/*---------------------------------------------------------------------------

	CMCoEcliptic.cpp  Ecliptical coordinate computig object

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : September 2009

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// system include
#include <string.h>
#include <math.h>



// project include
#include "CMCoEcliptic.h"
#include "CMCoRectangolar.h"
#include "CMCoEquatorial.h"
#include "CMFunctions.h"




/*---------------------------------------------------------------------------

	object constructor

---------------------------------------------------------------------------*/

CMCoEcliptic::CMCoEcliptic()
{
   Coo.l = 0;
   Coo.b = 0;
   Coo.d = 1;
}



/*---------------------------------------------------------------------------

	copy object constructor

---------------------------------------------------------------------------*/

CMCoEcliptic::CMCoEcliptic( CMCoEcliptic *Copy )
{
   Coo = Copy->Coo;
}



/*---------------------------------------------------------------------------

	set coordinate

---------------------------------------------------------------------------*/

int CMCoEcliptic::Set( ECL_COO* ParPos )
{
   Coo = *ParPos;
   return 0;
}



/*---------------------------------------------------------------------------

	get coordinate

---------------------------------------------------------------------------*/

const CMCoEcliptic::ECL_COO* CMCoEcliptic::Get( void )
{
   return &Coo;
}


/*---------------------------------------------------------------------------

	compute equatorial coordinate

---------------------------------------------------------------------------*/

CMCoEquatorial* CMCoEcliptic::GetCoEqu( double gg, bool Precision )
{
   double                              e0;                  // mean ecliptic obliquity
   double                              e;                   // true ecliptic obliquity
   double                              DeltaPhi;            // longitude nutation
   double                              DeltaEpsilon;        // obliquity nutation
   double                              l;                   // apperent longitude
   double                              b;                   // latitude
   double                              c;                   // temp for computing
   double                              cn;                  // temp for computing
   double                              cd;                  // temp for computing
   CMCoEquatorial*                    Out;                  // output position

   // create object for equatorial coordinate
   Out = new CMCoEquatorial();
   Out->SetDataEqu(gg);

   // compute ecliptic obliquity
   e0 = CM_ObliquityComputing( gg );

   // compute nutation
   DeltaPhi=0;
   DeltaEpsilon=0;
   if ( Precision == 1 )
      CM_NutationComputing( gg, &DeltaPhi, &DeltaEpsilon );

   // applay nutation
   e = e0 + DeltaEpsilon/3600;
   l = Coo.l + DeltaPhi/3600;
   b = Coo.b;

   // right ascension computing (meeus 8.3)
   cn =  sin(l/RAD) * cos(e /RAD)  - tan (b/RAD) * sin(e /RAD);
   cd = cos( l / RAD );
   c = atan( cn / cd ) * RAD;
   if ( cd < 0 )
      c += 180;
   if ( c < 0 )
      c += 360;
   Out->GetRA()->Set( c/15 );

   // declination computing (meeus 8.4)
   c = sin(b/RAD) * cos (e/RAD) + cos(b/RAD) * sin(e/RAD) * sin(l/RAD);
   c = asin( c ) * RAD;
   Out->GetDec()->Set(c);

   // set distance
   Out->SetDist(Coo.d);

   return Out;
}



/*---------------------------------------------------------------------------

	compute rectangolar coordinate

---------------------------------------------------------------------------*/

CMCoRectangolar* CMCoEcliptic::GetCoRet( void )
{
   CMCoRectangolar::RET_COO           Ret;                 // rectangolar position
   CMCoRectangolar*                   Obj;                 // rectangolar object

   Ret.X = Coo.d * cos( Coo.b / RAD ) * cos( Coo.l / RAD);
   Ret.Y = Coo.d * cos( Coo.b / RAD ) * sin( Coo.l / RAD);
   Ret.Z = Coo.d * sin( Coo.b / RAD );
   Obj = new CMCoRectangolar();
   Obj->Set(&Ret);

   return Obj;
}



/*---------------------------------------------------------------------------

	add value to coordinate

---------------------------------------------------------------------------*/

void CMCoEcliptic::Add( double QL, double QB )
{
   Coo.l += QL;
   Coo.b += QB;
}

