/*---------------------------------------------------------------------------

	CMCoEquatorial.cpp  Equatorial coordinate computig object

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : September 2009

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// system include
#include <string.h>
#include <math.h>



// project include
#include "CMCoEquatorial.h"
#include "CMCoHorizontal.h"
#include "CMCoLocal.h"
#include "CMFunctions.h"
#include "CMEvent.h"
#include "CMFunctions.h"




/*---------------------------------------------------------------------------

	object constructor

---------------------------------------------------------------------------*/

CMCoEquatorial::CMCoEquatorial()
{
   RA = new CMHours( );
   Dec = new CMDegree( );
   gg = 2415020;
   Dist = 1;
}



/*---------------------------------------------------------------------------

	copy object constructor

---------------------------------------------------------------------------*/

CMCoEquatorial::CMCoEquatorial( const CMCoEquatorial *Copy )
{
   CMHours::HOURS                      TempRA;
   CMDegree::DEGREE                    TempDec;

   RA = new CMHours( );
   Dec = new CMDegree( );

   TempRA = *Copy->GetRA()->Get();
   RA->Set( &TempRA );
   TempDec = *Copy->GetDec()->Get( );
   Dec->Set( &TempDec );
   gg = Copy->gg;
   Dist = Copy->Dist;
}



/*---------------------------------------------------------------------------

	destroyer

---------------------------------------------------------------------------*/

CMCoEquatorial::~CMCoEquatorial()
{
   delete RA;
   delete Dec;
}



/*---------------------------------------------------------------------------

	get object righ asension

---------------------------------------------------------------------------*/

CMHours* const CMCoEquatorial::GetRA( void ) const
{
   return RA;
}



/*---------------------------------------------------------------------------

	get object declination

---------------------------------------------------------------------------*/

CMDegree* const CMCoEquatorial::GetDec( void ) const
{
   return Dec;
}



/*---------------------------------------------------------------------------

	set equinoz epoch

---------------------------------------------------------------------------*/

void CMCoEquatorial::SetDataEqu( double Pargg )
{
   gg = Pargg;
}



/*---------------------------------------------------------------------------

	get ecliptic coordinate

   compute means or apparent coordinate

---------------------------------------------------------------------------*/

CMCoEcliptic* CMCoEquatorial::GetCoEcl( double gg, bool Apparent )
{
   double                           e0;                     // ecliptic mean obliquity
   double                           e;                      // ecliptic true obliquity
   double                           DeltaPhi;               // longitude nutation
   double                           DeltaEpsilon;           // obliquity nutation
   double                           alfa;                   // RA in degree
   double                           delta;                  // declination in degree
   double                           c;                      // temp
   double                           cn;
   double                           cd;
   CMCoEcliptic*                    Obj;                    // ecliptic coordinate object
   CMCoEcliptic::ECL_COO            Ecl;                    // ecliptic coordinate data

   // ecliptic obliquity computing
   e0 = CM_ObliquityComputing( gg );

   // nutation computing
   DeltaPhi=0;
   DeltaEpsilon=0;
   if ( Apparent == 1 )
      CM_NutationComputing( gg, &DeltaPhi, &DeltaEpsilon );

   // nutation applying
   e = e0 + DeltaEpsilon/3600;

   // create ecliptic coordinate object
   Obj = new CMCoEcliptic();

   // assign distance
   Ecl.d = Dist;

   alfa = RA->GetC()*15;
   delta = Dec->GetC();

   // longitude computing (meeus 8.1)
   cn = sin( alfa / RAD ) * cos ( e / RAD) + tan ( delta / RAD) * sin ( e / RAD );
   cd = cos( alfa / RAD );
   c = atan ( cn / cd ) * RAD;
   if ( cd < 0 )
      c += 180;
   if ( c < 0 )
      c += 360;
   Ecl.l = c;
   // latitude computing (meeus 8.2)
   c = sin( delta / RAD) * cos ( e / RAD ) - cos ( delta / RAD ) * sin ( e / RAD) * sin ( alfa / RAD );
   Ecl.b = asin( c ) * RAD;

   Obj->Set(&Ecl);

   return Obj;
}



/*---------------------------------------------------------------------------

   get horizontal coordinate

---------------------------------------------------------------------------*/

CMCoHorizontal* CMCoEquatorial::GetCoOri( double gg, CMCoLocal *Site, bool Apparent ) const
{
   CMEvent                             *Event;              // event for computing
   CMHours                             *SidObj;             // Greenwich sideral time object
   CMHours::HOURS                      SidValue;            // sideral time value
   CMHours                             *H;                  // Hour angle object
   double                              HValue;              // Hour angle in degree
   double                              Declination;         // declination in degree
   double                              Latitude;            // latitude
   double                              c;                   // temp
   double                              cn;
   double                              cd;
   CMCoHorizontal*                     Obj;                 // horizontal coordinate object

   // compute sideral time
   Event = new CMEvent();
   SidObj = new CMHours();
   Event->SetGG( gg );
   SidValue = CM_SideralTimeComputing( gg, Apparent );
   SidObj->Set(&SidValue);

   // compute hour angle
   H = new CMHours( SidObj );
   H->Add(Site->GetLongitude());
   H->Sub(RA);

   // object creation
   Obj = new CMCoHorizontal();

   // get angle in degree
   HValue = H->GetC() * 15;
   Declination = Dec->GetC();
   Latitude = Site->GetLatitude()->GetC();

   // compute azimut ( Meeus 8.5)
   cn = sin ( HValue / RAD );
   cd = cos ( HValue / RAD ) * sin ( Latitude / RAD ) - tan ( Declination / RAD) * cos ( Latitude / RAD );
   c = atan ( cn / cd ) * RAD;
   if ( cd < 0 )
      c += 180;
   if ( c < 0 )
      c += 360;
   Obj->GetAzimut()->Set( c / 15 );

   // compute elevation (Meeus 8.6)
   c = sin( Latitude / RAD ) * sin ( Declination / RAD ) + cos ( Latitude / RAD ) * cos ( Declination / RAD ) * cos ( HValue / RAD );
   c = asin ( c ) * RAD;
   Obj->GetElevation()->Set( c );

   // set distance
   Obj->SetDistance(Dist);

   delete Event;
   delete H;
   delete SidObj;

   return Obj;
}



/*---------------------------------------------------------------------------

	get topocentric equatorial coordinate
   Meeus ( 29.3 29.3 )

---------------------------------------------------------------------------*/

CMCoEquatorial* CMCoEquatorial::GetCoTopo( double gg, CMCoLocal *Site ) const
{
   CMEvent                             *Event;              // event for computing
   CMHours                             *SidObj;             // Greenwich sideral time object
   CMHours::HOURS                      SidValue;            // sideral time value
   CMHours                             *H;                  // Hour angle object
   double                              HValue;              // Hour angle in degree
   double                              RightAscension;      // right ascension in degree
   double                              Declination;         // declination in degree
   double                              Latitude;            // latitude
   double                              Elevation;           // elevation over horizon
   double                              u;                   // earth position parameters
   double                              SigmaSinPhi1;
   double                              SigmaCosPhi1;
   double                              pi;                  // parallax
   double                              RADelta;             // RA delta due parallax
   double                              NewDeclination;      // declination duw parallax
   double                              cd;                  // temp
   double                              cn;
   double                              c;
   CMCoEquatorial                      *Obj;                // equtorial object

   // compute sideral time
   Event = new CMEvent();
   Event->SetGG( gg );
   SidObj = new CMHours();
   SidValue = CM_SideralTimeComputing( gg, true );
   SidObj->Set(&SidValue);

   // compute hour angle
   H = new CMHours( SidObj );
   H->Sub(Site->GetLongitude());
   H->Sub(RA);

   // get hour angle in degree
   HValue = H->GetC() * 15;

   // get coordinate in degree
   RightAscension = RA->GetC()*15;
   Declination = Dec->GetC();

   // get local coordinate in degree
   Latitude =  Site->GetLatitude()->GetC();
   Elevation = Site->GetElevation();

   // compute parallax parameters ( Meeus 6 )
   c = EARTH_DEFORMATION * tan ( Latitude / RAD );
   u = atan ( c ) * RAD;
   SigmaSinPhi1 = EARTH_DEFORMATION * sin ( u / RAD ) + sin ( Latitude / RAD ) * Elevation / EARTH_RADIS;
   SigmaCosPhi1 =                     cos ( u / RAD ) + cos ( Latitude / RAD ) * Elevation / EARTH_RADIS;

   // compute parallax
   c = EARTH_RADIS / (UA * Dist);
   pi = asin ( c ) * RAD;

   // formule Meeus 29.2
   cn =                           - SigmaCosPhi1 * sin ( pi / RAD ) * sin ( HValue / RAD );
   cd = cos ( Declination / RAD ) - SigmaCosPhi1 * sin ( pi / RAD ) * cos ( HValue / RAD );
   RADelta = atan ( cn / cd ) * RAD;

   // formule 29.3
   cn = ( sin ( Declination / RAD ) - SigmaSinPhi1 * sin ( pi / RAD ) ) * cos ( RADelta / RAD );
   cd =   cos ( Declination / RAD ) - SigmaCosPhi1 * sin ( pi / RAD )   * cos ( HValue / RAD );
   NewDeclination = atan( cn / cd ) * RAD;

   // create object with new coordinate
   Obj = new CMCoEquatorial( this);
   Obj->GetRA()->Set( (RightAscension + RADelta) / 15 );
   Obj->GetDec()->Set( NewDeclination );

   return Obj;
}



/*---------------------------------------------------------------------------

	compute coordinate changing equinoz
   Meeus (14.1)

---------------------------------------------------------------------------*/

CMCoEquatorial* CMCoEquatorial::ChangeEqu( double Newgg ) const
{
   double                              RightAscension;      // right ascension in degree
   double                              Declination;         // declination in degree
   double                              Tau0;                // actual equinox epoch in julian centiries from 1900
   double                              Tau;                 // new equinox epoch in julian centiries from 1900
   double                              Epsilon;             // tranformation parameters
   double                              Zeta;
   double                              Teta;
   double                              A;
   double                              B;
   double                              C;
   CMCoEquatorial                      *Obj;                // euqtorial object

   Obj = new CMCoEquatorial( this);

   RightAscension = RA->GetC()*15;
   Declination = Dec->GetC();

   // compute epoch from 1900
   Tau0= (gg - 2415020.313)/36524.2199;
   Tau = (Newgg - gg)/36524.2199;

   // compute transformation parameters in degree
   Epsilon = (2304.250 + 1.396 * Tau0) * Tau + 0.302 * pow( Tau, 2) + 0.018 * pow( Tau, 3);
   Zeta = Epsilon + 0.791 * pow(Tau, 2) + 0.001 * pow (Tau, 3);
   Teta = (2004.682 - 0.853 * Tau0) * Tau - 0.426 * pow( Tau, 2) - 0.042 * pow (Tau, 3);
   Epsilon /= 3600;
   Zeta /= 3600;
   Teta /= 3600;

   // compute ABC
   A = cos ( Declination / RAD) * sin ( (RightAscension+Epsilon) / RAD);
   B = cos ( Teta/RAD) * cos( Declination / RAD) * cos ( (RightAscension+Epsilon) / RAD ) - sin ( Teta / RAD) * sin ( Declination / RAD );
   C = sin ( Teta / RAD ) * cos ( Declination / RAD ) * cos ( (RightAscension + Epsilon) /RAD ) + cos ( Teta / RAD ) * sin ( Declination / RAD );

   // get new RA and declination
   RightAscension = atan ( A/B ) * RAD;
   if ( B < 0 )
      RightAscension += 180;
   if ( RightAscension < 0 )
      RightAscension += 360;
   RightAscension += Zeta;
   if ( RightAscension > 360 )
      RightAscension -= 360;
   Declination = asin( C ) * RAD;

   // assign to new object
   Obj->GetRA()->Set( RightAscension / 15 );
   Obj->GetDec()->Set( Declination );
   Obj->SetDataEqu(Newgg);

   return Obj;
}



/*---------------------------------------------------------------------------

	set distance

---------------------------------------------------------------------------*/

void CMCoEquatorial::SetDist( double ParDist )
{
   Dist = ParDist;
}



/*---------------------------------------------------------------------------

	get distance

---------------------------------------------------------------------------*/

double CMCoEquatorial::GetDist( void ) const
{
   return Dist;
}

