/*---------------------------------------------------------------------------
                                    
	CMCoHorizontal.cpp  horizontal coordinate computig object

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : September 2009

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// system include
#include <string.h>
#include <math.h>



// project include
#include "CMCoHorizontal.h"
#include "CMCoEquatorial.h"
#include "CMCoLocal.h"
#include "CMEvent.h"
#include "CMFunctions.h"



/*---------------------------------------------------------------------------

	object constructor

---------------------------------------------------------------------------*/

CMCoHorizontal::CMCoHorizontal()
{
   Azimut = new CMHours( );
   Elevation = new CMDegree( );
}



/*---------------------------------------------------------------------------

	copy object constructor

---------------------------------------------------------------------------*/

CMCoHorizontal::CMCoHorizontal( CMCoHorizontal *Copy )
{
   CMHours::HOURS                      TempAr;              // temp objects
   CMDegree::DEGREE                    TempDec;

   Azimut = new CMHours( );
   Elevation = new CMDegree( );

   TempAr = *Copy->GetAzimut()->Get();
   Azimut->Set( &TempAr );
   TempDec = *Copy->GetElevation()->Get( );
   Elevation->Set( &TempDec );
}



/*---------------------------------------------------------------------------

	destroyer

---------------------------------------------------------------------------*/

CMCoHorizontal::~CMCoHorizontal()
{
   delete Azimut;
   delete Elevation;
}



/*---------------------------------------------------------------------------

	get azimut

---------------------------------------------------------------------------*/

CMHours* const CMCoHorizontal::GetAzimut( void )
{
   return Azimut;
}



/*---------------------------------------------------------------------------

	get elevatrion

---------------------------------------------------------------------------*/

CMDegree* const CMCoHorizontal::GetElevation( void )
{
   return Elevation;
}



/*---------------------------------------------------------------------------

	get equatorial coordinate

---------------------------------------------------------------------------*/

CMCoEquatorial* CMCoHorizontal::GetCoEqu( double gg, CMCoLocal *Site, bool Apparent )
{
   CMEvent                             *Event;              // event for computing
   CMHours                             *SidObj;             // Greenwich sideral time object
   CMHours::HOURS                      SidValue;            // sideral time value
   CMHours                             *H;                  // Hour angle object
   double                              Latitude;            // latitude
   CMHours                             *RA;                 // right ascension
   double                              Azimut_gg;           // azimut in degree
   double                              Elevation_gg;        // elevation in gradi
   double                              c;                   // temp
   double                              cn;
   double                              cd;
   CMCoEquatorial*                     Obj;                 // equatorial coordinate object

   // compute sideral time
   Event = new CMEvent();
   Event->SetGG( gg );
   SidObj = new CMHours();
   SidValue = CM_SideralTimeComputing( gg, Apparent );
   SidObj->Set(&SidValue);

   // object creation
   Obj = new CMCoEquatorial();

   // get angles in degree
   Azimut_gg = Azimut->GetC();
   Elevation_gg = Elevation->GetC();
   Latitude = Site->GetLatitude()->GetC();

   // compute hour angle
   cn = sin ( Azimut_gg / RAD );
   cd = cos ( Azimut_gg / RAD ) * sin ( Latitude / RAD ) + tan ( Elevation_gg / RAD) * cos ( Latitude / RAD );
   c = atan ( cn / cd ) * RAD;
   if ( cd < 0 )
      c += 180;
   if ( c < 0 )
      c += 360;
   H = new CMHours( );
   H->Set( c/15 );

   // compute right ascension
   RA = new CMHours( SidObj );
   RA->Sub(Site->GetLongitude());
   RA->Sub(H);
   Obj->GetRA()->Set( RA->Get() );

   // compute elevation
   c = sin( Latitude / RAD ) * sin ( Elevation_gg / RAD ) - cos ( Latitude / RAD ) * cos ( Elevation_gg / RAD ) * cos ( Azimut_gg / RAD );
   c = asin ( c ) * RAD;
   Obj->GetDec()->Set( c );

   delete Event;
   delete H;
   delete RA;
   delete SidObj;

   return Obj;
}



/*---------------------------------------------------------------------------

	Set distance

---------------------------------------------------------------------------*/

void CMCoHorizontal::SetDistance( double ParDistance )
{
   Distance = ParDistance;
}



/*---------------------------------------------------------------------------

	get distance

---------------------------------------------------------------------------*/

double CMCoHorizontal::GetDistance( void )
{
   return Distance;
}

