/*---------------------------------------------------------------------------

	CMEvent.cpp  Event computing object

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : August 2009

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// system include
#include <math.h>



// project include
#include "CMEvent.h"
#include "CMFunctions.h"



/*---------------------------------------------------------------------------

	constructor

---------------------------------------------------------------------------*/

CMEvent::CMEvent()
{
   ValidC = true;
   ValidGG = false;
   EventC.Year = 2000;
   EventC.Month = 1;
   EventC.Day = 1;
   EventC.Hour = 0;
   EventC.Minute = 0;
   EventC.Sec = 0;
}



/*---------------------------------------------------------------------------

	copy constructor

---------------------------------------------------------------------------*/

CMEvent::CMEvent( CMEvent *Copy )
{
   EventC = Copy->EventC;
   EventGG = Copy->EventGG;
   ValidC = Copy->ValidC;
   ValidGG = Copy->ValidGG;
}


/*---------------------------------------------------------------------------

	set date

---------------------------------------------------------------------------*/

int CMEvent::SetDate( DATE *Event )
{
   EventC = *Event;
   ValidC = true;
   ValidGG = false;
   return 0;
}



/*---------------------------------------------------------------------------

	set date in JD

---------------------------------------------------------------------------*/

int CMEvent::SetGG( double Event )
{
   EventGG = Event;
   ValidC = false;
   ValidGG = true;
   return 0;
}



/*---------------------------------------------------------------------------

	get date

---------------------------------------------------------------------------*/

const CMEvent::DATE* CMEvent::GetDate()
{
   if ( ValidC == false )
   {
      if ( ValidGG == true )
         CalcDate();
      else
         EventC.Year = 0;
   }
   return &EventC;
}



/*---------------------------------------------------------------------------

	get date in JD

---------------------------------------------------------------------------*/

double CMEvent::GetGG()
{
   if ( ValidGG == false )
   {
      if ( ValidC == true )
         CalcGG();
      else
         return 0;
   }
   return EventGG;
}



/*---------------------------------------------------------------------------

	add days to current event

---------------------------------------------------------------------------*/

void CMEvent::AddTime( double Delta )
{
   // is the JD data correct?
   if ( ValidGG == false )
   {
      if ( ValidC == true )
         CalcGG();
   }
   // add to JD date
   EventGG += Delta;
   // data not valid
   ValidC = false;
}



/*---------------------------------------------------------------------------

	compute JD from date
	both Gregorian and Julian calendar

---------------------------------------------------------------------------*/

void CMEvent::CalcGG( void )
{
   int                                 Month;
   int                                 Year;
   int                                 A;
   int                                 B;
   double                              JD;
   double                              Day;
   bool                                Greg;                // Gregorian date

   // Gregorian or Julian calendar ?
   Greg = false;
   if ( EventC.Year > 1582 )
      Greg = true;
   if ( EventC.Year == 1582 )
   {
      if ( EventC.Month > 10 )
         Greg = true;
      if ( EventC.Month == 12 )
      {
         if ( EventC.Day >= 15 )
            Greg = true;
      }
   }

   Month = EventC.Month;
   Year = EventC.Year;
   if ( Month < 3 )
   {
      Month += 12;
      Year--;
   }
   A = B = 0;
   if ( Greg == true )
   {
      A = Year/100;
      B = 2 - A + (A/4);
   }
   JD  = (int)( (double)(Year+4716) * 365.25 );
   JD += (int)( (double)(Month+1)* 30.6001 );
   JD += EventC.Day;
   JD += B;
   JD -= 1524.5;

   Day = EventC.Sec;
   Day /= 60;
   Day += EventC.Minute;
   Day /= 60;
   Day += EventC.Hour;
   Day /= 24;

   JD += Day;

   EventGG = JD;
}



/*---------------------------------------------------------------------------

	compute date from JD
	both Gregorian and Julian calendar

---------------------------------------------------------------------------*/

void CMEvent::CalcDate( void )
{
   int                                 Month;
   int                                 Year;
   double                              Alfa;
   int                                 Alfa2;
   int                                 A;
   double                              B;
   int                                 C;
   int                                 D;
   double                              E;
   double                              JD;
   int                                 Z;
   double                              F;
   double                              Day;

   JD = EventGG + 0.5;
   Z = JD;
   F = JD - Z;

   if ( Z < 2299161 )
      A = Z;
   else
   {
      Alfa = (Z-1867216.25)/36524.25;
      Alfa2 = Alfa;
      A = Z + 1 +Alfa2 - (int)(Alfa/4);
   }
   B = A + 1524;
   C = (B-122.1)/365.25;
   D = 365.25 * C;
   E = (int)((B-D)/30.6001);
   Day = B - D - (int)(30.6001*E) + F;
   if ( E < 13.5 )
      Month = E - 1;
   else
      Month = E - 13;
   if ( Month > 2 )
      Year = C - 4716;
   else
      Year = C - 4715;

   EventC.Year = Year;
   EventC.Month = Month;
   EventC.Day = Day;
   Day -= EventC.Day;

   Day *= 24;
   EventC.Hour = Day;
   Day -= EventC.Hour;
   Day *= 60;
   EventC.Minute = Day;
   Day -= EventC.Minute;
   Day *= 60;
   Day += 0.5;
   EventC.Sec = Day;

}






