/*---------------------------------------------------------------------------

	CMFunctions.cpp  Generic astronomical functions

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : October 2009

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// system include
#include <math.h>
#include <string.h>



// project include
#include "CMFunctions.h"



/*---------------------------------------------------------------------------

	compute orbital parameters

---------------------------------------------------------------------------*/

void CM_OrbitalParamCompute( double gg, const MEEUS_COEFF_ELE *CoeEleMeeus, MEEUS_ELE_ORB &EleOrb )
{
   double                              T;                   // Julian centiries from 1900
   double                              c;                   // temp

   // compute Julian centiries from 1900
   T = (gg - 2415020)/36525;

   // compute orbital parameters
   EleOrb.L = CM_PoliCompute( T, &CoeEleMeeus->L );
   EleOrb.L = CM_DegreeReduce( EleOrb.L );
   EleOrb.a = CM_PoliCompute( T, &CoeEleMeeus->a );
   EleOrb.e = CM_PoliCompute( T, &CoeEleMeeus->e );
   EleOrb.i = CM_PoliCompute( T, &CoeEleMeeus->i );
   EleOrb.omega = CM_PoliCompute( T, &CoeEleMeeus->omega );
   EleOrb.omega = CM_DegreeReduce( EleOrb.omega );
   EleOrb.OMEGA = CM_PoliCompute( T, &CoeEleMeeus->OMEGA );
   EleOrb.OMEGA = CM_DegreeReduce( EleOrb.OMEGA );

   // perielio longitude
   EleOrb.pi = EleOrb.OMEGA + EleOrb.omega;
   EleOrb.pi = CM_DegreeReduce( EleOrb.pi );

   // mean anomaly
   EleOrb.M = EleOrb.L - EleOrb.omega - EleOrb.OMEGA;
   EleOrb.M = CM_DegreeReduce( EleOrb.M );

   // solve keplero equation
   EleOrb.E = CM_KepleroEqu( EleOrb.e, EleOrb.M );

   // true anomaly
   c = tan( EleOrb.E/ RAD /2 ) * sqrt( (1+EleOrb.e)/(1-EleOrb.e) );
   c = atan( c );
   c = c * RAD;
   if ( (EleOrb.E/2) > 90 && (EleOrb.E) < 270 )
      c += 180;
   if ( c < 0 )
      c += 360;
   c *= 2;
   if ( c > 360 )
      c -= 360;
   EleOrb.v = c;
}



/*---------------------------------------------------------------------------

	compute position

---------------------------------------------------------------------------*/

CMCoEcliptic* CM_PositionCompute( MEEUS_ELE_ORB &EleOrb )
{
   double                              c;                   // temp
   CMCoEcliptic                        *PosEcl;             // eclictic position  object
   CMCoEcliptic::ECL_COO               Ecl;                 // eclictic position value
   double                              u;                   // latitude argument

   // vector radius
   Ecl.d = EleOrb.a * ( 1 - EleOrb.e * cos( EleOrb.E / RAD ) );

   // compute latitude argument, eclictic longitude and latitude
   // from planet plan to eclictic plan
   u = EleOrb.L + EleOrb.v - EleOrb.M - EleOrb.OMEGA;
   u = CM_DegreeReduce( u );
   c = cos( EleOrb.i / RAD ) * tan (u / RAD);
   c = atan( c ) * RAD;
   if ( u > 90 && u < 270 )
      c += 180;
   if ( c < 0 )
      c += 360;
   Ecl.l = c + EleOrb.OMEGA;
   Ecl.l = CM_DegreeReduce( Ecl.l );

   // latitude
   c = sin( u /RAD ) * sin ( EleOrb.i /RAD );
   Ecl.b = asin( c ) * RAD;
   PosEcl = new CMCoEcliptic();
   PosEcl->Set(&Ecl);

   return PosEcl;
}



/*---------------------------------------------------------------------------

	compute Keplero equation with Jean Meeus chapter 22 second way

---------------------------------------------------------------------------*/

double CM_KepleroEqu( double e, double M )
{
   double                              e0;
   double                              E;
   double                              E0;
   double                              Corr;

   // good for any eccentricity
   e0 = e * RAD;
   E0=M;
   while ( 1 )
   {
      Corr =  (M + e0* sin( E0 / RAD ) - E0) / (1 - e * cos( E0 / RAD ));
      E = E0 + Corr;
      if ( fabs( Corr ) <= 0.00000001 )
         break;
      E0 = E;
   }

   return E;
}



/*---------------------------------------------------------------------------

	reduce in range 0-360 degree

---------------------------------------------------------------------------*/

double CM_DegreeReduce(double Val)
{
   int                                 Turn;

   Turn = Val/360;
   if ( Val < 0 )
      Turn--;
   Turn *= 360;

   Val -= Turn;

   return Val;
}



/*---------------------------------------------------------------------------

	compute paramates with polynomials

---------------------------------------------------------------------------*/

double CM_PoliCompute( double T, const MEEUS_COEFF_POLI *Coe )
{
   double                              Val;
   double                              T1;

   T1 = T;
   Val = Coe->a0 + Coe->a1 * T1;
   T1 *= T;
   Val += Coe->a2 * T1;
   T1 *= T;
   Val += Coe->a3 * T1;

   return Val;
}



/*---------------------------------------------------------------------------

	obliquity computing (Meeus 18.4)

---------------------------------------------------------------------------*/

double CM_ObliquityComputing( double gg )
{
   double                              T;                   // Julian centiries from 1900

   T = (gg - 2415020)/36525;
   return CM_PoliCompute( T, &Obliquity );
}



/*---------------------------------------------------------------------------

	nutation computing (meeus 15)

---------------------------------------------------------------------------*/

void CM_NutationComputing( double GG, double *DeltaPhi, double *DeltaEpsilon )
{
   double                              T;                   // Julian centiries from 1900
   static double                       gg=0;                // last computing JD
   static double                       DPhi;                // nutation parameters
   static double                       DEpsilon;
   double                              L;                   // sun mean longitude
   double                              L1;                  // moon mean longitude
   double                              M;                   // sun mean anomaly
   double                              M1;                  // moon mean anomaly
   double                              Omega;               // moon ascending node

   // compute only when date change
   if ( GG != gg )
   {
      gg = GG;                                              // save for next time

      // compute parameters nutation involved
      T = (gg - 2415020) / 36525;
      L = CM_PoliCompute( T, &SunLong );
      L = CM_DegreeReduce( L );
      L1 = CM_PoliCompute( T, &MoonLong );
      L1 = CM_DegreeReduce( L1 );
      M = CM_PoliCompute( T, &SunAnom );
      M = CM_DegreeReduce( M );
      M1 = CM_PoliCompute( T, &MoonAnom );
      M1 = CM_DegreeReduce( M1 );
      Omega = CM_PoliCompute( T, &MoonNode );
      Omega = CM_DegreeReduce( Omega );

      DPhi = - (17.2327 + 0.01737 * T ) * sin ( Omega / RAD )
             - (1.2729  + 0.00013 * T ) * sin ( 2 * L / RAD )
             +  0.2088                  * sin ( 2 * Omega / RAD )
             -  0.2037                  * sin ( 2 * L1 / RAD )
             + (0.1261  - 0.00031 * T ) * sin ( M / RAD )
             +  0.0675                  * sin ( M1 / RAD )
             - (0.0497  - 0.00012 * T ) * sin ((2 * L + M) / RAD )
             -  0.0342                  * sin ((2 * L1 - Omega) / RAD )
             -  0.0261                  * sin ((2 * L1 + M1) / RAD )
             +  0.0214                  * sin ((2 * L  - M1) / RAD )
             -  0.0149                  * sin ((2 * L - 2 * L1 + M1) / RAD )
             +  0.0124                  * sin ((2 * L - Omega ) / RAD )
             +  0.0114                  * sin ((2 * L1 - M1) /RAD);

      DEpsilon =    (9.2100 + 0.00091 * T )  * cos ( Omega / RAD )
                  + (0.5522 - 0.00029 * T )  * cos ( 2 * L / RAD )
                  -  0.0904                  * cos ( 2 * Omega / RAD )
                  +  0.0884                  * cos ( 2 * L1 / RAD )
                  +  0.0216                  * cos ((2 * L + M) / RAD )
                  +  0.0183                  * cos ((2 * L1 - Omega ) / RAD )
                  +  0.0113                  * cos ((2 * L1 + M1) / RAD )
                  -  0.0093                  * cos ((2 * L - M ) / RAD )
                  -  0.0066                  * cos ((2 * L - Omega ) / RAD );

   }

   *DeltaPhi = DPhi;
   *DeltaEpsilon = DEpsilon;
}



/*---------------------------------------------------------------------------

	sideral time computing (Meeus 7)

---------------------------------------------------------------------------*/

CMHours::HOURS CM_SideralTimeComputing( double gg, bool Apparent )
{
   double                              T;                   // Julian centiries from 1900
   int                                 ggInt;               // integer part of gg
   double                              ggOnlyDate;          // only data of gg
   double                              ggOnlyTime;          // only time of gg
   double                              OnlyTime;            // only time in hours
   double                              SidTime_Double;      // sideral time in hours
   int                                 SidTime_Int;         // integer part of siderla time
   double                              e0;                  // obliquity
   double                              DeltaPhi;            // nutation parameters
   double                              DeltaEpsilon;
   double                              Corr;                // nutation correction
   CMHours::HOURS                      SideralTime;         // sideral time value
   CMHours::HOURS                      SideralTime2;        // daily sideral time

   // compute jd day begin
   ggInt = gg;                                              // only integer part
   ggOnlyDate = ggInt;                                      // jd date start with 0.5
   ggOnlyDate += 0.5;
   if ( ggOnlyDate > gg )                                   // if greated is the day after
      ggOnlyDate -= 1;                                      // go back one day

   // compute siderl time at day begining
   T = (ggOnlyDate - 2415020)/36525;                        // meeus 7.1
   SidTime_Double = 0.276919398 + 100.0021359 * T + 0.000001075 * T * T;     // meeus 7.3
   SidTime_Int = SidTime_Double;
   SidTime_Double -= SidTime_Int;
   SidTime_Double *= 24;
   SideralTime.hh = SidTime_Double;
   SidTime_Double -= SideralTime.hh;
   SidTime_Double *= 60;
   SideralTime.mm = SidTime_Double;
   SidTime_Double -= SideralTime.mm;
   SidTime_Double *= 60;
   SideralTime.ss = SidTime_Double;

   // compute daily siderl time
   ggOnlyTime = gg - ggOnlyDate;
   OnlyTime = ggOnlyTime*24;
   OnlyTime *= 1.002737908;
   while ( OnlyTime >= 24 )
      OnlyTime -= 24;
   SideralTime2.hh = OnlyTime;
   OnlyTime -= SideralTime2.hh;
   OnlyTime *= 60;
   SideralTime2.mm = OnlyTime;
   OnlyTime -= SideralTime2.mm;
   OnlyTime *= 60;
   SideralTime2.ss = OnlyTime;

   // compose sideral time
   SideralTime.ss += SideralTime2.ss;
   if ( SideralTime.ss >= 60 )
   {
      SideralTime.ss -= 60;
      SideralTime.mm ++;
   }
   SideralTime.mm += SideralTime2.mm;
   if ( SideralTime.mm >= 60 )
   {
      SideralTime.mm -= 60;
      SideralTime.hh ++;
   }
   SideralTime.hh += SideralTime2.hh;
   if ( SideralTime.hh >= 24 )
      SideralTime.hh -= 24;

   if ( Apparent == true )
   {
      e0 = CM_ObliquityComputing( gg );
      CM_NutationComputing( gg, &DeltaPhi, &DeltaEpsilon );
      Corr = DeltaPhi * cos ( e0 / RAD ) / 15;

      SideralTime.ss += Corr;
      if ( SideralTime.ss >= 60 )
      {
         SideralTime.ss -= 60;
         SideralTime.mm ++;
      }
      if ( SideralTime.ss < 0 )
      {
         SideralTime.ss += 60;
         SideralTime.mm --;
      }
      if ( SideralTime.mm >= 60 )
      {
         SideralTime.mm -= 60;
         SideralTime.hh ++;
      }
      if ( SideralTime.mm < 0 )
      {
         SideralTime.mm += 60;
         SideralTime.hh --;
      }
      if ( SideralTime.hh >= 24 )
         SideralTime.hh -= 24;
      if ( SideralTime.hh < 0 )
         SideralTime.hh += 24;
   }

   return SideralTime;
}



/*---------------------------------------------------------------------------

	moon position computing (Meeus 30)

---------------------------------------------------------------------------*/

CMCoEcliptic* CM_MoonPositionComputing( double gg )
{
   double                              T;                   // Julian centiries from 1900
   double                              T2;                  // square Julian centiries from 1900
   double                              T3;                  // cube Julian centiries from 1900
   double                              L1;                  // mean moon longitude
   double                              M;                   // mean sun anomaly
   double                              M1;                  // amean moon anomaly
   double                              D;                   // mean moon elongation
   double                              F;                   // mean ascending node distance
   double                              OMEGA;               // ascending node longitude
   double                              A1;                  // additional terms
   double                              A2;
   double                              A3;
   double                              A4;
   double                              A5;
   double                              A6;
   double                              A7;
   double                              A8;
   double                              A9;
   double                              A10;
   double                              e;                   // e term
   double                              lambda;              // moon longitude
   double                              B;                   // row moon latitude
   double                              Omega1;
   double                              Omega2;
   double                              b;                   // moon longitude
   double                              pi;                  // moon parallax
   double                              d;                   // distance
   CMCoEcliptic                        *PosEcl;             // eclitic position value
   CMCoEcliptic::ECL_COO               Ecl;                 // eclitic position object

   // compute Julian centiries
   T = (gg - 2415020)/36525;
   T2 = T * T;
   T3 = T2 * T;

   // compute means moon parameters
   L1    = 270.434164 + 481267.8831 * T - 0.001133 * T2 + 0.0000019 * T3;
   L1 = CM_DegreeReduce( L1 );
   M     = 358.475833 +  35999.0498 * T - 0.000150 * T2 + 0.0000033 * T3;
   M  = CM_DegreeReduce( M );
   M1    = 296.104608 + 477198.8491 * T + 0.009192 * T2 + 0.0000144 * T3;
   M1 = CM_DegreeReduce( M1 );
   D     = 350.737486 + 445267.1142 * T - 0.001436 * T2 + 0.0000019 * T3;
   D  = CM_DegreeReduce( D );
   F     =  11.250889 + 483202.0251 * T - 0.003211 * T2 + 0.0000003 * T3;
   F  = CM_DegreeReduce( F );
   OMEGA = 259.183275 -   1934.1420 * T + 0.002078 * T2 + 0.0000022 * T3;
   OMEGA = CM_DegreeReduce( OMEGA );

   // compute additional terms
   A1 =  0.000233 * sin ( (51.2+20.2*T) / RAD);
   A2 = -0.001778 * sin ( (51.2+20.2*T) / RAD);
   A3 =  0.000817 * sin ( (51.2+20.2*T) / RAD);
   A4 =  0.002011 * sin ( (51.2+20.2*T) / RAD);
   A5 =  0.003964 * sin ( (346.560 + 132.870 * T - 0.0091731 * T2) / RAD );
   A6 =  0.001964 * sin ( OMEGA / RAD );
   A7 =  0.002541 * sin ( OMEGA / RAD );
   A8 =  0.001964 * sin ( OMEGA / RAD );
   A9 = -0.024961 * sin ( OMEGA / RAD );
   A10= -0.004328 * sin ( ( OMEGA + 275.05 - 2.30 * T) / RAD );

   // applay additional terms
   L1 += A1 + A5 + A6;
   M  += A2;
   M1 += A3 + A5 + A7;
   D  += A4 + A5 + A8;
   F  += A5 + A9 + A10;

   // compute e term
   e = 1 - 0.002495 * T - 0.00000752 * T2;

   // compute longitude
   lambda = L1 + CM_MoonSinComputing( M, M1, D, F, e, Moon_lambda, Num_Moon_lambda );

   // compute latitude
   B = CM_MoonSinComputing( M, M1, D, F, e, Moon_B, Num_Moon_B );
   Omega1 = 0.0004664 * cos ( OMEGA / RAD );
   Omega2 = 0.0000754 * cos ( (OMEGA+275.05-2.3*T) / RAD );
   b = B * ( 1 - Omega1 - Omega2 );

   // compute parallax
   pi = 0.950724 + CM_MoonCosComputing( M, M1, D, F, e, Moon_pi, Num_Moon_pi );
   // calcolo distanza in ua
   d = EARTH_RADIS / sin ( pi/ RAD );                       // distanza in kilometri
   d = d/UA;                                                // distanza in ua

   PosEcl = new CMCoEcliptic();
   Ecl.l = lambda;
   Ecl.b = b;
   Ecl.d = d;
   PosEcl->Set(&Ecl);


/*
           6378.14
sin(pi) = ---------------------------
          149597870.691 * delta (ua)
                                   */
   return PosEcl;
}



/*---------------------------------------------------------------------------

	moon sin series computing

---------------------------------------------------------------------------*/

double CM_MoonSinComputing( double M, double M1, double D, double F, double e, const MEEUS_ELE_MOON *Ele, int NumEle )
{
   double                              Res;                 // result
   double                              Arg;                 // sin argument
   double                              Term;                // sin temr
   int                                 Index;               // scan index

   Res = 0;
   // scan serie elements
   for ( Index=0; Index<NumEle; Index++)
   {
      // build sin argument
      Arg =  Ele->Mult_M  * M;
      Arg += Ele->Mult_M1 * M1;
      Arg += Ele->Mult_D  * D;
      Arg += Ele->Mult_F  * F;
      // compute sin term
      Term = Ele->Coe * sin (Arg / RAD);
      if ( Ele->Pot_e == 1 )
         Term *= e;
      if ( Ele->Pot_e == 2 )
         Term *= e * e;
      // totalize in result
      Res += Term;
      Ele++;
   }

   return Res;
}



/*---------------------------------------------------------------------------

	moon cos series computing

---------------------------------------------------------------------------*/

double CM_MoonCosComputing( double M, double M1, double D, double F, double e, const MEEUS_ELE_MOON *Ele, int NumEle )
{
   double                              Res;                 // result
   double                              Arg;                 // cos argument
   double                              Term;                // cos term
   int                                 Index;               // scan index

   Res = 0;
   // scan serie elements
   for ( Index=0; Index<NumEle; Index++)
   {
      // build cos argument
      Arg =  Ele->Mult_M  * M;
      Arg += Ele->Mult_M1 * M1;
      Arg += Ele->Mult_D  * D;
      Arg += Ele->Mult_F  * F;
      // compute cos term
      Term = Ele->Coe * cos (Arg / RAD);
      if ( Ele->Pot_e == 1 )
         Term *= e;
      if ( Ele->Pot_e == 2 )
         Term *= e * e;
      // totalize in result
      Res += Term;
      Ele++;
   }

   return Res;
}



/*---------------------------------------------------------------------------

	Add Hours
	
	Time2 = Time1 + Time2
	
---------------------------------------------------------------------------*/

void CM_AddTime( CMHours::HOURS * Time1, CMHours::HOURS * Time2 )
{
   Time2->ss += Time1->ss;
   if ( Time2->ss > 59 )
   {
      Time2->ss -= 60;
      Time2->mm += 1;
   }
   Time2->mm += Time1->mm;
   if ( Time2->mm > 59 )
   {
      Time2->mm -= 60;
      Time2->hh += 1;
   }
   Time2->hh += Time1->hh;
   if ( Time2->hh > 23 )
      Time2->hh -= 24;
}



/*---------------------------------------------------------------------------

	subctract Hours

	Time2 = Time2 - Time1
	
---------------------------------------------------------------------------*/

void CM_SubTime( CMHours::HOURS * Time1, CMHours::HOURS * Time2 )
{
   Time2->ss -= Time1->ss;
   if ( Time2->ss < 0 )
   {
      Time2->ss += 60;
      Time2->mm -= 1;
   }
   Time2->mm -= Time1->mm;
   if ( Time2->mm < 0 )
   {
      Time2->mm += 60;
      Time2->hh -= 1;
   }
   Time2->hh -= Time1->hh;
   if ( Time2->hh < 0 )
      Time2->hh += 24;
} 



/*---------------------------------------------------------------------------

	convert Hours in degree

---------------------------------------------------------------------------*/

double CM_Convert( CMHours::HOURS *Pos )
{
   double                              Degree;

   Degree = Pos->ss;
   Degree /= 60;
   Degree += Pos->mm;
   Degree /= 60;
   Degree += Pos->hh;
   Degree *= 15;

   return Degree;
}



/*---------------------------------------------------------------------------

	convert degree in Hours

---------------------------------------------------------------------------*/

double CM_Convert( CMDegree::DEGREE *Pos )
{
   double                              Degree;

   Degree = Pos->ss;
   Degree /= 60;
   Degree += Pos->mm;
   Degree /= 60;
   Degree += Pos->gg;
   if ( Pos->sign[0] == '-' )
      Degree = - Degree;

   return Degree;
}



/*---------------------------------------------------------------------------

	Add declination

---------------------------------------------------------------------------*/

void CM_AddDec( CMDegree::DEGREE * Dec1, CMDegree::DEGREE * Dec2 )
{
   double                              Degree1;             // composed coordinate
   double                              Degree2;             // composed coordinate

   Degree1 = Dec1->gg;                                      // assign degree
   Degree1 *= 60;                                           // makes it minutes
   Degree1 += Dec1->mm;                                     // add minutes
   Degree1 *= 60;                                           // makes it seconds
   Degree1 += Dec1->ss;                                     // add seconds
   if ( Dec1->sign[0] == '-' )                              // make is negative if it needs
      Degree1 = - Degree1;

   Degree2 = Dec2->gg;                                      // assign degree
   Degree2 *= 60;                                           // makes it minutes
   Degree2 += Dec2->mm;                                     // add minutes
   Degree2 *= 60;                                           // makes it seconds
   Degree2 += Dec2->ss;                                     // add seconds
   if ( Dec2->sign[0] == '-' )                              // make is negative if it needs
      Degree2 = - Degree2;

   Degree1 += Degree2;                                      // add declination

   // assign sign
   strcpy( Dec1->sign, "+");
   if ( Degree1 < 0 )
   {
      strcpy( Dec1->sign, "-");
      Degree1 = - Degree1;
   }

   Dec1->gg = Degree1 / 3600;                               // assign degrees
   Degree1 = Degree1 - Dec1->gg * 3600;                     // residual degrees
   Dec1->mm = Degree1 / 60;                                 // assign minutes
   Degree1 = Degree1 - Dec1->mm * 60;                       // residual minutes
   Dec1->ss = Degree1;                                      // assign seconds
}



/*---------------------------------------------------------------------------

   planet elements

---------------------------------------------------------------------------*/

const MEEUS_COEFF_ELE      EleMercury=
{
   {  178.179078, 149474.07078,  0.0003011,     0},
   {  0.3870986,  0,             0,             0},
   {  0.20561421, 0.00002046,    -0.000000030,  0},
   {  7.002881,   0.0018608,     -0.0000183,    0},
   {  28.753753,  0.3702806,     0.0001208,     0},
   {  47.145944,  1.1852083,     0.0001739,     0}
};



const MEEUS_COEFF_ELE      EleVenus=
{
   {  342.767053, 58519.21191,   0.0003097,     0},
   {  0.7233316,  0,             0,             0},
   {  0.00682069, -0.00004774,   0.000000091,   0},
   {  3.393631,   0.0010058,     -0.0000010,    0},
   {  54.384186,  0.5081861,     -0.0013864,    0},
   {  75.779647,  0.8998500,     0.0004100,     0}
};



const MEEUS_COEFF_ELE      EleEarth=
{
   {  279.69668-180,  36000.76892,   0.0003035,     0},
   {  1.0000002,  0,             0,             0},
   {  0.01675104, -0.0000418,    -0.000000126,  0},
   {  0,          0,             0,             0},
   {  281.22085-180,  1.71917,   0.0004525,     0.0000033},
   {  0,          0,             0,             0}
};



const MEEUS_COEFF_ELE      EleEarth2000=
{
   {  101.093014,  35999.3728838,-0.00000596,   0.000000005},
   {  1.0000002,  0,             0,             0},
   {  0.01675104, -0.0000418,    -0.000000126,  0},
   {  -0.0130762, 0.0130855,     -0.00000927,   0.000000014},
   {  287.511505,0.5647920,      0.00013610,    0.000003333},
   {  175.105679, -0.2416582,    0.00000794,    -0.000000028}
};



const MEEUS_COEFF_ELE      EleMars=
{
   {  293.737334, 19141.69551,   0.0003107,     0},
   {  1.5236883,  0,             0,             0},
   {  0.09331290, 0.000092064,   -0.000000077,  0},
   {  1.850333,   -0.0006750,    0.0000126,     0},
   {  285.431761, 1.0697667,     0.0001313,     0.00000414},
   {  48.786442,  0.7709917,     -0.0000014,    -0.00000533}
};



const MEEUS_COEFF_ELE      EleJupiter=
{
   {  238.049257, 3036.301986,   0.0003347,     -0.00000165},
   {  5.202561,   0,             0,             0},
   {  0.04833475, 0.000164180,   -0.0000004676, -0.0000000017},
   {  1.308736,   -0.0056961,    0.0000039,     0},
   {  273.277558, 0.5994317,     0.00070405,    0.00000508},
   {  99.443414,  1.0105300,     0.00035222,    -0.00000851}
};



const MEEUS_COEFF_ELE      EleSaturn=
{
   {  266.564377, 1223.509884,   0.0003245,     -0.0000058},
   {  9.554747,   0,             0,             0},
   {  0.05589232, -0.00034550, -0.000000728,    0.00000000074},
   {  2.492519,   -0.0039189,    -0.00001549,   0.00000004},
   {  338.307800, 1.0852207,     0.00097854,    0.00000992},
   {  112.790414, 0.8731951,     -0.00015218,   -0.00000531}
};



const MEEUS_COEFF_ELE      EleUran=
{
   {  244.197470, 429.863546,    0.0003160,     -0.00000060},
   {  19.21814,   0,             0,             0},
   {  0.0463444,  -0.00002658,   0.000000077,   0},
   {  0.772464,   0.0006253,     0.0000395,     0},
   {  98.071581,  0.9857650,     -0.0010745,    -0.00000061},
   {  73.477111,  0.4986678,     0.0013117,     0}
};



const MEEUS_COEFF_ELE      EleNeptun=
{
   {  84.457994,  219.885914,    0.0003205,     -0.00000060},
   {  30.10957,   0,             0,             0},
   {  0.00899704, 0.000006330,   -0.000000002,  0},
   {  1.779242,   -0.0095436,    -0.0000091,    0},
   {  276.045975, 0.3256394,     -0.00014095,   0.000004113},
   {  130.681389, 1.0989350,     0.00024987,    -0.000004718}
};



/*---------------------------------------------------------------------------

	element for nutation computing

---------------------------------------------------------------------------*/

const MEEUS_COEFF_POLI       SunLong=
{     279.6967,  36000.7689,    0.000303,    0};
const MEEUS_COEFF_POLI       MoonLong=
{     270.4342,  481267.8831,   -0.001133,   0};
const MEEUS_COEFF_POLI       SunAnom=
{     358.4758,  35999.0498,    -0.000150,   0};
const MEEUS_COEFF_POLI       MoonAnom=
{     296.1046,  477198.8491,   0.009192,    0};
const MEEUS_COEFF_POLI       MoonNode=
{     259.1833,  -1934.142,  0.002078,       0};
const MEEUS_COEFF_POLI       Obliquity=
{     23.452294,  -0.0130125,    -0.00000164,   0.000000503};



/*---------------------------------------------------------------------------

	element for mon position computing

---------------------------------------------------------------------------*/

const MEEUS_ELE_MOON          Moon_lambda[]=
{              // M  M1 D  F   e
   {  6.288750,   0, 1, 0, 0,  0},
   {  1.274018,   0,-1, 2, 0,  0},
   {  0.658309,   0, 0, 2, 0,  0},
   {  0.213616,   0, 2, 0, 0,  0},
   { -0.185596,   1, 0, 0, 0,  1},
   { -0.114336,   0, 0, 0, 2,  0},
   {  0.058793,   0,-2, 2, 0,  0},
   {  0.057212,  -1,-1, 2, 0,  1},
   {  0.053320,   0, 1, 2, 0,  0},
   {  0.045874,  -1, 0, 2, 0,  1},
   {  0.041024,  -1, 1, 0, 0,  1},
   { -0.034718,   0, 0, 1, 0,  0},
   { -0.030465,   1, 1, 0, 0,  1},
   {  0.015326,   0, 0, 2,-2,  0},
   { -0.012528,   0, 1, 0, 2,  0},
   { -0.010980,   0,-1, 0, 2,  0},
   {  0.010674,   0,-1, 4, 0,  0},
   {  0.010034,   0, 3, 0, 0,  0},
   {  0.008548,   0,-2, 4, 0,  0},
   { -0.007910,   1,-1, 2, 0,  1},
   { -0.006783,   1, 0, 2, 0,  1},
   {  0.005162,   0, 1,-1, 0,  0},
   {  0.005000,   1, 0, 1, 0,  1},
   {  0.004049,  -1, 1, 2, 0,  1},
   {  0.003996,   0, 2, 2, 0,  0},
   {  0.003862,   0, 0, 4, 0,  0},
   {  0.003665,   0,-3, 2, 0,  0},
   {  0.002695,  -1, 2, 0, 0,  1},
   {  0.002602,   0, 1,-2,-2,  0},
               // M  M1 D  F   e
   {  0.002396,  -1,-2, 2, 0,  1},
   { -0.002349,   0, 1, 1, 0,  0},
   {  0.002249,  -2, 0, 2, 0,  2},
   { -0.002125,   1, 2, 0, 0,  1},
   { -0.002079,   2, 0, 0, 0,  2},
   {  0.002059,  -2,-1, 2, 0,  2},
   { -0.001773,   0, 1, 2,-2,  0},
   { -0.001595,   0, 0, 2, 2,  0},
   {  0.001220,  -1,-1, 4, 0,  1},
   { -0.001110,   0, 2, 0, 2,  0},
   {  0.000892,   0, 1,-3, 0,  0},
   { -0.000811,   1, 1, 2, 0,  1},
   {  0.000761,  -1,-2, 4, 0,  1},
   {  0.000717,  -2, 1, 0, 0,  2},
   {  0.000704,  -2, 1,-2, 0,  2},
   {  0.000693,   1,-2, 2, 0,  1},
   {  0.000598,  -1, 0, 2,-2,  1},
   {  0.000550,   0, 1, 4, 0,  0},
   {  0.000538,   0, 4, 0, 0,  0},
   {  0.000521,  -1, 0, 4, 0,  1},
   {  0.000486,   0, 2,-1, 0,  0},
};
const int Num_Moon_lambda = sizeof(Moon_lambda)/sizeof(MEEUS_ELE_MOON);

const MEEUS_ELE_MOON          Moon_B[]=
{              // M  M1 D  F   e
   {  5.128189,   0, 0, 0, 1,  0},
   {  0.280606,   0, 1, 0, 1,  0},
   {  0.277693,   0, 1, 0,-1,  0},
   {  0.173238,   0, 0, 2,-1,  0},
   {  0.055413,   0,-1, 2, 1,  0},
   {  0.046272,   0,-1, 2,-1,  0},
   {  0.032573,   0, 0, 2, 1,  0},
   {  0.017198,   0, 2, 0, 1,  0},
   {  0.009267,   0, 1, 2,-1,  0},
   {  0.008823,   0, 2, 0,-1,  0},
   {  0.008247,  -1, 0, 2,-1,  1},
   {  0.004323,   0,-2, 2,-1,  0},
   {  0.004200,   0, 1, 2, 1,  0},
   {  0.003372,  -1, 0,-2, 1,  1},
   {  0.002472,  -1,-1, 2, 1,  1},
   {  0.002222,  -1, 0, 2, 1,  1},
   {  0.002072,  -1,-1, 2,-1,  1},
   {  0.001877,  -1, 1, 0, 1,  1},
   {  0.001828,   0,-1, 4,-1,  0},
   { -0.001803,   1, 0, 0, 1,  1},
   { -0.001750,   0, 0, 0, 3,  0},
   {  0.001570,  -1, 1, 0,-1,  1},
   { -0.001487,   0, 0, 1, 1,  0},
   { -0.001481,   1, 1, 0, 1,  1},
   {  0.001417,  -1,-1, 0, 1,  1},
   {  0.001350,  -1, 0, 0, 1,  1},
   {  0.001330,   0, 0,-1, 1,  0},
   {  0.001106,   0, 3, 0, 1,  0},
   {  0.001220,   0, 0, 4,-1,  0},
   {  0.000833,   0,-1, 4, 1,  0},
   {  0.000781,   0, 1, 0,-3,  0},
   {  0.000670,   0,-2, 4, 1,  0},
   {  0.000606,   0, 0, 2,-3,  0},
   {  0.000597,   0, 2, 2,-1,  0},
   {  0.000492,  -1, 1, 2,-1,  1},
   {  0.000450,   0, 2,-2,-1,  0},
   {  0.000439,   0, 3, 0,-1,  0},
   {  0.000423,   0, 2, 2, 1,  0},
   {  0.000422,   0,-3, 2,-1,  0},
   { -0.000367,   1,-1, 2, 1,  1},
   { -0.000353,   1, 0, 2, 1,  1},
   {  0.000331,   0, 0, 4, 1,  0},
   {  0.000317,  -1, 1, 2, 1,  1},
   {  0.000306,  -2, 0, 2,-1,  2},
   { -0.000283,   0, 1, 0, 3,  0},
};
const int Num_Moon_B = sizeof(Moon_B)/sizeof(MEEUS_ELE_MOON);

const MEEUS_ELE_MOON          Moon_pi[]=
{              // M  M1 D  F   e
   {  0.051818,   0, 1, 0, 0,  0},
   {  0.009531,   0,-1, 2, 0,  0},
   {  0.007843,   0, 0, 2, 0,  0},
   {  0.002824,   0, 2, 0, 0,  0},
   {  0.000857,   0, 1, 2, 0,  0},
   {  0.000533,  -1, 0, 2, 0,  1},
   {  0.000401,  -1,-1, 2, 0,  1},
   {  0.000320,  -1, 1, 0, 0,  1},
   { -0.000271,   0, 0, 1, 0,  0},
   { -0.000264,   1, 1, 0, 0,  1},
   { -0.000198,   0,-1, 0, 2,  0},
   {  0.000173,   0, 3, 0, 0,  0},
   {  0.000167,   0,-1, 4, 0,  0},
   { -0.000111,   1, 0, 0, 0,  1},
   {  0.000103,   0,-2, 4, 0,  0},
   { -0.000084,   0, 2,-2, 0,  0},
   { -0.000083,   1, 0, 2, 0,  1},
   {  0.000079,   0, 2, 2, 0,  0},
   {  0.000072,   0, 0, 4, 0,  0},
   {  0.000064,  -1, 1, 2, 0,  1},
   { -0.000063,   1,-1, 2, 0,  1},
   {  0.000041,   1, 0, 1, 0,  1},
   {  0.000035,  -1, 2, 0, 0,  1},
   { -0.000033,   0, 3,-2, 0,  0},
   { -0.000030,   0, 1, 1, 0,  0},
   { -0.000029,   0, 0,-2, 2,  0},
   { -0.000029,   1, 2, 0, 0,  1},
   {  0.000026,  -2, 0, 2, 0,  2},
   { -0.000023,   0, 1,-2, 2,  0},
   {  0.000019,  -1,-1, 4, 0,  1},
};
const int Num_Moon_pi = sizeof(Moon_pi)/sizeof(MEEUS_ELE_MOON);







