/*---------------------------------------------------------------------------

	CMFunctions.h  Header for astronomical functions

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : September 2009

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// antiripetizione include
#ifndef CMFunctionsH
#define CMFunctionsH



// project include
#include "CMCoEcliptic.h"
#include "CMEvent.h"
#include "CMDegree.h"



// definitions
#define RAD                            (180/M_PI)
#define EARTH_RADIS                    (double)6378.140
#define UA                             (double)149597870.691
#define EARTH_DEFORMATION              (double)0.99664719



// meeus polynomial coefficents for one element
typedef struct
{
   double                              a0;                  // constant
   double                              a1;                  // parameter for T
   double                              a2;                  // parameter for T square
   double                              a3;                  // parameter for T cube
} MEEUS_COEFF_POLI;

// meeus polynomial coefficents for all orbital elements
typedef struct
{
   MEEUS_COEFF_POLI                    L;                   // longitude
   MEEUS_COEFF_POLI                    a;                   // semi axis
   MEEUS_COEFF_POLI                    e;                   // eccentricity
   MEEUS_COEFF_POLI                    i;                   // inclination
   MEEUS_COEFF_POLI                    omega;               // perielion argument
   MEEUS_COEFF_POLI                    OMEGA;               // node longitude
} MEEUS_COEFF_ELE;

// orbital elements
typedef struct
{
   double                              pi;                  // perielion longitude
   double                              M;                   // mean anomaly
   double                              L;                   // mean longitude
   double                              a;                   // semi axis
   double                              e;                   // eccentricity
   double                              i;                   // inclination
   double                              omega;               // perielion argument
   double                              OMEGA;               // node longitude
   double                              E;                   // eccentric anomaly
   double                              v;                   // true anomaly
} MEEUS_ELE_ORB;

// elememts for moon computing
typedef struct
{
   double                              Coe;                 // coefficient
   int                                 Mult_M;              // M multiplier
   int                                 Mult_M1;             // M2 multiplier
   int                                 Mult_D;              // D multiplier
   int                                 Mult_F;              // F multiplier
   int                                 Pot_e;               // e pow
} MEEUS_ELE_MOON;

// compute orbital parameters
void                                   CM_OrbitalParamCompute ( double gg, const MEEUS_COEFF_ELE *CoeEleMeeus, MEEUS_ELE_ORB &EleOrb );
// compute position
CMCoEcliptic*                          CM_PositionCompute( MEEUS_ELE_ORB &EleOrb );
// compute Keplero equation
double                                 CM_KepleroEqu( double e, double M );
// reduce in range 0-360 degree
double                                 CM_DegreeReduce( double Val);
// compute paramates with polynomials
double                                 CM_PoliCompute( double T, const MEEUS_COEFF_POLI * Coe );
// obliquity computing
double                                 CM_ObliquityComputing( double gg );
// nutation computing
void                                   CM_NutationComputing( double GG, double *DeltaPhi, double *DeltaEpsilon );
// sideral time computing
CMHours::HOURS                         CM_SideralTimeComputing( double GG, bool Apparente );
// moon position computing
CMCoEcliptic*                          CM_MoonPositionComputing( double gg );
// moon sin series computing
double                                 CM_MoonSinComputing( double M, double M1, double D, double F, double e, const MEEUS_ELE_MOON *Ele, int NumEle );
// moon cos series computing
double                                 CM_MoonCosComputing( double M, double M1, double D, double F, double e, const MEEUS_ELE_MOON *Ele, int NumEle );
// Add Hours
void                                   CM_AddTime( CMHours::HOURS * Time1, CMHours::HOURS * Time2 );
// subctract Hours
void                                   CM_SubTime( CMHours::HOURS * Time1, CMHours::HOURS * Time2 );
// convert Hours in degree
double                                 CM_Convert( CMHours::HOURS *Pos );
// convert degree in Hours
double                                 CM_Convert( CMDegree::DEGREE *Pos );
// Add Declination
void                                   CM_AddDec( CMDegree::DEGREE * Dec1, CMDegree::DEGREE * Dec2 );


extern const MEEUS_COEFF_ELE           EleMercury;
extern const MEEUS_COEFF_ELE           EleVenus;
extern const MEEUS_COEFF_ELE           EleEarth;
extern const MEEUS_COEFF_ELE           EleEarth2000;
extern const MEEUS_COEFF_ELE           EleMars;
extern const MEEUS_COEFF_ELE           EleJupiter;
extern const MEEUS_COEFF_ELE           EleSaturn;
extern const MEEUS_COEFF_ELE           EleNeptun;
extern const MEEUS_COEFF_ELE           EleUran;

extern const MEEUS_COEFF_POLI          SunLong;
extern const MEEUS_COEFF_POLI          MoonLong;
extern const MEEUS_COEFF_POLI          SunAnom;
extern const MEEUS_COEFF_POLI          MoonAnom;
extern const MEEUS_COEFF_POLI          MoonNode;
extern const MEEUS_COEFF_POLI          Obliquity;

extern const MEEUS_ELE_MOON            Moon_lambda[];
extern const int                       Num_Moon_lambda;
extern const MEEUS_ELE_MOON            Moon_B[];
extern const int                       Num_Moon_B;
extern const MEEUS_ELE_MOON            Moon_pi[];
extern const int                       Num_Moon_pi;

#endif
