/*---------------------------------------------------------------------------

	CMHours.cpp  Hours coordinate computing

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : October 2009

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// project include
#include "CMHours.h"



/*---------------------------------------------------------------------------

	object constructor

---------------------------------------------------------------------------*/

CMHours::CMHours(  )
{
   // reset internal data
   Ar.hh=0;
   Ar.mm=0;
   Ar.ss=0;
}



/*---------------------------------------------------------------------------

	object copy constructor

---------------------------------------------------------------------------*/

CMHours::CMHours( const CMHours *Copy )
{
   // copy internal data from external object
   Ar = Copy->Ar;
}



/*---------------------------------------------------------------------------

	set composed coordinate

---------------------------------------------------------------------------*/

void CMHours::Set( const HOURS *Par, bool Neg )
{
   if ( Neg == true )                  // negative paramater?
   {
      // exec complement
      Ar.ss = 60 - Par->ss;
      Ar.mm = 60 - Par->mm - 1;
      Ar.hh = 24 - Par->hh - 1;
      // adjust overflow
      if ( Ar.ss >= 60 )
      {
         Ar.ss -= 60;
         Ar.mm++;
      }
      if ( Ar.mm >= 60 )
      {
         Ar.mm -= 60;
         Ar.hh++;
      }
      if ( Ar.hh >= 24 )
         Ar.hh -= 24;
   }
   else
      Ar = *Par;                                            // copy parameter
}



/*---------------------------------------------------------------------------

	set packed coordinate (in Hours with decimal)

---------------------------------------------------------------------------*/

void CMHours::Set( double Par )
{
   double                              Residual;            // decimal part
   bool                                Neg;                 // negative parameter

   // makes parameter positive and store negative status
   Neg = false;
   if ( Par < 0 )
   {
      Par = - Par;
      Neg = true;
   }

   Ar.hh = Par;                                             // assigns hours
   Residual = Par - Ar.hh;                                  // residual hours
   Residual *= 60;                                          // makes it minutes
   Ar.mm = Residual;                                        // assigns minutes
   Residual = Residual - Ar.mm;                             // residual minutes
   Residual *= 60;                                          // makes it seconds
   Ar.ss = Residual;                                        // assign seconds

   if ( Neg == true )                                       // makes it negative if it needs
      Set( &Ar, true );
}



/*---------------------------------------------------------------------------

	get composed coordinate

---------------------------------------------------------------------------*/

const CMHours::HOURS* CMHours::Get( void) const
{
   return &Ar;
}



/*---------------------------------------------------------------------------

	get packed coordinate

---------------------------------------------------------------------------*/

double CMHours::GetC( void ) const
{
   double                              Hours;               // composed coordinate

   Hours = Ar.ss;                                           // assign seconds
   Hours /= 60;                                             // makes it minutes
   Hours += Ar.mm;                                          // add minutes
   Hours /= 60;                                             // makes it hours
   Hours += Ar.hh;                                          // add hours

   return Hours;
}



/*---------------------------------------------------------------------------

	add object coordinate to the object

---------------------------------------------------------------------------*/

void CMHours::Add( CMHours *Par )
{
   // add external objects coordinate
   Ar.ss += Par->Ar.ss;
   Ar.mm += Par->Ar.mm;
   Ar.hh += Par->Ar.hh;

   // adjust overflow
   if ( Ar.ss >= 60 )
   {
      Ar.ss -= 60;
      Ar.mm += 1;
   }
   if ( Ar.mm >= 60 )
   {
      Ar.mm -= 60;
      Ar.hh += 1;
   }
   if ( Ar.hh >= 24 )
      Ar.hh -= 24;
}



/*---------------------------------------------------------------------------

	add composed coordinate to the object

---------------------------------------------------------------------------*/

void CMHours::Add( CMHours::HOURS *Par )
{
   // add parameter coordinate
   Ar.ss += Par->ss;
   Ar.mm += Par->mm;
   Ar.hh += Par->hh;

   // adjust overflow
   if ( Ar.ss >= 60 )
   {
      Ar.ss -= 60;
      Ar.mm += 1;
   }
   if ( Ar.mm >= 60 )
   {
      Ar.mm -= 60;
      Ar.hh += 1;
   }
   if ( Ar.hh >= 24 )
      Ar.hh -= 24;
}



/*---------------------------------------------------------------------------

	add packed coordinate to the object

---------------------------------------------------------------------------*/

void CMHours::Add( double Par )
{
   double                              CurrentHours;        // corrent packed coordinate

   CurrentHours = GetC();                                   // get current coordinate
   CurrentHours += Par;                                     // add parameter
   Set( CurrentHours );                                     // set new coordinate
}



/*---------------------------------------------------------------------------

	subctract object coordinate from the object

---------------------------------------------------------------------------*/

void CMHours::Sub( CMHours *Par )
{
   // add external object coordinate
   Ar.ss -= Par->Ar.ss;
   Ar.mm -= Par->Ar.mm;
   Ar.hh -= Par->Ar.hh;

   // adjust overflow
   if ( Ar.ss < 0 )
   {
      Ar.ss += 60;
      Ar.mm -= 1;
   }
   if ( Ar.mm < 0 )
   {
      Ar.mm += 60;
      Ar.hh -= 1;
   }
   if ( Ar.hh < 0 )
      Ar.hh += 24;
}



/*---------------------------------------------------------------------------

	subctract composed coordinate from the object

---------------------------------------------------------------------------*/

void CMHours::Sub( CMHours::HOURS *Par )
{
   // add parameter coordinate
   Ar.ss -= Par->ss;
   Ar.mm -= Par->mm;
   Ar.hh -= Par->hh;

   // adjust overflow
   if ( Ar.ss < 0 )
   {
      Ar.ss += 60;
      Ar.mm -= 1;
   }
   if ( Ar.mm < 0 )
   {
      Ar.mm += 60;
      Ar.hh -= 1;
   }
   if ( Ar.hh < 0 )
      Ar.hh += 24;
}



/*---------------------------------------------------------------------------

	subctract packed coordinate from the object

---------------------------------------------------------------------------*/

void CMHours::Sub( double Par )
{
   double                              CurrentHours;        // corrent packed coordinate

   CurrentHours = GetC();                                   // get current coordinate
   CurrentHours -= Par;                                     // subtract parameter
   Set( CurrentHours );                                     // set new coordinate   
}

