/*---------------------------------------------------------------------------

	CMMPlanet.cpp  Minor planet computing object

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : September 2009

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// system include
#include <string.h>



// project include
#include "CMMPlanet.h"
#include "CMCoRectangolar.h"




/*---------------------------------------------------------------------------

	constructor

---------------------------------------------------------------------------*/

CMMPlanet::CMMPlanet() :
CMObject()
{
   Nutation = false;                                        // default computing
   PlanetView = NULL;                                       // null view planet
   // create position objects
   PosizioneEclReale = new CMCoEcliptic();
   PosizioneEclFK5 = new CMCoEcliptic();
   PosizioneEclVSOP = new CMCoEcliptic();
   RealRetPosition = new CMCoRectangolar;
   PosizioneRetView = new CMCoRectangolar;
}



/*---------------------------------------------------------------------------

	copy constructor

---------------------------------------------------------------------------*/

CMMPlanet::CMMPlanet( CMMPlanet *Copy ) :
CMObject( Copy )
{
   EleOrb = Copy->EleOrb;
   PlanetView = 0;
   Nutation = Copy->Nutation;
   PosizioneEclReale = new CMCoEcliptic( Copy->PosizioneEclReale );
   PosizioneEclFK5 = new CMCoEcliptic( Copy->PosizioneEclFK5 );
   PosizioneEclVSOP = new CMCoEcliptic( Copy->PosizioneEclVSOP );
   RealRetPosition = new CMCoRectangolar( Copy->RealRetPosition );
   PosizioneRetView = new CMCoRectangolar( Copy->PosizioneRetView );
   if ( Copy->PlanetView != 0 )
      PlanetView = new CMPlanet( Copy->PlanetView );
}



/*---------------------------------------------------------------------------

	destroyer

---------------------------------------------------------------------------*/

CMMPlanet::~CMMPlanet(  )
{
   delete PosizioneEclReale;
   delete PosizioneEclFK5;
   delete PosizioneEclVSOP;
   delete RealRetPosition;
   delete PosizioneRetView;
   if ( PlanetView != 0 )
      delete PlanetView;
}



/*---------------------------------------------------------------------------

	set planet of view

---------------------------------------------------------------------------*/

int CMMPlanet::SetPlanetView( CMPlanet *Planet )
{
   PlanetView = Planet;
   PlanetView->SetOpt(CMPlanet::MEEUS, false, false);
   return 0;
}



/*---------------------------------------------------------------------------

	set date

---------------------------------------------------------------------------*/

void CMMPlanet::SetData( double PData )
{
   CMObject::SetData( PData );

   // set planet view also
   if ( PlanetView != 0 )
      PlanetView->SetData( PData );
   return;
}



/*---------------------------------------------------------------------------

	set computing option

---------------------------------------------------------------------------*/

void CMMPlanet::SetOpt( bool ParNutation, bool ParTimeLight )
{
   Nutation = ParNutation;
   TimeLight = ParTimeLight;
   Dirty = true;
}



/*---------------------------------------------------------------------------

	set orbital elements

---------------------------------------------------------------------------*/

void CMMPlanet::SetEleOrb( double ParJEph, MEEUS_ELE_ORB *ParEleOrb, double ParM0, double ParN  )
{
   JEph = ParJEph;
   EleOrb = *ParEleOrb;
   M0 = ParM0;
   N = ParN;
}



/*---------------------------------------------------------------------------

	get orbital elements

---------------------------------------------------------------------------*/

const MEEUS_ELE_ORB* CMMPlanet::GetEleOrb( void )
{
   return &EleOrb;
}



/*---------------------------------------------------------------------------

	compute position

---------------------------------------------------------------------------*/

void CMMPlanet::ComputePosition( void )
{
   double                              gg;                  // event julian day
   double                              Days;                // day after ephemerid epoch
   double                              c;                   // temp
   CMCoEcliptic::ECL_COO               Ecl;                 // ecliptic position value
   double                              Corr;                // time light corretion
   double                              SunDistance;         // sun distance
   double                              ViewDistance;        // point of view distance
   double                              Rid;

   // compute day after ephemerid epoch
   gg = Date;
   Days = Date - JEph;

   // compute orbilal parameters
   CalcParamOrb( Days, EleOrb );

   // compute ecliptic coordinate with Meeus
   delete PosizioneEclReale;
   PosizioneEclReale = CM_PositionCompute(EleOrb);
   // get eliocentric rectangolar coordinate
   delete RealRetPosition;
   RealRetPosition = PosizioneEclReale->GetCoRet();

   // compute apparent rectangolar coordinate
   delete PosizioneRetView;
   PosizioneRetView = new CMCoRectangolar( RealRetPosition);
   if ( PlanetView != 0 )
   {
      PlanetView->SetData(gg);
      PlanetView->ComputePosition();
      PosizioneRetView->Sub(PlanetView->RealRetPosition);
   }

   // execute time light correction
   if ( TimeLight == true )
   {
      // compute distance
      c = PosizioneRetView->GetDist();
      Corr = c * 0.00577551830441213;
      Days -= Corr;
      gg -= Corr;

      CalcParamOrb( Days, EleOrb );

      // compute ecliptic coordinate with Meeus
      delete PosizioneEclReale;
      PosizioneEclReale = CM_PositionCompute(EleOrb);
      // get eliocentric rectangolar coordinate
      delete RealRetPosition;
      RealRetPosition = PosizioneEclReale->GetCoRet();

      // compute apparent rectangolar coordinate
      delete PosizioneRetView;
      PosizioneRetView = new CMCoRectangolar( RealRetPosition);
      if ( PlanetView != 0 )
      {
         PlanetView->SetData(gg);
         PlanetView->ComputePosition();
         PosizioneRetView->Sub(PlanetView->RealRetPosition);
      }
   }

   // compute magnitude
   ViewDistance = PosizioneRetView->GetDist();
   Ecl = *PosizioneEclReale->Get();
   SunDistance = Ecl.d;
   Rid = pow( ViewDistance, 2 ) * pow(SunDistance, 2);
   c = log10( Rid ) * 2.5;
   c += MagBase;
   c *= 100;
   c = floor( c );
   c /= 100;

   // ecliptic position
   {
      delete PosizioneEclFK5;
      PosizioneEclFK5 = PosizioneRetView->GetCoEcl();
   }


   // get equatorial position at J2000
   delete Position;
   Position = PosizioneEclFK5->GetCoEqu(2451544.5, false) ;

   return;
}



/*---------------------------------------------------------------------------

	compute orbital parameters

---------------------------------------------------------------------------*/

void CMMPlanet::CalcParamOrb( double Days, MEEUS_ELE_ORB &EleOrb )
{
   double                              c;                   // temo

   // compute base orbital parameters
   EleOrb.M = M0 + Days * N;
   EleOrb.M = CM_DegreeReduce( EleOrb.M );
   EleOrb.omega = EleOrb.pi - EleOrb.OMEGA;
   EleOrb.omega = CM_DegreeReduce( EleOrb.omega );
   EleOrb.L = EleOrb.M + EleOrb.pi;
   EleOrb.L = CM_DegreeReduce( EleOrb.L );

   // compute keplero equation
   EleOrb.E = CM_KepleroEqu( EleOrb.e, EleOrb.M );

   // compute true anomaly
   c = tan( EleOrb.E/ RAD /2 ) * sqrt( (1+EleOrb.e)/(1-EleOrb.e) );
   c = atan( c );
   c = c * RAD;
   if ( (EleOrb.E/2) > 90 && (EleOrb.E) < 270 )
      c += 180;
   if ( c < 0 )
      c += 360;
   c *= 2;
   if ( c > 360 )
      c -= 360;
   EleOrb.v = c;
}



/*---------------------------------------------------------------------------

	set base magnitude

---------------------------------------------------------------------------*/

void CMMPlanet::SetMagBase( double PMagBase )
{
   MagBase = PMagBase;
}



/*---------------------------------------------------------------------------

	get magnitude

---------------------------------------------------------------------------*/

float CMMPlanet::GetMag( void )
{
   return Mag;
}


