/*---------------------------------------------------------------------------

	CMPlanet.cpp Planet computing object

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : September 2009

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// system include
#include <string.h>



// project include
#include "CMPlanet.h"
#include "CMCoRectangolar.h"




/*---------------------------------------------------------------------------

	Object constructor

---------------------------------------------------------------------------*/

CMPlanet::CMPlanet(PLANET ParPlanet) :
CMObject()
{
   Mode = MEEUS;                                            // defult computing
   Nutation = false;
   PlanetView = NULL;                                       // nul planet of view
   Planet = ParPlanet;                                      // save planet code

   // create position objects
   RealEclPos = new CMCoEcliptic();
   FK5EclPos = new CMCoEcliptic();
   VSOPEclPos = new CMCoEcliptic();
   RealRetPosition = new CMCoRectangolar;
   ApparentRetPos = new CMCoRectangolar;

   // initialize planet parameters
   switch (Planet)
   {
      case SUN:
         SetName("Sole", CMObject::NAME_OWN);
         CoeEleMeeus = 0;
         Series = 0;
         break;
      case MERCURY:
         SetName("Mercurio", CMObject::NAME_OWN);
         CoeEleMeeus = &EleMercury;
         Series = &VSOP_Mercury;
         break;
      case VENUS:
         SetName("Venere", CMObject::NAME_OWN);
         CoeEleMeeus = &EleVenus;
         Series = &VSOP_Venus;
         break;
      case EARTH:
         SetName("Terra", CMObject::NAME_OWN);
         CoeEleMeeus = &EleEarth;
         Series = &VSOP_Earth;
         break;
      case EARTH2000:
         SetName("Terra", CMObject::NAME_OWN);
         CoeEleMeeus = &EleEarth2000;
         Series = &VSOP_Earth;
         break;
      case MARS:
         SetName("Marte", CMObject::NAME_OWN);
         CoeEleMeeus = &EleMars;
         Series = &VSOP_Mars;
         break;
      case JUPITER:
         SetName("Giove", CMObject::NAME_OWN);
         CoeEleMeeus = &EleJupiter;
         Series = &VSOP_Jupiter;
         break;
      case SATURN:
         SetName("Saturno", CMObject::NAME_OWN);
         CoeEleMeeus = &EleSaturn;
         Series = &VSOP_Saturn;
         break;
      case URAN:
         SetName("Urano", CMObject::NAME_OWN);
         CoeEleMeeus = &EleUran;
         Series = &VSOP_Uran;
         break;
      case NEPTUN:
         SetName("Nettuno", CMObject::NAME_OWN);
         CoeEleMeeus = &EleNeptun;
         Series = &VSOP_Neptun;
         break;
   }
}



/*---------------------------------------------------------------------------

	Object copy constructor

---------------------------------------------------------------------------*/

CMPlanet::CMPlanet( CMPlanet *Copy ) :
CMObject( Copy )
{
   CoeEleMeeus = Copy->CoeEleMeeus;
   EleOrb = Copy->EleOrb;
   Planet = Copy->Planet;
   PlanetView = 0;
   Mode = Copy->Mode;
   Nutation = Copy->Nutation;
   RealEclPos = new CMCoEcliptic( Copy->RealEclPos );
   FK5EclPos = new CMCoEcliptic( Copy->FK5EclPos );
   VSOPEclPos = new CMCoEcliptic( Copy->VSOPEclPos );
   RealRetPosition = new CMCoRectangolar( Copy->RealRetPosition );
   ApparentRetPos = new CMCoRectangolar( Copy->ApparentRetPos );
   if ( Copy->PlanetView != 0 )
      PlanetView = new CMPlanet( Copy->PlanetView );
}



/*---------------------------------------------------------------------------

	destroyer

---------------------------------------------------------------------------*/

CMPlanet::~CMPlanet(  )
{
   delete RealEclPos;
   delete FK5EclPos;
   delete VSOPEclPos;
   delete RealRetPosition;
   delete ApparentRetPos;
   if ( PlanetView != 0 )
      delete PlanetView;
}



/*---------------------------------------------------------------------------

	set efemerid date

---------------------------------------------------------------------------*/

void CMPlanet::SetData( double PData )
{
   CMObject::SetData( PData );

   // set planet of view also
   if ( PlanetView != 0 )
      PlanetView->SetData( PData );
   return;
}



/*---------------------------------------------------------------------------

	set planet of view

---------------------------------------------------------------------------*/

int CMPlanet::SetPlanetView( CMPlanet *Planet )
{
   PlanetView = Planet;
   return 0;
}



/*---------------------------------------------------------------------------

	set computing options

---------------------------------------------------------------------------*/

void CMPlanet::SetOpt( COMP_MODE ParMode, bool ParNutation, bool ParTimeLight )
{
   Mode = ParMode;
   Nutation = ParNutation;
   TimeLight = ParTimeLight;
   Dirty = true;

   if ( PlanetView != 0 )
      PlanetView->SetOpt(ParMode, ParNutation, false);
}



/*---------------------------------------------------------------------------

	get orbital elemets

---------------------------------------------------------------------------*/

const MEEUS_ELE_ORB* CMPlanet::GetEleOrb( void )
{
   return &EleOrb;
}


/*---------------------------------------------------------------------------

	Calcola la posizione

---------------------------------------------------------------------------*/

void CMPlanet::ComputePosition( void )
{
   double                              gg;                  // event julianday
   double                              T;                   // Julian centiries from 1900
   double                              c;                   // temp
   CMCoEcliptic::ECL_COO               Ecl;                 // ecliptic ccoordinate value
   double                              QL,QB;               // VSOP to FK5 corrections
   double                              Corr;                // time light correction
   CMCoRectangolar::RET_COO            Ret;                 // rectangolar coordinate value

   // compute Julian centiries from 1900
   gg = Date;
   T = (gg - 2415020)/36525;

   // clear orbital paramaters
   memset( &EleOrb, 0, sizeof(EleOrb) );

   // sun has coorinate 0
   if ( Planet == SUN )
   {
      delete RealRetPosition;
      RealRetPosition = new CMCoRectangolar();
      Ret.X = 0;
      Ret.Y = 0;;
      Ret.Z = 0;
      RealRetPosition->Set(&Ret);
   }
   else
   {
      // compute orbital parameter
      CM_OrbitalParamCompute( gg, CoeEleMeeus, EleOrb );

      // compute rectangolar coordinate with Meeus
      if ( Mode == MEEUS && CoeEleMeeus != 0 )
      {
         // compute ecliptic coordinate
         delete RealEclPos;
         RealEclPos = CM_PositionCompute(EleOrb);
         // get eliocentric rectangolar coordinate
         delete RealRetPosition;
         RealRetPosition = RealEclPos->GetCoRet();
      }

      // compute rectangolar coordinate with VSOP
      if ( Mode == VSOP )
      {
         delete RealRetPosition;
         RealRetPosition = CM_VSOP_ComputePosition(gg, Series  );
      }
   }

   // compute apparent rectangolar coordinate from planet of view
   delete ApparentRetPos;
   ApparentRetPos = new CMCoRectangolar( RealRetPosition);
   if ( PlanetView != 0 )
   {
      PlanetView->ComputePosition();
      ApparentRetPos->Sub(PlanetView->RealRetPosition);
   }

   // applay time light correction
   if ( TimeLight == true )
   {
      // compute distance and time light
      c = ApparentRetPos->GetDist();
      Corr = c * 0.00577551830441213;
      gg -= Corr;

      // compute only if it is not sub
      if ( Planet != SUN  )
      {
         // compute orbital parameter
         CM_OrbitalParamCompute( gg, CoeEleMeeus, EleOrb );

         // compute rectangolar coordinate with Meeus
         if ( Mode == MEEUS )
         {
            // compute ecliptic coordinate
            delete RealEclPos;
            RealEclPos = CM_PositionCompute(EleOrb);
            // get eliocentric rectangolar coordinate
            delete RealRetPosition;
            RealRetPosition = RealEclPos->GetCoRet();
         }

         // compute rectangolar coordinate with VSOP
         if ( Mode == VSOP )
         {
            delete RealRetPosition;
            RealRetPosition = CM_VSOP_ComputePosition(gg, Series  );
         }
      }

      // compute apparent rectangolar coordinate from planet of view
      delete ApparentRetPos;
      ApparentRetPos = new CMCoRectangolar( RealRetPosition);
      if ( PlanetView != 0 )
      {
         PlanetView->SetData(gg);
         PlanetView->ComputePosition();
         ApparentRetPos->Sub(PlanetView->RealRetPosition);
      }
   }

   // get ecliptic coordinate
   if ( Mode == MEEUS )
   {
      delete FK5EclPos;
      FK5EclPos = ApparentRetPos->GetCoEcl();
   }

   // applay correction from VSOP to FK5
   if ( Mode == VSOP )
   {
      // get ecliptic coordinate
      delete VSOPEclPos;
      delete FK5EclPos;
      VSOPEclPos = ApparentRetPos->GetCoEcl();
      FK5EclPos = new CMCoEcliptic( VSOPEclPos );

      // compute correction
      Ecl = *VSOPEclPos->Get();
      T = (gg - 2451545) / 36525;
      Ecl.l -= 1.397 * T + 0.00031 * T * T;

      QL = -0.09033 + 0.03916 *( cos(Ecl.l/RAD) + sin(Ecl.l/RAD) ) * tan(Ecl.b/RAD);
      QL /= 3600;

      QB = 0.03916 * ( cos(Ecl.l/RAD) - sin(Ecl.l/RAD));
      QB /= 3600;
      FK5EclPos->Add( QL, QB );
   }

   // get equatorial position
   delete Position;
   Position = FK5EclPos->GetCoEqu(gg, Nutation) ;

   return;
}


