/*---------------------------------------------------------------------------

	VSOP.cpp VSOP elemement compunting

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : October 2009

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// system include
#include <math.h>



// project include
#include "CMCoRectangolar.h"
#include "CMVSOP.h"
#include "CMFunctions.h"



/*---------------------------------------------------------------------------

   compute rectangolar VSOP coordinate

---------------------------------------------------------------------------*/


CMCoRectangolar* CM_VSOP_ComputePosition(double jd, VSOP_Object *Object )
{
   double                              t;                      // time in julian centuries from 2000
   double                              l;                      // longitudie
   double                              b;                      // latitude
   double                              r;                      // vector radius
   int                                 i;                      // series index
   double                              T;                      // time power
   CMCoEcliptic::ECL_COO               Ecl;                    // ecliptic position value
   CMCoEcliptic                        *EclPos;                // ecliptic position object
   CMCoRectangolar                     *RetPos;                // rectangolar position object

   // compute time in julian centuries from 2000
   t = (jd - 2451545.0) / 365250.0;

   // compute longitude series
   T = 1;
   l = 0;
   for (i = 0; i < 6; i++)
   {
      l += CM_VSOP_AddSeries( &Object->SerieL[i], t) * T;
      T = t * T;
   }

   // compute latitude series
   T = 1;
   b = 0;
   for (i = 0; i < 6; i++)
   {
      b += CM_VSOP_AddSeries( &Object->SerieB[i] , t) * T;
      T = t * T;
   }

   // compute vector radius series
   T = 1;
   r = 0;
   for (i = 0; i < 6; i++)
   {
      r += CM_VSOP_AddSeries( &Object->SerieR[i], t) * T;
      T = t * T;
   }

   // goto 0-360 degree
   l *= RAD;
   b *= RAD;
   l = CM_DegreeReduce( l );

   // compute rectangolar coordinate
   Ecl.l = l;
   Ecl.b = b;
   Ecl.d = r;
   EclPos = new CMCoEcliptic();
   EclPos->Set(&Ecl);
   RetPos = EclPos->GetCoRet();
   delete EclPos;

   return  RetPos;
}



/*---------------------------------------------------------------------------

	sum a VSOP serie

---------------------------------------------------------------------------*/

double CM_VSOP_AddSeries(const VSOP_Series *Serie, double t )
{
   double                              x;                   // sum
   VSOP_Tern                           *term;               // tern pointer
   int                                 i;                   // series scan index

   if (Serie->TernNum < 1)
      return 0;

   x = 0;
   term = &Serie->Terns[0];
   for (i = 0; i < Serie->TernNum; i++, term++)
      x += term->A * cos(term->B + term->C * t);

   return x;
}



/*---------------------------------------------------------------------------

	Mercury elements

---------------------------------------------------------------------------*/

static VSOP_Tern Mercury_L0[] =
{
   { 4.40250710144,0.00000000000,0.00000000000 },
};

static VSOP_Tern Mercury_L1[] =
{
   { 26088.14706222746,0.00000000000,0.00000000000 },
};

static VSOP_Tern Mercury_L2[] =
{
   { 0.00053049845,0.00000000000,0.00000000000 },
};

static VSOP_Tern Mercury_L3[] =
{
   { 0.00000188077,0.03466830117,52175.80628314840 },
};

static VSOP_Tern Mercury_L4[] =
{
   { 0.00000188077,0.03466830117,52175.80628314840 },
};

static VSOP_Tern Mercury_L5[] =
{
   { 0.00000000877,3.14159265359,0.00000000000 },
};

static VSOP_Tern Mercury_B0[] =
{
   { 0.11737528962,1.98357498767,26087.90314157420 },
};

static VSOP_Tern Mercury_B1[] =
{
   { 0.00429151362,3.50169780393,26087.90314157420 },
};

static VSOP_Tern Mercury_B2[] =
{
   { 0.00011830934,4.79065585784,26087.90314157420 },
};

static VSOP_Tern Mercury_B3[] =
{
   { 0.00000235423,0.35387524604,26087.90314157420 },
};

static VSOP_Tern Mercury_B4[] =
{
   { 0.00000004276,1.74579932115,26087.90314157420 },
};

static VSOP_Tern Mercury_B5[] =
{
   { 0.00000000106,3.94555784256,26087.90314157420 },
};

static VSOP_Tern Mercury_R0[] =
{
   { 0.39528271652,0.00000000000,0.00000000000 },
};

static VSOP_Tern Mercury_R1[] =
{
   { 0.00217347739,4.65617158663,26087.90314157420 },
};

static VSOP_Tern Mercury_R2[] =
{
   { 0.00003117867,3.08231840296,26087.90314157420 },
};

static VSOP_Tern Mercury_R3[] =
{
   { 0.00000032676,1.67971635359,26087.90314157420 },
};

static VSOP_Tern Mercury_R4[] =
{
   { 0.00000000394,0.36735403840,26087.90314157420 },
};

static VSOP_Tern Mercury_R5[] =
{
   { 0.00000000006,3.98900269603,26087.90314157420 },
};

VSOP_Series Mercury_L[] =
{
   {  Mercury_L0, ARRAY_SIZE(Mercury_L0)  },
   {  Mercury_L1, ARRAY_SIZE(Mercury_L1)  },
   {  Mercury_L2, ARRAY_SIZE(Mercury_L2)  },
   {  Mercury_L3, ARRAY_SIZE(Mercury_L3)  },
   {  Mercury_L4, ARRAY_SIZE(Mercury_L4)  },
   {  Mercury_L5, ARRAY_SIZE(Mercury_L5)  },
};

VSOP_Series Mercury_B[] =
{
   {  Mercury_B0, ARRAY_SIZE(Mercury_B0)  },
   {  Mercury_B1, ARRAY_SIZE(Mercury_B1)  },
   {  Mercury_B2, ARRAY_SIZE(Mercury_B2)  },
   {  Mercury_B3, ARRAY_SIZE(Mercury_B3)  },
   {  Mercury_B4, ARRAY_SIZE(Mercury_B4)  },
   {  Mercury_B5, ARRAY_SIZE(Mercury_B5)  },
};

VSOP_Series Mercury_R[] =
{
   {  Mercury_R0, ARRAY_SIZE(Mercury_R0)  },
   {  Mercury_R1, ARRAY_SIZE(Mercury_R1)  },
   {  Mercury_R2, ARRAY_SIZE(Mercury_R2)  },
   {  Mercury_R3, ARRAY_SIZE(Mercury_R3)  },
   {  Mercury_R4, ARRAY_SIZE(Mercury_R4)  },
   {  Mercury_R5, ARRAY_SIZE(Mercury_R5)  },
};

VSOP_Object VSOP_Mercury =
{
   Mercury_L,
   Mercury_B,
   Mercury_R
};


/*---------------------------------------------------------------------------

	Venus elements

---------------------------------------------------------------------------*/

static VSOP_Tern Venus_L0[] =
{
   { 3.17614666774,0.00000000000,0.00000000000 },
};

static VSOP_Tern Venus_L1[] =
{
   { 10213.52943052898,0.00000000000,0.00000000000 },
};

static VSOP_Tern Venus_L2[] =
{
   { 0.00054127076,0.00000000000,0.00000000000 },
};

static VSOP_Tern Venus_L3[] =
{
   { 0.00000135742,4.80389020993,10213.28554621100 },
};

static VSOP_Tern Venus_L4[] =
{
   { 0.00000114016,3.14159265359,0.00000000000 },
};

static VSOP_Tern Venus_L5[] =
{
   { 0.00000000874,3.14159265359,0.00000000000 },
};

static VSOP_Tern Venus_B0[] =
{
   { 0.05923638472,0.26702775813,10213.28554621100 },
};

static VSOP_Tern Venus_B1[] =
{
   { 0.00513347602,1.80364310797,10213.28554621100 },
};

static VSOP_Tern Venus_B2[] =
{
   { 0.00022377665,3.38509143877,10213.28554621100 },
};

static VSOP_Tern Venus_B3[] =
{
   { 0.00000646671,4.99166565277,10213.28554621100 },
};

static VSOP_Tern Venus_B4[] =
{
   { 0.00000014102,0.31537190181,10213.28554621100 },
};

static VSOP_Tern Venus_B5[] =
{
   { 0.00000000239,2.05201727566,10213.28554621100 },
};

static VSOP_Tern Venus_R0[] =
{
   { 0.72334820905,0.00000000000,0.00000000000 },
};

static VSOP_Tern Venus_R1[] =
{
   { 0.00034551039,0.89198710598,10213.28554621100 },
};

static VSOP_Tern Venus_R2[] =
{
   { 0.00001406587,5.06366395190,10213.28554621100 },
};

static VSOP_Tern Venus_R3[] =
{
   { 0.00000049582,3.22263554520,10213.28554621100 },
};

static VSOP_Tern Venus_R4[] =
{
   { 0.00000000573,0.92229697820,10213.28554621100 },
};

static VSOP_Tern Venus_R5[] =
{
   { 0.00000000045,0.30032866722,10213.28554621100 },
};


VSOP_Series venus_L[] =
{
   {  Venus_L0, ARRAY_SIZE(Venus_L0)  },
   {  Venus_L1, ARRAY_SIZE(Venus_L1)  },
   {  Venus_L2, ARRAY_SIZE(Venus_L2)  },
   {  Venus_L3, ARRAY_SIZE(Venus_L3)  },
   {  Venus_L4, ARRAY_SIZE(Venus_L4)  },
   {  Venus_L5, ARRAY_SIZE(Venus_L5)  },
};

VSOP_Series venus_B[] =
{
   {  Venus_B0, ARRAY_SIZE(Venus_B0)  },
   {  Venus_B1, ARRAY_SIZE(Venus_B1)  },
   {  Venus_B2, ARRAY_SIZE(Venus_B2)  },
   {  Venus_B3, ARRAY_SIZE(Venus_B3)  },
   {  Venus_B4, ARRAY_SIZE(Venus_B4)  },
   {  Venus_B5, ARRAY_SIZE(Venus_B5)  },
};

VSOP_Series venus_R[] =
{
   {  Venus_R0, ARRAY_SIZE(Venus_R0)  },
   {  Venus_R1, ARRAY_SIZE(Venus_R1)  },
   {  Venus_R2, ARRAY_SIZE(Venus_R2)  },
   {  Venus_R3, ARRAY_SIZE(Venus_R3)  },
   {  Venus_R4, ARRAY_SIZE(Venus_R4)  },
   {  Venus_R5, ARRAY_SIZE(Venus_R5)  },
};

VSOP_Object VSOP_Venus =
{
   venus_L,
   venus_B,
   venus_R
};



/*---------------------------------------------------------------------------

	Earth elemets

---------------------------------------------------------------------------*/

static VSOP_Tern Earth_L0[] =
{
   { 1.75347045673,0.00000000000,0.00000000000 },
};

static VSOP_Tern Earth_L1[] =
{
   { 6283.31966747491,0.00000000000,0.00000000000 },
};

static VSOP_Tern Earth_L2[] =
{
   { 0.00052918870,0.00000000000,0.00000000000 },
};

static VSOP_Tern Earth_L3[] =
{
   { 0.00052918870,0.00000000000,0.00000000000 },
};

static VSOP_Tern Earth_L4[] =
{
   { 0.00000114084,3.14159265359,0.00000000000 },
};

static VSOP_Tern Earth_L5[] =
{
   { 0.00000000878,3.14159265359,0.00000000000 },
};

static VSOP_Tern Earth_B0[] =
{
   { 0.00000279620,3.19870156017,84334.66158130829 },
};

static VSOP_Tern Earth_B1[] =
{
   { 0.00000009030,3.89729061890,5507.55323866740 },
};

static VSOP_Tern Earth_B2[] =
{
   { 0.00000001662,1.62703209173,84334.66158130829 },
};

static VSOP_Tern Earth_B3[] =
{
   { 0.00000000011,0.23877262399,7860.41939243920 },
};

static VSOP_Tern Earth_B4[] =
{
   { 0.00000000005,0.84308705203,1047.74731175470 },
};

static VSOP_Tern Earth_B5[] =
{
   { 0.00000000000,0.00000000000,0.00000000000 },
};

static VSOP_Tern Earth_R0[] =
{
   { 1.00013988799,0.00000000000,0.00000000000 },
};

static VSOP_Tern Earth_R1[] =
{
   { 0.00103018608,1.10748969588,6283.07584999140 },
};

static VSOP_Tern Earth_R2[] =
{
   { 0.00004359385,5.78455133738,6283.07584999140 },
};

static VSOP_Tern Earth_R3[] =
{
   { 0.00000144595,4.27319435148,6283.07584999140 },
};

static VSOP_Tern Earth_R4[] =
{
   { 0.00000003858,2.56384387339,6283.07584999140 },
};

static VSOP_Tern Earth_R5[] =
{
   { 0.00000000086,1.21579741687,6283.07584999140 },
};



VSOP_Series earth_L[] =
{
   {  Earth_L0, ARRAY_SIZE(Earth_L0)  },
   {  Earth_L1, ARRAY_SIZE(Earth_L1)  },
   {  Earth_L2, ARRAY_SIZE(Earth_L2)  },
   {  Earth_L3, ARRAY_SIZE(Earth_L3)  },
   {  Earth_L4, ARRAY_SIZE(Earth_L4)  },
   {  Earth_L5, ARRAY_SIZE(Earth_L5)  },
};

VSOP_Series earth_B[] =
{
   {  Earth_B0, ARRAY_SIZE(Earth_B0)  },
   {  Earth_B1, ARRAY_SIZE(Earth_B1)  },
   {  Earth_B2, ARRAY_SIZE(Earth_B2)  },
   {  Earth_B3, ARRAY_SIZE(Earth_B3)  },
   {  Earth_B4, ARRAY_SIZE(Earth_B4)  },
   {  Earth_B5, ARRAY_SIZE(Earth_B5)  },
};

VSOP_Series earth_R[] =
{
   {  Earth_R0, ARRAY_SIZE(Earth_R0)  },
   {  Earth_R1, ARRAY_SIZE(Earth_R1)  },
   {  Earth_R2, ARRAY_SIZE(Earth_R2)  },
   {  Earth_R3, ARRAY_SIZE(Earth_R3)  },
   {  Earth_R4, ARRAY_SIZE(Earth_R4)  },
   {  Earth_R5, ARRAY_SIZE(Earth_R5)  },
};

VSOP_Object VSOP_Earth =
{
   earth_L,
   earth_B,
   earth_R
};



/*---------------------------------------------------------------------------

	Mars elements

---------------------------------------------------------------------------*/

static VSOP_Tern Mars_L0[] =
{
   { 6.20347711583,0.00000000000,0.00000000000 },
};

static VSOP_Tern Mars_L1[] =
{
   { 3340.85627474342,0.00000000000,0.00000000000 },
};

static VSOP_Tern Mars_L2[] =
{
   { 0.00058015791,2.04979463279,3340.61242669980 },
};

static VSOP_Tern Mars_L3[] =
{
   { 0.00001482423,0.44434694876,3340.61242669980 },
};

static VSOP_Tern Mars_L4[] =
{
   { 0.00000113969,3.14159265359,0.00000000000 },
};

static VSOP_Tern Mars_L5[] =
{
   { 0.00000000868,3.14159265359,0.00000000000 },
};

static VSOP_Tern Mars_B0[] =
{
   { 0.03197134986,3.76832042432,3340.61242669980 },
};

static VSOP_Tern Mars_B1[] =
{
   { 0.00350068845,5.36847836211,3340.61242669980 },
};

static VSOP_Tern Mars_B2[] =
{
   { 0.00016726690,0.60221392419,3340.61242669980 },
};

static VSOP_Tern Mars_B3[] =
{
   { 0.00000606506,1.98050633529,3340.61242669980 },
};

static VSOP_Tern Mars_B4[] =
{
   { 0.00000013369,0.00000000000,0.00000000000 },
};

static VSOP_Tern Mars_B5[] =
{
   { 0.00000000457,4.86794125358,3340.61242669980 },
};

static VSOP_Tern Mars_R0[] =
{
   { 1.53033488276,0.00000000000,0.00000000000 },
};

static VSOP_Tern Mars_R1[] =
{
   { 0.01107433340,2.03250524950,3340.61242669980 },
};

static VSOP_Tern Mars_R2[] =
{
   { 0.00044242247,0.47930603943,3340.61242669980 },
};

static VSOP_Tern Mars_R3[] =
{
   { 0.00001113107,5.14987350142,3340.61242669980 },
};

static VSOP_Tern Mars_R4[] =
{
   { 0.00000019552,3.58211650473,3340.61242669980 },
};

static VSOP_Tern Mars_R5[] =
{
   { 0.00000000476,2.47617204701,6681.22485339960 },
};

VSOP_Series Mars_L[] =
{
   {  Mars_L0, ARRAY_SIZE(Mars_L0)  },
   {  Mars_L1, ARRAY_SIZE(Mars_L1)  },
   {  Mars_L2, ARRAY_SIZE(Mars_L2)  },
   {  Mars_L3, ARRAY_SIZE(Mars_L3)  },
   {  Mars_L4, ARRAY_SIZE(Mars_L4)  },
   {  Mars_L5, ARRAY_SIZE(Mars_L5)  },
};

VSOP_Series Mars_B[] =
{
   {  Mars_B0, ARRAY_SIZE(Mars_B0)  },
   {  Mars_B1, ARRAY_SIZE(Mars_B1)  },
   {  Mars_B2, ARRAY_SIZE(Mars_B2)  },
   {  Mars_B3, ARRAY_SIZE(Mars_B3)  },
   {  Mars_B4, ARRAY_SIZE(Mars_B4)  },
   {  Mars_B5, ARRAY_SIZE(Mars_B5)  },
};

VSOP_Series Mars_R[] =
{
   {  Mars_R0, ARRAY_SIZE(Mars_R0)  },
   {  Mars_R1, ARRAY_SIZE(Mars_R1)  },
   {  Mars_R2, ARRAY_SIZE(Mars_R2)  },
   {  Mars_R3, ARRAY_SIZE(Mars_R3)  },
   {  Mars_R4, ARRAY_SIZE(Mars_R4)  },
   {  Mars_R5, ARRAY_SIZE(Mars_R5)  },
};

VSOP_Object VSOP_Mars =
{
   Mars_L,
   Mars_B,
   Mars_R
};



/*---------------------------------------------------------------------------

	Jupiter elements

---------------------------------------------------------------------------*/

static VSOP_Tern Jupiter_L0[] =
{
   { 0.59954691495,0.00000000000,0.00000000000 },
};

static VSOP_Tern Jupiter_L1[] =
{
   { 529.93480757497,0.00000000000,0.00000000000 },
};

static VSOP_Tern Jupiter_L2[] =
{
   { 0.00047233598,4.32148323554,7.11354700080 },
};

static VSOP_Tern Jupiter_L3[] =
{
   { 0.00006501665,2.59862880482,7.11354700080 },
};

static VSOP_Tern Jupiter_L4[] =
{
   { 0.00000669483,0.85282421090,7.11354700080 },
};

static VSOP_Tern Jupiter_L5[] =
{
   { 0.00000049577,5.25658966184,7.11354700080 },
};

static VSOP_Tern Jupiter_B0[] =
{
   { 0.02268615703,3.55852606718,529.69096509460 },
};

static VSOP_Tern Jupiter_B1[] =
{
   { 0.00177351787,5.70166488486,529.69096509460 },
};

static VSOP_Tern Jupiter_B2[] =
{
   { 0.00008094051,1.46322843658,529.69096509460 },
};

static VSOP_Tern Jupiter_B3[] =
{
   { 0.00000251624,3.38087923084,529.69096509460 },
};

static VSOP_Tern Jupiter_B4[] =
{
   { 0.00000015050,4.52956999637,522.57741809380 },
};

static VSOP_Tern Jupiter_B5[] =
{
   { 0.00000001445,0.09198554072,522.57741809380 },
};

static VSOP_Tern Jupiter_R0[] =
{
   { 5.20887429471,0.00000000000,0.00000000000 },
};

static VSOP_Tern Jupiter_R1[] =
{
   { 0.01271801596,2.64937511122,529.69096509460 },
};

static VSOP_Tern Jupiter_R2[] =
{
   { 0.00079644833,1.35865896596,529.69096509460 },
};

static VSOP_Tern Jupiter_R3[] =
{
   { 0.00003519257,6.05800633846,529.69096509460 },
};

static VSOP_Tern Jupiter_R4[] =
{
   { 0.00000128628,0.08419309557,536.80451209540 },
};

static VSOP_Tern Jupiter_R5[] =
{
   { 0.00000011188,4.75249399945,536.80451209540 },
};

VSOP_Series Jupiter_L[] =
{
   {  Jupiter_L0, ARRAY_SIZE(Jupiter_L0)  },
   {  Jupiter_L1, ARRAY_SIZE(Jupiter_L1)  },
   {  Jupiter_L2, ARRAY_SIZE(Jupiter_L2)  },
   {  Jupiter_L3, ARRAY_SIZE(Jupiter_L3)  },
   {  Jupiter_L4, ARRAY_SIZE(Jupiter_L4)  },
   {  Jupiter_L5, ARRAY_SIZE(Jupiter_L5)  },
};

VSOP_Series Jupiter_B[] =
{
   {  Jupiter_B0, ARRAY_SIZE(Jupiter_B0)  },
   {  Jupiter_B1, ARRAY_SIZE(Jupiter_B1)  },
   {  Jupiter_B2, ARRAY_SIZE(Jupiter_B2)  },
   {  Jupiter_B3, ARRAY_SIZE(Jupiter_B3)  },
   {  Jupiter_B4, ARRAY_SIZE(Jupiter_B4)  },
   {  Jupiter_B5, ARRAY_SIZE(Jupiter_B5)  },
};

VSOP_Series Jupiter_R[] =
{
   {  Jupiter_R0, ARRAY_SIZE(Jupiter_R0)  },
   {  Jupiter_R1, ARRAY_SIZE(Jupiter_R1)  },
   {  Jupiter_R2, ARRAY_SIZE(Jupiter_R2)  },
   {  Jupiter_R3, ARRAY_SIZE(Jupiter_R3)  },
   {  Jupiter_R4, ARRAY_SIZE(Jupiter_R4)  },
   {  Jupiter_R5, ARRAY_SIZE(Jupiter_R5)  },
};

VSOP_Object VSOP_Jupiter =
{
   Jupiter_L,
   Jupiter_B,
   Jupiter_R
};



/*---------------------------------------------------------------------------

	Saturn elements

---------------------------------------------------------------------------*/

static VSOP_Tern Saturn_L0[] =
{
   { 0.87401354029,0.00000000000,0.00000000000 },
};

static VSOP_Tern Saturn_L1[] =
{
   { 213.54295595986,0.00000000000,0.00000000000 },
};

static VSOP_Tern Saturn_L2[] =
{
   { 0.00116441181,1.17987850633,7.11354700080 },
};

static VSOP_Tern Saturn_L3[] =
{
   { 0.00016038734,5.73945377424,7.11354700080 },
};

static VSOP_Tern Saturn_L4[] =
{
   { 0.00001661894,3.99826248978,7.11354700080 },
};

static VSOP_Tern Saturn_L5[] =
{
   { 0.00000123615,2.25923345732,7.11354700080 },
};

static VSOP_Tern Saturn_B0[] =
{
   { 0.04330678040,3.60284428399,213.29909543800 },
};

static VSOP_Tern Saturn_B1[] =
{
   { 0.00397554998,5.33289992556,213.29909543800 },
};

static VSOP_Tern Saturn_B2[] =
{
   { 0.00020629977,0.50482422817,213.29909543800 },
};

static VSOP_Tern Saturn_B3[] =
{
   { 0.00000666252,1.99006340181,213.29909543800 },
};

static VSOP_Tern Saturn_B4[] =
{
   { 0.00000080384,1.11918414679,206.18554843720 },
};

static VSOP_Tern Saturn_B5[] =
{
   { 0.00000007895,2.81927558645,206.18554843720 },
};

static VSOP_Tern Saturn_R0[] =
{
   { 9.55758135801,0.00000000000,0.00000000000 },
};

static VSOP_Tern Saturn_R1[] =
{
   { 0.06182981282,0.25843515034,213.29909543800 },
};

static VSOP_Tern Saturn_R2[] =
{
   { 0.00436902464,4.78671673044,213.29909543800 },
};

static VSOP_Tern Saturn_R3[] =
{
   { 0.00020315005,3.02186626038,213.29909543800 },
};

static VSOP_Tern Saturn_R4[] =
{
   { 0.00001202050,1.41499446465,220.41264243880 },
};

static VSOP_Tern Saturn_R5[] =
{
   { 0.00000128612,5.91282565136,220.41264243880 },
};

VSOP_Series Saturn_L[] =
{
   {  Saturn_L0, ARRAY_SIZE(Saturn_L0)  },
   {  Saturn_L1, ARRAY_SIZE(Saturn_L1)  },
   {  Saturn_L2, ARRAY_SIZE(Saturn_L2)  },
   {  Saturn_L3, ARRAY_SIZE(Saturn_L3)  },
   {  Saturn_L4, ARRAY_SIZE(Saturn_L4)  },
   {  Saturn_L5, ARRAY_SIZE(Saturn_L5)  },
};

VSOP_Series Saturn_B[] =
{
   {  Saturn_B0, ARRAY_SIZE(Saturn_B0)  },
   {  Saturn_B1, ARRAY_SIZE(Saturn_B1)  },
   {  Saturn_B2, ARRAY_SIZE(Saturn_B2)  },
   {  Saturn_B3, ARRAY_SIZE(Saturn_B3)  },
   {  Saturn_B4, ARRAY_SIZE(Saturn_B4)  },
   {  Saturn_B5, ARRAY_SIZE(Saturn_B5)  },
};

VSOP_Series Saturn_R[] =
{
   {  Saturn_R0, ARRAY_SIZE(Saturn_R0)  },
   {  Saturn_R1, ARRAY_SIZE(Saturn_R1)  },
   {  Saturn_R2, ARRAY_SIZE(Saturn_R2)  },
   {  Saturn_R3, ARRAY_SIZE(Saturn_R3)  },
   {  Saturn_R4, ARRAY_SIZE(Saturn_R4)  },
   {  Saturn_R5, ARRAY_SIZE(Saturn_R5)  },
};

VSOP_Object VSOP_Saturn =
{
   Saturn_L,
   Saturn_B,
   Saturn_R
};



/*---------------------------------------------------------------------------

	Uran elements

---------------------------------------------------------------------------*/

static VSOP_Tern Uran_L0[] =
{
   { 5.48129294299,0.00000000000,0.00000000000 },
};

static VSOP_Tern Uran_L1[] =
{
   { 75.02543121646,0.00000000000,0.00000000000 },
};

static VSOP_Tern Uran_L2[] =
{
   { 0.00053033277,0.00000000000,0.00000000000 },
};

static VSOP_Tern Uran_L3[] =
{
   { 0.00000120936,0.02418789918,74.78159856730 },
};

static VSOP_Tern Uran_L4[] =
{
   { 0.00000113855,3.14159265359,0.00000000000 },
};

static VSOP_Tern Uran_L5[] =
{
   { 0.00000000873,3.14159265359,0.00000000000 },
};

static VSOP_Tern Uran_B0[] =
{
   { 0.01346277639,2.61877810545,74.78159856730 },
};

static VSOP_Tern Uran_B1[] =
{
   { 0.00206366162,4.12394311407,74.78159856730 },
};

static VSOP_Tern Uran_B2[] =
{
   { 0.00009211656,5.80044305785,74.78159856730 },
};

static VSOP_Tern Uran_B3[] =
{
   { 0.00000267832,1.25097888291,74.78159856730 },
};

static VSOP_Tern Uran_B4[] =
{
   { 0.00000005719,2.85499529315,74.78159856730 },
};

static VSOP_Tern Uran_B5[] =
{
   { 0.00000000000,0.00000000000,0.00000000000 },
};

static VSOP_Tern Uran_R0[] =
{
   { 19.21264847881,0.00000000000,0.00000000000 },
};

static VSOP_Tern Uran_R1[] =
{
   { 0.01479896370,3.67205705317,74.78159856730 },
};

static VSOP_Tern Uran_R2[] =
{
   { 0.00022439904,0.69953118760,74.78159856730 },
};

static VSOP_Tern Uran_R3[] =
{
   { 0.00001164382,4.73453291602,74.78159856730 },
};

static VSOP_Tern Uran_R4[] =
{
   { 0.00000052996,3.00838033088,74.78159856730 },
};

static VSOP_Tern Uran_R5[] =
{
   { 0.00000000000,0.00000000000,0.000000000000},
};

VSOP_Series Uran_L[] =
{
   {  Uran_L0, ARRAY_SIZE(Uran_L0)  },
   {  Uran_L1, ARRAY_SIZE(Uran_L1)  },
   {  Uran_L2, ARRAY_SIZE(Uran_L2)  },
   {  Uran_L3, ARRAY_SIZE(Uran_L3)  },
   {  Uran_L4, ARRAY_SIZE(Uran_L4)  },
   {  Uran_L5, ARRAY_SIZE(Uran_L5)  },
};

VSOP_Series Uran_B[] =
{
   {  Uran_B0, ARRAY_SIZE(Uran_B0)  },
   {  Uran_B1, ARRAY_SIZE(Uran_B1)  },
   {  Uran_B2, ARRAY_SIZE(Uran_B2)  },
   {  Uran_B3, ARRAY_SIZE(Uran_B3)  },
   {  Uran_B4, ARRAY_SIZE(Uran_B4)  },
   {  Uran_B5, ARRAY_SIZE(Uran_B5)  },
};

VSOP_Series Uran_R[] =
{
   {  Uran_R0, ARRAY_SIZE(Uran_R0)  },
   {  Uran_R1, ARRAY_SIZE(Uran_R1)  },
   {  Uran_R2, ARRAY_SIZE(Uran_R2)  },
   {  Uran_R3, ARRAY_SIZE(Uran_R3)  },
   {  Uran_R4, ARRAY_SIZE(Uran_R4)  },
   {  Uran_R5, ARRAY_SIZE(Uran_R5)  },
};

VSOP_Object VSOP_Uran =
{
   Uran_L,
   Uran_B,
   Uran_R
};


/*---------------------------------------------------------------------------

	Neptun elements

---------------------------------------------------------------------------*/

static VSOP_Tern Neptun_L0[] =
{
   { 5.31188633047,0.00000000000,0.00000000000 },
};

static VSOP_Tern Neptun_L1[] =
{
   { 38.37687716731,0.00000000000,0.00000000000 },
};

static VSOP_Tern Neptun_L2[] =
{
   { 0.00053892649,0.00000000000,0.00000000000 },
};

static VSOP_Tern Neptun_L3[] =
{
   { 0.00000031254,0.00000000000,0.00000000000 },
};

static VSOP_Tern Neptun_L4[] =
{
   { 0.00000113998,3.14159265359,0.00000000000 },
};

static VSOP_Tern Neptun_L5[] =
{
   { 0.00000000874,3.14159265359,0.00000000000 },
};

static VSOP_Tern Neptun_B0[] =
{
   { 0.03088622933,1.44104372626,38.13303563780 },
};

static VSOP_Tern Neptun_B1[] =
{
   { 0.00227279214,3.80793089870,38.13303563780 },
};

static VSOP_Tern Neptun_B2[] =
{
   { 0.00009690766,5.57123750291,38.13303563780 },
};

static VSOP_Tern Neptun_B3[] =
{
   { 0.00000273423,1.01688979072,38.13303563780 },
};

static VSOP_Tern Neptun_B4[] =
{
   { 0.00000005728,2.66872693322,38.13303563780 },
};

static VSOP_Tern Neptun_B5[] =
{
   { 0.00000000113,4.70646877989,38.13303563780 },
};

static VSOP_Tern Neptun_R0[] =
{
   { 30.07013206102,0.00000000000,0.00000000000 },
};

static VSOP_Tern Neptun_R1[] =
{
   { 0.00236338502,0.70498011235,38.13303563780 },
};

static VSOP_Tern Neptun_R2[] =
{
   { 0.00004247412,5.89910679117,38.13303563780 },
};

static VSOP_Tern Neptun_R3[] =
{
   { 0.00000166297,4.55243893489,38.13303563780 },
};

static VSOP_Tern Neptun_R4[] =
{
   { 0.00000004333,0.10459484545,395.57870223900 },
};

static VSOP_Tern Neptun_R5[] =
{
   { 0.00000000000,0.00000000000,0.000000000000},
};

VSOP_Series Neptun_L[] =
{
   {  Neptun_L0, ARRAY_SIZE(Neptun_L0)  },
   {  Neptun_L1, ARRAY_SIZE(Neptun_L1)  },
   {  Neptun_L2, ARRAY_SIZE(Neptun_L2)  },
   {  Neptun_L3, ARRAY_SIZE(Neptun_L3)  },
   {  Neptun_L4, ARRAY_SIZE(Neptun_L4)  },
   {  Neptun_L5, ARRAY_SIZE(Neptun_L5)  },
};

VSOP_Series Neptun_B[] =
{
   {  Neptun_B0, ARRAY_SIZE(Neptun_B0)  },
   {  Neptun_B1, ARRAY_SIZE(Neptun_B1)  },
   {  Neptun_B2, ARRAY_SIZE(Neptun_B2)  },
   {  Neptun_B3, ARRAY_SIZE(Neptun_B3)  },
   {  Neptun_B4, ARRAY_SIZE(Neptun_B4)  },
   {  Neptun_B5, ARRAY_SIZE(Neptun_B5)  },
};

VSOP_Series Neptun_R[] =
{
   {  Neptun_R0, ARRAY_SIZE(Neptun_R0)  },
   {  Neptun_R1, ARRAY_SIZE(Neptun_R1)  },
   {  Neptun_R2, ARRAY_SIZE(Neptun_R2)  },
   {  Neptun_R3, ARRAY_SIZE(Neptun_R3)  },
   {  Neptun_R4, ARRAY_SIZE(Neptun_R4)  },
   {  Neptun_R5, ARRAY_SIZE(Neptun_R5)  },
};

VSOP_Object VSOP_Neptun =
{
   Neptun_L,
   Neptun_B,
   Neptun_R
};




