/******************************************************************************

	CCD.cpp | CCD manager

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : April 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// System include
#include <math.h>
#include <math.hpp>
#include <dir.h>



// project include
#include "CCD.h"
#pragma package(smart_init)



/****************************************************************************

   object constructor

*****************************************************************************/

CCD::CCD()
{
   int                                 i;                // index
   int                                 Size;             // struct size
   DWORD		                           Start;            // start time for server waiting
   DWORD                               Now;              // actual time for server wainting
   char									      Com[MAXPATH];     // command for server run
   char									      Par[MAXPATH];     // command line parameter for server run
   STARTUPINFO							      SInfo;            // info parameter for server run
   PROCESS_INFORMATION				      PInfo;            // process retur parameter for server run

   // initialize object data
   DpRam = 0;
   InternalImage = 0;
   InternalImageW = 0;
   InternalBmp = 0;
   InternalZoomBmp = 0;
   InternalHistogramBmp = 0;
   memset( &InternalInfo, 0, sizeof(InternalInfo));
   InternalInfo.State = CCD_STATE_CLOSED;
   InternalInfo.HImageSum = 0;
   InternalInfo.VImageSum = 0;

   // erase palette definition
   Size = sizeof( GrayPalette );
   memset ( &GrayPalette, 0, Size );
   Size = sizeof( GrayZoomPalette );
   memset ( &GrayZoomPalette, 0, Size );

   // gray palette building for bitmap
   GrayPalette.PalHeader.palVersion = 0x300;
   GrayPalette.PalHeader.palNumEntries = GRAY_PALETTE_ENTRY;
   for ( i=0; i<GRAY_LEVELS; i++ )
   {
      GrayPalette.PalHeader.palPalEntry[i].peRed = (BYTE)i;
      GrayPalette.PalHeader.palPalEntry[i].peGreen = (BYTE)i;
      GrayPalette.PalHeader.palPalEntry[i].peBlue = (BYTE)i;
      GrayPalette.PalHeader.palPalEntry[i].peFlags = 0;
   }

   // add an extracolor (yellow) for cicle arrond the star
   GrayPalette.PalHeader.palPalEntry[COLOR_STARCICLE].peRed = 255;
   GrayPalette.PalHeader.palPalEntry[COLOR_STARCICLE].peGreen = 255;
   GrayPalette.PalHeader.palPalEntry[COLOR_STARCICLE].peBlue = 0;
   GrayPalette.PalHeader.palPalEntry[COLOR_STARCICLE].peFlags = 0;

   Hp = CreatePalette((const tagLOGPALETTE *)&GrayPalette);

   // gray palette building for zoomed bitmap
   GrayZoomPalette.PalHeader.palVersion = 0x300;
   GrayZoomPalette.PalHeader.palNumEntries = GRAY_PALETTE_ENTRY;
   for ( i=0; i<(GRAY_LEVELS); i++ )
   {
      GrayZoomPalette.PalHeader.palPalEntry[i].peRed = (BYTE)i;
      GrayZoomPalette.PalHeader.palPalEntry[i].peGreen = (BYTE)i;
      GrayZoomPalette.PalHeader.palPalEntry[i].peBlue = (BYTE)i;
      GrayZoomPalette.PalHeader.palPalEntry[i].peFlags = 0;
   }
   // add an extracolor (Brown) for out border indication
   GrayZoomPalette.PalHeader.palPalEntry[COLOR_OUTBORDER].peRed = 128;
   GrayZoomPalette.PalHeader.palPalEntry[COLOR_OUTBORDER].peGreen = 64;
   GrayZoomPalette.PalHeader.palPalEntry[COLOR_OUTBORDER].peBlue = 0;
   GrayZoomPalette.PalHeader.palPalEntry[COLOR_OUTBORDER].peFlags = 0;

   // add an extracolor (yellow) for cicle arrond the star
   GrayPalette.PalHeader.palPalEntry[COLOR_STARCICLE].peRed = 255;
   GrayPalette.PalHeader.palPalEntry[COLOR_STARCICLE].peGreen = 255;
   GrayPalette.PalHeader.palPalEntry[COLOR_STARCICLE].peBlue = 0;
   GrayPalette.PalHeader.palPalEntry[COLOR_STARCICLE].peFlags = 0;
   // add an extracolor (red) for border & C indication
   GrayZoomPalette.PalHeader.palPalEntry[COLOR_BORDER].peRed = 255;
   GrayZoomPalette.PalHeader.palPalEntry[COLOR_BORDER].peGreen = 0;
   GrayZoomPalette.PalHeader.palPalEntry[COLOR_BORDER].peBlue = 0;
   GrayZoomPalette.PalHeader.palPalEntry[COLOR_BORDER].peFlags = 0;

   HpZoom = CreatePalette((const tagLOGPALETTE *)&GrayZoomPalette);

   // create histogram bitmap
   InternalHistogramBmp = new Graphics::TBitmap();          // create bitmap for histogram
   InternalHistogramBmp->Width = 250;                       // assign size to histogram bitmap
   InternalHistogramBmp->Height = 140;

   // check mapped file existence
 	HDpRam = OpenFileMapping( FILE_MAP_WRITE, false, FILENAME_CCD_SERVER );

   // if mapped file is not present it need ccd server run
   if ( HDpRam == NULL )
   {
      strcpy( Com, "CCDServer.exe" );                       // command
      strcpy( Par, Com );                                   // command line parameter
      memset( &SInfo, 0,  sizeof( SInfo ) );                // clear run information
      SInfo.cb = sizeof( SInfo );                           // lenght at th head
      SInfo.wShowWindow = SW_HIDE;                          // server run hide
   	CreateProcess( Com, Par, NULL, NULL, false,           // run server
                     NORMAL_PRIORITY_CLASS,
                     NULL, NULL, &SInfo, &PInfo );
   }

	Start = GetTickCount();                                  // store actual time
   Now = Start;
   while ( HDpRam == NULL && (Now-Start) < SERVER_TIMEOUT ) // wait until timeout or mapped file open
   {
      Sleep( 500 );                                         // in order to don't charge CPU in the while
		// tries to open mapped file
   	HDpRam = OpenFileMapping( FILE_MAP_WRITE, false, FILENAME_CCD_SERVER );
		Now = GetTickCount();                                 // get corrent time
   }

   // open view in the mapped file
	if ( HDpRam != NULL )
   	DpRam = (DP_CCD_SERVER*)MapViewOfFile(HDpRam, FILE_MAP_WRITE, 0, 0, 0);
}



/****************************************************************************

   Object destroyer

*****************************************************************************/

CCD::~CCD()
{
   Close();                                                 // close ccd device if open
   if ( InternalImage != 0 )                                // free buffer for internal image
      free ( InternalImage );
   if ( InternalImageW != 0 )                               // free buffer for internal image for working
      free ( InternalImageW );
   if ( InternalBmp != 0 )                                  // free bitmap
      delete InternalBmp;
   if ( InternalZoomBmp != 0 )                              // free zoom bitmap
      delete InternalZoomBmp;
   if ( InternalHistogramBmp != 0 )                         // free zoom bitmap
      delete InternalHistogramBmp;
   if ( InternalInfo.VImageSum != 0 )
      delete InternalInfo.VImageSum;
   if ( InternalInfo.HImageSum != 0 )
      delete InternalInfo.HImageSum;

   if ( DpRam != 0 )
   {
      DpRam->Command = DP_COM_EXIT;
      UnmapViewOfFile( DpRam );
   }

   if ( HDpRam != 0 )
      CloseHandle( HDpRam );
}



/****************************************************************************

   Open CCD device

*****************************************************************************/

CCD::ERR CCD::Open( HANDLE FormHandle, char *Name, char *ParImageFolder )
{
   // only if opennig was correct
   if ( DpRam == 0 )
      return ERR_CCDSERVERNOTRUNNING;

   // run open command on ccd server
   strcpy( DpRam->CCDName, Name );                          // assign CCD name
   DpRam->Command = DP_COM_OPEN;                            // assign command

   // wait until command is executed
   while ( DpRam->Command != DP_COM_NULL )
   {
      Sleep ( 100 );                                        // in order to don't charge CPU in the while
   }

   // create bitmaps
   InternalBmp = new Graphics::TBitmap();                   // create bitmap for image
   InternalBmp->PixelFormat = pf8bit;                       // bitmap with 8 bit pixel format
   InternalBmp->Palette = Hp;                               // assign gray palette to bitmap
   InternalZoomBmp = new Graphics::TBitmap();               // create bitmap for zoomed image
   InternalZoomBmp->PixelFormat = pf8bit;                   // bitmap with 8 bit pixel format
   InternalZoomBmp->Palette = HpZoom;                       // assign gray palette to bitmap
   InternalZoomBmp->Width = PIXEL_ZOOMED*ZOOM_FACTOR;       // assign size to zoomed bitmap
   InternalZoomBmp->Height = PIXEL_ZOOMED*ZOOM_FACTOR;

   // set ccd state
   InternalInfo.State = CCD_STATE_IDLE;

   // create image name
   strcpy(ImageName, ParImageFolder);
   strcat(ImageName, CCD_RAWFILE);

  	return ERR_NONE;
}



/****************************************************************************

   Close CCD device

*****************************************************************************/

CCD::ERR CCD::Close(  )
{
   // only if opennig was correct
   if ( DpRam == 0 )
      return ERR_CCDSERVERNOTRUNNING;

   // run close command on ccd server
   DpRam->Command = DP_COM_CLOSE;

   // wait until command is executed
   while ( DpRam->Command != DP_COM_NULL )
   {
      Sleep ( 100 );                                        // in order to don't charge CPU in the while
   }

   // set ccd state
   InternalInfo.State = CCD_STATE_CLOSED;

  	return ERR_NONE;
}



/****************************************************************************

   Take a shot  (time in millisecond)

*****************************************************************************/

CCD::ERR CCD::Shot( int Time )
{
   // only if opennig was correct
   if ( DpRam == 0 )
      return ERR_CCDSERVERNOTRUNNING;
   if ( DpRam->Opened == false )
      return ERR_DEVICENOTOPEN;

   // if ccd state is not IDLE and is not IMAGE it is an error
   if ( InternalInfo.State != CCD_STATE_IDLE && InternalInfo.State != CCD_STATE_IMAGE )
      return ERR_WRONGSTATE;

   // store initial time and exposure time
   GetSystemTime ( &InternalInfo.InitShotTime );
   InternalInfo.Exposure = Time;
   InternalInfo.Exposure /= 1000;

   // run shot command on ccd server
   DpRam->ExposureTime = Time;                              // assign time in millisecond
   DpRam->ExposureTime /= 1000;                             // assign time in second
   DpRam->Command = DP_COM_SHOT;                            // assign command

   // store initial time
   Start = GetTickCount();

   // wait until command is executed
   while ( DpRam->Command != DP_COM_NULL )
   {
      Sleep ( 100 );                                        // in order to don't charge CPU in the while
   }

   // set ccd state
   InternalInfo.State = CCD_STATE_EXPOSING;

  	return ERR_NONE;
}



/****************************************************************************

   Abort short in exposing

*****************************************************************************/

CCD::ERR CCD::AbortShot( void )
{
   // only if opennig was correct
   if ( DpRam == 0 )
      return ERR_CCDSERVERNOTRUNNING;
   if ( DpRam->Opened == false )
      return ERR_DEVICENOTOPEN;

   // if ccd state is not EXPOSING it is an error
   if ( InternalInfo.State != CCD_STATE_EXPOSING )
      return ERR_WRONGSTATE;

   // run abort command on ccd server
   DpRam->Command = DP_COM_ABORT;                           // assign command

   // store initial time
   Start = GetTickCount();

   // wait until command is executed
   while ( DpRam->Command != DP_COM_NULL )
   {
      Sleep ( 100 );                                        // in order to don't charge CPU in the while
   }

   InternalInfo.State = CCD_STATE_IDLE;

  	return ERR_NONE;
}



/****************************************************************************

   Check shot is finished

*****************************************************************************/

CCD::ERR CCD::IsShotFinisched( int *Time )
{
   ERR                                 Err;                 // error code

   // only if opennig was correct
   if ( DpRam == 0 )
      return ERR_CCDSERVERNOTRUNNING;
   if ( DpRam->Opened == false )
      return ERR_DEVICENOTOPEN;

   // if ccd state is not EXPOSING it is an error
   if ( InternalInfo.State != CCD_STATE_EXPOSING )
      return ERR_WRONGSTATE;

   Err = ERR_NONE;                                          // default is finished

   *Time = DpRam->ExposingTime*1000;                        // update exposing time
   if ( DpRam->ExposingStatus == DP_STATUS_EXPOSING )       // if server says not finished ..
      Err = ERR_NOTFINISHED;                                // return not finisch

  	return Err;
}



/****************************************************************************

   Get shot image

*****************************************************************************/

CCD::ERR CCD::GetShot( void )
{
   HANDLE                              hFile;               // file handle
   DWORD                               ToRead;              // byte to read
   DWORD                               Read;                // byte read
   int                                 IndexPixel;          // current pixel
   unsigned short                      *pLine;              // line image pointer
   unsigned short                      PixelValue;          // value of the current pixel
   int                                 IndexHisto;          // current histogram element
   int                                 MaxHistoValue;       // maximum value in histogram
   int                                 MaxHistoIndex;       // index of vaimum value in histogram
   double                              Sum;                 // sum for sigma computing
   double                              Diff;                // difference for sigma computing
   double                              Deno;                // denominator for sigma computing
   double                              Sigma;               // sigma
   unsigned short                      HighestValue;        // Highest Value in the image
   int                                 XHigh;               // x of Highest pixel
   int                                 YHigh;               // y of highest pixel
   double                              XStar;               // x centroid star position
   double                              YStar;               // y centroid star position
   double                              FWHMStar;            // Full width at half maximum star
   int                                 Ground;              // Ground near star
   double                              FWHMSum;             // sum for FWHM mean computing
   int                                 FWHMNum;             // number of star used in FWHM computing
   int                                 XScan;               // scanning column
   int                                 YScan;               // scanning raw
   int                                 PixelValue2;         // pixel value in integer
   int                                 PixelSum;            // pixel sum

   // only if opennig was correct
   if ( DpRam == 0 )
      return ERR_CCDSERVERNOTRUNNING;
   if ( DpRam->Opened == false )
      return ERR_DEVICENOTOPEN;

   InternalInfo.Width = DpRam->Width;                       // get image size from server
   InternalInfo.Height = DpRam->Height;
   ToRead = InternalInfo.Width*InternalInfo.Height*2;       // image file size
   if ( InternalImage == 0 )
      InternalImage = (unsigned short*)malloc( ToRead);     // memory allocation for internal image
   if ( InternalImageW == 0 )
      InternalImageW = (unsigned short*)malloc( ToRead);    // memory allocation for internal image for working
   if ( InternalInfo.HImageSum == 0 )
      InternalInfo.HImageSum = (int*)malloc( InternalInfo.Height * 4); // memory allocation for horizontal image sum
   if ( InternalInfo.VImageSum == 0 )
      InternalInfo.VImageSum = (int*)malloc( InternalInfo.Width * 4);  // memory allocation for vertical image sum

   // load image from file with fixed name
   hFile = CreateFile( ImageName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL );
   if ( hFile != INVALID_HANDLE_VALUE )
   {
      ReadFile ( hFile, InternalImage, ToRead, &Read, NULL);// read in one shot
      CloseHandle( hFile);                                  // close file
   }

   // set ccd state
   InternalInfo.State = CCD_STATE_IMAGE;

   // scan image in order to find minimum, maximum, mean and build histogram
   InternalInfo.MinValue = 65535;                           // minimum starts at maximum value
   InternalInfo.MaxValue = 0;                               // maximum starts at minimum value
   pLine = InternalImage;                                   // point to image
   for ( IndexHisto=0; IndexHisto<VALUE_RANGE; IndexHisto++)// scan all elements in histogram
      InternalInfo.Histogram[IndexHisto] = 0;               // erase element
   for ( IndexPixel=0; IndexPixel<(InternalInfo.Height*InternalInfo.Width); IndexPixel++ ) // scan all pixel in the image
   {
      PixelValue = *pLine++;                                // get pixel value

      if ( PixelValue > InternalInfo.MaxValue )             // if current pixel value is greater than current maximim
         InternalInfo.MaxValue = PixelValue;                // update current maximum
      if ( PixelValue < InternalInfo.MinValue )             // if current pixel value is less than current minimum
         InternalInfo.MinValue = PixelValue;                // update current minimum

      InternalInfo.Histogram[PixelValue] ++;                // increment pixel number having this value
   }

   // findout the maximum value in historam
   MaxHistoValue=0;                                         // reset value
   MaxHistoIndex=0;
   for ( IndexHisto=0; IndexHisto<VALUE_RANGE; IndexHisto++)// scan all elements in histogram
   {
      if ( InternalInfo.Histogram[IndexHisto] > MaxHistoValue ) // new value greater than the until now maximum ?
      {
         MaxHistoValue = InternalInfo.Histogram[IndexHisto];// store new maximum
         MaxHistoIndex = IndexHisto;                        // store its index
      }
   }
   InternalInfo.BackgrondLevel = MaxHistoIndex;                // background level is the index of maximum in histogram

   // compute background sigma
   Sum = 0;
   Deno = 0;
   for ( IndexHisto=0; IndexHisto<MaxHistoIndex; IndexHisto++)// scan elements in histogram from 0 to maximum value index
   {
      Diff = IndexHisto - MaxHistoIndex;                    // difference (error) between mean value and pixel value
      Diff *= Diff;                                         // square difference
      Diff *= InternalInfo.Histogram[IndexHisto]*2;         // weigh this point with histogram hight
      Sum += Diff;                                          // sum the
      Deno += InternalInfo.Histogram[IndexHisto]*2;         // number of point
   }
   if ( Deno == 0 )
      Sigma = 0;
   else
      Sigma = Sum / Deno;                                   // variance
   Sigma = pow( Sigma, 0.5 );                               // final sigma
   InternalInfo.BackgroundSigma = Sigma;

   // compute image sigma
   Sum = 0;
   Deno = 0;
   for ( IndexHisto=0; IndexHisto<VALUE_RANGE; IndexHisto++)// scan elements in histogram from 0 to maximum value index
   {
      Diff = IndexHisto - MaxHistoIndex;                    // difference (error) between image mean value and pixel value
      Diff *= Diff;                                         // square difference
      Diff *= InternalInfo.Histogram[IndexHisto];           // weigh this point with histogram hight
      Sum += Diff;                                          // sum the
      Deno += InternalInfo.Histogram[IndexHisto];           // number of point
   }
   Sigma = Sum / Deno;                                      // variance
   Sigma = pow( Sigma, 0.5 );                               // final sigma
   InternalInfo.ImgageSigma = Sigma;

   if ( InternalInfo.BackgroundSigma == 0 )
      InternalInfo.VisiblityIndex = 0;
   else
      InternalInfo.VisiblityIndex = (double)InternalInfo.ImgageSigma / (double)InternalInfo.BackgroundSigma; // compute visibility index

   XHighest = -1;

   // compute FWHM
   memcpy( InternalImageW, InternalImage, ToRead );         // copy image in another buffer to mangle it
   FWHMSum = 0;                                             // reset value for mean computing
   FWHMNum = 0;
   while ( FWHMNum < STAR_FWHM)                             // repeat for 10 brightest stars
   {
      HighestValue = SearchHighestPixel( &XHigh, &YHigh, 0, 1 );  // get highest value  in the image

      if ( HighestValue < InternalInfo.BackgrondLevel * 3 || HighestValue < 10) // it should be significant greater than background level
         break;
      XStar = XHigh;                                        // copy coordinate in double format
      YStar = YHigh;
      if ( XHigh < RADIUS_STAR || XHigh > (InternalInfo.Width - RADIUS_STAR) ||  // it shouldn't be near border
           YHigh < RADIUS_STAR || YHigh > (InternalInfo.Height - RADIUS_STAR)  )
      {
         RemoveStar( XStar, YStar, (unsigned short)InternalInfo.BackgrondLevel ); // remove star from image
         continue;
      }
      GetStarParam( &XStar, &YStar, &FWHMStar, &Ground );   // get real position, FWHM and ground near star
      if ( XHighest == -1 )                                 // in the first loop
      {
         XHighest = XStar;                                  // save the highest value position
         YHighest = YStar;
      }
      RemoveStar( XStar, YStar, (unsigned short) Ground );  // remove star from image
      if (Ground == 0 )
         continue;
      FWHMSum += FWHMStar;                                  // sum FWHM for mean comouting
      FWHMNum++;                                            // sum star number for mean comouting
   }
   if ( FWHMNum == 0 )
      InternalInfo.FWHM = 0;
   else
      InternalInfo.FWHM = FWHMSum / FWHMNum;                // compute FWHM mean

   // compute horizontal sum
   for ( YScan=0; YScan<InternalInfo.Height; YScan++ )      // scan all lines
   {
      PixelSum = 0;                                         // reset sum counter
      for ( XScan=0; XScan<InternalInfo.Width; XScan++)     // scan all columns
      {
         PixelValue2 = GetPixel( XScan, YScan, 0 );         // get pixel value
         PixelValue2 -= InternalInfo.BackgrondLevel;        // subtract background level
         PixelValue2 -= 3*InternalInfo.BackgroundSigma;     // subtract 3 background sigma
         if ( PixelValue2 < 0 )                             // clamp to zero
            PixelValue2 = 0;
         PixelSum += PixelValue2;                           // sum current pixel value
      }
      InternalInfo.HImageSum[YScan] = PixelSum;             // store into image info
   }

   // compute vertical sum
   for ( XScan=0; XScan<InternalInfo.Width; XScan++ )       // scan all columns
   {
      PixelSum = 0;                                         // reset sum counter
      for ( YScan=0; YScan<InternalInfo.Height; YScan++)    // scan all lines
      {
         PixelValue2 = GetPixel( XScan, YScan, 0 );         // sum current pixel value
         PixelValue2 -= InternalInfo.BackgrondLevel;        // subtract background level
         PixelValue2 -= 3*InternalInfo.BackgroundSigma;     // subtract 3 background sigma
         if ( PixelValue2 < 0 )                             // clamp to zero
            PixelValue2 = 0;
         PixelSum += PixelValue2;                           // sum current pixel value
      }
      InternalInfo.VImageSum[XScan] = PixelSum;             // store into image info
   }

  	return ERR_NONE;
}



/****************************************************************************

   Get informazioni about last shot

*****************************************************************************/

CCD::ERR CCD::GetShotInfo( SHOT_INFO* Info )
{
   // only if opennig was correct
   if ( DpRam == 0 )
      return ERR_CCDSERVERNOTRUNNING;
   if ( DpRam->Opened == false )
      return ERR_DEVICENOTOPEN;

   *Info = InternalInfo;                                    // copy data from internal structure
  	return ERR_NONE;
}



/****************************************************************************

   Get last shot image

*****************************************************************************/

CCD::ERR CCD::GetImage( unsigned short* Image )
{
   // only if opennig was correct
   if ( DpRam == 0 )
      return ERR_CCDSERVERNOTRUNNING;

   // if ccd state is not IMAGE it is an error
   if ( InternalInfo.State != CCD_STATE_IMAGE )
      return ERR_WRONGSTATE;

   // copy data from internal image
   memcpy( Image, InternalImage, InternalInfo.Width*InternalInfo.Height*2);

   // set ccd state
   //InternalInfo.State = CCD_STATE_IDLE;

  	return ERR_NONE;
}



/****************************************************************************

   Get last shot image as a gray bitmap

*****************************************************************************/

CCD::ERR CCD::GetBmp( Graphics::TBitmap ** Bmp, int Min, int Max )
{
   int                                 IndexRow;            // index for row scan
   int                                 IndexCol;            // index for column scan
   unsigned char                       *pPixel;             // pointer for bitmap scan
   unsigned short                      *pImage;             // pointer for image scan
   int                                 Value;               // pixel value

   // only if opennig was correct
   if ( DpRam == 0 )
      return ERR_CCDSERVERNOTRUNNING;
   if ( DpRam->Opened == false )
      return ERR_DEVICENOTOPEN;
   // only if shot present
   if ( InternalInfo.Width == 0 || InternalInfo.Height == 0 )
      return ERR_NOTFOUND;

   InternalBmp->Width = InternalInfo.Width;                 // bitmap size as shot image
   InternalBmp->Height = InternalInfo.Height;

   // build bitmap from the image with input minimum and maximum
   pImage = InternalImage;                                  // pointer to image
   for ( IndexRow=0; IndexRow<InternalInfo.Height; IndexRow++ )   // scan all the line
   {
      pPixel = (unsigned char*)InternalBmp->ScanLine[IndexRow];   // get line pointer in the bimap
      for ( IndexCol=0; IndexCol<InternalInfo.Width; IndexCol++ ) // scan pixel in the column
      {
         Value = *pImage++;                                 // get value from the image
         Value = ComputeGrayValue( Value, Min, Max);        // scale value
         *pPixel++ = (unsigned char)Value;
      }
   }

  	*Bmp = InternalBmp;                                      // yeld bitmap
   return ERR_NONE;
}



/****************************************************************************

   Get last shot image zoomed as a gray bitmap

*****************************************************************************/

CCD::ERR CCD::GetZoomBmp( Graphics::TBitmap ** Bmp, int Min, int Max, int CenterX, int CenterY )
{
   int                                 IndexRow;            // index for row scan
   int                                 IndexCol;            // index for column scan
   unsigned char                       *pPixel;             // pointer for bitmap scan
   unsigned short                      *pImage;             // pointer for image scan
   int                                 Value;               // pixel value
   int                                 InitX;               // init orizontal pixel
   int                                 InitY;               // init vertical pixel
   int                                 ImageX;              // x position in the normal image
   int                                 ImageY;              // y posiziont in the normal image

   // only if opennig was correct
   if ( DpRam == 0 )
      return ERR_CCDSERVERNOTRUNNING;
   if ( DpRam->Opened == false )
      return ERR_DEVICENOTOPEN;

   InitX=CenterX-PIXEL_ZOOMED/2;                            // set init image from center zoom point
   InitY=CenterY-PIXEL_ZOOMED/2;                            // these can be negative (out border)

   // build bitmap from the image with current minimum and maximum
   for ( IndexRow=0; IndexRow<InternalZoomBmp->Height; IndexRow++ )// scan all the line of bitmap
   {
      pPixel = (unsigned char*)InternalZoomBmp->ScanLine[IndexRow];   // get line pointer in the bimap
      ImageY = InitY + IndexRow / ZOOM_FACTOR ;             // compute sorce line from line in zoom image
      pImage = InternalImage+ ImageY * InternalInfo.Width;  // compute source image line
      for ( IndexCol=0; IndexCol<InternalZoomBmp->Width; IndexCol++ ) // scan pixel in the column of bitmap
      {
         ImageX = InitX + IndexCol / ZOOM_FACTOR;           // compute source column from colum in zoom image
         if ( ImageX < 0 || ImageX >= InternalInfo.Width ||
              ImageY < 0 || ImageY >= InternalInfo.Height ) // check current line and column are out out image border
            Value = COLOR_OUTBORDER;                        // out border pixel showed with out border color
         else
         {
            Value = pImage[ImageX];                         // get value from the image
            Value = ComputeGrayValue( Value, Min, Max);     // scale gray value
         }
         *pPixel++ = (unsigned char)Value;                  // assign bitmap pixel color
      }
   }

   // draw red cross in the center
   InternalZoomBmp->Canvas->Pen->Color = clRed;
   ImageX = InternalZoomBmp->Width / 2;
   ImageY = InternalZoomBmp->Height / 2;
   InternalZoomBmp->Canvas->MoveTo( ImageX-1, ImageY );
   InternalZoomBmp->Canvas->LineTo( ImageX+2, ImageY );
   InternalZoomBmp->Canvas->MoveTo( ImageX, ImageY-1 );
   InternalZoomBmp->Canvas->LineTo( ImageX, ImageY+2 );

  	*Bmp = InternalZoomBmp;                                  // yeld bitmap

   return ERR_NONE;
}



/****************************************************************************

   get Histogram bitmap

*****************************************************************************/

CCD::ERR CCD::GetHistogramBmp( Graphics::TBitmap **Bmp, int Min, int Max )
{
   TRect                               Rect;                // initialization rectange
   int                                 Index;               // histogram scanning index
   int                                 Start;               // starting histogram slice
   int                                 Stop;                // ending histogram slice
   int                                 Index2;              // slice scanning index
   double                              Cnt;                 // slice sum counter

   // rectangle inizialization
   Rect.Top = 0;
   Rect.Left = 0;
   Rect.Right = HISTO_SIZEX;
   Rect.Bottom = HISTO_SIZEY;

   // draw background color
   InternalHistogramBmp->Canvas->Brush->Color = clDkGray;
   InternalHistogramBmp->Canvas->FillRect(Rect);

   // draw histogram
   InternalHistogramBmp->Canvas->Pen->Color = clWhite;      // select white pen
   for ( Index=0; Index <HISTO_SIZEX; Index++)              // scan all horizontal pixel
   {
      Start = Index * CCD::VALUE_RANGE / HISTO_SIZEX;       // start histogram element
      Stop = (Index+1) * CCD::VALUE_RANGE / HISTO_SIZEX;    // stop histogram element
      Cnt = 0;                                              // sum counter
      for ( Index2=Start; Index2<Stop; Index2++)            // scan elements between start and stop -1
         Cnt += InternalInfo.Histogram[Index2];             // sum histogram contribute
      if ( Cnt > 1 )                                        // is not possibile log(0)
         Cnt = Log2(Cnt) * 7;                               // compute log vertical scale
      InternalHistogramBmp->Canvas->MoveTo(Index,HISTO_SIZEY-1);
      InternalHistogramBmp->Canvas->LineTo(Index,HISTO_SIZEY-1-Cnt);
   }

   // draw gamma line
   InternalHistogramBmp->Canvas->Pen->Color = clRed;        // select red pen
   Start = Min * HISTO_SIZEX / CCD::VALUE_RANGE;            // init gamma point
   Stop = Max * HISTO_SIZEX / CCD::VALUE_RANGE;             // ending gamma point
   InternalHistogramBmp->Canvas->MoveTo(Start,HISTO_SIZEY-1);  // draw line
   InternalHistogramBmp->Canvas->LineTo(Stop, 0);

  	*Bmp = InternalHistogramBmp;                             // yeld bitmap
   return ERR_NONE;
}



/****************************************************************************

   search highest pixel

*****************************************************************************/

unsigned short CCD::SearchHighestPixel( int *XHigh, int *YHigh, int WinSize, int ImageCode )
{
   int                                 XScan;               // X in scanning
   int                                 YScan;               // Y in scanning
   unsigned short                      HighestValue;        // highest pixel value
   unsigned short                      Value;               // pixel value
   int                                 XInit;               // x init scanning
   int                                 YInit;               // x init scanning
   int                                 XEnd;                // x end scanning
   int                                 YEnd;                // x end scanning

   // enstablish scanning limit coordinate
   XInit = 0;
   YInit = 0;
   XEnd = InternalInfo.Width;
   YEnd = InternalInfo.Height;
   if ( WinSize != 0 )
   {
      XInit = *XHigh - WinSize/2;                           // compute X init
      if ( XInit < 0 )                                      // check underflow
         XInit = 0;                                         // clamp
      YInit = *YHigh - WinSize/2;                           // compute Y init
      if ( YInit < 0 )                                      // check underflow
         YInit = 0;                                         // clamp
      XEnd = *XHigh + WinSize/2;                            // compute X end
      if ( XEnd >= InternalInfo.Width )                     // check overflow
         XEnd = InternalInfo.Width;                         // clamp
      YEnd = *YHigh + WinSize/2;                            // compute Y end
      if ( YEnd >= InternalInfo.Height )                    // check overflow
         YEnd = InternalInfo.Height;                        // clamp
   }

   // scan all pixel searching the highest
   HighestValue = 0;
   for ( YScan=YInit; YScan<YEnd; YScan++ )
   {
      for ( XScan=XInit; XScan<XEnd; XScan++ )
      {
         Value = GetPixel(XScan, YScan, ImageCode );
         if ( HighestValue < Value )
         {
            HighestValue = Value;
            *XHigh = XScan;
            *YHigh = YScan;
         }
      }
   }
   return HighestValue;
}



/****************************************************************************

   get image value from position

*****************************************************************************/

CCD::ERR CCD::GetImageValue( int x, int y, int *Value )
{
   unsigned short                   IntValue;
   // only if opennig was correct
   if ( DpRam == 0 )
      return ERR_CCDSERVERNOTRUNNING;
   if ( DpRam->Opened == false )
      return ERR_DEVICENOTOPEN;

   IntValue = GetPixel( x, y, 0 );
   *Value = IntValue;

   return ERR_NONE;
}



/****************************************************************************

   get highest position

*****************************************************************************/

CCD::ERR CCD::GetHighestPosition( double *ParXHighest, double *ParYHighest)
{
   *ParXHighest = XHighest;
   *ParYHighest = YHighest;
   return ERR_NONE;
}



/****************************************************************************

   compute gray value

*****************************************************************************/

int CCD::ComputeGrayValue( int Value, int Min, int Max)
{
   if ( Max > Min )                                         // if max greater the min => positive image
   {
      if ( Value <= Min )
         Value = 0;                                         // image value less then min => color is black
      else
      {
         if ( Value >= Max )
            Value = GRAY_LEVELS-1;                          // image value greater then max => colore white
         else
         {
            Value -= Min;                                   // in the middle proportional gray
            Value *= (GRAY_LEVELS-1);
            Value /= (Max-Min);
         }
      }
   }
   else                                                     // if min greater the max => negative image
   {
      if ( Value >= Min )
         Value = 0;                                         // image value greater then min => color is blck
      else
      {
         if ( Value <= Max )
            Value = GRAY_LEVELS-1;                          // image value less then max => colore white
         else
         {
            Value -= Min;                                   // in the middle proportional gray
            Value *= (GRAY_LEVELS-1);
            Value /= (Max-Min);
         }
      }
   }
   return Value;
}



/****************************************************************************

   Get pixel value

*****************************************************************************/

unsigned short CCD::GetPixel( int X, int Y, int ImageCode )
{
   unsigned short                      *p;                  // pointer to image

   // test if coordinate are in the correct range
   if ( X < 0 || X >= InternalInfo.Width || Y < 0 || Y >= InternalInfo.Height )
      return 0;

   // set pointer depending image
   p = InternalImage;
   if ( ImageCode == 1 )
      p = InternalImageW;

   p += Y * InternalInfo.Width + X;                         // move on the pixel required
   return *p;
}



/****************************************************************************

   Set pixel value

*****************************************************************************/

void CCD::SetPixel( int X, int Y, unsigned short Value )
{
   unsigned short                      *p;                  // pointer to image

   // test if coordinate are in the correct range
   if ( X < 0 || X >= InternalInfo.Width || Y < 0 || Y >= InternalInfo.Height )
      return;

   // set pointer
   p = InternalImageW + Y * InternalInfo.Width + X;
   *p = Value;;
}



/****************************************************************************

   Get star coordinate and FWHM

*****************************************************************************/

CCD::ERR CCD::GetStarParam( double *X, double *Y, double *FWHM, int *Ground )
{
   int                                 XInput;              // x input
   int                                 YInput;              // y in input
   int                                 XInit;               // x init scanning
   int                                 YInit;               // x init scanning
   int                                 XEnd;                // x end scanning
   int                                 YEnd;                // x end scanning
   int                                 XScan;               // x in scanning
   int                                 YScan;               // y in scanning
   double                              Distance;            // distance between current pixel and input position
   int                                 Value;               // current pixel value
   int                                 VSum[RADIUS_STAR*2+1];  // array for vertical pixel sum
   int                                 HSum[RADIUS_STAR*2+1];  // array for horizontal pixel sum
   int                                 GroundHisto[VALUE_RANGE]; // Histogram of ground value distribuition
   double                              Sum;                 // sum for mean computing
   double                              Samples;             // samples for mean computing
   double                              Mean;                // distribution mean
   double                              Sigma;               // standard deviation
   int                                 GroundValue;         // value of ground
   int                                 GroundCount;         // count of ground value in th histogram

   // enstablish scanning limit coordinate
   XInput = *X;                                             // store input
   YInput = *Y;
   XInit = XInput - RADIUS_GROUND;                          // compute X init
   if ( XInit < 0 )                                         // check underflow
      XInit = 0;                                            // clamp
   YInit = YInput - RADIUS_GROUND;                          // compute Y init
   if ( YInit < 0 )                                         // check underflow
      YInit = 0;                                            // clamp
   XEnd = XInput + RADIUS_GROUND;                           // compute X end
   if ( XEnd >= InternalInfo.Width )                        // check overflow
      XEnd = InternalInfo.Width;                            // clamp
   YEnd = YInput + RADIUS_GROUND;                           // compute Y end
   if ( YEnd >= InternalInfo.Height )                       // check overflow
      YEnd = InternalInfo.Height;                           // clamp

   // erase sum arrays
   memset( VSum, 0, sizeof(VSum) );
   memset( HSum, 0, sizeof(HSum) );
   memset( GroundHisto, 0, sizeof(GroundHisto) );

   // scan all x coordinate
   for ( XScan = XInit; XScan <= XEnd; XScan++ )
   {
      // scan all y coordinate
      for ( YScan = YInit; YScan <= YEnd; YScan++ )
      {
         // discover pixel position: in the star circle, in the dead area, in the ground area or outer
         Distance = pow( XScan-XInput, 2 ) + pow( YScan-YInput, 2);     // compute distance between current pixel and star center
         Distance = pow( Distance , 0.5 );
         if ( Distance > RADIUS_GROUND )                    // bypass if out of dark circle
            continue;
         if ( Distance > RADIUS_STAR && Distance < RADIUS_DEAD )  // bypass if in the dead area
            continue;

         Value = GetPixel(XScan, YScan, 0 );                // get pixel value

         if ( Distance <= RADIUS_STAR )                     // manage values inside star cicle
         {
            VSum[YScan-YInput+RADIUS_STAR] += Value;        // sum pixel value at vertical array
            HSum[XScan-XInput+RADIUS_STAR] += Value;        // sum pixel value at horizontal array
         }
         else                                               // manage values inside dark area
         {
             GroundHisto[Value] ++;                         // increase ground histogram corresponding at value
         }
      }
   }

   // enstablish scanning limit coordinate for star radius
   XInit = XInput - RADIUS_STAR;                            // compute X init
   if ( XInit < 0 )                                         // check underflow
      XInit = 0;                                            // clamp
   YInit = YInput - RADIUS_STAR;                            // compute Y init
   if ( YInit < 0 )                                         // check underflow
      YInit = 0;                                            // clamp
   XEnd = XInput + RADIUS_STAR;                             // compute X end
   if ( XEnd >= InternalInfo.Width )                        // check overflow
      XEnd = InternalInfo.Width;                            // clamp
   YEnd = YInput + RADIUS_STAR;                             // compute Y end
   if ( YEnd >= InternalInfo.Height )                       // check overflow
      YEnd = InternalInfo.Height;                           // clamp

   // compute vertical real star center
   Sum = 0;                                                 // reset value for mean computing
   Samples = 0;
   for ( YScan = YInit; YScan <= YEnd; YScan++ )
   {
      Sum += VSum[YScan-YInput+RADIUS_STAR] * YScan;        // add y scanning weighed with vertical sum value
      Samples += VSum[YScan-YInput+RADIUS_STAR];            // add weigh;
   }
   Mean = Sum / Samples;                                    // compute vertcal mean
   *Y = Mean;                                               // mean is the vertical star center

   // compute horizontal real star center
   Sum = 0;                                                 // reset value for mean computing
   Samples = 0;
   for ( XScan = XInit; XScan <= XEnd; XScan++ )
   {
      Sum += HSum[XScan-XInput+RADIUS_STAR] * XScan;        // add x scanning weighed with horizontal sum value
      Samples += HSum[XScan-XInput+RADIUS_STAR];            // add weigh;
   }
   Mean = Sum / Samples;                                    // compute horizontal mean
   *X = Mean;                                               // mean is the horizontal star center

   // compute background value (maximum in the background histogram )
   GroundCount = 0;
   for ( XScan = 0; XScan < VALUE_RANGE; XScan++ )          // scan all value in the histogram
   {
      if ( GroundHisto[XScan] > GroundCount )               // react only when scanning value is greater than stored
      {
         GroundCount = GroundHisto[XScan];                  // store new count
         GroundValue = XScan;                               // store groud value ( scan index);
      }
   }
   *Ground = GroundValue;                                   // yeld ground value

   // compute FWHM
   XInit = XInput - RADIUS_STAR;                            // compute X init scanning
   if ( XInit < 0 )                                         // check underflow
      XInit = 0;                                            // clamp
   XEnd = XInput + RADIUS_STAR;                             // compute X end  scanning
   if ( XEnd >= InternalInfo.Width )                        // check overflow
      XEnd = InternalInfo.Width;                            // clamp
   Sum = 0;                                                 // reset value for sigma computing
   Samples = 0;
   Sigma = 0;
   for ( XScan = XInit; XScan <= XEnd; XScan++ )            // scann horizontal line through the star
   {
      Value = GetPixel(XScan, YInput, 0 );                  // get pixel value
      Value -= GroundValue;                                 // remove ground
      if ( Value < 0 )                                      // prevent underground
         Value = 0;
      Distance = pow( XScan - XInput, 2 );                  // compute square distance between current pixel and star center
      Sum += Distance * (double)Value;                      // compute sum
      Samples += Value;                                     // add sampling count
   }

   // provvisorio
   Sum = 0;                                                 // reset value for sigma computing
   Samples = 0;
   GroundValue = 1000;
   // scan all x coordinate
   for ( XScan = XInit; XScan <= XEnd; XScan++ )
   {
      // scan all y coordinate
      for ( YScan = YInit; YScan <= YEnd; YScan++ )
      {
         // discover pixel position: in the star circle or outer
         Distance = pow( XScan - *X, 2 ) + pow( YScan - *Y, 2);   // compute distance between current pixel and real star center
         Distance = pow( Distance , 0.5 );
         if ( Distance > RADIUS_STAR )                      // bypass if out of star circle
            continue;

         Value = GetPixel(XScan, YScan, 0 );                // get pixel value
         Value -= GroundValue;                              // remove ground
         if ( Value < 0 )                                   // prevent underground
            Value = 0;
         Distance = pow( Distance, 2 );
         Sum += Distance * (double)Value;
         Samples += Value;
      }
   }
   // fine provvisorio


   if ( Samples != 0 )
   {
      Sigma = Sum / Samples;                                // compute standard deviation
      Sigma = pow ( Sigma , 0.5 );
   }
   *FWHM = Sigma * 2.355;                                   // FWHM is 2.355 time sigma

   return ERR_NONE;
}



/****************************************************************************

   remove star

*****************************************************************************/

CCD::ERR CCD::RemoveStar( double X, double Y, unsigned short Ground )
{
   int                                 XInput;              // x input
   int                                 YInput;              // y in input
   int                                 XInit;               // x init scanning
   int                                 YInit;               // x init scanning
   int                                 XEnd;                // x end scanning
   int                                 YEnd;                // x end scanning
   int                                 XScan;               // x in scanning
   int                                 YScan;               // y in scanning
   double                              Distance;            // distance between current pixel and input position

   // enstablish scanning limit coordinate for star radius
   XInput = X;                                              // store input
   YInput = Y;
   XInit = XInput - RADIUS_STAR;                            // compute X init
   if ( XInit < 0 )                                         // check underflow
      XInit = 0;                                            // clamp
   YInit = YInput - RADIUS_STAR;                            // compute Y init
   if ( YInit < 0 )                                         // check underflow
      YInit = 0;                                            // clamp
   XEnd = XInput + RADIUS_STAR;                             // compute X end
   if ( XEnd >= InternalInfo.Width )                        // check overflow
      XEnd = InternalInfo.Width;                            // clamp
   YEnd = YInput + RADIUS_STAR;                             // compute Y end
   if ( YEnd >= InternalInfo.Height )                       // check overflow
      YEnd = InternalInfo.Height;                           // clamp

   // scan all x coordinate
   for ( XScan = XInit; XScan <= XEnd; XScan++ )
   {
      // scan all y coordinate
      for ( YScan = YInit; YScan <= YEnd; YScan++ )
      {
         // discover pixel position: in the star circle or outer
         Distance = pow( XScan - X, 2 ) + pow( YScan - Y, 2);   // compute distance between current pixel and star center
         Distance = pow( Distance , 0.5 );
         if ( Distance > RADIUS_STAR )                      // bypass if out of star circle
            continue;

         SetPixel(XScan, YScan, Ground );           // Set pixel value
      }
   }
   return ERR_NONE;
}














