/******************************************************************************

	CCD.h | Header of CCD manager

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : April 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// include antirepeat
#ifndef CCDH
#define CCDH



// system include
#include <vcl.h>
#pragma hdrstop



// project include
#include "DpCCDServer.h"



// define
#define CCD_RAWFILE                    "RawImage"



/******************************************************************************

	CCD manager class

******************************************************************************/

class CCD
{
   public:
      // mixed definition
      typedef enum
      {
         GRAY_PALETTE_ENTRY = 256,                          // number of element in gray palette
         GRAY_LEVELS = 250,                                 // numers of gray level
         COLOR_OUTBORDER = 250,                             // numer of color for outborder
         COLOR_STARCICLE = 251,                             // numer of color for cicle arrond the star
         COLOR_BORDER = 252,                                // number of color for border
         PIXEL_ZOOMED = 21,                                 // number of pixel zoomed
         ZOOM_FACTOR = 5,                                   // zoom factor
         VALUE_RANGE = 65536,                               // range of available value
         HISTO_SIZEX = 250,                                 // x size for hstogram bitmap
         HISTO_SIZEY = 140,                                 // y size for hstogram bitmap
         SERVER_TIMEOUT = 5000,                             // server timeout in millisecond
         RADIUS_STAR = 9,                                   // radius for star measurement
         RADIUS_DEAD = 12,                                  // radius for dead zone
         RADIUS_GROUND = 20,                                // radius for ground measurement
         STAR_FWHM = 10,                                    // number of star for FWHM computing
      } DEFINE;

      typedef enum
      {
         CCD_STATE_CLOSED,                                  // no operation available on CCD
         CCD_STATE_IDLE,                                    // ready to shot
         CCD_STATE_EXPOSING,                                // exposing
         CCD_STATE_IMAGE,                                   // image downloaded
      } CCD_STATE;

      // shot information structure
      typedef struct
      {
         int                           Width;               // column number
         int                           Height;              // raw number
         int                           MinValue;            // minimum image value
         int                           MaxValue;            // maximum image value
         int                           BackgrondLevel;      // level of the background image
         int                           BackgroundSigma;     // sigma of the only background image
         float                         VisiblityIndex;      // index of stars visibility
         int                           ImgageSigma;         // sigma of the whole image
         double                        FWHM;                // star size in FWHM
         int                           Histogram[VALUE_RANGE]; // Histogram of value distribuition
         CCD_STATE                     State;               // ccd state
         SYSTEMTIME                    InitShotTime;        // time of shot init
         double                        Exposure;            // esposure
         int                           *HImageSum;          // horizontal image sum
         int                           *VImageSum;          // vertical image sum
      }SHOT_INFO;

      // gray palette structure
      typedef struct
      {
        TLogPalette                    PalHeader;           // windows palette header
        TPaletteEntry                  dummy[GRAY_PALETTE_ENTRY-1];
      } GRAY_PALETTE;

      // error definition
      typedef enum
      {
         ERR_NONE,                                          // no error
         ERR_NOTFOUND,                                      // not found
         ERR_CCDSERVERNOTRUNNING=1000,                      // CCD server is not running
         ERR_DEVICENOTINSTALLED,                            // CCD device not installed
         ERR_DEVICENOTANSWER,                               // CCD device don't answer
         ERR_DEVICEALREADYOPEN,                             // CCD device already open
         ERR_DEVICENOTOPEN,                                 // CCD device not open
         ERR_NOTFINISHED,                                   // short not finish
         ERR_WRONGSTATE,                                    // ccd not in correct state
      } ERR;

      // object constructor
                                       CCD( );
      // object destroyer
      virtual                          ~CCD();

      // open CCD device
      ERR                              Open ( HANDLE FormHandle, char *Name, char *ImageFolder );
      // close CCD device
      ERR                              Close();
      // take a shot
      ERR                              Shot( int Time );
      // Check shot is finished
      ERR                              IsShotFinisched( int *Time );
      // Abort shot
      ERR                              AbortShot( void );
      // Get shot image
      ERR                              GetShot( void );
      // get informazioni about last shot
      ERR                              GetShotInfo( SHOT_INFO* Info );
      // get last shot image
      ERR                              GetImage( unsigned short* Image );
      // get last shot image as a gray bitmap
      ERR                              GetBmp( Graphics::TBitmap **, int Min, int Max );
      // get last shot image zoomed as a gray bitmap
      ERR                              GetZoomBmp( Graphics::TBitmap **, int Min, int Max, int CenterX, int CenterY );
      // get Histogram bitmap
      ERR                              GetHistogramBmp( Graphics::TBitmap **Bmp, int Min, int Max );
      // search highest pixel
      unsigned short                   SearchHighestPixel( int *XHigh, int *YHigh, int WinSize, int ImageCode );
      // get image value from position
      ERR                              GetImageValue( int x, int y, int *Value );
      // get highest position
      ERR                              GetHighestPosition( double *XHighest, double *YHighest);

  protected:
      char                             ImageName[MAXPATH];  // image name
      unsigned short                   *InternalImage;      // image buffer
      unsigned short                   *InternalImageW;     // image buffer for working
      Graphics::TBitmap                *InternalBmp;        // image as a bitmap
      Graphics::TBitmap                *InternalZoomBmp;    // zoom bitmap
      Graphics::TBitmap                *InternalHistogramBmp;  // histogram bitmap
      GRAY_PALETTE                     GrayPalette;         // gray palette definition for image
      GRAY_PALETTE                     GrayZoomPalette;     // gray palette definition for zoom image
      HPALETTE                         Hp;                  // palette handle
      HPALETTE                         HpZoom;              // palette handle for zoom image
      SHOT_INFO                        InternalInfo;        // internal shot info
      DWORD                            Start;               // shot start
      int                              ShotTime;            // shot time
  		HANDLE			                  HDpRam;              // virtaul dual port ram handle
      DP_CCD_SERVER	                  *DpRam;
      double                           XHighest;            // highest value x position
      double                           YHighest;            // highest value y position

      // compute gray value
      int                              ComputeGrayValue( int Value, int Min, int Max);
      // Get pixel value
      unsigned short                   GetPixel( int X, int Y, int ImageCode );
      // set pixel value
      void                             SetPixel( int X, int Y, unsigned short Value );
      // Get star coordinate and FWHM
      ERR                              GetStarParam( double *X, double *Y, double *FWHM, int *Ground );
      // remove star
      ERR                              RemoveStar( double X, double Y, unsigned short Ground );
};

#endif
