/******************************************************************************

	CloseSequence.cpp | Observatory closing sequence

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : June 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// System include
#pragma hdrstop



// project include
#include "CloseSequence.h"
#pragma package(smart_init)



/*---------------------------------------------------------------------------

   Object constructor

---------------------------------------------------------------------------*/

__fastcall CloseSequence::CloseSequence(void)
   : TThread( false )
{
   State = SEQ_STATE_IDLE;                                  // object starts in idle status
   PuntSubject = 0;                                         // no sublect
   Control = false;                                         // no contol taken
}



/****************************************************************************

   Object destroyer

*****************************************************************************/

_fastcall CloseSequence::~CloseSequence()
{
   if ( PuntSubject != 0 )                                  // if subject assigned
      PuntSubject->Detach(this);                            // detach this observer from it
}



/******************************************************************************

   set subject

******************************************************************************/

int CloseSequence::SetSubject(
   TentaclesMgr       *ParPuntSubject                       // pointer to subject
)
{
   PuntSubject = ParPuntSubject;                            // store pointer
   return PuntSubject->Attach( this );                      // attach this observer to subject
}



/*---------------------------------------------------------------------------

   Starts the sequence

---------------------------------------------------------------------------*/

void CloseSequence::StartSequence(HANDLE ParFormHandle, bool ParTelescopeHome, bool ParTrackingOff, bool ParDomeZero, bool ParCloseBShutter, bool ParCloseSShutter, bool ParDomeHome, bool ParPowerOff )
{
   // store input parameters
   FormHandle     = ParFormHandle;
   TelescopeHome  = ParTelescopeHome;
   TrackingOff    = ParTrackingOff;
   DomeZero       = ParDomeZero;
   CloseBShutter  = ParCloseBShutter;
   CloseSShutter  = ParCloseSShutter;
   DomeHome       = ParDomeHome;
   PowerOff       = ParPowerOff;

   // activate the sequence
   Counter = 0;
   State = SEQ_STATE_START;
}




/*---------------------------------------------------------------------------

   detect subject notify

---------------------------------------------------------------------------*/

void CloseSequence::Update( int ANotifyCode )
{
}



/*---------------------------------------------------------------------------

   sequence execution

---------------------------------------------------------------------------*/

void __fastcall CloseSequence::Execute(void)
{
   TentaclesMgr::STATUS                Status;              // subject status
   TentaclesMgr::ERR                   Error;

   // continue until thread is not external terminated
   while (  Terminated == false )
   {
      Sleep ( 1000 );                                       // leave time to others activities
      Counter++;
      // manage state machine
      switch ( State )
      {
         // sequence in idle (do nothing)
         case SEQ_STATE_IDLE:
            break;

         // sequence started
         case SEQ_STATE_START:
            Error = PuntSubject->GetStatus( &Status );      // get subject status
            if ( Error != TentaclesMgr::ERR_NONE )
            {
               MessageBox( FormHandle, "Impossibile leggere status", "Errore", MB_OK );
               State = SEQ_STATE_END;
               break;
            }
            // verify sequence starting conditions
            if ( Status.CommOpened == false || Status.CommOk == false )
            {
               MessageBox( FormHandle, "Non  possibile lanciare sequenza di chiusura, una delle sequenti condizioni non  verificata:\n   Comunicazione attiva\n   Comunicazione ok", "Errore", MB_OK );
               State = SEQ_STATE_END;
               break;
            }

            // take control of observatory
            Error = PuntSubject->TakeUp( this, "Sequenza di chiusura", true, false );
            if ( Error != TentaclesMgr::ERR_NONE )
            {
               MessageBox( FormHandle, "Non  possibile prendere controllo di osservatorio", "Errore", MB_OK );
               State = SEQ_STATE_END;
               break;
            }
            Control = true;                                 // control taken
            // if riquered telescope in home position
            if ( TelescopeHome == true )
            {
               Error = PuntSubject->RunCommand( this, TentaclesMgr::COM_SUBJ_TELE, TentaclesMgr::COM_STA_HOME);
               if ( Error != TentaclesMgr::ERR_NONE )
               {
                  MessageBox( FormHandle, "Non accettato telescopio in meridiano", "Errore", MB_OK );
                  State = SEQ_STATE_END;                    // goto end step
                  break;
               }
            }
            State = SEQ_STATE_TELESCOPEHOME;                // goto next step
            break;

         // wait telescope in home position
         case SEQ_STATE_TELESCOPEHOME:
            // if riquered telescope in home position
            if ( TelescopeHome == true )
            {
               Error = PuntSubject->GetStatus( &Status );   // get subject status
               if ( Error != TentaclesMgr::ERR_NONE )
               {
                  MessageBox( FormHandle, "Impossibile leggere status", "Errore", MB_OK );
                  State = SEQ_STATE_END;
                  break;
               }
               // is tracking on?
               if ( Status.TeleTracking == true )
                  break;
            }
            // if riquered dome in zero position
            if ( DomeZero == true )
            {
               // unlock dome
               Error = PuntSubject->RunCommand( this, TentaclesMgr::COM_SUBJ_DOME, TentaclesMgr::COM_STA_UNLOCK);
               if ( Error != TentaclesMgr::ERR_NONE )
               {
                  MessageBox( FormHandle, "Impossibile mette cupola in automatico", "Errore", MB_OK );
                  State = SEQ_STATE_END;
                  break;
               }
               // dome in zero position
               Error = PuntSubject->RunCommand( this, TentaclesMgr::COM_SUBJ_DOME, TentaclesMgr::COM_STA_ZERO);
               if ( Error != TentaclesMgr::ERR_NONE )
               {
                  MessageBox( FormHandle, "Non accettato posizionamento cupola su zero", "Errore", MB_OK );
                  State = SEQ_STATE_END;                    // goto end step
                  break;
               }
            }
            State = SEQ_STATE_DOMEZERO;                     // goto next step
            break;

         // wait dome in zero position
         case SEQ_STATE_DOMEZERO:
            if ( DomeZero == true )
            {
               Error = PuntSubject->GetStatus( &Status );   // get subject status
               if ( Error != TentaclesMgr::ERR_NONE )
               {
                  MessageBox( FormHandle, "Impossibile leggere status", "Errore", MB_OK );
                  State = SEQ_STATE_END;
                  break;
               }
               // is dome in zero position?
               if ( Status.DomeAlligned == false || Status.DomeMovement != 0 )
                  break;
            }
            Sleep ( 1000 );
            // if required balancing shutter closing
            if ( CloseBShutter == true )
            {
               // send sliding shutter opening
               Error = PuntSubject->RunCommand( this, TentaclesMgr::COM_SUBJ_BSHUTTER, TentaclesMgr::COM_STA_CLOSE);
               if ( Error != TentaclesMgr::ERR_NONE )
               {
                  MessageBox( FormHandle, "Non riesco ad chiudere ribaltabile", "Errore", MB_OK );
                  State = SEQ_STATE_END;                    // goto end step
                  break;
               }
            }
            State = SEQ_STATE_CLOSEBSHUTTER;                // goto next step
            break;

         // wait balancing shutter closing
         case SEQ_STATE_CLOSEBSHUTTER:
            // if required balancing shutter closing
            if ( CloseBShutter == true )
            {
               Error = PuntSubject->GetStatus( &Status );      // get subject status
               if ( Error != TentaclesMgr::ERR_NONE )
               {
                  MessageBox( FormHandle, "Impossibile leggere status", "Errore", MB_OK );
                  State = SEQ_STATE_END;
                  break;
               }
               // is dome balancing shutter close?
               if ( Status.DomeBalancingShutter != 1 )
                  break;
            }
            // if required sliding shutter closing
            if ( CloseSShutter == true )
            {
               // send sliding shutter closeing
               Error = PuntSubject->RunCommand( this, TentaclesMgr::COM_SUBJ_SSHUTTER, TentaclesMgr::COM_STA_CLOSE);
               if ( Error != TentaclesMgr::ERR_NONE )
               {
                  MessageBox( FormHandle, "Non riesco ad chiudere scorrevole", "Errore", MB_OK );
                  State = SEQ_STATE_END;                    // goto end step
                  break;
               }
            }
            State = SEQ_STATE_CLOSESSHUTTER;                // goto next step
            break;

         // wait sliding shutter closing
         case SEQ_STATE_CLOSESSHUTTER:
            // if required sliding shutter closing
            if ( CloseSShutter == true )
            {
               Error = PuntSubject->GetStatus( &Status );      // get subject status
               if ( Error != TentaclesMgr::ERR_NONE )
               {
                  MessageBox( FormHandle, "Impossibile leggere status", "Errore", MB_OK );
                  State = SEQ_STATE_END;
                  break;
               }
               // is dome sliding shutter close?
               if ( Status.DomeSlidingShutter != 1 )
                  break;
            }

            // if required dome in home position
            if ( DomeHome == true )
            {
               // unlock dome
               Error = PuntSubject->RunCommand( this, TentaclesMgr::COM_SUBJ_DOME, TentaclesMgr::COM_STA_UNLOCK);
               if ( Error != TentaclesMgr::ERR_NONE )
               {
                  MessageBox( FormHandle, "Impossibile mette cupola in automatico", "Errore", MB_OK );
                  State = SEQ_STATE_END;
                  break;
               }
               // dome in home position
               Error = PuntSubject->RunCommand( this, TentaclesMgr::COM_SUBJ_DOME, TentaclesMgr::COM_STA_HOME);
               if ( Error != TentaclesMgr::ERR_NONE )
               {
                  MessageBox( FormHandle, "Impossibile mette cupola in home", "Errore", MB_OK );
                  State = SEQ_STATE_END;
                  break;
               }
            }
            State = SEQ_STATE_DOMEHOME;                 // goto next step
            break;

         // wait dome in home position
         case SEQ_STATE_DOMEHOME:
            if ( DomeHome == true )
            {
               Error = PuntSubject->GetStatus( &Status );   // get subject status
               if ( Error != TentaclesMgr::ERR_NONE )
               {
                  MessageBox( FormHandle, "Impossibile leggere status", "Errore", MB_OK );
                  State = SEQ_STATE_END;
                  break;
               }
               // is dome in home position?
               if ( Status.DomeAzimut != 282 )
                  break;
            }
            // power off
            if ( PowerOff == true )
            {
               Error = PuntSubject->RunCommand( this, TentaclesMgr::COM_SUBJ_POWER, TentaclesMgr::COM_STA_OFF);
               if ( Error != TentaclesMgr::ERR_NONE )
               {
                  MessageBox( FormHandle, "Impossibile spegnere quadro", "Errore", MB_OK );
                  State = SEQ_STATE_END;
                  break;
               }
            }
            State = SEQ_STATE_END;                          // goto end step
            break;
      }

      // when sequence ended exit thread
      if ( State == SEQ_STATE_END )
         break;
   }

   // release all
   if ( Control == true )
   {
      Error = PuntSubject->Release( this, true, false );     // release observatory
      if ( Error != TentaclesMgr::ERR_NONE )
         MessageBox( FormHandle, "Impossibile rilasciare controllo", "Errore", MB_OK );
      else
         Control = false;
   }

   State = SEQ_STATE_END;
}



/*---------------------------------------------------------------------------

   get sequence status

---------------------------------------------------------------------------*/

CloseSequence::SEQ_STATE CloseSequence::GetState( int *ParCounter )
{
   *ParCounter = Counter;
   return State;
}

