/******************************************************************************

	ElavationGraph.cpp | Elevation Graphics manager

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : March 2012

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// project include
#include "ElevationGraph.h"
#include "CMCoHorizontal.h"
#pragma package(smart_init)



/****************************************************************************

   Object constructor

*****************************************************************************/

ElevationGraph::ElevationGraph()
{
   // reset internal param
   Sun = new CMPlanet( CMPlanet::SUN );
   Earth    = new CMPlanet(CMPlanet::EARTH );               // create Earth
   Sun->SetPlanetView(Earth);                               // Sun seen from Earth
   Sun->SetOpt( CMPlanet::MEEUS,false, false);              // set Meeus computing, without nutation and time light
}



/****************************************************************************

   Object destroyer

*****************************************************************************/

ElevationGraph::~ElevationGraph()
{
}



/****************************************************************************

   Set X axel

*****************************************************************************/

void ElevationGraph::SetX( int Start, int Stop, int Grid )
{
   // store parameter
   StartX = Start;
   StopX = Stop;
   GridX = Grid;
   if ( Stop < Start )
      StopX += 24;
}



/****************************************************************************

   Set Y axel

*****************************************************************************/

void ElevationGraph::SetY( int Start, int Stop, int Grid )
{
   // store parameter
   StartY = Start;
   StopY = Stop;
   GridY = Grid;
}



/****************************************************************************

   set date

*****************************************************************************/

void ElevationGraph::SetDate( double PJD )
{
   int                                 IntegerJD;
   double                              SunsetJD;
   double                              NightIniJD;
   double                              NightEndJD;
   double                              SunriseJD;
   const CMCoEquatorial*               EquPosition;
   CMCoHorizontal                      *LocPosition;
   double                              Elevation;
   double                              Delta;

   JD = PJD;

   IntegerJD = JD;
   SunsetJD = IntegerJD + 0.25;

   do
   {
      Sun->SetData ( SunsetJD );
      EquPosition = Sun->GetPosition();
      LocPosition = EquPosition->GetCoOri( SunsetJD, Site, false );
      Elevation = LocPosition->GetElevation()->GetC();
      Delta = Elevation - -1;
      if ( fabs(Delta) < ELEVATION_PRECISION )
         break;
      Delta /= 360;
      SunsetJD += Delta;
   }
   while( 1 );

   SunsetTime = SunsetJD - IntegerJD;
   SunsetTime *= 24;
   SunsetTime += 12;



   IntegerJD = JD;
   NightIniJD = IntegerJD + 0.25;

   do
   {
      Sun->SetData ( NightIniJD );
      EquPosition = Sun->GetPosition();
      LocPosition = EquPosition->GetCoOri( NightIniJD, Site, false );
      Elevation = LocPosition->GetElevation()->GetC();
      Delta = Elevation - -12;
      if ( fabs(Delta) < ELEVATION_PRECISION )
         break;
      Delta /= 360;
      NightIniJD += Delta;
   }
   while( 1 );

   NightIniTime = NightIniJD - IntegerJD;
   NightIniTime *= 24;
   NightIniTime += 12;



   IntegerJD = JD;
   NightEndJD = IntegerJD + 0.75;

   do
   {
      Sun->SetData ( NightEndJD );
      EquPosition = Sun->GetPosition();
      LocPosition = EquPosition->GetCoOri( NightEndJD, Site, false );
      Elevation = LocPosition->GetElevation()->GetC();
      Delta = Elevation - -12;
      if ( fabs(Delta) < ELEVATION_PRECISION )
         break;
      Delta /= 360;
      NightEndJD -= Delta;
   }
   while( 1 );

   NightEndTime = NightEndJD - IntegerJD;
   NightEndTime *= 24;
   NightEndTime += 12;



   IntegerJD = JD;
   SunriseJD = IntegerJD + 0.75;

   do
   {
      Sun->SetData ( SunriseJD );
      EquPosition = Sun->GetPosition();
      LocPosition = EquPosition->GetCoOri( SunriseJD, Site, false );
      Elevation = LocPosition->GetElevation()->GetC();
      Delta = Elevation - -1;
      if ( fabs(Delta) < ELEVATION_PRECISION )
         break;
      Delta /= 360;
      SunriseJD -= Delta;
   }
   while( 1 );

   SunriseTime = SunriseJD - IntegerJD;
   SunriseTime *= 24;
   SunriseTime += 12;

}



/****************************************************************************

   set site position

*****************************************************************************/

void ElevationGraph::SetSite ( CMCoLocal *PSite )
{
   // store parameter
   Site = PSite;
}



/****************************************************************************

   set object list

*****************************************************************************/

void ElevationGraph::SetList( CMObjectList *PList )
{
   List = PList;
}



/****************************************************************************

   set data color

*****************************************************************************/

void ElevationGraph::SetColor( TColor *AColor, int AObjectCount )
{
   if ( AObjectCount > MAX_OBJECT)
      AObjectCount = MAX_OBJECT;
   // store paramete
   memcpy(&TabColor, AColor, sizeof(TColor)*AObjectCount);
   ObjectCount = AObjectCount;
}



/****************************************************************************

   get graph bitmap

*****************************************************************************/

void ElevationGraph::GetBmp( Graphics::TBitmap **PBmp )
{
   Graphics::TBitmap                   *Bmp;                // pointer to input bitmap
   TRect                               Rect;                // rectangle for graphic drawing
   int                                 InitX;               // initial posizion on x axel
   int                                 EndX;                // ending posizion on x axel
   int                                 InitY;               // initial posizion on y axel
   int                                 EndY;                // ending posizion on y axel
   double                              ScaleX;              // scale pixel for unit on X axel
   double                              ScaleY;              // scale pixel for unit on Y axel
   double                              Xd;                  // line x coordiante for computing
   double                              Yd;                  // line y coordiante for computing
   int                                 X;                   // line x coordiante
   int                                 Y;                   // line y coordiante
   int                                 PosGridX;            // position for X grid
   int                                 PosGridY;            // position for Y grid
   int                                 Value;
   AnsiString                          Label;
   TSize Size;
   CMObject                            *Obj;
   double                              TempJD;
   double                                 XCurr;
   const CMCoEquatorial*               EquPosition;
   CMCoHorizontal                      *LocPosition;
   double                              Elevation;
   bool                                First;
   int                                 ObjectIndex;


   // save chart parameters
   Bmp = *PBmp;                                             // get pointer to input bitmap

   // initialize limit
   InitX = 0;
   InitY = 0;
   EndX  = Bmp->Width;
   EndY  = Bmp->Height;

   // draw background color

   Rect.Top = 0;                                            // rectangle inizialization as whole bitmap
   Rect.Left = 0;
   Rect.Right = Bmp->Width;
   Rect.Bottom = Bmp->Height;
   Bmp->Canvas->Brush->Color = TColor(clBlack);             // night & border color
   Bmp->Canvas->FillRect(Rect);                             // paint all

   // draw ruler color
   Rect.Top = EndY - RULER_SIZEY;                           // rectangle inizialization for time ruler
   Rect.Bottom = Bmp->Height-1;
   Rect.Left = 1;
   Rect.Right = Bmp->Width-1;
   Bmp->Canvas->Brush->Color = TColor(0x506050);            // ruler color
   Bmp->Canvas->FillRect(Rect);                             // paint ruler

   // clamp to graphic area
   InitX++;
   InitY++;
   EndX--;
   EndY -= RULER_SIZEY + 1 ;

   // compute  scale
   ScaleX = double( EndX - InitX) / double( StopX - StartX );
   ScaleY = double( EndY - InitY - 1) / double( StopY - StartY );

   // draw day before twilight
   Rect.Left = InitX;                                       // from graphic beginig
   Rect.Right = (SunsetTime-StartX) * ScaleX ;              // to sunset
   Rect.Top = InitY;
   Rect.Bottom = EndY;
   if ( Rect.Left < Rect.Right )                            // only if sunset is visibile
   {
      Bmp->Canvas->Brush->Color = TColor( 0x80ffff);        // day color
      Bmp->Canvas->FillRect( Rect);                         // paint day
   }

   // draw evening twilight
   Rect.Left = (SunsetTime-StartX) * ScaleX ;               // from sunset
   if ( Rect.Left < 0 )                                     // check underflow
      Rect.Left = 0;
   Rect.Right = (NightIniTime-StartX) * ScaleX ;            // to night
   if ( Rect.Right < 0 )                                    // check underflow
      Rect.Right = 0;
   Rect.Top = InitY;
   Rect.Bottom = EndY;
   if ( Rect.Left < Rect.Right )                            // only if sunset is visibile
   {
      Bmp->Canvas->Brush->Color = TColor( 0x802020);        // twilight color
      Bmp->Canvas->FillRect( Rect);                         // paint day
   }

   // draw morning twilight
   Rect.Left = (NightEndTime-StartX) * ScaleX;              // from night
   if ( Rect.Left > EndX )                                  // check overflow
      Rect.Left = EndX;
   Rect.Right = (SunriseTime-StartX) * ScaleX ;             // to sunrise
   if ( Rect.Right > EndX )                                 // check overflow
      Rect.Right = EndX;
   Rect.Top = InitY;
   Rect.Bottom = EndY;
   if ( Rect.Left < Rect.Right )                            // only if sunrise is visibile
   {
      Bmp->Canvas->Brush->Color = TColor( 0x802020);        // twilight color
      Bmp->Canvas->FillRect( Rect);                         // paint day
   }

   // draw day after twilight
   Rect.Left = (SunriseTime-StartX) * ScaleX ;              // from sunrise
   if ( Rect.Left > EndX )                                  // check overflow
      Rect.Left = EndX;
   Rect.Right = EndX ;                                      // to graphic end
   Rect.Top = InitY;
   Rect.Bottom = EndY;
   if ( Rect.Left < Rect.Right )                            // only if sunset is visibile
   {
      Bmp->Canvas->Brush->Color = TColor( 0x80ffff);        // day color
      Bmp->Canvas->FillRect( Rect);                         // paint day
   }

   // draw vertical grid
   Bmp->Canvas->Pen->Color = TColor(0x00a000);
   PosGridX = StartX+GridX;
   do
   {
      Xd = (PosGridX - (double)StartX) * ScaleX + 0.5;
      X = InitX + Xd;
      Bmp->Canvas->MoveTo(X, InitY);
      Bmp->Canvas->LineTo(X, EndY);
      PosGridX += GridX;
   } while ( PosGridX<StopX );

   // draw vertical labels
   Bmp->Canvas->Font->Color = TColor(0xffffff);
   Bmp->Canvas->Brush->Color = TColor(0x506050);            // ruler color
   PosGridX = StartX+GridX;
   Value = StartX + 1;
   do
   {
      Label = AnsiString( Value );
      Xd = (PosGridX - (double)StartX) * ScaleX + 0.5;
      X = InitX + Xd;
      Size = Bmp->Canvas->TextExtent(Label);
      Bmp->Canvas->TextOut(X-Size.cx/2, EndY+5, Label);
      PosGridX += GridX;
      Value += GridX;
      if ( Value > 23 )
         Value -= 24;
   } while ( PosGridX<StopX );

   // draw horizontal grid
   Bmp->Canvas->Pen->Color = TColor(0x00a000);
   for ( PosGridY = StartY+GridY; PosGridY<StopY; PosGridY += GridY )
   {
      Yd = (PosGridY - (double)StartY) * ScaleY + 0.5;
      Y = EndY - Yd;
      Bmp->Canvas->MoveTo(InitX, Y);
      Bmp->Canvas->LineTo(EndX, Y);
   }

   // draw horizontal label
   Bmp->Canvas->Font->Color = TColor(0x0);
   Bmp->Canvas->Brush->Color = TColor(0x80ffff);
   Value = StartY+GridY;
   for ( PosGridY = StartY+GridY; PosGridY<StopY; PosGridY += GridY )
   {
      Label = AnsiString( Value );
      Yd = (PosGridY - (double)StartY) * ScaleY + 0.5;
      Y = EndY - Yd;
      Size = Bmp->Canvas->TextExtent(Label);
      Bmp->Canvas->TextOut( InitX + 3, Y-Size.cy/2, Label);
      Value += GridY;
   }

   ObjectIndex=0;
   List->First();
   Obj = List->Next();
   while ( Obj != 0 )
   {
      First = true;

      Bmp->Canvas->Pen->Color = TabColor[ObjectIndex]	;
      XCurr = StartX;
      do
      {
         TempJD = JD + XCurr/24.0;
         Obj->SetData(TempJD);
         EquPosition = Obj->GetPosition();
         EquPosition = EquPosition->ChangeEqu( TempJD );

         LocPosition = EquPosition->GetCoOri( TempJD, Site, false );
         Elevation = LocPosition->GetElevation()->GetC();
         if ( Elevation > 0 )
         {
            Xd = (XCurr-StartX) * ScaleX + 0.5;
            X = InitX + Xd;

            Yd = Elevation * ScaleY + 0.5;
            Y = EndY - Yd;

            if ( First == true )
               Bmp->Canvas->MoveTo(X, Y);
            else
               Bmp->Canvas->LineTo(X, Y);
            First = false;
         }
         else
            First = true;

         XCurr += 0.5;
      } while ( XCurr <= StopX );

      Obj = List->Next();
      ObjectIndex++;
      if ( ObjectIndex >= ObjectCount )
         break;
   }

/*   ScanElement = FirstElement;
   if ( ScanElement == 0 )
      return;

   Bmp->Canvas->Pen->Color = AxisColor	;
   Xd = (ScanElement->Point.X - (double)StartX) * ScaleX + 0.5;
   Yd = (ScanElement->Point.Y - (double)StartY) * ScaleY + 0.5;
   X = InitX + Xd;
   Y = EndY - Yd;
   Bmp->Canvas->MoveTo(X, Y);

   while ( ScanElement->Next != 0 )
   {
      ScanElement = ScanElement->Next;
      Xd = (ScanElement->Point.X - (double)StartX) * ScaleX + 0.5;
      Yd = (ScanElement->Point.Y - (double)StartY) * ScaleY + 0.5;
      X = InitX + Xd;
      Y = EndY - Yd;
      Bmp->Canvas->LineTo(X, Y);
   }  */

}

