/*---------------------------------------------------------------------------

	Fits.h  Header oggetto di gestione file filts

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : March 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus and SNF
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// include antirepeat
#ifndef FitsH
#define FitsH


// system include
#include <windows.h>



// project include
#include "CMHours.h"
#include "CMDegree.h"


// defines
#define FITS_NUMVALUES                 36



class Fits
{
   public:
      // error enumeration
      typedef enum
      {
         ERR_NONE,                                          // no error
         ERR_FILENOTOPEN,                                   // it is not possibile to open file
         ERR_FILENOTREAD,                                   // it is not possibile to read file
         ERR_FILENOTWRITTEN,                                // it is not possibile to write file
         ERR_IMAGETOOCOMPLEX,                               // FITS fomat too complex
         ERR_TAGNOTFOUND,                                   // mandatory tags not found
         ERR_NOIMAGE,                                       // no image in the object
         ERR_VALUENOTFOUND,                                 // value not found
         ERR_ADDPARAM,                                      // error adding a parameter
         ERR_ENDPARAMETERS,                                 // end of parameters
      } ERR;



      // line in the file header
      typedef struct
      {
         char                          Key[8];              // value key
         char                          Ass[2];              // assegnation
         char                          Value[70];           // value
      }FITS_VALUE;

      // line block
      typedef struct
      {
         FITS_VALUE                    Param[FITS_NUMVALUES];  // array parameter
      }FITS_BLOCK;


      // block chain element
      typedef struct FITS_CHAINTag
      {
         FITS_VALUE                    Actual;              // actual block
         FITS_CHAINTag*                Next;                // pounter to next block
      }FITS_CHAIN;

      // contructor
                                       Fits();
      // copy constructor
                                       Fits( Fits *Copy );
      // destroyer
                                       ~Fits();
      // load file from disk
      ERR                              LoadFile( char *FileName);
      // save file to disk
      ERR                              SaveFile( char *FileName);
      // set image
      ERR                              SetImage( int DimY, int DimX, int ParBase, unsigned short *ImageBuffer);
      // get image size
      ERR                              GetImageSize( int* DimY, int* DimX );
      // get image
      ERR                              GetImage( unsigned short *ImageBuffer );
      // set temperature
      ERR                              SetTemperature( double Temp);
      // get temperature
      ERR                              GetTemperature( double *Temp);
      // set shot parameters
      ERR                              SetShotPar( SYSTEMTIME *Time, double Exp);
      // get shot parameters
      ERR                              GetShotPar( SYSTEMTIME *Time, double *Exp);
      // set coordinate
      ERR                              SetPosition( CMHours::HOURS *Ar, CMDegree::DEGREE *Dec, double YearEquinox);
      // get coordinate
      ERR                              GetPosition( CMHours::HOURS *Ar, CMDegree::DEGREE *Dec, double *YearEquinox);
      // set object name
      ERR                              SetObject( char *Object);
      // get object name
      ERR                              GetObject( char *Object);
      // set observatory parameters
      ERR                              SetObservatoryPar( char *Telescope, char *Instrument, char *Observer);
      // get observatory parameters
      ERR                              GetObservatoryPar( char *Telescope, char *Instrument, char *Observer);
      // Set generic value
      ERR                              SetValue( char *Key, char *Value);
      // get generic value
      ERR                              GetValue( char *Key, char *Value);
      // get to first value
      ERR                              GetFirst( FITS_VALUE **Now );
      // get next value element
      ERR                              GetNext( FITS_VALUE ** Now );
      // remove current value
      ERR                              Remove( void );

   private:

      unsigned short*                  Image;               // image data
      FITS_CHAIN*                      First;               // first chain block
      int                              SizeX;               // X image size
      int                              SizeY;               // y image size
      unsigned short                   Base;                // base image value
      FITS_CHAIN*                      Current;             // current chain block
      FITS_CHAIN*                      Previous;            // previois chain block

      // free current block chain
      ERR                              FreeChain(void);
      // change word order
      unsigned short                   GiraW(unsigned short i);
      // change long order
      unsigned long                    GiraL(unsigned long i);

      // add a boolern value
      ERR                              SetValueBool( char *Key, bool Value );
      // add a numeric value
      ERR                              SetValueNum( char *Key, double Value );
      // get a numeric value
      ERR                              GetValueNum( char *Key, double *Value);

};

#endif
