/******************************************************************************

	FlatSequence.cpp | flat field and its dark gathering sequence

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : June 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// System include
#pragma hdrstop



// project include
#include "FlatSequence.h"
#pragma package(smart_init)





/*---------------------------------------------------------------------------

   Object constructor

---------------------------------------------------------------------------*/

__fastcall FlatSequence::FlatSequence(void)
   : TThread( false )
{
   State = SEQ_STATE_IDLE;                                  // object starts in idle status
   PuntSubject = 0;                                         // no sublect
   Control = false;                                         // no contol taken
}



/****************************************************************************

   Object destroyer

*****************************************************************************/

_fastcall FlatSequence::~FlatSequence()
{
   if ( PuntSubject != 0 )                                  // if subject assigned
      PuntSubject->Detach(this);                            // detach this observer from it
}



/******************************************************************************

   set subject

******************************************************************************/

int FlatSequence::SetSubject(
   TentaclesMgr       *ParPuntSubject                       // pointer to subject
)
{
   PuntSubject = ParPuntSubject;                            // store pointer
   return PuntSubject->Attach( this );                      // attach this observer to subject
}



/*---------------------------------------------------------------------------

   Starts the sequence

---------------------------------------------------------------------------*/

void FlatSequence::StartSequence(HANDLE ParFormHandle, int ParExposure, int ParNumFlat, int ParNumDark)
{
   // store input parameters
   FormHandle = ParFormHandle;
   Exposure = ParExposure;
   NumFlat = ParNumFlat;
   NumDark = ParNumDark;

   // reset progressive conters
   ProgFlat = 0;
   ProgDark = 0;

   // activate the sequence
   State = SEQ_STATE_START;
}




/*---------------------------------------------------------------------------

   detect subject notify

---------------------------------------------------------------------------*/

void FlatSequence::Update( int ANotifyCode )
{
}



/*---------------------------------------------------------------------------

   sequence execution

---------------------------------------------------------------------------*/

void __fastcall FlatSequence::Execute(void)
{
   TentaclesMgr::STATUS                Status;              // subject status
   TentaclesMgr::ERR                   Error;

   // continue until thread is not external terminated
   while (  Terminated == false )
   {
      Sleep ( 500 );                                        // leave time to others activities

      // manage state machine
      switch ( State )
      {
         // sequence in idle (do nothing)
         case SEQ_STATE_IDLE:
            break;

         // sequence started
         case SEQ_STATE_START:
            Error = PuntSubject->GetStatus( &Status );      // get subject status
            if ( Error != TentaclesMgr::ERR_NONE )
            {
               MessageBox( FormHandle, "Impossibile leggere status", "Errore", MB_OK );
               State = SEQ_STATE_END;
               break;
            }
            // verify flat sequence starting conditions
            if ( Status.CommOpened   == false || Status.CommOk      == false ||
                 Status.TeleTracking == true  || Status.CCDEnabled  == false    )
            {
               MessageBox( FormHandle, "Non  possibile lanciare sequenza di Flat, una delle sequenti condizioni non  verificata:\n   Comunicazione attiva\n   Comunicazione ok\n   Moto orario fermo\n   CCD abilitato", "Errore", MB_OK );
               State = SEQ_STATE_END;
               break;
            }

            // take control of observatory and ccd
            Error = PuntSubject->TakeUp( this, "Sequenza di flat", true, true );
            if ( Error != TentaclesMgr::ERR_NONE )
            {
               MessageBox( FormHandle, "Non  possibile prendere controllo di osservatorio e CCD", "Errore", MB_OK );
               State = SEQ_STATE_END;
               break;
            }
            Control = true;                                 // control taken
            // turn on flat lights
            Error = PuntSubject->RunCommand( this, TentaclesMgr::COM_SUBJ_LIGHT, TentaclesMgr::COM_STA_ON);
            if ( Error != TentaclesMgr::ERR_NONE )
            {
               MessageBox( FormHandle, "Impossibile accendere luci flat", "Errore", MB_OK );
               State = SEQ_STATE_END;                       // goto end step
               break;
            }
            Error = PuntSubject->SetObjectName( "Flat" );   // set base name for shots
            if ( Error != TentaclesMgr::ERR_NONE )
            {
               MessageBox( FormHandle, "Non accettato nome foto", "Errore", MB_OK );
               State = SEQ_STATE_END;                       // goto end step
               break;
            }
            State = SEQ_STATE_SHOTFLAT;                     // goto next step
            break;

         // shot flat
         case SEQ_STATE_SHOTFLAT:
            Error =PuntSubject->Shot( this, Exposure);      // shot new flat image
            if ( Error != TentaclesMgr::ERR_NONE )
            {
               MessageBox( FormHandle, "Non  possibile scattare foto", "Errore", MB_OK );
               State = SEQ_STATE_END;                       // goto end step
               break;
            }
            ProgFlat++;                                     // increase shot progressive number
            State = SEQ_STATE_WAITFLAT;                     // goto next step
            break;

         // wait shot flat finisch
         case SEQ_STATE_WAITFLAT:
            Error = PuntSubject->IsShotFinisched(&Exposing);// check shot is finished
            if ( Error == TentaclesMgr::ERR_NONE )             // until not finished stay in this state
            {
               // get shot
               Error = PuntSubject->GetShot( this );
               if ( Error != TentaclesMgr::ERR_NONE )
               {
                  MessageBox( FormHandle, "Impossibile scaricare foto flat", "Errore", MB_OK );
                  State = SEQ_STATE_END;                    // goto end step
                  break;
               }
               // save the shot in nightly folder
               Error = PuntSubject->RunCommand( this, TentaclesMgr::COM_SUBJ_NIGHTLYFOLDER, TentaclesMgr::COM_STA_SAVE);
               if ( Error != TentaclesMgr::ERR_NONE )
               {
                  MessageBox( FormHandle, "Impossibile salvare foto flat", "Errore", MB_OK );
                  State = SEQ_STATE_END;                    // goto end step
                  break;
               }
               if ( ProgFlat >= NumFlat )                   // verify if flat sequence is finished
               {
                  // turn off flat lights
                  Error = PuntSubject->RunCommand( this, TentaclesMgr::COM_SUBJ_LIGHT, TentaclesMgr::COM_STA_OFF);
                  if ( Error != TentaclesMgr::ERR_NONE )
                  {
                     MessageBox( FormHandle, "Impossibile spegnere luci flat", "Errore", MB_OK );
                     State = SEQ_STATE_END;                 // goto end step
                     break;
                  }
                  Error = PuntSubject->SetObjectName( "Dark" );   // set base name for shots
                  if ( Error != TentaclesMgr::ERR_NONE )
                  {
                     MessageBox( FormHandle, "Non accettato nome foto", "Errore", MB_OK );
                     State = SEQ_STATE_END;                 // goto end step
                     break;
                  }
                  State = SEQ_STATE_SHOTDARK;               // goto next step
               }
               else
                  State = SEQ_STATE_SHOTFLAT;               // go again to flat shot
            }
            break;

         // shot dark
         case SEQ_STATE_SHOTDARK:
            Error = PuntSubject->Shot( this, Exposure);     // shot new dark image
            if ( Error != TentaclesMgr::ERR_NONE )
            {
               MessageBox( FormHandle, "Non  possibile scattare foto", "Errore", MB_OK );
               State = SEQ_STATE_END;                       // goto end step
               break;
            }
            ProgDark++;                                     // increase shot progressive number
            State = SEQ_STATE_WAITDARK;                     // goto next step
            break;

         // wait shot dark finisch
         case SEQ_STATE_WAITDARK:
            Error = PuntSubject->IsShotFinisched(&Exposing);// check shot is finished
            if ( Error == TentaclesMgr::ERR_NONE )          // until not finished stay in this state
            {
               // get shot
               Error = PuntSubject->GetShot(this);
               if ( Error != TentaclesMgr::ERR_NONE )
               {
                  MessageBox( FormHandle, "Impossibile scaricare foto dark", "Errore", MB_OK );
                  State = SEQ_STATE_END;                    // goto end step
                  break;
               }
               Error = PuntSubject->RunCommand( this, TentaclesMgr::COM_SUBJ_NIGHTLYFOLDER, TentaclesMgr::COM_STA_SAVE);
               if ( Error != TentaclesMgr::ERR_NONE )
               {
                  MessageBox( FormHandle, "Impossibile salvare foto dark", "Errore", MB_OK );
                  State = SEQ_STATE_END;                    // goto end step
                  break;
               }
               if ( ProgDark >= NumDark )                   // verify if dark sequence is finished
                  State = SEQ_STATE_END;                    // goto end step
               else
                  State = SEQ_STATE_SHOTDARK;               // go again to dark shot
            }
            break;
      }

      // when sequence ended exit thread
      if ( State == SEQ_STATE_END )
         break;
   }

   // manage premature sequence termination
   if ( State == SEQ_STATE_WAITFLAT || State == SEQ_STATE_WAITDARK )
   {
      Error = PuntSubject->AbortShot(this);
      if ( Error != TentaclesMgr::ERR_NONE )
         MessageBox( FormHandle, "Impossibile abortire foto", "Errore", MB_OK );
      // turn off flat lights
      PuntSubject->RunCommand( this, TentaclesMgr::COM_SUBJ_LIGHT, TentaclesMgr::COM_STA_OFF);
   }

   // release all
   if ( Control == true )
   {
      Error = PuntSubject->Release( this, true, true );     // release observatory and CCD
      if ( Error != TentaclesMgr::ERR_NONE )
         MessageBox( FormHandle, "Impossibile rilasciare controllo", "Errore", MB_OK );
      else
         Control = false;
   }

   State = SEQ_STATE_END;
}



/*---------------------------------------------------------------------------

   get sequence status

---------------------------------------------------------------------------*/

void FlatSequence::GetStatus( bool *End, int *ParExposing, int *ParProgFlat, int *ParProgDark )
{
   *End = State == SEQ_STATE_END;
   *ParExposing = Exposing;
   *ParProgFlat = ProgFlat;
   *ParProgDark = ProgDark;
}

