/******************************************************************************

	GRB.cpp | GRB alert detector

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : May 2011

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// System include
#include <dir.h>



// project include
#include "GRB.h"
#include "CMEvent.h"
#pragma package(smart_init)


/*---------------------------------------------------------------------------

   object constructor

---------------------------------------------------------------------------*/

GRB::GRB(void)
{
   // initialize internal data
   MailCount=0;
   FirstElement = 0;
   LastElement = 0;

   // load MAPI library
   MAPILogon = NULL;
   MAPILogoff = NULL;
   MAPIReadMail = NULL;
   MAPIFindNext = NULL;
   hlibMAPI = LoadLibrary("MAPI32.DLL");  // 32 bit clients
   if ( hlibMAPI != NULL )
   {
      MAPILogon = (MAPILOGON*) GetProcAddress (hlibMAPI, "MAPILogon") ;
      MAPILogoff = (MAPILOGOFF*) GetProcAddress (hlibMAPI, "MAPILogoff") ;
      MAPIReadMail = (MAPIREADMAIL*) GetProcAddress (hlibMAPI, "MAPIReadMail") ;
      MAPIFindNext = (MAPIFINDNEXT*) GetProcAddress (hlibMAPI, "MAPIFindNext") ;
   }
}



/*---------------------------------------------------------------------------

   Object destroyer

---------------------------------------------------------------------------*/

GRB::~GRB()
{
   // MAPI library relase
   if ( hlibMAPI != NULL )
      FreeLibrary(hlibMAPI);
   // clear current list
   ClearList();
}



/*---------------------------------------------------------------------------

   Set option

---------------------------------------------------------------------------*/

void GRB::SetOption ( int ParMinElevation, int ParMaxDelay )
{
   MinElevation = ParMinElevation;
   MaxDelay = ParMaxDelay;
}



/*---------------------------------------------------------------------------

   Set status edit

---------------------------------------------------------------------------*/

void GRB::SetStatusEdit ( TEdit *ParStatusEdit )
{
   StatusEdit = ParStatusEdit;
}



/*---------------------------------------------------------------------------

	download mail

---------------------------------------------------------------------------*/

GRB::ERR GRB::DownloadMail(void)
{
   AnsiString                          From;                // mail sender
   AnsiString                          Subject;             // mail subject
   CHAR                                MessageIdc[513];     // mail identifier string
   AnsiString                          MessageId;           // mail identifier AnsiString
   AnsiString                          Body;                // mail body text
   AnsiString                          Date;                // mail date and time
   CMEvent                             *Event;              // event object for JD computing
   CMEvent::DATE                       EventDate;           // data for event object
   SYSTEMTIME                          LocalTime;           // current data and time from PC clock
   double                              CurrentJD;           // julian day at current data and time
   double                              MailJD;              // mail reveive julian day
   LHANDLE                             Session;             // mail log on session handle
   unsigned long                       RetVal;              // MAPI function retun code
   MapiMessage                         *Message;            // MAPI message pointer

   Event = new CMEvent();                                   // create object for jd computing

   // get current JD
   GetSystemTime ( &LocalTime );                            // get PC time
   EventDate.Year = LocalTime.wYear;                        // set event structure
   EventDate.Month = LocalTime.wMonth;
   EventDate.Day = LocalTime.wDay;
   EventDate.Hour = LocalTime.wHour;
   EventDate.Minute = LocalTime.wMinute;
   EventDate.Sec = LocalTime.wSecond;
   Event->SetDate( &EventDate );                            // set event
   CurrentJD = Event->GetGG();                              // get current JD

   // log to default mail client
   RetVal = MAPILogon( 0, "", "", MAPI_NEW_SESSION, 0, &Session );
   if ( RetVal != 0 )                                       // operation non possibile
      return ERR_NOLOG;

   // find the first unread message
   RetVal = MAPIFindNext(Session, 0L, NULL, NULL, MAPI_LONG_MSGID | MAPI_UNREAD_ONLY, 0L, MessageIdc);
   // download messages when found
   while  ( RetVal == 0 )
   {
      // show mail download progress message
      StatusEdit->Text = "Scarico mail";
      StatusEdit->Update();

      // get mail parameters
      RetVal = MAPIReadMail( Session, 0, MessageIdc, MAPI_PEEK, 0, &Message);
      From        = AnsiString(Message->lpOriginator->lpszName ); // get sender
      From        = From.SubString(1,50);                      // cut sender to long
      Date        = AnsiString(Message->lpszDateReceived );    // get date
      Subject     = AnsiString( Message->lpszSubject );        // get subject
      Subject     = Subject.SubString(1,100);                  // cut subject to long
      MessageId   = AnsiString( MessageIdc );                  // get message identifier
      Body        = AnsiString( Message->lpszNoteText );       // get mail text

      // set event structure with mail date and time
      EventDate.Year = Date.SubString(1,4).ToInt();            
      EventDate.Month = Date.SubString(6,2).ToInt();
      EventDate.Day = Date.SubString(9,2).ToInt();
      EventDate.Hour = Date.SubString(12,2).ToInt();
      EventDate.Minute = Date.SubString(15,2).ToInt();
      EventDate.Sec = 0;
      Event->SetDate( &EventDate );                            // set event
      MailJD = Event->GetGG();                                 // get mail JD

      if ( (CurrentJD-MailJD) < 7 && Subject.SubString(1,4) == "GCN/" )
      {
         if ( IsMailPresent(From.c_str(), Subject.c_str(), Date.c_str()) == false )
         {
            // add mail in the list
            Add(From.c_str(), Subject.c_str(), Date.c_str(), Body.c_str() );
         }
      }

      // find the next unread message
      RetVal = MAPIFindNext(Session, // explicit session required
                   0L,        // always valid ulUIParam
                   NULL,      // NULL specifies interpersonal messages
                   MessageIdc,      // seed message ID; NULL=get first message
                   MAPI_LONG_MSGID | // needed for 512 byte rgchMsgID.
                   MAPI_UNREAD_ONLY, // only get unread messages.
                   0L,        // reserved; must be 0

                   MessageIdc);// buffer to get back a message ID.

   }

   // show waiting message
   StatusEdit->Text = "Attesa";
   RetVal = MAPILogoff( Session, 0, 0, 0 );

   delete Event;

   return ERR_NONE;
}



/*---------------------------------------------------------------------------

	get mail count

---------------------------------------------------------------------------*/

int GRB::GetMailsCount( void )
{
   return MailCount;
}



/*---------------------------------------------------------------------------

	Set list pointer to the first element

---------------------------------------------------------------------------*/

void GRB::First( void )
{
   ScanElement = FirstElement;
}



/*---------------------------------------------------------------------------

	Set list pointer to the last element

---------------------------------------------------------------------------*/

void GRB::Last( void )
{
   ScanElement = LastElement;
}



/*---------------------------------------------------------------------------

	get next mail in the list

---------------------------------------------------------------------------*/

GRB::ERR GRB::GetNextMail( char *From, char *Subject, char *Date, char **Text )
{
   // when scan element is null no other mail in the list
   if ( ScanElement == 0 )
      return ERR_ENDOFLIST;
   else
   {
      // yeld data fron scan element
      strcpy( From,    ScanElement->From );
      strcpy( Subject, ScanElement->Subject );
      strcpy( Date,    ScanElement->Date );
      *Text = ScanElement->Text;
      // prepare per next time scanelement to previous one
      ScanElement = ScanElement->Next;
   }

  	return ERR_NONE;
}



/*---------------------------------------------------------------------------

	get previous mail in the list

---------------------------------------------------------------------------*/

GRB::ERR GRB::GetPrevMail( char *From, char *Subject, char *Date, char **Text )
{
   // when scan element is null no other mail in the list
   if ( ScanElement == 0 )
      return ERR_ENDOFLIST;
   else
   {
      // yeld data fron scan element
      strcpy( From,    ScanElement->From );
      strcpy( Subject, ScanElement->Subject );
      strcpy( Date,    ScanElement->Date );
      *Text = ScanElement->Text;
      // prepare per next time scanelement to previous one
      ScanElement = ScanElement->Prev;
   }

  	return ERR_NONE;
}



/*---------------------------------------------------------------------------

   check if mail is already present

---------------------------------------------------------------------------*/

bool GRB::IsMailPresent(char *From, char *Subject, char *Date)
{
   char                                ScanFrom[50];        // scanning source of mail
   char                                ScanSubject[100];     // scanning mail subject
   char                                ScanDate[20];        // scanning mail date
   char                                *ScanText;
   char                                Equal;               // current mail is equal to test

   First();
   while ( GetNextMail( ScanFrom, ScanSubject, ScanDate, &ScanText ) == ERR_NONE )
   {
      Equal = true;
      if ( strcmp(From,    ScanFrom)    != 0)
         Equal = false;
      if ( strcmp(Subject, ScanSubject) != 0)
         Equal = false;
      if ( strcmp(Date,    ScanDate)    != 0)
         Equal = false;
      if ( Equal == true )
         return true;
   }
   return false;
}



/*---------------------------------------------------------------------------

   Understand mail

---------------------------------------------------------------------------*/

bool GRB::UnderstandMail( char *MailText, MAIL_INFO *MailInfo )
{
   DWORD                               ToRead;              // bytes to read
   Tokenizer                           *MailTokenizer;      // tokenizer for mail
   char                                Separators[] = {'-',0,'+',0,':',0,';',0,'[',0,']',0,',',0,'(',0,')',0,'{',0,'}',0,0};
   char                                Token[Tokenizer::MAXSTR];  // currente token
   int                                 Err;                 // error code
   int                                 Line;                // current line
   int                                 Column;              // current column
   int                                 Cntx;                // current context code
   int                                 Cntx2;               // next context code
   int                                 Skip;                // token to skip

   // clear mail info
   memset( MailInfo, 0, sizeof(MAIL_INFO));

   // set a tokenizer on mail buffer
   ToRead = strlen( MailText );
   MailTokenizer = new Tokenizer();
   MailTokenizer->SetBuffer( MailText, ToRead );

   // set separator
   MailTokenizer->SetSeparators( Separators );

   // Scan mail
   Cntx = -1;                                               // invalid context code
   do
   {
      Err = MailTokenizer->Read( Token, &Line, &Column );   // get new token
      if ( Err == Tokenizer::ERR_NONE )
      {
         if ( Column == 1 )                                  // token in first colum  means new parameter name
            Cntx = Cntx2 = 0;
         switch ( Cntx )
         {
            // wait for parameter name
            case 0:
               Cntx2 = -1;                                  // invalid next token
               if ( strcmp(Token, "TRIGGER_INDEX") == 0 )   // check for trigger index
                  Cntx2 = 10;
               if ( strcmp(Token, "GRB_RA") == 0 )          // check for right ascension
               {
                  Cntx2 = 20;
                  Skip = 14;
               }
               if ( strcmp(Token, "GRB_DEC") == 0 )         // check for declination
               {
                  Cntx2 = 30;
                  Skip = 15;
               }
               Cntx = 1;
               break;
            // wait for column
            case 1:
               if ( strcmp( Token, ":" ) != 0 )
                  Cntx = -1;                                 // line is not valid
               else
                  Cntx = Cntx2;
               break;
            // wait trigger index value
            case 10:
               MailInfo->TriggerIndex = atoi( Token);
               Cntx = -1;                                   // skip all other tokens in the same line
               break;
            // bypass token before RA value
            case 20:
               Skip --;
               if ( Skip == 0 )
                  Cntx = 21;
               break;
            // wait for hour RA
            case 21:
               MailInfo->RA.hh = atoi( Token );
               Cntx++;
               break;
            // wait for minite RA
            case 22:
               MailInfo->RA.mm = atoi( Token );
               Cntx++;
               break;
            // wait for second RA
            case 23:
               MailInfo->RA.ss = atoi( Token );
               Cntx = -1;
               break;
            // bypass token before RA value
            case 30:
               Skip --;
               if ( Skip == 0 )
                  Cntx = 31;
               break;
            // wait for signi declination
            case 31:
               strcpy( MailInfo->Dec.sign, Token );
               Cntx++;
               break;
            // wait for degree declination
            case 32:
               MailInfo->Dec.gg = atoi( Token );
               Cntx++;
               break;
            // wait for minite declination
            case 33:
               MailInfo->Dec.mm = atoi( Token );
               Cntx++;
               break;
            // wait for second declination
            case 34:
               MailInfo->Dec.ss = atoi( Token );
               Cntx = -1;
               break;
         }
      }
   }while ( Err == Tokenizer::ERR_NONE);

   delete MailTokenizer;

   return true;
}



/*---------------------------------------------------------------------------

   Clear list

---------------------------------------------------------------------------*/

void GRB::ClearList(void)
{
   LIST_MAIL                           *CurrentElement;     // Current element to delete
   LIST_MAIL                           *NextElement;        // Next element to delete

   // only if list is not empty
   CurrentElement = FirstElement;
   while (CurrentElement != 0 )
   {
      NextElement = CurrentElement->Next;                   // save next element
      free( CurrentElement->Text );                         // free text buffer
      free( CurrentElement);                                // free element list structure
      CurrentElement = NextElement;                         // now the next is the first
   }
   MailCount = 0;
   FirstElement = 0;
   LastElement = 0;
}



/*---------------------------------------------------------------------------

   Add an mail to list

---------------------------------------------------------------------------*/

void GRB::Add(char *From, char *Subject, char *Date, char *Text)
{
   LIST_MAIL                           *NewElement;         // new added element
   LIST_MAIL                           *NextElement;        // next insertion element
   LIST_MAIL                           *PrevElement;        // previous insertion element
   int                                 TextLen;             // text lenght
   char                                *Buffer;
   char                                NextDate[30];        // date in yyyymmddhhmmss format for next element

   // allocate buffer for text length
   TextLen = strlen( Text );
   Buffer = (char*)malloc( TextLen + 1);
   strcpy( Buffer, Text );

   // create and fill new element
   NewElement = (LIST_MAIL*)malloc(sizeof(LIST_MAIL));
   strcpy( NewElement->From, From);
   strcpy( NewElement->Subject, Subject);
   strcpy( NewElement->Date, Date);
   NewElement->Text = Buffer;
   NewElement->Next = 0;
   NewElement->Prev = 0;

   // when list is empty this is the first and the last
   if ( FirstElement == 0 )
   {
      LastElement = NewElement;
      FirstElement = NewElement;
   }
   else
   {
      // find the insert position in data&time order (most recent at the beginning)
      NextElement = FirstElement;
      PrevElement = 0;
      while ( NextElement != 0 )
      {
         strcpy( NextDate, NextElement->Date );
         if ( strcmp(Date, NextDate) > 0 )
            break;
         PrevElement = NextElement;
         NextElement = NextElement->Next;
      }

      // link new record to link chain
      NewElement->Next = NextElement;
      NewElement->Prev = PrevElement;

      // adjust previous record link
      if ( PrevElement == 0 )
         FirstElement = NewElement;
      else
         PrevElement->Next = NewElement;

      // adjust next record link
      if ( NextElement == 0 )
         LastElement = NewElement;
      else
         NextElement->Prev = NewElement;
   }
   MailCount++;                                           // increment object counter
}

