/******************************************************************************

	CCD.cpp | GSC browser

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : May 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// System include
#include <math.h>
#include <dir.h>
#include <stdio.h>



// project include
#include "GSC.h"
#include "CMStar.h"
#pragma package(smart_init)



// list of band code
static int                             BandCode[] = { 0, 1, 6, 8, 10, 11, 12, 13, 14, 18, 4, 3, 16, -1 };



/****************************************************************************

   Object constructor

*****************************************************************************/

GSC::GSC(char *PGSCPath)
{
   HANDLE                              hRegion;             // regions index file handle
   char                                FileName[MAXPATH];   // file name
   BY_HANDLE_FILE_INFORMATION          Info;                // file information
   DWORD                               ToRead;              // bytes to read
   DWORD                               Read;                // bytes read
   char                                *Buffer;             // file buffer
   char                                *Scan;               // scan file pointer
   int                                 Region;              // region number
   CMHours                             *RA;                 // RA for conversion
   CMDegree                            *Dec;                // declination for conversion
   CMHours::HOURS                      RAMin;               // RA minimum
   CMHours::HOURS                      RAMax;               // RA maximum
   CMDegree::DEGREE                    DecMin;              // Dec minimum
   CMDegree::DEGREE                    DecMax;              // Dec maximum
   double                              Dec1;                // first declination
   double                              Dec2;                // second declination

   // store GSC path
   strcpy( GSCPath, PGSCPath );

   // create local objects
   RA = new CMHours();
   Dec = new CMDegree();

   // load region file
   strcpy( FileName, GSCPath );
   strcat( FileName, "regions.dat" );
   hRegion = CreateFile( FileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL );
   if ( hRegion == INVALID_HANDLE_VALUE )
      return;
   GetFileInformationByHandle( hRegion, &Info );            // get file info
   ToRead = Info.nFileSizeLow;                              // file lenght
   Buffer = (char*) malloc( ToRead );                       // alloc buffer for whole file reading
   ReadFile ( hRegion, Buffer, ToRead, &Read, NULL);        // read all file in one block
   CloseHandle( hRegion);                                   // close file

   // Scan buffer extracting region parameters
   Scan = Buffer;                                           // scan pointer at buffer begining
   NumRegions=0;                                            // region index
   while ( Read > 0 )
   {
      // scan ascii parameter in the buffer
      sscanf( Scan, "%d %d %d %f %d %d %f %1s%d %f %1s%d %f", &Region,
                                                        &RAMin.hh, &RAMin.mm , &RAMin.ss,
                                                        &RAMax.hh, &RAMax.mm, &RAMax.ss,
                                                        &DecMin.sign, &DecMin.gg, &DecMin.ss,
                                                        &DecMax.sign, &DecMax.gg, &DecMax.ss );
      // adjust declinations
      DecMin.mm = DecMin.ss;                                // in DecMin.ss there is minutes minimum declination with decimal
      DecMin.ss -= DecMin.mm;                               // remove integer part
      DecMin.ss *= 60;                                      // tranform decimal part in seconds
      DecMax.mm = DecMax.ss;                                // in DecMax.ss there is minutes maximum declination with decimal
      DecMax.ss -= DecMax.mm;                               // remove integer part
      DecMax.ss *= 60;                                      // tranform decimal part in seconds

      // insert parameters into array
      Regions[NumRegions].Number = Region;                  // region number
      RA->Set( &RAMin );                                    // set RA to convert it in a unique value
      Regions[NumRegions].RAMin = RA->GetC();               // minimum RA
      RA->Set( &RAMax );                                    // set RA to convert it in a unique value
      Regions[NumRegions].RAMax = RA->GetC();               // maximum RA
      if ( Regions[NumRegions].RAMax == 0 )                 // in order to facilitate region & window comparation..
         Regions[NumRegions].RAMax = 24;                    // maximum limit = 0 is converted to 24
      Dec->Set( &DecMin );                                  // set declination to convert it in a unique value
      Dec1 = Dec->GetC();                                   // first declination
      Dec->Set( &DecMax );                                  // set declination to convert it in a unique value
      Dec2 = Dec->GetC();                                   // second declination
      if ( Dec1 < Dec2 )                                    // regions declination are not in order
      {
         Regions[NumRegions].DecMin = Dec1;                 // first declination s minimum
         Regions[NumRegions].DecMax = Dec2;                 // second declination is maximum
      }
      else
      {
         Regions[NumRegions].DecMin = Dec2;                 // second declination is minimum
         Regions[NumRegions].DecMax = Dec1;                 // first declination s maximum
      }

      NumRegions++;                                         // increment region index

      // search next new line character
      while ( *Scan != '\n' && Read > 0 )
      {
         Read--;
         Scan++;
      }

      // bypass new line character
      Read--;
      Scan++;
   }

   // release local objects
   free ( Buffer );
   delete RA;
   delete Dec;
}



/****************************************************************************

   Object destroyer

*****************************************************************************/

GSC::~GSC()
{
}



/****************************************************************************

   get star list inside a sky window

*****************************************************************************/

CMObjectList * GSC::GetStars( CMHours::HOURS *ARCenter, CMDegree::DEGREE *DecCenter, int Radius )
{
   CMObjectList                        *List;               // star list
   int                                 IndexRegion;         // index for regnio scanning
   CMHours                             *AR;                 // AR for conversion
   CMDegree                            *Dec;                // declination for conversion
   double                              Right;               // finding window right border
   double                              Left;                // finding window left border
   double                              Top;                 // finding window upper border
   double                              Bottom;              // finding window lower border
   double                              ARRadius;            // radius in hours
   double                              DecRadius;           // radius in degree
   HANDLE                              hRegion;             // region file handle
   char                                FileName[MAXPATH];   // file name
   char                                Str[10];             // string for conversion
   char                                Str2[10];            // string for conversion
   BY_HANDLE_FILE_INFORMATION          Info;                // file information
   DWORD                               ToRead;              // bytes to read
   DWORD                               Read;                // bytes read
   unsigned char                       *Buffer;             // file buffer
   HEADER                              Header;              // file header
   GSCREC                              Star;                // star definition
   unsigned char                       *Scan;               // scan pointer in the buffer
   CMStar                              *ObjStar;            // object star
   CMCoEquatorial                     *PosBase;            // star base position

   // create the list
   List = new CMObjectList();

   // create local objects
   AR = new CMHours();
   Dec = new CMDegree();

   // compute finding window borders
   ARRadius = Radius;
   ARRadius /= 3600;                                        // compute radius in degree
   if ( (DecCenter->gg + ARRadius) > 87 )                   // near pole no filter in AR
   {
      ARRadius = 0;
      Left = 0;
      Right = 0;
   }
   else
   {
      ARRadius /= cos( (double)DecCenter->gg / RAD );       // enlarge radius depending on declination
      ARRadius /= 15;                                       // compute radius in hours
      AR->Set(ARCenter);                                    // set the window center
      AR->Add( ARRadius );                                  // add radius
      Right = AR->GetC();                                   // compact value
      AR->Set( ARCenter);                                   // set the window center
      AR->Sub( ARRadius );                                  // subtract radius
      Left = AR->GetC();                                    // compact value
   }
   DecRadius = Radius;                                      // compute radius in degree
   DecRadius /= 3600;
   Dec->Set( DecCenter );                                   // set the window center
   Dec->Add( DecRadius );                                   // add radius
   Top = Dec->GetC();                                       // compact value
   Dec->Set( DecCenter );                                   // set the window center
   Dec->Sub( DecRadius );                                   // subctract radius
   Bottom = Dec->GetC();                                    // compact value

   // scan all regions in order to verify if finding window intersec them
   for ( IndexRegion=0; IndexRegion<NumRegions; IndexRegion++ )
   {
      // bypass non interseting regions
      if ( RegionIntersect( IndexRegion, Right, Left, Top, Bottom ) == false )
         continue;

      // load region file
      strcpy( FileName, GSCPath );                          // path for GSC catalog
      itoa( IndexRegion+10001, Str, 10 );                   // file name index starts from 1 with 0 padding
      strcat( FileName, Str+1 );
      strcat( FileName, ".GSC" );                           // fixed extension
      hRegion = CreateFile( FileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL );  // open file
      if ( hRegion == INVALID_HANDLE_VALUE )                // if file non open bypass region
      {
         MessageBox( 0, FileName, "Manca", 0);
         continue;
      }
      GetFileInformationByHandle( hRegion, &Info );         // get information for buffer size allocation
      ToRead = Info.nFileSizeLow;                           // file lenght
      Buffer = (unsigned char*) malloc( ToRead );           // allocate buffer for all file
      ReadFile ( hRegion, Buffer, ToRead, &Read, NULL);     // read all file in one block
      CloseHandle( hRegion);                                // close file

      // scan file
      Scan = GetHeader( Buffer, &Header );                  // get header information
      Read -= Header.Len;                                   // cut header lenght
      while ( Read > 0 )                                    // scan buffer until bytes again present
      {
         Scan = GetStar( Scan, &Header, &Star );            // get star inforrmation
         Read -= 12;                                        // cut star lenght

         // oggetti non stellari
         if ( Star.Class == 3 )
            continue;
         // bypass star don't fit in the finding window
         if ( Right != 0 && Left != 0 )                     // right and left == 0 means no test
         {
            if ( Right > Left )                             // normal situation
            {
               if ( Star.RA > Right || Star.RA < Left )     // bypass stars out of AR window
                  continue;
            }
            else
            {                                               // view window crossed bt AR 0.0
               if ( Star.RA < 12 && Star.RA > Right )       // bypass stars too right
                  continue;
               if ( Star.RA > 12 && Star.RA < Left )        // bypass start too left
                  continue;
            }
         }

         if ( Star.Dec > Top || Star.Dec < Bottom )         // bypass stars out of declination window
            continue;

         // this star is into the finding window
         ObjStar = new CMStar();                            // create new star object
         itoa( Star.Region, Str, 10 );                         // region
         strcat( Str, "-" );
         itoa( Star.Id, Str2, 10 );                         // id
         strcat( Str, Str2 );
         ObjStar->SetName( Str, CMObject::NAME_CATALOG );                           // set name as catalog number
         PosBase = new CMCoEquatorial();                   // object for star coordinate
         PosBase->GetRA()->Set( Star.RA );                  // set base AR
         PosBase->GetDec()->Set( Star.Dec );                // set base declination
         PosBase->SetDataEqu(2451545);                      // set equinox 
         ObjStar->SetBase(2451545, PosBase, 0, 0);          // set position
         ObjStar->SetMag( Star.Mag );                       // set magnitude
         delete PosBase;


         // add star to list
         List->Add(ObjStar);
      }

      free( Buffer );                                       // free file buffer
   }

   // release local object
   delete AR;
   delete Dec;

   return List;
}



/****************************************************************************

   get header information from file buffer

*****************************************************************************/

unsigned char * GSC::GetHeader( unsigned char *Buffer, HEADER *Header )
{
   int                                 HeaderLen;           // len of header in buffer
   char                                *Scan;               // scan pointer

   // initialization
	memset( Header, 0, sizeof(HEADER));
   Scan = (char*)Buffer;

   // identify header len
   HeaderLen = atoi(Scan);                                  // header len is in ASCII at top of the file
   Header->Len = HeaderLen;                                 // save len the the header structure
   Scan += 4;                                               // skip len in buffer

   // get information from header
	Header->Version = atoi(Scan);                            // file version
   Scan++;                                                  // goto next information
	while(*Scan != ' ')
      Scan++;
	Header->Region = atoi(Scan);                             // region number
   Scan++;                                                  // goto next information
	while(*Scan != ' ')
      Scan++;
	Header->ObjectNumber = atoi(Scan);                       // obejct number
   Scan++;                                                  // goto next information
	while(*Scan != ' ')
      Scan++;
	Header->RAMin = atof(Scan);                              // Minimum right ascension
   Scan++;                                                  // goto next information
	while(*Scan != ' ')
      Scan++;
	Header->RAMax = atof(Scan);                              // Maximum right ascension
   Scan++;                                                  // goto next information
	while(*Scan != ' ')
      Scan++;
	Header->DecMin = atof(Scan);                             // Minimum declination
   Scan++;                                                  // goto next information
	while(*Scan != ' ')
      Scan++;
	Header->DecMax = atof(Scan);                             // maximum declination
   Scan++;                                                  // goto next information
	while(*Scan != ' ')
      Scan++;
	Header->MagOffset = atof(Scan);                          // magnitude offset
   Scan++;                                                  // goto next information
	while(*Scan != ' ')
      Scan++;
	Header->RAScale = atof(Scan);                            // right ascension Scale
   Scan++;                                                  // goto next information
	while(*Scan != ' ')
      Scan++;
	Header->DecScale = atof(Scan);                           // Declination scale
   Scan++;                                                  // goto next information
	while(*Scan != ' ')
      Scan++;
	Header->ErrorScale = atof(Scan);                         // position error scale
   Scan++;                                                  // goto next information
	while(*Scan != ' ')
      Scan++;
	Header->MagScale = atof(Scan);                           // magnitude scale
   Scan++;                                                  // goto next information
	while(*Scan != ' ')
      Scan++;
	Header->PlateNum = atoi(Scan);                           // number of plates for this region
   Scan++;                                                  // goto next information
	while(*Scan != ' ')
      Scan++;
	Header->PlateList = Scan;                                // pointer to plate list names

   return Buffer+HeaderLen;                                 // first star starts at the end of header in file
}



/****************************************************************************

   get next star from file buffer

*****************************************************************************/

unsigned char * GSC::GetStar( unsigned char *Punt, HEADER *Header, GSCREC *Star )
{
   int                                 Id;                  // id from file
   unsigned int                        RA;                  // right ascension from file
   unsigned int                        Dec;                 // declination from file
   unsigned int                        PosError;            // position errore from file
   unsigned int                        Mag;                 // magnitude from file
   unsigned int                        MagError;            // magnitude error from file
	int                                 BandIndex;           // band index from file
   unsigned int                        Class;               // class object from file
   unsigned int                        PlateIndex;          // index of the region plate
   unsigned int                        MultipCode;          // *
	char                                *PlateName;          // pointer to plate name on header
	char                                *PlateDate;          // pointer to plate date on header

   // decode star ID
	Id = (Punt[0] & 127);
	Id <<= 7;
	Id |= (Punt[1] >> 1);

   // decode right ascension
	RA = ((Punt[1]&1)<< 8) | Punt[2];
	RA <<= 8;
	RA |= Punt[3];
	RA <<= 8;
	RA |= Punt[4];
	RA >>= 3;

   // decode declination
	Dec = Punt[4] & 7;
	Dec <<= 8;
	Dec |= Punt[5];
	Dec <<= 8;
	Dec |= Punt[6];

   // decode position error
	PosError = Punt[7];
	PosError <<= 1;
	PosError |= Punt[8] >> 7;

   // decode magnitude
	Mag = Punt[9];
	Mag <<= 3;
	Mag |= Punt[10] >> 5;

   // decode magnitude error
	MagError = Punt[8] & 127;

   // decode band index
	BandIndex = (Punt[10] >> 1) & 15;

   // decode class
	Class = (Punt[11] >> 4) & 7;

   // copy plate name
	PlateIndex = Punt[11] & 15;
	PlateName = Header->PlateList;
   PlateName += 5 * PlateIndex + 1;
	strncpy(Star->PlateName,PlateName,4);
	Star->PlateName[4] = '\0';

   // copy plate shot data
	PlateDate = Header->PlateList+5*Header->PlateNum;
	PlateDate += 9*PlateIndex + 1;
	if (*PlateDate != ' ')
      Star->PlateData = atof(PlateDate);
	else
      Star->PlateData = 0;

   // ?
	MultipCode = Punt[10] & 1;
	if(MultipCode == 0)
      Star->MultipCode = 'F';
	if(MultipCode == 1)
      Star->MultipCode = 'T';

   // assign data to output record
	Star->Id = Id;
	Star->RA = (double)RA / Header->RAScale + Header->RAMin;
	if (Star->RA < 0)
      Star->RA += 360;
	if (Star->RA >= 360)
      Star->RA -= 360;
   Star->RA /= 15;
	Star->Dec = (double)Dec / Header->DecScale + Header->DecMin;
	Star->PosError = (double)PosError / Header->ErrorScale;
	Star->Mag = (double)Mag / Header->MagScale + Header->MagOffset;
	Star->MagError = (double)MagError / Header->MagScale;
	Star->Region = Header->Region;
	Star->Band = BandCode[BandIndex];
   Star->Class = Class;

   return Punt+12;
}



/****************************************************************************

   Verify finding window region intersection

*****************************************************************************/

bool GSC::RegionIntersect( int IndexRegion, double Right, double Left, double Top, double Bottom )
{
   double               ARMin;;

   if ( Bottom > Regions[IndexRegion].DecMax )           // bypass region too low
      return false;
   if ( Top < Regions[IndexRegion].DecMin )              // bypass region to high
      return false;

   if ( Right != 0 && Left != 0 )                        // right and left == 0 means no test
   {
      if ( Right > Left )                                // normal situation
      {
         if ( Left > Regions[IndexRegion].RAMax )        // bypass region too left
            return false;
         if ( Right < Regions[IndexRegion].RAMin )       // bypass region too right
            return false;
      }
      else
      {                                                  // view window crossed bt AR 0.0
         if ( Left > Regions[IndexRegion].RAMax+24 )     // bypass region too left
            return false;
         ARMin = Regions[IndexRegion].RAMin;             // region right border for comparing
         if ( Regions[IndexRegion].RAMax == 24 )         // if it is 24 should use 0
            ARMin -= 24;
         if ( Right < ARMin )                            // bypass region too right
            return false;
      }
   }
   return true;
}



