/*---------------------------------------------------------------------------

	ImageAnalyse.cpp | Analyse an images in order to detect objects using
                      Source EXctractor program

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : October 2012

   This C++ object is used in more than one astromical software developed at
   Bassano Bresciano Observatory  
   Copyright (C) 2009,2010,2011,2012  Luca Pietro Strabla
   This file is part of Polipus and SNF
   Polypus ans SNF are free software: you can redistribute it and/or modify
   them under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus ans SNF are distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it

---------------------------------------------------------------------------*/



// System include
#include <dir.h>
#include <stdio.h>



// project include
#include "ImageAnalyse.h"
#pragma package(smart_init)



/*---------------------------------------------------------------------------

   object constructor

---------------------------------------------------------------------------*/

ImageAnalyse::ImageAnalyse( char *PPath, char *PName )
{
   strcpy( Path, PPath );                                   // store path for all SExtractor files
   strcpy( Name, PName );                                   // store name for SExtractor files
   HasParameters = false;                                   // no parameters are present
   FirstObject = 0;                                         // erase internal pointers
   LastObject = 0;
   ObjectCount = 0;                                         // erase object counter
}



/*---------------------------------------------------------------------------

   Object destroyer

---------------------------------------------------------------------------*/

ImageAnalyse::~ImageAnalyse()
{
   FreeChain();                                             // release all objects
}



/*---------------------------------------------------------------------------

   set analyse parameters

---------------------------------------------------------------------------*/

void ImageAnalyse::SetAnalyseParameters( ANALYSE* PAnalyseParameters )
{
   if ( PAnalyseParameters->MinDetectionPixel  != AnalyseParameters.MinDetectionPixel  ||
        PAnalyseParameters->DetectionThreshold != AnalyseParameters.DetectionThreshold ||
        PAnalyseParameters->ClassThreshold     != AnalyseParameters.ClassThreshold     ||
        PAnalyseParameters->SaturationLevel    != AnalyseParameters.SaturationLevel    ||
        PAnalyseParameters->SeeinfFWHM         != AnalyseParameters.SeeinfFWHM            )
      ParametersChanged = true;                             // parameters files need to be created

   AnalyseParameters = *PAnalyseParameters;                 // copy input parameters
   HasParameters = true;                                    // parameters are present
}



/*---------------------------------------------------------------------------

   image analyse

---------------------------------------------------------------------------*/

ImageAnalyse::ERR ImageAnalyse::Analyse( char *ImagePath )
{
   ERR                                 Error;               // error code
   char									      FileName[MAXPATH];   // file name in current directory
   char									      Com[MAXPATH];        // command for sextractor run
   char									      Par[MAXPATH];        // command line parameter for sextractor run
   STARTUPINFO							      SInfo;               // info parameter for sextractor run
   PROCESS_INFORMATION				      PInfo;               // process retur parameter for sextractor run

   // initialization
   Error = ERR_NONE;                                        // no error

   // check for parameters set
   if ( HasParameters == false )
      Error = ERR_NOPARAMETERS;

   // check for parameters change
   if ( Error == ERR_NONE && ParametersChanged == true )
      Error = CreateSExtractorFiles( );                     // create new source extractor files

   // copy image to analyse into current directory with short name
   strcpy( FileName, Name );                                // name is object name
   strcat ( FileName, ".fit" );                             // fit file
   CopyFile( ImagePath, FileName, false );                  // copy with overwrite

   // run source extractor
   strcpy( Com, "sex.exe" );                                // command
   strcpy( Par, Com );                                      // command line parameter
   strcat( Par, " " );
   strcat( Par, FileName );                                 // image to analyse
   strcat( Par, " -c ");                                    // option form main option file
   strcat( Par, Name );                                     // name as object name
   strcat( Par, ".sex" );                                   // source estractor extension
   memset( &SInfo, 0,  sizeof( SInfo ) );                   // clear run information
   SInfo.cb = sizeof( SInfo );                              // lenght at th head
   SInfo.wShowWindow = SW_HIDE;                             // hide running
   CreateProcess( Com, Par, NULL, NULL, false,              // run source extractor
               NORMAL_PRIORITY_CLASS,
               NULL, NULL, &SInfo, &PInfo );

   // wait for program end
   while ( GetProcessVersion( PInfo.dwProcessId ) != 0 )
      Sleep( 500 );
   return ERR_NONE;
}



/*---------------------------------------------------------------------------

   build object list

---------------------------------------------------------------------------*/

ImageAnalyse::ERR ImageAnalyse::BuildObjectList( void )
{
   ERR                                 Error;               // error code
   char									      FileName[MAXPATH];   // file name in current directory
   HANDLE                              hFile;               // file handle
   BY_HANDLE_FILE_INFORMATION          Info;                // file information
   DWORD                               ToRead;              // bytes to read
   DWORD                               Read;                // bytes read
   char                                *Buffer;             // file buffer
   char                                *Scan;               // scan pointer in the buffer
   OBJECT                              Object;              // object found in the image

   // initialization
   hFile = INVALID_HANDLE_VALUE;                            // file handle not valid
   Buffer = 0;                                              // nul buffer pointer
   FreeChain();

   // get output values
   strcpy( FileName, Path );
   strcat( FileName, Name );                                // compose output file name
   strcat ( FileName, ".cat" );
   hFile = CreateFile( FileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL );  // open file
   if ( hFile == INVALID_HANDLE_VALUE )                     // check open fail
      Error = ERR_SEXOUTPUT;
   if ( Error == ERR_NONE )
   {
      GetFileInformationByHandle( hFile, &Info );           // get information for buffer size allocation
      ToRead = Info.nFileSizeLow;                           // file lenght
      Buffer = (char*) malloc( ToRead );                    // allocate buffer for all file
      if ( Buffer == 0 )                                    // if not enought memory show error dialog
         Error = ERR_MEMORY;
   }
   if ( Error == ERR_NONE )
      ReadFile ( hFile, Buffer, ToRead, &Read, NULL);       // read all file in one block
   if ( hFile != INVALID_HANDLE_VALUE )
      CloseHandle( hFile);                                  // close file

   // read output file and put them into
   if ( Error == ERR_NONE )
   {
      // Scan buffer extracting region parameters
      Scan = Buffer;                                        // scan pointer at buffer begining
      while ( Read > 0 )
      {
         if ( *Scan != '#' )
         {
            // scan ascii parameters in the buffer
            sscanf( Scan, "%lf %lf %lf %lf %lf %d %lf", &Object.Flux, &Object.Magnitude,
               &Object.X, &Object.Y, &Object.FWHM, &Object.Flags, &Object.Classification);
            AddObject( &Object );
         }

         // search next new line character
         while ( *Scan != '\n' && Read > 0 )
         {
            Read--;
            Scan++;
         }

         // bypass new line character
         Read--;
         Scan++;
      }
      free (Buffer);
   }

   return Error;
}



/*---------------------------------------------------------------------------

   update output file

---------------------------------------------------------------------------*/

ImageAnalyse::ERR ImageAnalyse::UpdateOutputFile( void )
{
   return ERR_NONE;
}



/*---------------------------------------------------------------------------

   save object list

---------------------------------------------------------------------------*/

ImageAnalyse::ERR ImageAnalyse::SaveObject( void )
{
   ERR                                 Error;
   char									      FileName[MAXPATH];   // file name in current directory
   char                                Buffer[100];
   OBJECT_CHAIN*                       Scan;                // scan element
   HANDLE                              hFile;               // handle del file
   DWORD                               ToWrite;             // bytes to write
   DWORD                               Written;             // written bytes

   Error = ERR_NONE;
   
   strcpy( FileName, Name );                                // compose output file name
   strcat ( FileName, ".cat" );
   hFile = CreateFile( FileName, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS,	FILE_ATTRIBUTE_ARCHIVE, NULL );  // open file
   if ( hFile == INVALID_HANDLE_VALUE )                     // check open fail
      Error = ERR_SEXOUTPUT;

   Scan = FirstObject;
   while ( Scan != 0 )
   {
      sprintf( Buffer, " %lf %lf %lf %lf %lf %d %lf \r\n", Scan->Actual.Flux, Scan->Actual.Magnitude,
         Scan->Actual.X, Scan->Actual.Y, Scan->Actual.FWHM, Scan->Actual.Flags, Scan->Actual.Classification );
      ToWrite = strlen(Buffer);                      // image size
      WriteFile ( hFile, Buffer, ToWrite, &Written, NULL);  // write image

      Scan = Scan->Next;
   }

   CloseHandle( hFile);

   return Error;
}



/*---------------------------------------------------------------------------

   get object list

---------------------------------------------------------------------------*/

ImageAnalyse::OBJECT_CHAIN * ImageAnalyse::GetObjects( int *PObjectCount )
{
   *PObjectCount = ObjectCount;
   return FirstObject;
}



/*---------------------------------------------------------------------------

   free current block chain

---------------------------------------------------------------------------*/

ImageAnalyse::ERR ImageAnalyse::FreeChain( void )
{
   OBJECT_CHAIN*                       Now;                 // actual block chain element
   OBJECT_CHAIN*                       Next;                // next block chain element

   Now = FirstObject;                                       // first element
   while ( Now != 0 )                                       // scan chain until an element is present
   {
      Next = Now->Next;                                     // get next element
      free( Now );                                          // free actual element
      Now = Next;                                           // goto to the next
   }

   FirstObject = 0;                                         // null first element
   LastObject = 0;
   ObjectCount = 0;                                         // erase object count
   return ERR_NONE;
}



/*---------------------------------------------------------------------------

   Add object to chain

---------------------------------------------------------------------------*/

void ImageAnalyse::AddObject( OBJECT *NewObject )
{
   OBJECT_CHAIN*                       Prev;                // previous element
   OBJECT_CHAIN*                       Scan;                // scan element
   int                                 ScanIndex;           // index for chain scan
   OBJECT_CHAIN*                       New;                 // new chain element

   // Skip objects too border near
   //if ( (NewObject->Flags & 8 ) != 0 )
   //   return;
   // skip objects out of horizontal limits
   if ( NewObject->X < AnalyseParameters.XStart || NewObject->X > AnalyseParameters.XStop )
      return;
   // skip objects out of vertical limits
   if ( NewObject->Y < AnalyseParameters.YStart || NewObject->Y > AnalyseParameters.YStop )
      return;
   // if galaxis not wanted skip object out of FWHM limit
   if ( AnalyseParameters.NoGalaxies == true && NewObject->FWHM > AnalyseParameters.GalaxyFWHM )
      return;

   // list in bright order ?
   if ( AnalyseParameters.MostBright == false )
   {
      // list not in bright order

      New = (OBJECT_CHAIN*)malloc( sizeof(OBJECT_CHAIN) );  // create new parameter
      New->Next = 0;                                        // not next parameters
      New->Actual = *NewObject;                             // copy input parameters

      // check if it is the first object
      if ( LastObject == 0 )
         FirstObject = New;                                 // it is firt object: set as first
      else
         LastObject->Next = New;                            // it isn't the first: set as next of the last
      LastObject = New;                                     // now this object is the last
      ObjectCount++;                                        // increase object count
   }
   else
   {
      // list in bright order

      // search the correct magnitude positione where insert the current object
      Scan = FirstObject;                                   // start from the first objects
      Prev = 0;                                             // no previous object
      ScanIndex = 0;                                        // erase he counter
      while (  Scan != 0 && Scan->Actual.Magnitude < NewObject->Magnitude )  // scan objects until object is brighter
      {
         Prev = Scan;                                       // current object is the previous for next loop
         Scan = Scan->Next;                                 // goto next object
         ScanIndex++;                                       // count objects
      }

      // no insertion if there is a limit and the index if over limit
      if ( AnalyseParameters.MaxObjectCount == 0 || ScanIndex < AnalyseParameters.MaxObjectCount )
      {
         New = (OBJECT_CHAIN*)malloc( sizeof(OBJECT_CHAIN) ); // create new parameter
         New->Next = 0;                                     // not next parameters
         New->Actual = *NewObject;                          // copy input parameters
         ObjectCount++;                                     // increase object count

         New->Next = Scan;                                  // next is the last scan object (the first of less bright)
         if ( Prev == 0 )                                   // if this is brighter than the first
            FirstObject = New;                              // current object became the first
         else
            Prev->Next = New;                               // current object became the next of the previous

         ScanIndex++;                                       // go to next object

         // if a count limitation is active an object need to be removed
         if ( AnalyseParameters.MaxObjectCount != 0 )
         {
            while ( ScanIndex < AnalyseParameters.MaxObjectCount && Scan != 0 ) // scan until object to be removed
            {
               Prev = Scan;                                 // current object is the previous for next loop
               Scan = Scan->Next;                           // goto next object
               ScanIndex++;                                 // count objects
            }
            // if an object was found it have to be removed
            if ( Scan != 0 )
            {
               free( Scan );                                // free current element
               Prev->Next = 0;                              // previous is the last
               ObjectCount--;                               // decrease object counter
            }
         }
      }
   }
}



/*---------------------------------------------------------------------------

   main SExtractor file

---------------------------------------------------------------------------*/

char                                   MainFile[]=
"CATALOG_NAME       %A.cat\r\n"
"CATALOG_TYPE       ASCII_HEAD\r\n"
"\r\n"
"PARAMETERS_NAME    %A.param\r\n"
"\r\n"
"DETECT_TYPE        CCD\r\n"
"DETECT_MINAREA     %B\r\n"
"THRESH_TYPE        RELATIVE\r\n"
"DETECT_THRESH      %C\r\n"
"ANALYSIS_THRESH    %D\r\n"
"\r\n"
"FILTER             Y\r\n"
"FILTER_NAME        default.conv\r\n"
"\r\n"
"DEBLEND_NTHRESH    16\r\n"
"DEBLEND_MINCONT    0.0000001\r\n"
"\r\n"
"CLEAN              Y\r\n"
"CLEAN_PARAM        1.0\r\n"
"\r\n"
"MASK_TYPE          CORRECT\r\n"
"\r\n"
"PHOT_APERTURES     5\r\n"
"PHOT_AUTOPARAMS    2.5, 3.5\r\n"
"SATUR_LEVEL        %E\r\n"
"\r\n"
"MAG_ZEROPOINT      0.0\r\n"
"MAG_GAMMA          4.0\r\n"
"GAIN               1.0\r\n"
"PIXEL_SCALE        1\r\n"
"\r\n"
"SEEING_FWHM        %F\r\n"
"STARNNW_NAME       default.nnw\r\n"
" \r\n"
"BACK_SIZE          16\r\n"
"BACK_FILTERSIZE    7\r\n"
"\r\n"
"BACKPHOTO_TYPE     GLOBAL\r\n"
"BACKPHOTO_THICK    24\r\n"
"\r\n"
"CHECKIMAGE_TYPE    BACKGROUND,APERTURES,SEGMENTATION\r\n"
"CHECKIMAGE_NAME    %A_B.fits,%A_A.fits,%A_S.fits\r\n"
"\r\n"
"MEMORY_OBJSTACK    2000\r\n"
"MEMORY_PIXSTACK    100000\r\n"
"MEMORY_BUFSIZE     1024\r\n"
"\r\n"
"VERBOSE_TYPE       FULL\r\n";



/*---------------------------------------------------------------------------

   param SExtractor file

---------------------------------------------------------------------------*/

char                                   ParamFile[]=

"FLUX_ISOCOR\r\n"
"MAG_ISOCOR\r\n"
"X_IMAGE\r\n"
"Y_IMAGE\r\n"
"FWHM_IMAGE\r\n"
"FLAGS\r\n"
"CLASS_STAR\r\n";



/*---------------------------------------------------------------------------

   create sextractor files

---------------------------------------------------------------------------*/

ImageAnalyse::ERR ImageAnalyse::CreateSExtractorFiles ( void )
{
   ERR                                 Error;               // error code
   char                                OutputBuffer[2000];  // buffer for file creation
   char                                *Param[10];          // pointer to macro parameter
   char                                ParamB[20];          // parameter B value
   char                                ParamC[20];          // parameter C value
   char                                ParamD[20];          // parameter D value
   char                                ParamE[20];          // parameter E value
   char                                ParamF[20];          // parameter F value
   char                                FileName[MAXPATH];   // file name
   HANDLE                              hFile;               // file handle
   DWORD                               ToWrite;             // bytes to write
   DWORD                               Written;             // written bytes

   // initialization
   Error = ERR_NONE;                                        // no error
   hFile = INVALID_HANDLE_VALUE;                            // handle not valid
   Param[0] = Name;                                         // assign parameter to array
   Param[1] = ParamB;
   Param[2] = ParamC;
   Param[3] = ParamD;
   Param[4] = ParamE;
   Param[5] = ParamF;

   // main file creation
   strcpy( FileName, Name );                                // name as image one
   strcat( FileName, ".sex" );
   hFile = CreateFile( FileName, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS,	FILE_ATTRIBUTE_ARCHIVE, NULL );   // file creation
   if ( hFile == INVALID_HANDLE_VALUE )
      Error = ERR_SEXFILES;

   // main file body generation
   if ( Error == ERR_NONE )
   {
      // macro expansion for file body generation with correct parameters
      itoa( AnalyseParameters.MinDetectionPixel, ParamB, 10 ); // macro parameter creation
      gcvt( AnalyseParameters.DetectionThreshold, 3, ParamC );
      gcvt( AnalyseParameters.ClassThreshold, 3, ParamD );
      itoa( AnalyseParameters.SaturationLevel, ParamE, 10 );
      gcvt( AnalyseParameters.SeeinfFWHM, 3, ParamF );
      MacroExpansion( MainFile, OutputBuffer, Param );      // macro expansione

      // write file content
      ToWrite = strlen(OutputBuffer);                       // file size
      WriteFile ( hFile, OutputBuffer, ToWrite, &Written, NULL);// file write
      if ( Written != ToWrite )                             // check write
         Error = ERR_SEXFILES;
   }

   // file close
   if ( hFile != INVALID_HANDLE_VALUE )
      CloseHandle( hFile);
   hFile = INVALID_HANDLE_VALUE;                            // handle not longer valid

   // parameter file creation
   if ( Error == ERR_NONE )
   {
      strcpy( FileName, Name );                             // name as image one
      strcat( FileName, ".param" );
      hFile = CreateFile( FileName, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS,	FILE_ATTRIBUTE_ARCHIVE, NULL ); // file creation
      if ( hFile == INVALID_HANDLE_VALUE )
         Error = ERR_SEXFILES;

      if ( Error == ERR_NONE )
      {
         // write file content
         ToWrite = strlen(ParamFile);                       // file size
         WriteFile ( hFile, ParamFile, ToWrite, &Written, NULL);// file write
         if ( Written != ToWrite )                          // check write
            Error = ERR_SEXFILES;
      }
   }

   // file close
   if ( hFile != INVALID_HANDLE_VALUE )
      CloseHandle( hFile);

   ParametersChanged = false;                               // erase flag for parameter changed (paramater are aligned to sex files)
   return Error;
}



/*---------------------------------------------------------------------------

   macro expansion

---------------------------------------------------------------------------*/

void ImageAnalyse::MacroExpansion( char *Input, char *Output, char **Param )
{
   char                                *pRead;              // pointer for macro reading
   char                                *pWrite;             // pointer for output writing
   char                                *pParam;             // pointer to current parameter
   int                                 ParamIndex;          // parameter index

   // pointer initialization
   pRead = Input;
   pWrite = Output;

   while ( *pRead != 0 )                                    // scan input macro
   {
      // check for macro parameter place holder
      if ( *pRead == '%' )
      {
         ParamIndex = *(pRead+1) - 'A';                     // compute parameter index
         pParam = Param[ParamIndex];                        // get current parameter

         while ( *pParam != 0 )                             // copy parameter
            *pWrite++ = *pParam++;

         pRead += 2;                                        // skip macro parameter place holder
      }
      *pWrite++ = *pRead++;                                 // copy input to output
   }

   *pWrite = 0;                                             // copy terminator

}


