/*---------------------------------------------------------------------------

	ImageCompare.cpp | Compare two images in order to detect differences

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : November 2011

   This C++ object is used in more than one astromical software developed at
   Bassano Bresciano Observatory
   Copyright (C) 2009,2010,2011,2012  Luca Pietro Strabla
   This file is part of Polipus and SNF
   Polypus ans SNF are free software: you can redistribute it and/or modify
   them under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus ans SNF are distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it

---------------------------------------------------------------------------*/



// System include
#include <math.h>
#include <dir.h>
#include <stdio.h>



// project include
#include "ImageCompare.h"
#pragma package(smart_init)



/*---------------------------------------------------------------------------

   object constructor

---------------------------------------------------------------------------*/

ImageCompare::ImageCompare( char *PPath )
{
   strcpy( Path, PPath );                                   // store working path
   FirstDifferenceObject = 0;                               // erase point to first found object
   FirstAlignmentObject = 0;                                // erase point to first align object
   Allignement = ALIGNEMENT_NOTDONE;                        // images not alligned
   Compared = false;                                        // image not compared
   HotPixelTab = 0;                                         // erase pointer to hot pixel table

   TestAnalyse = new ImageAnalyse( PPath, "TEST");          // create object for test image analysing
   RefAnalyse = new ImageAnalyse( PPath, "REF");            // create object for reference image analysing
}



/*---------------------------------------------------------------------------

   Object destroyer

---------------------------------------------------------------------------*/

ImageCompare::~ImageCompare()
{
   FreeDifferenceList();                                    // free star found chain
   FreeAlignmentList();                                     // free star found chain

   delete TestAnalyse;                                      // remove object for test image analysing
   delete RefAnalyse;                                       // remove object for reference image analysing
   if ( HotPixelTab != 0 )                                  // free memory for hot pixel table
      free ( HotPixelTab );
}



/*---------------------------------------------------------------------------

   set hot pixel table

---------------------------------------------------------------------------*/

void ImageCompare::SetHotPixel( int PHotPixelCount, int *PHotPixelTab )
{
   if ( HotPixelTab != 0 )                                  // remove previous table
      free ( HotPixelTab );
   HotPixelTab = (int*) malloc( PHotPixelCount*8 );         // memory allocation for new table (2 INT for every pint)
   memcpy( HotPixelTab, PHotPixelTab, PHotPixelCount*8 );   // copy table
   HotPixelCount = PHotPixelCount;                          // save point count
}



/*---------------------------------------------------------------------------

   set solving parameters

---------------------------------------------------------------------------*/

void ImageCompare::SetSolvingParam( SOLVING* ASolving )
{
   Solving = *ASolving;                                     // parameters from client application
}



/*---------------------------------------------------------------------------

   set images

---------------------------------------------------------------------------*/

ImageCompare::ERR ImageCompare::SetImages( char *PRefImageName, char *PTestImageName )
{
   Fits::ERR                           FitsError;           // error code on Fits object
   ERR                                 Error;               // error code
   Fits                                *Image;              // fits image object

   // save input parameters
   strcpy( RefImageName, PRefImageName );
   strcpy( TestImageName, PTestImageName );

   // initialization
   Allignement = ALIGNEMENT_NOTDONE;                        // images not alligned
   Compared = false;                                        // images not compared
   XShift = 0;                                              // erase allignment parameters
   YShift = 0;
   Rotation = 0;
   FreeDifferenceList();                                    // free difference chain
   FreeAlignmentList();                                     // free alignment chain
   Error = ERR_NONE;

   Image = new Fits();                                      // create object for fits file reading

   // load test image
   FitsError = Image->LoadFile(TestImageName);              // load test image from file
   if ( FitsError != Fits::ERR_NONE )                       // check error code
      Error = ERR_NOTESTIMAGE;

   if ( Error == ERR_NONE )
   {
      Image->GetImageSize( &TestImageInfo.SizeY, &TestImageInfo.SizeX ); // get image size

      // load reference image
      FitsError = Image->LoadFile(RefImageName);            // load reference image from file
      if ( FitsError != Fits::ERR_NONE )                    // check error code
         Error = ERR_NOREFIMAGE;
   }
   if ( Error == ERR_NONE )
      Image->GetImageSize( &RefImageInfo.SizeY, &RefImageInfo.SizeX );   // get image size

   delete Image;                                            // destroy object for fits file reading
   return Error;
}



/*---------------------------------------------------------------------------

   alligne image automatically

---------------------------------------------------------------------------*/

ImageCompare::ERR ImageCompare::ImageAllignment ( void )
{
   int                                 CurrentShift;        // current shift used for alignment searching
   int                                 MaxShift;            // maximum shift is available

   // Set parameter for test image analysing
   AnalyseParameters.MaxObjectCount = MOST_BRIGHT_OBEJCTS;  // only most bright objects
   AnalyseParameters.MostBright = true;
   AnalyseParameters.MinDetectionPixel = Solving.MinDetectionPixel; // detection parameter from user
   AnalyseParameters.DetectionThreshold = Solving.DetectionThreshold;
   AnalyseParameters.ClassThreshold = Solving.ClassThreshold;
   AnalyseParameters.SaturationLevel = Solving.SaturationLevel;
   AnalyseParameters.SeeinfFWHM = 1;                        // it is only a scale factor
   // analyse test image
   TestAnalyse->SetAnalyseParameters(&AnalyseParameters);
   TestAnalyse->Analyse( TestImageName );

   // set parameters for reference image analysing
   AnalyseParameters.MaxObjectCount = NEAR_BRIGHT_OBJECTS;  // only bright near objects
   AnalyseParameters.MostBright = true;
   AnalyseParameters.MinDetectionPixel = Solving.MinDetectionPixel; // detection parameter from user
   AnalyseParameters.DetectionThreshold = Solving.DetectionThreshold;
   AnalyseParameters.ClassThreshold = Solving.ClassThreshold;
   AnalyseParameters.SaturationLevel = Solving.SaturationLevel;
   AnalyseParameters.SeeinfFWHM = 1;                        // it is only a scale factor
   // analyse reference image
   RefAnalyse->SetAnalyseParameters(&AnalyseParameters);
   RefAnalyse->Analyse( RefImageName );

   // compute maximum shift for these images
   MaxShift = TestImageInfo.SizeX/4;                        // at least half image to every direction (so a quarter imgage shift)
   if ( MaxShift > TestImageInfo.SizeY/4 )
      MaxShift = TestImageInfo.SizeY/4;

   // search allignement incrementing possibile shift
   CurrentShift = SHIFT_STEP;                               // starting with one shift step
   do          
   {
      if ( TryAllignment( CurrentShift ) == true )          // try alignement with current shift
         Allignement = ALIGNEMENT_DONE;                     // mark alignment was done
      if ( CurrentShift == MaxShift )                       // was shift the maximum ?
         break;
      CurrentShift += SHIFT_STEP;                           // increase shift
      if ( CurrentShift > MaxShift )                        // shift clamp
         CurrentShift = MaxShift;
   } while ( Allignement == ALIGNEMENT_NOTDONE );           // iteration until alignment is done

   if ( Allignement == ALIGNEMENT_DONE )                    // error management
      return ERR_NONE;

   return ERR_NOALLIGN;
}



/*---------------------------------------------------------------------------

   execute manual alignment

---------------------------------------------------------------------------*/

ImageCompare::ERR ImageCompare::ManualAllignment( TWO_STARS_ALLIGNMENT *ManualParam )
{
   ERR                                 Error;               // internal error
   double                              TanNum;              // tangent numerator
   double                              TanDen;              // tangent denumerator
   double                              TestAngle;           // star angle in test image
   double                              RefAngle;            // star angle in reference image
   double                              XCent;               // horizontal from center position
   double                              YCent;               // vertical from center posiiton
   double                              XRot;                // horizontal rotated posiiton
   double                              YRot;                // vertical rotated posiiton
   double                              Distance;            // two object distance
   double                              NearDistance;        // near distance
   double                              X;                   // coordinate for nearer object
   double                              Y;
   ImageAnalyse::OBJECT_CHAIN          *Objects;            // object chain pointer
   int                                 ObjectCount;         // objcet found

   // initialize
   Error = ERR_NONE;

   // Set parameter near object1 for test image analysing
   AnalyseParameters.XStart = ManualParam->X1Test - MANUAL_ALIGNMENT_BOX_SIZE; // box around object1
   AnalyseParameters.XStop = ManualParam->X1Test + MANUAL_ALIGNMENT_BOX_SIZE;
   AnalyseParameters.YStart = ManualParam->Y1Test - MANUAL_ALIGNMENT_BOX_SIZE;
   AnalyseParameters.YStop = ManualParam->Y1Test + MANUAL_ALIGNMENT_BOX_SIZE;
   AnalyseParameters.MaxObjectCount = 0;                    // all objects
   AnalyseParameters.MostBright = false;                    // not in magnitude order
   AnalyseParameters.MinDetectionPixel = Solving.MinDetectionPixel; // detection parameters from user
   AnalyseParameters.DetectionThreshold = Solving.DetectionThreshold;
   AnalyseParameters.ClassThreshold = Solving.ClassThreshold;
   AnalyseParameters.SaturationLevel = Solving.SaturationLevel;
   AnalyseParameters.SeeinfFWHM = 1;                        // it is only a scale factor
   // analyse test image
   TestAnalyse->SetAnalyseParameters(&AnalyseParameters);   // set parameters
   TestAnalyse->BuildObjectList();                          // build object list near object1

   // search the nearest object
   Objects = TestAnalyse->GetObjects( &ObjectCount );       // get the first object
   NearDistance = 10000;                                    // initial distance very far so any other are less
   while ( Objects != 0 )                                   // until object is present
   {
      // distance computing
      Distance = pow( ManualParam->X1Test - Objects->Actual.X, 2 ) + pow( ManualParam->Y1Test -Objects->Actual.Y, 2);     // compute distance between current pixel and star center
      Distance = pow( Distance , 0.5 );
      // check if is nearer than previous
      if ( Distance < NearDistance )                        // compare current distante to nearer
      {
         NearDistance = Distance;                           // current distante is the nearer
         X = Objects->Actual.X;                             // store current object coordinate
         Y = Objects->Actual.Y;
      }
      Objects = Objects->Next;                              // goto next object
   }
   if ( NearDistance == 10000 )                             // check if no object was found
      Error = ERR_NOALLIGN;

   if ( Error == ERR_NONE )
   {
      // update centroid position object 1 in test image
      ManualParam->X1Test = X;
      ManualParam->Y1Test = Y;

      // Set parameter near object2 for test image analysing
      AnalyseParameters.XStart = ManualParam->X2Test - MANUAL_ALIGNMENT_BOX_SIZE;  // box around object2
      AnalyseParameters.XStop = ManualParam->X2Test + MANUAL_ALIGNMENT_BOX_SIZE;
      AnalyseParameters.YStart = ManualParam->Y2Test - MANUAL_ALIGNMENT_BOX_SIZE;
      AnalyseParameters.YStop = ManualParam->Y2Test + MANUAL_ALIGNMENT_BOX_SIZE;
      // analyse test image near object2
      TestAnalyse->SetAnalyseParameters(&AnalyseParameters);// set parameters
      TestAnalyse->BuildObjectList();                       // build list object near object2

      // search the nearest object
      Objects = TestAnalyse->GetObjects( &ObjectCount );    // get the first object
      NearDistance = 10000;                                 // initial distance very far so any other are less
      while ( Objects != 0 )                                // until object is present
      {
         // distance computing
         Distance = pow( ManualParam->X2Test - Objects->Actual.X, 2 ) + pow( ManualParam->Y2Test -Objects->Actual.Y, 2);     // compute distance between current pixel and star center
         Distance = pow( Distance , 0.5 );
         // check if is nearer than previous
         if ( Distance < NearDistance )                     // compare current distante to nearer
         {
            NearDistance = Distance;                        // current distante is the nearer
            X = Objects->Actual.X;                          // store current object coordinate
            Y = Objects->Actual.Y;
         }
         Objects = Objects->Next;                           // goto next object
      }
      if ( NearDistance == 10000 )                          // check if no object was found
         Error = ERR_NOALLIGN;
   }

   if ( Error == ERR_NONE )
   {
      // update centroid position object2 in test image
      ManualParam->X2Test = X;
      ManualParam->Y2Test = Y;

      // Set parameter near object1 for reference image analysing
      AnalyseParameters.XStart = ManualParam->X1Ref - MANUAL_ALIGNMENT_BOX_SIZE;    // box around object1
      AnalyseParameters.XStop = ManualParam->X1Ref + MANUAL_ALIGNMENT_BOX_SIZE;
      AnalyseParameters.YStart = ManualParam->Y1Ref - MANUAL_ALIGNMENT_BOX_SIZE;
      AnalyseParameters.YStop = ManualParam->Y1Ref + MANUAL_ALIGNMENT_BOX_SIZE;
      AnalyseParameters.MaxObjectCount = 0;                 // all objects
      AnalyseParameters.MostBright = false;                 // not in magnitude order
      AnalyseParameters.MinDetectionPixel = Solving.MinDetectionPixel; // detection parameter from user
      AnalyseParameters.DetectionThreshold = Solving.DetectionThreshold;
      AnalyseParameters.ClassThreshold = Solving.ClassThreshold;
      AnalyseParameters.SaturationLevel = Solving.SaturationLevel;
      AnalyseParameters.SeeinfFWHM = 1;                     // it is only a scale factor
      // analyse reference image
      RefAnalyse->SetAnalyseParameters(&AnalyseParameters); // set parameters
      RefAnalyse->BuildObjectList();                        // build object list near object1

      // search the nearest object
      Objects = RefAnalyse->GetObjects( &ObjectCount );     // get the first object
      NearDistance = 10000;                                 // initial distance very far so any other are less
      while ( Objects != 0 )                                // until object is present
      {
         // distance computing
         Distance = pow( ManualParam->X1Ref - Objects->Actual.X, 2 ) + pow( ManualParam->Y1Ref -Objects->Actual.Y, 2);     // compute distance between current pixel and star center
         Distance = pow( Distance , 0.5 );
         // check if is nearer than previous
         if ( Distance < NearDistance )                     // compare current distante to nearer
         {
            NearDistance = Distance;                        // current distante is the nearer
            X = Objects->Actual.X;                          // store current object coordinate
            Y = Objects->Actual.Y;
         }
         Objects = Objects->Next;                           // goto next object
      }
      if ( NearDistance == 10000 )                          // check if no object was found
         Error =  ERR_NOALLIGN;
   }

   if ( Error == ERR_NONE )
   {
      // update centroid position object 1 in reference image
      ManualParam->X1Ref = X;
      ManualParam->Y1Ref = Y;

      // Set parameter near object2 for reference image analysing
      AnalyseParameters.XStart = ManualParam->X2Ref - MANUAL_ALIGNMENT_BOX_SIZE;    // box around object2
      AnalyseParameters.XStop = ManualParam->X2Ref + MANUAL_ALIGNMENT_BOX_SIZE;
      AnalyseParameters.YStart = ManualParam->Y2Ref - MANUAL_ALIGNMENT_BOX_SIZE;
      AnalyseParameters.YStop = ManualParam->Y2Ref + MANUAL_ALIGNMENT_BOX_SIZE;
      // analyse reference image
      RefAnalyse->SetAnalyseParameters(&AnalyseParameters); // set parameters
      RefAnalyse->BuildObjectList();                        // build object list near object2

      // search the nearest object
      Objects = RefAnalyse->GetObjects( &ObjectCount );     // get the first object
      NearDistance = 10000;                                 // initial distance very far so any other are less
      while ( Objects != 0 )                                // until object is present
      {
         // distance computing
         Distance = pow( ManualParam->X2Ref - Objects->Actual.X, 2 ) + pow( ManualParam->Y2Ref -Objects->Actual.Y, 2);     // compute distance between current pixel and star center
         Distance = pow( Distance , 0.5 );
         // check if is nearer than previous
         if ( Distance < NearDistance )                     // compare current distante to nearer
         {
            NearDistance = Distance;                        // current distante is the nearer
            X = Objects->Actual.X;                          // store current object coordinate
            Y = Objects->Actual.Y;
         }
         Objects = Objects->Next;                           // goto next object
      }
      if ( NearDistance == 10000 )                          // check if no object was found
         Error = ERR_NOALLIGN;
   }

   if ( Error == ERR_NONE )
   {
      // update centroid position object 2 in reference image
      ManualParam->X2Ref = X;
      ManualParam->Y2Ref = Y;

      // compute angle line between two star angle in test image
      TanNum = -(ManualParam->Y1Test-ManualParam->Y2Test);
      TanDen = ManualParam->X1Test-ManualParam->X2Test;
      if ( TanDen == 0 )
         TestAngle = 0;
      else
         TestAngle = atan( TanNum/TanDen ) * RAD;
      if ( TanDen < 0 )
         TestAngle += 180;
      if ( TestAngle < 0 )
         TestAngle += 360;

      // compute angle line between two star angle in reference image
      TanNum = -(ManualParam->Y1Ref-ManualParam->Y2Ref);
      TanDen = ManualParam->X1Ref-ManualParam->X2Ref;
      if ( TanDen == 0 )
         RefAngle = 0;
      else
         RefAngle = atan( TanNum/TanDen ) * RAD;
      if ( TanDen < 0 )
         RefAngle += 180;
      if ( RefAngle < 0 )
         RefAngle += 360;

      // compute allignment parameters
      Rotation = TestAngle - RefAngle;                      // reference rotation angle to get the test image

      XCent = ManualParam->X1Ref - RefImageInfo.SizeX/2;    // compute object 1 position center image related
      YCent = -(ManualParam->Y1Ref - RefImageInfo.SizeY/2);
      XRot = XCent * cos( Rotation/RAD ) - YCent * sin ( Rotation/RAD ); // rotate object 1 position
      YRot = XCent * sin( Rotation/RAD ) + YCent * cos ( Rotation/RAD );
      XRot += RefImageInfo.SizeX/2;                         // compute object 1 position with bmp coordinate system ...
      YRot = RefImageInfo.SizeY/2 - YRot;                   // this should be the object 1 position with only rotation
      XShift = ManualParam->X1Test - XRot;                  // compute reference image shift after rotation
      YShift = ManualParam->Y1Test - YRot;

      Allignement = ALIGNEMENT_DONE;                        // now images are aligned
   }

   Compared = false;                                        // image have to be compared

   return Error;
}



/*---------------------------------------------------------------------------

   get allignment status

---------------------------------------------------------------------------*/

ImageCompare::ALIGNEMENT_STATUS ImageCompare::GetAllignementStatus( void )
{
   return Allignement;
}



/*---------------------------------------------------------------------------

   get allignment parameters

---------------------------------------------------------------------------*/

void ImageCompare::GetAllignment( double *PXShift, double *PYShift, double *PRotation )
{
   *PXShift = XShift;
   *PYShift = YShift;
   *PRotation = Rotation;
}



/*---------------------------------------------------------------------------

   get Alignment object list

---------------------------------------------------------------------------*/

ImageCompare::OBJECT_CHAIN * ImageCompare::GetAlignmentObjects( void )
{
   return FirstAlignmentObject;
}



/*---------------------------------------------------------------------------

   get compare status

---------------------------------------------------------------------------*/

bool ImageCompare::GetCompareStatus( void )
{
   return Compared;
}



/*---------------------------------------------------------------------------

   Compare Image

---------------------------------------------------------------------------*/

ImageCompare::ERR ImageCompare::Compare( void )
{
   ERR                                 Error;               // internal error
   ImageAnalyse::OBJECT_CHAIN          *TestObjectsChain;   // test object chain pointer
   ImageAnalyse::OBJECT_CHAIN          *RefObjectsChain;    // reference object chain pointer
   int                                 TestObjectCount;     // object found on test image
   int                                 RefObjectCount;      // object found on reference image
   int                                 TestIndex;           // object index on test image
   int                                 RefIndex;            // object index on reference image
   int                                 Size;                // temporay per array size
   OBJECT                              ObjectParam;         // object parameters
   double                              XRefT;               // x theorical cooadinate in reference image
   double                              YRefT;               // Y theorical cooadinate in reference image
   double                              Distance;            // two object distance
   double                              LessDistance;        // less objects distance
   double                              DisTollerance;       // distance tollerance for objetc match
   bool                                Found;               // object found
   int                                 Border;              // boder arrond images without object test
   bool                                ToTest;              // object to be tested
   int                                 Count;               // matched object counter
   double                              MeanX;               // mean X value in test to reference magnitude relationship
   double                              MeanY;               // mean Y value in test to reference magnitude relationship
   double                              DeviationXY;         // deviation X vs Y in test to reference magnitude relationshi
   double                              DeviationXX;         // deviation X vs X in test to reference magnitude relationshi
   double                              MagAspected;         // aspected magnitude in reference image
   double                              MagIncrease;         // magnitute increse for current objects
   double                              Deviation;           // absolute deviation ( sigma * n.sigma) for magnitude increase detection
   char                                FileName[MAXPATH];   // match file name
   char                                Drive[MAXDRIVE];     // test image driver
   char                                Dir[MAXDIR];         // test image directory
   char                                Name[MAXFILE];       // test image file name
   char                                Ext[MAXEXT];         // test image file extension
   HANDLE                              hFile;               // file handle
   char                                Message[200];        // message on match file
   DWORD                               Written;             // written byte on match file
   int                                 BrighterCountr;      // counter for brighter objects
   int                                 NotFoundCounter;     // counter for not found objects
   int                                 HotIndex;            // hot pixel index

   // check if allignment was not possible
   if ( Allignement == ALIGNEMENT_NOTFOUND )
      return ERR_NOALLIGN;

   // check if it need allignment
   if ( Allignement == ALIGNEMENT_NOTDONE )
   {
      Error = ImageAllignment();                            // execute image allignment
      if ( Error != ERR_NONE )                              // check alignement error
         return Error;
   }

   // initialization
   FreeDifferenceList();                                    // free previous diffrence list
   Error = ERR_NONE;                                        // no error
   CompareInfo = 0;                                         // compare information array not allocated
   TestObjects = 0;                                         // test object array not allocated
   RefObjects = 0;                                          // reference object array not allocated

   // analyse test image
   AnalyseParameters.XStart = 0;                            // all objects in the image
   AnalyseParameters.XStop = TestImageInfo.SizeX;
   AnalyseParameters.YStart = 0;
   AnalyseParameters.YStop = TestImageInfo.SizeY;
   AnalyseParameters.MaxObjectCount = 0;                    // all objects
   AnalyseParameters.MostBright = true;                     // list in magnitude order
   AnalyseParameters.MinDetectionPixel = Solving.MinDetectionPixel; // detection parameter from user
   AnalyseParameters.DetectionThreshold = Solving.DetectionThreshold;
   AnalyseParameters.ClassThreshold = Solving.ClassThreshold;
   AnalyseParameters.SaturationLevel = Solving.SaturationLevel;
   AnalyseParameters.SeeinfFWHM = 1;                        // it is only a scale factor
   TestAnalyse->SetAnalyseParameters(&AnalyseParameters);   // set parameter
   TestAnalyse->BuildObjectList();                          // get object list

   // load all objects from test image in an array
   TestObjectsChain = TestAnalyse->GetObjects( &TestObjectCount );   // get first object on test image
   if ( TestObjectCount == 0 )
      Error = ERR_NOSTAR;

   if ( Error == ERR_NONE )
   {
      Size = TestObjectCount * sizeof(ImageAnalyse::OBJECT);// size for test object array
      TestObjects = (ImageAnalyse::OBJECT*) malloc( Size ); // allocate test object array
      TestIndex = 0;                                        // erase index
      while ( TestObjectsChain != 0 )                       // scan all objects in test image
      {
         TestObjects[TestIndex++] = TestObjectsChain->Actual;// store object into array
         TestObjectsChain = TestObjectsChain->Next;         // goto to next object
      }

      // create array for comparing information
      Size = TestObjectCount * sizeof(COMPARE_INFO);        // size for comparing array
      CompareInfo = (COMPARE_INFO*) malloc( Size );         // allocate comparing information array
      memset( CompareInfo, 0, Size );                       // array erase

      // analyse reference image
      RefAnalyse->SetAnalyseParameters(&AnalyseParameters); // set parameter
      RefAnalyse->BuildObjectList();                        // get object list

      // load all objects from reference image in an array
      RefObjectsChain = RefAnalyse->GetObjects( &RefObjectCount );   // get first object on test image
      if ( RefObjectCount == 0 )
         Error = ERR_NOSTAR;
   }

   if ( Error == ERR_NONE )
   {
      Size = RefObjectCount * sizeof(ImageAnalyse::OBJECT); // size for reference object array
      RefObjects = (ImageAnalyse::OBJECT*) malloc( Size );  // allocate reference object array
      RefIndex = 0;                                         // erase index
      while ( RefObjectsChain != 0 )                        // scan all objects in reference image
      {
         RefObjects[RefIndex++] = RefObjectsChain->Actual;  // store object into array
         RefObjectsChain = RefObjectsChain->Next;           // goto to next object
      }

      // compute mean FWHM on test image on brightest objects
      FWHM = GetStarFWHM();
      Border = FWHM * 5;                                    // border enought to be sure objects are completly in the image
      if ( Border < 10 )
         Border = 10;                                       // at least 10 pixel

      hFile = INVALID_HANDLE_VALUE;                         // invalidate file handle
      if ( Solving.CreateMatchFile == true )                // need file handle?
      {
         fnsplit( TestImageName, Drive, Dir, Name, Ext);    // fplit test image file
         strcpy( FileName, Drive);                          // create file name in the same folder ...
         strcat( FileName, Dir);
         strcat( FileName, Name );                          // with the same name ...
         strcat( FileName, ".txt" );                        // as text file
         hFile = CreateFile( FileName, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS,	FILE_ATTRIBUTE_ARCHIVE, NULL );
         if ( hFile != INVALID_HANDLE_VALUE )               // if open was succeful
         {
            strcpy( Message, "X Y MagTest MagRif Flag\r\n" );  // write header line ( for exel)
            WriteFile ( hFile, Message, strlen(Message), &Written, NULL);
         }
      }

      // scan all object in test image and find the corrispondent object in reference image
      for ( TestIndex=0; TestIndex<TestObjectCount; TestIndex++) // scann all object found on test image
      {
         CompareInfo[TestIndex].Status = OBJECT_NOTTESTED;  // set default status
         CompareInfo[TestIndex].RefIndex = -1;              // invalidate reference index
         ToTest = true;                                     // object to be tested
         // check if object is too border near
         if ( TestObjects[TestIndex].X < Border || TestObjects[TestIndex].X >= TestImageInfo.SizeX-Border ||
              TestObjects[TestIndex].Y < Border || TestObjects[TestIndex].Y >= TestImageInfo.SizeY-Border    )
            ToTest = false;                                 // object to near to border: no for test
         // check if object is too faint or can be an hot pixel
         if ( TestObjects[TestIndex].FWHM < FWHM/2.0 )
            ToTest = false;                                 // object to faint or hot pixel

         // compute object coordinate in the reference image
         if ( ToTest == true )
         {
            XRefT = TestObjects[TestIndex].X;               // get current object coordinate
            YRefT = TestObjects[TestIndex].Y;
            TestToRef( &XRefT, &YRefT );                    // convert in reference image coordinate
            // check if the coordinate in the reference image is inside border
            if ( XRefT < Border || XRefT >= RefImageInfo.SizeX-Border ||
                 YRefT < Border || YRefT >= RefImageInfo.SizeY-Border    )
               ToTest = false;                              // object to near to border: no for test
         }

         // check if the object is present in referce image
         if ( ToTest == true )
         {
            CompareInfo[TestIndex].AspectedX = XRefT;       // store aspected coordnate on reference image
            CompareInfo[TestIndex].AspectedY = YRefT;
            Found = false;                                  // not found by default
            LessDistance = 10000;                           // initial distance very far so any other are less
            for ( RefIndex=0; RefIndex<RefObjectCount; RefIndex++)   // scan all reference objects
            {
               // distance computing
               Distance = pow( XRefT - RefObjects[RefIndex].X, 2 ) + pow( YRefT -RefObjects[RefIndex].Y, 2);     // compute distance between current pixel and star center
               Distance = pow( Distance , 0.5 );
               DisTollerance = (FWHM + RefObjects[RefIndex].FWHM)/4;
               if ( DisTollerance < 1.5 )
                  DisTollerance = 1.5;
               if ( Distance <= DisTollerance )
               {
                  Found = true;
                  if ( Distance < LessDistance )
                  {
                     CompareInfo[TestIndex].Status = OBJECT_FOUND;// set found status
                     CompareInfo[TestIndex].RefIndex = RefIndex;  // set index in reference image
                     LessDistance = Distance;
                  }
               }
            }
            // if match file was open store found information
            if ( hFile != INVALID_HANDLE_VALUE && Found == true)
            {
               RefIndex = CompareInfo[TestIndex].RefIndex;  // get reference object index
               sprintf( Message, "%lf %lf %lf %lf %d\r\n", TestObjects[TestIndex].X, TestObjects[TestIndex].Y,
               TestObjects[TestIndex].Magnitude, RefObjects[RefIndex].Magnitude, TestObjects[TestIndex].Flags  );
               WriteFile ( hFile, Message, strlen(Message), &Written, NULL);     // write file
            }
         }
         if ( ToTest == true && Found == false )
            CompareInfo[TestIndex].Status = OBJECT_NOTFOUND;
      }
      if ( hFile != INVALID_HANDLE_VALUE )
         CloseHandle( hFile);                               // close file


      //
      // compute least squares regression line between test magnitude (x) and reference magnitude (y)
      //

      // firt step: compute x and y mean values
      Count = 0;                                            // erase matched object counter
      MeanX = 0;                                            // erase means
      MeanY = 0;
      for ( TestIndex=0; TestIndex<TestObjectCount; TestIndex++) // scann all object found on test image
      {
         if ( CompareInfo[TestIndex].Status != OBJECT_FOUND )  // not matched objects are skipped
            continue;
         if ( (TestObjects[TestIndex].Flags & 4) != 0 )     // sature objects are skipped
            continue;

         RefIndex = CompareInfo[TestIndex].RefIndex;        // get reference object index
         MeanX += TestObjects[TestIndex].Magnitude;         // sum test magnitude for mean computing
         MeanY += RefObjects[RefIndex].Magnitude;           // sum reference magnitude for mean computing
         Count++;                                           // count object used
      }
      MeanX /= Count;                                       // compute means
      MeanY /= Count;

      // second step: compute standard deviation
      DeviationXY = 0;                                      // erase deviations
      DeviationXX = 0;
      for ( TestIndex=0; TestIndex<TestObjectCount; TestIndex++) // scann all object found on test image
      {
         if ( CompareInfo[TestIndex].Status != OBJECT_FOUND )  // not matched objects are skipped
            continue;
         if ( (TestObjects[TestIndex].Flags & 4) != 0 )     // sature objects are skipped
            continue;

         RefIndex = CompareInfo[TestIndex].RefIndex;        // get reference object index
         DeviationXX += pow( (TestObjects[TestIndex].Magnitude - MeanX ), 2); // sum test vs reference deviation
         DeviationXY += (RefObjects[RefIndex].Magnitude - MeanY) * (TestObjects[TestIndex].Magnitude - MeanX );  // sum test vs test deviation
      }

      // third step: regression line M and Q computing
      MagRelationM = DeviationXY / DeviationXX;
      MagRelationQ = MeanY - MagRelationM * MeanX;


      // find the faintest real objects
      Count=0;                                              // erase object counter
      MaxTestMag = 0;                                       // erase maximum magnitude used for comparation
      FaintestMag = 0;                                      // so to undertand is the first
      for ( TestIndex=TestObjectCount-1; TestIndex> 0; TestIndex--) // scan object from less bright to the most
      {
         if ( CompareInfo[TestIndex].Status == OBJECT_FOUND  ) // only matched object
         {
            if ( FaintestMag == 0 )
               FaintestMag = TestObjects[TestIndex].Magnitude; // store the faintest magnitude matched
            Count++;                                        // counts objcts
            if ( Count >= 20 )                              // only 20 objects are needed
               break;
         }
      }
      MaxTestMag = FaintestMag;                             // starting point for maximum reference magnitude
      Deviation = GetDeviation( TestIndex, 0, 20 );         // compute deviation from test ve reference relation ship in this point
      MaxTestMag -= Deviation * Solving.SigmaFaintMag;      // increase de maximum magnitude by user sigmas

      // check objects for magnitude e minimum magnitude
      for ( TestIndex=0; TestIndex<TestObjectCount; TestIndex++) // scann all object found on test image
      {
         // verify is there is a good reason match wasn't found
         if ( CompareInfo[TestIndex].Status == OBJECT_NOTFOUND ) // only not matched object are wanted
         {
            if ( TestObjects[TestIndex].Magnitude > MaxTestMag )  // is magnitude greater the maximum ?
               CompareInfo[TestIndex].Status = OBJECT_TOOFAINT;   // object was not found because too faint
         }
         // verify if a found object increase magnitude
         if ( CompareInfo[TestIndex].Status == OBJECT_FOUND &&    // only matched objects ...
              (TestObjects[TestIndex].Flags & 4) == 0 &&    // not sature objects ..
               TestObjects[TestIndex].FWHM > FWHM * 0.8)    // not hot pixels
         {
            if ( TestObjects[TestIndex].Magnitude < MaxTestMag )  // is magnitude lower the the maximum ?
            {
               Deviation = GetDeviation( TestIndex+1, TestObjects[TestIndex].Magnitude+1.0, 20 );  // compute deviation from test ve reference relation ship in this point
               Deviation = 0;
               RefIndex = CompareInfo[TestIndex].RefIndex;     // get index of found object in reference image
               MagAspected = TestObjects[TestIndex].Magnitude * MagRelationM + MagRelationQ; // compute aspected magnitude on reference image
               MagIncrease = RefObjects[RefIndex].Magnitude - MagAspected;  // compute magnitude increase
               Deviation *= Solving.SigmaMagIncrease;       // compute sigmas deviation
               if ( Deviation < Solving.MinMagIncrease )    // deviation at leat the minimum
                  Deviation = Solving.MinMagIncrease;
               if ( MagIncrease > Deviation )               // check if magnitude increase is more than threshold deviation
                  CompareInfo[TestIndex].Status = OBJECT_BRIGHTER;   // this object increse the magnitude
            }
         }
      }

      // create difference list
      BrighterCountr = 0;                                   // counter for brighter objects
      NotFoundCounter = 0;                                  // counter for not found objects
      for ( TestIndex=0; TestIndex<TestObjectCount; TestIndex++) // scann all object found on test image
      {
         if ( HotPixelTab != 0 )                            // is table pixel assigned ?
         {
            Found = false;                                  // default condition. no bypass
            for ( HotIndex=0; HotIndex<HotPixelCount; HotIndex++ )   // scan all element in hot pixel table
            {
               if ( fabs( TestObjects[TestIndex].X - HotPixelTab[HotIndex*2] )  < 0.5 &&  // compare corrent posizion with hot pixel current element
                    fabs( TestObjects[TestIndex].Y - HotPixelTab[HotIndex*2+1]) < 0.5 )
               {
                  Found = true;                             // pixel to bypass
                  break;
               }
            }
            if ( Found == true )                           // bypass current pixel
               continue;
         }

         // file for all matched objects?
         if ( Solving.SkipReferencMatch == false )
         {
            // is an object brighter than reference image ?
            if ( CompareInfo[TestIndex].Status == OBJECT_BRIGHTER )
            {
               // put in the list at maximum 2 brighter objects
               //if ( BrighterCountr < 2 )                    // already two object detect?
               {
                  memset ( &ObjectParam, 0, sizeof(ObjectParam) );   // erase difference structure
                  RefIndex = CompareInfo[TestIndex].RefIndex;        // get index of found object in reference image
                  // put all information in the difference structure
                  ObjectParam.TestX = TestObjects[TestIndex].X;
                  ObjectParam.TestY = TestObjects[TestIndex].Y;
                  ObjectParam.TestMag = TestObjects[TestIndex].Magnitude;
                  ObjectParam.RefX  = CompareInfo[TestIndex].AspectedX;
                  ObjectParam.RefY  = CompareInfo[TestIndex].AspectedY;
                  ObjectParam.RefMag = RefObjects[RefIndex].Magnitude;
                  ObjectParam.AspectedMag = TestObjects[TestIndex].Magnitude * MagRelationM + MagRelationQ;
                  ObjectParam.Status = CompareInfo[TestIndex].Status;
                  // add difference structure to the list
                  AddDifferenceObject( &ObjectParam );
               }
               //else
               //   CompareInfo[TestIndex].Status = OBJECT_REMOVED; // object removed from the list
               BrighterCountr++;                            // count brighter object
            }
            // is an object not found ?
            if ( CompareInfo[TestIndex].Status == OBJECT_NOTFOUND )
            {
               // put in the list at maximum 2 brighter objects
               //if ( NotFoundCounter < 2 )                   // already two object detect?
               {
                  memset ( &ObjectParam, 0, sizeof(ObjectParam) ); // erase difference structure
                  // put all information in the difference structure
                  ObjectParam.TestX = TestObjects[TestIndex].X;
                  ObjectParam.TestY = TestObjects[TestIndex].Y;
                  ObjectParam.TestMag = TestObjects[TestIndex].Magnitude;
                  ObjectParam.RefX  = CompareInfo[TestIndex].AspectedX;
                  ObjectParam.RefY  = CompareInfo[TestIndex].AspectedY;
                  ObjectParam.Status = CompareInfo[TestIndex].Status;
                  // add difference structure to the list
                  AddDifferenceObject( &ObjectParam );
               }
               NotFoundCounter++;                           // count not found object
            }
         }
         else
         {
            // file for all matched objects !!!
            if ( CompareInfo[TestIndex].Status != OBJECT_NOTTESTED )
            {
               memset ( &ObjectParam, 0, sizeof(ObjectParam) );      // erase difference structure
               // put all information in the difference structure
               ObjectParam.TestX = TestObjects[TestIndex].X;
               ObjectParam.TestY = TestObjects[TestIndex].Y;
               ObjectParam.TestMag = TestObjects[TestIndex].Magnitude;
               ObjectParam.RefX  = CompareInfo[TestIndex].AspectedX;
               ObjectParam.RefY  = CompareInfo[TestIndex].AspectedY;
               ObjectParam.Status = OBJECT_NOTFOUND;
               // add difference structure to the list
               AddDifferenceObject( &ObjectParam );
            }
         }
      }
   }

   // free used arrays
   if ( RefObjects != 0 )
      free ( RefObjects );                                  // free reference object array
   if ( CompareInfo != 0 )
      free ( CompareInfo );                                 // free compare information array
   if ( TestObjects )
      free ( TestObjects );                                 // free test object array

   Compared = true;                                         // this image is compared
   return ERR_NONE;
}



/*---------------------------------------------------------------------------

   get new object list

---------------------------------------------------------------------------*/

ImageCompare::OBJECT_CHAIN * ImageCompare::GetDifferenceObjects( void )
{
   return FirstDifferenceObject;
}



/*---------------------------------------------------------------------------

   get limit magnitudes

---------------------------------------------------------------------------*/

void ImageCompare::GetMagnitude( double* Faintest, double*Maximum )
{
   *Faintest = FaintestMag;
   *Maximum = MaxTestMag;
}



/*---------------------------------------------------------------------------

   try an allignment with defined maximum shift

---------------------------------------------------------------------------*/

bool ImageCompare::TryAllignment( int MaxShift )
{
   ImageAnalyse::ERR                   Error;               // error code
   ImageAnalyse::OBJECT                BrightestStars[MOST_BRIGHT_OBEJCTS];  // brightest stars in the test image center
   ImageAnalyse::OBJECT                FirstStarAround[NEAR_BRIGHT_OBJECTS]; // stars around the first star in the reference image
   ImageAnalyse::OBJECT                SecondStarAround[NEAR_BRIGHT_OBJECTS];// stars around the second star in the reference image
   int                                 FirstTestStarIndex;  // first test star index in BrightestStars array
   int                                 SecondTestStarIndex; // second test star index in BrightestStars array
   int                                 FirstRefStarIndex;   // first reference star index in FirstStarAround array
   double                              SecondTestDistance;  // distance of second star in test image
   int                                 Index;               // scanning table index
   double                              X;                   // horizontal from center position
   double                              Y;                   // vertical from center posiiton
   int                                 FirstIndex;          // first star search index
   int                                 SecondIndex;         // second star search index
   int                                 BrightIndex;         // bright start scan index
   double                              Distance;            // two star distance
   TWO_STARS_ALLIGNMENT                AlignParam;          // aglignment parameter
   bool                                Found;               // star found
   int                                 MatchStarCount;      // matched star count
   ImageAnalyse::OBJECT_CHAIN          *Objects;            // object chain element
   int                                 ObjectCount;         // object in the chain
   double                              FWHM;                // mean FWHM
   double                              DisTollerance;       // distance tollerance for objetc match

   // initialization
   memset( BrightestStars, 0, sizeof(BrightestStars));      // erase all arrays
   memset( FirstStarAround, 0, sizeof(FirstStarAround));
   memset( SecondStarAround, 0, sizeof(SecondStarAround));

   // set analyse area as total less the shift
   AnalyseParameters.XStart = MaxShift;
   AnalyseParameters.XStop = TestImageInfo.SizeX - MaxShift;
   AnalyseParameters.YStart = MaxShift;
   AnalyseParameters.YStop = TestImageInfo.SizeY - MaxShift;
   AnalyseParameters.MaxObjectCount = MOST_BRIGHT_OBEJCTS;  // only most bright objects
   AnalyseParameters.MostBright = true;
   AnalyseParameters.NoGalaxies = false;                    // now I don't know galaxy FWHM
   TestAnalyse->SetAnalyseParameters(&AnalyseParameters);   // set parameter
   Error = TestAnalyse->BuildObjectList();                  // build object list with these parameters
   if ( Error != ImageAnalyse::ERR_NONE )
      return false;

   // compute mean FWHM
   FWHM = GetStarFWHM();
   DisTollerance = FWHM/3.0;                                // distance tollerance depending on FWHM ..
   if ( DisTollerance < 1.5 )                               // but at leat 1.5 pixel
      DisTollerance = 1.5;

   // set analyse area without galaxies
   AnalyseParameters.NoGalaxies = true;                     // set no galaxies
   AnalyseParameters.GalaxyFWHM = FWHM*2;                   // galaxy when FWHM is the double of star FWHM
   TestAnalyse->SetAnalyseParameters(&AnalyseParameters);   // set parameter
   Error = TestAnalyse->BuildObjectList();                  // build object list with these parameters
   if ( Error != ImageAnalyse::ERR_NONE )
      return false;

   // copy 10 most bright stars
   Objects = TestAnalyse->GetObjects( &ObjectCount );       // get first object
   Index = 0;                                               // erase counter
   while ( Objects != 0 && Index < MOST_BRIGHT_OBEJCTS )    // until 10 object found
   {
      BrightestStars[Index++] = Objects->Actual;            // store objects
      Objects = Objects->Next;                              // go to next object
   }
   if ( Index < MIN_ALIGNMENT_OBJECTS )                     // when objects are not enought
      return false;                                         // it is not possibile make allignment

   // find out the farther star from the first
   FirstTestStarIndex = 0;                                  // first star index (the most bright)
   SecondTestDistance = 0;                                  // second star index scanning start from here
   for ( Index=0; Index<MOST_BRIGHT_OBEJCTS; Index++ )      // scan all 10 most bright stars
   {
      if ( BrightestStars[Index].X == 0 )                   // skip empty entry
         break;

      // compute distance from the first star to the current star
      Distance = pow( BrightestStars[Index].X-BrightestStars[FirstTestStarIndex].X, 2 ) +
                 pow( BrightestStars[Index].Y-BrightestStars[FirstTestStarIndex].Y, 2);
      Distance = pow( Distance , 0.5 );
      // current distance is greater than the previous ?
      if ( Distance > SecondTestDistance )
      {
         SecondTestDistance = Distance;                     // store distance
         SecondTestStarIndex = Index;                       // store second star index
      }
   }

   // set paramater for all object near the first star
   AnalyseParameters.XStart = BrightestStars[FirstTestStarIndex].X - MaxShift;
   AnalyseParameters.XStop = BrightestStars[FirstTestStarIndex].X + MaxShift;
   AnalyseParameters.YStart = BrightestStars[FirstTestStarIndex].Y - MaxShift;
   AnalyseParameters.YStop = BrightestStars[FirstTestStarIndex].Y + MaxShift;
   AnalyseParameters.NoGalaxies = false;
   RefAnalyse->SetAnalyseParameters(&AnalyseParameters);    // set parameter
   RefAnalyse->BuildObjectList();                           // build object list with these parameters

   // copy 20 most bright stars near the first
   Objects = RefAnalyse->GetObjects( &ObjectCount );        // get first object
   Index = 0;                                               // erase counter
   while ( Objects != 0 && Index < NEAR_BRIGHT_OBJECTS )    // until 20 object found
   {
      FirstStarAround[Index++] = Objects->Actual;           // store object
      Objects = Objects->Next;                              // goto next object
   }
   if ( Index < 1 )                                         // when no star was found
      return false;                                         // it is not possibile do alignment

   // set paramater for all object near the second star
   AnalyseParameters.XStart = BrightestStars[SecondTestStarIndex].X - MaxShift;
   AnalyseParameters.XStop = BrightestStars[SecondTestStarIndex].X + MaxShift;
   AnalyseParameters.YStart = BrightestStars[SecondTestStarIndex].Y - MaxShift;
   AnalyseParameters.YStop = BrightestStars[SecondTestStarIndex].Y + MaxShift;
   RefAnalyse->SetAnalyseParameters(&AnalyseParameters);    // set parameter
   RefAnalyse->BuildObjectList();                           // build object list with these parameters

   // copy 20 most bright stars near the second
   Objects = RefAnalyse->GetObjects( &ObjectCount );        // get first object
   Index = 0;                                               // erase counter
   while ( Objects != 0 && Index < NEAR_BRIGHT_OBJECTS )    // until 20 object found
   {
      SecondStarAround[Index++] = Objects->Actual;          // store object
      Objects = Objects->Next;                              // goto next object
   }
   if ( Index < 1 )                                         // when no star was found
      return false;                                         // it is not possibile do alignment

   // set parameter for all object in reference image
   AnalyseParameters.XStart = 0;
   AnalyseParameters.XStop = TestImageInfo.SizeX;
   AnalyseParameters.YStart = 0;
   AnalyseParameters.YStop = TestImageInfo.SizeY;
   AnalyseParameters.MaxObjectCount = 0;
   RefAnalyse->SetAnalyseParameters(&AnalyseParameters);    // set parameter
   RefAnalyse->BuildObjectList();                           // build object list with these parameters

   // evalute the best match
   FirstRefStarIndex = -1;                                  // invalidate index of the two stars on reference image
   // scan all possibile first star in reference image
   for ( FirstIndex = 0; FirstIndex<NEAR_BRIGHT_OBJECTS ; FirstIndex++)
   {
      if ( FirstStarAround[FirstIndex].X == 0 )             // skip empty entries
         break;
      // scan all possibile second star in reference image
      for ( SecondIndex = 0; SecondIndex<NEAR_BRIGHT_OBJECTS ; SecondIndex++)
      {
         if ( SecondStarAround[SecondIndex].X == 0 )        // skip empty entries
            break;

         // compute distance between current stars couple
         Distance = pow( FirstStarAround[FirstIndex].X-SecondStarAround[SecondIndex].X, 2 ) +
                    pow( FirstStarAround[FirstIndex].Y-SecondStarAround[SecondIndex].Y, 2);     // compute distance between current pixel and star center
         Distance = pow( Distance , 0.5 );

         // analyse only when the distance is into FWHM
         if ( fabs( Distance-SecondTestDistance) < DisTollerance )
         {
            FreeAlignmentList();                            // free star found chain
            // add the two stars to allignment chain
            AddAlignmnentObject( BrightestStars[FirstTestStarIndex].X, BrightestStars[FirstTestStarIndex].Y,
                                 FirstStarAround[FirstIndex].X, FirstStarAround[FirstIndex].Y);
            AddAlignmnentObject( BrightestStars[SecondTestStarIndex].X, BrightestStars[SecondTestStarIndex].Y,
                                 SecondStarAround[SecondIndex].X, SecondStarAround[SecondIndex].Y);

            // compute rotation and shif parameter with the current couple
            AlignParam.X1Test = BrightestStars[FirstTestStarIndex].X;
            AlignParam.Y1Test = BrightestStars[FirstTestStarIndex].Y;
            AlignParam.X2Test = BrightestStars[SecondTestStarIndex].X;
            AlignParam.Y2Test = BrightestStars[SecondTestStarIndex].Y;
            AlignParam.X1Ref  = FirstStarAround[FirstIndex].X;
            AlignParam.Y1Ref  = FirstStarAround[FirstIndex].Y;
            AlignParam.X2Ref  = SecondStarAround[SecondIndex].X;
            AlignParam.Y2Ref  = SecondStarAround[SecondIndex].Y;
            Allignment( &AlignParam );

            MatchStarCount = 0;                             // erase star match counter
            // scan others stars in order to verify the match
            for ( BrightIndex=0; BrightIndex<MOST_BRIGHT_OBEJCTS; BrightIndex++)
            {
               if ( BrightIndex == FirstTestStarIndex )     // skip first star
                  continue;
               if ( BrightIndex == SecondTestStarIndex )    // skip second star
                  continue;

               // Get current star coordinate in test image
               X = BrightestStars[BrightIndex].X;
               Y= BrightestStars[BrightIndex].Y;

               TestToRef( &X, &Y);                          // compute current star coordinate in reference image
               Found = false;                               // erase found flag
               Objects = RefAnalyse->GetObjects( &ObjectCount );     // get first object in reference image
               while ( Objects != 0 )                       // until object are present
               {
                  // compute distance between current star and bright star
                  Distance = pow( X-Objects->Actual.X, 2 ) + pow( Y-Objects->Actual.Y, 2);
                  Distance = pow( Distance , 0.5 );
                  // it is the bright start only when the distance is into FWHM
                  if ( Distance < DisTollerance )
                  {
                     Found = true;                          // star found
                     MatchStarCount++;                      // increase mached star
                     // add this star to allignment chain
                     AddAlignmnentObject( BrightestStars[BrightIndex].X, BrightestStars[BrightIndex].Y,
                                          Objects->Actual.X, Objects->Actual.Y);
                     break;
                  }
                  Objects = Objects->Next;                  // goto next object
               }
               if ( Found == false )                        // when not found current couple start are not correct
                  break;
               if ( MatchStarCount >= ALIGNMENT_OBJECTS-2 ) // when found enough star not other action is needed
                  break;
            }
            if ( Found == true )                            // when match is found ..
            {
               FirstRefStarIndex = FirstIndex;              // store two stars index
               break;
            }
         }
      }
      if ( FirstRefStarIndex != -1 )                        // when found no other action
         break;
   }

   // exit action
   if ( FirstRefStarIndex != -1 )
   {
       return true;                                         // found !!!
   }
   else
   {
      XShift = 0;                                           // not found: erase parameters
      YShift = 0;
      Rotation = 0;
      return false;
   }
}



/*---------------------------------------------------------------------------

   execute alignment

---------------------------------------------------------------------------*/

ImageCompare::ERR ImageCompare::Allignment( TWO_STARS_ALLIGNMENT *ManualParam )
{
   double                              TanNum;              // tangent numerator
   double                              TanDen;              // tangent denumerator
   double                              TestAngle;           // star angle in test image
   double                              RefAngle;            // star angle in reference image
   double                              XCent;               // horizontal from center position
   double                              YCent;               // vertical from center posiiton
   int                                 XRot;                // horizontal rotated posiiton
   int                                 YRot;                // vertical rotated posiiton

   // compute line between two star angle in test image
   TanNum = -(ManualParam->Y1Test-ManualParam->Y2Test);
   TanDen = ManualParam->X1Test-ManualParam->X2Test;
   if ( TanDen == 0 )
      TestAngle = 0;
   else
      TestAngle = atan( TanNum/TanDen ) * RAD;
   if ( TanDen < 0 )
      TestAngle += 180;
   if ( TestAngle < 0 )
      TestAngle += 360;

   // compute line between two star angle in reference image
   TanNum = -(ManualParam->Y1Ref-ManualParam->Y2Ref);
   TanDen = ManualParam->X1Ref-ManualParam->X2Ref;
   if ( TanDen == 0 )
      RefAngle = 0;
   else
      RefAngle = atan( TanNum/TanDen ) * RAD;
   if ( TanDen < 0 )
      RefAngle += 180;
   if ( RefAngle < 0 )
      RefAngle += 360;

   // compute allignment parameters
   Rotation = TestAngle - RefAngle;                         // reference rotation angle to get the test image

   XCent = ManualParam->X1Ref - RefImageInfo.SizeX/2;       // compute object 1 position center image related
   YCent = -(ManualParam->Y1Ref - RefImageInfo.SizeY/2);
   XRot = XCent * cos( Rotation/RAD ) - YCent * sin ( Rotation/RAD ); // rotate object 1 position
   YRot = XCent * sin( Rotation/RAD ) + YCent * cos ( Rotation/RAD );
   XRot += RefImageInfo.SizeX/2;                            // compute object 1 position with bmp coordinate system ...
   YRot = RefImageInfo.SizeY/2 - YRot;                      // this should be the object 1 position with only rotation
   XShift = ManualParam->X1Test - XRot;                     // compute reference image shift after rotation
   YShift = ManualParam->Y1Test - YRot;

   return ERR_NONE;
}



/*---------------------------------------------------------------------------

   test to reference coordinate conversion

---------------------------------------------------------------------------*/

void ImageCompare::TestToRef( double *X, double *Y)
{
   double                              XCent;               // horizontal from center position
   double                              YCent;               // vertical from center posiiton
   double                              XRot;                // horizontal rotated posiiton
   double                              YRot;                // vertical rotated posiiton

   XCent = *X - XShift;                                     // apply reverse shift
   YCent = *Y - YShift;
   XCent = XCent - TestImageInfo.SizeX/2;                   // convert bitmap coordinates in center related coordinates
   YCent = -(YCent - TestImageInfo.SizeY/2);
   XRot = XCent * cos( -Rotation/RAD ) - YCent * sin ( -Rotation/RAD ); // apply reverse rotation
   YRot = XCent * sin( -Rotation/RAD ) + YCent * cos ( -Rotation/RAD );
   XRot += RefImageInfo.SizeX/2;                            // convert center related coordinates to bitmap coordinates
   YRot = RefImageInfo.SizeY/2 - YRot;

   *X = XRot;
   *Y = YRot;
}



/*---------------------------------------------------------------------------

   Add alignment star to list

---------------------------------------------------------------------------*/

void ImageCompare::AddAlignmnentObject( double TestX, double TestY, double RefX, double RefY )
{
   OBJECT_CHAIN*                         Last;              // last element i block chain
   OBJECT_CHAIN*                         New;               // new element in blocl chain

   // get last chain element
   Last = FirstAlignmentObject;
   if ( Last != 0 )
   {
      while ( Last->Next != 0 )
         Last = Last->Next;
   }

   // create new parameter
   New = (OBJECT_CHAIN*)malloc( sizeof(OBJECT_CHAIN) );     // new parameter allocation
   New->Next = 0;

   // fill element with all parameters
   New->Actual.TestX = TestX;
   New->Actual.TestY = TestY;
   New->Actual.RefX = RefX;
   New->Actual.RefY = RefY;

   // put at the end
   if ( Last == 0 )
      FirstAlignmentObject = New;                           // if last is not present this is the first element
   else
      Last->Next = New;
}



/*---------------------------------------------------------------------------

   free current alignment list

---------------------------------------------------------------------------*/

ImageCompare::ERR ImageCompare::FreeAlignmentList( void )
{
   OBJECT_CHAIN*                       Now;                 // actual block chain element
   OBJECT_CHAIN*                       Next;                // next block chain element

   Now = FirstAlignmentObject;                              // first element
   while ( Now != 0 )                                       // scan chain until an element is present
   {
      Next = Now->Next;                                     // get next element
      free( Now );                                          // free actual element
      Now = Next;                                           // goto to the next
   }

   FirstAlignmentObject = 0;                                // null first element
   return ERR_NONE;
}



/*---------------------------------------------------------------------------

   Add star to chain

---------------------------------------------------------------------------*/

void ImageCompare::AddDifferenceObject( OBJECT *ObjectParam )
{
   OBJECT_CHAIN*                       Last;                // last element i block chain
   OBJECT_CHAIN*                       New;                 // new element in blocl chain

   // get last chain element
   Last = FirstDifferenceObject;
   if ( Last != 0 )
   {
      while ( Last->Next != 0 )
         Last = Last->Next;
   }

   // create new parameter
   New = (OBJECT_CHAIN*)malloc( sizeof(OBJECT_CHAIN) );     // allocate new entry
   New->Next = 0;                                           // next is null
   New->Actual = *ObjectParam;                              // copy input paramaters

   // put at the end
   if ( Last == 0 )
      FirstDifferenceObject = New;                          // if last is not present this is the first element
   else
      Last->Next = New;
}



/*---------------------------------------------------------------------------

   free current block chain

---------------------------------------------------------------------------*/

ImageCompare::ERR ImageCompare::FreeDifferenceList( void )
{
   OBJECT_CHAIN*                       Now;                 // actual block chain element
   OBJECT_CHAIN*                       Next;                // next block chain element

   Now = FirstDifferenceObject;                             // first element
   while ( Now != 0 )                                       // scan chain until an element is present
   {
      Next = Now->Next;                                     // get next element
      free( Now );                                          // free actual element
      Now = Next;                                           // goto to the next
   }

   FirstDifferenceObject = 0;                               // null first element
   return ERR_NONE;
}



/*---------------------------------------------------------------------------

    get magnitude relation deviation

---------------------------------------------------------------------------*/

double ImageCompare::GetDeviation( int StartIndex, double MagnitudeLimit, int MaxObject )
{
   int                                 TestObjectCount;     // object found on test image
   double                              Deviation;           // computed deviation
   int                                 Count;               // object count
   int                                 ScanIndex;           // index for object scanning
   int                                 RefIndex;            // index in reference list
   double                              MagAspected;         // magnitude aspected

   // initialization
   Deviation = 0;
   Count = 0;
   TestAnalyse->GetObjects( &TestObjectCount );
   for ( ScanIndex=StartIndex; ScanIndex<TestObjectCount; ScanIndex++) // scan test obejcts
   {
      if ( CompareInfo[ScanIndex].Status != OBJECT_FOUND )  // only matched object are wanted
         continue;
      if ( TestObjects[ScanIndex].FWHM < FWHM/2  )          // skip possibile hot pixel
         continue;
      if ( (TestObjects[ScanIndex].Flags & 4) != 0 )        // skip sature object
          continue;
      if ( TestObjects[ScanIndex].Magnitude > MagnitudeLimit ) // when magniture is too faint no other objects
          break;

      RefIndex = CompareInfo[ScanIndex].RefIndex;           // get matched object index
      MagAspected = TestObjects[ScanIndex].Magnitude * MagRelationM + MagRelationQ; // compute aspetec magnitude
      Deviation += pow( (RefObjects[RefIndex].Magnitude - MagAspected ), 2);   // sum deviation
      Count++;                                              // count objects
      if ( Count >= MaxObject )
         break;
   }
   if ( Count > 0 )
   {
      Deviation /= Count;                                   // mean square deviation
      Deviation = pow( Deviation, 0.5 );                    // deviation
    }
   return Deviation;
}



/*---------------------------------------------------------------------------

    compute star FWHM

---------------------------------------------------------------------------*/

double ImageCompare::GetStarFWHM( void )
{
   ImageAnalyse::OBJECT_CHAIN          *Objects;            // object chain pointer
   int                                 ObjectCount;         // objcet found
   double                              FWHMDeviation;       // deviation of FWHM
   double                              FWHMPrevDeviation;   // deviation of FWHM
   double                              FWHMMax;
   int                                 FWHMObjects;

   // compute mean FWHM on test image on brightest objects
   FWHMMax = 1000;
   FWHMPrevDeviation = 1000;
   do
   {
      FWHM = 0;
      FWHMObjects = 0;
      Objects = TestAnalyse->GetObjects( &ObjectCount );    // get first object
      while ( Objects != 0 )                                // until object found
      {
         if ( Objects->Actual.FWHM <= FWHMMax )
         {
            FWHM += Objects->Actual.FWHM;                   // sum FWHM
            FWHMObjects++;
            if ( FWHMObjects >= 10 )
               break;
         }
         Objects = Objects->Next;                           // go to next object
      }
      FWHM /= FWHMObjects;                                  // compute FWHM mean value

      FWHMDeviation = 0;
      FWHMObjects = 0;
      Objects = TestAnalyse->GetObjects( &ObjectCount );    // get first object
      while ( Objects != 0 )                                // until object found
      {
         if ( Objects->Actual.FWHM <= FWHMMax )
         {
            FWHMDeviation += pow(FWHM - Objects->Actual.FWHM, 2); // sum deviation
            FWHMObjects++;
            if ( FWHMObjects >= 10 )
               break;
         }
         Objects = Objects->Next;                           // go to next object
      }
      FWHMDeviation /= FWHMObjects;
      FWHMDeviation = pow( FWHMDeviation, 0.5 );
      FWHMMax = FWHM + 1.5 * FWHMDeviation;
      if ( FWHMDeviation >= FWHMPrevDeviation )
         break;
      FWHMPrevDeviation = FWHMDeviation;
   } while ( 1 );
   return FWHM;
}

