/******************************************************************************

	ImageCompare.h | Header of image comparer

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : November 2011

   This C++ object is used in more than one astromical software developed at
   Bassano Bresciano Observatory
   Copyright (C) 2009,2010,2011,2012  Luca Pietro Strabla
   This file is part of Polipus and SNF
   Polypus ans SNF are free software: you can redistribute it and/or modify
   them under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus ans SNF are distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it

******************************************************************************/



// include antirepeat
#ifndef ImageCompareH
#define ImageCompareH



// system include
#include <vcl.h>
#include <dir.h>
#pragma hdrstop



// project include
#include "ImageAnalyse.h"
#include "Fits.h"

#define                                RAD            (180/M_PI)


/******************************************************************************

	Image compare class

******************************************************************************/

class ImageCompare
{
   public:

      // definitions
      typedef enum
      {
         MAX_PIXEL_VALUE=65535,                             // maximum pixel value
         MIN_ALIGNMENT_OBJECTS=3,                           // minimum objects count for alignmnent
         ALIGNMENT_OBJECTS=5,                               // standard obejcts count for alignment
         MOST_BRIGHT_OBEJCTS=10,                            // most bright objects count  for alignment
         NEAR_BRIGHT_OBJECTS=20,                            // near most bright star count for alignment
         SHIFT_STEP=10,                                     // shift step for alignment shift searching
         MANUAL_ALIGNMENT_BOX_SIZE=10,                      // box size for manual alignment
      } DEFINITIONS;

      // compare status
      typedef enum
      {
         OBJECT_NOTTESTED,                                  // object was not compared (e.g. too border near)
         OBJECT_FOUND,                                      // object found on reference image
         OBJECT_NOTFOUND,                                   // object not found on reference image
         OBJECT_TOOFAINT,                                   // object not found on feference image but is too faint
         OBJECT_BRIGHTER,                                   // object found on reference image, in test image is brighter
         OBJECT_REMOVED,                                    // objcet was removed because to many detection
      } COMPARE_STATUS;

      // image info
      typedef struct
      {
         int                           SizeX;               // horizontal image size
         int                           SizeY;               // vertical image size
      } IMAGE_INFO;

      // solving parameter
      typedef struct
      {
         bool                          SkipReferencMatch;   // skip reference match
         int                           MinDetectionPixel;   // pixel count for object detection
         double                        DetectionThreshold;  // detection threshold in beckground sigma
         double                        ClassThreshold;      // object class detection in beckground sigma
         int                           SaturationLevel;     // CCD saturation level
         double                        SigmaFaintMag;       // sigma for faint object magnitude
         double                        SigmaMagIncrease;    // sigma for magnitude increase
         double                        MinMagIncrease;      // minimum magnitude increase
         bool                          CreateMatchFile;     // create match file 
      }SOLVING;

      // two stars allignement parameters
      typedef struct
      {
         double                        X1Test;              // object 1 on test image X position
         double                        Y1Test;              // object 1 on test image Y position
         double                        X2Test;              // object 2 on test image X position
         double                        Y2Test;              // object 2 on test image Y position
         double                        X1Ref;               // object 1 on reference image X position
         double                        Y1Ref;               // object 1 on reference image Y position
         double                        X2Ref;               // object 2 on reference image X position
         double                        Y2Ref;               // object 2 on reference image Y position
      }TWO_STARS_ALLIGNMENT;

      // comparing information
      typedef struct
      {
         int                           RefIndex;            // index of this object in reference image ( -1 = not found )
         int                           AspectedX;           // aspected horizontal object position in reference image
         int                           AspectedY;           // aspected vertical object position in reference image
         COMPARE_STATUS                Status;              // compare status
      }COMPARE_INFO;

      // star found information
      typedef struct
      {
         int                           TestX;               // horizontal object position in test image
         int                           TestY;               // vertical object position in test image
         double                        TestMag;             // magnitude in test image
         int                           RefX;                // horizontal object position in reference image
         int                           RefY;                // vertical object position in reference image
         double                        RefMag;              // magnitude in reference image
         double                        AspectedMag;         // aspected magnitude in reference image 
         COMPARE_STATUS                Status;              // comparation status
      }OBJECT;

      // block chain element
      typedef struct OBJECT_CHAINTag
      {
         OBJECT                        Actual;              // actual object
         OBJECT_CHAINTag*              Next;                // pounter to next object chain element
      }OBJECT_CHAIN;

      // error definition
      typedef enum
      {
         ERR_NONE,                                          // no error
         ERR_NOTESTIMAGE,                                   // test image not found or not corret
         ERR_NOREFIMAGE,                                    // reference image not found or not corret
         ERR_NOALLIGN,                                      // images not alligned
         ERR_NOSTAR,                                        // no star on the image
      } ERR;

      // allignment status definition
      typedef enum
      {
         ALIGNEMENT_NOTDONE,                                // allignement not done
         ALIGNEMENT_DONE,                                   // allignement done
         ALIGNEMENT_NOTFOUND,                               // allignement not found
      } ALIGNEMENT_STATUS;

      // object constructor
                                       ImageCompare( char *PPath );
      // object destroyer
      virtual                          ~ImageCompare();
      // set hot pixel
      void                             SetHotPixel( int PHotPixelCount, int *PHotPixelTab );
      // set solving parameter
      void                             SetSolvingParam( SOLVING* ASolving );
      // set images
      ERR                              SetImages( char *PRefImageName, char *PTestImageName );
      // alligne image automatically
      ERR                              ImageAllignment( void );
      // execute manual alignment
      ERR                              ManualAllignment( TWO_STARS_ALLIGNMENT *ManualParam );
      // get allignement status
      ALIGNEMENT_STATUS                GetAllignementStatus( void );
      // get allignment
      void                             GetAllignment( double *XShift, double *YShift, double *Rotation );
      // get alignment list
      OBJECT_CHAIN *                   GetAlignmentObjects( void );
      // get compare status
      bool                             GetCompareStatus( void );
      // compare plates
      ERR                              Compare ( );
      // get new object list
      OBJECT_CHAIN *                   GetDifferenceObjects( void );
      // get limit magnitudes
      void                             GetMagnitude( double* Faintest, double*Maximum );

  protected:
      char                             Path[MAXPATH];       // working path
      SOLVING                          Solving;             // solving parameters
      int                              HotPixelCount;       // hot pixel count
      int                              *HotPixelTab;        // hot pixel table, every pixel use two element (x,y)

      char                             TestImageName[MAXPATH];  // test image name
      IMAGE_INFO                       TestImageInfo;       // test image info
      ImageAnalyse                     *TestAnalyse;

      char                             RefImageName[MAXPATH]; // reference image name
      IMAGE_INFO                       RefImageInfo;        // reference image info
      ImageAnalyse                     *RefAnalyse;

      OBJECT_CHAIN                     *FirstDifferenceObject;   // first found object
      OBJECT_CHAIN                     *FirstAlignmentObject;// first align object

      ImageAnalyse::OBJECT             *TestObjects;        // test object array
      ImageAnalyse::OBJECT             *RefObjects;         // reference object array
      COMPARE_INFO                     *CompareInfo;        // array with comparing information
      double                           FWHM;                // mean FWHM
      double                           MagRelationM;
      double                           MagRelationQ;

      double                           XShift;              // x shift allignment
      double                           YShift;              // y shift allignment
      double                           Rotation;            // rotation allignment
      ALIGNEMENT_STATUS                Allignement;         // allignement status
      bool                             Compared;            // image compared
      ImageAnalyse::ANALYSE            AnalyseParameters;   // parameters for image analysing

      double                           FaintestMag;         // faintest test object
      double                           MaxTestMag;          // maximum magnitude on test image

      // try an allignment with defined maximum shift
      bool                             TryAllignment( int MaxShift );
      // execute alignment
      ERR                              Allignment( TWO_STARS_ALLIGNMENT *ManualParam );
      // test to reference coordinate conversion
      void                             TestToRef( double *X, double *Y);
      // add alignment object to list
      void                             AddAlignmnentObject( double TestX, double TestY, double RefX, double RefY );
      // free current alignment list
      ERR                              FreeAlignmentList(void);
      // Add star to chain
      void                             AddDifferenceObject( OBJECT *ObjectParam );
      // free current star chain
      ERR                              FreeDifferenceList(void);
      // get magnitude relation deviation
      double                           GetDeviation( int StartIndex, double MagnitudeLimit, int MaxObject );
      // compute star FWHM
      double                           GetStarFWHM( void );
};

#endif
