/******************************************************************************

	LX200.cpp | LX200 protocol management

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : March 2009

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// project include
#include "LX200.h"
#pragma package(smart_init)



/****************************************************************************

   Object constructor

*****************************************************************************/

LX200::LX200()
{
   // initilization
   hCom = INVALID_HANDLE_VALUE;                       // invalid handle
   InitializeCriticalSection( &CriticalSection );
}



/****************************************************************************

   Distruttore

*****************************************************************************/

LX200::~LX200()
{
   // close communication if it isn't
   if ( hCom != INVALID_HANDLE_VALUE )
      Close();
   DeleteCriticalSection( &CriticalSection );
}



/****************************************************************************

   Serial line opening
   return code not 0 means error

*****************************************************************************/

LX200::Open(
   int                                 Line,
   int                                 BaudDate
)
{
	char			                        NomeFile[10];		   // file name
   char			                        c[10];					// temp
	DCB 			                        dcb;						// communication parameter
	BOOL 			                        Ok;				      // successful operation
   COMMTIMEOUTS                        CommTimeouts;			// time-outs structure

   // open communication line
   strcpy( NomeFile, "COM" );
   itoa ( Line, c, 10);
   strcat( NomeFile, c );
	hCom = CreateFile( NomeFile, GENERIC_READ | GENERIC_WRITE,
    	0,                                                    // comm devices must be opened w/exclusive-access
    	NULL,                                                 // no security attrs
    	OPEN_EXISTING,                                        // comm devices must use OPEN_EXISTING
    	0,                                                    // not overlapped I/O
    	NULL                                                  // hTemplate must be NULL for comm devices
    	);

   // if error exit
	if (hCom == INVALID_HANDLE_VALUE)
      return 1;

   // get current communication parameters
	Ok = GetCommState(hCom, &dcb);
	if ( Ok == false)
      return 2;

   // set my parameters
	dcb.BaudRate = BaudDate;
	dcb.ByteSize = 8;
	dcb.Parity = NOPARITY;
	dcb.StopBits = TWOSTOPBITS	;
   dcb.fDtrControl = DTR_CONTROL_DISABLE;
   dcb.fRtsControl = RTS_CONTROL_DISABLE;
	Ok = SetCommState(hCom, &dcb);
	if ( Ok == false)
      return 3;

   // get current timeout
	Ok = GetCommTimeouts( hCom, &CommTimeouts );
	if ( Ok == false)
      return 4;

   // set my timeout
   CommTimeouts.ReadIntervalTimeout         = 100;
   CommTimeouts.ReadTotalTimeoutMultiplier  = 10;
   CommTimeouts.ReadTotalTimeoutConstant    = 2000;
   CommTimeouts.WriteTotalTimeoutMultiplier = 100;
   CommTimeouts.WriteTotalTimeoutConstant   = 1000;
	Ok = SetCommTimeouts( hCom, &CommTimeouts );
	if ( Ok == false)
      return 5;

  	return 0;
}



/****************************************************************************

   Serial line close

*****************************************************************************/

int LX200::Close()
{
   // exit if already closed
   if ( hCom == INVALID_HANDLE_VALUE )
      return 0;

   // close
	CloseHandle( hCom );
   hCom = INVALID_HANDLE_VALUE;
   return 0;
}



/****************************************************************************

   Get current telescope position

*****************************************************************************/

int LX200::GetTelescopePosition(
   CMHours::HOURS                     *ARetta,
   CMDegree::DEGREE                    *Dec
)
{
   char                                BufferTx[20];        // transmission buffer
   char                                BufferRx[20];        // receive buffer
   char                                Str[10];             // temp
   int                                 Error;               // error code

   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   // get RA
   strcpy( BufferTx, ":GR#" );
   Error = ExecCom( BufferTx, strlen( BufferTx), BufferRx, sizeof( BufferRx), 0);

   if ( Error == 0 )
   {
      // conver received data
      strncpy( Str, BufferRx, 2 );
      Str[2] = 0;
      if ( strcmp(Str, "07") != 0 )
         ARetta->hh = 0;
      ARetta->hh = atoi( Str );
      strncpy( Str, BufferRx+3, 2 );
      Str[2] = 0;
      ARetta->mm = atoi( Str );
      strncpy( Str, BufferRx+6, 2 );
      Str[2] = 0;
      ARetta->ss = atoi( Str );

      // get declination
      strcpy( BufferTx, ":GD#" );
      Error = ExecCom( BufferTx, strlen( BufferTx), BufferRx, sizeof( BufferRx), 0);
   }

   if ( Error == 0 )
   {
      // conver received data
      Dec->sign[0] = BufferRx[0];
      strncpy( Str, BufferRx+1, 2 );
      Str[2] = 0;
      Dec->gg = atoi( Str );
      strncpy( Str, BufferRx+4, 2 );
      Str[2] = 0;
      Dec->mm = atoi( Str );
      strncpy( Str, BufferRx+7, 2 );
      Str[2] = 0;
      Dec->ss = atoi( Str );
   }

   return Error;
}



/****************************************************************************

   set telescope velocity

*****************************************************************************/

int LX200::SetTelescopeVelocity(
   int                                 Velocity
)
{
   char                                Comand[10];          // command
   int                                 Error;               // error code

   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   switch ( Velocity )
   {
      case 1:
         strcpy( Comand, ":RG#");          break;
      case 2:
         strcpy( Comand, ":RC#");          break;
      case 3:
         strcpy( Comand, ":RM#");          break;
      case 4:
         strcpy( Comand, ":RS#");          break;
      default:
         Error = 1;
   }
   Error = ExecCom( Comand, strlen( Comand), NULL, 0, -1);

   return Error;
}



/****************************************************************************

   Set telecope position

*****************************************************************************/

int LX200::SetTelescopePosition( CMHours::HOURS *ARetta, CMDegree::DEGREE*Dec )
{
   char                             Command[20];            // command
   char                             BufferRx[20];           // error code
   char                             Str[10];                // temp
   int                              Error;                  // error code

   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   // send AR
   strcpy( Command, ":Sr ");
   itoa( ARetta->hh+100, Str, 10 );
   strcat ( Command, Str+1);
   strcat ( Command, ":" );
   itoa( ARetta->mm+100, Str, 10 );
   strcat ( Command, Str+1);
   strcat ( Command, ":" );
   itoa( ARetta->ss+100, Str, 10 );
   strcat ( Command, Str+1);
   strcat ( Command, "#" );
   Error = ExecCom( Command, strlen( Command), BufferRx, sizeof( BufferRx), 1);
   if ( Error == 0 )
   {
      if ( BufferRx[0] != '1' )
         Error = 1;
   }

   // send Declination
   strcpy( Command, ":Sd ");
   strcat( Command, Dec->sign );
   itoa( Dec->gg+100, Str, 10 );
   strcat ( Command, Str+1);
   strcat ( Command, ":" );
   itoa( Dec->mm+100, Str, 10 );
   strcat ( Command, Str+1);
   strcat ( Command, ":" );
   itoa( Dec->ss+100, Str, 10 );
   strcat ( Command, Str+1);
   strcat ( Command, "#" );
   if ( Error == 0 )
      Error = ExecCom( Command, strlen( Command), BufferRx, sizeof( BufferRx), 1);

   if ( Error == 0 )
   {
      if ( BufferRx[0] != '1' )
         Error = 1;
   }

   // send set position command
   strcpy( Command, ":CM#");
   if ( Error == 0 )
      Error = ExecCom( Command, strlen( Command), BufferRx, sizeof( BufferRx), 1);

   if ( Error == 0 )
   {
      if ( BufferRx[0] != '#' )
         Error = 1;
   }

   return Error;
}



/****************************************************************************

   Reset telecope position

*****************************************************************************/

int LX200::ResetTelescopeAlign( void )
{
   char                                Command[20];         // command
   char                                BufferRx[20];        // receive buffer
   int                                 Error;               // error code

   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   // send allignment reset
   strcpy( Command, ":!CM#");
   Error = ExecCom( Command, strlen( Command), BufferRx, sizeof( BufferRx), 1);
   if ( Error == 0 )
   {
      if ( BufferRx[0] != '#' )
         Error = 1;
   }

   return Error;
}



/****************************************************************************

   telescope goto command

*****************************************************************************/

int LX200::TelescopeGoto( CMHours::HOURS *ARetta, CMDegree::DEGREE*Dec )
{
   char                                Command[20];         // command
   char                                BufferRx[20];        // receive buffer
   char                                Str[10];             // temp
   int                                 Error;               // error code

   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   // send AR
   strcpy( Command, ":Sr ");
   itoa( ARetta->hh+100, Str, 10 );
   strcat ( Command, Str+1);
   strcat ( Command, ":" );
   itoa( ARetta->mm+100, Str, 10 );
   strcat ( Command, Str+1);
   strcat ( Command, ":" );
   itoa( ARetta->ss+100, Str, 10 );
   strcat ( Command, Str+1);
   strcat ( Command, "#" );
   Error = ExecCom( Command, strlen( Command), BufferRx, sizeof( BufferRx), 1);

   if ( Error == 0 )
   {
      if ( BufferRx[0] != '1' )
         Error = 1;
   }

   // send Declintion
   strcpy( Command, ":Sd ");
   strcat( Command, Dec->sign );
   itoa( Dec->gg+100, Str, 10 );
   strcat ( Command, Str+1);
   strcat ( Command, ":" );
   itoa( Dec->mm+100, Str, 10 );
   strcat ( Command, Str+1);
   strcat ( Command, ":" );
   itoa( Dec->ss+100, Str, 10 );
   strcat ( Command, Str+1);
   strcat ( Command, "#" );
   if ( Error == 0 )
      Error = ExecCom( Command, strlen( Command), BufferRx, sizeof( BufferRx), 1);

   if ( Error == 0 )
   {
      if ( BufferRx[0] != '1' )
         Error = 1;
   }

   // sent positioning command
   strcpy( Command, ":MS#");
   if ( Error == 0 )
      Error = ExecCom( Command, strlen( Command), BufferRx, sizeof( BufferRx), 1);

   if ( Error == 0 )
   {
      if ( BufferRx[0] != '0' )
         Error = 1;
   }

   return Error;
}



/****************************************************************************

   telescope in home position

*****************************************************************************/

int LX200::TelescopeHome(void)
{
   char                                Command[10];         // command
   int                                 Error;               // error code

   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   strcpy( Command, ":hS#");
   Error = ExecCom( Command, strlen( Command), NULL, 0, -1);

   return Error;
}



/****************************************************************************

   manual telescope movement to north

*****************************************************************************/

int LX200::GoTelescopeNorth(void)
{
   char                                Command[10];         // command
   int                                 Error;               // error code

   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   strcpy( Command, ":Mn#");
   Error = ExecCom( Command, strlen( Command), NULL, 0, -1);

   return Error;
}



/****************************************************************************

   manual telescope movement to south

*****************************************************************************/

int LX200::GoTelescopeSouth(void)
{
   char                                Command[10];         // command
   int                                 Error;               // error code

   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   strcpy( Command, ":Ms#");
   Error = ExecCom( Command, strlen( Command), NULL, 0, -1);

   return Error;
}



/****************************************************************************

   manual telescope movement to east

*****************************************************************************/

int LX200::GoTelescopeEast(void)
{
   char                             Command[10];            // command
   int                              Error;                  // error code

   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   strcpy( Command, ":Me#");
   Error = ExecCom( Command, strlen( Command), NULL, 0, -1);

   return Error;
}



/****************************************************************************

   manual telescope movement to west

*****************************************************************************/

int LX200::GoTelescopeWest(void)
{
   char                                Command[10];         // command
   int                                 Error;               // error code

   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   strcpy( Command, ":Mw#");
   Error = ExecCom( Command, strlen( Command), NULL, 0, -1);

   return Error;
}



/****************************************************************************

   stop telescope movement

*****************************************************************************/

int LX200::StopTelescope(void)
{
   char                                Command[10];         // command
   int                                 Error;               // error code

   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   strcpy( Command, ":Q#");
   Error = ExecCom( Command, strlen( Command), NULL, 0, -1);

   return Error;
}



/****************************************************************************

   set controller clock

*****************************************************************************/

int LX200::SetClock( SYSTEMTIME *Time )
{
   char                                Command[30];         // command
   int                                 Error;               // error code
   char                                Str1[10];            // temp

   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   strcpy( Command, ":!SDT ");
   itoa( Time->wYear, Str1, 10 );
   strcat( Command, Str1 );
   strcat( Command, "/" );
   itoa( Time->wMonth+100, Str1, 10 );
   strcat( Command, Str1+1 );
   strcat( Command, "/" );
   itoa( Time->wDay+100, Str1, 10 );
   strcat( Command, Str1+1 );
   strcat( Command, "/" );
   itoa( Time->wHour+100, Str1, 10 );
   strcat( Command, Str1+1 );
   strcat( Command, "." );
   itoa( Time->wMinute+100, Str1, 10 );
   strcat( Command, Str1+1 );
   strcat( Command, "." );
   itoa( Time->wSecond+100, Str1, 10 );
   strcat( Command, Str1+1 );
   strcat( Command, "#" );

   Error = ExecCom( Command, strlen( Command), NULL, 0, -1);

   return Error;
}



/****************************************************************************

   get telescope status

*****************************************************************************/

int LX200::GetTelescopeStatus(
   bool                                *Retain,
   bool                                *Calib,
   bool                                *MotAR,
   bool                                *MotDec,
   bool                                *MotoO,
   bool                                *Movimento,
   int                                 *PECValue,
   int                                 *PECIndex
)
{
   char                                BufferTx[20];        // transmission buffer
   char                                BufferRx[20];        // receive  buffer
   int                                 Error;               // error code
   char                                Temp1[10];           // temporary string

   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   // get status
   strcpy( BufferTx, ":!GS#" );                                               // LUCA
   Error = ExecCom( BufferTx, strlen( BufferTx), BufferRx, sizeof( BufferRx), 14);

   if ( Error == 0 )
   {
      // convert data
      if ( BufferRx[0] == '1' )
         *Retain = true;
      else
         *Retain = false;

      if ( BufferRx[1] == '1' )
         *Calib = true;
      else
         *Calib = false;

      if ( BufferRx[2] == '1' )
         *MotAR = true;
      else
         *MotAR = false;

      if ( BufferRx[3] == '1' )
         *MotDec = true;
      else
         *MotDec = false;

      if ( BufferRx[4] == '1' )
         *MotoO = true;
      else
         *MotoO = false;

      if ( BufferRx[5] == '1' )
         *Movimento = true;
      else
         *Movimento = false;

      strncpy( Temp1, BufferRx+6, 4 );
      Temp1[4] = 0;
      *PECValue = atoi ( Temp1 ) - 5000;

      strncpy( Temp1, BufferRx+10, 4 );
      Temp1[4] = 0;
      *PECIndex = atoi ( Temp1 );
   }

   return 0;
}



/****************************************************************************

   reset motor fault
   0 = AR
   1 = DEC

*****************************************************************************/

int LX200::ResetMotorFault( int Mot )
{
   char                                Command[10];         // command
   int                                 Error;               // error code

   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   Error = 0;
   if ( Mot == 0 )
   {
      strcpy( Command, ":!SRR#");
      Error = ExecCom( Command, strlen( Command), NULL, 0, -1);
   }
   if ( Mot == 1 )
   {
      strcpy( Command, ":!SRD#");
      Error = ExecCom( Command, strlen( Command), NULL, 0, -1);
   }

   return Error;
}



/****************************************************************************

   tun on/off telescope tracking movement

*****************************************************************************/

int LX200::SetTelescopeTracking( bool Stato )
{
   char                                Command[10];         // command
   int                                 Error;               // error code

   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   if ( Stato == 1 )
      strcpy( Command, ":!SM1#");
   else
      strcpy( Command, ":!SM0#");

   Error = ExecCom( Command, strlen( Command), NULL, 0, -1);

   return Error;
}



/****************************************************************************

   tun on/off dome automatic movement

*****************************************************************************/

int LX200::SetAutomaticDome( bool Stato )
{
   char                             Command[10];            // command
   int                              Error;                  // error code

   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   if ( Stato == 1 )
      strcpy( Command, ":!DA1#");
   else
      strcpy( Command, ":!DA0#");

   Error = ExecCom( Command, strlen( Command), NULL, 0, -1);

   return Error;
}



/****************************************************************************

   turn on/off flat light

*****************************************************************************/

int LX200::SetFlatLight( bool Stato )
{
   char                                Command[10];         // command
   int                                 Error;               // error code

   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   if ( Stato == 1 )
      strcpy( Command, ":!DF1#");
   else
      strcpy( Command, ":!DF0#");

   Error = ExecCom( Command, strlen( Command), NULL, 0, -1);

   return Error;
}



/****************************************************************************

   get dome status

*****************************************************************************/

int LX200::GetDomeStatus(
   int                                 *Azimut,
   int                                 *Scor,
   int                                 *Riba,
   bool                                *Flat,
   bool                                *Auto,
   bool                                *Calib,
   int                                 *ComCupola,
   bool                                *Phone,
   bool                                *Alarm,
   bool                                *Home,
   bool                                *Zero
)
{
   char                                BufferTx[30];        // tranmission buffer
   char                                BufferRx[30];        // receive buffer
   char                                Str[10];             // temp
   int                                 Error;               // error code

   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   // get status
   strcpy( BufferTx, ":!DI#" );
   Error = ExecCom( BufferTx, strlen( BufferTx), BufferRx, sizeof( BufferRx), 14);

   if ( Error == 0 )
   {
      // convert data
      strncpy( Str, BufferRx, 3 );
      Str[3] = 0;
      *Azimut = atoi( Str);

      *Scor = BufferRx[3] - '0';

      *Riba = BufferRx[4] - '0';

      if ( BufferRx[5] == '1' )
         *Flat = true;
      else
         *Flat = false;

      if ( BufferRx[6] == '1' )
         *Auto = true;
      else
         *Auto = false;

      if ( BufferRx[7] == '1' )
         *Calib = true;
      else
         *Calib = false;

      *ComCupola = BufferRx[8] - '0';

      if ( BufferRx[10] == '1' )
         *Phone = true;
      else
         *Phone = false;

      if ( BufferRx[11] == '1' )
         *Alarm = true;
      else
         *Alarm = false;

      if ( BufferRx[12] == '1' )
         *Home = true;
      else
         *Home = false;

      if ( BufferRx[13] == '1' )
         *Zero = true;
      else
         *Zero = false;
   }

   return Error;
}



/****************************************************************************

   get dome temperature

*****************************************************************************/

int LX200::GetDomeTemperature
(
   int                                 *TempAmb
)
{
   char                                BufferTx[20];        // transmittion buffer
   char                                BufferRx[20];        // receive buffer
   char                                Str[10];             // temp
   int                                 Error;               // error code

   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   // get status
   strcpy( BufferTx, ":!DT#" );
   Error = ExecCom( BufferTx, strlen( BufferTx), BufferRx, sizeof( BufferRx), 2);

   if ( Error == 0 )
   {
      // convert data
      strncpy( Str, BufferRx, 2);
      Str[2] = 0;
      *TempAmb = atoi( Str) - 50;
   }

   return Error;
}



/****************************************************************************

   open/close sliding shutter

*****************************************************************************/

int LX200::SetSShutter( bool Open )
{
   char                                Command[10];         // command
   int                                 Error;               // error code

   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   if ( Open == 1 )
      strcpy( Command, ":!DS0#");
   else
      strcpy( Command, ":!DS1#");

   Error = ExecCom( Command, strlen( Command), NULL, 0, -1);

   return Error;
}




/****************************************************************************

   open/close balancing shutter

*****************************************************************************/

int LX200::SetBShutter( bool Open )
{
   char                                Command[10];         // command
   int                                 Error;               // error code

   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   if ( Open == 1 )
      strcpy( Command, ":!Ds0#");
   else
      strcpy( Command, ":!Ds1#");

   Error = ExecCom( Command, strlen( Command), NULL, 0, -1);

   return Error;
}



/****************************************************************************

   CW dome rotation

*****************************************************************************/

int LX200::SetDomeCW( void )
{
   char                                Command[10];         // command
   int                                 Error;               // error code

   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   strcpy( Command, ":!DM1#");
   Error = ExecCom( Command, strlen( Command), NULL, 0, -1);

   return Error;
}



/****************************************************************************

   CCW dome rotation

*****************************************************************************/

int LX200::SetDomeCCW( void )
{
   char                                Command[10];         // command
   int                                 Error;               // error code

   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   strcpy( Command, ":!DM2#");
   Error = ExecCom( Command, strlen( Command), NULL, 0, -1);

   return Error;
}



/****************************************************************************

   Stop dome movement

*****************************************************************************/

int LX200::SetDomeStop( void )
{
   char                                Command[10];         // command
   int                                 Error;               // error code

   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   strcpy( Command, ":!DM0#");
   Error = ExecCom( Command, strlen( Command), NULL, 0, -1);

   return Error;
}



/****************************************************************************

   dome in zero position

*****************************************************************************/

int LX200::SetDomeZero( void )
{
   char                                Command[10];         // command
   int                                 Error;               // error code

   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   strcpy( Command, ":!DM3#");
   Error = ExecCom( Command, strlen( Command), NULL, 0, -1);

   return Error;
}



/****************************************************************************

   dome in home position

*****************************************************************************/

int LX200::SetDomeHome( void )
{
   char                                Command[10];         // command
   int                                 Error;               // error code

   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   strcpy( Command, ":!DM4#");
   Error = ExecCom( Command, strlen( Command), NULL, 0, -1);

   return Error;
}



/****************************************************************************

   turn on/off decondence

*****************************************************************************/

int LX200::SetDecondence( bool Open )
{
   char                                Command[10];         // command
   int                                 Error;               // error code

   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   if ( Open == 1 )
      strcpy( Command, ":!DR1#");
   else
      strcpy( Command, ":!DR0#");

   Error = ExecCom( Command, strlen( Command), NULL, 0, -1);

   return Error;
}



/****************************************************************************

   turn on/off timeout

*****************************************************************************/

int LX200::SetTimeoutControl( bool Stato )
{
   char                                Command[10];         // command
   int                                 Error;               // error code

   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   if ( Stato == 1 )
      strcpy( Command, ":!TO1#");
   else
      strcpy( Command, ":!TO0#");

   Error = ExecCom( Command, strlen( Command), NULL, 0, -1);

   return Error;
}



/****************************************************************************

   set automatic velocity

*****************************************************************************/

int LX200::SetAutoVelocity( int Velocity )
{
   char                                Command[10];         // command
   int                                 Error;               // error code
   char                                c[10];               // temp


   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   strcpy( Command, ":!VA");
   itoa( Velocity+1000, c, 10 );
   strcat( Command, c+1 );
   strcat( Command, "#" );

   Error = ExecCom( Command, strlen( Command), NULL, 0, -1);

   return Error;
}



/****************************************************************************

   turn on/off periodic error correction

*****************************************************************************/

int LX200::SetPEC( bool Stato, int Amplitude, int Phase )
{
   char                                Command[20];         // command
   int                                 Error;               // error code
   char                                Str[100];            // temporary string

   // exit if communication not open
   if ( hCom == INVALID_HANDLE_VALUE )
      return 1;

   if ( Stato == 1 )
      strcpy( Command, ":!PC1");
   else
      strcpy( Command, ":!PC0");

   itoa( Amplitude+1000, Str, 10 );                         // add aplitude
   strcat( Command, Str+1 );

   itoa( Phase+1000, Str, 10 );                             // add phase
   strcat( Command, Str+1 );

   strcat ( Command , "#" );

   Error = ExecCom( Command, strlen( Command), NULL, 0, -1);
   return Error;
}



/****************************************************************************

   communication execution

*****************************************************************************/

int LX200::ExecCom( char *BufferTx, int LenBufTx, char *BufferRx, int LenBufRx, int LenExpected )
{
   int                                 Error;               // error code

   EnterCriticalSection( &CriticalSection );

   Error = TxBuffer( BufferTx, LenBufTx );
   if ( Error == 0 && LenExpected != -1 )
   {
      Error = RxBuffer( BufferRx, LenBufRx, LenExpected );
   }

   LeaveCriticalSection( &CriticalSection );
   return Error;
}



/****************************************************************************

   send buffer on serial line

*****************************************************************************/

int LX200::TxBuffer(
   char                                *Buffer,
   int                                 Len
)
{
   DWORD					                  LenW;						// written bytes
   bool					                  Ok;						// operation esit
   DWORD					                  Errore;					// communition error
   COMSTAT				                  Status;					// line status

   Ok = WriteFile ( hCom, Buffer, Len, &LenW, NULL );
   if ( Ok == false || (int) (LenW) != Len )
   {
   	ClearCommError( hCom, &Errore, &Status );             // reset error
      PurgeComm( hCom, PURGE_RXCLEAR | PURGE_RXABORT );     // empty internal buffer
   	return 1;
   }
   return 0;
}



/****************************************************************************

   receive buffer from serial line

*****************************************************************************/

int LX200::RxBuffer(
   char                                *Buffer,
   int                                 LenBuf,
   int                                 LenExpected
)
{
   int                                 IndexInput;          // receive index in buffer
   DWORD                               Start;               // timestamp start receive
   DWORD                               Stop;                // timestamp end receive
   DWORD					                  Errore;				   // communication error
   COMSTAT				                  Status;				   // status line
   int                                 LenRx;               // bytes to read
   DWORD					                  LenR;						// read byes
   bool					                  Ok;						// operation esit
   int                                 Ret;                 // error code

   // initialize
   IndexInput=0;
   Start = GetTickCount();                                  // get current time

   Ret = 1;
   do
   {
      // get queue status
  	   ClearCommError( hCom, &Errore, &Status );
      if (Status.cbInQue >= 1)
      {
         // compute byte to read
         LenRx = Status.cbInQue;
         if ( LenExpected != 0 && LenRx > (LenExpected - IndexInput) )
             LenRx = LenExpected - IndexInput;
         if ( LenRx > (LenBuf-IndexInput) )
             LenRx = LenBuf-IndexInput;
         // read data
         Ok = ReadFile( hCom, Buffer+IndexInput, LenRx, &LenR, NULL );
         // if error in reading exits
         if ( Ok == false || (int) (LenR) != LenRx )
         {
            Ret = 2;
            break;
         }
         // increment index depenfing on received bytes
         IndexInput += LenRx;
         // trasmission completed?
         if ( LenExpected == 0 )
         {
            if ( Buffer[IndexInput-1] == '#' )
            {
               Ret = 0;
               break;
            }
         }
         else
         {
            if ( LenExpected == IndexInput )
            {
               Ret = 0;
               break;
            }
         }
      }

      Stop = GetTickCount();
   }
   while ( (Stop-Start) <= LX200_RXTOUT );

   if ( Ret != 0 )
   {
   	ClearCommError( hCom, &Errore, &Status );             // reset error
      PurgeComm( hCom, PURGE_RXCLEAR | PURGE_RXABORT );     // empty internal buffer
      Ret = -2;
   }
   
   return Ret;
}

