/******************************************************************************

	LogObserver.cpp log observer. 

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : 		July 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

*****************************************************************************/



// project include
#include "LogObserver.h"



/******************************************************************************

   constructor

******************************************************************************/

LogObserver::LogObserver( )
{
   // inizialize internal data
   PuntSubject=NULL;
   PuntMemo=NULL;
   *FileName=0;
}



/******************************************************************************

   Destroyer

******************************************************************************/

LogObserver::~LogObserver( )
{
   // if obderver is attached to subject
   if ( PuntSubject != NULL )
      PuntSubject->Detach( this );                          // release itself from subject
}



/******************************************************************************

   Set subject

******************************************************************************/

int LogObserver::SetSubject( TentaclesMgr *APuntSubject )
{
   PuntSubject = APuntSubject;                              // store pointer
   return PuntSubject->Attach( this );                      // attach itself to subject
}



/******************************************************************************

   set file for log storing

******************************************************************************/

void LogObserver::SetFileName( char *AFileName )
{
   strcpy( FileName, AFileName );
}



/******************************************************************************

   set memo to show messages

******************************************************************************/

void LogObserver::SetMemo( TMemo *APuntMemo )
{
   PuntMemo = APuntMemo;
}



/******************************************************************************

   Detect all notification

******************************************************************************/

void LogObserver::Update( int ANotifyCode )
{
   char                                Message[200];        // message to create
   int                                 Lenght;              // header message len
   SYSTEMTIME                          CurrentGMT;          // current GMT time
   const char                          *ShotName;           // shot name pointer
   TentaclesMgr::STATUS                Status;              // subject status
   double                              Temp1;               // temporary values
   int                                 Temp2;
   int                                 Temp3;
   char                                Str[20];             // conversion string
   char                                Str2[20];            // conversion string
   HANDLE                              hFile;               // file handle
   DWORD                               ToWrite;             // bytes to wtite
   DWORD                               Written;             // writte bytes
   CCD::SHOT_INFO                      ShotInfo;            // info about shot

   // get event timestamp
   GetSystemTime( &CurrentGMT );                            // get current GMT time
   // create date time string
   wsprintf( Message, "%04d/%02d/%02d %02d:%02d:%02d   ",
      CurrentGMT.wYear, CurrentGMT.wMonth, CurrentGMT.wDay,
      CurrentGMT.wHour, CurrentGMT.wMinute, CurrentGMT.wSecond );
   Lenght = strlen( Message );

   // Add message text dependig on notify code
   switch ( ANotifyCode )
   {
      // Program ON
      case TentaclesMgr::NOTIFY_PROGON:
         strcat ( Message, "--- AVVIATO PROGRAMMA --- Ver." );
         strcat ( Message, VERSION );
         break;
      // Program OFF
      case TentaclesMgr::NOTIFY_PROGOFF:
         wsprintf( Message+Lenght, "--- ARRESTATO PROGRAMMA ---" );
         break;
      // an observer take up CCD
      case TentaclesMgr::NOTIFY_CCD_TAKEUP:
         NameOnCCD = PuntSubject->GetNameOnCCD();
         wsprintf( Message+Lenght, "Preso controllo del CCD da modulo '%s'", NameOnCCD );
         break;
      // an observer release CCD
      case TentaclesMgr::NOTIFY_CCD_RELEASE:
         NameOnCCD = PuntSubject->GetNameOnCCD();
         wsprintf( Message+Lenght, "Rilasciato controllo del CCD da modulo '%s'", NameOnCCD );
         break;
      // an observer take up observatory
      case TentaclesMgr::NOTIFY_OBSERVATORY_TAKEUP:
         NameOnObservatory = PuntSubject->GetNameOnObservatory();
         wsprintf( Message+Lenght, "Preso controllo dell'osservatorio da modulo '%s'", NameOnObservatory );
         break;
      // an observer release observatory
      case TentaclesMgr::NOTIFY_OBSERVATORY_RELEASE:
         NameOnObservatory = PuntSubject->GetNameOnObservatory();
         wsprintf( Message+Lenght, "Rilasciato controllo dell'osservatorio da modulo '%s'", NameOnObservatory );
         break;
      // a shot is started
      case TentaclesMgr::NOTIFY_SHOTSTART:
         PuntSubject->GetStatus( &Status );                 // get shot time
         Temp1 = Status.ShotExposure;                       // convert in second
         Temp1 /= 1000;
         gcvt( Temp1, 3, Str );                             // convert in string
         wsprintf( Message+Lenght, "Avviata fotografia con esposizione %s secondi, Azimut:%d, Elevazione %d", Str, Status.TeleAzimut.gg, Status.TeleElevation.gg );
         break;
      // shot is aborted
      case TentaclesMgr::NOTIFY_SHOTABORT:
         strcat( Message, "Abortita fotografia" );
         break;
      // shot image downloaded
      case TentaclesMgr::NOTIFY_SHOTDOWNLOADED:
         PuntSubject->GetShotInfo( &ShotInfo );
         gcvt( ShotInfo.VisiblityIndex, 3, Str );
         gcvt( ShotInfo.FWHM, 3, Str2 );
         wsprintf( Message+Lenght, "Scaricata fotografia da CCD indice visibilit %s FWHM %s", Str, Str2 );
         break;
      // shot file saved
      case TentaclesMgr::NOTIFY_SHOTSAVED:
         ShotName = PuntSubject->GetLastShotName();  // get file name
         strcat( Message, "Salvata fotografia su file " );
         strcat( Message, ShotName );
         break;
      case TentaclesMgr::NOTYFY_TELESCOPEPOSITIONED:
         PuntSubject->GetStatus( &Status );                 // target position
         Temp2 = Status.TargetRA.ss;                        // convert in integer
         Temp3 = Status.TargetDec.ss;                       // convert in integer
         wsprintf( Message+Lenght, "Posizionato telescopio su %s AR %02dh%02d'%02d\" DE %s%02d%02d'%02d\"",
            Status.TargetObject, Status.TargetRA.hh, Status.TargetRA.mm, Temp2,
            Status.TargetDec.sign, Status.TargetDec.gg, Status.TargetDec.mm, Temp3 );
         break;
      case TentaclesMgr::NOTIFY_TELESCOPEALLIGED:
         PuntSubject->GetStatus( &Status );                 // target position
         Temp2 = Status.TargetRA.ss;                        // convert in integer
         Temp3 = Status.TargetDec.ss;                       // convert in integer
         wsprintf( Message+Lenght, "Allineato telescopio su %s AR %02dh%02d'%02d\" DE %s%02d%02d'%02d\"",
            Status.TargetObject, Status.TargetRA.hh, Status.TargetRA.mm, Temp2,
            Status.TargetDec.sign, Status.TargetDec.gg, Status.TargetDec.mm, Temp3 );
         break;
      case TentaclesMgr::NOTIFY_TELESCOPESHOTALLIGED:
         PuntSubject->GetStatus( &Status );                 // target position
         Temp2 = Status.TargetRA.ss;                        // convert in integer
         Temp3 = Status.TargetDec.ss;                       // convert in integer
         wsprintf( Message+Lenght, "Allineato telescopio da foto su %s AR %02dh%02d'%02d\" DE %s%02d%02d'%02d\"",
            Status.TargetObject, Status.TargetRA.hh, Status.TargetRA.mm, Temp2,
            Status.TargetDec.sign, Status.TargetDec.gg, Status.TargetDec.mm, Temp3 );
         break;
      // enable/disabled safe algorithm
      case TentaclesMgr::NOTIFY_SAFEALGORITM:
         PuntSubject->GetStatus( &Status );                 // get algorithm status
         if ( Status.SafeEnable == false )
            strcat( Message, "Disattivato controllo cielo nuvoloso" );
         else
         {
            gcvt( Status.SafeThreshold, 3, Str );
            wsprintf( Message+Lenght, "Attivato controllo cielo nuvoloso Soglia %s Tempo %d min.", Str, Status.SafeTime/60 );
         }
         break;
      // sky clouded
      case TentaclesMgr::NOTIFY_SKYCLOUDY:
         strcat( Message, "Rilevato cielo nuvoloso " );
         break;
      case TentaclesMgr::NOTIFY_GENERIC_COMMAND:
         PuntSubject->GetStatus( &Status );                 // get last command
         switch ( Status.CommandSubject )
         {
            // power management
            case TentaclesMgr::COM_SUBJ_POWER:
               if ( Status.CommandState == TentaclesMgr::COM_STA_ON )
                  strcat( Message, "Acceso quadro" );
               if ( Status.CommandState == TentaclesMgr::COM_STA_OFF )
                  strcat( Message, "Spento quadro" );
               break;
            // LX200 line management
            case TentaclesMgr::COM_SUBJ_LX200:
               if ( Status.CommandState == TentaclesMgr::COM_STA_ON )
                  strcat( Message, "Aperta linea di comunicazione LX200" );
               if ( Status.CommandState == TentaclesMgr::COM_STA_OFF )
                  strcat( Message, "Chiusa linea di comunicazione LX200" );
               break;
            // CCD management
            case TentaclesMgr::COM_SUBJ_CCD:
               if ( Status.CommandState == TentaclesMgr::COM_STA_ON )
                  strcat( Message, "Attivato CCD" );
               if ( Status.CommandState == TentaclesMgr::COM_STA_OFF )
                  strcat( Message, "Rilasciato CCD" );
               break;
            // flat light management
            case TentaclesMgr::COM_SUBJ_LIGHT:
               if ( Status.CommandState == TentaclesMgr::COM_STA_ON )
                  strcat( Message, "Accese luci flat" );
               if ( Status.CommandState == TentaclesMgr::COM_STA_OFF )
                  strcat( Message, "Spente luci flat" );
               break;
            // decondence management
            case TentaclesMgr::COM_SUBJ_DECOND:
               if ( Status.CommandState == TentaclesMgr::COM_STA_ON )
                  strcat( Message, "Acceso anticondensa" );
               if ( Status.CommandState == TentaclesMgr::COM_STA_OFF )
                  strcat( Message, "Spento anticondensa" );
               break;
            // telescope tracking management
            case TentaclesMgr::COM_SUBJ_TRACK:
               if ( Status.CommandState == TentaclesMgr::COM_STA_ON )
                  strcat( Message, "Acceso moto orario" );
               if ( Status.CommandState == TentaclesMgr::COM_STA_OFF )
                  strcat( Message, "Spento moto orario" );
               break;
            // clock management
            case TentaclesMgr::COM_SUBJ_CLOCKSYNC:
               strcat( Message, "Sincronizzato orologio controllore" );
               break;
            // dome management
            case TentaclesMgr::COM_SUBJ_DOME:
               if ( Status.CommandState == TentaclesMgr::COM_STA_LOCK )
                  strcat( Message, "Cupola in automatico su azimut telescopio" );
               if ( Status.CommandState == TentaclesMgr::COM_STA_UNLOCK )
                  strcat( Message, "Cupola in manuale" );
               if ( Status.CommandState == TentaclesMgr::COM_STA_HOME )
                  strcat( Message, "Movimento automatico cupola su posizione di riposo" );
               if ( Status.CommandState == TentaclesMgr::COM_STA_ZERO )
                  strcat( Message, "Movimento automatico cupola su posizione di zero" );
               if ( Status.CommandState == TentaclesMgr::COM_STA_STOP )
                  strcat( Message, "Arresto movimento cupola" );
               if ( Status.CommandState == TentaclesMgr::COM_STA_LEFT )
                  strcat( Message, "Movimento manuale cupola a sinistra" );
               if ( Status.CommandState == TentaclesMgr::COM_STA_RIGHT )
                  strcat( Message, "Movimento manuale cupola a destra" );
               break;
            // RA fault reset management
            case TentaclesMgr::COM_SUBJ_RA:
               if ( Status.CommandState == TentaclesMgr::COM_STA_ZERO )
                  strcat( Message, "Reset fault motore AR" );
               // DEC fault reset management
               case TentaclesMgr::COM_SUBJ_DEC:
               if ( Status.CommandState == TentaclesMgr::COM_STA_ZERO )
                  strcat( Message, "Reset fault motore AR" );
               break;
            // sliding shutter dome management
            case TentaclesMgr::COM_SUBJ_SSHUTTER:
               if ( Status.CommandState == TentaclesMgr::COM_STA_OPEN )
                  strcat( Message, "Apertura sportello scorrevole" );
               if ( Status.CommandState == TentaclesMgr::COM_STA_CLOSE )
                  strcat( Message, "Chiusura sportello scorrevole" );
               if ( Status.CommandState == TentaclesMgr::COM_STA_STOP )
                  strcat( Message, "Arresto movimento sportello scorrevole" );
               break;
            // balancing shutter dome management
            case TentaclesMgr::COM_SUBJ_BSHUTTER:
               if ( Status.CommandState == TentaclesMgr::COM_STA_OPEN )
                  strcat( Message, "Apertura sportello ribaltabile" );
               if ( Status.CommandState == TentaclesMgr::COM_STA_CLOSE )
                  strcat( Message, "Chiusura sportello ribaltabile" );
               if ( Status.CommandState == TentaclesMgr::COM_STA_STOP )
                  strcat( Message, "Arresto movimento sportello ribaltabile" );
               break;
            // telescope management
            case TentaclesMgr::COM_SUBJ_TELE:
               if ( Status.CommandState == TentaclesMgr::COM_STA_LEFT )
                  strcat( Message, "Movimento manuale telescopio ad est" );
               if ( Status.CommandState == TentaclesMgr::COM_STA_RIGHT )
                  strcat( Message, "Movimento manuale telescopio ad ovest" );
               if ( Status.CommandState == TentaclesMgr::COM_STA_UP )
                  strcat( Message, "Movimento manuale telescopio a nord" );
               if ( Status.CommandState == TentaclesMgr::COM_STA_DOWN )
                  strcat( Message, "Movimento manuale telescopio a sud" );
               if ( Status.CommandState == TentaclesMgr::COM_STA_STOP )
                  strcat( Message, "Arrestato movimento telescopio" );
               if ( Status.CommandState == TentaclesMgr::COM_STA_HOME )
                  strcat( Message, "Telescopio in posizione di riposo" );
               break;
            // controller management
            case TentaclesMgr::COM_SUBJ_CONTROLLER:
               if ( Status.CommandState == TentaclesMgr::COM_STA_ON )
                  strcat( Message, "Attivato controllo timeout su controllore" );
               if ( Status.CommandState == TentaclesMgr::COM_STA_OFF )
                  strcat( Message, "Disattivato controllo timeout su controllore" );
               break;
            // not managed
            default:
               return;
         }
         break;
      case TentaclesMgr::NOTIFY_GRBACTIVE:
         strcat( Message, "Attivata rilevazione GRB " );
         break;
      case TentaclesMgr::NOTIFY_GRBDEACTIVE:
         strcat( Message, "Disattivata rilevazione GRB " );
         break;
      case TentaclesMgr::NOTIFY_GRBACTIVESWIFT:
         strcat( Message, "Attivata rilevazione GRB da satellite SWIFT " );
         break;
      case TentaclesMgr::NOTIFY_GRBDEACTIVESWIFT:
         strcat( Message, "Disattivata rilevazione GRB da satellite SWIFT  " );
         break;
      case TentaclesMgr::NOTIFY_GRBDETECTED:
         strcat( Message, "Rilevato GRB " );
         break;
      case TentaclesMgr::NOTIFY_GRBTELESCOPEPOSITIONED:
         strcat( Message, "Posizionato telescopio su GRB " );
         break;
   }

   // add new string on memo
   PuntMemo->Lines->Add( AnsiString(Message) );

   // add new string on file
   strcat( Message, "\r\n" );                               // append new file for file
   hFile = CreateFile( FileName, GENERIC_READ |GENERIC_WRITE, 0, NULL, OPEN_ALWAYS	,	FILE_ATTRIBUTE_ARCHIVE, NULL );  // open log file in append mode
   if ( hFile == INVALID_HANDLE_VALUE )                     // if not possibile exit
      return ;
   SetFilePointer(hFile, 0, 0, FILE_END) ;                  // pointer to actual file end
   if ( ANotifyCode == TentaclesMgr::NOTIFY_PROGON )
      WriteFile ( hFile, "\r\n", 2, &Written, NULL);        // write empty line
   ToWrite = strlen(Message);                               // write size
   WriteFile ( hFile, Message, ToWrite, &Written, NULL);    // write message
   CloseHandle( hFile);                                     // close file
}




