/******************************************************************************

	OpenSequence.cpp | Observatory opening sequence

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : June 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// System include
#pragma hdrstop



// project include
#include "OpenSequence.h"
#pragma package(smart_init)





/*---------------------------------------------------------------------------

   Object constructor

---------------------------------------------------------------------------*/

__fastcall OpenSequence::OpenSequence(void)
   : TThread( false )
{
   State = SEQ_STATE_IDLE;                                  // object starts in idle status
   PuntSubject = 0;                                         // no sublect
   Control = false;                                         // no contol taken
}



/****************************************************************************

   Object destroyer

*****************************************************************************/

_fastcall OpenSequence::~OpenSequence()
{
   if ( PuntSubject != 0 )                                  // if subject assigned
      PuntSubject->Detach(this);                            // detach this observer from it
}



/******************************************************************************

   set subject

******************************************************************************/

int OpenSequence::SetSubject(
   TentaclesMgr       *ParPuntSubject                       // pointer to subject
)
{
   PuntSubject = ParPuntSubject;                            // store pointer
   return PuntSubject->Attach( this );                      // attach this observer to subject
}



/*---------------------------------------------------------------------------

   Starts the sequence

---------------------------------------------------------------------------*/

void OpenSequence::StartSequence(HANDLE ParFormHandle, bool ParDomeZero, bool ParOpenSShutter, bool ParOpenBShutter, bool ParDomeLock, bool ParSyncClock, bool ParTrackingOn )
{
   // store input parameters
   FormHandle     = ParFormHandle;
   DomeZero       = ParDomeZero;
   OpenSShutter   = ParOpenSShutter;
   OpenBShutter   = ParOpenBShutter;
   DomeLock       = ParDomeLock;
   SyncClock      = ParSyncClock;
   TrackingOn     = ParTrackingOn;

   // activate the sequence
   State = SEQ_STATE_START;
}




/*---------------------------------------------------------------------------

   detect subject notify

---------------------------------------------------------------------------*/

void OpenSequence::Update( int ANotifyCode )
{
}



/*---------------------------------------------------------------------------

   sequence execution

---------------------------------------------------------------------------*/

void __fastcall OpenSequence::Execute(void)
{
   TentaclesMgr::STATUS                Status;              // subject status
   TentaclesMgr::ERR                   Error;

   // continue until thread is not external terminated
   while (  Terminated == false )
   {
      Sleep ( 500 );                                        // leave time to others activities

      // manage state machine
      switch ( State )
      {
         // sequence in idle (do nothing)
         case SEQ_STATE_IDLE:
            break;

         // sequence started
         case SEQ_STATE_START:
            Error = PuntSubject->GetStatus( &Status );      // get subject status
            if ( Error != TentaclesMgr::ERR_NONE )
            {
               MessageBox( FormHandle, "Impossibile leggere status", "Errore", MB_OK );
               State = SEQ_STATE_END;
               break;
            }
            // verify flat sequence starting conditions
            if ( Status.CommOpened == false || Status.CommOk == false )
            {
               MessageBox( FormHandle, "Non  possibile lanciare sequenza di apertura, una delle sequenti condizioni non  verificata:\n   Comunicazione attiva\n   Comunicazione ok", "Errore", MB_OK );
               State = SEQ_STATE_END;
               break;
            }

            // take control of observatory
            Error = PuntSubject->TakeUp( this, "Sequenza di apertura", true, false );
            if ( Error != TentaclesMgr::ERR_NONE )
            {
               MessageBox( FormHandle, "Non  possibile prendere controllo di osservatorio", "Errore", MB_OK );
               State = SEQ_STATE_END;
               break;
            }
            Control = true;                                 // control taken
            // if riquered dome in zero position
            if ( DomeZero == true )
            {
               Error = PuntSubject->RunCommand( this, TentaclesMgr::COM_SUBJ_DOME, TentaclesMgr::COM_STA_ZERO);
               if ( Error != TentaclesMgr::ERR_NONE )
               {
                  MessageBox( FormHandle, "Non accettato posizionamento cupola su zero", "Errore", MB_OK );
                  State = SEQ_STATE_END   ;                // goto end step
                  break;
               }
            }
            State = SEQ_STATE_DOMEZERO;                   // goto next step
            break;

         // wait dome in zero position
         case SEQ_STATE_DOMEZERO:
            if ( DomeZero == true )
            {
               Error = PuntSubject->GetStatus( &Status );   // get subject status
               if ( Error != TentaclesMgr::ERR_NONE )
               {
                  MessageBox( FormHandle, "Impossibile leggere status", "Errore", MB_OK );
                  State = SEQ_STATE_END;
                  break;
               }
               // is dome in zero position?
               if ( Status.DomeAlligned == false || Status.DomeAzimut != 270 )
                  break;
            }
            // if required sliding shutter opening
            if ( OpenSShutter == true )
            {
               // send sliding shutter opening
               Error = PuntSubject->RunCommand( this, TentaclesMgr::COM_SUBJ_SSHUTTER, TentaclesMgr::COM_STA_OPEN);
               if ( Error != TentaclesMgr::ERR_NONE )
               {
                  MessageBox( FormHandle, "Non riesco ad aprire scorrevole", "Errore", MB_OK );
                  State = SEQ_STATE_END   ;                // goto end step
                  break;
               }
            }
            State = SEQ_STATE_OPENSSHUTTER;                 // goto next step
            break;

         // wait sliding shutter opening
         case SEQ_STATE_OPENSSHUTTER:
            // if required sliding shutter opening
            if ( OpenSShutter == true )
            {
               Error = PuntSubject->GetStatus( &Status );      // get subject status
               if ( Error != TentaclesMgr::ERR_NONE )
               {
                  MessageBox( FormHandle, "Impossibile leggere status", "Errore", MB_OK );
                  State = SEQ_STATE_END;
                  break;
               }
               // is dome sliding shutter open?
               if ( Status.DomeSlidingShutter != 0 )
                  break;
            }
            // if required balancing shutter opening
            if ( OpenBShutter == true )
            {
               // send sliding shutter opening
               Error = PuntSubject->RunCommand( this, TentaclesMgr::COM_SUBJ_BSHUTTER, TentaclesMgr::COM_STA_OPEN);
               if ( Error != TentaclesMgr::ERR_NONE )
               {
                  MessageBox( FormHandle, "Non riesco ad aprire ribaltabile", "Errore", MB_OK );
                  State = SEQ_STATE_END;                    // goto end step
                  break;
               }
            }
            State = SEQ_STATE_OPENBSHUTTER;                 // goto next step
            break;

         // wait balancing shutter opening
         case SEQ_STATE_OPENBSHUTTER:
            // if required balancing shutter opening
            if ( OpenBShutter == true )
            {
               Error = PuntSubject->GetStatus( &Status );      // get subject status
               if ( Error != TentaclesMgr::ERR_NONE )
               {
                  MessageBox( FormHandle, "Impossibile leggere status", "Errore", MB_OK );
                  State = SEQ_STATE_END;
                  break;
               }
               // is dome balancing shutter open?
               if ( Status.DomeBalancingShutter != 0 )
                  break;
            }

            // if required dome lock
            if ( DomeLock == true )
            {
               Error = PuntSubject->RunCommand( this, TentaclesMgr::COM_SUBJ_DOME, TentaclesMgr::COM_STA_LOCK);
               if ( Error != TentaclesMgr::ERR_NONE )
               {
                  MessageBox( FormHandle, "Impossibile mette cupola in automatico", "Errore", MB_OK );
                  State = SEQ_STATE_END;
                  break;
               }
            }

            // if required clock syncronization
            if ( SyncClock == true )
            {
               Error = PuntSubject->RunCommand( this, TentaclesMgr::COM_SUBJ_CLOCKSYNC, TentaclesMgr::COM_STA_ON);
               if ( Error != TentaclesMgr::ERR_NONE )
               {
                  MessageBox( FormHandle, "Impossibile mette cupola in automatico", "Errore", MB_OK );
                  State = SEQ_STATE_END;
                  break;
               }
            }

            // if required TrackingOn
            if ( TrackingOn == true )
            {
               Error = PuntSubject->RunCommand( this, TentaclesMgr::COM_SUBJ_TRACK, TentaclesMgr::COM_STA_ON);
               if ( Error != TentaclesMgr::ERR_NONE )
               {
                  MessageBox( FormHandle, "Impossibile mette cupola in automatico", "Errore", MB_OK );
                  State = SEQ_STATE_END;
                  break;
               }
            }

            State = SEQ_STATE_END;                          // goto end step
            break;
      }

      // when sequence ended exit thread
      if ( State == SEQ_STATE_END )
         break;
   }

   // release all
   if ( Control == true )
   {
      Error = PuntSubject->Release( this, true, false );     // release observatory
      if ( Error != TentaclesMgr::ERR_NONE )
         MessageBox( FormHandle, "Impossibile rilasciare controllo", "Errore", MB_OK );
      else
         Control = false;
   }

   State = SEQ_STATE_END;
}



/*---------------------------------------------------------------------------

   get sequence status

---------------------------------------------------------------------------*/

OpenSequence::SEQ_STATE OpenSequence::GetState( void )
{
   return State;
}

