/*---------------------------------------------------------------------------

	ScriptCPU.cpp  script CPU for observatory management

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : October 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// system include
#include <string.h>
#include <stdlib.h>
#include <ctype.h>


// project include
#include "ScriptCPU.h"



/*---------------------------------------------------------------------------

	object constructor

---------------------------------------------------------------------------*/

_fastcall ScriptCPU::ScriptCPU()
   : TThread( false )
{
   PuntSubject = 0;                                         // no sublect
   MainTask = new ScriptTask();                             // create tasks
   TimeTask = new ScriptTask();
   CloseTask = new ScriptTask();
   ErrorList = new ScriptErrorList();                       // create error list
   ExecMode = EXEC_NONE;                                    // wihtout script it is not possible run
   Begin = true;                                            // run at the begin
   Control = false;                                         // control of observatory and CCD is not taken
}



/*---------------------------------------------------------------------------

	object destroyer

---------------------------------------------------------------------------*/

_fastcall ScriptCPU::~ScriptCPU(  )
{
   if ( PuntSubject != 0 )                                  // if subject assigned
      PuntSubject->Detach(this);                            // detach this observer from it

   delete MainTask;                                         // destroy tasks
   delete TimeTask;
   delete CloseTask;
   delete ErrorList;                                        // destry error list
}



/*---------------------------------------------------------------------------

   set subject

---------------------------------------------------------------------------*/

int ScriptCPU::SetSubject(
   TentaclesMgr       *ParPuntSubject                       // pointer to subject
)
{
   PuntSubject = ParPuntSubject;                            // store pointer
   MainTask->SetSubject(PuntSubject, this, SafeElevation);
   TimeTask->SetSubject(PuntSubject, this, SafeElevation);
   CloseTask->SetSubject(PuntSubject, this, SafeElevation);
   return PuntSubject->Attach( this );                      // attach this observer to subject
}



/*---------------------------------------------------------------------------

   set GRB trigger

---------------------------------------------------------------------------*/

void ScriptCPU::SetGRBTrigger( int PGRBTriggerIndex )
{
   MainTask->SetGRBTrigger( PGRBTriggerIndex );
   TimeTask->SetGRBTrigger( PGRBTriggerIndex );
   CloseTask->SetGRBTrigger( PGRBTriggerIndex );
}



/*---------------------------------------------------------------------------

	set script to execute

---------------------------------------------------------------------------*/

void ScriptCPU::SetScript( char *MainScript, char *TimeScript, char *CloseScript )
{
   int                                 ErrorLine;           // line where is the error
   int                                 ErrorColumn;         // column where is the error
   int                                 ErrorCode;           // error code

   delete ErrorList;
   ErrorList = new ScriptErrorList();                       // create error list

   MainTask->SetScript(MainScript);                         // set script on main task
   // exec simulation of all steps in order to verify it is error free
   do
   {
      MainTask->Step( ErrorLine, ErrorColumn, ErrorCode, true);   // simulate single step
      if ( ErrorCode != ScriptTask::ERR_ENDOFSCRIPT &&
           ErrorCode != ScriptTask::ERR_NONE            )
         ErrorList->Add( 1, ErrorCode, ErrorLine, ErrorColumn);   // add error to the list
   } while (ErrorCode != ScriptTask::ERR_ENDOFSCRIPT &&           // exit when script is finished
            ErrorCode != ScriptTask::ERR_PREMATUREEND);

   TimeTask->SetScript(TimeScript);                         // set script on time task
   // exec simulation of all steps in order to verify it is error free
   do
   {
      TimeTask->Step( ErrorLine, ErrorColumn, ErrorCode, true);   // simulate single step
      if ( ErrorCode != ScriptTask::ERR_ENDOFSCRIPT &&
           ErrorCode != ScriptTask::ERR_NONE            )
         ErrorList->Add( 2, ErrorCode, ErrorLine, ErrorColumn);   // add error to the list
   } while (ErrorCode != ScriptTask::ERR_ENDOFSCRIPT &&           // exit when script is finished
            ErrorCode != ScriptTask::ERR_PREMATUREEND);

   CloseTask->SetScript(CloseScript);                       // set script on close task
   // exec simulation of all steps in order to verify it is error free
   do
   {
      CloseTask->Step( ErrorLine, ErrorColumn, ErrorCode, true);  // simulate single step
      if ( ErrorCode != ScriptTask::ERR_ENDOFSCRIPT &&
           ErrorCode != ScriptTask::ERR_NONE            )
         ErrorList->Add( 2, ErrorCode, ErrorLine, ErrorColumn);   // add error to the list
   } while (ErrorCode != ScriptTask::ERR_ENDOFSCRIPT &&           // exit when script is finished
            ErrorCode != ScriptTask::ERR_PREMATUREEND);

   // chek error presence
   if ( ErrorList->GetCount() == 0 )
      ExecMode = EXEC_STOP;                                       // ativate stop condition
   else
      ExecMode = EXEC_NONE;                                       // activate null condition (step/run is not possibile)
   Rewind();                                                      // rewind execution
}



/*---------------------------------------------------------------------------

	set execution parameters

---------------------------------------------------------------------------*/

void ScriptCPU::SetExecParameters( HANDLE ParFormHandle, int ParTimeScriptInterval, bool ParTimeSciptEnable,
                                    int ParHourCloseScript, int ParMinuteCloseScript, int ParSafeElevation )
{
   // store execution parameter
   FormHandle           = ParFormHandle;
   TimeScriptInterval   = ParTimeScriptInterval;
   TimeScriptEnable     = ParTimeSciptEnable;
   HourCloseScript      = ParHourCloseScript;
   if ( HourCloseScript < 12 )                              // hours after midnight are considered of next day
      HourCloseScript += 24;
   MinuteCloseScript    = ParMinuteCloseScript;
   SafeElevation = ParSafeElevation;
}



/*---------------------------------------------------------------------------

	get error list

---------------------------------------------------------------------------*/

ScriptErrorList* ScriptCPU::GetErrorList(void)
{
   return ErrorList;
}



/*---------------------------------------------------------------------------

	start execution until finish

---------------------------------------------------------------------------*/

void ScriptCPU::Start( void )
{
   TentaclesMgr::ERR                   Error;

   if ( ExecMode == EXEC_STOP )                             // run is accepted only in stop condition
   {
      // take control of observatory and ccd
      Error = PuntSubject->TakeUp( this, "Programma", true, true );
      if ( Error != TentaclesMgr::ERR_NONE )
         MessageBox( FormHandle, "Non  possibile prendere controllo di osservatorio e CCD", "Errore", MB_OK );
      else
      {
         ExecMode = EXEC_RUN;
         Control = true;
      }
   }
}



/*---------------------------------------------------------------------------

	Stop execution on the next istruction

---------------------------------------------------------------------------*/

void ScriptCPU::Stop( void )
{
   if ( ExecMode == EXEC_RUN )                              // stop is accepted only un run condition
   {
      ExecMode = EXEC_STOPREQ;
      MainTask->EndStep();                                  // request of end of step
      TimeTask->EndStep();
      CloseTask->EndStep();
   }
}



/*---------------------------------------------------------------------------

	exec only one step

---------------------------------------------------------------------------*/

void ScriptCPU::Step( void )
{
   TentaclesMgr::ERR                   Error;

   if ( ExecMode == EXEC_STOP )                             // step is accepted only in stop condition
   {
      // take control of observatory and ccd
      Error = PuntSubject->TakeUp( this, "Programma", true, true );
      if ( Error != TentaclesMgr::ERR_NONE )
         MessageBox( FormHandle, "Non  possibile prendere controllo di osservatorio e CCD", "Errore", MB_OK );
      else
      {
         ExecMode = EXEC_STEP;
         Control = true;
      }
   }
}



/*---------------------------------------------------------------------------

	execute rewind

---------------------------------------------------------------------------*/

void ScriptCPU::Rewind( void )
{
   if ( ExecMode == EXEC_RUN )                              // stop request is exectuted
      ExecMode = EXEC_STOPREQ;

   // reset task execution
   MainTask->Reset();                                       
   TimeTask->Reset();
   CloseTask->Reset();

   // reset variables
   Begin = true;                                            // program at begin
   TimerScript = 0;                                         // reset timer script
   MainScriptRun = false;                                   // main script is not running
   TimeScriptRun = false;                                   // time script is not running
   CloseScriptRun = false;                                  // Close script is not running
   CloseScriptEnd = false;                                  // Close script is not finished
   PrevTick = GetTickCount();                               // take tick count as start time
   PuntSubject->SetSafeAlgorithm( false, 0, 0 );            // disable safe algorithm
}



/*---------------------------------------------------------------------------

	get current status

---------------------------------------------------------------------------*/

void ScriptCPU::GetStatus( bool *ParRun, int *Script, int *Line, bool *ParBegin )
{
   *ParRun = ExecMode == EXEC_STEP || ExecMode == EXEC_RUN || ExecMode == EXEC_STOPREQ;
   *Script = CurrentScript;
   *Line = CurrentLine;
   *ParBegin = Begin;
}



/*---------------------------------------------------------------------------

   detect subject notify

---------------------------------------------------------------------------*/

void ScriptCPU::Update( int ANotifyCode )
{
}



/*---------------------------------------------------------------------------

   execute program scheduling

---------------------------------------------------------------------------*/

void __fastcall ScriptCPU::Execute(void)
{
   int                                 Column;              // column where is the error
   int                                 ErrorCode;           // error code
   int                                 CurrentTick;         // current tick count
   SYSTEMTIME                          CurrentTime;         // current data for Nightly folder naming
   TentaclesMgr::ERR                   Error;
   TentaclesMgr::STATUS                Status;              // observatory status
   int                                 Elevation;           // signed current telecope elevation

   while ( Terminated == false )                            // this thread continue until it is terminated
   {
      Sleep( 200 );                                         // leaves time to other thread
      if ( ExecMode == EXEC_STEP || ExecMode == EXEC_RUN )  // scheduning only in step or run mode
      {
         //
         // execute a step
         //

         // at beging active main program
         if ( Begin == true )
         {
            MainScriptRun = true;
            Begin = false;
         }

         // check for close script activation for elevation under safe limit
         PuntSubject->GetStatus( &Status);
         Elevation = Status.TeleElevation.gg;
         if ( *Status.TeleElevation.sign == '-' )
            Elevation = - Elevation;
         if ( Elevation < SafeElevation )
            CloseScriptRun = true;                          // close script have to run

         // chek time for close script activation

         GetSystemTime( &CurrentTime );                     // get current UT time
         if ( CurrentTime.wHour < 12 )                      // hours after midnight are considered of next day
             CurrentTime.wHour += (unsigned short)24;
         if ( CurrentTime.wHour > HourCloseScript )         // when current hour is greater the programmed close hour
            CloseScriptRun = true;                          // close script have to run
         if ( CurrentTime.wHour == HourCloseScript && CurrentTime.wMinute >= MinuteCloseScript )   // when current hour is equal programmed hors and minutes is equal o gratater
            CloseScriptRun = true;                          // close script have to run

         // check for time script activation
         if ( TimeScriptEnable == true && TimeScriptRun == false )   // if time script is enabled and not jet running
         {
            CurrentTick = GetTickCount();                   // get current tick
            TimerScript += ( CurrentTick - PrevTick );      // add tick from last sample
            if ( TimerScript / 60000 >= TimeScriptInterval )   // check tick versus time interval
            {
               TimeScriptRun = true;                        // it is time to run time script
               TimerScript -= 60000 * TimeScriptInterval;   // retrigger timer for next scheluning
            }
            PrevTick = CurrentTick;                         // save tick for next time
         }

         // check program activation by priority
         if ( CloseScriptRun == true )                      // check close script activation
         {
            CurrentScript = 3;                              // code of close script
            CloseTask->Step( CurrentLine, Column, ErrorCode, false); // exec a step in close script
            if ( ErrorCode == ScriptTask::ERR_ENDOFSCRIPT ) // check if script is finished
            {
               ExecMode == EXEC_STOPREQ;                    // close script is finihed, put CPU in stop
               Rewind();                                    // rewind execution
            }
         }
         else
         {
            if ( TimeScriptRun == true )                    // check time script activation
            {
               CurrentScript = 2;                           // code of time script
               TimeTask->Step( CurrentLine, Column, ErrorCode, false); // exec a step in time script
               if ( ErrorCode == ScriptTask::ERR_ENDOFSCRIPT )    // check if script is finished
               {
                  TimeScriptRun = false;                    // time script is finished
                  TimeTask->Reset();                        // rewind time script for a new execution
               }
            }
            else
            {
               if ( MainScriptRun == true )                 // check main script activation
               {
                  CurrentScript = 1;                        // code of main script
                  MainTask->Step( CurrentLine, Column, ErrorCode, false);  // exec a step in main script
                  if ( ErrorCode == ScriptTask::ERR_ENDOFSCRIPT )    // check if script is finished
                  {
                     MainScriptRun = false;                 // main script is finished
                     CloseScriptRun = true;                 // close script is activated before activation time
                  }
               }
            }
         }

         // at the end of the step CPU is put in stop is execution mode is step
         if ( ExecMode == EXEC_STEP )
            ExecMode = EXEC_STOPREQ;

      }

      // when execution goes in stop observatory and ccd are released
      if ( ExecMode == EXEC_STOPREQ && Control == true )
      {
         Error = PuntSubject->Release( this, true, true );     // release observatory and CCD
         if ( Error != TentaclesMgr::ERR_NONE )
            MessageBox( FormHandle, "Impossibile rilasciare controllo", "Errore", MB_OK );
         ExecMode = EXEC_STOP;
         Control = false;
      }

   }
}

