/******************************************************************************

	ScriptCPU.h | Header of script CPU for observatory management

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : October 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// antirepeat
#ifndef ScriptCPUH
#define ScriptCPUH



// project include
#include "Observer.h"
#include "TentaclesMgr.h"
#include "ScriptTask.h"
#include "ScriptErrorList.h"



/******************************************************************************

	Script CPU class

******************************************************************************/

class ScriptCPU : public TThread, public Observer
{
   public:

      // execution mode
      typedef enum
      {
         EXEC_NONE,                                         // execution is not possibile
         EXEC_STOP,                                         // execution in stop
         EXEC_STOPREQ,                                      // stop request
         EXEC_STEP,                                         // exec only one step and after that go in stop
         EXEC_RUN,                                          // exec all the instructions until end
         EXEC_END,                                          // execution reach end of all instructions
      } EXEC_MODE;

      // object constructor
      _fastcall                        ScriptCPU( );
      // object destroyer
      virtual _fastcall                ~ScriptCPU();
      // set subject
      int                              SetSubject( TentaclesMgr *ParPuntSubject );
      // set GRB trigger
      void                             SetGRBTrigger( int PGRBTriggerIndex );
      // set script to execute
      void                             SetScript( char *MainScript, char *TimeScript, char *CloseScript );
      // set execution parameters
      void                             SetExecParameters( HANDLE ParFormHandle, int ParTimeScriptInterval, bool ParTimeSciptEnable,
                                          int ParHourCloseScript, int ParMinuteCloseScript, int ParSafeElevation );
      // get error list
      ScriptErrorList*                 GetErrorList(void);
      // execute start
      void                             Start(void );
      // execute stop
      void                             Stop(void );
      // execute step
      void                             Step(void );
      // execute rewind
      void                             Rewind(void );
      // get current status
      void                             GetStatus( bool *Run, int *Script, int *Line, bool *ParBegin );

   private:
      // notify change in subject
      virtual void                     Update( int ANotifyCode );

   protected:
      HANDLE                           FormHandle;          // handle of the parent form
      TentaclesMgr                     *PuntSubject;        // pointer to subject
      ScriptTask                       *MainTask;           // pointer to exec assigned to main script
      ScriptTask                       *TimeTask;           // pointer to exec assigned to time script
      ScriptTask                       *CloseTask;          // pointer to exec assigned to close script
      ScriptErrorList                  *ErrorList;          // pointer to the last error list
      EXEC_MODE                        ExecMode;            // execution mode
      int                              CurrentScript;       // script currently executed
      int                              CurrentLine;         // line currenty executed
      bool                             Begin;               // execution at the begin
      int                              TimeScriptInterval;  // interval for time script (minutes)
      bool                             TimeScriptEnable;    // enable time script
      int                              HourCloseScript;     // hour for close script activation
      int                              MinuteCloseScript;   // minute for close script activation
      int                              TimerScript;         // timer for time script activation (in milliseconds)
      bool                             MainScriptRun;       // main script is running
      bool                             TimeScriptRun;       // time script is running
      bool                             CloseScriptRun;      // Close script is running
      bool                             CloseScriptEnd;      // Close script is finished
      int                              PrevTick;            // previous tick count
      bool                             Control;             // control activated
      int                              SafeElevation;       // safe elevation in degree for program working

      // Threaded function
      void __fastcall                  Execute(void);       // execute program scheduling
};
#endif


