/*---------------------------------------------------------------------------

	ScriptTask.cpp  script exec for observatory management

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : October 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// system include
#include <string.h>
#include <stdlib.h>
#include <ctype.h>


// project include
#include "ScriptTask.h"
#include "KeyWords.h"
#include "CMEvent.h"



/*---------------------------------------------------------------------------

	object constructor

---------------------------------------------------------------------------*/

ScriptTask::ScriptTask()
{
   Script=0;
   GRBTriggerIndex = 0;
}



/*---------------------------------------------------------------------------

	object destroyer

---------------------------------------------------------------------------*/

ScriptTask::~ScriptTask(  )
{
   if ( Script != 0 )
      free ( Script );
}



/*---------------------------------------------------------------------------

   set subject

---------------------------------------------------------------------------*/

void ScriptTask::SetSubject(
   TentaclesMgr         *ParPuntSubject,                    // pointer to subject
   Observer             *ParExecObserver,
   int                  ParSafeElevation
)
{
   PuntSubject = ParPuntSubject;                            // store pointer
   ExecObserver = ParExecObserver;
   SafeElevation = ParSafeElevation;
}



/*---------------------------------------------------------------------------

   set GRB trigger

---------------------------------------------------------------------------*/

void ScriptTask::SetGRBTrigger( int PGRBTriggerIndex )
{
   GRBTriggerIndex = PGRBTriggerIndex;
}



/*---------------------------------------------------------------------------

	set editor graphic interface

---------------------------------------------------------------------------*/

void ScriptTask::SetScript( char *ParScript )
{
   int                                 Lenght;              // script lenght

   Lenght = strlen( ParScript ) + 1;                        // get script lenght + rerminator
   Script = (char *) malloc (Lenght);                       // allocate memory
   strcpy( Script, ParScript );                             // copy script
   End = Script + Lenght;                                   // end of script
   Reset();                                                 // reset execution
}



/*---------------------------------------------------------------------------

	reset execution

---------------------------------------------------------------------------*/

void ScriptTask::Reset( void )
{
   int                                 Index;               // scan table index

   Scanner = Script;                                        // next execution is the firt token in the script
   Line=1;                                                  // first line
   Column=1;                                                // first column
   StackPointer=0;                                          // reset stack pointer
   for ( Index=0; Index<MAX_VARIABLE; Index++ )             // scan all elements in the table
      Variables[Index].Valid = false;                       // mark position as free
   PrevCommandLine = -1;                                    // make invalid previous command line
   strcpy( ObjectName, "" );                                // clera corrent object name
}



/*---------------------------------------------------------------------------

	execute next step

---------------------------------------------------------------------------*/

void ScriptTask::Step( int &StepLine, int &StepColumn, int &Error, bool Simulate )
{
   char                                Command[40];         // command
   int                                 CommandLine;         // line where found command
   int                                 CommandColumn;       // column where found command
   char                                *CommandScanner;     // scanner position where found command
   bool                                TokenOk;             // token found

   EndStepRequest = false;                                  // reset end of step request

   // get command
   do
   {
      TokenOk = GetToken( Command, &CommandLine, &CommandColumn, &CommandScanner, true );
      // no token when command is attended means regular end of file
      if ( TokenOk == false )
      {
         Error = ERR_ENDOFSCRIPT;
         return;
      }
   } while ( CommandLine == PrevCommandLine );

   StepLine = CommandLine;                                  // output line and column are command ones
   StepColumn = CommandColumn;
   PrevCommandLine = CommandLine;                           // save command line for next time
   LowerCase( Command );                                    // command in lower case
   Error = ERR_NONE;                                        // until now no error

  // manage goto command
   if ( strcmp(Command, KEYWORD_GOTO ) == 0 )
   {
      ExecGoto( StepLine, StepColumn, Error, Simulate );
      return;
   }

   // manage traking command
   if ( strcmp(Command, KEYWORD_TRACKING ) == 0 )
   {
      ExecTracking( StepLine, StepColumn, Error, Simulate );
      return;
   }

   // manage dome command
   if ( strcmp(Command, KEYWORD_DOME ) == 0 )
   {
      ExecDome( StepLine, StepColumn, Error, Simulate );
      return;
   }

   // manage sliding shutter command
   if ( strcmp(Command, KEYWORD_SSHUTTER ) == 0 )
   {
      ExecSShutter( StepLine, StepColumn, Error, Simulate );
      return;
   }

   // manage balancing shutter command
   if ( strcmp(Command, KEYWORD_BSHUTTER ) == 0 )
   {
      ExecBShutter( StepLine, StepColumn, Error, Simulate );
      return;
   }

   // manage decondence command
   if ( strcmp(Command, KEYWORD_DECONDENCE ) == 0 )
   {
      ExecDecondence( StepLine, StepColumn, Error, Simulate );
      return;
   }

   // manage power command
   if ( strcmp(Command, KWYWORD_POWER ) == 0 )
   {
      ExecPower( StepLine, StepColumn, Error, Simulate );
      return;
   }

   // manage shot command
   if ( strcmp(Command, KEYWORD_SHOT ) == 0 )
   {
      ExecShot( StepLine, StepColumn, Error, Simulate );
      return;
   }

   // manage flat command
   if ( strcmp(Command, KEYWORD_FLAT ) == 0 )
   {
      ExecFlat( StepLine, StepColumn, Error, Simulate );
      return;
   }

   // manage dark command
   if ( strcmp(Command, KEYWORD_DARK ) == 0 )
   {
      ExecDark( StepLine, StepColumn, Error, Simulate );
      return;
   }

   // manage sky check command
   if ( strcmp(Command, KEYWORD_SKYCHECK ) == 0 )
   {
      ExecSkyCheck( StepLine, StepColumn, Error, Simulate );
      return;
   }

   // manage wait command
   if ( strcmp(Command, KEYWORD_WAIT ) == 0 )
   {
      ExecWait( StepLine, StepColumn, Error, Simulate );
      return;
   }

   // manage repeat command
   if ( strcmp(Command, KEYWORD_REPEAT ) == 0 )
   {
      ExecRepeat( StepLine, StepColumn, CommandScanner, Error, Simulate );
      return;
   }

   // manage end repeat command
   if ( strcmp(Command, KEYWORD_ENDREPEAT ) == 0 )
   {
      ExecEndRepeat( Error);
      return;
   }

   // manage velocity command
   if ( strcmp(Command, KEYWORD_VELOCITY ) == 0 )
   {
      ExecVelocity( StepLine, StepColumn, Error, Simulate );
      return;
   }

   // manage telescope alignment
   if ( strcmp(Command, KEYWORD_ALLIGN ) == 0 )
   {
      ExecTeleAlignment( StepLine, StepColumn, Error, Simulate );
      return;
   }

   // manage sn search
   if ( strcmp(Command, KEYWORD_SEARHSN ) == 0 )
   {
      ExecSNSearch( StepLine, StepColumn, Error, Simulate );
      return;
   }

   Error = ERR_SINTAX;
}



/*---------------------------------------------------------------------------

	end of step request

---------------------------------------------------------------------------*/

void ScriptTask::EndStep( void )
{
   EndStepRequest = true;
}



/*---------------------------------------------------------------------------

	execute goto istruction

---------------------------------------------------------------------------*/

void ScriptTask::ExecGoto( int &StepLine, int &StepColumn, int &Error, bool Simulate )
{
   char                                Par1[40];            // first parameter
   char                                Par2[40];            // second parameter
   char                                Par3[40];            // third parameter
   int                                 ParLine;             // line where found parameter
   int                                 ParColumn;           // column where found parameter
   bool                                TokenOk;             // token found
   TentaclesMgr::ERR                   CommError;           // command error code
   CMHours::HOURS                      RA;
   CMDegree::DEGREE                    Dec;
   int                                 RA_hh;
   int                                 RA_mm;
   int                                 RA_ss;
   char                                Dec_Sign;
   int                                 Dec_gg;
   int                                 Dec_mm;
   int                                 Dec_ss;
   CMObjectList                        *List;
   CMObject*                           Obj;                 // generig object
   const char                          *Name;               // object name
   const CMCoEquatorial                *EquPos;             // object equatorial position
   CMCoEquatorial                      *EquPos2000;         // object equatorial position
   double                              JDCurrent;           // current julian day
   double                              JD2000;              // 2000 julian day
   SYSTEMTIME                          Time;                // current data and time
   CMEvent                             *Now;                // current data and time event object
   CMEvent::DATE                       NowDate;             // current data and time event data
   int                                 Hour;
   int                                 Minutes;
   bool                                Found;
   int                                 Par2Len;
   TentaclesMgr::STATUS                Status;              // observatory status
   int                                 Elevation;           // current telescope elevation
   char                                Temp[10];

   // get kind of object
   TokenOk = GetToken( Par1, &ParLine, &ParColumn, 0, false );
   // no token means premature ond of file
   if ( TokenOk == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PREMATUREEND;                             // error code
      return;
   }
   LowerCase( Par1 );                                       // parameter in lower case
   // kind of object should be one of these
   if ( /*strcmp( Par1, PARAMETER_PLANET)     != 0 &&  */
        strcmp( Par1, PARAMETER_EQUATORIAL) != 0 &&
        strcmp( Par1, PARAMETER_EQU2000) != 0 &&
        strcmp( Par1, PARAMETER_GRB) != 0 &&
        //strcmp( Par1, PARAMETER_ALTAZIMUT)  != 0 &&
        strcmp( Par1, PARAMETER_MPLANET)    != 0 /*&&
        strcmp( Par1, PARAMETER_NEBULA)     != 0 &&
        strcmp( Par1, PARAMETER_CLUSTER)    != 0 &&
        strcmp( Par1, PARAMETER_GALAXY)     != 0 */&&
        //strcmp( Par1, PARAMETER_STAR)       != 0 &&
        strcmp( Par1, PARAMETER_HOME)       != 0  )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_WRONGPARAMETER;                           // error code
      return;
   }

   // get object identifier
   if ( strcmp( Par1, PARAMETER_HOME)       == 0     )
   {
      TokenOk = GetToken( Par2, &ParLine, &ParColumn, 0, false );
      // no token means premature end of file
      if ( TokenOk == false )
      {
         StepLine = ParLine;                                // error position
         StepColumn = ParColumn;
         Error = ERR_PREMATUREEND;                          // error code
         return;
      }

      // command is correct
      Error = ERR_NONE;

      // in simulation mode command in not executed
      if (Simulate == true )
         return;

      // exec command
      CommError = PuntSubject->RunCommand( ExecObserver, TentaclesMgr::COM_SUBJ_TELE, TentaclesMgr::COM_STA_HOME);
      if ( CommError != TentaclesMgr::ERR_NONE )
         Error = ERR_EXECUTION;

      do
      {
         Sleep ( 1000 );
         PuntSubject->GetStatus( &Status);
      } while ( Status.TeleMoving == true && EndStepRequest == false);

      return;
   }

   // manage equatorial positioning
   if ( strcmp( Par1, PARAMETER_EQUATORIAL) == 0 )
   {
      TokenOk = GetToken( Par2, &ParLine, &ParColumn, 0, false ); // get coordinate
      // no token means premature end of file
      if ( TokenOk == false )
      {
         StepLine = ParLine;                                // error position
         StepColumn = ParColumn;
         Error = ERR_PREMATUREEND;                          // error code
         return;
      }
      if ( IsCoo( Par2, &RA_hh, &RA_mm, &RA_ss, &Dec_Sign, &Dec_gg, &Dec_mm, &Dec_ss ) == false )
      {
         StepLine = ParLine;                                // error position
         StepColumn = ParColumn;
         Error = ERR_WRONGPARAMETER;                        // error code
         return;
      }
      TokenOk = GetToken( Par3, &ParLine, &ParColumn, 0, false ); // get object name
      // no token means premature end of file
      if ( TokenOk == false )
      {
         StepLine = ParLine;                                // error position
         StepColumn = ParColumn;
         Error = ERR_PREMATUREEND;                          // error code
         return;
      }
      if ( strcmp( Par3, "") == 0  )
      {
         StepLine = ParLine;                                // error position
         StepColumn = ParColumn;
         Error = ERR_WRONGPARAMETER;                        // error code
         return;
      }
      strcpy( ObjectName, Par3 );                           // set object name

      // command is correct
      Error = ERR_NONE;

      // in simulation mode command in not executed
      if (Simulate == true )
         return;

      // verify tescope whitout movement
      PuntSubject->GetStatus( &Status);
      if ( Status.TeleMoving == true )
      {
         PuntSubject->RunCommand( ExecObserver, TentaclesMgr::COM_SUBJ_TELE, TentaclesMgr::COM_STA_STOP);
         do
         {
            Sleep ( 1000 );
            PuntSubject->GetStatus( &Status);
         }
         while ( Status.TeleMoving == true && EndStepRequest == false );
      }
      if ( EndStepRequest == true )
         return;

      // command execution
      RA.hh = RA_hh;
      RA.mm = RA_mm;
      RA.ss = RA_ss;
      Dec.sign[0] = Dec_Sign;
      Dec.sign[1] = 0;
      Dec.gg = Dec_gg;
      Dec.mm = Dec_mm;
      Dec.ss = Dec_ss;
      CommError = PuntSubject->TelescopePos( ExecObserver, &RA, &Dec, Par3, false );
      if ( CommError != TentaclesMgr::ERR_NONE )
         Error = ERR_EXECUTION;

      do
      {
         Sleep ( 1000 );
         PuntSubject->GetStatus( &Status);
         Elevation = Status.TeleElevation.gg;
         if ( *Status.TeleElevation.sign == '-' )
            Elevation = - Elevation;
         if ( Elevation < SafeElevation )
            EndStepRequest = true;
      } while ( Status.TeleMoving == true && EndStepRequest == false );

      return;
   }

   // manage equatorial positioning with equinox = 2000
   if ( strcmp( Par1, PARAMETER_EQU2000) == 0 )
   {
      TokenOk = GetToken( Par2, &ParLine, &ParColumn, 0, false ); // get coordinate
      // no token means premature end of file
      if ( TokenOk == false )
      {
         StepLine = ParLine;                                // error position
         StepColumn = ParColumn;
         Error = ERR_PREMATUREEND;                          // error code
         return;
      }
      if ( IsCoo( Par2, &RA_hh, &RA_mm, &RA_ss, &Dec_Sign, &Dec_gg, &Dec_mm, &Dec_ss ) == false )
      {
         StepLine = ParLine;                                // error position
         StepColumn = ParColumn;
         Error = ERR_WRONGPARAMETER;                        // error code
         return;
      }
      TokenOk = GetToken( Par3, &ParLine, &ParColumn, 0, false ); // get object name
      // no token means premature end of file
      if ( TokenOk == false )
      {
         StepLine = ParLine;                                // error position
         StepColumn = ParColumn;
         Error = ERR_PREMATUREEND;                          // error code
         return;
      }
      if ( strcmp( Par3, "") == 0  )
      {
         StepLine = ParLine;                                // error position
         StepColumn = ParColumn;
         Error = ERR_WRONGPARAMETER;                        // error code
         return;
      }
      strcpy( ObjectName, Par3 );                           // set object name

      // command is correct
      Error = ERR_NONE;

      // in simulation mode command in not executed
      if (Simulate == true )
         return;

      // verify tescope whitout movement
      PuntSubject->GetStatus( &Status);
      if ( Status.TeleMoving == true )
      {
         PuntSubject->RunCommand( ExecObserver, TentaclesMgr::COM_SUBJ_TELE, TentaclesMgr::COM_STA_STOP);
         do
         {
            Sleep ( 1000 );
            PuntSubject->GetStatus( &Status);
         }
         while ( Status.TeleMoving == true && EndStepRequest == false );
      }
      if ( EndStepRequest == true )
         return;

      // command execution

      // create equ2000 coordinate
      RA.hh = RA_hh;
      RA.mm = RA_mm;
      RA.ss = RA_ss;
      Dec.sign[0] = Dec_Sign;
      Dec.sign[1] = 0;
      Dec.gg = Dec_gg;
      Dec.mm = Dec_mm;
      Dec.ss = Dec_ss;



      // compute current
      Now = new CMEvent();                                  // create event object
      GetSystemTime( &Time );                               // get current GMT time
      NowDate.Year = Time.wYear;                            // fill event structure
      NowDate.Month = Time.wMonth;
      NowDate.Day = Time.wDay;
      NowDate.Hour = Time.wHour;
      NowDate.Minute = Time.wMinute;
      NowDate.Sec = Time.wSecond;
      Now->SetDate(&NowDate);                               // set event object
      JDCurrent = Now->GetGG();                             // get julian day
      NowDate.Year = 2000;                                  // fill event structure
      NowDate.Month = 1;
      NowDate.Day = 1;
      NowDate.Hour = 0;
      NowDate.Minute = 0;
      NowDate.Sec = 0;
      Now->SetDate(&NowDate);                               // set event object
      JD2000 = Now->GetGG();                                // get julian day
      delete Now;                                           // destroy object

      EquPos2000 = new CMCoEquatorial();
      EquPos2000->GetRA()->Set( &RA );
      EquPos2000->GetDec()->Set( &Dec );
      EquPos2000->SetDataEqu(JD2000);
      EquPos = EquPos2000->ChangeEqu( JDCurrent );             // get equatorial position at actual epoch
      RA = *EquPos->GetRA()->Get();                         // get object RA
      Dec = *EquPos->GetDec()->Get();                       // get object declination

      CommError = PuntSubject->TelescopePos( ExecObserver, &RA, &Dec, Par3, false );
      if ( CommError != TentaclesMgr::ERR_NONE )
         Error = ERR_EXECUTION;

      do
      {
         Sleep ( 1000 );
         PuntSubject->GetStatus( &Status);
         Elevation = Status.TeleElevation.gg;
         if ( *Status.TeleElevation.sign == '-' )
            Elevation = - Elevation;
         if ( Elevation < SafeElevation )
            EndStepRequest = true;
      } while ( Status.TeleMoving == true && EndStepRequest == false );

      return;
   }

   // manage minor planet positioning
   if ( strcmp( Par1, PARAMETER_MPLANET) == 0 )
   {
      TokenOk = GetToken( Par2, &ParLine, &ParColumn, 0, false ); // get minor planet code
      // no token means premature end of file
      if ( TokenOk == false )
      {
         StepLine = ParLine;                                // error position
         StepColumn = ParColumn;
         Error = ERR_PREMATUREEND;                          // error code
         return;
      }

      TokenOk = GetToken( Par3, &ParLine, &ParColumn, 0, false ); // get time for position
      // no token means premature end of file
      if ( TokenOk == false )
      {
         StepLine = ParLine;                                // error position
         StepColumn = ParColumn;
         Error = ERR_PREMATUREEND;                          // error code
         return;
      }
      // parameter should be a time
      if ( IsTime( Par3, &Hour, &Minutes ) == false )
      {
         StepLine = ParLine;                                // error position
         StepColumn = ParColumn;
         Error = ERR_WRONGPARAMETER;                        // error code
         return;
      }
      if ( Hour < 12 )
         Hour += 24;

      // compute current
      Now = new CMEvent();                                  // create event object
      GetSystemTime( &Time );                               // get current GMT time
      NowDate.Year = Time.wYear;                            // fill event structure
      NowDate.Month = Time.wMonth;
      NowDate.Day = Time.wDay;
      NowDate.Hour = 0;
      NowDate.Minute = 0;
      NowDate.Sec = 0;
      Now->SetDate(&NowDate);                               // set event object
      JDCurrent = Now->GetGG();                             // get julian day at day begin
      if ( Time.wHour < 12 )
         JDCurrent--;
      delete Now;                                           // destroy object
      JDCurrent += (double)Hour/24 + (double)Minutes/1440;

      // Add minor planet to list
      CommError = PuntSubject->AddMinorPlanet( Par2 );
      if ( CommError != TentaclesMgr::ERR_NONE && CommError != TentaclesMgr::ERR_ALREADYPRESENT )
      {
         StepLine = ParLine;                                // error position
         StepColumn = ParColumn;
         Error = ERR_WRONGPARAMETER;                        // error code
         return;
      }
      List = PuntSubject->GetObjectList( TentaclesMgr::OBJECT_MINORPLANETS );
      List->First();                                        // reset internal cursor list
      Obj = List->Next();                                   // get first object
      Found = false;
      Par2Len = strlen( Par2 );
      while (Obj != 0 )                                     // works only until object is avalable
      {
         Name = Obj->GetName(CMObject::NAME_CATALOG);       // get object catalog name
         if ( strcmp(Name+5-Par2Len, Par2) == 0 )           // if it is my object
         {
            Obj->SetData(JDCurrent);                       // get set time for position
            EquPos = Obj->GetPosition();                    // get equatorial position J2000
            EquPos = EquPos->ChangeEqu( JDCurrent );       // get equatorial position at actual epoch
            RA = *EquPos->GetRA()->Get();                   // get object RA
            Dec = *EquPos->GetDec()->Get();                 // get object declination
            Found = true;
            break;
         }
         Obj = List->Next();                                // go to next object in the list
      }
      if ( Found == false )
      {
         StepLine = ParLine;                                // error position
         StepColumn = ParColumn;
         Error = ERR_PREMATUREEND;                          // error code
         return;
      }

      // command is correct
      strcpy( ObjectName, Par2 );                           // set object name
      Error = ERR_NONE;

      // in simulation mode command in not executed
      if (Simulate == true )
         return;

      // verify tescope whitout movement
      PuntSubject->GetStatus( &Status);
      if ( Status.TeleMoving == true )
      {
         PuntSubject->RunCommand( ExecObserver, TentaclesMgr::COM_SUBJ_TELE, TentaclesMgr::COM_STA_STOP);
         do
         {
            Sleep ( 1000 );
            PuntSubject->GetStatus( &Status);
         } while ( Status.TeleMoving == true && EndStepRequest == false );
      }
      if ( EndStepRequest == true )
         return;

      // command execution
      CommError = PuntSubject->TelescopePos( ExecObserver, &RA, &Dec, Par2, false );
      if ( CommError != TentaclesMgr::ERR_NONE )
         Error = ERR_EXECUTION;
      do
      {
         Sleep ( 1000 );
         PuntSubject->GetStatus( &Status);
         Elevation = Status.TeleElevation.gg;
         if ( *Status.TeleElevation.sign == '-' )
            Elevation = - Elevation;
         if ( Elevation < SafeElevation )
            EndStepRequest = true;
      } while ( Status.TeleMoving == true && EndStepRequest == false);

      return;
   }

   // manage GRB positioning
   if ( strcmp( Par1, PARAMETER_GRB) == 0 )
   {
      // command is correct
      Error = ERR_NONE;
      strcpy( ObjectName, "GRB" );                           // set object name
      strcpy( ObjectName, "GRB_TRIGGER_" );
      itoa( GRBTriggerIndex, Temp, 10);
      strcat( ObjectName, Temp );

      // in simulation mode command in not executed
      if (Simulate == true )
         return;

      // verify tescope whitout movement
      PuntSubject->GetStatus( &Status);
      if ( Status.TeleMoving == true )
      {
         PuntSubject->RunCommand( ExecObserver, TentaclesMgr::COM_SUBJ_TELE, TentaclesMgr::COM_STA_STOP);
         do
         {
            Sleep ( 1000 );
            PuntSubject->GetStatus( &Status);
         } while ( Status.TeleMoving == true && EndStepRequest == false );
      }
      if ( EndStepRequest == true )
         return;

      // position telescope in GRB
      CommError = PuntSubject->TelescopeGRBPos( ExecObserver );
      if ( CommError != TentaclesMgr::ERR_NONE )
         Error = ERR_EXECUTION;

      do
      {
         Sleep ( 1000 );
         PuntSubject->GetStatus( &Status);
      } while ( Status.TeleMoving == true && EndStepRequest == false );

      return;
   }


   return;
}



/*---------------------------------------------------------------------------

	execute tracking instruction

---------------------------------------------------------------------------*/

void ScriptTask::ExecTracking( int &StepLine, int &StepColumn, int &Error, bool Simulate )
{
   char                                Par1[40];            // first parameter
   int                                 ParLine;             // line where found parameter
   int                                 ParColumn;           // column where found parameter
   bool                                TokenOk;             // token found
   TentaclesMgr::ERR                   CommError;           // command error code

   // get parameter
   TokenOk = GetToken( Par1, &ParLine, &ParColumn, 0, false );
   // no token means premature ond of file
   if ( TokenOk == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PREMATUREEND;                             // error code
      return;
   }
   LowerCase( Par1 );                                       // parameter in lower case
   //parameter should be one of these
   if ( strcmp( Par1, PARAMETER_ON)  != 0 &&
        strcmp( Par1, PARAMETER_OFF) != 0  )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_WRONGPARAMETER;                           // error code
      return;
   }

   // command is correct
   Error = ERR_NONE;

   // in simulation mode command in not executed
   if (Simulate == true )
      return;

   // command execution
   if ( strcmp( Par1, PARAMETER_ON) == 0 )
      CommError = PuntSubject->RunCommand( ExecObserver, TentaclesMgr::COM_SUBJ_TRACK, TentaclesMgr::COM_STA_ON);
   if ( strcmp( Par1, PARAMETER_OFF) == 0 )
      CommError = PuntSubject->RunCommand( ExecObserver, TentaclesMgr::COM_SUBJ_TRACK, TentaclesMgr::COM_STA_OFF);
   if ( CommError != TentaclesMgr::ERR_NONE )
      Error = ERR_EXECUTION;

   return;
}



/*---------------------------------------------------------------------------

	execute dome instruction

---------------------------------------------------------------------------*/

void ScriptTask::ExecDome( int &StepLine, int &StepColumn, int &Error, bool Simulate )
{
   char                                Par1[40];            // first parameter
   int                                 ParLine;             // line where found parameter
   int                                 ParColumn;           // column where found parameter
   bool                                TokenOk;             // token found
   TentaclesMgr::STATUS                Status;              // observatory status
   TentaclesMgr::ERR                   CommError;           // command error code

   // get parameter
   TokenOk = GetToken( Par1, &ParLine, &ParColumn, 0, false );
   // no token means premature ond of file
   if ( TokenOk == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PREMATUREEND;                             // error code
      return;
   }
   LowerCase( Par1 );                                       // parameter in lower case
   //parameter should be one of these
   if ( strcmp( Par1, PARAMETER_AUTO) != 0 &&
        strcmp( Par1, PARAMETER_ZERO) != 0 &&
        strcmp( Par1, PARAMETER_HOME) != 0  )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_WRONGPARAMETER;                           // error code
      return;
   }

   // command is correct
   Error = ERR_NONE;

   // in simulation mode command in not executed
   if (Simulate == true )
      return;

   // command execution
   if ( strcmp( Par1, PARAMETER_AUTO) == 0 )
      CommError = PuntSubject->RunCommand( ExecObserver, TentaclesMgr::COM_SUBJ_DOME, TentaclesMgr::COM_STA_LOCK);
   if ( strcmp( Par1, PARAMETER_ZERO) == 0 )
   {
      CommError = PuntSubject->RunCommand( ExecObserver, TentaclesMgr::COM_SUBJ_DOME, TentaclesMgr::COM_STA_UNLOCK);
      CommError = PuntSubject->RunCommand( ExecObserver, TentaclesMgr::COM_SUBJ_DOME, TentaclesMgr::COM_STA_ZERO);
      if ( CommError == TentaclesMgr::ERR_NONE )
      {
         do
         {
            Sleep ( 1000 );
            PuntSubject->GetStatus( &Status);
         } while ( (Status.DomeAlligned == false || Status.DomeAzimut != 270) && EndStepRequest == false );
      }
   }
   if ( strcmp( Par1, PARAMETER_HOME) == 0 )
   {
      CommError = PuntSubject->RunCommand( ExecObserver, TentaclesMgr::COM_SUBJ_DOME, TentaclesMgr::COM_STA_UNLOCK);
      CommError = PuntSubject->RunCommand( ExecObserver, TentaclesMgr::COM_SUBJ_DOME, TentaclesMgr::COM_STA_HOME);
      if ( CommError == TentaclesMgr::ERR_NONE )
      {
         do
         {
            Sleep ( 1000 );
            PuntSubject->GetStatus( &Status);
         } while ( (Status.DomeAlligned == false || Status.DomeHome == false) && EndStepRequest == false );
      }
   }
   if ( CommError != TentaclesMgr::ERR_NONE )
      Error = ERR_EXECUTION;

   return;
}



/*---------------------------------------------------------------------------

	execute sliding shutter instruction

---------------------------------------------------------------------------*/

void ScriptTask::ExecSShutter( int &StepLine, int &StepColumn, int &Error, bool Simulate )
{
   char                                Par1[40];            // first parameter
   int                                 ParLine;             // line where found parameter
   int                                 ParColumn;           // column where found parameter
   bool                                TokenOk;             // token found
   TentaclesMgr::STATUS                Status;              // observatory status
   TentaclesMgr::ERR                   CommError;           // command error code

   // get parameter
   TokenOk = GetToken( Par1, &ParLine, &ParColumn, 0, false );
   // no token means premature ond of file
   if ( TokenOk == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PREMATUREEND;                             // error code
      return;
   }
   LowerCase( Par1 );                                       // parameter in lower case
   //parameter should be one of these
   if ( strcmp( Par1, PARAMETER_OPEN) != 0  &&
        strcmp( Par1, PARAMETER_CLOSE) != 0    )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_WRONGPARAMETER;                           // error code
      return;
   }

   // command is correct
   Error = ERR_NONE;

   // in simulation mode command in not executed
   if (Simulate == true )
      return;

   // command execution
   if ( strcmp( Par1, PARAMETER_OPEN) == 0 )
   {
      CommError = PuntSubject->RunCommand( ExecObserver, TentaclesMgr::COM_SUBJ_SSHUTTER, TentaclesMgr::COM_STA_OPEN);
      if ( CommError == TentaclesMgr::ERR_NONE )
      {
         do
         {
            Sleep ( 1000 );
            PuntSubject->GetStatus( &Status);
         } while ( Status.DomeSlidingShutter != 0 && EndStepRequest == false );
      }
   }
   if ( strcmp( Par1, PARAMETER_CLOSE) == 0 )
   {
      CommError = PuntSubject->RunCommand( ExecObserver, TentaclesMgr::COM_SUBJ_SSHUTTER, TentaclesMgr::COM_STA_CLOSE);
      if ( CommError == TentaclesMgr::ERR_NONE )
      {
         do
         {
            Sleep ( 1000 );
            PuntSubject->GetStatus( &Status);
         } while ( Status.DomeSlidingShutter != 1 && EndStepRequest == false );
      }
   }
   if ( CommError != TentaclesMgr::ERR_NONE )
      Error = ERR_EXECUTION;

   return;
}



/*---------------------------------------------------------------------------

	execute balancing shutter instruction

---------------------------------------------------------------------------*/

void ScriptTask::ExecBShutter( int &StepLine, int &StepColumn, int &Error, bool Simulate )
{
   char                                Par1[40];            // first parameter
   int                                 ParLine;             // line where found parameter
   int                                 ParColumn;           // column where found parameter
   bool                                TokenOk;             // token found
   TentaclesMgr::STATUS                Status;              // observatory status
   TentaclesMgr::ERR                   CommError;           // command error code

   // get parameter
   TokenOk = GetToken( Par1, &ParLine, &ParColumn, 0, false );
   // no token means premature ond of file
   if ( TokenOk == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PREMATUREEND;                             // error code
      return;
   }
   LowerCase( Par1 );                                       // parameter in lower case
   //parameter should be one of these
   if ( strcmp( Par1, PARAMETER_OPEN) != 0  &&
        strcmp( Par1, PARAMETER_CLOSE) != 0    )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_WRONGPARAMETER;                           // error code
      return;
   }

   // command is correct
   Error = ERR_NONE;

   // in simulation mode command in not executed
   if (Simulate == true )
      return;

   // command execution
   if ( strcmp( Par1, PARAMETER_OPEN) == 0 )
   {
      CommError = PuntSubject->RunCommand( ExecObserver, TentaclesMgr::COM_SUBJ_BSHUTTER, TentaclesMgr::COM_STA_OPEN);
      if ( CommError == TentaclesMgr::ERR_NONE )
      {
         do
         {
            Sleep ( 1000 );
            PuntSubject->GetStatus( &Status);
         } while ( Status.DomeBalancingShutter != 0 && EndStepRequest == false );
      }
   }
   if ( strcmp( Par1, PARAMETER_CLOSE) == 0 )
   {
      CommError = PuntSubject->RunCommand( ExecObserver, TentaclesMgr::COM_SUBJ_BSHUTTER, TentaclesMgr::COM_STA_CLOSE);
      if ( CommError == TentaclesMgr::ERR_NONE )
      {
         do
         {
            Sleep ( 1000 );
            PuntSubject->GetStatus( &Status);
         } while ( Status.DomeBalancingShutter != 1 && EndStepRequest == false );
      }
   }
   if ( CommError != TentaclesMgr::ERR_NONE )
      Error = ERR_EXECUTION;

   return;
}



/*---------------------------------------------------------------------------

	execute decondence instruction

---------------------------------------------------------------------------*/

void ScriptTask::ExecDecondence( int &StepLine, int &StepColumn, int &Error, bool Simulate )
{
   char                                Par1[40];            // first parameter
   int                                 ParLine;             // line where found parameter
   int                                 ParColumn;           // column where found parameter
   bool                                TokenOk;             // token found
   TentaclesMgr::ERR                   CommError;           // command error code

   // get parameter
   TokenOk = GetToken( Par1, &ParLine, &ParColumn, 0, false );
   // no token means premature ond of file
   if ( TokenOk == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PREMATUREEND;                             // error code
      return;
   }
   LowerCase( Par1 );                                       // parameter in lower case
   //parameter should be one of these
   if ( strcmp( Par1, PARAMETER_ON)  != 0 &&
        strcmp( Par1, PARAMETER_OFF) != 0  )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_WRONGPARAMETER;                           // error code
      return;
   }

   // command is correct
   Error = ERR_NONE;

   // in simulation mode command in not executed
   if (Simulate == true )
      return;

   // command execution
   if ( strcmp( Par1, PARAMETER_ON) == 0 )
      CommError = PuntSubject->RunCommand( ExecObserver, TentaclesMgr::COM_SUBJ_DECOND, TentaclesMgr::COM_STA_ON);
   if ( strcmp( Par1, PARAMETER_OFF) == 0 )
      CommError = PuntSubject->RunCommand( ExecObserver, TentaclesMgr::COM_SUBJ_DECOND, TentaclesMgr::COM_STA_OFF);
   if ( CommError != TentaclesMgr::ERR_NONE )
      Error = ERR_EXECUTION;

   return;
}



/*---------------------------------------------------------------------------

	execute power instruction

---------------------------------------------------------------------------*/

void ScriptTask::ExecPower( int &StepLine, int &StepColumn, int &Error, bool Simulate )
{
   char                                Par1[40];            // first parameter
   int                                 ParLine;             // line where found parameter
   int                                 ParColumn;           // column where found parameter
   bool                                TokenOk;             // token found
   TentaclesMgr::ERR                   CommError;           // command error code

   // get parameter
   TokenOk = GetToken( Par1, &ParLine, &ParColumn, 0, false );
   // no token means premature ond of file
   if ( TokenOk == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PREMATUREEND;                             // error code
      return;
   }
   LowerCase( Par1 );                                       // parameter in lower case
   //parameter should be one of these
   if ( strcmp( Par1, PARAMETER_ON)  != 0 &&
        strcmp( Par1, PARAMETER_OFF) != 0  )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_WRONGPARAMETER;                           // error code
      return;
   }

   // command is correct
   Error = ERR_NONE;

   // in simulation mode command in not executed
   if (Simulate == true )
      return;

   // command execution
   if ( strcmp( Par1, PARAMETER_ON) == 0 )
      CommError = PuntSubject->RunCommand( ExecObserver, TentaclesMgr::COM_SUBJ_POWER, TentaclesMgr::COM_STA_ON);
   if ( strcmp( Par1, PARAMETER_OFF) == 0 )
      CommError = PuntSubject->RunCommand( ExecObserver, TentaclesMgr::COM_SUBJ_POWER, TentaclesMgr::COM_STA_OFF);
   if ( CommError != TentaclesMgr::ERR_NONE )
      Error = ERR_EXECUTION;

   return;
}



/*---------------------------------------------------------------------------

	execute shot instruction

---------------------------------------------------------------------------*/

void ScriptTask::ExecShot( int &StepLine, int &StepColumn, int &Error, bool Simulate )
{
   char                                Par1[40];            // first parameter
   char                                Par2[40];            // second parameter
   int                                 ParLine;             // line where found parameter
   int                                 ParColumn;           // column where found parameter
   bool                                TokenOk;             // token found
   int                                 ShotTime;            // shot time in seconds
   TentaclesMgr::ERR                   CommError;           // command error code
   int                                 Exposing;            // current exposing time
   CCD::SHOT_INFO                      ShotInfo;            // last shot information

   // get time (seconds)
   TokenOk = GetToken( Par1, &ParLine, &ParColumn, 0, false );
   // no token means premature ond of file
   if ( TokenOk == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PREMATUREEND;                             // error code
      return;
   }
   // should be a number
   if ( IsNumber( Par1 ) == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_WRONGPARAMETER;                           // error code
      return;
   }
   // convert number
   ShotTime = atoi( Par1 );
   // time between 1 and 300
   if ( ShotTime < 1 || ShotTime > 300 )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PARAMOUTOFRANGE;                          // error code
      return;
   }

   // get object name (optional)
   TokenOk = GetToken( Par2, &ParLine, &ParColumn, 0, false );
   // no token means premature ond of file
   if ( TokenOk == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PREMATUREEND;                             // error code
      return;
   }
   if ( *Par2 != 0 )
      strcpy( ObjectName, Par2 );

   // command is correct
   Error = ERR_NONE;

   // in simulation mode command in not executed
   if (Simulate == true )
      return;

   // command execution
   CommError = PuntSubject->SetObjectName( ObjectName );    // set base name for shots
   if ( CommError == TentaclesMgr::ERR_NONE )
      CommError = PuntSubject->Shot( ExecObserver, ShotTime*1000);   // shot new image
   if ( CommError == TentaclesMgr::ERR_NONE )
   {
      do
      {
         Sleep ( 1000 );
         CommError = PuntSubject->IsShotFinisched(&Exposing); // check shot is finished
         if ( CommError == TentaclesMgr::ERR_NONE )         // until not finished stay in this state
         {
            PuntSubject->GetShot(ExecObserver);             // get shot
            CommError = PuntSubject->GetShotInfo(&ShotInfo);// get shot info in order to verify clouds
            if ( CommError == TentaclesMgr::ERR_CLOUDY )
            {
               Error = ERR_ENDOFSCRIPT;
               return;
            }
            PuntSubject->RunCommand( ExecObserver, TentaclesMgr::COM_SUBJ_NIGHTLYFOLDER, TentaclesMgr::COM_STA_SAVE);
            break;
         }
         else
         {
            if ( EndStepRequest == true )
            {
               CommError = PuntSubject->AbortShot(ExecObserver);
               if ( CommError == TentaclesMgr::ERR_NONE )
                  break;
            }
         }
      } while ( 1 );
   }
   if ( CommError != TentaclesMgr::ERR_NONE )
      Error = ERR_EXECUTION;

   return;
}



/*---------------------------------------------------------------------------

	execute flat instruction

---------------------------------------------------------------------------*/

void ScriptTask::ExecFlat( int &StepLine, int &StepColumn, int &Error, bool Simulate )
{
   char                                Par1[40];            // first parameter
   char                                Par2[40];            // second parameter
   char                                Par3[40];            // third parameter
   int                                 ParLine;             // line where found parameter
   int                                 ParColumn;           // column where found parameter
   bool                                TokenOk;             // token found
   int                                 FlatShot;            // number of flat shot
   int                                 DarkShot;            // number of dark shot
   int                                 ShotTime;            // shot time in seconds
   TentaclesMgr::ERR                   CommError;           // command error code
   int                                 Prog;                // progressive counter for shot
   int                                 Exposing;            // current exposing time

   // get flat shot
   TokenOk = GetToken( Par1, &ParLine, &ParColumn, 0, false );
   // no token means premature ond of file
   if ( TokenOk == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PREMATUREEND;                             // error code
      return;
   }
   // should be a number
   if ( IsNumber( Par1 ) == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_WRONGPARAMETER;                           // error code
      return;
   }
   // convert number
   FlatShot = atoi( Par1 );
   // time between 1 and 100
   if ( FlatShot < 1 || FlatShot > 100 )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PARAMOUTOFRANGE;                          // error code
      return;
   }

   // get dark shot
   TokenOk = GetToken( Par2, &ParLine, &ParColumn, 0, false );
   // no token means premature ond of file
   if ( TokenOk == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PREMATUREEND;                             // error code
      return;
   }
   // should be a number
   if ( IsNumber( Par2 ) == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_WRONGPARAMETER;                           // error code
      return;
   }
   // convert number
   DarkShot = atoi( Par2 );
   // time between 1 and 100
   if ( DarkShot < 1 || DarkShot > 100 )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PARAMOUTOFRANGE;                          // error code
      return;
   }

   // get time (seconds)
   TokenOk = GetToken( Par3, &ParLine, &ParColumn, 0, false );
   // no token means premature ond of file
   if ( TokenOk == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PREMATUREEND;                             // error code
      return;
   }
   // should be a number
   if ( IsNumber( Par3 ) == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_WRONGPARAMETER;                           // error code
      return;
   }
   // convert number
   ShotTime = atoi( Par3 );
   // time between 1 and 100
   if ( ShotTime < 1 || ShotTime > 100 )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PARAMOUTOFRANGE;                          // error code
      return;
   }

   // command is correct
   Error = ERR_NONE;

   // in simulation mode command in not executed
   if (Simulate == true )
      return;

   //
   // command execution
   //

   // turn on flat lights
   CommError = PuntSubject->RunCommand( ExecObserver, TentaclesMgr::COM_SUBJ_LIGHT, TentaclesMgr::COM_STA_ON);
   if ( CommError == TentaclesMgr::ERR_NONE )
      CommError = PuntSubject->SetObjectName( "Flat" );     // set base name for shots
   if ( CommError == TentaclesMgr::ERR_NONE )
   {
      for ( Prog=0; Prog < FlatShot; Prog++)
      {
         CommError = PuntSubject->Shot( ExecObserver, ShotTime*1000);   // shot new flat image
         if ( CommError != TentaclesMgr::ERR_NONE )
            break;
         do
         {
            Sleep ( 500 );
            CommError = PuntSubject->IsShotFinisched(&Exposing);  // check shot is finished
            if ( CommError == TentaclesMgr::ERR_NONE )      // until not finished stay in this state
            {
               CommError = PuntSubject->GetShot( ExecObserver );  // get shot
               if ( CommError != TentaclesMgr::ERR_NONE )
                  break;
               // save the shot in nightly folder
               CommError = PuntSubject->RunCommand( ExecObserver, TentaclesMgr::COM_SUBJ_NIGHTLYFOLDER, TentaclesMgr::COM_STA_SAVE);
               break;
            }
            else
            {
               if ( EndStepRequest == true )
               {
                  CommError = PuntSubject->AbortShot(ExecObserver);
                  if ( CommError == TentaclesMgr::ERR_NONE )
                     break;
               }
            }
         } while ( 1 );
         if ( CommError != TentaclesMgr::ERR_NONE || EndStepRequest == true)
            break;
      }
   }

   if ( EndStepRequest == true )
      return;

   if ( CommError == TentaclesMgr::ERR_NONE )
      CommError = PuntSubject->RunCommand( ExecObserver, TentaclesMgr::COM_SUBJ_LIGHT, TentaclesMgr::COM_STA_OFF); // turn off flat lights
   else
      PuntSubject->RunCommand( ExecObserver, TentaclesMgr::COM_SUBJ_LIGHT, TentaclesMgr::COM_STA_OFF); // turn off flat lights

   if ( CommError == TentaclesMgr::ERR_NONE )
      CommError = PuntSubject->SetObjectName( "Dark" );     // set base name for shots
   if ( CommError == TentaclesMgr::ERR_NONE )
   {
      for ( Prog=0; Prog < DarkShot; Prog++)
      {
         CommError = PuntSubject->Shot( ExecObserver, ShotTime*1000);   // shot new flat image
         if ( CommError != TentaclesMgr::ERR_NONE )
            break;
         do
         {
            Sleep ( 500 );
            CommError = PuntSubject->IsShotFinisched(&Exposing);  // check shot is finished
            if ( CommError == TentaclesMgr::ERR_NONE )      // until not finished stay in this state
            {
               CommError = PuntSubject->GetShot( ExecObserver );  // get shot
               if ( CommError != TentaclesMgr::ERR_NONE )
                  break;
               // save the shot in nightly folder
               CommError = PuntSubject->RunCommand( ExecObserver, TentaclesMgr::COM_SUBJ_NIGHTLYFOLDER, TentaclesMgr::COM_STA_SAVE);
               break;
            }
            else
            {
               if ( EndStepRequest == true )
               {
                  CommError = PuntSubject->AbortShot(ExecObserver);
                  if ( CommError == TentaclesMgr::ERR_NONE )
                     break;
               }
            }
         } while ( 1 );
         if ( CommError != TentaclesMgr::ERR_NONE || EndStepRequest == true)
            break;
      }
   }
   if ( CommError != TentaclesMgr::ERR_NONE )
      Error = ERR_EXECUTION;

   return;
}



/*---------------------------------------------------------------------------

	execute dark instruction

---------------------------------------------------------------------------*/

void ScriptTask::ExecDark( int &StepLine, int &StepColumn, int &Error, bool Simulate )
{
   char                                Par1[40];            // first parameter
   char                                Par2[40];            // second parameter
   int                                 ParLine;             // line where found parameter
   int                                 ParColumn;           // column where found parameter
   bool                                TokenOk;             // token found
   int                                 DarkShot;            // number of dark shot
   int                                 ShotTime;            // shot time in seconds
   TentaclesMgr::STATUS                Status;              // observatory status
   char                                Name[100];           // shot name
   char                                c[10];               // for conversion
   TentaclesMgr::ERR                   CommError;           // command error code
   int                                 Prog;                // progressive counter for shot
   int                                 Exposing;            // current exposing time

   // get dark shot
   TokenOk = GetToken( Par1, &ParLine, &ParColumn, 0, false );
   // no token means premature ond of file
   if ( TokenOk == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PREMATUREEND;                             // error code
      return;
   }
   // should be a number
   if ( IsNumber( Par1 ) == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_WRONGPARAMETER;                           // error code
      return;
   }
   // convert number
   DarkShot = atoi( Par1 );
   // time between 1 and 100
   if ( DarkShot < 1 || DarkShot > 100 )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PARAMOUTOFRANGE;                          // error code
      return;
   }

   // get time (seconds)
   TokenOk = GetToken( Par2, &ParLine, &ParColumn, 0, false );
   // no token means premature ond of file
   if ( TokenOk == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PREMATUREEND;                             // error code
      return;
   }
   // should be a number
   if ( IsNumber( Par2 ) == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_WRONGPARAMETER;                           // error code
      return;
   }
   // convert number
   ShotTime = atoi( Par2 );
   // time between 1 and 300
   if ( ShotTime < 1 || ShotTime > 300 )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PARAMOUTOFRANGE;                          // error code
      return;
   }

   // command is correct
   Error = ERR_NONE;

   // in simulation mode command in not executed
   if (Simulate == true )
      return;

   // command execution

   // generate base shot name
   CommError = PuntSubject->GetStatus( &Status );           // get subject status
   strcpy( Name, "Dark_" );
   itoa ( ShotTime, c, 10 );
   strcat( Name, c );
   if ( Status.DomeTemp >= 0 )
      strcat( Name, "+" );
   itoa ( Status.DomeTemp, c, 10 );
   strcat( Name, c );
   if ( CommError == TentaclesMgr::ERR_NONE )
      CommError = PuntSubject->SetObjectName( Name );       // set base name for shots

   if ( CommError == TentaclesMgr::ERR_NONE )
   {
      for ( Prog=0; Prog < DarkShot; Prog++)
      {
         CommError = PuntSubject->Shot( ExecObserver, ShotTime*1000);   // shot new dark image
         if ( CommError != TentaclesMgr::ERR_NONE )
            break;
         do
         {
            Sleep ( 500 );
            CommError = PuntSubject->IsShotFinisched(&Exposing);  // check shot is finished
            if ( CommError == TentaclesMgr::ERR_NONE )      // until not finished stay in this state
            {
               CommError = PuntSubject->GetShot( ExecObserver );  // get shot
               if ( CommError != TentaclesMgr::ERR_NONE )
                  break;
               // save the shot in nightly folder
               CommError = PuntSubject->RunCommand( ExecObserver, TentaclesMgr::COM_SUBJ_DARKFOLDER, TentaclesMgr::COM_STA_SAVE);
               break;
            }
            else
            {
               if ( EndStepRequest == true )
               {
                  CommError = PuntSubject->AbortShot(ExecObserver);
                  if ( CommError == TentaclesMgr::ERR_NONE )
                     break;
               }
            }
         } while ( 1 );
         if ( CommError != TentaclesMgr::ERR_NONE || EndStepRequest == true)
            break;
      }
   }
   if ( CommError != TentaclesMgr::ERR_NONE )
      Error = ERR_EXECUTION;

   return;
}



/*---------------------------------------------------------------------------

	execute sky check instruction

---------------------------------------------------------------------------*/

void ScriptTask::ExecSkyCheck( int &StepLine, int &StepColumn, int &Error, bool Simulate )
{
   char                                Par1[40];            // first parameter
   char                                Par2[40];            // second parameter
   int                                 ParLine;             // line where found parameter
   int                                 ParColumn;           // column where found parameter
   bool                                TokenOk;             // token found
   float                               Quality;             // quality treshould
   int                                 ClmbTime;            // Climb time before activate close script

   // get dark shot
   TokenOk = GetToken( Par1, &ParLine, &ParColumn, 0, false );
   // no token means premature ond of file
   if ( TokenOk == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PREMATUREEND;                             // error code
      return;
   }
   // should be a float
   if ( IsFloat( Par1 ) == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_WRONGPARAMETER;                           // error code
      return;
   }
   // convert number
   Quality = atof( Par1 );
   // time between 1 and 100
   if ( Quality < 1 || Quality > 50 )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PARAMOUTOFRANGE;                          // error code
      return;
   }

   // get climb time (minutes)
   TokenOk = GetToken( Par2, &ParLine, &ParColumn, 0, false );
   // no token means premature ond of file
   if ( TokenOk == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PREMATUREEND;                             // error code
      return;
   }
   // should be a number
   if ( IsNumber( Par2 ) == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_WRONGPARAMETER;                           // error code
      return;
   }
   // convert number
   ClmbTime = atoi( Par2 );
   // time between 1 and 300
   if ( ClmbTime < 1 || ClmbTime > 60 )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PARAMOUTOFRANGE;                          // error code
      return;
   }

   // command is correct
   Error = ERR_NONE;

   // in simulation mode command in not executed
   if (Simulate == true )
      return;

   // command execution
   PuntSubject->SetSafeAlgorithm( ExecObserver, Quality, ClmbTime*60 );

   return;
}



/*---------------------------------------------------------------------------

	execute wait instruction

---------------------------------------------------------------------------*/

void ScriptTask::ExecWait( int &StepLine, int &StepColumn, int &Error, bool Simulate )
{
   char                                Par1[40];            // first parameter
   int                                 ParLine;             // line where found parameter
   int                                 ParColumn;           // column where found parameter
   bool                                TokenOk;             // token found
   int                                 WaitTime;            // wait time in seconds

   // get time (seconds)
   TokenOk = GetToken( Par1, &ParLine, &ParColumn, 0, false );
   // no token means premature ond of file
   if ( TokenOk == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PREMATUREEND;                             // error code
      return;
   }
   // should be a number
   if ( IsNumber( Par1 ) == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_WRONGPARAMETER;                           // error code
      return;
   }
   // convert number
   WaitTime = atoi( Par1 );
   // time between 1 and 100
   if ( WaitTime < 1 || WaitTime > 100 )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PARAMOUTOFRANGE;                          // error code
      return;
   }

   // command is correct
   Error = ERR_NONE;

   // in simulation mode command in not executed
   if (Simulate == true )
      return;

   // command execution
   while ( WaitTime > 0 && EndStepRequest == false )
   {
      Sleep( 1000 );
      WaitTime--;
   }

   return;
}



/*---------------------------------------------------------------------------

	execute repeat instruction

---------------------------------------------------------------------------*/

void ScriptTask::ExecRepeat( int &StepLine, int &StepColumn, char *StepScanner, int &Error, bool Simulate )
{
   char                                Par1[40];            // first parameter
   char                                Par2[40];            // second parameter
   int                                 ParLine;             // line where found parameter
   int                                 ParColumn;           // column where found parameter
   bool                                TokenOk;             // token found
   int                                 Times;               // times to repeat
   char                                VarName[40];         // variabile for times storing
   char                                C1[10];              // termporary for conversion
   bool                                Bypass;              // bypass all body
   int                                 VarIndex;            // variabile index
   int                                 Indent;              // repeat indent level
   bool                                SetError;            // error already set
   SYSTEMTIME                          Time;                // current data and time
   int                                 Hour;                // hour repeat end
   int                                 Minutes;             // minutes repeat end

   // command is correct for default
   Error = ERR_NONE;
   SetError = false;

   // stack must have spaces for new entry
   if ( StackPointer >= MAX_STACK )
   {
      Error = ERR_STACKFULL;                                // error code
      SetError = true;
   }

   // get type of repeat
   TokenOk = GetToken( Par1, &ParLine, &ParColumn, 0, false );
   // no token means premature ond of file
   if ( TokenOk == false && SetError == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PREMATUREEND;                             // error code
      SetError = true;
   }
   //parameter should be one of these
   if ( strcmp( Par1, PARAMETER_TIMES) != 0 &&
        strcmp( Par1, PARAMETER_UPTO)  != 0 &&
        SetError == false  )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_WRONGPARAMETER;                           // error code
      SetError = true;
   }


   // get repeat limit
   TokenOk = GetToken( Par2, &ParLine, &ParColumn, 0, false );
   // no token means premature ond of file
   if ( TokenOk == false && SetError == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PREMATUREEND;                             // error code
      SetError = true;
   }
   // check for repet for times
   if ( strcmp( Par1, PARAMETER_TIMES) == 0 )
   {
      // should be a number
      if ( IsNumber( Par2 ) == false && SetError == false )
      {
         StepLine = ParLine;                                // error position
         StepColumn = ParColumn;
         Error = ERR_WRONGPARAMETER;                        // error code
         SetError = true;
      }
      // convert number
      Times = atoi( Par2 );
      // time between 1 and 100
      if ( (Times < 1 || Times > 1000 ) && SetError == false )
      {
         StepLine = ParLine;                                // error position
         StepColumn = ParColumn;
         Error = ERR_PARAMOUTOFRANGE;                       // error code
         SetError = true;
      }
   }
   // check for repet up to time
   if ( strcmp( Par1, PARAMETER_UPTO) == 0 && SetError == false )
   {
      // should be a number
      if ( IsTime( Par2, &Hour, &Minutes ) == false )
      {
         StepLine = ParLine;                                // error position
         StepColumn = ParColumn;
         Error = ERR_WRONGPARAMETER;                        // error code
         SetError = true;
      }
      if ( Hour < 12 )
         Hour += 24;
   }


   // in simulation mode command is executed as 1 times
   if (Simulate == true )
   {
      strcpy( Par1, PARAMETER_TIMES );
      Times = 1;
   }

   //
   // command execution
   //

   Bypass = false;

   // check for repet for times
   if ( strcmp( Par1, PARAMETER_TIMES) == 0 )
   {
      // build variable name for times execution
      strcpy( VarName, "Repeat_" );
      itoa ( StepLine, C1, 10 );
      strcat( VarName, C1 );

      VarIndex = FindVariabile( VarName );                  // find variabile
      if ( VarIndex < 0 )                                   // variabile not present means : is the first time
      {
         VarIndex = AddVariable( VarName );                 // add variabile
         Variables[VarIndex].Value = Times;                 // set variable value
      }
      else
         Error = ERR_NONE;                                  // error detection only the firt loop
      if ( Variables[VarIndex].Value <= 0 )                 // if counter is 0 repeat is finifhed
      {
         RemoveVariable(VarIndex);                          // remove variabile from the list
         Bypass = true;                                     // force bypass of repeat body istructions
      }
   }

   // check for repet upto time
   if ( strcmp( Par1, PARAMETER_UPTO) == 0 )
   {
      GetSystemTime( &Time );                               // get current GMT time
      if ( Time.wHour < 12 )
         Time.wHour += (WORD)24;

      // verify if current time is over limit
      if ( Time.wHour > Hour )
         Bypass = true;                                     // force bypass of repeat body istructions
      if ( Time.wHour == Hour && Time.wMinute >= Minutes )
         Bypass = true;                                     // force bypass of repeat body istructions
   }

   if ( Bypass == false )
      PushPC( StepScanner, StepLine, StepColumn );          // save istruction start position into stack
   else
   {
      Indent = 1;                                           // actuall repeat indent level
      do
      {
         // get type of repeat
         TokenOk = GetToken( Par1, &ParLine, &ParColumn, 0, true );
         // no token means premature ond of file
         if ( TokenOk == false )
         {
            Error = ERR_PREMATUREEND;                       // error code
            return;
         }
         LowerCase( Par1 );                                 // token in lower case
         if ( strcmp(Par1, KEYWORD_REPEAT ) == 0 )          // repeat increase indent level
            Indent++;
         if ( strcmp(Par1, KEYWORD_ENDREPEAT ) == 0 )       // repeat increase indent level
            Indent--;
      } while ( Indent != 0 );                              // repeat until indent level arrive to 0
   }

   return;
}



/*---------------------------------------------------------------------------

	execute end repeat instruction

---------------------------------------------------------------------------*/

void ScriptTask::ExecEndRepeat(int &Error )
{
   char                                VarName[40];         // variabile for times storing
   char                                C1[10];              // termporary for conversion
   int                                 VarIndex;            // variabile index
   bool                                Ok;                  // something in the stack

   // build variable name for times execution
   strcpy( VarName, "Repeat_" );
   itoa ( Stack[StackPointer-1].Line, C1, 10 );
   strcat( VarName, C1 );

   VarIndex = FindVariabile( VarName );                     // find variabile
   if ( VarIndex >= 0 )
      Variables[VarIndex].Value--;                          // decrement variabile

   Ok = PullPC();                                           // recovery from stack
   if ( Ok == true )
      Error = ERR_NONE;
   else
      Error = ERR_STACKEMPTY;
   return;
}



/*---------------------------------------------------------------------------

	execute velocity instruction

---------------------------------------------------------------------------*/

void ScriptTask::ExecVelocity( int &StepLine, int &StepColumn, int &Error, bool Simulate )
{
   char                                Par1[40];            // first parameter
   int                                 ParLine;             // line where found parameter
   int                                 ParColumn;           // column where found parameter
   bool                                TokenOk;             // token found
   int                                 Velocity;            // velocity in percentage
   TentaclesMgr::ERR                   CommError;           // command error code

   // get time (seconds)
   TokenOk = GetToken( Par1, &ParLine, &ParColumn, 0, false );
   // no token means premature ond of file
   if ( TokenOk == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PREMATUREEND;                             // error code
      return;
   }
   // should be a number
   if ( IsNumber( Par1 ) == false )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_WRONGPARAMETER;                           // error code
      return;
   }
   // convert number
   Velocity = atoi( Par1 );
   // time between 1 and 100
   if ( Velocity < 1 || Velocity > 100 )
   {
      StepLine = ParLine;                                   // error position
      StepColumn = ParColumn;
      Error = ERR_PARAMOUTOFRANGE;                          // error code
      return;
   }

   // command is correct
   Error = ERR_NONE;

   // in simulation mode command in not executed
   if (Simulate == true )
      return;

   // command execution
   CommError = PuntSubject->SetAutoVelocity( ExecObserver, Velocity);
   if ( CommError != TentaclesMgr::ERR_NONE )
      Error = ERR_EXECUTION;

   return;
}



/*---------------------------------------------------------------------------

	execute telescope alignment

---------------------------------------------------------------------------*/

void ScriptTask::ExecTeleAlignment( int &StepLine, int &StepColumn, int &Error, bool Simulate )
{
   char                                Par1[40];            // first parameter
   int                                 ParLine;             // line where found parameter
   int                                 ParColumn;           // column where found parameter
   int                                 ShotTime;            // shot time in seconds
   TentaclesMgr::ERR                   CommError;           // command error code
   int                                 Exposing;            // current exposing time
   CCD::SHOT_INFO                      ShotInfo;            // last shot information

   // get time (seconds)
   GetToken( Par1, &ParLine, &ParColumn, 0, false );
   // empty token means no shot
   if ( *Par1 != 0 )
   {
      // should be a number
      if ( IsNumber( Par1 ) == false )
      {
         StepLine = ParLine;                                   // error position
         StepColumn = ParColumn;
         Error = ERR_WRONGPARAMETER;                           // error code
         return;
      }
      // convert number
      ShotTime = atoi( Par1 );
      // time between 1 and 300
      if ( ShotTime < 1 || ShotTime > 300 )
      {
         StepLine = ParLine;                                   // error position
         StepColumn = ParColumn;
         Error = ERR_PARAMOUTOFRANGE;                          // error code
         return;
      }
   }

   // command is correct
   Error = ERR_NONE;

   // in simulation mode command in not executed
   if (Simulate == true )
      return;

   // command execution

   // shot if required
   if ( *Par1 != 0 )
   {
      CommError = PuntSubject->SetObjectName( ObjectName );    // set base name for shots
      if ( CommError == TentaclesMgr::ERR_NONE )
         CommError = PuntSubject->Shot( ExecObserver, ShotTime*1000);   // shot new image
      if ( CommError == TentaclesMgr::ERR_NONE )
      {
         do
         {
            Sleep ( 1000 );
            CommError = PuntSubject->IsShotFinisched(&Exposing); // check shot is finished
            if ( CommError == TentaclesMgr::ERR_NONE )         // until not finished stay in this state
            {
               PuntSubject->GetShot(ExecObserver);             // get shot
               CommError = PuntSubject->GetShotInfo(&ShotInfo);// get shot info in order to verify clouds
               if ( CommError == TentaclesMgr::ERR_CLOUDY )
               {
                  Error = ERR_ENDOFSCRIPT;
                  return;
               }
               break;
            }
            else
            {
               if ( EndStepRequest == true )
               {
                  CommError = PuntSubject->AbortShot(ExecObserver);
                  if ( CommError == TentaclesMgr::ERR_NONE )
                     break;
               }
            }
         } while ( 1 );
      }
      if ( CommError != TentaclesMgr::ERR_NONE )
         Error = ERR_EXECUTION;
   }

   // put request for allignment execution
   // found problem win the direct run in this thread
   // so now store the information for button pushing simulation
   PuntSubject->SetAlignmentRequest( ExecObserver );
   // whait command is executed
   while ( PuntSubject->TestAlignmentFinished() == false )
      Sleep ( 1000 );

   // save short file with new cocrdinate
   PuntSubject->RunCommand( ExecObserver, TentaclesMgr::COM_SUBJ_NIGHTLYFOLDER, TentaclesMgr::COM_STA_SAVE);
   return;
}



/*---------------------------------------------------------------------------

	SN search

---------------------------------------------------------------------------*/

void ScriptTask::ExecSNSearch( int &StepLine, int &StepColumn, int &Error, bool Simulate )
{
   ImageCompare::OBJECT_CHAIN            *ObjectList;

   // command is correct
   Error = ERR_NONE;

   // in simulation mode command in not executed
   if (Simulate == true )
      return;

   // command execution
   PuntSubject->SetCompareRequest(&ObjectList);

   do
   {
      Sleep ( 1000 );
      
   } while ( PuntSubject->TestCompareRequest() == true );

   return;
}



/*---------------------------------------------------------------------------

	get token from a string

---------------------------------------------------------------------------*/

bool ScriptTask::GetToken( char *Token, int *TokenLine, int *TokenColumn, char **TokenScanner, bool WantCommand )
{
   // bypass all chars spaces and less than space
   while ( *Scanner <= ' ' && Scanner != End )
   {
      Column++;                                             // increment actual column
      if ( *Scanner == '\n' )                               // detect new line
      {
         if ( WantCommand == false)                         // accept new line only when want command
         {
            *Token = 0;                                     // empty token
            return true;
         }
         Column = 1;                                        // next chartacter is on firt column
         Line++;                                            // next line
      }
      Scanner++;                                            // advance scan position
   }

   // when scan reach end tokening is finished
   if ( Scanner == End )
      return false;

   // return token begining
   if ( TokenLine != 0 )
      *TokenLine = Line;
   if ( TokenColumn != 0 )
      *TokenColumn = Column;
   if ( TokenScanner != 0 )
      *TokenScanner = Scanner;

   // copy string in the token
   while ( *Scanner > ' ' && Scanner != End )
   {
      *Token++ = *Scanner++;
      Column++;
   }

   *Token = 0;                                              // token terminator
   return true;
}



/*---------------------------------------------------------------------------

	check string to verify number

---------------------------------------------------------------------------*/

bool ScriptTask::IsNumber( char *Str )
{
   while ( *Str != 0 )                                      // until character are present
   {
      if ( *Str < '0' || *Str > '9' )                       // check if current character is a number
         return false;
      Str++;                                                // goto next character
   }

   return true;                                             // all number
}



/*---------------------------------------------------------------------------

	check string to verify float

---------------------------------------------------------------------------*/

bool ScriptTask::IsFloat( char *Str )
{
   bool                                DecimalPoint;        // found decimal point

   DecimalPoint = false;
   while ( *Str != 0 )                                      // until character are present
   {
      if ( *Str < '0' || *Str > '9' )                       // check if current character is a number
      {
         if ( *Str == '.' && DecimalPoint == false)
            DecimalPoint = true;
         else
            return false;
      }
      Str++;                                                // goto next character
   }

   return true;                                             // all number
}



/*---------------------------------------------------------------------------

	check string to verify coordinate

---------------------------------------------------------------------------*/

bool ScriptTask::IsCoo( char *Str, int *RA_hh, int *RA_mm, int *RA_ss, char *Sign, int *Dec_gg, int *Dec_mm, int *Dec_ss )
{
   char                                Str2[5];             // string of single coordinate element
   int                                 Index;               // copying index
   int                                 Value;               // value

   Index=0;
   while ( *Str >= '0' && *Str <= '9' && Index < 3)         // copy until number
      Str2[Index++] = *Str++;
   Str2[Index] = 0;
   Value = atoi( Str2 );                                    // convert in number
   if ( Value < 0 || Value > 359 || Index == 0 )            // test for degrees
      return false;
   *RA_hh = Value;

   if ( *Str++ != '.' )                                     // test for separator
      return false;

   Index=0;
   while ( *Str >= '0' && *Str <= '9' && Index < 3)         // copy until number
      Str2[Index++] = *Str++;
   Str2[Index] = 0;
   Value = atoi( Str2 );                                    // convert in number
   if ( Value < 0 || Value > 59 || Index == 0 )             // test for minutes
      return false;
   *RA_mm = Value;

   if ( *Str++ != '.' )                                     // test for separator
      return false;

   Index=0;
   while ( *Str >= '0' && *Str <= '9' && Index < 3)         // copy until number
      Str2[Index++] = *Str++;
   Str2[Index] = 0;
   Value = atoi( Str2 );                                    // convert in number
   if ( Value < 0 || Value > 59 || Index == 0 )             // test for seconds
      return false;
   *RA_ss = Value;

   if ( *Str != '+' && *Str != '-' )                        // test for separator
      return false;
   *Sign = *Str;

   Str++;
   Index=0;
   while ( *Str >= '0' && *Str <= '9' && Index < 3)         // copy until number
      Str2[Index++] = *Str++;
   Str2[Index] = 0;
   Value = atoi( Str2 );                                    // convert in number
   if ( Value < 0 || Value > 89 || Index == 0 )             // test for degree
      return false;
   *Dec_gg = Value;

   if ( *Str++ != '.' )                                     // test for separator
      return false;

   Index=0;
   while ( *Str >= '0' && *Str <= '9' && Index < 3)         // copy until number
      Str2[Index++] = *Str++;
   Str2[Index] = 0;
   Value = atoi( Str2 );                                    // convert in number
   if ( Value < 0 || Value > 59 || Index == 0 )             // test for minutes
      return false;
   *Dec_mm = Value;

   if ( *Str++ != '.' )                                     // test for separator
      return false;

   Index=0;
   while ( *Str >= '0' && *Str <= '9' && Index < 3)         // copy until number
      Str2[Index++] = *Str++;
   Str2[Index] = 0;
   Value = atoi( Str2 );                                    // convert in number
   if ( Value < 0 || Value > 59 || Index == 0 )             // test for seconds
      return false;
   *Dec_ss = Value;

   if ( *Str++ != 0 )                                       // test for terminator
      return false;

   return true;                                             // correct equatorial coordinate
}



/*---------------------------------------------------------------------------

	check string to verify time

---------------------------------------------------------------------------*/

bool ScriptTask::IsTime( char *Str, int *Hour, int *Minutes )
{
   char                                Str2[5];             // string of single coordinate element
   int                                 Index;               // copying index
   int                                 Value;               // value

   Index=0;
   while ( *Str >= '0' && *Str <= '9' && Index < 3)         // copy until number
      Str2[Index++] = *Str++;
   Str2[Index] = 0;
   Value = atoi( Str2 );                                    // convert in number
   if ( Value < 0 || Value > 23 || Index == 0 )             // test for hours
      return false;
   *Hour = Value;

   if ( *Str++ != '.' )                                     // test for separator
      return false;

   Index=0;
   while ( *Str >= '0' && *Str <= '9' && Index < 3)         // copy until number
      Str2[Index++] = *Str++;
   Str2[Index] = 0;
   Value = atoi( Str2 );                                    // convert in number
   if ( Value < 0 || Value > 59 || Index == 0 )             // test for minutes
      return false;
   *Minutes = Value;

   if ( *Str++ != 0 )                                       // test for terminator
      return false;

   return true;                                             // correct equatorial coordinate
}



/*---------------------------------------------------------------------------

	push program counter in the stack

---------------------------------------------------------------------------*/

bool ScriptTask::PushPC( char *ParScanner, int ParLine, int ParColumn )
{
   // check for stack full
   if ( StackPointer >= MAX_STACK )
      return false;

   // put elements in the stack
   Stack[StackPointer].Scanner = ParScanner;
   Stack[StackPointer].Line = ParLine;
   Stack[StackPointer].Column = ParColumn;
   StackPointer++;                                          // increment stack pointer
   return true;
}



/*---------------------------------------------------------------------------

	pull program counter from the stack

---------------------------------------------------------------------------*/

bool ScriptTask::PullPC( void )
{
   // check for stack emplty
   if ( StackPointer == 0 )
      return false;

   StackPointer--;                                          // decrement stack pointer
   // get elements from the stack
   Scanner = Stack[StackPointer].Scanner;
   Line = Stack[StackPointer].Line;
   Column = Stack[StackPointer].Column;
   return true;
}



/*---------------------------------------------------------------------------

	add variable to the table

---------------------------------------------------------------------------*/

int ScriptTask::AddVariable(char *Name )
{
   int                                 Index;               // scan table index

   // verify if variabile is already present
   Index = FindVariabile( Name );
   if ( Index != -1 )
      return -1;

   for ( Index=0; Index<MAX_VARIABLE; Index++ )             // scan all elements in the table
   {
      if ( Variables[Index].Valid == false )                // check for position free
      {
         strcpy( Variables[Index].Name, Name );             // copy the name
         Variables[Index].Valid = true;                     // mark position as used
         return Index;                                      // return the position
      }
   }
   return -2;                                               // variable space full
}



/*---------------------------------------------------------------------------

	 find variabile in the table

---------------------------------------------------------------------------*/

int ScriptTask::FindVariabile( char *Name )
{
   int                                 Index;               // scan table index

   for ( Index=0; Index<MAX_VARIABLE; Index++ )             // scan all elements in the table
   {
      if ( Variables[Index].Valid == true )                 // check only position used
      {
         if ( strcmp( Variables[Index].Name, Name ) == 0 )  // compare name
            return Index;                                   // return the position
      }
   }
   return -1;                                               // variable not found
}



/*---------------------------------------------------------------------------

	remove variabile from the table

---------------------------------------------------------------------------*/

void ScriptTask::RemoveVariable( int VariableIndex )
{
   // index should be in the range
   if ( VariableIndex < 0 || VariableIndex >= MAX_VARIABLE )
      return;

   Variables[VariableIndex].Valid = false;                 // mark position as free
}



/*---------------------------------------------------------------------------

	trasform string in lower case

---------------------------------------------------------------------------*/

void ScriptTask::LowerCase( char *Str )
{
   while ( *Str != 0 )                                     // until character are present
      *Str++ = (char)tolower(*Str);                        // transform in lower case
}

