/******************************************************************************

	ScriptTask.h | Header of script exec for observatory management

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : October 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// antirepeat
#ifndef ScriptTaskH
#define ScriptTaskH



// project include
#include "Observer.h"
#include "TentaclesMgr.h"



// definitions
#define                                MAX_STACK      100
#define                                MAX_VARIABLE   100             



/******************************************************************************

	Script exec class

******************************************************************************/

class ScriptTask
{
   public:

      // error definition
      typedef enum
      {
         ERR_NONE,                                          // no error
         ERR_ENDOFSCRIPT,                                   // regular end of script
         ERR_PREMATUREEND,                                  // premature end of file
         ERR_SINTAX,                                        // sintax error
         ERR_WRONGPARAMETER,                                // wrong parameter
         ERR_PARAMOUTOFRANGE,                               // parameter out of range
         ERR_NOPARAMETER,                                   // parameter missing
         ERR_STACKEMPTY,                                    // to position in stack where to return
         ERR_STACKFULL,                                     // stack full
         ERR_EXECUTION,                                     // error in execution
      } ERR;

      // stack information structure
      typedef struct
      {
         char                          *Scanner;            // pointer to next script character to execute
         int                           Line;                // line to execute
         int                           Column;              // column with token
      }STACK;

      // variale information structure
      typedef struct
      {
         char                          Name[20];            // variable name
         int                           Value;               // variabile value
         bool                          Valid;               // variable alive
      }VARIABLE;

      // object constructor
                                       ScriptTask( );
      // object destroyer
      virtual                          ~ScriptTask();
      // set subject
      void                             SetSubject( TentaclesMgr *ParPuntSubject, Observer *ParExecObserver, int ParSafeElevation );
      // set GRB trigger
      void                             SetGRBTrigger( int PGRBTriggerIndex );
      // set script to execute
      void                             SetScript( char *Script );
      // reset execution
      void                             Reset(void);
      // execute next step
      void                             Step( int &Line, int &Column, int &Error, bool Simulate );
      // end of step request
      void                             EndStep( void );

   protected:
      // execute goto
      void                             ExecGoto( int &Line, int &Column, int &Error, bool Simulate );
      // execute wait
      void                             ExecTracking( int &Line, int &Column, int &Error, bool Simulate );
      // execute dome
      void                             ExecDome( int &Line, int &Column, int &Error, bool Simulate );
      // execute sliding shutter
      void                             ExecSShutter( int &Line, int &Column, int &Error, bool Simulate );
      // execute balancing shutter
      void                             ExecBShutter( int &Line, int &Column, int &Error, bool Simulate );
      // execute decondence
      void                             ExecDecondence( int &Line, int &Column, int &Error, bool Simulate );
      // execute power
      void                             ExecPower( int &Line, int &Column, int &Error, bool Simulate );
      // execute shot
      void                             ExecShot( int &Line, int &Column, int &Error, bool Simulate );
      // execute flat
      void                             ExecFlat( int &Line, int &Column, int &Error, bool Simulate );
      // execute dark
      void                             ExecDark( int &Line, int &Column, int &Error, bool Simulate );
      // execute dark
      void                             ExecSkyCheck( int &Line, int &Column, int &Error, bool Simulate );
      // execute wait
      void                             ExecWait( int &Line, int &Column, int &Error, bool Simulate );
      // execute repeat
      void                             ExecRepeat( int &Line, int &Column, char *Scanner, int &Error, bool Simulate );
      // execute end repeat
      void                             ExecEndRepeat( int &Error );
      // execute velocity
      void                             ExecVelocity( int &Line, int &Column, int &Error, bool Simulate );
      // execute telescope alignment
      void                             ExecTeleAlignment( int &Line, int &Column, int &Error, bool Simulate );
      // execute SN search
      void                             ExecSNSearch( int &Line, int &Column, int &Error, bool Simulate );
      // get token from a string
      bool                             GetToken( char *Token, int *TokenLine, int *TokenColumn, char **TokenScanner, bool WantCommand );
      // check string to verify number
      bool                             IsNumber( char *Str );
      // check string to verify float
      bool                             IsFloat( char *Str );
      // check string to verify coordinate
      bool                             IsCoo( char *Str, int *RA_hh, int *RA_mm, int *RA_ss, char *Sign, int *Dec_gg, int *Dec_mm, int *Dec_ss );
      // check string to verify coordinate
      bool                             IsTime( char *Str, int *Hour, int *Minutes );
      // push program counter in the stack
      bool                             PushPC( char *ParScanner, int ParLine, int ParColumn );
      // pull program counter from the stack
      bool                             PullPC( void );
      // add variable to the table
      int                              AddVariable(char *Name );
      // find variabile in the table
      int                              FindVariabile( char *Name );
      // remove variabile from the table
      void                             RemoveVariable( int VariableIndex );
      // trasform string in lower case
      void                             LowerCase( char *Str );

      TentaclesMgr                     *PuntSubject;        // pointer to subject
      Observer                         *ExecObserver;       // Exec observer for tentacles command
      char                             *Script;             // script text
      char                             *Scanner;            // pointer to next script character to execute
      char                             *End;                // end of script
      int                              Line;                // line to execute
      int                              Column;              // column with token
      int                              StackPointer;        // first free element in stack pointer
      STACK                            Stack[MAX_STACK];    // stack pointer table
      VARIABLE                         Variables[MAX_VARIABLE];   // variables table
      int                              PrevCommandLine;     // previous command line
      char                             ObjectName[50];      // object name on the telescope
      bool                             EndStepRequest;      // end of step request
      int                              SafeElevation;       // safe elevation in degree for program working
      int                              GRBTriggerIndex;
};
#endif


