/******************************************************************************

	ShotSequence.cpp | Shot sequence management

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : June 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// System include
#include <stdio.h>
#pragma hdrstop



// project include
#include "ShotSequence.h"
#pragma package(smart_init)





/*---------------------------------------------------------------------------

   Object constructor

---------------------------------------------------------------------------*/

__fastcall ShotSequence::ShotSequence(TMemo *ParServiceMemo)
   : TThread( false )
{
   State = SEQ_STATE_IDLE;                                  // object starts in idle status
   PuntSubject = 0;                                         // no sublect
   Control = false;
   Cloudy = false;
   ServiceMemo = ParServiceMemo;
}



/****************************************************************************

   Object destroyer

*****************************************************************************/

_fastcall ShotSequence::~ShotSequence()
{
   if ( PuntSubject != 0 )                                  // if subject assigned
      PuntSubject->Detach(this);                            // detach this observer from it
}



/******************************************************************************

   set subject

******************************************************************************/

int ShotSequence::SetSubject(
   TentaclesMgr       *ParPuntSubject                       // pointer to subject
)
{
   PuntSubject = ParPuntSubject;                            // store pointer
   return PuntSubject->Attach( this );                      // attach this observer to subject
}



/*---------------------------------------------------------------------------

   Starts the sequence

---------------------------------------------------------------------------*/

void ShotSequence::StartSequence(HANDLE ParFormHandle, int ParExposure, int ParNumShot, char* ParObject,
                                 int ParVentilazInterval, int ParVentilazTime, int ParVentilazPause, bool ParPECheck)
{
   // store input parameters
   FormHandle = ParFormHandle;
   Exposure = ParExposure;
   NumShot = ParNumShot;
   strcpy( Object, ParObject );
   VentilazInterval = ParVentilazInterval;
   VentilazTime = ParVentilazTime;
   VentilazPause = ParVentilazPause;
   PECheck = ParPECheck;

   // reset progressive conter
   ProgShot = 0;

   // get starting time for ventilation
   StartInterval = GetTickCount();

   // activate the sequence
   State = SEQ_STATE_START;
}




/*---------------------------------------------------------------------------

   detect subject notify

---------------------------------------------------------------------------*/

void ShotSequence::Update( int ANotifyCode )
{
}


/*---------------------------------------------------------------------------

   sequence execution

---------------------------------------------------------------------------*/

void __fastcall ShotSequence::Execute(void)
{
   TentaclesMgr::STATUS                Status;              // subject status
   TentaclesMgr::ERR                   Error;
   CCD::SHOT_INFO                      ShotInfo;            // last shot information
   DWORD                               Now;                 // actual time
   double                              XHighest;            // highest value x position
   double                              YHighest;            // highest value y position
   char                                Message[50];         // service message

   // continue until thread is not external terminated
   while (  Terminated == false )
   {
      Sleep ( 500 );                                        // leave time to others activities

      // manage state machine
      switch ( State )
      {
         // sequence in idle (do nothing)
         case SEQ_STATE_IDLE:
            break;

         // sequence started
         case SEQ_STATE_START:
            Error = PuntSubject->GetStatus( &Status );      // get subject status
            if ( Error != TentaclesMgr::ERR_NONE )
            {
               MessageBox( FormHandle, "Impossibile leggere status", "Errore", MB_OK );
               State = SEQ_STATE_END;
               break;
            }
            // verify shot sequence starting conditions
            if ( Status.CCDEnabled  == false    )
            {
               MessageBox( FormHandle, "Non  possibile lanciare sequenza di foto, una delle sequenti condizioni non  verificata:\n   CCD abilitato", "Errore", MB_OK );
               State = SEQ_STATE_END;
               break;
            }

            // take control of  ccd
            Error = PuntSubject->TakeUp( this, "Sequenza di foto", false, true );
            if ( Error != TentaclesMgr::ERR_NONE )
            {
               MessageBox( FormHandle, "Non  possibile prendere controllo di CCD", "Errore", MB_OK );
               State = SEQ_STATE_END;
               break;
            }
            Control = true;
            Error = PuntSubject->SetObjectName( Object );   // set base name for shots
            if ( Error != TentaclesMgr::ERR_NONE )
            {
               MessageBox( FormHandle, "Non accettato nome foto", "Errore", MB_OK );
               State = SEQ_STATE_END;                       // goto end step
               break;
            }
            State = SEQ_STATE_CHECK_VENT_ON;                // goto next step
            break;

         // check if ventilation should be started
         case SEQ_STATE_CHECK_VENT_ON:
            if ( VentilazInterval == 0 )                    // parameter = zero mean no ventilation
            {
               State = SEQ_STATE_SHOT;
               break;
            }
            // verify if it is ventilation time
            Now = GetTickCount();
            if ( (int)(Now-StartInterval)/60000 < VentilazInterval )
            {
               State = SEQ_STATE_SHOT;
               break;
            }

            // it is ventilation time
            StartInterval = Now;                            // store for next time
            PuntSubject->RunCommand( this, TentaclesMgr::COM_SUBJ_DECOND, TentaclesMgr::COM_STA_ON); // start ventilation
            State = SEQ_STATE_CHECK_VENT_OFF;
            break;

         // check if ventilation should be stopped
         case SEQ_STATE_CHECK_VENT_OFF:
            // verify if it is time to stop
            Now = GetTickCount();
            if ( (int)(Now-StartInterval)/1000 < VentilazTime )
               break;

            // it is ventilation stop time
            PauseTime = Now;                             // store for pause time
            PuntSubject->RunCommand( this, TentaclesMgr::COM_SUBJ_DECOND, TentaclesMgr::COM_STA_OFF); // stop ventilation
            State = SEQ_STATE_CHECK_VENT_PAUSE;
            break;

         // check if it is time to restart shot
         case SEQ_STATE_CHECK_VENT_PAUSE:
            // verify if pause is expired
            Now = GetTickCount();
            if ( (int)(Now-PauseTime)/1000 < VentilazPause )
               break;

            // it is time to start another shot
            State = SEQ_STATE_SHOT;
            break;

         // shot image
         case SEQ_STATE_SHOT:
            Error = PuntSubject->GetStatus( &Status );      // get subject status
            if ( Error != TentaclesMgr::ERR_NONE )
            {
               MessageBox( FormHandle, "Impossibile leggere status", "Errore", MB_OK );
               State = SEQ_STATE_END;
               break;
            }
            Error = PuntSubject->Shot( this, Exposure);     // shot new image
            if ( Error != TentaclesMgr::ERR_NONE )
            {
               MessageBox( FormHandle, "Non  possibile scattare foto", "Errore", MB_OK );
               State = SEQ_STATE_END;                       // goto end step
               break;
            }
            ProgShot++;                                     // increase shot progressive number
            State = SEQ_STATE_WAIT;                         // goto next step
            break;

         // wait shot finisch
         case SEQ_STATE_WAIT:
            Error = PuntSubject->IsShotFinisched(&Exposing);// check shot is finished
            if ( Error == TentaclesMgr::ERR_NONE )          // until not finished stay in this state
            {
               // get shot
               Error = PuntSubject->GetShot(this);
               if ( Error != TentaclesMgr::ERR_NONE )
               {
                  MessageBox( FormHandle, "Impossibile scaricare foto", "Errore", MB_OK );
                  State = SEQ_STATE_END;                    // goto end step
                  break;
               }
               // get shot info in order to verify clouds
               Error = PuntSubject->GetShotInfo(&ShotInfo);// get shot information
               if ( Error == TentaclesMgr::ERR_CLOUDY )
               {
                  State = SEQ_STATE_END;                    // goto end
                  Cloudy = true;
                  break;
               }
               if ( PECheck == true )
               {
                  // get highest value position
                  Error = PuntSubject->GetHighestPosition( &XHighest, &YHighest);
                  if ( Error != TentaclesMgr::ERR_NONE )
                  {
                     MessageBox( FormHandle, "Impossibile leggere posizione pi luminosa", "Errore", MB_OK );
                     State = SEQ_STATE_END;                    // goto end
                     break;
                  }
                  sprintf( Message, "%04d   %f", Status.PECIndex, XHighest );  // create string
                  ServiceMemo->Lines->Add(Message);
               }
               Error = PuntSubject->RunCommand( this, TentaclesMgr::COM_SUBJ_NIGHTLYFOLDER, TentaclesMgr::COM_STA_SAVE);
               if ( Error != TentaclesMgr::ERR_NONE )
               {
                  MessageBox( FormHandle, "Impossibile salvare foto", "Errore", MB_OK );
                  State = SEQ_STATE_END;                    // goto end step
                  break;
               }
               if ( ProgShot >= NumShot )                   // verify if sequence is finished
                  State = SEQ_STATE_END;                    // goto end step
               else
                  State = SEQ_STATE_CHECK_VENT_ON;          // go again to shot
            }
            break;
      }


      // when sequence ended exit thread
      if ( State == SEQ_STATE_END )
         break;
   }


   // manage premature sequence termination
   if ( State == SEQ_STATE_WAIT )
   {
      Error = PuntSubject->AbortShot(this);
      if ( Error != TentaclesMgr::ERR_NONE )
         MessageBox( FormHandle, "Impossibile abortire foto", "Errore", MB_OK );
   }
   // release all
   if ( Control == true )
   {
      Error = PuntSubject->Release( this, false, true ); // release CCD
      if ( Error != TentaclesMgr::ERR_NONE )
         MessageBox( FormHandle, "Impossibile rilasciare controllo", "Errore", MB_OK );
      else
         Control = false;
   }

   State = SEQ_STATE_END;
}



/*---------------------------------------------------------------------------

   get sequence status

---------------------------------------------------------------------------*/

void ShotSequence::GetStatus( bool *End, int *ParExposing, int *ParProgShot, bool *ParCloudy )
{
   *End = State == SEQ_STATE_END;
   *ParExposing = Exposing;
   *ParProgShot = ProgShot;
   *ParCloudy = Cloudy;
}

