/******************************************************************************

	TentaclesMgr.cpp | Observatory tentacles manager

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : March 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// System include
#include <stdio.h>



// definition
#define NO_PROMPT_ON_HRCHECK_FAILURE



// project include
#include "TentaclesMgr.h"
#include "CMStar.h"
#include "CMMoon.h"
#include "CMPlanet.h"
#include "CMMPlanet.h"
#include "CMDeepSky.h"
#include "CMEvent.h"
#include "PinPoint_OCX.h"
#include "PinPoint_TLB.h"
#pragma package(smart_init)



/****************************************************************************

   Object constructor

*****************************************************************************/

TentaclesMgr::TentaclesMgr()
: Subject()
{
   // object data initialization
   hPowerLine = INVALID_HANDLE_VALUE;
   LX200Line = 0;
   PowerLine = 0;
   LX200BaudRate = 0;
   ObjCom = 0;
   ObjThreadCom = 0;
   ObjCcd = 0;
   ObjGSC = 0;
   ObjFitsImage = 0;
   RefStarsList = 0;
   PlanetList = 0;
   MPlanetsList = 0;
   MultipleStarList = 0;
   ClustersList = 0;
   NebuleList = 0;
   GalaxiesList = 0;
   NGCList = 0;
   FormHandle = 0;
   *CcdName = 0;
   *ShotName = 0;
   *NameOnObservatory = 0;
   *NameOnCCD = 0;
   *Telescope = 0;
   *Observers = 0;
   EnableSetTime = false;
   SetTimeStamp = 0;
   ObserverOnObservatory = 0;
   ObserverOnCCD = 0;
   ChartObjects = 0;
   AllignmentRequest = 0;
   CompareRequest = 0;
   InitTargetPos = 0;
   memset( ShotList, 0, sizeof(ShotList));
   memset( DarkList, 0, sizeof(DarkList));
   memset( &InternalStatus, 0, sizeof( InternalStatus ) );
   InternalStatus.ShotRA.hh = 25;                           // invalid shot coordinate (means not valid)
   Now = new CMEvent();                                     // create event object

   // create comparation objects
   RACompObj1 = new CMHours();
   DecCompObj1 = new CMDegree();
   RACompObj2 = new CMHours();
   DecCompObj2 = new CMDegree();

   // create plate compare object
   CompareObj = new ImageCompare("");
}



/****************************************************************************

   Object destroyer

*****************************************************************************/

TentaclesMgr::~TentaclesMgr()
{
   Deactivate();
   if ( ChartObjects != 0 )
      delete ChartObjects;

   delete Now;

   // delete comparation objects
   delete RACompObj1;
   delete DecCompObj1;
   delete RACompObj2;
   delete DecCompObj2;

   // delete plate compare object
   delete CompareObj;
}



/****************************************************************************

   Set parallel port

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::SetPower( int PLine )
{
   // store input parameters
   PowerLine = PLine;

   return ERR_NONE;
}



/****************************************************************************

   Set LX200 communication parameters

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::SetLX200( int PLX200Line, int PLX200BaudRate )
{
   // store input parameters
   LX200Line = PLX200Line;
   LX200BaudRate = PLX200BaudRate;
   return ERR_NONE;
}



/****************************************************************************

   Set ccd parameters

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::SetCCD ( HANDLE PFormHandle, char *PCcdName, double PPixelScale, char *ParImageFolder )
{
   // store input parameters
   FormHandle = PFormHandle;
   strcpy( CcdName, PCcdName);
   PixelScale = PPixelScale;
   strcpy(ImageFolder, ParImageFolder);
   return ERR_NONE;
}



/****************************************************************************

   Set plate solver parameters

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::SetPlateSolver ( char *PGSCFolders, int PMaxMagResolution )
{
   strcpy( GSCResolutionFolder, PGSCFolders );
   MaxMagResolution = PMaxMagResolution;
   return ERR_NONE;
}



/****************************************************************************

   set short folders

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::SetShotFolders( char *PShotFolder, char *PDarkFolder )
{
   // store input parameters
   strcpy( ShotFolder, PShotFolder );
   strcpy( DarkFolder, PDarkFolder );
   return ERR_NONE;
}



/****************************************************************************

   Set GSC folder

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::SetGSCFolder ( char *PGSCFolder )
{
   // store input parameter
   strcpy( GSCFolder, PGSCFolder);
   return ERR_NONE;
}



/****************************************************************************

   Set objects folder

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::SetObjectsFolder ( char *PObjectsFolder )
{
   // store input parameter
   strcpy( ObjectsFolder, PObjectsFolder);
   return ERR_NONE;
}



/****************************************************************************

   Set SN reference parameters

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::SetSN ( char *PSNFolder, double PPixelScale )
{
/*   CompareObj->SetFolder( GSCResolutionFolder, PSNFolder, ShotFolder);
   PixelScale = PPixelScale;
   CompareObj->SetOption( PixelScale); */
   return ERR_NONE;
}



/****************************************************************************

   Set observatory Parameters

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::SetObservatoryPar ( char *PTelescope, char *PObservers,
                                                    CMDegree::DEGREE ParObsLatitude, CMHours::HOURS ParObsLongitude )
{
   // store input parameter
   strcpy( Telescope, PTelescope );
   strcpy( Observers, PObservers );
   ObsLatitude = ParObsLatitude;
   ObsLongitude = ParObsLongitude;
   return ERR_NONE;
}



/****************************************************************************

   Activate object

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::Activate( void )
{
	char			                        NomeFile[10];		   // file name
   char			                        c[10];					// temp
  	DCB 			                        dcb;						// communication parameter

   // notify program is on
   Notify( NULL, NOTIFY_PROGON );

   // serial power communication parameter have to be set
   if ( PowerLine == 0 )
      return ERR_LPOWER_PAR;
      
   // LX200 communication paramemeters have to be set
   if ( LX200Line == 0 || LX200BaudRate == 0 )
      return ERR_LX200_PAR;

   // communication object creation
   if ( ObjCom == 0 )
      ObjCom = new LX200();

   if ( ObjThreadCom == 0 )
      ObjThreadCom = new LX200Thread(ObjCom, LX200_UPDATETIME);

   // CCD object creation
   if ( ObjCcd == 0 )
      ObjCcd = new CCD();

   // GSC object creation
   if ( ObjGSC == 0 )
      ObjGSC = new GSC( GSCFolder);

   // create object managing fits file image
   if ( ObjFitsImage == 0 )
      ObjFitsImage = new Fits();

   // create sky objects lists
   CreateSkyObjectsLists();

   // open serial line for power on
   strcpy( NomeFile, "COM" );
   itoa ( PowerLine, c, 10);
   strcat( NomeFile, c );
	hPowerLine = CreateFile( NomeFile, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL );
   if ( hPowerLine == INVALID_HANDLE_VALUE )
      return ERR_LPOWER_OPEN;

   // get current communication parameters
	GetCommState(hPowerLine, &dcb);

   // set my parameters
	dcb.BaudRate = 9600;                                     // doesn't matter
	dcb.ByteSize = 8;                                        // doesn't matter
	dcb.Parity = NOPARITY;                                   // doesn't matter
	dcb.StopBits = TWOSTOPBITS	;                             // doesn't matter
   dcb.fDtrControl = DTR_CONTROL_DISABLE;                   // user to turn off
   dcb.fRtsControl = RTS_CONTROL_DISABLE;                   // doesn't matter
	SetCommState(hPowerLine, &dcb);


   return ERR_NONE;
}



/****************************************************************************

   Deactivate object

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::Deactivate( void )
{
   // notify program is on
   Notify( NULL, NOTIFY_PROGOFF );

   // close and destroy communication thread object if it was created
   if ( ObjThreadCom != 0 )
   {
      ObjThreadCom->Terminate();                            // terminate thread
      while ( ObjThreadCom->Active == true )                // wait until thread is really terminated
         Sleep( 100 );                                      // leave CPU free
      delete ObjThreadCom;                                  // destroy object
      ObjThreadCom = 0;                                     // reset pointer
   }
   // close and destroy communication object if it was created
   if ( ObjCom != 0 )
   {
      if ( InternalStatus.CommOpened == true )
         ObjCom->Close();                                   // line close
      delete ObjCom;                                        // destroy object
      ObjCom = 0;                                           // reset pointer
   }

   // close and destroy CCD object if it was created
   if ( ObjCcd != 0 )
   {
      if ( InternalStatus.CommOpened == true )
         ObjCcd->Close();                                   // object close
      delete ObjCcd;                                        // destroy object
      ObjCcd = 0;                                           // reset pointer
   }

   // close and destroy GSC object
   if ( ObjGSC != 0 )
   {
      delete ObjGSC;
      ObjGSC = 0;
   }

   // close and destroy object managing fits file image
   if ( ObjFitsImage != 0 )
   {
      delete ObjFitsImage;
      ObjFitsImage = 0;
   }

   // destroy sky objects lists
   if ( RefStarsList != 0 )
   {
      delete RefStarsList;
      RefStarsList = 0;
   }
   if ( PlanetList != 0 )
   {
      delete PlanetList;
      PlanetList = 0;
   }
   if ( MPlanetsList != 0 )
   {
      delete MPlanetsList;
      MPlanetsList = 0;
   }
   if ( MultipleStarList != 0 )
   {
      delete MultipleStarList;
      MultipleStarList = 0;
   }
   if ( ClustersList != 0 )
   {
      delete ClustersList;
      ClustersList = 0;
   }
   if ( NebuleList != 0 )
   {
      delete NebuleList;
      NebuleList = 0;
   }
   if ( GalaxiesList != 0 )
   {
      delete GalaxiesList;
      GalaxiesList = 0;
   }
   if ( NGCList != 0 )
   {
      delete NGCList;
      NGCList = 0;
   }

   // close serial line
   if ( hPowerLine != INVALID_HANDLE_VALUE )
   {
      CloseHandle( hPowerLine );
      hPowerLine = INVALID_HANDLE_VALUE;
   }

   return ERR_NONE;
}



/****************************************************************************

   Observer asks to take up observatory and/or ccd

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::TakeUp( Observer *Sender, char *Name, bool Observatory, bool CCD )
{
   // check in order to verify Observatory is busy and asked
   if ( Observatory == true && ObserverOnObservatory != 0 )
      return ERR_OBSERVATORYBUSY;

   // check in order to verify CCD is busy and asked
   if ( CCD == true && ObserverOnCCD != 0 )
      return ERR_CCDBUSY;

   // if observatory is asked it is taken up and notification is sent
   if ( Observatory == true )
   {
      ObserverOnObservatory = Sender;
      strcpy( NameOnObservatory, Name );
      Notify( Sender, NOTIFY_OBSERVATORY_TAKEUP );
   }
   // if CCD is asked it is taken up and notification is sent
   if ( CCD == true )
   {
      ObserverOnCCD = Sender;
      strcpy( NameOnCCD, Name );
      Notify( Sender, NOTIFY_CCD_TAKEUP );
   }

   return ERR_NONE;
}



/****************************************************************************

   Observer releases observatory and/or ccd

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::Release( Observer *Sender, bool Observatory, bool CCD )
{
   // check in order to verify Observatory is releasing from the correct observer taked it up
   if ( Observatory == true && ObserverOnObservatory != Sender )
      return ERR_COMMAND;

   // check in order to verify CCD is releasing from the correct observer taked it up
   if ( CCD == true && ObserverOnCCD != Sender )
      return ERR_COMMAND;

   // if observatory is releasing it is released and notification is sent
   if ( Observatory == true )
   {
      Notify( Sender, NOTIFY_OBSERVATORY_RELEASE );
      ObserverOnObservatory = 0;
      strcpy( NameOnObservatory, "" );
   }
   // if CCD is releasing it is released and notification is sent
   if ( CCD == true )
   {
      Notify( Sender, NOTIFY_CCD_RELEASE );
      ObserverOnCCD = 0;
      strcpy( NameOnCCD, "" );
   }

   return ERR_NONE;
}



/****************************************************************************

   Get name of observer had taken exclusive use of observatory

*****************************************************************************/

const char* TentaclesMgr::GetNameOnObservatory( void )
{
   return NameOnObservatory;
}



/****************************************************************************

   Get name of observer had taken exclusive use of CCD

*****************************************************************************/

const char* TentaclesMgr::GetNameOnCCD( void )
{
   return NameOnCCD;
}



/****************************************************************************

   Get status of all controlled devices

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::GetStatus( STATUS *Status)
{
   LX200Thread::STATUS                 ThreadStatus;
   DWORD                               TimeStamp;
   SYSTEMTIME                          CurrentGMT;          // current GMT time
   CMEvent::DATE                       NowDate;             // current data and time event data
   double                              NowJD;               // current julian day
   CMHours::HOURS                      SidTime;             // current sideral time
   double                              HourAngle;           // telescope hour angle
   double                              Declination;         // telescope declination in degree
   double                              Temp1;               // termporary double for computing
   double                              Temp2;               // termporary double for computing
   double                              Azimut;              // telescope azimut
   double                              Elevation;           // telescope elevation
   double                              Latitude;            // observatory latitude in degrees
   int                                 Temp;                // temporary variabile
   bool                                ThreadStatusOk;      // Thread status received

   // get information from LX200 thread communication
   InternalStatus.CommOk = ObjThreadCom->GetCommStatus();   // get communication status
   ThreadStatusOk = ObjThreadCom->GetStatus( &ThreadStatus );  // get information from telescope/dome/others

   // copy information on whole information structure
   if ( InternalStatus.CommOpened == true && ThreadStatusOk == true)  // only when communication is opened and status was received
   {
      InternalStatus.TeleRA                  = ThreadStatus.TeleRA;
      InternalStatus.TeleDec                 = ThreadStatus.TeleDec;
      InternalStatus.DataRetained            = ThreadStatus.DataRetained;
      InternalStatus.TeleAlligned            = ThreadStatus.TeleAlligned;
      InternalStatus.MotorRAFault            = ThreadStatus.MotorRAFault;
      InternalStatus.MotorDecFault           = ThreadStatus.MotorDecFault;
      InternalStatus.TeleMoving              = ThreadStatus.TeleMoving;
      InternalStatus.DomeAzimut              = ThreadStatus.DomeAzimut;
      InternalStatus.DomeSlidingShutter      = ThreadStatus.DomeSlidingShutter;
      InternalStatus.DomeBalancingShutter    = ThreadStatus.DomeBalancingShutter;
      InternalStatus.DomeAlligned            = ThreadStatus.DomeAlligned;
      InternalStatus.DomeMovement            = ThreadStatus.DomeMovement;
      InternalStatus.DomeHome                = ThreadStatus.DomeHome;
      InternalStatus.DomeZero                = ThreadStatus.DomeZero;
      InternalStatus.TeleAlarm               = ThreadStatus.TeleAlarm;
      InternalStatus.DomeTemp                = ThreadStatus.DomeTemp;
      InternalStatus.PECValue                = ThreadStatus.PECValue;
      InternalStatus.PECIndex                = ThreadStatus.PECIndex;
      if ( InitTargetPos == false )
      {
         InitTargetPos = true;
         InternalStatus.TargetRA = InternalStatus.TeleRA;
         InternalStatus.TargetDec = InternalStatus.TeleDec;
      }
   }

   // get current data and time
   GetSystemTime( &CurrentGMT );                            // get current GMT time

   // compute current local sideral time
   NowDate.Year = CurrentGMT.wYear;                         // fill event structure
   NowDate.Month = CurrentGMT.wMonth;
   NowDate.Day = CurrentGMT.wDay;
   NowDate.Hour = CurrentGMT.wHour;
   NowDate.Minute = CurrentGMT.wMinute;
   NowDate.Sec = CurrentGMT.wSecond;
   Now->SetDate(&NowDate);                                  // set event object
   NowJD = Now->GetGG();                                    // get current julian day
   SidTime = CM_SideralTimeComputing( NowJD, false );       // compute greenwich sideral time
   CM_AddTime( &ObsLongitude, &SidTime );                   // compute local sideral time

   // compute efemerid epoque
   YearEquinox = CurrentGMT.wYear;                          // current year
   YearEquinox += (double)(CurrentGMT.wMonth-1)/12.0;       // add mounth from January
   Temp = YearEquinox * 100.0;                              // exec round
   YearEquinox = (double)Temp / 100;

   // compute telescope azimut
   HourAngle = CM_Convert( &SidTime ) - CM_Convert( &InternalStatus.TeleRA );   // telescope hour angle
   Declination = CM_Convert( &InternalStatus.TeleDec );     // telescope declination in degree
   Latitude = CM_Convert ( &ObsLatitude );                  // observatory latitude
   HourAngle /= 180/M_PI;                                   // telescope hour angle in radiant
   Declination /= 180/M_PI;                                 // telescope declination in radiant
   Latitude /= 180/M_PI;                                    // observatory latitude in radiant
   Temp1 = sin(HourAngle);                                  // first parameter for trasnformation
   Temp2 = cos(HourAngle)*sin(Latitude) - tan(Declination)*cos(Latitude);  // second parameter for trasnformation
   if ( Temp2 == 0 )                                        // to evalid zero divide exception
   {
      if ( Temp1 > 0 )                                      // assign azimut depending on sign of first parameter
         Azimut = M_PI_2;
      else
         Azimut = - M_PI_2;
   }
   else
      Azimut = atan( Temp1/Temp2 );                         // compute azimut
   Azimut *= 180/M_PI;
   if ( Temp2 < 0 )                                         // arrange quadrant result
      Azimut += 180;
   if ( Azimut < 0 )                                        // arrange quadrant result
      Azimut += 360;
   InternalStatus.TeleAzimut.gg = Azimut;                   // get azimut in degree
   Temp1 = Azimut - InternalStatus.TeleAzimut.gg;           // compute decimal part
   Temp1 *= 60;                                             // compute minute
   InternalStatus.TeleAzimut.mm = Temp1;                    // compute integer minutes
   Temp1 = Azimut - InternalStatus.TeleAzimut.mm;           // compute decimal part
   InternalStatus.TeleAzimut.ss = Temp1 * 60;               // compute second

   // compute and update telescope elevation
   Temp1 = sin(Latitude)*sin(Declination) + cos(Latitude)*cos(Declination)*cos(HourAngle);   // compute elevetion asin in radiant
   Elevation = asin(Temp1) * 180/M_PI;                      // compute elevation
   strcpy( InternalStatus.TeleElevation.sign, "+" );        // default positive elevation
   if ( Elevation < 0 )                                     // check for negative elevation
   {
      Elevation = - Elevation;                              // change sign
      strcpy(InternalStatus.TeleElevation.sign, "-");       // minus
   }
   InternalStatus.TeleElevation.gg = Elevation;             // get elevation in degree
   Elevation -= InternalStatus.TeleElevation.gg;            // compute decimal part
   Elevation *= 60;                                         // compute minutes
   InternalStatus.TeleElevation.mm = Elevation;             // comoute integer minutes
   Elevation -= InternalStatus.TeleElevation.mm;            // compute decimal part
   InternalStatus.TeleElevation.ss = Elevation * 60;        // compute seconds

   // if enabled wait time ...
   if ( EnableSetTime == true )
   {
      TimeStamp = GetTickCount();                           // get actual time
      if ( (TimeStamp-SetTimeStamp) > LX200_READWAIT )      // is wait time expired
         EnableSetTime = false;                             // remove wait time
   }

   // during wait time information modified by command are not updated
   if ( EnableSetTime == false )
   {
      InternalStatus.DomeLock                = ThreadStatus.DomeLock;
      InternalStatus.TeleTracking            = ThreadStatus.TeleTracking;
      InternalStatus.FlatLight               = ThreadStatus.FlatLight;
      InternalStatus.DeCondence              = ThreadStatus.DeCondenses;
   }

   // update information about safe algorithm
   InternalStatus.SafeEnable = SafeEnable;
   InternalStatus.SafeThreshold = SafeThreshold;
   InternalStatus.SafeTime = SafeTime;

   // copy internal information
   *Status = InternalStatus;

   if ( AllignmentRequest == 1 )
   {
      AllignmentRequest = 2;
      ShotAllignment( ObserverAllignment );
      AllignmentRequest = 0;
   }

   if ( CompareRequest == 1 )
   {
      CompareRequest = 2;
      Compare( CompareObjectList );
      CompareRequest = 0;
   }

   return ERR_NONE;
}



/****************************************************************************

   Run a command

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::RunCommand( Observer *Sender, COM_SUBJECT Subj, COM_STA State, int Par1, int Par2 )
{
   int                                 Ret;                 // communication retuned value
   ERR                                 Error;               // error code for return
   char                                Buffer[10];
   SYSTEMTIME                          Time;                // current time for clock sincronization
   DWORD					                  LenW;						// written bytes
  	DCB 			                        dcb;						// communication parameter

   // check if this observer can run the command
   if ( ObserverOnObservatory != 0  && ObserverOnObservatory != Sender)
      return ERR_OBSERVATORYBUSY;

   Error = ERR_COMMAND;

   // save last command
   InternalStatus.CommandSubject = Subj;
   InternalStatus.CommandState = State;

   // power command
   if ( Subj == COM_SUBJ_POWER )
   {
      if ( State == COM_STA_ON )
      {
         Buffer[0] = 0;
         WriteFile ( hPowerLine, Buffer, 1, &LenW, NULL );  // send a zero character to turn power on

         InternalStatus.PowerEnabled = true;                // store the on condition
         Notify( Sender, NOTIFY_GENERIC_COMMAND );          // notify to all observer
      }
      if ( State == COM_STA_OFF )
      {
         GetCommState(hPowerLine, &dcb);                    // get communication parameters
         dcb.fDtrControl = DTR_CONTROL_ENABLE;              // set DTR to +12V
	      SetCommState(hPowerLine, &dcb);                    // apply the change on line
         Sleep( 100 );                                      // wait for power off
         dcb.fDtrControl = DTR_CONTROL_DISABLE;             // set DTR to - 12V
	      SetCommState(hPowerLine, &dcb);                    // applay the change on line

         InternalStatus.PowerEnabled = false;               // store the off condition
         Notify( Sender, NOTIFY_GENERIC_COMMAND );          // notify to all observer
      }
      return ERR_NONE;
   }

   // LX200 command
   if ( Subj == COM_SUBJ_LX200 )
   {
      Error = ERR_NONE;
      if ( State == COM_STA_ON )
      {
         Ret = ObjCom->Open(LX200Line, LX200BaudRate);      // comunication open
         if ( Ret == 0 )
            InternalStatus.CommOpened = true;
         else
            Error = ERR_LX200_OPEN;
      }
      else
      {
         Ret = ObjCom->Close();                             // communication close
         if ( Ret == 0 )
            InternalStatus.CommOpened = false;
         else
            Error = ERR_LX200_OPEN;
      }
      Notify( Sender, NOTIFY_GENERIC_COMMAND );
      return Error;
   }

   // ccd command
   if ( Subj == COM_SUBJ_CCD )
   {
      if ( State == COM_STA_ON )
      {
         Ret = ObjCcd->Open( FormHandle, CcdName, ImageFolder );         // ccd open
         if ( Ret == 0 )
            InternalStatus.CCDEnabled = true;
      }
      if ( State == COM_STA_OFF )
      {
         Ret = ObjCcd->Close(  );                           // ccd close
         if ( Ret == 0 )
            InternalStatus.CCDEnabled = false;
      }
      Notify( Sender, NOTIFY_GENERIC_COMMAND );
      return (TentaclesMgr::ERR)Ret;
   }

   // manage commands on LX200, the line have to be open
   if ( InternalStatus.CommOpened == true )
   {
      Ret=0;
      switch ( Subj )                                       // depending on target switch
      {
         // flat light
         case COM_SUBJ_LIGHT:
            EnableSetTime = true;
            SetTimeStamp = GetTickCount();                  // get actuall timestamp to stop termorary readback until thread refresch its data
            if ( State == COM_STA_ON )
            {
               Ret = ObjCom->SetFlatLight( true );          // turn on flat light
               InternalStatus.FlatLight = true;
            }
            if ( State == COM_STA_OFF )
            {
               Ret = ObjCom->SetFlatLight( false );         // tirn off flat light
               InternalStatus.FlatLight = false;
            }
            break;

         // de condence
         case COM_SUBJ_DECOND:
            EnableSetTime = true;
            SetTimeStamp = GetTickCount();                  // get actuall timestamp to stop termorary readback until thread refresch its data
            if ( State == COM_STA_ON )
            {
               Ret = ObjCom->SetDecondence( true );         // turn on de condence device
               InternalStatus.DeCondence = true;
            }
            if ( State == COM_STA_OFF )
            {
               Ret = ObjCom->SetDecondence( false );        // turn off de condence device
               InternalStatus.DeCondence = false;
            }
            break;

         // telescope tracking
         case COM_SUBJ_TRACK:
            EnableSetTime = true;
            SetTimeStamp = GetTickCount();                  // get actuall timestamp to stop termorary readback until thread refresch its data
            if ( State == COM_STA_ON )
            {
               Ret = ObjCom->SetTelescopeTracking( true );  // turn on telescope tracking motor
               InternalStatus.TeleTracking = true;
            }
            if ( State == COM_STA_OFF )
            {
               Ret = ObjCom->SetTelescopeTracking( false ); // turn off telescope tracking motor
               InternalStatus.TeleTracking = false;
            }
            break;

         // controlloer clock sincronization
         case COM_SUBJ_CLOCKSYNC:
            GetSystemTime( &Time );                         // recupero data ed ora attuale
            Ret = ObjCom->SetClock( &Time );                // invio al telescopio
            break;

         // dome
         case COM_SUBJ_DOME:
            EnableSetTime = true;
            SetTimeStamp = GetTickCount();                  // get actuall timestamp to stop termorary readback until thread refresch its data
            if ( State == COM_STA_LOCK )
            {
               Ret = ObjCom->SetAutomaticDome( true );      // lock dome to telecope azimut
               InternalStatus.DomeLock = true;
            }
            if ( State == COM_STA_UNLOCK )
            {
               Ret = ObjCom->SetAutomaticDome( false );     // unlock dome position
               InternalStatus.DomeLock = false;
            }
            if ( State == COM_STA_HOME )
               Ret = ObjCom->SetDomeHome();                 // dome in home position
            if ( State == COM_STA_ZERO )
               Ret = ObjCom->SetDomeZero();                 // dome at power station
            if ( State == COM_STA_STOP )
               Ret = ObjCom->SetDomeStop();                 // stop dome movement
            if ( State == COM_STA_LEFT )
               Ret = ObjCom->SetDomeCCW();                  // dome CCW movement
            if ( State == COM_STA_RIGHT )
               Ret = ObjCom->SetDomeCW();                   // dome CW movement
            break;

         // telescope RA motor
         case COM_SUBJ_RA:
            if ( State == COM_STA_ZERO )
               Ret = ObjCom->ResetMotorFault(0);            // Right ascension motor fault reset
            break;

         // telescope declination motor
         case COM_SUBJ_DEC:
            if ( State == COM_STA_ZERO )
               Ret = ObjCom->ResetMotorFault(1);;           // declination motor fault reset
            break;

         // dome sliding shutter
         case COM_SUBJ_SSHUTTER:
            if ( State == COM_STA_OPEN )
               Ret = ObjCom->SetSShutter(true);             // open sliding shutter
            if ( State == COM_STA_CLOSE )
               Ret = ObjCom->SetSShutter(false);            // close sliding shutter
            if ( State == COM_STA_STOP )
               Ret = ObjCom->SetDomeStop();                 // stop sliding shutter movement
            break;

         // dome balancing shutter
         case COM_SUBJ_BSHUTTER:
            if ( State == COM_STA_OPEN )
               Ret = ObjCom->SetBShutter(true);             // open balancing shutter
            if ( State == COM_STA_CLOSE )
               Ret = ObjCom->SetBShutter(false);            // close balancing shutter
            if ( State == COM_STA_STOP )
               Ret = ObjCom->SetDomeStop();                 // stop balancing shutter movement
            break;

         // telescope
         case COM_SUBJ_TELE:
            if ( State == COM_STA_LEFT )
            {
               Ret = ObjCom->SetTelescopeVelocity( Par1 );  // telescope left movement
               if ( Ret != 0 )
                  break;
               Ret = ObjCom->GoTelescopeEast();
            }
            if ( State == COM_STA_RIGHT )
            {
               Ret = ObjCom->SetTelescopeVelocity( Par1 );  // telescope right movement
               if ( Ret != 0 )
                  break;
               Ret = ObjCom->GoTelescopeWest();
            }
            if ( State == COM_STA_UP )
            {
               Ret = ObjCom->SetTelescopeVelocity( Par1 );  // telescope up movement
               if ( Ret != 0 )
                  break;
               Ret = ObjCom->GoTelescopeNorth();
            }
            if ( State == COM_STA_DOWN )
            {
               Ret = ObjCom->SetTelescopeVelocity( Par1 );  // telescope down movement
               if ( Ret != 0 )
                  break;
               Ret = ObjCom->GoTelescopeSouth();
            }
            if ( State == COM_STA_STOP )
               Ret = ObjCom->StopTelescope();               // telescope stop movement
            if ( State == COM_STA_HOME )
               Ret = ObjCom->TelescopeHome();               // telescope in home position
            break;

         // contoller timeout
         case COM_SUBJ_CONTROLLER:
            if ( State == COM_STA_ON )
               Ret = ObjCom->SetTimeoutControl( true );     // turn on timeout control
            if ( State == COM_STA_OFF )
               Ret = ObjCom->SetTimeoutControl( false );    // turn off timout control
            break;

         // PEC
         case COM_SUBJ_PEC:
            if ( State == COM_STA_ON )
               Ret = ObjCom->SetPEC( true, Par1, Par2 );     // turn on PEC
            if ( State == COM_STA_OFF )
               Ret = ObjCom->SetPEC( false, Par1, Par2 );    // turn off PEC
            break;

      }
      Notify( Sender, NOTIFY_GENERIC_COMMAND );
      if ( Ret == 0 )
         Error = ERR_NONE;
      else
         Error = ERR_LX200_OPEN;
   }

   // nightly shot folder
   if ( Subj == COM_SUBJ_NIGHTLYFOLDER )
   {
      if ( State == COM_STA_SAVE )
      {
         Error = SaveFile( ShotFolder, ShotList );          // save shot in nightly shot folder
         Notify( Sender, NOTIFY_SHOTSAVED );
      }
      return Error;
   }

   // dark shot folder
   if ( Subj == COM_SUBJ_DARKFOLDER )
   {
      if ( State == COM_STA_SAVE )
      {
         Error = SaveFile( ShotFolder, DarkList );          // save shot in dark shot folder
         Notify( Sender, NOTIFY_SHOTSAVED );
      }
      return Error;
   }

   return Error;
}



/****************************************************************************

   Position/Align telescope

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::TelescopePos( Observer *Sender, CMHours::HOURS *RA, CMDegree::DEGREE*Dec, char *Object, bool Allign )
{
   int                                 Ret;                 // communication retun value

   // check if this observer can run the command
   if ( ObserverOnObservatory != 0  && ObserverOnObservatory != Sender)
      return ERR_OBSERVATORYBUSY;
      if ( InternalStatus.TeleTracking == false )
      return ERR_NOTRACKING;

   // position or allign telescope
   if ( Allign == false )
   {
      Ret = ObjCom->TelescopeGoto( RA, Dec );            // run positioning on LX200 object
      if ( Ret == 0 )
      {
         InternalStatus.TargetRA = *RA;                     // store target position
         InternalStatus.TargetDec = *Dec;
         strcpy( InternalStatus.TargetObject, Object );     // store object name
         Notify( Sender, NOTYFY_TELESCOPEPOSITIONED );      // notify event
      }
   }
   else
   {
      Ret = ObjCom->SetTelescopePosition( RA, Dec );
      if ( Ret == 0 )
      {
         InternalStatus.TargetRA = *RA;                     // store target position
         InternalStatus.TargetDec = *Dec;
         InternalStatus.TeleRA = *RA;                       // store actual telescope position
         InternalStatus.TeleDec = *Dec;
         strcpy( InternalStatus.TargetObject, Object );     // store object name
         Notify( Sender, NOTIFY_TELESCOPEALLIGED );         // notify event
         if ( InternalStatus.ShotRA.hh < 24 )               // if short position is valid
         {
            InternalStatus.ShotRA = InternalStatus.TargetRA;// uptade shot position
            InternalStatus.ShotDec = InternalStatus.TargetDec;
         }
      }
   }

   // if coomunication failed return error
   if ( Ret != 0 )
      return ERR_POS;

   return ERR_NONE;
}



/****************************************************************************

   set automatic velocity

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::SetAutoVelocity( Observer *Sender, int Velocity )
{
   int                                 Ret;                 // communication retun value

   // check if this observer can run the command
   if ( ObserverOnObservatory != 0  && ObserverOnObservatory != Sender)
      return ERR_OBSERVATORYBUSY;

   Ret = ObjCom->SetAutoVelocity( Velocity );               // run positioning on LX200 object

   // if coomunication failed return error
   if ( Ret != 0 )
      return ERR_COMMAND;

   return ERR_NONE;
}



/****************************************************************************

   Take a shot  (time in millisecond)

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::Shot( Observer *Sender, int Time )
{
   CCD::ERR                            Result;              // CCD error code

   // check if this observer can run the command
   if ( ObserverOnCCD != 0  && ObserverOnCCD != Sender)
      return ERR_CCDBUSY;

   Result = ObjCcd->Shot( Time );                           // run shot
   if ( Result != CCD::ERR_NONE )
      return ERR_NOSHOT;

   InternalStatus.ShotRA = InternalStatus.TeleRA;           // store telescope shot position
   InternalStatus.ShotDec = InternalStatus.TeleDec;
   InternalStatus.ShotInProgress = true;                    // set shot is in progress
   InternalStatus.ShotExposure = Time;                      // store shot time
   Notify( Sender, NOTIFY_SHOTSTART );                      // notify event
  	return ERR_NONE;
}




/****************************************************************************

   Abort shot exposing

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::AbortShot( Observer *Sender )
{
   CCD::ERR                            Result;              // CCD error code

   // check if this observer can run the command
   if ( ObserverOnCCD != 0  && ObserverOnCCD != Sender)
      return ERR_CCDBUSY;

   Result = ObjCcd->AbortShot();                            // abort current shot
   if ( Result != CCD::ERR_NONE )
      return ERR_NOABORTSHOT;

   InternalStatus.ShotInProgress = false;                   // reset shot is in progress
   Notify( Sender, NOTIFY_SHOTABORT );                      // notify event

  	return ERR_NONE;
}



/****************************************************************************

   Verify is shot is finished

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::IsShotFinisched( int *Time )
{
   CCD::ERR                            Result;              // CCD error code


   Result = ObjCcd->IsShotFinisched( Time );                // test short finish
   InternalStatus.ShotExposing = *Time;                     // store current exposing time
   if ( Result == CCD::ERR_NOTFINISHED )
      return ERR_NOTFINISHED;

   InternalStatus.ShotInProgress = false;                   // reset shot is in progress
  	return ERR_NONE;
}



/****************************************************************************

   Get shot

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::GetShot( Observer *Sender )
{
   CCD::ERR                            Result;              // error code

   // check if this observer can run the command
   if ( ObserverOnCCD != 0  && ObserverOnCCD != Sender)
      return ERR_CCDBUSY;

   Result = ObjCcd->GetShot();                              // get last shot from ccd
   if ( Result != CCD::ERR_NONE )
      return ERR_NOSHOT;

   Notify( Sender, NOTIFY_SHOTDOWNLOADED );                 // notify event
  	return ERR_NONE;
}



/****************************************************************************

   Get informazioni about last shot

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::GetShotInfo( CCD::SHOT_INFO* Info )
{
   CCD::ERR                            Result;              // error code
   DWORD                               Lap;                 // safety timer

   Result = ObjCcd->GetShotInfo( Info);                     // get information about last shot
   if ( Result != CCD::ERR_NONE )
      return ERR_NOSHOT;

   // if safety active
   if ( SafeEnable == true )
   {
      // quality too low (may be clouds)
      if ( Info->VisiblityIndex < SafeThreshold )
      {
         // it is the first time
         if ( SafeTimerStarted == false )
            SafeTimer = GetTickCount();                     // store initial time
         SafeTimerStarted = true;                           // timer is enabled
         Lap = GetTickCount();                              // current time
         // Safety time is expired
         if ( (int)(Lap-SafeTimer) >= SafeTime*1000 )
         {
            Notify( 0, NOTIFY_SKYCLOUDY );
            return ERR_CLOUDY;
         }
      }
      else
         // quality good enoght
         SafeTimerStarted = false;
   }


  	return ERR_NONE;
}



/****************************************************************************

   set the object name for shot

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::SetObjectName( char *PObjectName )
{
   strcpy( ObjectName, PObjectName );
   return ERR_NONE;
}



/****************************************************************************

   Select star near given coordinate on shot

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::SelectStarOnShot( int *x, int *y, CMHours::HOURS *RAStar, CMDegree::DEGREE*DecStar)
{
   CCD::SHOT_INFO                      Info;                // last shot information
   CCD::ERR                            Result;              // resutl of CCD call
   CMHours                             *ObjRA;              // object for RA computing
   CMDegree                            *ObjDec;             // object for declination computing
   double                              DecCenter;           // declination of the shot center
   int                                 DeltaX;              // x delta vs shot center
   int                                 DeltaY;              // y delta vs shot center
   double                              DeltaRA;             // right ascension delta vs shot center
   double                              DeltaDec;            // declination delta vs shot center

   Result = ObjCcd->GetShotInfo( &Info );                   // get last shot information
   if ( Result != CCD::ERR_NONE )
      return ERR_NOSHOT;

   ObjCcd->SearchHighestPixel( x, y, CCD::RADIUS_STAR*2, 0 );

   // initialize object with shot center
   ObjRA = new CMHours();                                   // create RA object
   ObjRA->Set( &InternalStatus.ShotRA );                    // set shot RA center
   ObjDec = new CMDegree();                                 // create declination object
   ObjDec->Set( &InternalStatus.ShotDec );                  // set declination center
   DecCenter = ObjDec->GetC();                              // declination in degree

   // compute star RA
   DeltaX = *x - Info.Width / 2;                            // compute x delta
   DeltaRA = - DeltaX;                                      // RA direction contrary window coordinates
   DeltaRA *= PixelScale;                                   // convert in arc sec
   DeltaRA /= 3600;                                         // convert in degree
   DeltaRA /= 15;                                           // convert in hours
   DeltaRA /= cos( DecCenter / RAD );                       // adapt to declination shrinkage
   ObjRA->Add( DeltaRA );                                   // add RA delta to shot center
   *RAStar = *ObjRA->Get();                                 // return star RA

   // compute star declination
   DeltaY = *y - Info.Height/2;                             // compute y delta
   DeltaDec = - DeltaY;                                     // declination direction contrary window coordinates
   DeltaDec *= PixelScale;                                  // convert in arc sec
   DeltaDec /= 3600;                                        // convert in degree
   ObjDec->Add( DeltaDec );                                 // add declination to shot center
   *DecStar = *ObjDec->Get();                               // return star declination

   // destroy object
   delete ObjRA;                                            // delete RA object
   delete ObjDec;                                           // delete declination object

   return ERR_NONE;
}



/****************************************************************************

   Get last shot image as a gray bitmap

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::GetBmp( Graphics::TBitmap ** Bmp, int Min, int Max )
{
   CCD::ERR                            Result;              // error code
   int                                 ObjX;                // object x coordinate
   int                                 ObjY;                // object y coordinate

   Result = ObjCcd->GetBmp( Bmp, Min, Max );                // get last shot as bmp
   if ( Result != CCD::ERR_NONE )
      return ERR_NOSHOT;

   // draw CCD center
   ObjX = (*Bmp)->Width/2;
   ObjY = (*Bmp)->Height/2;
   (*Bmp)->Canvas->Pen->Color = clYellow;
   (*Bmp)->Canvas->MoveTo(ObjX, ObjY-2);
   (*Bmp)->Canvas->LineTo(ObjX, ObjY+3);
   (*Bmp)->Canvas->MoveTo(ObjX-2, ObjY);
   (*Bmp)->Canvas->LineTo(ObjX+3, ObjY);


  	return ERR_NONE;
}



/****************************************************************************

   Get last shot image zoomed as a gray bitmap

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::GetZoomBmp( Graphics::TBitmap ** Bmp, int Min, int Max, int CenterX, int CenterY )
{
   CCD::ERR                            Result;              // error code

   Result = ObjCcd->GetZoomBmp( Bmp, Min, Max, CenterX, CenterY ); // get zoomed bmp of last shot
   if ( Result != CCD::ERR_NONE )
      return ERR_NOSHOT;

  	return ERR_NONE;
}



/****************************************************************************

   get Histogram bitmap

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::GetHistogramBmp( Graphics::TBitmap ** Bmp, int Min, int Max )
{
   CCD::ERR                            Result;              // error code

   Result = ObjCcd->GetHistogramBmp( Bmp, Min, Max);        // get histogramm bmp
   if ( Result != CCD::ERR_NONE )
      return ERR_NOSHOT;

  	return ERR_NONE;
}



/****************************************************************************

   get object list fron object class

*****************************************************************************/

CMObjectList * TentaclesMgr::GetObjectList( OBJECT_CLASS Class )
{
   switch ( Class )
   {
      // get planet list
      case OBJECT_PLANETS:
         return PlanetList;

      // get reference stars list
      case OBJECT_REFSTARS:
         return RefStarsList;

      // get minor planet list
      case OBJECT_MINORPLANETS:
         return MPlanetsList;

      // get double stars list
      case OBJECT_MULTISTARS:
         return MultipleStarList;

      // get cluster list
      case OBJECT_CLUSTERS:
         return ClustersList;

      // get nebulas list
      case OBJECT_NEBULAS:
         return NebuleList;

      // get galaxies list
      case OBJECT_GALAXIES:
         return GalaxiesList;

      // get NGC list
      case OBJECT_NGC:
         return NGCList;
   }

   return 0;
}



/****************************************************************************

   get chart for the bimap with given center position and cdd rectangle

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::GetChart( Graphics::TBitmap **PBmp,
                                    CMHours::HOURS *RACenter, CMDegree::DEGREE*DecCenter,
                                    int Radius, int CCDWidth, int CCDHeight)
{
   Graphics::TBitmap                   *Bmp;                // pointer to input bitmap
   double                              Scale;               // chart scale in arcsec/pixel
   SYSTEMTIME                          LocalTime;           // current data and time
   CMEvent::DATE                       Date;                // data for event object
   CMEvent                             *Event;              // event object
   double                              CurrentJD;           // julian day at current data and time
   double                              HoraryJD;            // julian day at full hours
   double                              RACent;              // RA chart center in degree
   double                              DecCent;             // declination chart center in degree
   CMObject*                           Obj;                 // current evaluating object
   CMObject*                           ObjNext;             // next evaluating object
   CMHours                             *ObjRACenter;        // object for RA computing
   CMDegree                            *ObjDecCenter;       // object for declination computing
   TRect                               Rect;                // rectangle for graphic drawing
   const CMCoEquatorial                *EquPos;             // object equatorial position
   double                              RAObj;               // RA of current object
   double                              DecStar;             // declination of current star
   double                              Magnitude;           // object magnitude
   bool                                InChart;             // object is into the chart
   int                                 ObjX;                // object x coordinate
   int                                 ObjY;                // object y coordinate
   int                                 ObjX2;               // object x coordinate for minor planet traking
   int                                 ObjY2;               // object y coordinate for minor planet traking
   int                                 ObjRadius;           // object iradius in pixel
   int                                 ObjHRadius;          // object half radius in pixel
   int                                 Hour;                // hour counter
   char                                Name[30];            // object name
   int                                 Lenght;              // nemae extension

   // save chart parameters
   Bmp = *PBmp;                                             // get pointer to input bitmap
   ChartRACenter = *RACenter;
   ChartDecCenter = *DecCenter;
   ChartWidth = Bmp->Width;
   ChartHeight = Bmp->Height;
   ChartRadius = Radius;

   // set event object with current data and time
   GetSystemTime ( &LocalTime );                            // get PC time
   Date.Year = LocalTime.wYear;                             // set event structure
   Date.Month = LocalTime.wMonth;
   Date.Day = LocalTime.wDay;
   Date.Hour = LocalTime.wHour;
   Date.Minute = LocalTime.wMinute;
   Date.Sec = LocalTime.wSecond;
   Event = new CMEvent();
   Event->SetDate( &Date );                                 // set event
   CurrentJD = Event->GetGG();                              // get JD
   delete Event;

   // compute chart center in degree
   ObjRACenter = new CMHours();                               // create RA object
   ObjRACenter->Set( RACenter );                            // set RA chart center
   RACent = ObjRACenter->GetC()*15;                         // get RA in degree
   delete ObjRACenter;                                      // delete RA object
   ObjDecCenter = new CMDegree();                           // create dec object
   ObjDecCenter->Set( DecCenter );                          // set dec chart center
   DecCent = ObjDecCenter->GetC();                          // get dec in degree
   delete ObjDecCenter;                                     // delete dec object

   // rectangle inizialization as whole bitmap
   Rect.Top = 0;
   Rect.Left = 0;
   Rect.Right = Bmp->Width;
   Rect.Bottom = Bmp->Height;

   // draw background color
   Bmp->Canvas->Brush->Color = clBlack;
   Bmp->Canvas->FillRect(Rect);

   // create list of star arround the center
   if ( ChartObjects != 0 )
      delete ChartObjects;
   ChartObjects = ObjGSC->GetStars( RACenter, DecCenter, Radius );

   // compute scale arcsec/pixel
   Scale = Radius;
   Scale /= 3600;
   Scale /= Bmp->Width;

   // set color for NGC object drawing
   Bmp->Canvas->Pen->Color = clGreen;
   Bmp->Canvas->Font->Color = clGreen;
   Bmp->Canvas->Brush->Color = clBlack;
   Bmp->Canvas->Brush->Style = bsClear;

   // draw NGC objects
   NGCList->First();                                        // reset internal cursor list
   ObjNext = NGCList->Next();                               // get first object
   while (ObjNext != 0 )                                    // works only until object is avalable
   {
      Obj = ObjNext;                                        // star to draw
      ObjNext = NGCList->Next();                            // prepare next object
      Obj->SetData(CurrentJD);                              // set current data and time
      RAObj = Obj->GetPosition()->GetRA()->GetC()*15;       // get RA of the star
      DecStar = Obj->GetPosition()->GetDec()->GetC();       // get declination of the star

      // compute star position
      InChart = ObjectToChart( RACent, DecCent, Scale, Bmp->Width, Bmp->Height, RAObj, DecStar, &ObjX, &ObjY );
      if ( InChart == false )                               // bypass stars out of chart
         continue;

      strcpy( Name, Obj->GetName(CMObject::NAME_CATALOG) ); // get catalog name
      strcat( Name, " " );                                  // space
      strcat( Name, Obj->GetName(CMObject::NAME_OWN) );     // messier name
      Bmp->Canvas->Ellipse(ObjX-10, ObjY-10, ObjX+10, ObjY+10);  // draw circle
      Lenght = Bmp->Canvas->TextWidth( Name );              // measure name horizontal extension
      Bmp->Canvas->TextOutA(ObjX - Lenght/2, ObjY-25, Name ); // draw name
   }

   // set color for star drawing
   Bmp->Canvas->Pen->Color = clWhite;

   // draws stars
   ChartObjects->First();                                   // set list at first element
   ObjNext =  ChartObjects->Next();                         // get first star
   while ( ObjNext != 0 )                                   // until star are present in the list ..
   {
      Obj = ObjNext;                                        // star to draw
      ObjNext = ChartObjects->Next();                       // prepare next star
      Obj->SetData(CurrentJD);                              // set current data and time
      RAObj = Obj->GetPosition()->GetRA()->GetC()*15;       // get RA of the star
      DecStar = Obj->GetPosition()->GetDec()->GetC();       // get declination of the star

      // compute star position
      InChart = ObjectToChart( RACent, DecCent, Scale, Bmp->Width, Bmp->Height, RAObj, DecStar, &ObjX, &ObjY );
      if ( InChart == false )                               // bypass stars out of chart
         continue;

      Magnitude = Obj->GetMag();                            // get star magnitude
      ObjRadius = 17 - Magnitude;                           // compute start radius with mignitude
      if ( ObjRadius < 1 )                                  // at leat 1
         ObjRadius = 1;

      // draw start depending on radius
      Bmp->Canvas->Brush->Color = clWhite;                  // select white color
      switch ( ObjRadius )
      {
         case 1:
            // draw star as a dot
            Rect.Top = ObjY;
            Rect.Left = ObjX;
            Rect.Right = ObjX+1;
            Rect.Bottom = ObjY+1;
            Bmp->Canvas->FillRect(Rect);
            break;
         case 2:
            // draw star as two dot
            Rect.Top = ObjY;
            Rect.Left = ObjX;
            Rect.Right = ObjX+1;
            Rect.Bottom = ObjY+1;
            Bmp->Canvas->FillRect(Rect);
            Bmp->Canvas->Brush->Color = clGray;
            Rect.Top = ObjY;
            Rect.Left = ObjX+1;
            Rect.Right = ObjX+2;
            Rect.Bottom = ObjY+1;
            Bmp->Canvas->FillRect(Rect);
            break;
         case 3:
            // draw star as a cross
            Bmp->Canvas->MoveTo(ObjX-1,ObjY);
            Bmp->Canvas->LineTo(ObjX+2,ObjY);
            Bmp->Canvas->MoveTo(ObjX,ObjY-1);
            Bmp->Canvas->LineTo(ObjX,ObjY+2);
            break;
         default:
            // draw star as circle
            ObjHRadius = ObjRadius / 2;
            Bmp->Canvas->Ellipse(ObjX-ObjHRadius ,ObjY-ObjHRadius, ObjX-ObjHRadius+ObjRadius+1, ObjY-ObjHRadius+ObjRadius+1);
            break;
      }
   }

   // draws minor planet
   MPlanetsList->First();                                   // reset internal cursor list
   ObjNext = MPlanetsList->Next();                          // get first object
   while (ObjNext != 0 )                                    // works only until object is avalable
   {
      Obj = ObjNext;                                        // star to draw
      ObjNext = (CMStar*) MPlanetsList->Next();             // prepare next star

      // compute julian date at previous full hour
      HoraryJD = CurrentJD;                                 // start from current HD
      HoraryJD *= 24;                                       // compute in hours
      HoraryJD = floor ( HoraryJD );                        // loose decimal
      HoraryJD /= 24.0;                                     // previous full hour

      // color for minor planet track
      Bmp->Canvas->Brush->Color = clGreen;
      Bmp->Canvas->Pen->Color = clGreen;

      for ( Hour=0; Hour<7; Hour++ )                        // compute 7 positions (duration 6 hours)
      {
         Obj->SetData(HoraryJD);                            // set computing time
         EquPos = Obj->GetPosition();                            // get equatorial position (J2000)
         EquPos = EquPos->ChangeEqu( HoraryJD );            // get equatorial position (ephoc time)
         RAObj = EquPos->GetRA()->GetC()*15;                // get RA of minor planet
         DecStar = EquPos->GetDec()->GetC();                // get declination of minor planet

         // compute minor planet position on the chart
         InChart = ObjectToChart( RACent, DecCent, Scale, Bmp->Width, Bmp->Height, RAObj, DecStar, &ObjX2, &ObjY2 );
         // store first position for traking draw
         if ( Hour == 0 )
         {
            ObjX = ObjX2;
            ObjY = ObjY2;
         }

         HoraryJD += 1.0/24.0;                              // go to next full hors

         // if position out of chart it don't draw
         if ( InChart == false )
            continue;

         // draw box to mark horaly position
         Rect.Top = ObjY2-1;
         Rect.Left = ObjX2-1;
         Rect.Bottom = ObjY2+3;
         Rect.Right = ObjX2+3;
         Bmp->Canvas->FillRect( Rect);
      }

      // draw traking line
      Bmp->Canvas->MoveTo(ObjX,ObjY);
      Bmp->Canvas->LineTo(ObjX2,ObjY2);

      // compute current position
      Obj->SetData(CurrentJD);                              // current time
      EquPos = Obj->GetPosition();                          // get equatorial position (J2000)
      EquPos = EquPos->ChangeEqu( CurrentJD );              // get equatorial position (ephoc time)
      RAObj = EquPos->GetRA()->GetC()*15;                   // get RA of minor planet
      DecStar = EquPos->GetDec()->GetC();                   // get declination of minor planet

      // compute minor planet position on the chart
      InChart = ObjectToChart( RACent, DecCent, Scale, Bmp->Width, Bmp->Height, RAObj, DecStar, &ObjX, &ObjY );

      // if position out of chart it don't draw
      if ( InChart == false )
         continue;

      // current position highlighted
      ObjRadius = 4;
      Bmp->Canvas->Brush->Color = clLime;
      Bmp->Canvas->Pen->Color = clLime;
      // draw minor planet as circle
      ObjHRadius = ObjRadius / 2;
      Bmp->Canvas->Ellipse(ObjX-ObjHRadius ,ObjY-ObjHRadius, ObjX-ObjHRadius+ObjRadius+1, ObjY-ObjHRadius+ObjRadius+1);
   }

   // draw CCD rectangle
   Rect.Top = (Bmp->Height - CCDHeight)/2;
   Rect.Left = (Bmp->Width - CCDWidth)/2 - 1;
   Rect.Bottom = Rect.Top + CCDHeight ;
   Rect.Right = Rect.Left + CCDWidth + 2;
   Bmp->Canvas->Brush->Color = clRed;
   Bmp->Canvas->FrameRect(Rect);

   // draw CCD center
   ObjX = Bmp->Width/2;
   ObjY = Bmp->Height/2;
   Bmp->Canvas->Pen->Color = clRed;
   Bmp->Canvas->MoveTo(ObjX, ObjY-2);
   Bmp->Canvas->LineTo(ObjX, ObjY+3);
   Bmp->Canvas->MoveTo(ObjX-2, ObjY);
   Bmp->Canvas->LineTo(ObjX+3, ObjY);

   return ERR_NONE;
}



/****************************************************************************

   Select star near given coordinate on bitmap

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::SelectStarOnChart( int *x, int *y, CMHours::HOURS *RACenter, CMDegree::DEGREE*DecCenter)
{
   SYSTEMTIME                          LocalTime;           // current data and time
   CMEvent::DATE                       Date;                // data for event object
   CMEvent                             *Event;              // event object
   double                              CurrentJD;           // current julian day
   CMHours                             *ObjRACenter;        // object for RA center computing
   CMDegree                            *ObjDecCenter;       // object for declination center computing
   double                              RACent;              // RA chart center in degree
   double                              DecCent;             // declination chart center in degree
   CMHours                             *ObjRASearch;        // object for RA search computing
   CMDegree                            *ObjDecSearch;       // object for declination search computing
   double                              RASearch;            // RA search in degree
   double                              DecSearch;           // declination search in degree
   int                                 Deltax;              // delta point x in pixel
   int                                 Deltay;              // delta point y in pixel
   double                              DeltaRA;             // delta point in RA
   double                              DeltaDec;            // delta point in declination
   CMStar                              *Star;               // current star in scannin
   CMStar                              *NearestStar;        // nearest star
   double                              DecStar;             // declination star
   double                              RAStar;              // RA star
   double                              Distance;            // distance between star and point
   double                              NearestDistance;     // nearest distance
   double                              HourAngle;           // hour angle simulation
   double                              Latitude;            // latitude simulation
   double                              Azimut;              // azimut simulation
   double                              Elevation;           // evelation simulation
   double                              Scale;               // chart scale in arcsec/pixel
   double                              cn;                  // temporary for computing
   double                              cd;                  // temporary for computing

   // set event object with current data and time
   GetSystemTime ( &LocalTime );                            // get PC time
   Date.Year = LocalTime.wYear;                             // set event structure
   Date.Month = LocalTime.wMonth;
   Date.Day = LocalTime.wDay;
   Date.Hour = LocalTime.wHour;
   Date.Minute = LocalTime.wMinute;
   Date.Sec = LocalTime.wSecond;
   Event = new CMEvent();
   Event->SetDate( &Date );                                 // set event
   CurrentJD = Event->GetGG();                              // get JD
   delete Event;

   // compute chart center in degree
   ObjRACenter = new CMHours();                             // create RA object
   ObjRACenter->Set( &ChartRACenter );                      // set RA chart center
   RACent = ObjRACenter->GetC()*15;                         // get RA in degree
   delete ObjRACenter;                                      // delete RA object
   ObjDecCenter = new CMDegree();                           // create dec object
   ObjDecCenter->Set( &ChartDecCenter );                    // set dec chart center
   DecCent = ObjDecCenter->GetC();                          // get dec in degree
   delete ObjDecCenter;                                     // delete dec object

   // compute RA chart position corresponding to point
   ObjRASearch = new CMHours();                             // create RA object
   ObjRASearch->Set( &ChartRACenter );                      // set RA chart center
   Deltax = *x - ChartWidth / 2;                            // delta to chart center in pixel
   DeltaRA = - Deltax;                                      // delta RA in opposite direction
   DeltaRA *= ChartRadius;                                  // scale pixel to arcsec
   DeltaRA /= ChartWidth;
   DeltaRA /= 3600;                                         // compute degree
   DeltaRA /= 15;                                           // convert in hours
   DeltaRA /= cos( DecCent / RAD );                         // adapt to declination srinking
   ObjRASearch->Add( DeltaRA );                             // find RA correspinding to point
   RASearch = ObjRASearch->GetC()*15;                       // get RA in degree
   delete  ObjRASearch;

   // compute declination chart position corresponding to point
   ObjDecSearch = new CMDegree();                           // create dec object
   ObjDecSearch->Set( &ChartDecCenter );                    // set dec chart center
   Deltay = *y - ChartHeight / 2;                           // delta to chart center in pixel
   DeltaDec = - Deltay;                                     // delta declination in opposite direction
   DeltaDec *= ChartRadius;                                 // scale pixel to arcsec
   DeltaDec /= ChartWidth;
   DeltaDec /= 3600;                                        // compute degree
   ObjDecSearch->Add( DeltaDec );                           // find declination correspinding to point
   DecSearch = ObjDecSearch->GetC();                        // get declination in degree
   delete ObjDecSearch;

   // scan list in order to find the nearest
   ChartObjects->First();                                   // set list at first element
   Star = (CMStar*) ChartObjects->Next();                   // get first star
   NearestStar = 0;                                         // no start found until now
   NearestDistance=0;
   while ( Star != 0 )                                      // until star are present in the list ..
   {
      Star->SetData(CurrentJD);                             // set current data and time
      RAStar = Star->GetPosition()->GetRA()->GetC()*15;     // get star RA
      DecStar = Star->GetPosition()->GetDec()->GetC();      // get star declination
      Distance = pow( RAStar - RASearch, 2 ) + pow( DecStar-DecSearch, 2 );   // compute distance
      Distance = pow(Distance, 0.5 );
      Distance *= 3600;                                     // distance in arcsec
      if ( Distance < 50 )                                  // take care on distence less than 50 arcsec
      {
         if ( NearestStar == 0 || Distance < NearestDistance )    // check i this distence is less than previous
         {
            NearestStar = Star;                             // store star
            NearestDistance = Distance;                     // store distance
         }
      }
      Star = (CMStar*) ChartObjects->Next();                // go to next star in the list
   }

   if ( NearestStar == 0 )                                  // if no star found, error
      return ERR_STARNOTFOUD;

   // yeld star coordinates
   *RACenter = *NearestStar->GetPosition()->GetRA()->Get();
   *DecCenter = *NearestStar->GetPosition()->GetDec()->Get();

   // compute scale arcsec/pixel
   Scale = ChartRadius;
   Scale /= 3600;
   Scale /= ChartWidth;

   // get star coordinates in degree
   RAStar = NearestStar->GetPosition()->GetRA()->GetC()*15;
   DecStar = NearestStar->GetPosition()->GetDec()->GetC();

   // compute coordinate star on the chart
   HourAngle = RACent - RAStar;                             // simulate hour angle
   Latitude = DecCent + 90;                                 // simulate latitude

   // simulate azimut computing
   cn = sin ( HourAngle / RAD );                            // compute numerator
   cd = cos ( HourAngle / RAD ) * sin ( Latitude / RAD ) - tan ( DecStar / RAD) * cos ( Latitude / RAD );  // compute denominator
   Azimut = atan ( cn / cd ) * RAD;                         // compute azimut
   if ( cd < 0 )                                            // arrange sector 2,3
      Azimut += 180;
   // simulate elevation computing
   Elevation = sin( Latitude / RAD ) * sin ( DecStar / RAD ) + cos ( Latitude / RAD ) * cos ( DecStar / RAD ) * cos ( HourAngle / RAD );
   Elevation = asin ( Elevation ) * RAD;

   // compute x position
   cn = Azimut;                                             // azimut
   if ( cn > 180 )                                          // in the range -180 - +180
      cn -= 360;
   cn /= Scale;                                             // compute in pixel
   cn += ChartWidth / 2;                                    // add bitmap center
   *x = cn;                                                 // this is x coordinate

   // compute y position
   cn = - Elevation;                                        // elevation in opposite direction of bitmap coordinate
   cn /= Scale;                                             // compute in pixel
   cn += ChartHeight / 2;                                   // add bitmap center
   *y = cn;                                                 // this is y coordinate

   return ERR_NONE;
}



/****************************************************************************

   get image value from position

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::GetImageValue( int x, int y, int *Value )
{
   CCD::ERR                            Result;              // error code

   Result = ObjCcd->GetImageValue( x, y, Value );           // get image value
   if ( Result != CCD::ERR_NONE )
      return ERR_NOSHOT;

  	return ERR_NONE;
}



/****************************************************************************

   Save file with progressive name

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::SaveFile( char *Folder, SHOT *List )
{
   int                                 IndexName;           // index for name counter scan
   int                                 Counter;             // conter to add to name
   char                                FileName[MAXPATH];   // file name
   char                                Str[10];             // temporary
   bool                                NameFault;           // name already used
   CCD::SHOT_INFO                      Info;                // information about shot
   unsigned short                      *Image;              // buffer image original
   unsigned short                      *Image2;             // buffer image flipped
   int                                 RowIndex;            // index for row scanning
   int                                 ColumnIndex;         // index for column scanning
   unsigned short                      *pRead;              // reading pointer for image flipping
   unsigned short                      *pWrite;             // writing pointer for image flipping
   SYSTEMTIME                          LocalTime;           // current data and time
   CMEvent                             *Event;              // event object
   CMEvent::DATE                       Date;                // data for event object
   double                              J2000;               // 2000 epoch julian day
   double                              CurrentJD;           // julian day at current data and time
   CMCoEquatorial                      *EquPos;             // object equatorial position at current equinox
   CMCoEquatorial                      *EquPos2000;         // object equatorial position at 2000 equinox
   CMHours::HOURS                      RA2000;              // right acension at 2000 equinox
   CMDegree::DEGREE                    Dec2000;             // declination at 2000 equinox

   // CCD object have to be present
   if ( ObjCcd == NULL )
      return ERR_NOSHOT;

   // CCD have to have an image not saved
   ObjCcd->GetShotInfo( &Info);
   if ( Info.State != CCD::CCD_STATE_IMAGE )
      return ERR_NOSHOT;

   // create directory if it is not already present
   if ( GetFileAttributes( Folder ) ==  0xFFFFFFFF )
      CreateDirectory( Folder, NULL );

   // manage progressive name conter
   if ( List == 0 )
   {
      strcpy( FileName, Folder);                               // filename start with folder
      strcat( FileName, ObjectName );                          // add object name
      strcat( FileName, ".fit" );                              // add file extension
   }
   else
   {
      for ( IndexName = 0; IndexName < MAX_SHOT; IndexName++)  // scan counter
      {
         if ( strcmp( List[IndexName].ObjectName, ObjectName) == 0 )  // check the name
         {
            Counter = ++List[IndexName].LastCount;             // increment counter and assign
            break;
         }
         if ( List[IndexName].ObjectName[0] == 0 )             // name empty means not found so I'lll insert it
         {
            strcpy( List[IndexName].ObjectName, ObjectName );  // insert the name
            Counter = ++List[IndexName].LastCount;             // increment counter and assign ( 1 )
            break;
         }
      }
      if ( IndexName >= MAX_SHOT )                             // space run out
         return ERR_TOMANYSHOT;

      // build the name file and verify if it is free
      do
      {
         strcpy( FileName, Folder);                            // filename start with folder
         strcat( FileName, ObjectName );                       // add object base name
         itoa( Counter, Str, 10);                              // convert name to string
         strcat( FileName, "_" );                              // add underscore
         strcat( FileName, Str );                              // add conter
         strcat( FileName, ".fit" );                           // add file extension
         NameFault = false;                                    // reset fault condition
         if ( GetFileAttributes( FileName ) !=  0xFFFFFFFF )   // check if file name is already present
         {
            NameFault = true;                                  // fault in the name
            Counter++;                                         // increment name counter
            List[IndexName].LastCount = Counter;               // update counter in the table
         }
      } while ( NameFault == true );                           // repeat until error is present

      // save the last short name
      strcpy( ShotName, ObjectName);                           // object base name
      strcat( ShotName, "_" );                                 // add underscore
      strcat( ShotName, Str );                                 // add conter
   }

   Image = (unsigned short*)malloc( Info.Width * Info.Height * 2);
   Image2 = (unsigned short*)malloc( Info.Width * Info.Height * 2);
   ObjCcd->GetImage( Image );

   // horizontal image flip
   for ( RowIndex=0; RowIndex<Info.Height; RowIndex++ )
   {
      pRead = Image  + (RowIndex+1) * Info.Width;
      pWrite = Image2 +  RowIndex    * Info.Width;
      for ( ColumnIndex=0; ColumnIndex<Info.Width; ColumnIndex++ )
      {
         --pRead;
         *pWrite++ = *pRead ;
      }
   }

   // set event object with J2000 data and time
   Event = new CMEvent();                                   // create object for jd computing
   Date.Year = 2000;                                        // set event structure
   Date.Month = 1;
   Date.Day = 1;
   Date.Hour = 0;
   Date.Minute = 0;
   Date.Sec = 0;
   Event->SetDate( &Date );                                 // set event
   J2000 = Event->GetGG();                                  // get JD
   // set event to current data and time
   GetSystemTime ( &LocalTime );                            // get PC time
   Date.Year = LocalTime.wYear;                             // set event structure
   Date.Month = LocalTime.wMonth;
   Date.Day = LocalTime.wDay;
   Date.Hour = LocalTime.wHour;
   Date.Minute = LocalTime.wMinute;
   Date.Sec = LocalTime.wSecond;
   Event->SetDate( &Date );                                 // set event
   CurrentJD = Event->GetGG();                              // get current JD
   delete Event;

   EquPos = new CMCoEquatorial();
   EquPos->GetRA()->Set(&InternalStatus.ShotRA );
   EquPos->GetDec()->Set(&InternalStatus.ShotDec );
   EquPos->SetDataEqu(CurrentJD);
   EquPos2000 = EquPos->ChangeEqu( J2000 );                 // get equatorial position (ephoc time)
   RA2000 = *EquPos2000->GetRA()->Get();
   Dec2000 = *EquPos2000->GetDec()->Get();
   delete EquPos;
   delete EquPos2000;

   ObjFitsImage->SetImage( Info.Height, Info.Width, 32768, Image);
   ObjFitsImage->SetTemperature( InternalStatus.DomeTemp );
   ObjFitsImage->SetShotPar( &Info.InitShotTime , Info.Exposure);
   ObjFitsImage->SetPosition( &RA2000, &Dec2000, 2000);
   ObjFitsImage->SetObject(ObjectName);
   ObjFitsImage->SetObservatoryPar(Telescope, CcdName, Observers);
   ObjFitsImage->SaveFile(FileName);
   free ( Image );
   free ( Image2 );



   return ERR_NONE;
}



/****************************************************************************

   Create lists of sky objects

*****************************************************************************/

void TentaclesMgr::CreateSkyObjectsLists( void )
{
   bool                                Ok;                  // permission to continue
   SYSTEMTIME                          LocalTime;           // current local time
   CMEvent::DATE                       Date;                // current local time for event object
   CMEvent                             *Event;              // event object
   double                              CurrentJD;           // current julian day
   double                              J2000;               // 2000 epoch julian day
   char                                FileName[MAXPATH];   // file name
   HANDLE                              hCatalog;            // catalog file handle
   char                                Message[200];        // error message
   BY_HANDLE_FILE_INFORMATION          Info;                // file information
   DWORD                               ToRead;              // bytes to read
   DWORD                               Read;                // bytes read
   char                                *Buffer;             // file buffer
   CMCoEquatorial                      *CatalogPos;         // object catalog position
   CMHours::HOURS                      RA;                  // object RA
   CMDegree::DEGREE                    Dec;                 // object Dec
   char                                Bayer[20];           // catalog name
   char                                Const[20];           // constllation
   char                                OwnName[20];         // object own Name
   char                                ObjectType[20];      // NGC object type
   char                                Temp[20];            // temporary string
   float                               RAProperM;           // proper motion in RA
   float                               DecProperM;          // proper motion in Dec
   float                               ApparentSize;        // object apparent size
   float                               Magnitude;           // object apparent magnitude
   char                                *Scan;               // scan pointer in the buffer
   CMStar                              *Star;               // object star
   CMMoon                              *Moon;               // object Moon
   CMPlanet                            *Planet;             // object planet
   CMPlanet                            *Earth;              // object Earth for point of view
   CMDeepSky                           *DeepSky;            // deep sky object
   char                                ObjInfo[1000];       // object information
   char                                *pObjInfoWrite;      // pointer for object info writing
   char                                *pObjNameWrite;      // pointer for object name writing
   CMDeepSky::TYPE                     Type;                // object type code
   bool                                MessierFlag;         // Messier name is scanning

   // create needed objects
   Event = new CMEvent();                                   // event object
   CatalogPos = new CMCoEquatorial();                       // object with catalog object position

   // set event object with current data and time
   GetSystemTime ( &LocalTime );                            // get PC time
   Date.Year = LocalTime.wYear;                             // set event structure
   Date.Month = LocalTime.wMonth;
   Date.Day = LocalTime.wDay;
   Date.Hour = LocalTime.wHour;
   Date.Minute = LocalTime.wMinute;
   Date.Sec = LocalTime.wSecond;
   Event->SetDate( &Date );                                 // set event
   CurrentJD = Event->GetGG();                              // get JD

   // set event object with J2000 data and time
   Date.Year = 2000;                                        // set event structure
   Date.Month = 1;
   Date.Day = 1;
   Date.Hour = 0;
   Date.Minute = 0;
   Date.Sec = 0;
   Event->SetDate( &Date );                                 // set event
   J2000 = Event->GetGG();                                  // get JD

   //
   // create the list of planets
   //
   PlanetList = new CMObjectList();

   Moon = new CMMoon( );                                    // create moon object
   Moon->SetData(CurrentJD);                                // set actual date
   PlanetList->Add(Moon);                                   // add to list

   Planet   = new CMPlanet(CMPlanet::SUN);                  // create Sun
   Earth    = new CMPlanet(CMPlanet::EARTH );               // create Earth
   Planet->SetPlanetView(Earth);                            // Sun seen from Earth
   Planet->SetOpt( CMPlanet::MEEUS,true, true);             // set Meeus computing, nutation and time light
   Planet->SetData(CurrentJD);                              // set current time
   PlanetList->Add(Planet);                                 // add to list

   Planet   = new CMPlanet(CMPlanet::MERCURY);              // create Mercury
   Earth    = new CMPlanet(CMPlanet::EARTH );               // create Earth
   Planet->SetPlanetView(Earth);                            // Mercury seen from Earth
   Planet->SetOpt( CMPlanet::MEEUS,true, true);             // set Meeus computing, nutation and time light
   Planet->SetData(CurrentJD);                              // set current time
   PlanetList->Add(Planet);                                 // add to list

   Planet   = new CMPlanet(CMPlanet::VENUS);                // create Venus
   Earth    = new CMPlanet(CMPlanet::EARTH );               // create Earth
   Planet->SetPlanetView(Earth);                            // Venus seen from Earth
   Planet->SetOpt( CMPlanet::MEEUS,true, true);             // set Meeus computing, nutation and time light
   Planet->SetData(CurrentJD);                              // set current time
   PlanetList->Add(Planet);                                 // add to list

   Planet   = new CMPlanet(CMPlanet::MARS);                 // create Mars
   Earth    = new CMPlanet(CMPlanet::EARTH );               // create Earth
   Planet->SetPlanetView(Earth);                            // Mars seen from Earth
   Planet->SetOpt( CMPlanet::MEEUS,true, true);             // set Meeus computing, nutation and time light
   Planet->SetData(CurrentJD);                              // set current time
   PlanetList->Add(Planet);                                 // add to list

   Planet   = new CMPlanet(CMPlanet::JUPITER);              // create Jupiter
   Earth    = new CMPlanet(CMPlanet::EARTH );               // create Earth
   Planet->SetPlanetView(Earth);                            // Jupiter seen from Earth
   Planet->SetOpt( CMPlanet::MEEUS,true, true);             // set Meeus computing, nutation and time light
   Planet->SetData(CurrentJD);                              // set current time
   PlanetList->Add(Planet);                                 // add to list

   Planet   = new CMPlanet(CMPlanet::SATURN);               // create Saturn
   Earth    = new CMPlanet(CMPlanet::EARTH );               // create Earth
   Planet->SetPlanetView(Earth);                            // Saturn seen from Earth
   Planet->SetOpt( CMPlanet::MEEUS,true, true);             // set Meeus computing, nutation and time light
   Planet->SetData(CurrentJD);                              // set current time
   PlanetList->Add(Planet);                                 // add to list

   Planet   = new CMPlanet(CMPlanet::URAN);                 // create Uran
   Earth    = new CMPlanet(CMPlanet::EARTH );               // create Earth
   Planet->SetPlanetView(Earth);                            // Uran seen from Earth
   Planet->SetOpt( CMPlanet::MEEUS,true, true);             // set Meeus computing, nutation and time light
   Planet->SetData(CurrentJD);                              // set current time
   PlanetList->Add(Planet);                                 // add to list

   Planet   = new CMPlanet(CMPlanet::NEPTUN);               // create Neptun
   Earth    = new CMPlanet(CMPlanet::EARTH );               // create Earth
   Planet->SetPlanetView(Earth);                            // Neptun seen from Earth
   Planet->SetOpt( CMPlanet::MEEUS,true, true);             // set Meeus computing, nutation and time light
   Planet->SetData(CurrentJD);                              // set current time
   PlanetList->Add(Planet);                                 // add to list

   //
   // create the list of reference stars
   //
   RefStarsList = new CMObjectList();

   // load reference stars file
   Ok = true;
   strcpy( FileName, ObjectsFolder );                       // path for objects catalog
   strcat( FileName, "AllignStars.txt" );                   // file name
   hCatalog = CreateFile( FileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL );  // open file
   if ( hCatalog == INVALID_HANDLE_VALUE )                  // if open fails show error dialog
   {
      strcpy( Message, "Catalogo stelle di riferimento non trovato:\n" );
      strcat( Message, FileName );
      MessageBox( 0, Message, "Errore", 0);
      Ok = false;
   }
   if ( Ok == true )
   {
      GetFileInformationByHandle( hCatalog, &Info );        // get information for buffer size allocation
      ToRead = Info.nFileSizeLow;                           // file lenght
      Buffer = (char*) malloc( ToRead );                    // allocate buffer for all file
      if ( Buffer == 0 )                                    // if not enought memory show error dialog
      {
         strcpy( Message, "Memoria insufficiente per caricare catalogo:\n" );
         strcat( Message, FileName );
         MessageBox( 0, Message, "Errore ", 0);
         CloseHandle( hCatalog);                            // close file
         Ok = false;
      }
   }
   if ( Ok == true )
   {
      ReadFile ( hCatalog, Buffer, ToRead, &Read, NULL);    // read all file in one block
      CloseHandle( hCatalog);                               // close file

      // Scan buffer extracting region parameters
      Scan = Buffer;                                        // scan pointer at buffer begining
      while ( Read > 0 )
      {
         // scan ascii parameters in the buffer
         sscanf( Scan, "%d %d %f %s %d %d %f %s %s", &RA.hh, &RA.mm, &RA.ss,
                                                     &Dec.sign, &Dec.gg, &Dec.mm, &Dec.ss,
                                                     &Bayer, &Const);
         *OwnName = 0;                                      // default name
         if ( Scan[38] != ' ' )                             // own name is not always present
            sscanf( Scan+38, "%s", &OwnName);               // if present load it
         sscanf( Scan+49, "%f %f", &RAProperM, &DecProperM );  // scan proper motion parameters

         CatalogPos->GetRA()->Set( &RA );                   // set catalog right ascension
         CatalogPos->GetDec()->Set( &Dec);                  // set declination
         CatalogPos->SetDataEqu(J2000);                     // set equinotion date
         Star = new CMStar();                               // create new star object
         Star->SetName( OwnName, CMObject::NAME_OWN );      // own name assignment
         Star->SetName( Const, CMObject::NAME_COST );       // constellation assignment
         strcat( Bayer, " " );                              // create catalog name
         strcat( Bayer, Const );
         Star->SetName( Bayer, CMObject::NAME_CATALOG );    // catalog name assignment
         Star->SetBase(J2000, CatalogPos, RAProperM/1000, DecProperM/1000);      // set ephemerid data and proper motion
         Star->SetData(CurrentJD);                          // set current date
         RefStarsList->Add(Star);                           // add star to list

         // search next new line character
         while ( *Scan != '\n' && Read > 0 )
         {
            Read--;
            Scan++;
         }

         // bypass new line character
         Read--;
         Scan++;
      }
      free (Buffer);
   }

   //
   // create the list of minor planet
   //
   MPlanetsList = new CMObjectList();

   //
   // create the list of multiple stars
   //
   MultipleStarList = new CMObjectList();

   // load reference stars file
   Ok = true;
   strcpy( FileName, ObjectsFolder );                       // path for objects catalog
   strcat( FileName, "MultipleStars.txt" );                 // file name
   hCatalog = CreateFile( FileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL );  // open file
   if ( hCatalog == INVALID_HANDLE_VALUE )                  // if open fails show error dialog
   {
      strcpy( Message, "Catalogo stelle multiple non trovato:\n" );
      strcat( Message, FileName );
      MessageBox( 0, Message, "Errore", 0);
      Ok = false;
   }
   if ( Ok == true )
   {
      GetFileInformationByHandle( hCatalog, &Info );        // get information for buffer size allocation
      ToRead = Info.nFileSizeLow;                           // file lenght
      Buffer = (char*) malloc( ToRead );                    // allocate buffer for all file
      if ( Buffer == 0 )                                    // if not enought memory show error dialog
      {
         strcpy( Message, "Memoria insufficiente per caricare catalogo:\n" );
         strcat( Message, FileName );
         MessageBox( 0, Message, "Errore ", 0);
         CloseHandle( hCatalog);                            // close file
         Ok = false;
      }
   }
   if ( Ok == true )
   {
      ReadFile ( hCatalog, Buffer, ToRead, &Read, NULL);    // read all file in one block
      CloseHandle( hCatalog);                               // close file

      // Scan buffer extracting region parameters
      Scan = Buffer;                                        // scan pointer at buffer begining
      while ( Read > 0 )
      {
         // scan ascii parameters in the buffer
         sscanf( Scan, "%d %d %f %s %d %d %f %s %s", &RA.hh, &RA.mm, &RA.ss,
                                                     &Dec.sign, &Dec.gg, &Dec.mm, &Dec.ss,
                                                     &Bayer, &Const);
         *OwnName = 0;                                      // default name
         if ( Scan[38] != ' ' )                             // own name is not always present
            sscanf( Scan+38, "%s", &OwnName);               // if present load it
         sscanf( Scan+49, "%f %f", &RAProperM, &DecProperM );  // scan proper motion parameters

         CatalogPos->GetRA()->Set( &RA );                   // set catalog right ascension
         CatalogPos->GetDec()->Set( &Dec);                  // set declination
         CatalogPos->SetDataEqu(J2000);                     // set equinotion date
         Star = new CMStar();                               // create new star object
         Star->SetName( OwnName, CMObject::NAME_OWN );      // own name assignment
         Star->SetName( Const, CMObject::NAME_COST );       // constellation assignment
         strcat( Bayer, " " );                              // create catalog name
         strcat( Bayer, Const );
         Star->SetName( Bayer, CMObject::NAME_CATALOG );    // catalog name assignment
         Star->SetBase(J2000, CatalogPos, RAProperM/1000, DecProperM/1000);      // set ephemerid data and proper motion
         Star->SetData(CurrentJD);                          // set current date

         // get object information text from column 69
         if ( Read > 68 )                                   // should be enought file
         {
            Scan += 68;                                     // skip previous columns
            Read -= 68;                                     // reduce file lenght
            pObjInfoWrite = ObjInfo;                        // pointr to destination info text
            while ( *Scan != '\n' && *Scan != '"' && Read > 0 ) // run unitl end of text or end of line or end of file
            {
               if ( *Scan == '\\' && *(Scan+1) == 'n' )     // mark for new line ?
               {
                  *pObjInfoWrite++ = '\r';                  // write return
                  *pObjInfoWrite++ = '\n';                  // write line feed
                  Scan += 2;                                // skip 2 characters
                  Read -= 2;                                // reduce file lenght
               }
               else
               {
                  *pObjInfoWrite++ = *Scan;                 // copy characters
                  Scan++;                                   // skip 1 character
                  Read--;                                   // reduce file lenght
               }
           }
            *pObjInfoWrite = 0;                             // set line terminator
         }
         Star->SetInfo(ObjInfo);                            // set object information 

         MultipleStarList->Add(Star);                       // add star to list

         // search next new line character
         while ( *Scan != '\n' && Read > 0 )
         {
            Read--;
            Scan++;
         }

         // bypass new line character
         Read--;
         Scan++;
      }
      free (Buffer);
   }

   //
   // create the list of cluster
   //
   ClustersList = new CMObjectList();

   // load cluster file
   Ok = true;
   strcpy( FileName, ObjectsFolder );                       // path for objects catalog
   strcat( FileName, "Clusters.txt" );                      // file name
   hCatalog = CreateFile( FileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL );  // open file
   if ( hCatalog == INVALID_HANDLE_VALUE )                  // if open fails show error dialog
   {
      strcpy( Message, "Catalogo ammassi non trovato:\n" );
      strcat( Message, FileName );
      MessageBox( 0, Message, "Errore", 0);
      Ok = false;
   }
   if ( Ok == true )
   {
      GetFileInformationByHandle( hCatalog, &Info );        // get information for buffer size allocation
      ToRead = Info.nFileSizeLow;                           // file lenght
      Buffer = (char*) malloc( ToRead );                    // allocate buffer for all file
      if ( Buffer == 0 )                                    // if not enought memory show error dialog
      {
         strcpy( Message, "Memoria insufficiente per caricare catalogo:\n" );
         strcat( Message, FileName );
         MessageBox( 0, Message, "Errore ", 0);
         CloseHandle( hCatalog);                            // close file
         Ok = false;
      }
   }
   if ( Ok == true )
   {
      ReadFile ( hCatalog, Buffer, ToRead, &Read, NULL);    // read all file in one block
      CloseHandle( hCatalog);                               // close file

      // Scan buffer extracting region parameters
      Scan = Buffer;                                        // scan pointer at buffer begining
      while ( Read > 0 )
      {
         // scan ascii parameters in the buffer
         sscanf( Scan, "%d %d %f %s %d %d %f %s %s", &RA.hh, &RA.mm, &RA.ss,
                                                     &Dec.sign, &Dec.gg, &Dec.mm, &Dec.ss,
                                                     &Bayer, &Const);
         *OwnName = 0;                                      // default name
         if ( Scan[38] != ' ' )                             // own name is not always present
            sscanf( Scan+38, "%s", &OwnName);               // if present load it

         CatalogPos->GetRA()->Set( &RA );                   // set catalog right ascension
         CatalogPos->GetDec()->Set( &Dec);                  // set declination
         CatalogPos->SetDataEqu(J2000);                     // set equinotion date
         Star = new CMStar();                               // create new star object
         Star->SetName( OwnName, CMObject::NAME_OWN );      // own name assignment
         Star->SetName( Const, CMObject::NAME_COST );       // constellation assignment
         Star->SetName( Bayer, CMObject::NAME_CATALOG );    // catalog name assignment
         Star->SetBase(J2000, CatalogPos, 0, 0);            // set ephemerid data
         Star->SetData(CurrentJD);                          // set current date

         // get object information text from column 52
         if ( Read > 51 )                                   // should be enought file
         {
            Scan += 51;                                     // skip previous columns
            Read -= 51;                                     // reduce file lenght
            pObjInfoWrite = ObjInfo;                        // pointr to destination info text
            while ( *Scan != '\n' && *Scan != '"' && Read > 0 ) // run unitl end of text or end of line or end of file
            {
               if ( *Scan == '\\' && *(Scan+1) == 'n' )     // mark for new line ?
               {
                  *pObjInfoWrite++ = '\r';                  // write return
                  *pObjInfoWrite++ = '\n';                  // write line feed
                  Scan += 2;                                // skip 2 characters
                  Read -= 2;                                // reduce file lenght
               }
               else
               {
                  *pObjInfoWrite++ = *Scan;                 // copy characters
                  Scan++;                                   // skip 1 character
                  Read--;                                   // reduce file lenght
               }
           }
            *pObjInfoWrite = 0;                             // set line terminator
         }
         Star->SetInfo(ObjInfo);                            // set object information 

         ClustersList->Add(Star);                           // add cluster to list

         // search next new line character
         while ( *Scan != '\n' && Read > 0 )
         {
            Read--;
            Scan++;
         }

         // bypass new line character
         Read--;
         Scan++;
      }
      free (Buffer);
   }

   //
   // create the list of nebule
   //
   NebuleList = new CMObjectList();

   // load nebule file
   Ok = true;
   strcpy( FileName, ObjectsFolder );                       // path for objects catalog
   strcat( FileName, "Nebulas.txt" );                       // file name
   hCatalog = CreateFile( FileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL );  // open file
   if ( hCatalog == INVALID_HANDLE_VALUE )                  // if open fails show error dialog
   {
      strcpy( Message, "Catalogo nebulose non trovato:\n" );
      strcat( Message, FileName );
      MessageBox( 0, Message, "Errore", 0);
      Ok = false;
   }
   if ( Ok == true )
   {
      GetFileInformationByHandle( hCatalog, &Info );        // get information for buffer size allocation
      ToRead = Info.nFileSizeLow;                           // file lenght
      Buffer = (char*) malloc( ToRead );                    // allocate buffer for all file
      if ( Buffer == 0 )                                    // if not enought memory show error dialog
      {
         strcpy( Message, "Memoria insufficiente per caricare catalogo:\n" );
         strcat( Message, FileName );
         MessageBox( 0, Message, "Errore ", 0);
         CloseHandle( hCatalog);                            // close file
         Ok = false;
      }
   }
   if ( Ok == true )
   {
      ReadFile ( hCatalog, Buffer, ToRead, &Read, NULL);    // read all file in one block
      CloseHandle( hCatalog);                               // close file

      // Scan buffer extracting region parameters
      Scan = Buffer;                                        // scan pointer at buffer begining
      while ( Read > 0 )
      {
         // scan ascii parameters in the buffer
         sscanf( Scan, "%d %d %f %s %d %d %f %s %s", &RA.hh, &RA.mm, &RA.ss,
                                                     &Dec.sign, &Dec.gg, &Dec.mm, &Dec.ss,
                                                     &Bayer, &Const);
         *OwnName = 0;                                      // default name
         if ( Scan[38] != ' ' )                             // own name is not always present
            sscanf( Scan+38, "%s", &OwnName);               // if present load it

         CatalogPos->GetRA()->Set( &RA );                   // set catalog right ascension
         CatalogPos->GetDec()->Set( &Dec);                  // set declination
         CatalogPos->SetDataEqu(J2000);                     // set equinotion date
         Star = new CMStar();                               // create new star object
         Star->SetName( OwnName, CMObject::NAME_OWN );      // own name assignment
         Star->SetName( Const, CMObject::NAME_COST );       // constellation assignment
         Star->SetName( Bayer, CMObject::NAME_CATALOG );    // catalog name assignment
         Star->SetBase(J2000, CatalogPos, 0, 0);            // set ephemerid data
         Star->SetData(CurrentJD);                          // set current date

         // get object information text from column 53
         if ( Read > 52 )                                   // should be enought file
         {
            Scan += 52;                                     // skip previous columns
            Read -= 52;                                     // reduce file lenght
            pObjInfoWrite = ObjInfo;                        // pointr to destination info text
            while ( *Scan != '\n' && *Scan != '"' && Read > 0 ) // run unitl end of text or end of line or end of file
            {
               if ( *Scan == '\\' && *(Scan+1) == 'n' )     // mark for new line ?
               {
                  *pObjInfoWrite++ = '\r';                  // write return
                  *pObjInfoWrite++ = '\n';                  // write line feed
                  Scan += 2;                                // skip 2 characters
                  Read -= 2;                                // reduce file lenght
               }
               else
               {
                  *pObjInfoWrite++ = *Scan;                 // copy characters
                  Scan++;                                   // skip 1 character
                  Read--;                                   // reduce file lenght
               }
           }
            *pObjInfoWrite = 0;                             // set line terminator
         }
         Star->SetInfo(ObjInfo);                            // set object information 

         NebuleList->Add(Star);                             // add nebula to list

         // search next new line character
         while ( *Scan != '\n' && Read > 0 )
         {
            Read--;
            Scan++;
         }

         // bypass new line character
         Read--;
         Scan++;
      }
      free (Buffer);
   }

   //
   // create the list of galaxies
   //
   GalaxiesList = new CMObjectList();

   // load reference stars file
   Ok = true;
   strcpy( FileName, ObjectsFolder );                       // path for objects catalog
   strcat( FileName, "Galaxies.txt" );                      // file name
   hCatalog = CreateFile( FileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL );  // open file
   if ( hCatalog == INVALID_HANDLE_VALUE )                  // if open fails show error dialog
   {
      strcpy( Message, "Catalogo galassie non trovato:\n" );
      strcat( Message, FileName );
      MessageBox( 0, Message, "Errore", 0);
      Ok = false;
   }
   if ( Ok == true )
   {
      GetFileInformationByHandle( hCatalog, &Info );        // get information for buffer size allocation
      ToRead = Info.nFileSizeLow;                           // file lenght
      Buffer = (char*) malloc( ToRead );                    // allocate buffer for all file
      if ( Buffer == 0 )                                    // if not enought memory show error dialog
      {
         strcpy( Message, "Memoria insufficiente per caricare catalogo:\n" );
         strcat( Message, FileName );
         MessageBox( 0, Message, "Errore ", 0);
         CloseHandle( hCatalog);                            // close file
         Ok = false;
      }
   }
   if ( Ok == true )
   {
      ReadFile ( hCatalog, Buffer, ToRead, &Read, NULL);    // read all file in one block
      CloseHandle( hCatalog);                               // close file

      // Scan buffer extracting region parameters
      Scan = Buffer;                                        // scan pointer at buffer begining
      while ( Read > 0 )
      {
         // scan ascii parameters in the buffer
         sscanf( Scan, "%d %d %f %s %d %d %f %s %s", &RA.hh, &RA.mm, &RA.ss,
                                                     &Dec.sign, &Dec.gg, &Dec.mm, &Dec.ss,
                                                     &Bayer, &Const);
         *OwnName = 0;                                      // default name
         if ( Scan[38] != ' ' )                             // own name is not always present
            sscanf( Scan+38, "%s", &OwnName);               // if present load it

         CatalogPos->GetRA()->Set( &RA );                   // set catalog right ascension
         CatalogPos->GetDec()->Set( &Dec);                  // set declination
         CatalogPos->SetDataEqu(J2000);                     // set equinotion date
         Star = new CMStar();                               // create new star object
         Star->SetName( OwnName, CMObject::NAME_OWN );      // own name assignment
         Star->SetName( Const, CMObject::NAME_COST );       // constellation assignment
         Star->SetName( Bayer, CMObject::NAME_CATALOG );    // catalog name assignment
         Star->SetBase(J2000, CatalogPos, 0, 0);            // set ephemerid data
         Star->SetData(CurrentJD);                          // set current date

         // get object information text from column 52
         if ( Read > 51 )                                   // should be enought file
         {
            Scan += 51;                                     // skip previous columns
            Read -= 51;                                     // reduce file lenght
            pObjInfoWrite = ObjInfo;                        // pointr to destination info text
            while ( *Scan != '\n' && *Scan != '"' && Read > 0 ) // run unitl end of text or end of line or end of file
            {
               if ( *Scan == '\\' && *(Scan+1) == 'n' )     // mark for new line ?
               {
                  *pObjInfoWrite++ = '\r';                  // write return
                  *pObjInfoWrite++ = '\n';                  // write line feed
                  Scan += 2;                                // skip 2 characters
                  Read -= 2;                                // reduce file lenght
               }
               else
               {
                  *pObjInfoWrite++ = *Scan;                 // copy characters
                  Scan++;                                   // skip 1 character
                  Read--;                                   // reduce file lenght
               }
           }
            *pObjInfoWrite = 0;                             // set line terminator
         }
         Star->SetInfo(ObjInfo);                            // set object information 

         GalaxiesList->Add(Star);                           // add galaxy to list

         // search next new line character
         while ( *Scan != '\n' && Read > 0 )
         {
            Read--;
            Scan++;
         }

         // bypass new line character
         Read--;
         Scan++;
      }
      free (Buffer);
   }

   //
   // create the list of NGC objects
   //
   NGCList = new CMObjectList();

   // load NGC catalog file
   Ok = true;
   strcpy( FileName, ObjectsFolder );                       // path for objects catalog
   strcat( FileName, "NGC2000.DAT" );                      // file name
   hCatalog = CreateFile( FileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL );  // open file
   if ( hCatalog == INVALID_HANDLE_VALUE )                  // if open fails show error dialog
   {
      strcpy( Message, "Catalogo NGC non trovato:\n" );
      strcat( Message, FileName );
      MessageBox( 0, Message, "Errore", 0);
      Ok = false;
   }
   if ( Ok == true )
   {
      GetFileInformationByHandle( hCatalog, &Info );        // get information for buffer size allocation
      ToRead = Info.nFileSizeLow;                           // file lenght
      Buffer = (char*) malloc( ToRead );                    // allocate buffer for all file
      if ( Buffer == 0 )                                    // if not enought memory show error dialog
      {
         strcpy( Message, "Memoria insufficiente per caricare catalogo:\n" );
         strcat( Message, FileName );
         MessageBox( 0, Message, "Errore ", 0);
         CloseHandle( hCatalog);                            // close file
         Ok = false;
      }
   }
   if ( Ok == true )
   {
      ReadFile ( hCatalog, Buffer, ToRead, &Read, NULL);    // read all file in one block
      CloseHandle( hCatalog);                               // close file

      // Scan buffer extracting region parameters
      Scan = Buffer;                                        // scan pointer at buffer begining
      while ( Read > 0 )
      {
         *ObjInfo = 0;                                      // erase information
         *OwnName = 0;                                      // erase Messier name
         MessierFlag = false;                               // no Messier name is scanning

         GetString ( Scan, Bayer, 3 );                      // get catalog name
         if ( strcmp( Bayer, "IC" ) == 0 )                  // is IC catalog ?
            GetString ( Scan+3, Temp, 4 );                  // get catalog number
         else
            GetString ( Scan+4, Temp, 4 );                  // get catalog number
         strcat( Bayer, Temp);                              // compose catalog name object

         // get right ascension
         GetString ( Scan+13, Temp, 2 );                    // get hh
         RA.hh = atoi( Temp );
         GetString ( Scan+16, Temp, 2 );                    // get mm
         RA.mm = atoi( Temp );
         GetString ( Scan+19, Temp, 1 );                    // get mm 1/10
         RA.ss = atoi( Temp ) * 6;

         // get declination
         GetString ( Scan+22, Dec.sign, 1 );                // get sign
         GetString ( Scan+23, Temp, 2 );                    // get gg
         Dec.gg = atoi( Temp );
         GetString ( Scan+26, Temp, 2 );                    // get mm
         Dec.mm = atoi( Temp );
         Dec.ss = 0;

         // get other object parameters
         GetString ( Scan+9, ObjectType, 3 );               // get object type
         GetString ( Scan+32, Const, 3 );                   // get constellation
         GetString ( Scan+36, Temp, 5 );                    // get apparent size
         ApparentSize = atof( Temp );
         GetString ( Scan+43, Temp, 4 );                    // get magnitude
         Magnitude = atof( Temp );

         // type coding
         Type = CMDeepSky::NOTDEFINED;
         if ( strcmp(ObjectType, "Gx" ) == 0 )
            Type = CMDeepSky::GALAXY;
         if ( strcmp(ObjectType, "OC" ) == 0 )
            Type = CMDeepSky::OPENCLASTER;
         if ( strcmp(ObjectType, "Gb" ) == 0 )
            Type = CMDeepSky::GLOBULARCLUSTER;
         if ( strcmp(ObjectType, "Nb" ) == 0 )
            Type = CMDeepSky::NEBULA;
         if ( strcmp(ObjectType, "Pl" ) == 0 )
            Type = CMDeepSky::PLANETARYNEBULLA;
         if ( strcmp(ObjectType, "C+N" ) == 0 )
            Type = CMDeepSky::NEBULACLUSTER;
         if ( strcmp(ObjectType, "Ast" ) == 0 )
            Type = CMDeepSky::ASTERISM;
         if ( strcmp(ObjectType, "Kt" ) == 0 )
            Type = CMDeepSky::KNOT;
         if ( strcmp(ObjectType, "***" ) == 0 )
            Type = CMDeepSky::TRIPLESTAR;
         if ( strcmp(ObjectType, "D*" ) == 0 )
            Type = CMDeepSky::DOUBLESTAR;
         if ( strcmp(ObjectType, "*" ) == 0 )
            Type = CMDeepSky::SINGLESTAR;

         // get object information text from column 50
         if ( Read > 49 )                                   // should be enought file
         {
            Scan += 49;                                     // skip previous columns
            Read -= 49;                                     // reduce file lenght
            pObjInfoWrite = ObjInfo;                        // pointer to destination info text
            pObjNameWrite = OwnName;                        // pointer to Messier name
            while ( *Scan != '\n' && *Scan != '"' && Read > 0 ) // run unitl end of text or end of line or end of file
            {
               if ( *Scan == '\\' && *(Scan+1) == 'n' )     // mark for new line ?
               {
                  *pObjInfoWrite++ = '\r';                  // write return
                  *pObjInfoWrite++ = '\n';                  // write line feed
                  Scan += 2;                                // skip 2 characters
                  Read -= 2;                                // reduce file lenght
               }
               else
               {
                  *pObjInfoWrite++ = *Scan;                 // copy characters
                  if ( *Scan == ' ' && pObjNameWrite != OwnName )
                     MessierFlag = false;                   // Messier name disactivation
                  if ( *Scan != ' ' && MessierFlag == true )// Messier name ?
                     *pObjNameWrite++ = *Scan;              // copy characters
                  if ( *Scan == '=' && *(Scan+2) == 'M' )   // Messier name identification
                     MessierFlag = true;                    // Messier name acivation
                  Scan++;                                   // skip 1 character
                  Read--;                                   // reduce file lenght
               }
           }
            *pObjInfoWrite = 0;                             // set line terminator
            *pObjNameWrite = 0;                             // set line terminator
         }

         CatalogPos->GetRA()->Set( &RA );                   // set catalog right ascension
         CatalogPos->GetDec()->Set( &Dec);                  // set declination
         CatalogPos->SetDataEqu(J2000);                     // set equinotion date
         DeepSky = new CMDeepSky();                         // create new deep sky object
         DeepSky->SetName( OwnName, CMObject::NAME_OWN );   // own name assignment
         DeepSky->SetName( Const, CMObject::NAME_COST );    // constellation assignment
         DeepSky->SetName( Bayer, CMObject::NAME_CATALOG ); // catalog name assignment
         DeepSky->SetBase(J2000, CatalogPos);               // set ephemerid data
         DeepSky->SetData(CurrentJD);                       // set current date
         DeepSky->SetMag(Magnitude);                        // set apparent magnitude
         DeepSky->SetType(Type);                            // set object size
         DeepSky->SetSize(ApparentSize);                    // set apparent size
         DeepSky->SetInfo(ObjInfo);                         // set object information

         NGCList->Add(DeepSky);                             // add object to list

         // search next new line character
         while ( *Scan != '\n' && Read > 0 )
         {
            Read--;
            Scan++;
         }

         // bypass new line character
         Read--;
         Scan++;
      }
      free (Buffer);
   }


   // destoy objects
   delete Event;
   delete CatalogPos;
}



/****************************************************************************

   enable/disable safe algorithm to determinate claud sky

*****************************************************************************/

void TentaclesMgr::SetSafeAlgorithm( bool PSafeEnable, double PSafeThreshold, int PSafeTime)
{
   // save input parameters
   SafeEnable = PSafeEnable;
   SafeThreshold = PSafeThreshold;
   SafeTime = PSafeTime;

   // if disable reset timer
   if ( SafeEnable == false )
      SafeTimerStarted = false;
   Notify( 0, NOTIFY_SAFEALGORITM );
}



/****************************************************************************

   Add minor planet to list

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::AddMinorPlanet( char *MinorPlanetCode )
{
   char                                SearchDesign[10];
   int                                 Len;
   bool                                Found;               // element found
   bool                                Ok;                  // permission to continue
   char                                FileName[MAXPATH];   // file name
   HANDLE                              hCatalog;            // catalog file handle
   char                                Message[200];        // error message
   BY_HANDLE_FILE_INFORMATION          Info;                // file information
   int                                 FileLen;             // file lenght
   int                                 FileOffset;          // file reading offset
   int                                 BufferSize;          // buffer size
   int                                 ToRead;              // bytes to read
   DWORD                               Read;                // bytes read
   char                                *Buffer;             // file buffer
   char                                *Scan;               // scan pointer in the buffer
   CMMPlanet                           *MinorPlanet;        // object planet
   CMPlanet                            *Earth;              // object Earth for point of view
   char                                Design[10];          // minor planet number
   char                                Design2[21];         // minor planet number
   char                                Design3[21];         // minor planet number
   char                                H[10];               // absoulute magnitude
   char                                G[10];               // slope parameter
   char                                Epoch[10];           // orbit parameter epoch
   char                                M0[15];              // mean anomaly
   char                                p[15];               // perielion;
   char                                n[15];               // node
   char                                i[15];               // inclination
   char                                e[15];               // eccentricity
   char                                a[15];               // semiaxis
   char                                dm[15];              // daily motion
   bool                                First;
   MEEUS_ELE_ORB                       EleOrb;
   double                              JEph;
   double                              CurrentJD;           // current julian day
   char                                year[3];
   char                                month[2];
   char                                day[2];
   int                                 Years;
   int                                 Months;
   int                                 Days;
   CMEvent::DATE                       Date;                // current local time for event object
   CMEvent                             *Event;              // event object
   SYSTEMTIME                          LocalTime;           // current local time
   CMObject*                           Obj;                 // generig object
   CMObject*                           ObjNext;                 // generig object

   strcpy( SearchDesign, "00000" );
   Len = strlen( MinorPlanetCode );
   strcpy( SearchDesign+5-Len, MinorPlanetCode );

   Ok = true;
   BufferSize = 100000;
   FileOffset = 0;
   First=true;
   Found=false;
   strcpy( FileName, ObjectsFolder );                       // path for objects catalog
   strcat( FileName, "MPCOrb.dat" );                        // file name
   hCatalog = CreateFile( FileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL );  // open file
   if ( hCatalog == INVALID_HANDLE_VALUE )                  // if open fails show error dialog
   {
      strcpy( Message, "Catalogo mpc non trovato:\n" );
      strcat( Message, FileName );
      MessageBox( 0, Message, "Errore", 0);
      Ok = false;
   }
   if ( Ok == true )
   {
      GetFileInformationByHandle( hCatalog, &Info );        // get information for file size
      FileLen = Info.nFileSizeLow;                          // file lenght
      Buffer = (char*) malloc( BufferSize );                // allocate buffer
      if ( Buffer == 0 )                                    // if not enought memory show error dialog
      {
         strcpy( Message, "Memoria insufficiente per allocare buffer lettura catalogo:\n" );
         strcat( Message, FileName );
         MessageBox( 0, Message, "Errore ", 0);
         CloseHandle( hCatalog);                            // close file
         Ok = false;
      }
   }

   if ( Ok == true )
   {
      while ( FileOffset < FileLen && Found == false)
      {
         ToRead = FileLen - FileOffset;
         if ( ToRead > BufferSize )
            ToRead = BufferSize;
         SetFilePointer( hCatalog, FileOffset, NULL, FILE_BEGIN );
         ReadFile ( hCatalog, Buffer, ToRead, &Read, NULL); // read all file in one block
         Scan = Buffer;                                     // scan pointer at buffer begining

         // bypass lines don't start with number (header file)
         while ( *Scan < '0' || *Scan > '9' && First == true)
         {
            // search next new line character
            while ( *Scan != '\n' && Read > 0 )
            {
               Read--;
               Scan++;
               FileOffset++;
            }

            // bypass new line character
            Read--;
            Scan++;
            FileOffset++;
         }

         First = false;

         // Scan buffer extracting region parameters
         while ( Read > 0 && (Scan-Buffer) < (BufferSize-1000) )
         {
            // scan ascii parameters in the buffer
            sscanf( Scan, "%s ", &Design);
            if ( strcmp(Design, SearchDesign) == 0 )
            {
               // scan ascii parameters in the buffer
               sscanf( Scan, "%s %s %s %s %s %s %s %s %s %s %s", &Design, &H, &G, &Epoch, &M0,
                                                           &p, &n, &i, &e, &dm, &a);
               sscanf( Scan+175, "%s ", &Design2);
               Found=true;
               break;
            }

            // search next new line character
            while ( *Scan != '\n' && Read > 0 )
            {
               Read--;
               Scan++;
               FileOffset++;
            }

            // bypass new line character
            Read--;
            Scan++;
            FileOffset++;
         }
      }
      free (Buffer);
      CloseHandle( hCatalog);                               // close file
   }

   if ( Found == true )
   {

      MPlanetsList->First();                                // reset internal cursor list
      ObjNext = MPlanetsList->Next();                       // get first object
      while (ObjNext != 0 )                                 // works only until object is avalable
      {
         Obj = ObjNext;                                     // star to draw
         ObjNext = (CMStar*) MPlanetsList->Next();          // prepare next star
         strcpy( Design3, Obj->GetName(CMObject::NAME_OWN ));
         if ( strcmp( Design3, Design2) == 0 )
            return ERR_ALREADYPRESENT;
      }

      // determinazione epoca
      switch (*Epoch)
      {
         case 'I':
            Years = 1800;
            break;
         case 'J':
            Years = 1900;
            break;
         case 'K':
            Years = 2000;
            break;
      }
      year[0] = *(Epoch + 1);
      year[1] = *(Epoch + 2);
      year[2] = 0;
      Years += strtol (year,0,10);
      month[0] = *(Epoch + 3);
      month[1] = 0;
      Months = strtol (month, 0, 16);
      day[0] = *(Epoch + 4);
      day[1] = 0;
      Days = strtol (day, 0, 31);

      Event = new CMEvent();                                // event object
      // set event object with current data and time
      GetSystemTime ( &LocalTime );                         // get PC time
      Date.Year = LocalTime.wYear;                          // set event structure
      Date.Month = LocalTime.wMonth;
      Date.Day = LocalTime.wDay;
      Date.Hour = LocalTime.wHour;
      Date.Minute = LocalTime.wMinute;
      Date.Sec = LocalTime.wSecond;
      Event->SetDate( &Date );                              // set event
      CurrentJD = Event->GetGG();                           // get JD

      // set event object with epoca elementi
      Date.Year = Years;                                    // set event structure
      Date.Month = Months;
      Date.Day = Days;
      Date.Hour = 0;
      Date.Minute = 0;
      Date.Sec = 0;
      Event->SetDate( &Date );                              // set event
      JEph = Event->GetGG();                                // get JD

      memset( &EleOrb, 0, sizeof(EleOrb));
      EleOrb.pi = atof( p );                                // perielion argument
      EleOrb.a =  atof(a);                                  // semi axis
      EleOrb.e =  atof(e);                                  // eccentricity
      EleOrb.i =  atof(i);                                  // inclination
      EleOrb.OMEGA =  atof(n);                              // ascending node longitude
      EleOrb.pi += EleOrb.OMEGA;                            // perielion del longitude
      EleOrb.pi = CM_DegreeReduce( EleOrb.pi );

      MinorPlanet   = new CMMPlanet();
      MinorPlanet->SetEleOrb( JEph, &EleOrb, atof(M0), atof(dm) );
      MinorPlanet->SetOpt( true, true);                     // set nutation and time light
      MinorPlanet->SetData(CurrentJD);                      // set current time
      MinorPlanet->SetMagBase(atof(H));                     // imposta magnitudine base
      Earth       = new CMPlanet(CMPlanet::EARTH2000 );     // create Earth
      MinorPlanet->SetPlanetView(Earth);                    // Sun seen from Earth
      MinorPlanet->SetName( Design, CMObject::NAME_CATALOG );  // own name assignment
      MinorPlanet->SetName( Design2, CMObject::NAME_OWN );  // own name assignment
      MPlanetsList->Add(MinorPlanet);                       // add star to list
      return ERR_NONE;
   }

   return ERR_NOTFOUND;
}



/****************************************************************************

   Remove minor planet from the list

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::RemoveMinorPlanet( char *MinorPlanetCode )
{
   char                                Code[21];            // scanning minor planet code
   CMObject*                           Obj;                 // generig object

   MPlanetsList->First();                                   // reset internal cursor list
   Obj = MPlanetsList->Next();                              // get first object
   while (Obj != 0 )                                        // works only until object is avalable
   {
      strcpy( Code, Obj->GetName(CMObject::NAME_CATALOG ));
      if ( strcmp( Code, MinorPlanetCode) == 0 )
      {
         MPlanetsList->Remove();
         return ERR_NONE;
      }
      Obj = MPlanetsList->Next();                           // goto next object
   }

   return ERR_NOTFOUND;
}



/****************************************************************************

   set GRB position

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::SetGRBPos( CMHours::HOURS *RA, CMDegree::DEGREE*Dec )
{
   GRBRA = *RA;
   GRBDec = *Dec;
   return ERR_NONE;
}



/****************************************************************************

   Position telescope on current GRB

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::TelescopeGRBPos( Observer *Sender)
{
   int                                 Ret;                 // communication retun value

   // check if this observer can run the command
   if ( ObserverOnObservatory != 0  && ObserverOnObservatory != Sender)
      return ERR_OBSERVATORYBUSY;
   if ( InternalStatus.TeleTracking == false )
      return ERR_NOTRACKING;

   Ret = ObjCom->TelescopeGoto( &GRBRA, &GRBDec );            // run positioning on LX200 object
   if ( Ret == 0 )
   {
      InternalStatus.TargetRA = GRBRA;                      // store target position
      InternalStatus.TargetDec = GRBDec;
      strcpy( InternalStatus.TargetObject, "GRB" );         // store object name
      Notify( Sender, NOTIFY_GRBTELESCOPEPOSITIONED );      // notify event
   }

   // if coomunication failed return error
   if ( Ret != 0 )
      return ERR_POS;

   return ERR_NONE;
}



/****************************************************************************

   exec telescope allignment from shot

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::ShotAllignment( Observer *Sender)
{
   char                                FileName[MAXPATH];   // allignment file name
   wchar_t                             UFileName[MAXPATH];  // allignment file name in unicode
   wchar_t                             UGSCResolutionFolder[MAXPATH];   // catalog directory in unicode
   TPlate                              *Plate;              // plate solver object
   bool                                Solved;              // plate solved
   double                              SolvedRA;            // solved right ascension
   double                              SolvedDec;           // solved declination
   CMHours::HOURS                      RA;                  // solved right ascension in CMHours::HOURS format
   CMDegree::DEGREE                    Dec;                 // solved declination in CMDegree::GRADI format
   SYSTEMTIME                          LocalTime;           // current data and time
   CMEvent::DATE                       Date;                // data for event object
   CMEvent                             *Event;              // event object
   double                              CurrentJD;           // julian day at current data and time
   double                              J2000;               // 2000 epoch julian day
   CMCoEquatorial                      *J200PlatePos;       // 2000 equinox enter plate position
   const CMCoEquatorial                *CurrentPlatePos;    // current equinox enter plate position
   int                                 Ret;                 // communication retun value
   char                                SavedObjectName[30]; // saved base object name

   // check if this observer can run the command
   if ( ObserverOnObservatory != 0  && ObserverOnObservatory != Sender)
      return ERR_OBSERVATORYBUSY;
   if ( InternalStatus.TeleTracking == false )
      return ERR_NOTRACKING;

   // save last short
   strcpy( SavedObjectName, ObjectName );                   // save current object name
   SetObjectName( TMGR_LASTSHORT );                         // set fix name for shot save
   Ret = SaveFile( ImageFolder, 0 );                        // save shot in program folder
   SetObjectName( SavedObjectName );                        // restore current object name

   // create file name for plate solver
   strcpy( FileName, ImageFolder);                          // filename start with folder
   strcat( FileName, TMGR_LASTSHORT );                      // add last short name
   strcat( FileName, ".fit" );                              // add file extension
   MultiByteToWideChar( CP_ACP, 0, FileName, -1, UFileName, sizeof(UFileName) ); // convert name in unicode

   // create and configure plate resolver
   Plate = new TPlate(0);                                   // object creation without owner
   Plate->AttachFITS( UFileName );                          // assign last short
   Plate->set_Catalog(ppGSC_ACT);                           // Assign Guide Star 1.1 catalog, adjusted within PinPoint by corrections from the ACT catalog, as implemented by Bill Gray of Project Pluto.
   MultiByteToWideChar( CP_ACP, 0, GSCResolutionFolder, -1, UGSCResolutionFolder, sizeof(UGSCResolutionFolder) ); // convert folder in unicode
   Plate->set_CatalogPath(UGSCResolutionFolder);            // assign catalog path
   Plate->set_CatalogExpansion( 0.5 );                      // assign search tollerance in frame size
   Plate->set_CatalogMaximumMagnitude( MaxMagResolution );  // assign maximum magnitude from catalog
   Plate->set_CentroidAlgorithm ( ppAperture);              // assign algorithm to aperture and annulus
   Plate->set_ArcsecPerPixelHoriz ( PixelScale );           // assign horizontal pixel scale (arcsec/pixel)
   Plate->set_ArcsecPerPixelVert ( PixelScale );            // assign vertical pixel scale (arcsec/pixel)
   Plate->set_ColorBand ( ppRBand);                         // assign the nearest band color image CCD is more sensible in R band
   Plate->set_ProjectionType ( ppTAN );                     // assign the gnomonic projection (tangent plane)
   Plate->set_MaxSolveTime ( 5);                            // assign maximum plate solving time (sec.)
   Plate->set_SigmaAboveMean( 3.0);                         // assign sigma for object detection
   Plate->set_RightAscension(Plate->TargetRightAscension);  // assign plate right ascension as FITS target right ascension
   Plate->set_Declination( Plate->TargetDeclination);       // assign plate declination as FITS target declination

   // plate solving
   try
   {
      Solved = Plate->Solve();
      SolvedRA = Plate->RightAscension;                     // get solved coordinate J2000 based in double format
      SolvedDec = Plate->Declination;
   }
   catch(...)
   {
      Solved = false;
   }
   delete Plate;                                            // delete plate object
   if ( Solved == false )                                   // if not solved game over
      return ERR_SHOTALLIGNMENT;

   // coordinate conversion in CM mode
   RA.hh = SolvedRA;                                        // assign hours
   SolvedRA -= RA.hh;                                       // remove hours so to have fractional part
   SolvedRA *= 60;                                          // obtain minutes
   RA.mm = SolvedRA;                                        // assign minutes
   SolvedRA -= RA.mm;                                       // remove minutes so to have fractional part
   SolvedRA *= 60;                                          // obtain seconds
   RA.ss = SolvedRA;                                        // assign seconds

   Dec.sign[0] = '+';                                       // assign positive declination
   Dec.sign[1] = 0;
   if ( SolvedDec < 0 )                                     // check negative declination
   {
      SolvedDec = - SolvedDec;                              // make declination positive
      Dec.sign[0] = '-';                                    // put sign negative
   }
   Dec.gg = SolvedDec;                                      // assign degrees
   SolvedDec -= Dec.gg;                                     // remove degrees so to have fractional part
   SolvedDec *= 60;                                         // obtain minutes
   Dec.mm = SolvedDec;                                      // assign minutes
   SolvedDec -= Dec.mm;                                     // remove minutes so to have fractional part
   SolvedDec *= 60;                                         // obtain seconds
   Dec.ss = SolvedDec;                                      // assign seconds

   // set event object with current data and time
   Event = new CMEvent();
   GetSystemTime ( &LocalTime );                            // get PC time
   Date.Year = LocalTime.wYear;                             // set event structure
   Date.Month = LocalTime.wMonth;
   Date.Day = LocalTime.wDay;
   Date.Hour = LocalTime.wHour;
   Date.Minute = LocalTime.wMinute;
   Date.Sec = LocalTime.wSecond;
   Event->SetDate( &Date );                                 // set event
   CurrentJD = Event->GetGG();                              // get JD

   // set event object with J2000 data and time
   Date.Year = 2000;                                        // set event structure
   Date.Month = 1;
   Date.Day = 1;
   Date.Hour = 0;
   Date.Minute = 0;
   Date.Sec = 0;
   Event->SetDate( &Date );                                 // set event
   J2000 = Event->GetGG();                                  // get JD
   delete Event;

   J200PlatePos = new CMCoEquatorial();                     // object for plate position
   J200PlatePos->SetDataEqu(J2000);                         // assign 2000 equinox
   J200PlatePos->GetRA()->Set(&RA);                         // assign plate right ascension
   J200PlatePos->GetDec()->Set(&Dec);                       // assign plate declination
   CurrentPlatePos = J200PlatePos->ChangeEqu(CurrentJD);    // ask current equinox coordinate
   RA = *CurrentPlatePos->GetRA()->Get();                   // get current equinox plate right ascension
   Dec = *CurrentPlatePos->GetDec()->Get();                 // get current equinox plate declination
   delete J200PlatePos;                                     // remove objects
   delete CurrentPlatePos;

   // allign telescope to this position
   Ret = ObjCom->SetTelescopePosition( &RA, &Dec );
   // check if coomunication failed
   if ( Ret == 0 )                                          // when no error allignment done with success
   {
      InternalStatus.TargetRA = RA;                         // store target position
      InternalStatus.TargetDec = Dec;
      InternalStatus.TeleRA = RA;                           // store actual telescope position
      InternalStatus.TeleDec = Dec;
      Notify( Sender, NOTIFY_TELESCOPESHOTALLIGED );        // notify event
      InternalStatus.ShotRA = InternalStatus.TargetRA;      // uptade shot position
      InternalStatus.ShotDec = InternalStatus.TargetDec;
   }
   else
      return ERR_POS;

   return ERR_NONE;
}



/****************************************************************************

   request allignement execution

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::SetAlignmentRequest( Observer *Sender )
{
   ObserverAllignment = Sender;
   AllignmentRequest = 1;
   return ERR_NONE;
}



/****************************************************************************

   test request allignement

*****************************************************************************/

bool TentaclesMgr::TestAlignmentRequest( void )
{
   return AllignmentRequest == 1;
}



/****************************************************************************

   test allignement finished

*****************************************************************************/

bool TentaclesMgr::TestAlignmentFinished( void )
{
   return AllignmentRequest == 0;
}



/****************************************************************************

   request compare execution

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::SetCompareRequest( ImageCompare::OBJECT_CHAIN **ObjectList )
{
   CompareObjectList = ObjectList;
   CompareRequest = 1;
   return ERR_NONE;
}



/****************************************************************************

   test request allignement

*****************************************************************************/

bool TentaclesMgr::TestCompareRequest( void )
{
   return CompareRequest == 2;
}



/****************************************************************************

   request compare execution

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::GetHighestPosition( double *XHighest, double *YHighest)
{
   CCD::ERR                            Result;              // error code

   Result = ObjCcd->GetHighestPosition( XHighest, YHighest); // get information from CCD oobject
   if ( Result != CCD::ERR_NONE )
      return ERR_NOSHOT;
   return ERR_NONE;
}



/****************************************************************************

   compare two plates

*****************************************************************************/

TentaclesMgr::ERR TentaclesMgr::Compare ( ImageCompare::OBJECT_CHAIN **ObjectList )
{
//   CompareObj->Compare ( ObjectName, ShotName, ObjectList );
   return ERR_NONE;
}



/****************************************************************************

   Compute object coordinate on chart

*****************************************************************************/

bool TentaclesMgr::ObjectToChart( double RACent, double DecCent, double Scale, int Width, int Height,
                              double RAStar, double DecStar, int *x, int *y )
{
   double                              HourAngle;           // hour angle simulation
   double                              Latitude;            // latitude simulation
   double                              cn;                  // temporary for computing
   double                              cd;                  // temporary for computing
   double                              cx;                  // temporary for X computing
   double                              cy;                  // temporary for Y computing
   double                              Azimut;              // azimut simulation
   double                              Elevation;           // evelation simulation

   HourAngle = RACent - RAStar;                             // simulate hour angle
   Latitude = DecCent + 90;                                 // simulate latitude

   // simulate azimut computing
   cn = sin ( HourAngle / RAD );                            // compute numerator
   cd = cos ( HourAngle / RAD ) * sin ( Latitude / RAD ) - tan ( DecStar / RAD) * cos ( Latitude / RAD );  // compute denominator
   Azimut = atan ( cn / cd ) * RAD;                         // compute azimut
   if ( cd < 0 )                                            // arrange sector 2,3
      Azimut += 180;

   // simulate elevation computing
   Elevation = sin( Latitude / RAD ) * sin ( DecStar / RAD ) + cos ( Latitude / RAD ) * cos ( DecStar / RAD ) * cos ( HourAngle / RAD );
   Elevation = asin ( Elevation ) * RAD;

   // compute x position
   cx = Azimut;                                             // azimut
   if ( cx > 180 )                                          // in the range -180 - +180
      cx -= 360;
   cx /= Scale;                                             // compute in pixel

   // compute y position
   cy = - Elevation;                                        // elevation in opposite direction of bitmap coordinate
   cy /= Scale;                                             // compute in pixel

   cx += Width / 2;                                         // add image center
   *x = cx;                                                 // this is x coordinate
   cy += Height / 2;                                        // add image center
   *y = cy;                                                 // this is y coordinate

   if ( cy > Height || cy < 0 )                             // bypass start out of image
      return false;

   if ( cx > Width || cx < 0 )                              // bypass start out of image
      return false;  

   return true;
}



/****************************************************************************

   get a string from text buffer

*****************************************************************************/

void TentaclesMgr::GetString ( char *Buffer, char *Dest, int MaxChars )
{
   // skip initial spaces
   while ( MaxChars > 0 && *Buffer == ' ' )
   {
      Buffer++;                                             // skip character
      MaxChars--;                                           // reduce characters
   }
   // copy string
   while ( MaxChars > 0 && *Buffer != ' ' )
   {
      *Dest++ = *Buffer++;                                  // copy caharcter
      MaxChars--;                                           // reduce characters
   }

   // set string terminator
   *Dest = 0;
}

