/******************************************************************************

	TentaclesMgr.h | Header of Observatory tentacles manager

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : March 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// antirepeat
#ifndef TentaclesMgrH
#define TentaclesMgrH



// system include
#include <vcl.h>
#include <dir.h>
#pragma hdrstop



// project include
#include "Subject.h"
#include "LX200.h"
#include "LX200Thread.h"
#include "CCD.h"
#include "Fits.h"
#include "GSC.h"
#include "ImageCompare.h"



// definition
#define TMGR_LASTSHORT                    "LastShot"



// function definitions for inpout32.dll
typedef short _stdcall (*IN32)(short PortAddr);
typedef void _stdcall (*OUT32)(short PortAddr, short Datum );



/******************************************************************************

	Observatory branches manager class

******************************************************************************/

class TentaclesMgr : public Subject
{
   public:

      // mixed definition
      typedef enum
      {
         MAX_SHOT = 100,                                    // number of maximum shot name counter
         LX200_UPDATETIME = 500,                            // update time for LX200 protocol
         LX200_READWAIT = 1000,                             // wait time in the updating LX200 information
         POWER_OFF = 0,                                     // power off command
         POWER_ON = 1,                                      // power on command
         MAX_PIXELCOMP = 20,                                // maximum pixel compensation
         MAX_DELTACOMP = 500,                               // maximum compensation in arcsec
      } DEFINE;

      // structure containg subfix conter for every shot saved
      typedef struct
      {
         char                          ObjectName[30];      // base object name
         int                           LastCount;           // last used count
      }SHOT;

      // error definitions
      typedef enum
      {
         ERR_NONE,                                          // no error
         ERR_LPOWER_PAR,                                    // error in serial power parameters
         ERR_LPOWER_OPEN,                                   // serial power connection not open
         ERR_LX200_PAR,                                     // error in LX200 parameters
         ERR_LX200_OPEN,                                    // LX200 connection not open
         ERR_COMMAND,                                       // wrong command
         ERR_OBSERVATORYBUSY,                               // exclusive use of the observatory already acquired
         ERR_CCDBUSY,                                       // exclusive use of the ccd already acquired
         ERR_TOMANYSHOT,                                    // to many short name
         ERR_NOSHOT,                                        // no shot available
         ERR_NOABORTSHOT,                                   // no abort shot available
         ERR_POS,                                           // positioning error
         ERR_STARNOTFOUD,                                   // star not found
         ERR_NOTFINISHED,                                   // short not finish
         ERR_CLOUDY,                                        // sky couldy
         ERR_NOTRACKING,                                    // tracking off
         ERR_NOTFOUND,                                      // element not found
         ERR_ALREADYPRESENT,                                // element already present
         ERR_SHOTALLIGNMENT,                                // allignment by shot is not possibile
      } ERR;

      // notify definition
      typedef enum
      {
         NOTIFY_PROGON,                                     // program ON
         NOTIFY_PROGOFF,                                    // program OFF
         NOTIFY_CCD_TAKEUP,                                 // exclusive use of the observatory acquired
         NOTIFY_CCD_RELEASE,                                // exclusive use of the observatory released
         NOTIFY_OBSERVATORY_TAKEUP,                         // exclusive use of the ccd acquired
         NOTIFY_OBSERVATORY_RELEASE,                        // exclusive use of the ccd released
         NOTIFY_GENERIC_COMMAND,                            // generic command run
         NOTIFY_SHOTSTART,                                  // started shot
         NOTIFY_SHOTABORT,                                  // aborteded shot
         NOTIFY_SHOTDOWNLOADED,                             // downloaded shot image from CCD
         NOTIFY_SHOTSAVED,                                  // saved shot file
         NOTYFY_TELESCOPEPOSITIONED,                        // telescope positioned
         NOTIFY_TELESCOPEALLIGED,                           // telescope alligned
         NOTIFY_TELESCOPESHOTALLIGED,                       // telescope alligned by shot
         NOTIFY_SAFEALGORITM,                               // safe algorithm enable/disabled
         NOTIFY_SKYCLOUDY,                                  // sky is cloudy
         NOTIFY_GRBACTIVE,                                  // GRB detection actived
         NOTIFY_GRBDEACTIVE,                                // GRB detection deactived
         NOTIFY_GRBACTIVESWIFT,                             // GRB detection with SWIFT actived
         NOTIFY_GRBDEACTIVESWIFT,                           // GRB detection with SWIFT deactived
         NOTIFY_GRBDETECTED,                                // GRB detected
         NOTIFY_GRBTELESCOPEPOSITIONED                      // telecope positioned on GRB
      } NOTIFY;

      // subjects of command
      typedef enum
      {
         COM_SUBJ_POWER,                                    // command on power supply
         COM_SUBJ_LX200,                                    // command on LX200 connection
         COM_SUBJ_CONTROLLER,                               // controller timeout
         COM_SUBJ_CCD,                                      // command on CCD
         COM_SUBJ_LIGHT,                                    // command on flat light
         COM_SUBJ_TRACK,                                    // command on telescope tracking movement
         COM_SUBJ_DECOND,                                   // command on de condenses device
         COM_SUBJ_CLOCKSYNC,                                // command on controller clock sincronization
         COM_SUBJ_DOME,                                     // command on dome position
         COM_SUBJ_RA,                                       // command on telescope right acension motor
         COM_SUBJ_DEC,                                      // command on telescope declination motor
         COM_SUBJ_SSHUTTER,                                 // command on dome sliding shutter
         COM_SUBJ_BSHUTTER,                                 // command on dome balancing shutter
         COM_SUBJ_TELE,                                     // command on telescope position
         COM_SUBJ_NIGHTLYFOLDER,                            // command on folder of current night
         COM_SUBJ_DARKFOLDER,                               // command on dark folder
         COM_SUBJ_PEC,                                      // periodic error correction
      } COM_SUBJECT;

      // command state
      typedef enum
      {
         COM_STA_OFF,                                       // command power off
         COM_STA_ON,                                        // command power on
         COM_STA_OPEN,                                      // command open
         COM_STA_CLOSE,                                     // command close
         COM_STA_STOP,                                      // command stop movement
         COM_STA_LOCK,                                      // command lock
         COM_STA_UNLOCK,                                    // command unlock
         COM_STA_HOME,                                      // command go to home position
         COM_STA_ZERO,                                      // command go to zero position
         COM_STA_LEFT,                                      // command go in left direction
         COM_STA_RIGHT,                                     // command go in right direction
         COM_STA_UP,                                        // command go in up direction
         COM_STA_DOWN,                                      // command go in down direction
         COM_STA_SAVE,                                      // command file save
      } COM_STA;

      // sky object class
      typedef enum
      {
         OBJECT_REFSTARS,                                   // reference stars
         OBJECT_PLANETS,                                    // planets
         OBJECT_MINORPLANETS,                               // minor planets
         OBJECT_MULTISTARS,                                 // multiple stars
         OBJECT_CLUSTERS,                                   // clusters
         OBJECT_NEBULAS,                                    // nebulas
         OBJECT_GALAXIES,                                   // galaxy
         OBJECT_NGC,                                        // NGC list
      } OBJECT_CLASS;

      // observatory status structure
      typedef struct
      {
         bool                          PowerEnabled;        // Power supply enabled
         bool                          DataRetained;        // Data retained correctly by controller
         bool                          CommOpened;          // LX200 communication opened
         bool                          CommOk;              // LX200 communication correct
         bool                          CCDEnabled;          // CCD is enabled
         CMHours::HOURS                TeleRA;              // Current telescope right ascension
         CMDegree::DEGREE              TeleDec;             // Current telescope declination
         bool                          TeleAlligned;        // Telescopel is assligned
         CMDegree::DEGREE              TeleAzimut;          // Current telescope azimut
         CMDegree::DEGREE              TeleElevation;       // Current telescope elevation
         bool                          TeleTracking;        // Telescope tracking motor state
         bool                          TeleMoving;          // Telescope is moving
         bool                          TeleAlarm;           // Telescope alarm position
         int                           DomeAzimut;          // Current dome azimut
         bool                          DomeLock;            // Dome lock to telescope azimut state
         bool                          DomeAlligned;        // Dome is alligned
         int                           DomeMovement;        // Dome movement state
         int                           DomeSlidingShutter;  // Sliding shutter state
         int                           DomeBalancingShutter;// Balancing shutter state
         float                         DomeTemp;            // Temperature inside dome
         bool                          DomeHome;            // Dome in home position
         bool                          DomeZero;            // Dome in zero position
         bool                          FlatLight;           // Flat light state
         bool                          DeCondence;          // De condenses device state
         bool                          MotorRAFault;        // Fault on right ascension motor
         bool                          MotorDecFault;       // Fault on declination motor
         CMHours::HOURS                ShotRA;              // telescope right ascension at last shot
         CMDegree::DEGREE              ShotDec;             // telescope declination at last shot
         bool                          ShotInProgress;      // a shot is in progress
         int                           ShotExposure;        // exposure for last shot
         int                           ShotExposing;        // current exposing time
         CMHours::HOURS                TargetRA;            // target movement telescope right ascension
         CMDegree::DEGREE              TargetDec;           // target movement telescope declination
         char                          TargetObject[51];    // target telescope object name
         COM_SUBJECT                   CommandSubject;      // subject of last command
         COM_STA                       CommandState;        // state of last command
         bool                          SafeEnable;          // safe algorithm is active
         double                        SafeThreshold;       // safe algorithm threshold
         int                           SafeTime;            // safe algorithm time in seconds
         int                           PECValue;            // periodic error correction value
         int                           PECIndex;            // periodic error correction phase index
      }STATUS;

      // Object constructor
                                       TentaclesMgr( );
      // Object destroyer
      virtual                          ~TentaclesMgr();
      // Set parallel port
      ERR                              SetPower( int PLine );
      // Set LX200 communication parameter
      ERR                              SetLX200( int PLine, int PBaudRate );
      // Set ccd parameters
      ERR                              SetCCD ( HANDLE PFormHandle, char *PCcdName, double PPixelScale, char *ImageFolder );
      // Set plate solver parameter
      ERR                              SetPlateSolver ( char *PGSCFolders, int MaxMagResolution );
      // set shot folders
      ERR                              SetShotFolders( char *PShotFolder, char *PDarkFolder );
      // Set gsc folder
      ERR                              SetGSCFolder ( char *PGSCFolders );
       // Set objects folder
      ERR                              SetObjectsFolder ( char *PObjectsFolders );
       // Set SN reference parameters
      ERR                              SetSN ( char *PSNFolder, double PPixelScale );
      // Set observatory Parameters
      ERR                              SetObservatoryPar ( char *Telescope, char *Observers,
                                                            CMDegree::DEGREE ParObsLatitude, CMHours::HOURS ParObsLongitude );
      // Activate object
      ERR                              Activate(void);
      // Deactivate object
      ERR                              Deactivate(void);
      // Observer asks to take up observatory and/or ccd
      ERR                              TakeUp( Observer *Sender, char *Name, bool Observatory, bool CCD );
      // Observer releases observatory and/or ccd
      ERR                              Release( Observer *Sender, bool Observatory, bool CCD );
      // get name of observer had taken exclusive use of observatory
      const char*                      GetNameOnObservatory( void );
      // get name of observer had taken exclusive use of CCD
      const char*                      GetNameOnCCD( void );
      // Get status of all controlled devices
      ERR                              GetStatus( STATUS *Status);
      // Run a command
      ERR                              RunCommand( Observer *Sender, COM_SUBJECT Subj, COM_STA State, int Par1=0, int Par2=0);
      // set telescope coordinate
      ERR                              TelescopePos( Observer *Sender, CMHours::HOURS *RA, CMDegree::DEGREE *Dec, char *Object, bool Allign );
      // set automatic velocity
      ERR                              SetAutoVelocity( Observer *Sender, int Velocity );

      // Take a shot
      ERR                              Shot( Observer *Sender, int Time );
      // Abort shot
      ERR                              AbortShot( Observer *Sender );
      // Verify is shot is finished
      ERR                              IsShotFinisched( int *Time );
      // Get shot
      ERR                              GetShot( Observer *Sender );
      // Get informazioni about last shot
      ERR                              GetShotInfo( CCD::SHOT_INFO* Info );
      // set the object name
      ERR                              SetObjectName( char *PObjectName );
      // select star nearest given coordinate on shot
      ERR                              SelectStarOnShot( int *x, int *y, CMHours::HOURS *RAStar, CMDegree::DEGREE *DecStar);

      // Get last shot image as a gray bitmap
      ERR                              GetBmp( Graphics::TBitmap **, int Min, int Max );
      // Get last shot image zoomed as a gray bitmap
      ERR                              GetZoomBmp( Graphics::TBitmap **, int Min, int Max, int CenterX, int CenterY );
      // get Histogram bitmap
      ERR                              GetHistogramBmp( Graphics::TBitmap **Bmp, int Min, int Max );
      // get last shot name
      const char*                      GetLastShotName( void ){return ShotName;}
      // get object list from object class
      CMObjectList *                   GetObjectList( OBJECT_CLASS Class );
      // get chart
      ERR                              GetChart( Graphics::TBitmap **, CMHours::HOURS *RACenter, CMDegree::DEGREE *DecCenter, int Radius, int CCDwidth, int CCDHeight);
      // select star nearest given coordinate on bitmap
      ERR                              SelectStarOnChart( int *x, int *y, CMHours::HOURS *RACenter, CMDegree::DEGREE *DecCenter);
      // get image value from position
      ERR                              GetImageValue( int x, int y, int *Value );
      // enable/disable safe algorithm to determinate cloudy sky
      void                             SetSafeAlgorithm( bool PSafeEnable, double PSafeThreshold, int PSafeTime);
      // Add minor planet to list
      ERR                              AddMinorPlanet( char *MinorPlanetCode );
      // remove minor planet from the list
      ERR                              RemoveMinorPlanet( char *MinorPlanetCode );
      // set GRB coordinate
      ERR                              SetGRBPos( CMHours::HOURS *RA, CMDegree::DEGREE *Dec );
      // set telescope coordinate
      ERR                              TelescopeGRBPos( Observer *Sender);
      // exec telescope allignment from shot
      ERR                              ShotAllignment( Observer *Sender);
      // request allignement execution
      ERR                              SetAlignmentRequest( Observer *Sender );
      // test request allignement
      bool                             TestAlignmentRequest( void );
      // test allignement finished
      bool                             TestAlignmentFinished( void );
      // request comparation execution
      ERR                              SetCompareRequest( ImageCompare::OBJECT_CHAIN **ObjectList );
      // test comparation allignement
      bool                             TestCompareRequest( void );
      // get highest position
      ERR                              GetHighestPosition( double *XHighest, double *YHighest);

   protected:
      bool                             InitTargetPos;             // target position initialized
      char                             ImageFolder[MAXPATH];      // image folder
      LX200                            *ObjCom;                   // object managing LX200 communication
      LX200Thread                      *ObjThreadCom;             // object managing LX200 communication in concurrent thread
      CCD                              *ObjCcd;                   // object managing CCD
      GSC                              *ObjGSC;                   // object GSC browser
      HANDLE                           hPowerLine;                // handle for power on line
      int                              PowerLine;                 // power on line
      int                              LX200Line;                 // LX200 line communication
      int                              LX200BaudRate;             // LX200 baud rate
      HANDLE                           FormHandle;                // handle of the main form
      char                             CcdName[50];               // CCD name
      char                             GSCFolder[MAXPATH];        // GSC folder
      char                             ObjectsFolder[MAXPATH];    // folder for objects catalogs
      char                             ShotFolder[MAXPATH];       // folder for nightly shots
      char                             DarkFolder[MAXPATH];       // folder for dark library
      Observer                         *ObserverOnObservatory;    // observer had taken exclusive use of observatory
      Observer                         *ObserverOnCCD;            // observer had taken exclusive use of ccd
      char                             NameOnObservatory[80];     // name of observer had taken exclusive use of observatory
      char                             NameOnCCD[80];             // name of observer had taken exclusive use of ccd
      char                             ObjectName[30];            // base object name
      SHOT                             ShotList[MAX_SHOT];        // table object in shot folder
      SHOT                             DarkList[MAX_SHOT];        // table object in dark folder
      Fits                             *ObjFitsImage;             // object managing fits file image
      STATUS                           InternalStatus;            // internal status information
      char                             ShotName[50];              // last shot name
      CMObjectList *                   ChartObjects;              // list of object on the last chart
      CMHours::HOURS                   ChartRACenter;             // righ ascension of the chart center
      CMDegree::DEGREE                 ChartDecCenter;            // declination of the chart center
      int                              ChartWidth;                // chart width in pixel
      int                              ChartHeight;               // chart height in pixel
      int                              ChartRadius;               // chart radius in arcsec
      double                           PixelScale;                // size of pixel in arcsec
      bool                             EnableSetTime;             // enable timer for stop the readback status changed by set command
      DWORD                            SetTimeStamp;              // timestamp (GetTickCount) for stop the readback status changed by set command
      CMObjectList                     *PlanetList;               // list of planet
      CMObjectList                     *RefStarsList;             // list of reference stars
      CMObjectList                     *MPlanetsList;             // list of minor planets
      CMObjectList                     *MultipleStarList;         // list of multiple stars
      CMObjectList                     *ClustersList;             // list of clusters
      CMObjectList                     *NebuleList;               // list of nebule
      CMObjectList                     *GalaxiesList;             // list of galaxies
      CMObjectList                     *NGCList;                  // list of NGC object
      char                             Telescope[80];             // telecope type
      char                             Observers[80];             // observers list
      bool                             SafeEnable;                // safe algorithm is active
      double                           SafeThreshold;             // safe algorithm threshold
      int                              SafeTime;                  // safe algorithm time in seconds
      bool                             SafeTimerStarted;          // safe timer is started
      DWORD                            SafeTimer;                 // safe timer
      CMHours                          *RACompObj1;               // RA comparation object
      CMDegree                         *DecCompObj1;              // declination comparation object
      CMHours                          *RACompObj2;               // RA comparation object
      CMDegree                         *DecCompObj2;              // declination comparation object
      CMHours::HOURS                   GRBRA;                     // GRB right ascension at last shot
      CMDegree::DEGREE                 GRBDec;                    // GRB declination at last shot
      CMEvent                          *Now;                      // current data and time event object
      CMDegree::DEGREE                 ObsLatitude;               // composed observatory latitude
      CMHours::HOURS                   ObsLongitude;              // composed observatory longitude in hours
      double                           Latitude;                  // observatory latitude
      double                           YearEquinox;               // current epoch for ecpitic coordinate
      char                             GSCResolutionFolder[MAXPATH]; // GSC resolution folder
      int                              MaxMagResolution;          // maximum magnitude for plate resolution
      int                              AllignmentRequest;         // allignment requeset pending ( 0=no request, 1, request, 2=running)
      Observer                         *ObserverAllignment;       // observer requiring alignment
      int                              CompareRequest;            // compare requeset pending ( 0=no request, 1=request, 2=running )
      ImageCompare::OBJECT_CHAIN       **CompareObjectList;       // compare object list pointer
      ImageCompare                     *CompareObj;               // object for plate compare

      // save file with progressive name
      ERR                              SaveFile( char *Folder, SHOT *List );
      // create sky object lists
      void                             CreateSkyObjectsLists( void );
      //  Compute object coordinate on chart
      bool                             ObjectToChart( double RACent, double DecCent, double Scale, int Width, int Height,
                                             double RAStar, double DecStar, int *x, int *y );
      // compare two plates
      ERR                              Compare ( ImageCompare::OBJECT_CHAIN **ObjectList );
      // get a string from text buffer
      void                             GetString ( char *Buffer, char *Dest, int MaxChars );
};

#endif
