/******************************************************************************

	PolypusMF.cpp : Polypus main form

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : March 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// System include
#include <vcl.h>
#include <windows.h>
#include <math.h>
#include <io.h>                                
#include <sys\stat.h>
#include <fcntl.h>
#include <Inifiles.hpp>
#include <registry.hpp>
#pragma hdrstop



// project include
#include "PolypusMF.h"
#include "Config.h"
#include "Diary.h"
#include "AddMinorPlanetForm.h"
#include "LicenceForm.h"
#include "CMCoLocal.h"
#include "CMEvent.h"
#include "CMCoHorizontal.h"
#include "CMFunctions.h"
#include "CMDeepSky.h"
#include "CMMoon.h"
#include "Tokenizer.h"
#pragma package(smart_init)
#pragma resource "*.dfm"



// global variables
TMainForm *MainForm;



/*---------------------------------------------------------------------------

   Form constructor

---------------------------------------------------------------------------*/

__fastcall TMainForm::TMainForm(TComponent* Owner)
   : TForm(Owner)
{
   char                                FileName[MAXPATH];
   char                                Drive[MAXDRIVE];
   char                                Dir[MAXDIR];
   char                                Name[MAXFILE];
   char                                Ext[MAXEXT];

   // inizialize object data
   ExposureActive=false;
   ZoomLocked = false;
   ZoomX = 0;
   ZoomY = 0;
   AdjustChatFrame = false;
   StarShotSelected = false;
   StarChartSelected = false;
   EnableRefreshShot = false;
   SetChartFromTelescope = 0;
   EnableScaligChangeDetect = true;
   ObjFlatSequence = 0;
   ObjDarkSequence = 0;
   ObjShotSequence = 0;
   ObjOpenSequence = 0;
   ObjCloseSequence = 0;
   StartedDiary = false;
   memset( &StatusOld, 0, sizeof(StatusOld) );
   FirstStatus = true;
   CPULoaded = false;
   OldLineRunning = -1;
   OldScriptRunning = -1;
   GRBScriptChanged = false;
   MainScriptChanged = false;
   TimeScriptChanged = false;
   CloseScriptChanged = false;
   CPURunning = false;
   OldGRBOn = true;
   AutomaticRun = false;
   Now = new CMEvent();                                     // create event object
   PlanningList = 0;

   // detect program folder
   strcpy( FileName, ParamStr(0).c_str() );
   fnsplit(FileName, Drive, Dir, Name, Ext);
   strcpy( ProgramFolder, Drive);
   strcat( ProgramFolder, Dir);

   // assange main form in starting condition
   Top=0;                                                   // top of the screen
   Left=0;                                                  // left of the screen
}



/*---------------------------------------------------------------------------

   Show form

---------------------------------------------------------------------------*/

void __fastcall TMainForm::FormShow(TObject *Sender)
{
   TentaclesMgr                        *ObjTentacles;       // subject in observer pattern (this object manages all observatory devices)
   TentaclesMgr::ERR                   Error;               // error in tentacles manager
   char                                FileName[MAXPATH];   // log file name
   SYSTEMTIME                          CurrentTime;         // current data for Nightly folder naming
   char                                Str[20];             // temporary string
   Graphics::TBitmap                   *Bmp;                // starting bitmap
   TRect                               Rect;                // rectangle for bitmap drawing
   AnsiString                          AStr;                // temporary string
   TRegistry                           *Reg;                // registry obejct
   AnsiString                          Licence;             // licence enable

   // ask for licence if not jet accepted
   Reg = new TRegistry();
   Licence = "";
   if ( Reg->OpenKey( "Software\\Polypus", false ) == true )
   {
      Licence = Reg->ReadString("License");
   }
   delete Reg;

   if ( Licence != "Y")
      LicenceClick( 0 );

   // load configuration data from ini file
   LoadConfigurationData();
   ChartField = PixelScale * (double)ChartImage->Width;

   // build site coordinate object
   Site = new CMCoLocal();                                  // create object for observatory site position
   Site->GetLongitude()->Set( &ObsLongitude );              // assign observatory longitude
   Site->GetLatitude()->Set( &ObsLatitude );                // assign observatory latitude

   // build nightly shot folder name from date
   strcpy ( PatShotFolder, ShotFolder);
   GetSystemTime( &CurrentTime );
   wsprintf( Str, "%04d-%02d-%02d\\", CurrentTime.wYear, CurrentTime.wMonth, CurrentTime.wDay );
   strcat( PatShotFolder, Str );

   // create nightly shot folder
   if ( GetFileAttributes( PatShotFolder ) ==  0xFFFFFFFF )
      CreateDirectory( PatShotFolder, NULL );

   //
   // build up the observer pattern structure
   //

   ObjTentacles = new TentaclesMgr();                       // create object managing all observatory devices
   ObjTentacles->SetPower(SerialPower);                     // assign serial port for controlloer power on
   ObjTentacles->SetLX200(SPortNumber, SPortBaudRate);      // assign serial port for LX200 connection
   ObjTentacles->SetGSCFolder( GSCFolder );                 // assign folder for GSC catalog
   ObjTentacles->SetObjectsFolder( ObjectsFolder );         // assign folder for sky objects catalogs
   ObjTentacles->SetShotFolders( PatShotFolder, DarkFolder);// assign folders for shot
   ObjTentacles->SetObservatoryPar ( Telescope, ObserverList, ObsLatitude, ObsLongitude );// assign observatory parameters
   ObjTentacles->SetPlateSolver( GSCResolutionFolder, MaxMagResolution );
   ObjTentacles->SetSN(SNFolder, PixelScale);


   MFObserver = new ManualControlObserver();                // crate observer for manual device driving (because VCL dosn't support multiple heredity
   MFObserver->SetSubject( ObjTentacles );                  // assign subject to observer
   MFObserver->SetMF(this);                                 // assign main form to subject (turn arround VCl problem)
   ObjTentacles->Attach(MFObserver);                        // attach subject to observer

   strcpy( FileName, PatShotFolder );
   strcat( FileName, "GiornaleDiBordo.txt" );
   ObjLogObserver = new LogObserver();
   ObjLogObserver->SetSubject( ObjTentacles );              // assign subject to observer
   ObjLogObserver->SetMemo(LogMemo);                        // assign memo to subject
   ObjLogObserver->SetFileName(FileName);                   // assign file name
   ObjTentacles->Attach(ObjLogObserver);                    // attach subject to observer

   Error = ObjTentacles->Activate();                        // activate tentacles
   ShowError ( Error );                                     // show error message

   // create CPU object
   CPU = new ScriptCPU();                                   // create CPI (it is an observer)
   CPU->SetSubject( ObjTentacles );                         // assign subject

   // load retain data
   LoadRetainData();

   // load dark list
   LoadDarkList();

   // initialize shot histogram image
   Bmp = new Graphics::TBitmap();                           // create bitmap for hitogram image
   Bmp->Width = HistogramImage->Width;                      // assign size to hitogram bitmap from image size
   Bmp->Height = HistogramImage->Height;
   HistogramImage->Picture->Bitmap = Bmp;                   // assign bitmap to image
   UpdateHistogram();                                       // create the default image

   // initialize chart image
   Rect.Top = 0;                                            // rectangle whit chart size
   Rect.Left = 0;
   Rect.Right = ChartImage->Width;
   Rect.Bottom = ChartImage->Height;
   ChartImage->Canvas->Brush->Color = clBlack;              // background color
   ChartImage->Picture->Bitmap->Canvas->FillRect(Rect);     // fill bitmap

   // set bottom bitmaps
   ImageList->GetBitmap( BITMAP_ON, OpenSequenceButton->Glyph);   // open sequence with on button
   ImageList->GetBitmap( BITMAP_ON, CloseSequenceButton->Glyph);  // close sequence with on button
   ImageList->GetBitmap( BITMAP_LOCK, DomeLockButton->Glyph);     // lock dome button
   ImageList->GetBitmap( BITMAP_UNLOCK, DomeUnlockButton->Glyph); // unlock dome button
   ImageList->GetBitmap( BITMAP_HOME, DomeHomeButton->Glyph);     // home dome button
   ImageList->GetBitmap( BITMAP_ZERO, DomeZeroButton->Glyph);     // zero dome button
   ImageList->GetBitmap( BITMAP_STOP, DomeStopButton->Glyph);     // stop dome button
   ImageList->GetBitmap( BITMAP_LEFT, DomeLeftButton->Glyph);     // left dome button
   ImageList->GetBitmap( BITMAP_RIGHT, DomeRightButton->Glyph);   // right dome button
   ImageList->GetBitmap( BITMAP_ON, TrackingOnButton->Glyph );    // tracking on button
   ImageList->GetBitmap( BITMAP_OFF, TrackingOffButton->Glyph );  // tracking off button
   ImageList->GetBitmap( BITMAP_HOME, TelescopeHomeButton->Glyph );// telescope home button
   ImageList->GetBitmap( BITMAP_STOP, TelescopeStopButton->Glyph );// telescope stop button
   ImageList->GetBitmap( BITMAP_UP, TelescopeUpButton->Glyph );   // telescope up button
   ImageList->GetBitmap( BITMAP_DOWN, TelescopeDownButton->Glyph );// telescope down button
   ImageList->GetBitmap( BITMAP_LEFT, TelescopeLeftButton->Glyph );// telescope left button
   ImageList->GetBitmap( BITMAP_RIGHT, TelescopeRightButton->Glyph );// telescope right button
   ImageList->GetBitmap( BITMAP_OPEN, SShutterOpenButton->Glyph );   // Sliding shutter open button
   ImageList->GetBitmap( BITMAP_CLOSE, SShutterCloseButton->Glyph ); // Sliding shutter close button
   ImageList->GetBitmap( BITMAP_STOP, SShutterStopButton->Glyph );   // Sliding shutter stop button
   ImageList->GetBitmap( BITMAP_OPEN, BShutterOpenButton->Glyph );   // Balancing shutter open button
   ImageList->GetBitmap( BITMAP_CLOSE, BShutterCloseButton->Glyph ); // Balancing shutter close button
   ImageList->GetBitmap( BITMAP_STOP, BShutterStopButton->Glyph );   // Balancing shutter stop button
   ImageList->GetBitmap( BITMAP_ON, FlatLightOnButton->Glyph );      // flat light on button
   ImageList->GetBitmap( BITMAP_OFF, FlatLightOffButton->Glyph );    // flat light off button
   ImageList->GetBitmap( BITMAP_ON, DeCondOnButton->Glyph );         // decondence on button
   ImageList->GetBitmap( BITMAP_OFF, DeCondOffButton->Glyph );       // decondence off button
   ImageList->GetBitmap( BITMAP_ON, PowerOnButton->Glyph );          // power on button
   ImageList->GetBitmap( BITMAP_OFF, PowerOffButton->Glyph );        // power off button
   ImageList->GetBitmap( BITMAP_ON, LX200OnButton->Glyph );          // LX200 communication on button
   ImageList->GetBitmap( BITMAP_OFF, LX200OffButton->Glyph );        // LX200 communication off button
   ImageList->GetBitmap( BITMAP_SYNC, ClockSyncButton->Glyph );      // clock syncronization button
   ImageList->GetBitmap( BITMAP_RESET, RAMotorResetButton->Glyph );  // RA motor reset button
   ImageList->GetBitmap( BITMAP_RESET, DecMotorResetButton->Glyph ); // declination motor reset button

   // check all element in openinig sequence
   OpenSequenceListBox->Checked[0] = true;
   OpenSequenceListBox->Checked[1] = true;
   OpenSequenceListBox->Checked[2] = true;
   OpenSequenceListBox->Checked[3] = true;
   OpenSequenceListBox->Checked[4] = true;
   OpenSequenceListBox->Checked[5] = true;

   // check all element in closing sequence
   CloseSequenceListBox->Checked[0] = true;
   CloseSequenceListBox->Checked[1] = true;
   CloseSequenceListBox->Checked[2] = true;
   CloseSequenceListBox->Checked[3] = true;
   CloseSequenceListBox->Checked[4] = true;
   CloseSequenceListBox->Checked[5] = true;
   CloseSequenceListBox->Checked[6] = true;

   // center the CCD image object
   CCDImage->Top += (CCDImage->Height-FrameHeight)/2;
   CCDImage->Left += (CCDImage->Width-FrameWidth)/2;
   CCDImage->Height = FrameHeight;
   CCDImage->Width = FrameWidth;

   // create bitmap for shot exposing bar graph
   Bmp = new Graphics::TBitmap();                           // create bitmap
   Bmp->Width = ShotExposingImage->Width;                   // assign size from image
   Bmp->Height = ShotExposingImage->Height;
   Rect.Top = 0;
   Rect.Left = 0;
   Rect.Right = Bmp->Width;
   Rect.Bottom = Bmp->Height;
   Bmp->Canvas->Brush->Color = clBlack;
   Bmp->Canvas->FillRect(Rect);
   ShotExposingImage->Picture->Bitmap = Bmp;                // assign bitmap to image

   // create program editors
   GRBScriptEditor = new ScriptEditor();                   // create editor
   GRBScriptEditor->SetEditor( GRBScriptBox, 50, 5, GRBScriptBox->Width-10, GRBScriptBox->Height-55 );     // applay editor to the box
   LoadGRBScriptFile();                                     // load script
   MainScriptEditor = new ScriptEditor();                   // create editor
   MainScriptEditor->SetEditor( MainScriptBox, 75, 5, MainScriptBox->Width-10, MainScriptBox->Height-80 );     // applay editor to the box
   LoadMainScriptFile();                                    // load script
   TimeScriptEditor = new ScriptEditor();                   // create editor
   TimeScriptEditor->SetEditor( TimeScriptBox, 75, 5, MainScriptBox->Width-10, MainScriptBox->Height-80 );     // applay editor to the box
   LoadTimeScriptFile();                                    // load script
   CloseScriptEditor = new ScriptEditor();                  // create editor
   CloseScriptEditor->SetEditor( CloseScriptBox, 75, 5, MainScriptBox->Width-10, MainScriptBox->Height-80 );   // applay editor to the box
   LoadCloseScriptFile();                                    // load script

   // show selected object coordinates
   UpdateObjectList(0);

   Pages->ActivePageIndex = 0;                              // select firt page
   ShotPages->ActivePageIndex = 0;                          // select first page

   // update caption
   AStr = Caption + " " + VERSION + " - Bassano Bresciano Observatory";
   Caption = AStr;

   // column heading in GRB page
   MailGrid->Cells[0][0] = "Da";
   MailGrid->Cells[1][0] = "Data";
   MailGrid->Cells[2][0] = "Oggetto";

   // create GRB detector
   GRBDetector = new GRB( );
   GRBDetector->SetOption ( MinElevation, MaxDelay );
   GRBDetector->SetStatusEdit(MailStatus);
   UpdateMailGrid();

   // planning setup
   PlaningGraph = new ElevationGraph();                     // create planning graph
   PlanningList = new CMObjectList( true );                 // create planning object list as reference list
   PlaningGraph->SetList(PlanningList);                     // assign planning list to planning object
   MonthCalendar1->Date = MonthCalendar1->Date.CurrentDate();  // assign current date
   PlanningDrawGrid->Enabled = false;                       // is not possbile edit planning object list
   UpdatePlanningGraph();                                   // show empty planning view


   // create table color for planning list
   TabColor[0] = clRed;
   TabColor[1] = clLime;
   TabColor[2] = clFuchsia;
   TabColor[3] = clAqua;
   TabColor[4] = clTeal;
   TabColor[5] = clGray;
   TabColor[6] = clPurple;
   TabColor[7] = clMaroon;
   TabColor[8] = clGreen;
   TabColor[9] = clOlive;

   // adjust pages
   PollingTimer->Enabled = true;                            // enable timer for staus update
}



/*---------------------------------------------------------------------------

   Verify the possibility of program closing

---------------------------------------------------------------------------*/

void __fastcall TMainForm::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   int                                 RetVal;              // retun value from messagebox

   // check activities
   if ( ObjFlatSequence != 0 || ObjDarkSequence != 0 || ObjShotSequence != 0 ||
        CPURunning == true )
   {
      RetVal = MessageBox( this->Handle, "Attivit in corso vuoi veramente chiudere il programma?", "Attenzione", MB_YESNO );
      if ( RetVal == IDNO	)
         CanClose = false;
         return;
   }

   // check for flat sequence
   if ( ObjFlatSequence != 0 )
   {
      ObjFlatSequence->Terminate();
      CanClose = false;
   }
   // check for flat sequence
   if ( ObjDarkSequence != 0 )
   {
      ObjDarkSequence->Terminate();
      CanClose = false;
   }
   // check for shot sequence
   if ( ObjShotSequence != 0 )
   {
      ObjShotSequence->Terminate();
      CanClose = false;
   }
   // check for GRB script saving
   if ( GRBScriptChanged == true )
   {
      RetVal = MessageBox( this->Handle, "Programma GRB modificato e non salvato, Vuoi salvare?", "Attenzione", MB_YESNOCANCEL );
      if ( RetVal == IDYES	)
         SaveGRBScript(0);
      if ( RetVal == IDCANCEL	)
         CanClose = false;
   }
   // check for main script saving
   if ( MainScriptChanged == true )
   {
      RetVal = MessageBox( this->Handle, "Programma principale modificato e non salvato, Vuoi salvare?", "Attenzione", MB_YESNOCANCEL );
      if ( RetVal == IDYES	)
         SaveMainScript(0);
      if ( RetVal == IDCANCEL	)
         CanClose = false;
   }
   // check for time script saving
   if ( TimeScriptChanged == true )
   {
      RetVal = MessageBox( this->Handle, "Programma a tempo modificato e non salvato, Vuoi salvare?", "Attenzione", MB_YESNOCANCEL );
      if ( RetVal == IDYES	)
         SaveTimeScript(0);
      if ( RetVal == IDCANCEL	)
         CanClose = false;
   }
   // check for close script saving
   if ( CloseScriptChanged == true )
   {
      RetVal = MessageBox( this->Handle, "Programma di chiusura modificato e non salvato, Vuoi salvare?", "Attenzione", MB_YESNOCANCEL );
      if ( RetVal == IDYES	)
         SaveCloseScript(0);
      if ( RetVal == IDCANCEL	)
         CanClose = false;
   }

   // close is it possibile ?
   if ( CanClose == true )
   {
      PollingTimer->Enabled = false;                        // timer must be disactivated
      SaveConfigurationData();                              // save information to ini file
   }
}



/*---------------------------------------------------------------------------

   Destroy form

---------------------------------------------------------------------------*/

void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
   TentaclesMgr                        *ObjTentacles;       // subject: object managing all observatory devices

   // save retain data
   SaveRetainData();


   ObjTentacles = MFObserver->GetSubject();                 // get object from observer
   ObjTentacles->Deactivate();                              // deactivate tentacle
   delete MFObserver;                                       // destroy main form observer
   delete ObjLogObserver;                                   // destroy log observer
   delete ObjTentacles;                                     // destroy subject
   delete GRBDetector;                                      // delete GRB manager
   delete PlaningGraph;                                     // delete planning graph
   delete Site;                                             // delete observatory position
}



/*---------------------------------------------------------------------------

   Show configuration dialog box

---------------------------------------------------------------------------*/

void __fastcall TMainForm::ItemConfiguraClick(TObject *Sender)
{
   TForm1                              *FormCfg;            // configuration dialog box
   int                                 ReturnCode;          // return dialog value
   TentaclesMgr::COM_STA               State;               // state command
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   FormCfg = new TForm1(this);                              // create configuration dialog box
   ReturnCode = FormCfg->ShowModal();                       // show  dialog box
   delete FormCfg;                                          // delete dialog box

   if ( ReturnCode != mrOk )                                // no new information
      return;                                               // exit

   // set PEC on controller
   if ( Status.CommOpened == true )
   {
      State = TentaclesMgr::COM_STA_OFF;                    // prepare set PEC state depending on configuration
      if ( EnablePEC == true )
         State = TentaclesMgr::COM_STA_ON;
      Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_PEC, State, PECAmplitude, PECPhase);
      ShowError ( Error );                                  // show error message
   }

   SaveConfigurationData();                                 // save information to ini file
}



/*---------------------------------------------------------------------------

   Polling update of all status information

---------------------------------------------------------------------------*/

void __fastcall TMainForm::StatusUpdate(TObject *Sender)
{
   SYSTEMTIME                          CurrentGMT;          // current GMT time
   CMEvent::DATE                       NowDate;             // current data and time event data
   CMHours::HOURS                      SidTime;             // current sideral time
   int                                 Second;              // Second in int format
   char                                Str[50];             // temporary string
   char                                Str2[50];            // temporary string
   double                              Temp1;               // termporary double for computing
   int                                 Temp3;               // termporary integer for computing
   TRect                               Rect;                // rectanlge for bar graph drawing
   TentaclesMgr::ERR                   Error;               // error code from devices manager
   bool                                End;                 // end exposure flag
   int                                 ExposingTime;        // current esposing time
   int                                 Flat;                // current flat number
   int                                 Dark;                // current dark number
   int                                 Shot;                // current shot number
   OpenSequence::SEQ_STATE             OpenState;           // current open state
   CloseSequence::SEQ_STATE            CloseState;          // current close state
   TForm2                              *DlgDiary;           // dialog for diary startup
   int                                 ReturnCode;          // return dialog value
   int                                 Index;               // index for observers set and check
   bool                                Cloudy;              // sequence ended because sky cloudy
   int                                 ScriptRunning;       // code of script running
   int                                 LineRunning;         // line of script running
   bool                                CPUBegin;            // CPU at beging
   int                                 CloseCounter;        // progress counter for close sequence

   // get actual observatory devices state
   MFObserver->GetSubject()->GetStatus( &Status );

   //
   //  update general panel
   //

   // update current data and time
   GetSystemTime( &CurrentGMT );                            // get current GMT time
   wsprintf( Str, "%02d/%02d/%04d", CurrentGMT.wDay, CurrentGMT.wMonth, CurrentGMT.wYear );  // create date string
   DateLabel->Caption = AnsiString(Str);                    // update date
   wsprintf( Str, "%02d:%02d:%02d", CurrentGMT.wHour, CurrentGMT.wMinute, CurrentGMT.wSecond ); // create time string
   TimeLabel->Caption = AnsiString(Str);                    // update time

   // update current local sideral time
   NowDate.Year = CurrentGMT.wYear;                         // fill event structure
   NowDate.Month = CurrentGMT.wMonth;
   NowDate.Day = CurrentGMT.wDay;
   NowDate.Hour = CurrentGMT.wHour;
   NowDate.Minute = CurrentGMT.wMinute;
   NowDate.Sec = CurrentGMT.wSecond;
   Now->SetDate(&NowDate);                                  // set event object
   NowJD = Now->GetGG();                                    // get current julian day
   SidTime = CM_SideralTimeComputing( NowJD, false );

   CM_AddTime( &ObsLongitude, &SidTime );                   // compute local sideral time
   Second = SidTime.ss;                                     // convert siderl time second
   wsprintf( Str, "T.Sid.%02d:%02d:%02d", SidTime.hh, SidTime.mm, Second );  // create string
   SideralTimeLabel->Caption = AnsiString(Str);             // update sidel time

   // update power supply enable
   // detect power became on
   if ( Status.PowerEnabled == true &&
        (StatusOld.PowerEnabled == false || PowerOnButton->Down == false || FirstStatus == true ) )
   {
      PowerOnButton->Down = true;                           // push on button
      PowerLabel->Caption = "Quadro ON";                    // show on message
      PowerLabel->Font->Color = clBlack;                    // normal color
   }
   // detect power became off
   if ( Status.PowerEnabled == false &&
        (StatusOld.PowerEnabled == true || PowerOffButton->Down == false || FirstStatus == true ) )
   {
      PowerOffButton->Down = true;                          // push off button
      PowerLabel->Caption = "Quadro OFF";                   // show off message
      PowerLabel->Font->Color = clRed;                      // error color
   }

   // update GRB label
   if ( GRBDetected == true && OldGRBDetect == false )
   {
      GRBLabel->Caption = "GRB in corso";
      GRBLabel->Font->Color = clRed;                        // alarm color
   }
   if ( GRBDetected == false && OldGRBDetect == true )
      OldGRBOn = !GRBEnableCheckBox->Checked;
   if ( GRBEnableCheckBox->Checked == true && OldGRBOn == false )
   {
      GRBLabel->Caption = "GRB ON";
      GRBLabel->Font->Color = clBlack;                      // normal color
   }
   if ( GRBEnableCheckBox->Checked == false && OldGRBOn == true )
   {
      GRBLabel->Caption = "GRB OFF";
      GRBLabel->Font->Color = clBlack;                      // normal color
   }
   OldGRBOn = GRBEnableCheckBox->Checked;
   OldGRBDetect = GRBDetected;


   // update communication status

   // set open commmunication button
   if ( Status.CommOpened == true && LX200OnButton->Down == false )
      LX200OnButton->Down = true;                           // push on button
   // detect communication became off
   if ( Status.CommOpened == false &&
        (StatusOld.CommOpened == true || LX200OffButton->Down == false || FirstStatus == true ) )
   {
      LX200OffButton->Down = true;                          // push off button
      CommLabel->Caption = "Comm.OFF";                      // show off message
      CommLabel->Font->Color = clRed;                       // error color
   }
   // detect communication became on and good
   if ( (Status.CommOpened == true && Status.CommOk == true) &&
        (StatusOld.CommOpened == false || StatusOld.CommOk == false ) )
   {
      CommLabel->Caption = "Comm.OK";                       // show ok message
      CommLabel->Font->Color = clBlack;                     // normal color
      if ( CharFollowsTelescopeCheckBox->Checked == true )
         SetChartFromTelescope = 4;                         // activate chart update
   }
   // detect communication became on without connection
   if ( (Status.CommOpened == true && Status.CommOk == false) &&
        (StatusOld.CommOpened == false || StatusOld.CommOk == true ) )
   {
      CommLabel->Caption = "No comm.";                      // show error message
      CommLabel->Font->Color = clRed;                       // error color
   }

   // update retain data on controller
   // detect retain data became good
   if ( Status.DataRetained == true &&
        (StatusOld.DataRetained == false || FirstStatus == true ) )
      RetainLabel->Caption = "Dati OK";

   // detect retain data became bad
   if ( Status.DataRetained == false &&
      (StatusOld.DataRetained == true || FirstStatus == true ) )
      RetainLabel->Caption = "Dati persi";



   //
   // Update telescope panel
   //

   // update RA
   Second = Status.TeleRA.ss;                               // convert second in int format
   wsprintf( Str, "AR %02dh%02d'%02d\"", Status.TeleRA.hh, Status.TeleRA.mm, Second ); // create RA string
   TelescopeRALabel->Caption = Str;                         // update RA

   // update declination
   Second = Status.TeleDec.ss;                              // convert second in int format
   wsprintf( Str, "DE %s%02d%02d'%02d\"", Status.TeleDec.sign, Status.TeleDec.gg, Status.TeleDec.mm, Second );  // create declination string
   TelescopeDeclinationLabel->Caption = AnsiString(Str);    // update declination

   // detect RA and declination become valid and stable
   if ( (Status.CommOk    == true  && Status.TeleTracking    == true  && Status.TeleAlligned    == true  && Status.TeleMoving == false ) &&
        (StatusOld.CommOk == false || StatusOld.TeleTracking == false || StatusOld.TeleAlligned == false || StatusOld.TeleMoving == true || FirstStatus == true ) )
   {
      TelescopeRALabel->Font->Color = clBlack;              // RA in normal color
      TelescopeDeclinationLabel->Font->Color = clBlack;     // declination in normal color
   }
   // detect RA and declination become not valid
   if ( (Status.CommOk    == false || Status.TeleTracking    == false || Status.TeleAlligned    == false || Status.TeleMoving == true)  &&
        (StatusOld.CommOk == true  && StatusOld.TeleTracking == true  && StatusOld.TeleAlligned == true  || Status.TeleMoving == false) || FirstStatus == true )
   {
      TelescopeRALabel->Font->Color = clRed;                // RA in error color
      TelescopeDeclinationLabel->Font->Color = clRed;       // declination in error color
   }

   // update telescope azimut
   wsprintf( Str, "AZI %03d%02d'", Status.TeleAzimut.gg, Status.TeleAzimut.mm );         // create azimut string
   TelescopeAzimutLabel->Caption = AnsiString(Str);         // update azimut

   // update telescope elevation
   wsprintf( Str2, "ALT %s%02d%02d'", Status.TeleElevation.sign, Status.TeleElevation.gg, Status.TeleElevation.mm ); // create elevation string
   TelescopeElevationLabel->Caption = AnsiString(Str2);     // update elevation

   // detect tracking became on
   if ( Status.TeleTracking == true &&
       (StatusOld.TeleTracking == false || TrackingOnButton->Down == false || FirstStatus == true ) )
   {
      TrackingOnButton->Down = true;                        // push on button
      TelescopeTrackingLabel->Caption = "M.Orario ON";      // show message
      TelescopeTrackingLabel->Font->Color = clBlack;        // normal color
   }
   // detect tracking became on
   if ( Status.TeleTracking == false &&
        (StatusOld.TeleTracking == true || TrackingOffButton->Down == false || FirstStatus == true ) )
   {
      TrackingOffButton->Down = true;                       // push off button
      TelescopeTrackingLabel->Caption = "M.Orario OFF";     // show message
      TelescopeTrackingLabel->Font->Color = clRed;          // error color
   }

   // detect telescope allignment became on
   if ( Status.TeleAlligned == true && (StatusOld.TeleAlligned == false || FirstStatus == true ) )
   {
      TelescopeAllignedLabel->Caption = "Allineato";         // show message
      TelescopeAllignedLabel->Font->Color = clBlack;         // normal color
   }
   // detect telescope allignment became off
   if ( Status.TeleAlligned == false && (StatusOld.TeleAlligned == true || FirstStatus == true ) )
   {
      TelescopeAllignedLabel->Caption = "Disallineato";      // show message
      TelescopeAllignedLabel->Font->Color = clRed;           // error color
   }

   // detect no motors fult
   if ( (Status.MotorRAFault == false && Status.MotorDecFault == false ) &&
        (StatusOld.MotorRAFault == true || StatusOld.MotorDecFault  == true || FirstStatus == true ) )
   {
      TelescopeMotorsLabel->Caption = "Motori OK";          // show message
      TelescopeMotorsLabel->Font->Color = clBlack;          // normal color
   }
   // detect AR motors fult
   if ( Status.MotorRAFault == true &&
         (StatusOld.MotorRAFault == false || FirstStatus == true ) )
   {
      TelescopeMotorsLabel->Caption = "Fault AR";           // show message
      TelescopeMotorsLabel->Font->Color = clRed;            // error color
   }
   // detect no motors fult
   if ( (Status.MotorRAFault == false && Status.MotorDecFault == true ) &&
        (StatusOld.MotorRAFault == true || StatusOld.MotorDecFault == false || FirstStatus == true ) )
   {
      TelescopeMotorsLabel->Caption = "Fault DE";           // show message
      TelescopeMotorsLabel->Font->Color = clRed;            // error color
   }

   // show PEC status
   wsprintf( Str, " PEC %04d/%d", Status.PECIndex, Status.PECValue );  // create date string
   TelescopePECLabel->Caption = Str;

   //
   // update dome panel
   //

   // update Sliding shutter status
   if ( Status.DomeSlidingShutter != StatusOld.DomeSlidingShutter || FirstStatus == true )
   {
      switch (Status.DomeSlidingShutter )                   // depending status
      {
         case 0:
            SShutterLabel->Caption = "Sc.Aperto";           // show message
            SShutterLabel->Font->Color = clBlack;           // normal color
            break;
         case 1:
            SShutterLabel->Caption = "Sc.Chiuso";           // show message
            SShutterLabel->Font->Color = clRed;             // error color
            break;
         case 2:
            SShutterLabel->Caption = "Sc.Ape.Auto.";        // show message
            SShutterLabel->Font->Color = clRed;             // error color
            break;
         case 3:
            SShutterLabel->Caption = "Sc.Ape.Man.";         // show message
            SShutterLabel->Font->Color = clRed;             // error color
            break;
         case 4:
            SShutterLabel->Caption = "Sc.Chi.Auto";         // show message
            SShutterLabel->Font->Color = clRed;             // error color
            break;
         case 5:
            SShutterLabel->Caption = "Sc.Chi.Man.";         // show message
            SShutterLabel->Font->Color = clRed;             // error color
            break;
      }
   }

   // update balancing shutter status
   if ( Status.DomeBalancingShutter != StatusOld.DomeBalancingShutter || FirstStatus == true )
   {
      switch (Status.DomeBalancingShutter )                 // depending status
      {
         case 0:
            BShutterLabel->Caption = "Ri.Aperto";           // show message
            BShutterLabel->Font->Color = clBlack;           // normal color
            break;
         case 1:
            BShutterLabel->Caption = "Ri.Chiuso";           // show message
            BShutterLabel->Font->Color = clRed;             // error color
            break;
         case 2:
            BShutterLabel->Caption = "Ri.Ape.Auto.";        // show message
            BShutterLabel->Font->Color = clRed;             // error color
            break;
         case 3:
            BShutterLabel->Caption = "Ri.Ape.Man.";         // show message
            BShutterLabel->Font->Color = clRed;             // error color
            break;
         case 4:
            BShutterLabel->Caption = "Ri.Chi.Auto";         // show message
            BShutterLabel->Font->Color = clRed;             // error color
            break;
         case 5:
            BShutterLabel->Caption = "Ri.Chi.Man.";         // show message
            BShutterLabel->Font->Color = clRed;             // error color
            break;
      }
   }

   // update dome azimut
   if ( (Status.DomeAzimut != StatusOld.DomeAzimut ||
         Status.DomeHome   != StatusOld.DomeHome       )
         || FirstStatus == true )
   {
      wsprintf( Str, "AZI %03d", Status.DomeAzimut );      // create azimut string
      if ( Status.DomeHome == true )
         strcat( Str, " H");
      if ( Status.DomeZero == true )
         strcat( Str, " Z");
      DomeAzimutLabel->Caption = AnsiString(Str);           // update azimut
   }

   // detect telescope azimut became within dome window (less than 5 degee )
   if ( fabs( Status.DomeAzimut-Status.TeleAzimut.gg) <= 4 &&
      ( fabs( StatusOld.DomeAzimut-AzimutOld) > 4 || FirstStatus == true) )
      DomeAzimutLabel->Font->Color = clBlack;              // normal color
   // detect telescope azimut became out dome window (greater than 4 degee )
   if ( fabs( Status.DomeAzimut-Status.TeleAzimut.gg) > 4  &&
      ( fabs( StatusOld.DomeAzimut-AzimutOld) <= 4 || FirstStatus == true) )
      DomeAzimutLabel->Font->Color = clRed;                 // error color

   // detect flat light on
   if ( Status.FlatLight == true &&
      ( StatusOld.FlatLight == false || FlatLightOnButton->Down == false || FirstStatus == true ) )
   {
      FlatLightOnButton->Down = true;                       // push on button
      FlatLightLabel->Caption = "Luci flat ON";             // show message
      FlatLightLabel->Font->Color = clRed;                  // error color
   }
   // detect flat light off
   if ( Status.FlatLight == false &&
      (StatusOld.FlatLight == true || FlatLightOffButton->Down == false || FirstStatus == true ) )
   {
      FlatLightOffButton->Down = true;                      // push off button
      FlatLightLabel->Caption = "Luci flat OFF";            // show message
      FlatLightLabel->Font->Color = clBlack;                // normal color
   }

   // detect decondence device on
   if ( Status.DeCondence == true &&
      ( StatusOld.DeCondence == false || DeCondOnButton->Down == false || FirstStatus == true ) )
   {
      DeCondOnButton->Down = true;
      DecondLabel->Caption = "Ventilaz ON";                 // push on button
      DecondLabel->Font->Color = clRed;                     // show message
   }                                                        // error color
   // detect decondence device off
   if ( Status.DeCondence == false &&
      ( StatusOld.DeCondence == true || DeCondOffButton->Down == false || FirstStatus == true ) )
   {
      DeCondOffButton->Down = true;                         // push off button
      DecondLabel->Caption = "Ventilaz OFF";                // show message
      DecondLabel->Font->Color = clBlack;                   // normal color
   }

   // detect dome allignment on
   if ( Status.DomeAlligned == true &&
      (StatusOld.DomeAlligned == false || FirstStatus == true ) )
   {
      DomeAllignedLabel->Caption = "Allineata";             // show message
      DomeAllignedLabel->Font->Color = clBlack;             // normal color
   }
   // detect dome allignment on
   if ( Status.DomeAlligned == false &&
      (StatusOld.DomeAlligned == true || FirstStatus == true ) )
   {
      DomeAllignedLabel->Caption = "Disallineata";          // show message
      DomeAllignedLabel->Font->Color = clRed;               // error color
   }

   // detect dome lock on
   if ( Status.DomeLock == true &&
      ( StatusOld.DomeLock == false || DomeLockButton->Down == false || FirstStatus == true ) )
   {
      DomeLockButton->Down = true;                          // puch dome lock button
      DomeStateLabel->Caption = "Automatico";               // show message
      DomeStateLabel->Font->Color = clBlack;                // normal color
   }
   // detect dome lock off
   if ( Status.DomeLock == false &&
      ( StatusOld.DomeLock == true || DomeUnlockButton->Down == false || FirstStatus == true ) )
   {
      DomeUnlockButton->Down = true;                        // puch dome unlock button
      DomeStateLabel->Font->Color = clRed;                  // alwais error color
      StatusOld.DomeMovement = -1;                          // force status chance in order to show status
   }
   // update dome state in lock off state
   if ( Status.DomeLock == false )
   {
      if ( Status.DomeMovement != StatusOld.DomeMovement || FirstStatus == true )
      {
         switch (Status.DomeMovement)                       // message depanding on dome state
         {
            case 0:
               DomeStateLabel->Caption = "Ferma";           // show message
               break;
            case 1:
               DomeStateLabel->Caption = "Rot.oraria";      // show message
               break;
            case 2:
               DomeStateLabel->Caption = "Rot.antioraria";  // show message
               break;
            case 3:
               DomeStateLabel->Caption = "Pos.zero";        // show message
               break;
            case 4:
               DomeStateLabel->Caption = "Pos.riposo";      // show message
               break;
         }
      }
   }

   // update dome temperature
   gcvt( Status.DomeTemp, 3, Str2);                         // generate string with only 3 digit
   wsprintf( Str, "Temp. %s", Str2 );                      // generate string
   DomeTemperatureLabel->Caption = AnsiString(Str);         // update temperature



   //
   // update shot panel
   //

   // detect shot is finished
   if ( Status.ShotInProgress == false && StatusOld.ShotInProgress == true )
   {
      Rect.top = 1;                                         // rectangle leaving one pixel in every border
      Rect.left = 1;
      Rect.bottom = ShotExposingImage->Height-1;
      Rect.right = ShotExposingImage->Width-1;
      ShotExposingImage->Picture->Bitmap->Canvas->Brush->Color = clGreen;  // color meaning finish
      ShotExposingImage->Picture->Bitmap->Canvas->FillRect( Rect );        // show complete bar graph
   }

   // detect shot in in progress
   if ( Status.ShotInProgress == true )
   {
      Temp3 = 0;                                            // default bar graph width
      if ( Status.ShotExposure != 0 )                       // if exposure time available compute bar graph width
         Temp3 = (Status.ShotExposing * ShotExposingImage->Width-2 ) /  Status.ShotExposure ;
      if ( Temp3 > ShotExposingImage->Width-2 )             // if width greate than maximum clamp it
         Temp3 = ShotExposingImage->Width-2;
      Rect.top = 1;                                         // define rectangle to paint leving one pixel for border
      Rect.left = 1;
      Rect.bottom = ShotExposingImage->Height-1;
      Rect.right = Temp3+1;
      ShotExposingImage->Picture->Bitmap->Canvas->Brush->Color = clRed;    // color means exposing
      ShotExposingImage->Picture->Bitmap->Canvas->FillRect( Rect );        // show bar graph for exposing
      ShotExposingImage->Picture->Bitmap->Canvas->Brush->Color = clYellow; // color for empty part
      if ( Temp3 < ShotExposingImage->Width-2 )             // empty par only if bar graph is not full
      {
         Rect.left = Rect.right;                            // define the right rectengle
         Rect.right = ShotExposingImage->Width-1;
         ShotExposingImage->Picture->Bitmap->Canvas->FillRect( Rect );     // show empty part
      }
   }

   // detect shot time is changed
   if ( Status.ShotExposure != StatusOld.ShotExposure )
   {
      Temp1 = Status.ShotExposure;                          // new exposure in millisecond
      Temp1 /= 1000;                                        // convert in second
      ShotExposureLabel->Caption = "Esp."+ AnsiString(Temp1)+"s.";   // update exposure time
   }


   //
   // check shot and sequence status
   //

   // check single exposition
   if ( ExposureActive == true )
   {
      Error = MFObserver->GetSubject()->IsShotFinisched(&Temp3); // ask if shot is finished
      Temp1 = Temp3;                                        // convert in float
      Temp1 /= 1000;                                        // convert in second
      ExposingEdit->Text = AnsiString(Temp1);               // update exposing
      if ( Error == TentaclesMgr::ERR_NONE )                // if shot finished
      {
         ExposureActive = false;                            // reset exposing flag
         MFObserver->GetSubject()->GetShot(MFObserver);     // get shot from ccd
         ShotStartButton->Enabled = true;                   // activate stat button
         ShotStopButton->Enabled = false;                   // deactivate stop button
      }
   }

   // chek flat sequence
   if ( ObjFlatSequence != 0 )
   {
      ObjFlatSequence->GetStatus( &End, &ExposingTime, &Flat, &Dark );   // get sequence status
      Temp1 = ExposingTime;                                 // convert in float
      Temp1 /= 1000;                                        // convert in second
      FlatExposingEdit->Text = Temp1;                       // update exposing time
      FlatProgEdit->Text = Flat;                            // update flat number
      DarkFlatProgEdit->Text = Dark;                        // update dark number
      // check if sequence if finished
      if ( End == true )
      {
         delete ObjFlatSequence;                            // delete object
         ObjFlatSequence = 0;                               // clear pointer
         FlatStartButton->Enabled = true;                   // activate stat button
         FlatStopButton->Enabled = false;                   // deactivate stop button
      }
   }

   // chek flat sequence dark
   if ( ObjDarkSequence != 0 )
   {
      ObjDarkSequence->GetStatus( &End, &ExposingTime, &Dark ); // get sequence status
      Temp1 = ExposingTime;                                 // convert in float
      Temp1 /= 1000;                                        // convert in second
      DarkExposingEdit->Text = Temp1;                       // update exposing time
      DarkProgEdit->Text = Dark;                            // update dark number
      // check if sequence if finished
      if ( End == true )
      {
         delete ObjDarkSequence;                            // delete object
         ObjDarkSequence = 0;                               // clear pointer
         DarkStartButton->Enabled = true;                   // activate stat button
         DarkStopButton->Enabled = false;                   // deactivate stop button
      }
   }

   // chek shot sequence dark
   if ( ObjShotSequence != 0 )
   {
      ObjShotSequence->GetStatus( &End, &ExposingTime, &Shot, &Cloudy ); // get sequence status
      Temp1 = ExposingTime;                                 // convert in float
      Temp1 /= 1000;                                        // convert in second
      ExposingEdit->Text = Temp1;                           // update exposing time
      SequenceProgEdit->Text = Shot;                        // update shot number
      if ( End == true )
      {
         delete ObjShotSequence;                            // delete object
         ObjShotSequence = 0;                               // clear pointer
         SequenceStartButton->Enabled = true;               // disactivate stat button
         SequenceStopButton->Enabled = false;               // activate stop button

         // if cky cloudy activate safe procedure
         if ( Cloudy == true )
         {
            CloseSequenceButtonClick( NULL );               // run closing sequence
            CloseSequenceCheckBox->Checked = false;
         }

         // disactivate safe algorithm
         if ( Status.SafeEnable == true )
            MFObserver->GetSubject()->SetSafeAlgorithm( false, 0, 0 );
      }
   }

   // check opening sequence
   if ( ObjOpenSequence != 0 )
   {
      OpenState = ObjOpenSequence->GetState();              // get open state sequence
      switch ( OpenState )                                  // depending on state select item in sequence listbox
      {
         // wait dome on zero position
         case OpenSequence::SEQ_STATE_DOMEZERO:
            OpenSequenceListBox->ItemIndex = 0;
            break;

         // wait sliding shutter opening
         case OpenSequence::SEQ_STATE_OPENSSHUTTER:
            OpenSequenceListBox->ItemIndex = 1;
            break;

         // wait balancing shutter opening
         case OpenSequence::SEQ_STATE_OPENBSHUTTER:
            OpenSequenceListBox->ItemIndex = 2;
            break;

         // sequence terminated
         case OpenSequence::SEQ_STATE_END:
            OpenSequenceListBox->ItemIndex = -1;            // select nothing
            delete ObjOpenSequence;                         // delete sequence
            ObjOpenSequence = 0;                            // reset sequence pointer
            OpenSequenceButton->Glyph = 0;                  // reset sequence button
            ImageList->GetBitmap( BITMAP_ON, OpenSequenceButton->Glyph);  // select stat sequence button
            break;
      }
   }

   // check closing sequence
   if ( ObjCloseSequence != 0 )
   {
      CloseState = ObjCloseSequence->GetState( &CloseCounter);            // get close state sequence
      switch ( CloseState )                                 // depending on state select item in sequence listbox
      {
         // wait telescope on home position
         case CloseSequence::SEQ_STATE_TELESCOPEHOME:
            CloseSequenceListBox->ItemIndex = 0;
            break;

         // wait dome on zero position
         case CloseSequence::SEQ_STATE_DOMEZERO:
            CloseSequenceListBox->ItemIndex = 2;
            break;

         // wait balancing shutter closing
         case CloseSequence::SEQ_STATE_CLOSEBSHUTTER:
            CloseSequenceListBox->ItemIndex = 3;
            break;

         // wait sliding shutter closing
         case CloseSequence::SEQ_STATE_CLOSESSHUTTER:
            CloseSequenceListBox->ItemIndex = 4;
            break;

         // wait dome in home position
         case CloseSequence::SEQ_STATE_DOMEHOME:
            CloseSequenceListBox->ItemIndex = 5;
            break;

         // sequence terminated
         case CloseSequence::SEQ_STATE_END:
            CloseSequenceListBox->ItemIndex = -1;           // select nothing
            delete ObjCloseSequence;                        // delete sequence
            ObjCloseSequence = 0;                           // reset sequence pointer
            CloseSequenceButton->Glyph = 0;                 // reset sequence button
            ImageList->GetBitmap( BITMAP_ON, CloseSequenceButton->Glyph);  // select stat sequence button
            break;
      }
   }

   // refresh shot information if required
   if ( EnableRefreshShot == true )
   {
      EnableRefreshShot = false;                            // reset refresh request
      RefreshShot();                                        // exec refresh
   }

   // set chart from telescope if required
   if ( SetChartFromTelescope > 0 )
   {
      SetChartFromTelescope--;                              // reset request
      if ( SetChartFromTelescope == 0 )
      {
         // copy telescope target position to chart input field
         ChartRA_hhEdit->Text = Status.TargetRA.hh;
         ChartRA_mmEdit->Text = Status.TargetRA.mm;
         ChartRA_ssEdit->Text = Status.TargetRA.ss;
         ChartDec_SignEdit->Text = Status.TargetDec.sign;
         ChartDec_ggEdit->Text = Status.TargetDec.gg;
         ChartDec_mmEdit->Text = Status.TargetDec.mm;
         ChartDec_ssEdit->Text = Status.TargetDec.ss;

         // simulate chart goto button pressing
         ChartGotoButtonClick( 0 );
      }
   }

   StatusOld = Status;                                      // actual status is status old for next loop
   AzimutOld = Status.TeleAzimut.gg;                        // actual telescope azimut is the old for next loop
   FirstStatus = 0;                                         // next time is not the first time

   // if required ask for diary at first communication ok
   if ( EnableDiary == true && StartedDiary == false && Status.CommOpened == true && Status.CommOk == true)
   {
      StartedDiary = true;                                  // only once
      DlgDiary = new TForm2(this);                          // create configuration dialog box
      for ( Index=0; Index<NumObserver; Index++ )           // scan all observer
      {
         DlgDiary->LocalObserverListBox->Items->Add(Observers[Index]);  // fill local observer list
         DlgDiary->RemoteObserverListBox->Items->Add(Observers[Index]); // fill remote observer list
      }
      ReturnCode = DlgDiary->ShowModal();
      if ( ReturnCode == mrOk )                             // if confermed
      {
         Diary  = "Data: " +  DateLabel->Caption + "\r\n";  // set data
         Diary += "Ora inizio: " +  TimeLabel->Caption + "\r\n";   // set stating time
         Diary += DomeTemperatureLabel->Caption + "\r\n";   // set temperature
         Diary += "Osservatori presenti : " ;
         for ( Index=0; Index<NumObserver; Index++ )  // add present observers
         {
            if ( DlgDiary->LocalObserverListBox->Checked[Index] == true )  // if observer selected
               Diary += AnsiString(Observers[Index]) + " " ;// add name
         }
         Diary += "\r\n";                                   // end of line
         Diary += "Osservatori connessi : " ;
         for ( Index=0; Index<NumObserver; Index++ )  // add remote connected observers
         {
            if ( DlgDiary->RemoteObserverListBox->Checked[Index] == true )  // if observer selected
               Diary += AnsiString(Observers[Index]) + " " ;// add name
         }
         Diary += "\r\n";                                   // end of line
         Diary += "Scopo osservazioni :\r\n";               // scope
         for ( Index=0; Index<DlgDiary->ScopeMemo->Lines->Count; Index++ )  // add scope lines
         {
            Diary += DlgDiary->ScopeMemo->Lines->Strings[Index];
            Diary += "\r\n";                                // end of line
         }
         SaveDiary();
      }
   }

   //
   // script editor management
   //

   if ( GRBScriptEditor->IsModified() == true )            // detect script change
   {
      GRBScriptChanged = true;                             // script needs saving
   }
   if ( MainScriptEditor->IsModified() == true )            // detect script change
   {
      MainScriptChanged = true;                             // script needs saving
      CPULoaded = false;                                    // CPU needs loading
   }
   if ( TimeScriptEditor->IsModified() == true )            // detect script change
   {
      TimeScriptChanged = true;                             // script needs saving
      CPULoaded = false;                                    // CPU needs loading
   }
   if ( CloseScriptEditor->IsModified() == true )           // detect script change
   {
      CloseScriptChanged = true;                            // script needs saving
      CPULoaded = false;                                    // CPU needs loading
   }
   // refresh editor aspect with running information
   CPU->GetStatus( &CPURunning, &ScriptRunning, &LineRunning, &CPUBegin);  // get running information
   if ( LineRunning != OldLineRunning || ScriptRunning != OldScriptRunning) // refresh line selection only when something change
   {
      switch ( ScriptRunning )
      {
         case 1:
            MainScriptEditor->SelectLine(LineRunning);      // select line running
            TimeScriptEditor->SelectLine(0);                // no selection
            CloseScriptEditor->SelectLine(0);               // no selection
            break;
         case 2:
            MainScriptEditor->SelectLine(0);                // no selection
            TimeScriptEditor->SelectLine(LineRunning);      // select line running
            CloseScriptEditor->SelectLine(0);               // no selection
            break;
         case 3:
            MainScriptEditor->SelectLine(0);                // no selection
            TimeScriptEditor->SelectLine(0);                // no selection
            CloseScriptEditor->SelectLine(LineRunning);     // select line running
            // disable GRB detection
            if ( GRBEnableCheckBox->Checked == true )
               GRBEnableCheckBox->Checked = false;
            break;
      }
      OldLineRunning = LineRunning;                         // save for next time
      OldScriptRunning = ScriptRunning;
   }
   // make evidence in script running and disable writing when any script is running
   MainScriptEditor->SetStatus(CPURunning && ScriptRunning == 1, CPURunning);
   TimeScriptEditor->SetStatus(CPURunning && ScriptRunning == 2, CPURunning);
   CloseScriptEditor->SetStatus(CPURunning && ScriptRunning == 3, CPURunning);
   // enable buttons depending on CPU status
   StopScriptButton->Enabled = CPURunning;
   StepScriptButton->Enabled = !CPURunning;
   RunScriptButton->Enabled = !CPURunning;
   RewindScriptButton->Enabled = !CPUBegin & !CPURunning;
   // exec program automatic start
   if ( AutomaticRun == true && CPURunning == false )
   {
      AutomaticRun = false;
      RunScriptButtonClick(NULL);
   }

   // automatic close sequence start
   if ( CloseSequenceCheckBox->Checked == true && CPURunning == false)
   {
      if ( CurrentGMT.wHour   == atoi( CloseSequenceHHEdit->Text.c_str() ) &&
           CurrentGMT.wMinute == atoi( CloseSequenceMMEdit->Text.c_str() )    )
      {
         // if shot sequence is running...
         if ( ObjShotSequence != 0 )
            ObjShotSequence->Terminate();                  // terminate sequence

         CloseSequenceCheckBox->Checked = false;
         CloseSequenceButtonClick(0);
      }
   }

   // check is allignemt is needed
   if ( MFObserver->GetSubject()->TestAlignmentRequest() == true )
      MFObserver->GetSubject()->ShotAllignment( (Observer*)-1 );
}



/*---------------------------------------------------------------------------

   turn on main power

---------------------------------------------------------------------------*/

void __fastcall TMainForm::PowerOnButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_POWER, TentaclesMgr::COM_STA_ON);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   turn off main power

---------------------------------------------------------------------------*/

void __fastcall TMainForm::PowerOffButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_POWER, TentaclesMgr::COM_STA_OFF);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   turn on communication

---------------------------------------------------------------------------*/

void __fastcall TMainForm::LX200OnButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager
   TentaclesMgr::COM_STA               State;               // state command

   // open communication
   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_LX200, TentaclesMgr::COM_STA_ON);
   ShowError ( Error );                                     // show error message

   if ( Error != TentaclesMgr::ERR_NONE )                   // if erron don't continue
      return;

   State = TentaclesMgr::COM_STA_OFF;                       // prepare set timeout state depending on configuration
   if ( EnableCommTimeout == true )
      State = TentaclesMgr::COM_STA_ON;
   // set timeout on controller
   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_CONTROLLER, State);
   ShowError ( Error );                                     // show error message

   State = TentaclesMgr::COM_STA_OFF;                       // prepare set PEC state depending on configuration
   if ( EnablePEC == true )
      State = TentaclesMgr::COM_STA_ON;
   // set PEC on controller
   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_PEC, State, PECAmplitude, PECPhase);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   turn off communication

---------------------------------------------------------------------------*/

void __fastcall TMainForm::LX200OffButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_LX200, TentaclesMgr::COM_STA_OFF);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   turn on flat lights

---------------------------------------------------------------------------*/

void __fastcall TMainForm::FlatLightOnButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_LIGHT, TentaclesMgr::COM_STA_ON);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   turn off flat lights

---------------------------------------------------------------------------*/

void __fastcall TMainForm::FlatLightOffButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_LIGHT, TentaclesMgr::COM_STA_OFF);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   turn on tracking motor

---------------------------------------------------------------------------*/

void __fastcall TMainForm::TrackingOnButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_TRACK, TentaclesMgr::COM_STA_ON);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   turn off tracking motor

---------------------------------------------------------------------------*/

void __fastcall TMainForm::TrackingOffButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_TRACK, TentaclesMgr::COM_STA_OFF);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   turn on decondence device

---------------------------------------------------------------------------*/

void __fastcall TMainForm::DeCondOnButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_DECOND, TentaclesMgr::COM_STA_ON);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   turn off decondence device

---------------------------------------------------------------------------*/

void __fastcall TMainForm::DeCondOffButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_DECOND, TentaclesMgr::COM_STA_OFF);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   set controller clock

---------------------------------------------------------------------------*/

void __fastcall TMainForm::ClockSyncButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_CLOCKSYNC, TentaclesMgr::COM_STA_ON);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   lock dome position on telescope azimut

---------------------------------------------------------------------------*/

void __fastcall TMainForm::DomeLockButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_DOME, TentaclesMgr::COM_STA_LOCK);
   ShowError ( Error );                                     // show error message

   if ( Error != TentaclesMgr::ERR_NONE )                   // if error don't continue
      return;

   // disable dome manual movement buttons
   DomeHomeButton->Enabled = false;
   DomeZeroButton->Enabled = false;
   DomeStopButton->Enabled = false;
   DomeLeftButton->Enabled = false;
   DomeRightButton->Enabled = false;
}



/*---------------------------------------------------------------------------

   unlock dome position from telescope azimut

---------------------------------------------------------------------------*/

void __fastcall TMainForm::DomeUnlockButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_DOME, TentaclesMgr::COM_STA_UNLOCK);
   ShowError ( Error );                                     // show error message

   if ( Error != TentaclesMgr::ERR_NONE )                   // if error don't continue
      return;

   // disable dome manual movement buttons
   DomeHomeButton->Enabled = true;
   DomeZeroButton->Enabled = true;
   DomeStopButton->Enabled = true;
   DomeLeftButton->Enabled = true;
   DomeRightButton->Enabled = true;
}



/*---------------------------------------------------------------------------

   dome in home position

---------------------------------------------------------------------------*/

void __fastcall TMainForm::DomeHomeButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_DOME, TentaclesMgr::COM_STA_HOME);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   dome in zero position

---------------------------------------------------------------------------*/

void __fastcall TMainForm::DomeZeroButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_DOME, TentaclesMgr::COM_STA_ZERO);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   stop dome movement

---------------------------------------------------------------------------*/

void __fastcall TMainForm::DomeStopButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_DOME, TentaclesMgr::COM_STA_STOP);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   dome manual movement on the left

---------------------------------------------------------------------------*/

void __fastcall TMainForm::DomeLeftButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_DOME, TentaclesMgr::COM_STA_LEFT);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   dome manual movement on the right

---------------------------------------------------------------------------*/

void __fastcall TMainForm::DomeRightButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_DOME, TentaclesMgr::COM_STA_RIGHT);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   reset fault on RA motor

---------------------------------------------------------------------------*/

void __fastcall TMainForm::RAMotorResetButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_RA, TentaclesMgr::COM_STA_ZERO);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   reset fault on declination motor

---------------------------------------------------------------------------*/

void __fastcall TMainForm::DecMotorResetButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_DEC, TentaclesMgr::COM_STA_ZERO);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   open sliding shutter

---------------------------------------------------------------------------*/

void __fastcall TMainForm::SShutterOpenButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_SSHUTTER, TentaclesMgr::COM_STA_OPEN);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   stop sliding shutter

---------------------------------------------------------------------------*/

void __fastcall TMainForm::SShutterStopButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                      Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_SSHUTTER, TentaclesMgr::COM_STA_STOP);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   close sliding shutter

---------------------------------------------------------------------------*/

void __fastcall TMainForm::SShutterCloseButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_SSHUTTER, TentaclesMgr::COM_STA_CLOSE);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   open balancing shutter

---------------------------------------------------------------------------*/

void __fastcall TMainForm::BShutterOpenButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_BSHUTTER, TentaclesMgr::COM_STA_OPEN);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   stop balancing shutter

---------------------------------------------------------------------------*/

void __fastcall TMainForm::BShutterStopButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_BSHUTTER, TentaclesMgr::COM_STA_STOP);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   close balancing shutter

---------------------------------------------------------------------------*/

void __fastcall TMainForm::BShutterCloseButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_BSHUTTER, TentaclesMgr::COM_STA_CLOSE);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   manual telescope movement to left

---------------------------------------------------------------------------*/

void __fastcall TMainForm::TelescopeLeftButtonClick(TObject *Sender)
{
   int                                 Velocity;            // manual velocity
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Velocity = TelescopeVelocity->Position;                  // get actual manual velocity
   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_TELE, TentaclesMgr::COM_STA_LEFT, Velocity);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   manual telescope movement to right

---------------------------------------------------------------------------*/

void __fastcall TMainForm::TelescopeRightButtonClick(TObject *Sender)
{
   int                                 Velocity;            // manual velocity
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Velocity = TelescopeVelocity->Position;                  // get actual manual velocity
   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_TELE, TentaclesMgr::COM_STA_RIGHT, Velocity);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   manual telescope movement to up

---------------------------------------------------------------------------*/

void __fastcall TMainForm::TelescopeUpButtonClick(TObject *Sender)
{
   int                                 Velocity;            // manual velocity
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Velocity = TelescopeVelocity->Position;                  // get actual manual velocity
   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_TELE, TentaclesMgr::COM_STA_UP, Velocity);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   manual telescope movement to down

---------------------------------------------------------------------------*/

void __fastcall TMainForm::TelescopeDownButtonClick(TObject *Sender)
{
   int                                 Velocity;            // manual velocity
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Velocity = TelescopeVelocity->Position;                  // get actual manual velocity
   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_TELE, TentaclesMgr::COM_STA_DOWN, Velocity);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   stop telescope movement

---------------------------------------------------------------------------*/

void __fastcall TMainForm::TelescopeStopButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_TELE, TentaclesMgr::COM_STA_STOP);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   telescope in home position

---------------------------------------------------------------------------*/

void __fastcall TMainForm::TelescopeHomeButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_TELE, TentaclesMgr::COM_STA_HOME);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   Activate open sequence

---------------------------------------------------------------------------*/

void __fastcall TMainForm::OpenSequenceButtonClick(TObject *Sender)
{
   bool                                DomeZero;            // required dome in zero position
   bool                                OpenSShutter;        // required sliding shutter opening
   bool                                OpenBShutter;        // required balancing shutter opening
   bool                                DomeLock;            // required dome lock on telescope azimut position
   bool                                SyncClock;           // required controller clock sincronization
   bool                                TrackingOn;          // required turn on tracking motor

   // if open sequence is running...
   if ( ObjOpenSequence != 0 )
   {
      ObjOpenSequence->Terminate();                         // terminate sequence
      return;
   }

   // collect single step sequence enables
   DomeZero       = OpenSequenceListBox->Checked[0];
   OpenSShutter   = OpenSequenceListBox->Checked[1];
   OpenBShutter   = OpenSequenceListBox->Checked[2];
   DomeLock       = OpenSequenceListBox->Checked[3];
   SyncClock      = OpenSequenceListBox->Checked[4];
   TrackingOn     = OpenSequenceListBox->Checked[5];

   ObjOpenSequence = new OpenSequence();                    // create sequence (it is an observer)
   ObjOpenSequence->SetSubject( MFObserver->GetSubject() ); // assign subject
   ObjOpenSequence->StartSequence(this->Handle, DomeZero, OpenSShutter, OpenBShutter, DomeLock, SyncClock, TrackingOn );  // start sequence
   OpenSequenceButton->Glyph = 0;                           // clear button image
   ImageList->GetBitmap( BITMAP_OFF, OpenSequenceButton->Glyph);  // assign image for stop
}



/*---------------------------------------------------------------------------

   Activate close sequence

---------------------------------------------------------------------------*/

void __fastcall TMainForm::CloseSequenceButtonClick(TObject *Sender)
{
   bool                                TelescopeHome;       // required telescope in home position
   bool                                TrackingOff;         // required turn off tracking motor
   bool                                DomeZero;            // required dome in zero position
   bool                                CloseBShutter;       // required balancing shutter closing
   bool                                CloseSShutter;       // required sliding shutter closing
   bool                                DomeHome;            // required dome in home position
   bool                                PowerOff;            // required general power off

   // if close sequence is running...
   if ( ObjCloseSequence != 0 )
   {
      ObjCloseSequence->Terminate();                        // terminate sequence
      return;
   }

   // disable GRB detection
   GRBEnableCheckBox->Checked = false;

   // collect single step sequence enables
   TelescopeHome  = CloseSequenceListBox->Checked[0];
   TrackingOff    = CloseSequenceListBox->Checked[1];
   DomeZero       = CloseSequenceListBox->Checked[2];
   CloseBShutter  = CloseSequenceListBox->Checked[3];
   CloseSShutter  = CloseSequenceListBox->Checked[4];
   DomeHome       = CloseSequenceListBox->Checked[5];
   PowerOff       = CloseSequenceListBox->Checked[6];

   ObjCloseSequence = new CloseSequence();                  // create sequence (it is an observer)
   ObjCloseSequence->SetSubject( MFObserver->GetSubject() );// assign subject
   ObjCloseSequence->StartSequence(this->Handle, TelescopeHome, TrackingOff, DomeZero, CloseBShutter, CloseSShutter, DomeHome, PowerOff );  // start sequence
   CloseSequenceButton->Glyph = 0;                          // clear button image
   ImageList->GetBitmap( BITMAP_OFF, CloseSequenceButton->Glyph);    // assign image for stop
}



/*---------------------------------------------------------------------------

   CCD activation

---------------------------------------------------------------------------*/

void __fastcall TMainForm::ActivateCCDButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   // set ccd name
   Error = MFObserver->GetSubject()->SetCCD( this->Handle, CCDName, PixelScale, ProgramFolder);
   ShowError ( Error );                                     // show error message

   if ( Error != TentaclesMgr::ERR_NONE )                   // if error don't continue
      return;

   // activate ccd
   MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_CCD, TentaclesMgr::COM_STA_ON);
   ShowError ( Error );                                     // show error message

   if ( Error != TentaclesMgr::ERR_NONE )                   // if error don't continue
      return;

   ActivateCCDButton->Enabled = false;                      // disable activate button
   ReleaseCCDButton->Enabled = true;                        // enable release button
}



/*---------------------------------------------------------------------------

   CCD release

---------------------------------------------------------------------------*/

void __fastcall TMainForm::ReleaseCCDButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_CCD, TentaclesMgr::COM_STA_OFF);
   ShowError ( Error );                                     // show error message

   if ( Error != TentaclesMgr::ERR_NONE )                   // if error don't continue
      return;

   ActivateCCDButton->Enabled = true;                       // enable activate button
   ReleaseCCDButton->Enabled = false;                       // disable release button
}



/*---------------------------------------------------------------------------

   shot single image

---------------------------------------------------------------------------*/

void __fastcall TMainForm::ShotStartButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager
   float                               ShotTime;            // time for exposure
   int                                 IntShotTime;         // time for exposure format integer

   ShotTime = atof( ExposureComboBox->Text.c_str());        // get time
   ShotTime *= 1000;                                        // time in millisec.
   IntShotTime = ShotTime;                                  // format integer

   // check for correct time
   if ( IntShotTime < 1 || IntShotTime > 200000 )
   {
      MessageBox( this->Handle, "Tempo esposizione errato", "Errore", MB_OK );
      return;
   }

   Error = MFObserver->GetSubject()->Shot( MFObserver, IntShotTime); // run shot
   ShowError ( Error );                                     // show error message

   if ( Error != TentaclesMgr::ERR_NONE )                   // if error don't continue
      return;

   ExposureActive = true;                                   // tun on exposure is active
   ShotStartButton->Enabled = false;                        // disactivate stat button
   ShotStopButton->Enabled = true;                          // activete stop button
}



/*---------------------------------------------------------------------------

   stop shot single image

---------------------------------------------------------------------------*/

void __fastcall TMainForm::ShotStopButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->AbortShot(MFObserver);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   save single image

---------------------------------------------------------------------------*/

void __fastcall TMainForm::ImageSaveButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   // check for object name
   if ( ObjectNameEdit->Text == "" )
   {
      MessageBox( this->Handle, "Manca nome oggetto", "Errore", MB_OK );
      return;
   }

   // set image base name
   Error = MFObserver->GetSubject()->SetObjectName( ObjectNameEdit->Text.c_str() );
   ShowError ( Error );                                     // show error message

   if ( Error != TentaclesMgr::ERR_NONE )                   // if error don't continue
      return;

   // exec save
   Error = MFObserver->GetSubject()->RunCommand( MFObserver, TentaclesMgr::COM_SUBJ_NIGHTLYFOLDER, TentaclesMgr::COM_STA_SAVE);
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   Activate shot sequence

---------------------------------------------------------------------------*/

void __fastcall TMainForm::SequenceStartButtonClick(TObject *Sender)
{
   float                               ShotTime;            // time for exposure
   int                                 IntShotTime;         // time for exposure format integer
   int                                 SequenceTarget;      // number of shot in the sequence
   double                              SafeThresholdVal;    // threshold for safe algorithm
   int                                 SafeTimeVal;         // time for safe algorithm
   int                                 VentilazInterval;
   int                                 VentilazTime;
   int                                 VentilazPause;
   bool                                PECheck;             // check periodic error

   ShotTime = atof( ExposureComboBox->Text.c_str());        // get time
   ShotTime *= 1000;                                        // time in millisec.
   IntShotTime = ShotTime;                                  // format integer

   SequenceTarget = atoi( SequenceTargetEdit->Text.c_str() );  // get shot number in the sequence
   SequenceProgEdit->Text = "";                            // clear progressive conter

   // check for object name
   if ( ObjectNameEdit->Text == "" )
   {
      MessageBox( this->Handle, "Manca nome oggetto", "Errore", MB_OK );
      return;
   }

   // check for correct time
   if ( IntShotTime    < 1 || IntShotTime    > 200000 ||
        SequenceTarget < 1 || SequenceTarget > 500       )
   {
      MessageBox( this->Handle, "Errore nei parametri sequenza", "Errore", MB_OK );
      return;
   }

   // activate safe algorithm
   if ( SafeActive->Checked == true )
   {
      SafeThresholdVal = atof (SafeThresholdEdit->Text.c_str() );
      SafeTimeVal = atoi( SafeTimeEdit->Text.c_str() ) * 60;
      MFObserver->GetSubject()->SetSafeAlgorithm( true, SafeThresholdVal, SafeTimeVal );
   }

   // prepare ventilation parameters
   VentilazInterval = 0;
   VentilazTime = 0;
   VentilazPause = 0;
   if ( VentilationActive->Checked == true )
   {
      VentilazInterval = atoi( VentilationIntervalEdit->Text.c_str() );
      VentilazTime = atoi( VentilationTimeEdit->Text.c_str() );
      VentilazPause = atoi( VentilationPauseEdit->Text.c_str() );
   }

   PECheck = PECheckBox->Checked;

   ObjShotSequence = new ShotSequence(ServiceMemo);          // create sequence object (it is a observer)
   ObjShotSequence->SetSubject( MFObserver->GetSubject() ); // subject assignment
   ObjShotSequence->StartSequence(this->Handle, IntShotTime, SequenceTarget, ObjectNameEdit->Text.c_str(),
                                    VentilazInterval, VentilazTime, VentilazPause, PECheck);  // start sequence

   SequenceStartButton->Enabled = false;                    // disactivate stat button
   SequenceStopButton->Enabled = true;                      // activate stop button

   // record in diary the shot sequence
   if ( Diary != "" )                                       // only if diary is activated
   {
      Diary += "Sequenza fotografie: Oggetto = " + ObjectNameEdit->Text + " Esposizione = " + ExposureComboBox->Text + " s. Num " + SequenceTargetEdit->Text + "\r\n";  // set data
      SaveDiary();                                          // salvo file
   }
}



/*---------------------------------------------------------------------------

   Stop shot sequence

---------------------------------------------------------------------------*/

void __fastcall TMainForm::SequenceStopButtonClick(TObject *Sender)
{
   // if shot sequence is running...
   if ( ObjShotSequence != 0 )
      ObjShotSequence->Terminate();                         // terminate sequence
}



/*---------------------------------------------------------------------------

   manual change of maximum value in image scaling

---------------------------------------------------------------------------*/

void __fastcall TMainForm::MaximumEditChange(TObject *Sender)
{
   char                                Str[10];             // temporary string
   int                                 Temp1;               // temporary integer value

   // bypass elaboration is dectect is disabled
   if ( EnableScaligChangeDetect == false )
      return;

   strcpy( Str, MaximumEdit->Text.c_str() );                // get new value
   Temp1 = atoi(Str);                                       // integer value
   // clamp value
   if ( Temp1 > 65536 )
   {
      Temp1 = 65536;
      MaximumEdit->Text = Temp1;
   }
   if ( Temp1 < 0 )
   {
      Temp1 = 0;
      MaximumEdit->Text = Temp1;
   }
   MaximumScrollBar->Position = Temp1;                      // assign value to scroll bar
   UpdateShotImages();                                      // update shot images
   UpdateHistogram();                                       // update histogram image
}



/*---------------------------------------------------------------------------

   Manual change of minimum value in image scaling

---------------------------------------------------------------------------*/

void __fastcall TMainForm::MinimumEditChange(TObject *Sender)
{
   char                                Str[10];             // temporary string
   int                                 Temp1;               // temporary integer value

   // bypass elaboration is dectect is disabled
   if ( EnableScaligChangeDetect == false )
      return;

   strcpy( Str, MinimumEdit->Text.c_str() );                // get new value
   Temp1 = atoi(Str);                                       // integer value
   // clamp value
   if ( Temp1 > 65536 )
   {
      Temp1 = 65536;
      MaximumEdit->Text = Temp1;
   }
   if ( Temp1 < 0 )
   {
      Temp1 = 0;
      MaximumEdit->Text = Temp1;
   }
   MinimumScrollBar->Position = Temp1;                      // assign value to scroll bar
   UpdateShotImages();                                      // update shot images
   UpdateHistogram();                                       // update histogram image
}



/*---------------------------------------------------------------------------

   Manual change of maximum scroll bar image scaling

---------------------------------------------------------------------------*/

void __fastcall TMainForm::MaximumScrollBarChange(TObject *Sender)
{
   MaximumEdit->Text = MaximumScrollBar->Position;          // assign value from scroll bar
}



/*---------------------------------------------------------------------------

   Manual change of minimum scroll bar image scaling

---------------------------------------------------------------------------*/

void __fastcall TMainForm::MinimumScrollBarChange(TObject *Sender)
{
   MinimumEdit->Text = MinimumScrollBar->Position;          // assign value from scroll bar
}



/*---------------------------------------------------------------------------

   Activate flat sequence

---------------------------------------------------------------------------*/

void __fastcall TMainForm::FlatStartButtonClick(TObject *Sender)
{
   float                               ShotTime;            // time for exposure
   int                                 IntShotTime;         // time for exposure format integer
   int                                 FlatFlatTarget;      // number of flat shot
   int                                 FlatDarkTarget;      // number of dark shot

   ShotTime = atof( FlatExposureComboBox->Text.c_str());    // get time
   ShotTime *= 1000;                                        // time in millisec.
   IntShotTime = ShotTime;                                  // format integer

   FlatFlatTarget = atoi( FlatTargetEdit->Text.c_str() );   // flat sequence target
   FlatDarkTarget = atoi( DarkFlatTargetEdit->Text.c_str() ); // dark sequence target

   // check for correct time and shots
   if ( IntShotTime    < 1 || IntShotTime    > 200000 ||
        FlatFlatTarget < 1 || FlatFlatTarget > 50     ||
        FlatDarkTarget < 1 || FlatDarkTarget > 50        )
   {
      MessageBox( this->Handle, "Errore nei parametri sequenza", "Errore", MB_OK );
      return;
   }
   FlatProgEdit->Text = "";                                 // clear progressive target
   DarkFlatProgEdit->Text = "";                             // clear progressive target

   ObjFlatSequence = new FlatSequence();                    // create flat sequence object (it is a observer)
   ObjFlatSequence->SetSubject( MFObserver->GetSubject() ); // subject assignment
   ObjFlatSequence->StartSequence(this->Handle, IntShotTime, FlatFlatTarget, FlatDarkTarget ); // start sequence

   FlatStartButton->Enabled = false;                        // disactivate stat button
   FlatStopButton->Enabled = true;                          // activate stop button

   // record in diary the flat sequence
   if ( Diary != "" )                                       // only if diary is activated
   {
      Diary += "Eseguito flat: Esposizione = " + FlatExposureComboBox->Text + " s. Num flat = " + FlatTargetEdit->Text + " Num dark " + DarkFlatTargetEdit->Text + "\r\n";  // set data
      SaveDiary();                                          // salvo file
   }
}



/*---------------------------------------------------------------------------

   Stop flat sequence

---------------------------------------------------------------------------*/

void __fastcall TMainForm::FlatStopButtonClick(TObject *Sender)
{
   // if shot sequence is running...
   if ( ObjFlatSequence != 0 )
      ObjFlatSequence->Terminate();                         // terminate sequence
}



/*---------------------------------------------------------------------------

   Activate dark sequence

---------------------------------------------------------------------------*/

void __fastcall TMainForm::DarkStartButtonClick(TObject *Sender)
{
   float                               ShotTime;            // time for exposure
   int                                 IntShotTime;         // time for exposure format integer
   int                                 DarkTarget;          // number of dark shot

   ShotTime = atof( DarkExposureComboBox->Text.c_str());    // get time
   ShotTime *= 1000;                                        // time in millisec.
   IntShotTime = ShotTime;                                  // format integer

   DarkTarget = atoi( DarkTargetEdit->Text.c_str() );   // dark sequence target

   // check for correct time and shots
   if ( IntShotTime  < 1 || IntShotTime   > 200000 ||
        DarkTarget   < 1 || DarkTarget    > 50       )
   {
      MessageBox( this->Handle, "Errore nei parametri sequenza", "Errore", MB_OK );
      return;
   }
   DarkProgEdit->Text = "";                                 // clear progressive target

   ObjDarkSequence = new DarkSequence();                    // create flat sequence object (it is a observer)
   ObjDarkSequence->SetSubject( MFObserver->GetSubject() ); // subject assignment
   ObjDarkSequence->StartSequence(this->Handle, IntShotTime, DarkTarget ); // start sequence

   DarkStartButton->Enabled = false;                        // disactivate stat button
   DarkStopButton->Enabled = true;                          // activate stop button

   // record in diary the dark sequence
   if ( Diary != "" )                                       // only if diary is activated
   {
      Diary += "Eseguito dark: Esposizione = " + DarkExposureComboBox->Text + " s. Num dark " + DarkTargetEdit->Text + "\r\n";  // set data
      SaveDiary();                                          // salvo file
   }
}



/*---------------------------------------------------------------------------

   Stop dark sequence

---------------------------------------------------------------------------*/

void __fastcall TMainForm::DarkStopButtonClick(TObject *Sender)
{
   // if shot sequence is running...
   if ( ObjDarkSequence != 0 )
      ObjDarkSequence->Terminate();                         // terminate sequence
}



/*---------------------------------------------------------------------------

   Detect mouse move on shot image

---------------------------------------------------------------------------*/

void __fastcall TMainForm::CCDImageMouseMove(TObject *Sender, TShiftState Shift, int X, int Y)
{
   int                                 MinimumValue;        // minimum value
   int                                 MaximumValue;        // maximum value
   Graphics::TBitmap                   *Bmp;                // zoomed image
   int                                 Ret;                 // retun code value
   int                                 Value;               // value at mouse position

   if ( ZoomLocked == true )
      return;

   MinimumValue = atoi ( MinimumEdit->Text.c_str() );       // get current image scaling values
   MaximumValue = atoi ( MaximumEdit->Text.c_str() );
   Ret = MFObserver->GetSubject()->GetZoomBmp( &Bmp, MinimumValue, MaximumValue, X, Y); // get zoomed image
   if ( Ret != TentaclesMgr::ERR_NONE)                      // if error don't continue
      return;

   ZoomImage->Picture->Bitmap = Bmp;                        // assign bitmap to image
   ZoomX = X;                                               // store zoomed position
   ZoomY = Y;
   PointerXEdit->Text = X;                                  // update coordinates
   PointerYEdit->Text = Y;
   Ret = MFObserver->GetSubject()->GetImageValue( X, Y, &Value ); // get image value at position
   if ( Ret != TentaclesMgr::ERR_NONE)
      return;
   PointerValueEdit->Text = Value;                          // update value
}



/*---------------------------------------------------------------------------

   Mouse click on shot image

---------------------------------------------------------------------------*/

void __fastcall TMainForm::CCDImageMouseDown(TObject *Sender, TMouseButton Button, TShiftState Shift, int X, int Y)
{
   int                                 MinimumValue;        // minimum value
   int                                 MaximumValue;        // maximum value
   Graphics::TBitmap                   *Bmp;                // shot image without star selected
   int                                 Ret;                 // retun code value
   TentaclesMgr::ERR                   Error;               // error code
   TPoint                              Click;               // click point position

   // click with shift toggle zoom lock image
   if ( Shift.Contains(ssShift) )
   {
      ZoomLocked = ! ZoomLocked;                            // toggle zoom look image
      return;
   }

   // click with right button shows shot popup menu
   if ( Button == mbRight )
   {
      Click.x = X;                                          // fill point with mouse position
      Click.y = Y;
      Click = CCDImage->ClientToScreen(Click);              // convert position in screen coordinates
      // enable menu item depending on selections
      TelescopeAllignmentItem->Enabled = StarShotSelected & StarChartSelected;
      ShotMenu->Popup(Click.x, Click.y );                   // shows shot popup menu
      return;
   }

   // left button click select star
   MinimumValue = atoi ( MinimumEdit->Text.c_str() );       // get current image scaling values
   MaximumValue = atoi ( MaximumEdit->Text.c_str() );
   Ret = MFObserver->GetSubject()->GetBmp( &Bmp, MinimumValue, MaximumValue); // get image without any star selectd
   if ( Ret == CCD::ERR_NONE )                              // action only without error
   {
      CCDImage->Picture->Bitmap = Bmp;                      // assign bitmap to shot image
      // select star next click position
      Error = MFObserver->GetSubject()->SelectStarOnShot( &X, &Y, &RAStarShot, &DecStarShot);
      if ( Error == TentaclesMgr::ERR_NONE )                // action only if star found
      {
         CCDImage->Picture->Bitmap->Canvas->Pen->Color = clYellow;   // select selection color
         CCDImage->Picture->Bitmap->Canvas->Brush->Style = bsClear;  // brush don't fill inside circle
         CCDImage->Picture->Bitmap->Canvas->Ellipse(X-SelectionRadius+1,Y-SelectionRadius+1, X+SelectionRadius, Y+SelectionRadius);  // draw circle arround selected star
         StarShotSelected = true;
      }
   }
}



/*---------------------------------------------------------------------------

   Telescope allignment from shot image

---------------------------------------------------------------------------*/

void __fastcall TMainForm::TelescopeAllignmentItemClick(TObject *Sender)
{
   CMHours                             *RACenterObj;        // object with center RA
   CMDegree                            *DecCenterObj;       // object with center declination
   CMHours::HOURS                      RACenter;            // center RA
   CMDegree::DEGREE                    DecCenter;           // center declination
   TentaclesMgr::ERR                   Error;               // error code from devices manager


   RACenterObj = new CMHours();                             // create RA object
   RACenterObj->Set( &RAStarChart );                        // Assign star RA from chart (real value)
   RACenterObj->Sub( &RAStarShot );                         // Subtract star RA due to actual allignment = -allignment error = allignment correction
   RACenterObj->Add( &Status.ShotRA );                      // add correction to actual short center = real allignment value
   RACenter = *RACenterObj->Get();                          // estract RA
   delete RACenterObj;                                      // destroy object

   DecCenterObj = new CMDegree();                           // create declination object
   DecCenterObj->Set( &DecStarChart );                      // Assign star Dec from chart (real value)
   DecCenterObj->Sub( &DecStarShot);                        // Subtract star Dec due to actual allignment = -allignment error = allignment correction
   DecCenterObj->Add( &Status.ShotDec);                     // add correction to actual short center = real allignment value
   DecCenter = *DecCenterObj->Get();                        // estract declination
   delete DecCenterObj;                                     // destroy object

   // allign telescope
   Error = MFObserver->GetSubject()->TelescopePos( MFObserver, &RACenter, &DecCenter, "", true );
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   Move telescope so to center selected star

---------------------------------------------------------------------------*/

void __fastcall TMainForm::StarCenterItemClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error =    MFObserver->GetSubject()->TelescopePos( MFObserver, &RAStarShot, &DecStarShot, "", false );
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   Exec automatic telescope allignment from last shot

---------------------------------------------------------------------------*/

void __fastcall TMainForm::AligmentButtonClick(TObject *Sender)
{
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   Error = MFObserver->GetSubject()->ShotAllignment( MFObserver );
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   Goto chart to input coordinates

---------------------------------------------------------------------------*/

void __fastcall TMainForm::ChartGotoButtonClick(TObject *Sender)
{
   Graphics::TBitmap                   *Bmp;                // chart image
   CMHours::HOURS                      RACenter;            // center RA
   CMDegree::DEGREE                    DecCenter;           // center declination
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   // disable char follows telecope position when function is called by button
   if ( Sender != 0 )
      CharFollowsTelescopeCheckBox->Checked = false;

   // collect coordinates element from inut fields
   RACenter.hh = atoi (ChartRA_hhEdit->Text.c_str() );
   RACenter.mm = atoi (ChartRA_mmEdit->Text.c_str() );
   RACenter.ss = atof (ChartRA_ssEdit->Text.c_str() );

   strcpy( DecCenter.sign, ChartDec_SignEdit->Text.c_str() );
   DecCenter.gg = atoi (ChartDec_ggEdit->Text.c_str() );
   DecCenter.mm = atoi (ChartDec_mmEdit->Text.c_str() );
   DecCenter.ss = atof (ChartDec_ssEdit->Text.c_str() );

   // get bitmap from image
   Bmp = ChartImage->Picture->Bitmap;
   // create chart bitmap
   Error = MFObserver->GetSubject()->GetChart( &Bmp, &RACenter, &DecCenter, ChartField, FrameWidth, FrameHeight);
   RAChart = RACenter;
   DecChart = DecCenter;
   ShowError ( Error );                                     // show error message
   if ( Error == TentaclesMgr::ERR_NONE)
      StarChartSelected = false;                            // remove star selection
}



/*---------------------------------------------------------------------------

   Detect mouse down on chart image

---------------------------------------------------------------------------*/

void __fastcall TMainForm::ChartImageMouseDown(TObject *Sender, TMouseButton Button, TShiftState Shift, int X, int Y)
{
   CMHours::HOURS                      RACenter;            // center RA
   CMDegree::DEGREE                    DecCenter;           // center declination
   TentaclesMgr::ERR                   Error;               // error code from devices manager
   TPoint                              Click;               // click position

   // click with right button shows shot popup menu
   if ( Button == mbRight )
   {
      Click.x = X;                                          // fill point with mouse position
      Click.y = Y;
      Click = ChartImage->ClientToScreen(Click);            // convert position in screen coordinates
      // enable menu item depending on selections
      ChartAllignmentItem->Enabled = StarShotSelected & StarChartSelected;
      LogDeviationItem->Enabled = StarShotSelected & StarChartSelected;
      ChartMenu->Popup(Click.x, Click.y );                  // shows shot popup menu
      return;
   }

   // with CTRL active CCD frame movement
   if ( Shift.Contains(ssCtrl)	)
   {
      // collect coordinates element from inut fields
      RACenter.hh = atoi (ChartRA_hhEdit->Text.c_str() );
      RACenter.mm = atoi (ChartRA_mmEdit->Text.c_str() );
      RACenter.ss = atof (ChartRA_ssEdit->Text.c_str() );

      strcpy( DecCenter.sign, ChartDec_SignEdit->Text.c_str() );
      DecCenter.gg = atoi (ChartDec_ggEdit->Text.c_str() );
      DecCenter.mm = atoi (ChartDec_mmEdit->Text.c_str() );
      DecCenter.ss = atof (ChartDec_ssEdit->Text.c_str() );

      BmpOriginalChart = new Graphics::TBitmap();           // create bitmap for chart without frame
      BmpOriginalChart->Width = ChartImage->Picture->Width; // bitmap size as chart
      BmpOriginalChart->Height = ChartImage->Picture->Height;
      // get chart bitmap without frame
      MFObserver->GetSubject()->GetChart( &BmpOriginalChart, &RACenter, &DecCenter, ChartField, FrameWidth, FrameHeight);
      StarChartSelected = false;                            // remove star selection

      BmpFramedChart = new Graphics::TBitmap();             // create bitmap for chart with frame
      BmpFramedChart->Width = ChartImage->Picture->Width;   // bitmap size as chart
      BmpFramedChart->Height = ChartImage->Picture->Height;

      XMouseDown = X;                                       // store click coordinates
      YMouseDown = Y;
      AdjustChatFrame=true;                                 // ativate chart adjustment with mouse
      return;
   }

   // select star on chart
   // collect coordinates element from inut fields
   RACenter.hh = atoi (ChartRA_hhEdit->Text.c_str() );
   RACenter.mm = atoi (ChartRA_mmEdit->Text.c_str() );
   RACenter.ss = atof (ChartRA_ssEdit->Text.c_str() );

   strcpy( DecCenter.sign, ChartDec_SignEdit->Text.c_str() );
   DecCenter.gg = atoi (ChartDec_ggEdit->Text.c_str() );
   DecCenter.mm = atoi (ChartDec_mmEdit->Text.c_str() );
   DecCenter.ss = atof (ChartDec_ssEdit->Text.c_str() );

   BmpOriginalChart = new Graphics::TBitmap();              // create bitmap for chart without frame
   BmpOriginalChart->Width = ChartImage->Picture->Width;    // bitmap size as chart
   BmpOriginalChart->Height = ChartImage->Picture->Height;
   // get chart bitmap without frame
   MFObserver->GetSubject()->GetChart( &BmpOriginalChart, &RACenter, &DecCenter, ChartField, FrameWidth, FrameHeight);
   ChartImage->Picture->Bitmap = BmpOriginalChart;          // assign to chart
   delete BmpOriginalChart;                                 // delete bitmap

   // select star near coordinates
   Error = MFObserver->GetSubject()->SelectStarOnChart( &X, &Y, &RAStarChart, &DecStarChart);
   if ( Error == TentaclesMgr::ERR_NONE )                   // if found..
   {
      ChartImage->Picture->Bitmap->Canvas->Pen->Color = clYellow;    // star selection color
      ChartImage->Picture->Bitmap->Canvas->Brush->Style = bsClear;   // no paint inside cicle
      ChartImage->Picture->Bitmap->Canvas->Ellipse(X-9,Y-9, X+10, Y+10);   // draw cicle arround star
      StarChartSelected = true;
   }
}



/*---------------------------------------------------------------------------

   Detect mouse move on chart image

---------------------------------------------------------------------------*/

void __fastcall TMainForm::ChartImageMouseMove(TObject *Sender, TShiftState Shift, int X, int Y)
{
   TRect                               RectS;               // source copy rectangle
   TRect                               RectD;               // destination copy rectangle

   // move CCD frame rectangle only if function is activated
   if (AdjustChatFrame == true )
   {
      BmpFramedChart->Canvas->Draw(0,0,BmpOriginalChart );  // initialize framed bitmap with original bitmap

      // compute frame position from actal mouse position and mouse down  position
      RectD.Top = (ChartImage->Height - FrameHeight)/2 + Y - YMouseDown;
      RectD.Left = (ChartImage->Width - FrameWidth)/2 + X - XMouseDown;
      RectD.Bottom = RectD.Top + FrameHeight;
      RectD.Right = RectD.Left + FrameWidth;

      BmpFramedChart->Canvas->Brush->Color = clAqua;        // select color for moving frame
      BmpFramedChart->Canvas->FrameRect(RectD);             // draw frame on bitmap

      // destination rectangle is the whole bitmap
      RectD.Top = 0;
      RectD.Left = 0;
      RectD.Bottom = ChartImage->Height;
      RectD.Right = ChartImage->Width;

      ChartImage->Picture->Bitmap->Canvas->CopyRect(RectD,BmpFramedChart->Canvas,RectD);  // copy bitmap in chart image
   }
}



/*---------------------------------------------------------------------------

   Detect mouse up on chart image

---------------------------------------------------------------------------*/

void __fastcall TMainForm::ChartImageMouseUp(TObject *Sender, TMouseButton Button, TShiftState Shift, int X, int Y)
{
   CMHours                             *RACenterObj;        // object with center RA
   CMDegree                            *DecCenterObj;       // object with center declination
   CMHours::HOURS                      RACenter;            // center RA
   CMDegree::DEGREE                    DecCenter;           // center declination
   double                              Declination;         // declination in degree
   double                              DeltaRA;             // frame movement in RA
   double                              DeltaDec;            // frame movement in declination
   char                                Str[20];             // temporary for conversion
   Graphics::TBitmap                   *Bmp;                // chart image
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   // if not ativate chart adjustment exit
   if (AdjustChatFrame == false )
      return;

   AdjustChatFrame=false;                                   // stop chart adjustment
   delete BmpOriginalChart;                                 // delete used bitmap
   delete BmpFramedChart;

   // collect actual chart coordinates from inut fields
   RACenter.hh = atoi (ChartRA_hhEdit->Text.c_str() );
   RACenter.mm = atoi (ChartRA_mmEdit->Text.c_str() );
   RACenter.ss = atof (ChartRA_ssEdit->Text.c_str() );

   strcpy( DecCenter.sign, ChartDec_SignEdit->Text.c_str() );
   DecCenter.gg = atoi (ChartDec_ggEdit->Text.c_str() );
   DecCenter.mm = atoi (ChartDec_mmEdit->Text.c_str() );
   DecCenter.ss = atof (ChartDec_ssEdit->Text.c_str() );

   // create objects with actual chart coordinates
   RACenterObj = new CMHours();
   RACenterObj->Set( &RACenter );
   DecCenterObj = new CMDegree();
   DecCenterObj->Set( &DecCenter );

   Declination = DecCenterObj->GetC();                      // get declination in degreee

   // compute movement in RA
   DeltaRA = XMouseDown - X;                                // compute horizontal movement in pixel
   DeltaRA *= ChartField;                                   // scale pixel in arcsec (at equator)
   DeltaRA /= ChartImage->Width;
   DeltaRA /= 3600;                                         // convert in degree (at equator)
   DeltaRA /= 15;                                           // convert in hours (at equator)
   DeltaRA /= cos( Declination / RAD );                     // convert in hous at declination ( approximate)

   // compute new RA
   RACenterObj->Add( DeltaRA );                             // add movement to actual chart center
   RACenter = *RACenterObj->Get();                          // get new chart center
   ChartRA_hhEdit->Text = RACenter.hh;                      // assign new hour to input field
   ChartRA_mmEdit->Text = RACenter.mm;                      // assign new minute to input field
   gcvt( RACenter.ss, 3, Str );                             // convert new second with only one decimal
   ChartRA_ssEdit->Text = Str;                              // assign new second to input field

   // compute movement in declination
   DeltaDec = YMouseDown - Y;                               // compute vertical movement in pixel
   DeltaDec *= ChartField;                                  // scale pixel in arcsec
   DeltaDec /= ChartImage->Width;
   DeltaDec /= 3600;                                        // convert in degree

   // compute new declination
   DecCenterObj->Add( DeltaDec );                           // add movement to actual chart center
   DecCenter = *DecCenterObj->Get();                        // get new chart center
   ChartDec_SignEdit->Text = DecCenter.sign;                // assign new sign to input field
   ChartDec_ggEdit->Text = DecCenter.gg;                    // assign new degree to input field
   ChartDec_mmEdit->Text = DecCenter.mm;                    // assign new minute to input field
   gcvt( DecCenter.ss, 3, Str );                            // convert new second with only one decimal
   ChartDec_ssEdit->Text = Str;                             // assign new second to input field
   delete DecCenterObj;
   delete RACenterObj;

   // get bitmap from image
   Bmp = ChartImage->Picture->Bitmap;

   // create chart bitmap
   Error = MFObserver->GetSubject()->GetChart( &Bmp, &RACenter, &DecCenter, ChartField, FrameWidth, FrameHeight);
   RAChart = RACenter;
   DecChart = DecCenter;
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   Chart allignment from shot

---------------------------------------------------------------------------*/

void __fastcall TMainForm::ChartAllignmentFromShot(TObject *Sender)
{
   CMHours                             *RACenterObj;        // object with center RA
   CMDegree                            *DecCenterObj;       // object with center declination
   CMHours::HOURS                      RACenter;            // center RA
   CMDegree::DEGREE                    DecCenter;           // center declination
   char                                Str[20];             // temporary for conversion
   Graphics::TBitmap                   *Bmp;                // chart image
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   RACenterObj = new CMHours();                             // create RA object
   RACenterObj->Set( &RAStarChart );                        // Assign star RA from chart
   RACenterObj->Sub( &RAStarShot );                         // compute difference between RA from chart and RA from telescope (chart allignment error)
   RACenterObj->Add( &Status.ShotRA );                      // add actual telescope position
   RACenter = *RACenterObj->Get();                          // estract RA
   delete RACenterObj;                                      // destroy object

   DecCenterObj = new CMDegree();                            // create declination object
   DecCenterObj->Set( &DecStarChart );                      // Assign star declination from chart
   DecCenterObj->Sub( &DecStarShot);                        // compute difference between dec. from chart and dec. from telescope (chart allignment error)
   DecCenterObj->Add( &Status.ShotDec);                     // add actual telescope position
   DecCenter = *DecCenterObj->Get();                        // estract declination
   delete DecCenterObj;                                     // destroy object

   // get bitmap from image
   Bmp = ChartImage->Picture->Bitmap;
   // create chart bitmap
   Error = MFObserver->GetSubject()->GetChart( &Bmp, &RACenter, &DecCenter, ChartField, FrameWidth, FrameHeight);
   if ( Error == TentaclesMgr::ERR_NONE)
      StarChartSelected = false;                            // remove star selection

   RAChart = RACenter;
   DecChart = DecCenter;

   ChartRA_hhEdit->Text = RACenter.hh;                      // assign new hour to input field
   ChartRA_mmEdit->Text = RACenter.mm;                      // assign new minute to input field
   gcvt( RACenter.ss, 3, Str );                             // convert new second with only one decimal
   ChartRA_ssEdit->Text = Str;                              // assign new second to input field

   ChartDec_SignEdit->Text = DecCenter.sign;                // assign new sign to input field
   ChartDec_ggEdit->Text = DecCenter.gg;                    // assign new degree to input field
   ChartDec_mmEdit->Text = DecCenter.mm;                    // assign new minute to input field
   gcvt( DecCenter.ss, 3, Str );                            // convert new second with only one decimal
   ChartDec_ssEdit->Text = Str;                             // assign new second to input field
}



/*---------------------------------------------------------------------------

   Log deviation between coordinate from shot and chart

---------------------------------------------------------------------------*/

void __fastcall TMainForm::LogDeviation(TObject *Sender)
{
   char                                Message[200];        // message with log information
   char                                Str[80];             // temporary string
   double                              ARDev;               // deviation in RA in arcsec
   double                              DecDev;              // deviation in declination in arcsec
   CMHours                             *RAObj;              // temporary RA object
   CMDegree                            *DecObj;             // temporary declination object
   int                                 RA_ss;               // RA second in integer format
   int                                 Dec_ss;              // declination second in integer format

   // generate shot position message
   strcpy( Message, "Pos.foto : ");
   RA_ss = RAStarShot.ss;
   Dec_ss = DecStarShot.ss;
   wsprintf( Str, "%2dh %2dm %2ds %s %2d %2d' %2d\"", RAStarShot.hh, RAStarShot.mm, RA_ss,
                                                       DecStarShot.sign, DecStarShot.gg, DecStarShot.mm, Dec_ss );
   strcat( Message, Str );

   // generate chart position message
   strcat( Message, " Pos.carta : ");
   RA_ss = RAStarChart.ss;
   Dec_ss = DecStarChart.ss;
   wsprintf( Str, "%2dh %2dm %2ds %s %2d %2d' %2d\"", RAStarChart.hh, RAStarChart.mm, RA_ss,
                                                       DecStarChart.sign, DecStarChart.gg, DecStarChart.mm, Dec_ss );
   strcat( Message, Str );

   // compute AR deviation
   RAObj = new CMHours();                                   // create object
   RAObj->Set( &RAStarChart );                              // assign position from chart
   RAObj->Sub( &RAStarShot );                               // subtract position from shot
   ARDev = RAObj->GetC();                                   // get difference ( in hours)
   if ( ARDev > 12 )                                        // adapt sign
      ARDev = ARDev -24;
   ARDev *= 15;                                             // convert in degree
   ARDev *= 3600;                                           // convert in arcsec
   delete RAObj;                                            // destoy object

   DecObj = new CMDegree();                                 // create object
   DecObj->Set( &DecStarChart );                            // assign position from chart
   DecObj->Sub( &DecStarShot);                              // subtract position from shot
   DecDev = DecObj->GetC();                                 // get difference
   DecDev *= 3600;                                          // convert in arcsec
   delete DecObj;                                           // destoy object

   // generate deviation message
   strcat( Message, " Dif.foto AR = " );
   gcvt( ARDev, 4, Str );
   strcat( Message, Str );
   strcat( Message, " dec = " );
   gcvt( DecDev, 4, Str );
   strcat( Message, Str );

   // add message to list
   ServiceMemo->Lines->Add(Message);
}



/*---------------------------------------------------------------------------

   Update object list depending on class type

---------------------------------------------------------------------------*/

void __fastcall TMainForm::UpdateObjectList(TObject *Sender)
{
   TentaclesMgr::OBJECT_CLASS          Class;               // selection class
   CMMoon                              *Moon;               // Moon object
   CMObjectList                        *List;               // star list
   CMObject*                           Obj;                 // generig object
   CMDeepSky                           *DeepSky;            // deep sky object
   CMHours::HOURS                      RA;                  // right ascensione
   CMDegree::DEGREE                    Dec;                 // declination
   AnsiString                          CatalogName;         // element list
   AnsiString                          OwnName;             // element list
   AnsiString                          Element;             // element list
   char                                Str[10];             // temporary string
   CMDegree::DEGREE                    Elevation;           // object elevation
   CMEvent                             *Now;                // current data and time event object
   CMEvent::DATE                       NowDate;             // current data and time event data
   SYSTEMTIME                          Time;                // current data and time
   double                              NowJD;               // current julian day
   const CMCoEquatorial               *EquPos;              // object equatorial position
   CMCoHorizontal                     *HorPos;              // object horizontal position
   bool                                Skip;                // object to skip

   // assign object class depending on witch radiobutton is pressed
   Class = TentaclesMgr::OBJECT_REFSTARS;
   if ( ObjectTypeComboBox->ItemIndex == 1 )
      Class = TentaclesMgr::OBJECT_PLANETS;
   if ( ObjectTypeComboBox->ItemIndex == 2 )
      Class = TentaclesMgr::OBJECT_MINORPLANETS;
   if ( ObjectTypeComboBox->ItemIndex == 3 )
      Class = TentaclesMgr::OBJECT_MULTISTARS;
   if ( ObjectTypeComboBox->ItemIndex == 4 )
      Class = TentaclesMgr::OBJECT_CLUSTERS;
   if ( ObjectTypeComboBox->ItemIndex == 5 )
      Class = TentaclesMgr::OBJECT_NEBULAS;
   if ( ObjectTypeComboBox->ItemIndex == 6 )
      Class = TentaclesMgr::OBJECT_GALAXIES;
   if ( ObjectTypeComboBox->ItemIndex == 7 )
   {
      Class = TentaclesMgr::OBJECT_NGC;
      NGCTypeComboBox->Visible = true;
   }
   else
      NGCTypeComboBox->Visible = false;

   // enable/disable add/remove buttons
   AddButton->Enabled = ObjectTypeComboBox->ItemIndex == 2;
   RemoveButton->Enabled = AddButton->Enabled;

   // get object list of object class selected
   List = MFObserver->GetSubject()->GetObjectList(Class);

   // erase current information
   ObjectInfoMemo->Lines->Clear();

   // compute current
   Now = new CMEvent();                                     // create event object
   GetSystemTime( &Time );                                  // get current GMT time
   NowDate.Year = Time.wYear;                               // fill event structure
   NowDate.Month = Time.wMonth;
   NowDate.Day = Time.wDay;
   NowDate.Hour = Time.wHour;
   NowDate.Minute = Time.wMinute;
   NowDate.Sec = Time.wSecond;
   Now->SetDate(&NowDate);                                  // set event object
   NowJD = Now->GetGG();                                    // get current julian day
   delete Now;                                              // destroy object

   // clear old list
   ObjListBox->Items->Clear();

   // build new list
   if ( List != NULL )
   {
      List->First();                                        // reset internal cursor list
      Obj = List->Next();                                   // get first object
      while (Obj != 0 )                                     // works only until object is avalable
      {
         EquPos = Obj->GetPosition();                       // get equatorial position
         EquPos = EquPos->ChangeEqu( NowJD );               // get current equinox
         Moon = dynamic_cast<CMMoon*> ( Obj );
         if ( Moon != 0 )
            EquPos = EquPos->GetCoTopo(NowJD, Site);

         HorPos = EquPos->GetCoOri( NowJD, Site, true);     // get horizontal position
         Elevation = *HorPos->GetElevation()->Get();        // get elevation
         Skip = false;                                      // object not to skip

         // filter NGC object type
         if ( ObjectTypeComboBox->ItemIndex == 7 && NGCTypeComboBox->ItemIndex != 0 )
         {
            DeepSky = dynamic_cast<CMDeepSky*>(Obj);        // is it a deep skip object
            if ( DeepSky != 0 )
            {
               if ( DeepSky->GetType() != (NGCTypeComboBox->ItemIndex -1) )
                  Skip = true;
            }
         }
         // filter over horizon if enabled
         if ( Elevation.sign[0] == '-' && OnlyVisibleCheckBox->Checked == true )
            Skip = true;

         // filtered ?
         if ( Skip == false )
         {
            RA = *EquPos->GetRA()->Get();                   // get object RA
            Dec = *EquPos->GetDec()->Get();                 // get object declination
            OwnName = Obj->GetName(CMObject::NAME_OWN);     // get object own name
            CatalogName = Obj->GetName(CMObject::NAME_CATALOG); // get catalog name


            Element = CatalogName;
            if ( Element != "" )                            // if not present ..
               Element += " ";
            Element += OwnName;
            if ( Element.Length() > 19 )
               Element = Element.SubString(1,19);
            // build message
            while ( Element.Length() < 19 )                 // fill space to allign coordinates
               Element += " ";
            Element += " ";
            itoa( 100 + RA.hh, Str, 10);                    // add AR coordinate at message
            Element += AnsiString( Str+1);
            Element += "h ";
            itoa( 100 + RA.mm, Str, 10);
            Element += AnsiString( Str+1);
            Element += "m ";
            itoa( 100 + RA.ss, Str, 10);
            Element += AnsiString( Str+1);
            Element += "s ";

            Element += Dec.sign;                            // add declination coordinate at message
            itoa( 100 + Dec.gg, Str, 10);
            Element += AnsiString( Str+1);
            Element += " ";
            itoa( 100 + Dec.mm, Str, 10);
            Element += AnsiString( Str+1);
            Element += "m ";
            itoa( 100 + Dec.ss, Str, 10);
            Element += AnsiString( Str+1);
            Element += "s  ";

            Element += Elevation.sign;                      // add evelation at message
            itoa( 100 + Elevation.gg, Str, 10);
            Element += AnsiString( Str+1);
            Element += " ";

            // put message in the list
            ObjListBox->Items->AddObject(Element, (TObject*)Obj);
         }
         Obj = List->Next();                                // go to next object in the list
         delete HorPos;                                     // delete horizontal object
      }
   }
}



/*---------------------------------------------------------------------------

   detect selection change on objct list

---------------------------------------------------------------------------*/

void __fastcall TMainForm::ObjListBoxClick(TObject *Sender)
{
   int                                 SelectedItem;        // selected item in list box
   CMObject                            *SelectedObj;        // selected obejct in the listbos
   const CMCoEquatorial               *Position;            // equatorial position
   CMHours::HOURS                      RA;                  // right ascension
   CMDegree::DEGREE                    Dec;                 // declination
   char                                Str[20];             // temporary string
   SYSTEMTIME                          Time;                // current data and time
   CMEvent                             *Now;                // current data and time event object
   double                              NowJD;               // current julian day
   CMEvent::DATE                       NowDate;             // current data and time event data
   char                                ObjInfo[1000];       // object information

   SelectedItem = ObjListBox->ItemIndex;                    // get number of selected item
   if ( SelectedItem == -1 )                                // something should be selected
      return;

   // get selected object
   SelectedObj = (CMObject*)ObjListBox->Items->Objects[SelectedItem];
   if ( SelectedObj == 0 )                                  // pointer should be valid
      return;

   // compute current
   Now = new CMEvent();                                     // create event object
   GetSystemTime( &Time );                                  // get current GMT time
   NowDate.Year = Time.wYear;                               // fill event structure
   NowDate.Month = Time.wMonth;
   NowDate.Day = Time.wDay;
   NowDate.Hour = Time.wHour;
   NowDate.Minute = Time.wMinute;
   NowDate.Sec = Time.wSecond;
   Now->SetDate(&NowDate);                                  // set event object
   NowJD = Now->GetGG();                                    // get current julian day
   delete Now;                                              // destroy object

   Position = SelectedObj->GetPosition();                   // get equatorial position
   if ( ObjectTypeComboBox->ItemIndex == 2 )
      Position = Position->ChangeEqu( NowJD );

   RA = *Position->GetRA()->Get();                          // get right ascension
   Dec = *Position->GetDec()->Get();                        // get declination

   // put coordinates in the input fields
   gcvt( RA.ss, 3, Str );
   ObjRA_hhEdit->Text = RA.hh;
   ObjRA_mmEdit->Text = RA.mm;
   ObjRA_ssEdit->Text = Str;
   gcvt( Dec.ss, 3, Str );
   ObjDec_SignEdit->Text = Dec.sign;
   ObjDec_ggEdit->Text = Dec.gg;
   ObjDec_mmEdit->Text = Dec.mm;
   ObjDec_ssEdit->Text = Str;

   // get object name
   strcpy( SelectedObject, SelectedObj->GetName(CMObject::NAME_OWN) );
   if ( *SelectedObject == 0 )                               // if not present ..
      strcpy( SelectedObject, SelectedObj->GetName(CMObject::NAME_CATALOG) );

   // get object info
   SelectedObj->GetInfo( ObjInfo, sizeof( ObjInfo) );
   ObjectInfoMemo->Lines->Clear();
   ObjectInfoMemo->Lines->Add(ObjInfo);
}



/*---------------------------------------------------------------------------

   set chart position from object

---------------------------------------------------------------------------*/

void __fastcall TMainForm::ObjCharGotoButtonClick(TObject *Sender)
{
   // disable char follows telecope position
   CharFollowsTelescopeCheckBox->Checked = false;

   // copy object selected input field to chart input field
   ChartRA_hhEdit->Text = ObjRA_hhEdit->Text;
   ChartRA_mmEdit->Text = ObjRA_mmEdit->Text;
   ChartRA_ssEdit->Text = ObjRA_ssEdit->Text;
   ChartDec_SignEdit->Text = ObjDec_SignEdit->Text;
   ChartDec_ggEdit->Text = ObjDec_ggEdit->Text;
   ChartDec_mmEdit->Text = ObjDec_mmEdit->Text;
   ChartDec_ssEdit->Text = ObjDec_ssEdit->Text;

   // simulate chart goto button pressing
   ChartGotoButtonClick( 0 );
}



/*---------------------------------------------------------------------------

   telescope goto position from object

---------------------------------------------------------------------------*/

void __fastcall TMainForm::ObjGotoButtonClick(TObject *Sender)
{
   CMHours::HOURS                      RA;                  // object right ascension
   CMDegree::DEGREE                    Dec;                 // object declination
   TentaclesMgr::ERR                   Error;               // error code from devices manager
   int                                 Velocity;            // velocit

   // fill coordinate structured variabile from input field
   RA.hh = atoi ( ObjRA_hhEdit->Text.c_str() );
   RA.mm = atoi ( ObjRA_mmEdit->Text.c_str() );
   RA.ss = atof ( ObjRA_ssEdit->Text.c_str() );
   strcpy( Dec.sign, ObjDec_SignEdit->Text.c_str() );
   Dec.gg = atoi ( ObjDec_ggEdit->Text.c_str() );
   Dec.mm = atoi ( ObjDec_mmEdit->Text.c_str() );
   Dec.ss = atof ( ObjDec_ssEdit->Text.c_str() );

   // set telescope velocity
   Velocity = atoi ( TelecopeAutoVelocityEdit->Text.c_str() );
   Error = MFObserver->GetSubject()->SetAutoVelocity( MFObserver, Velocity );
   if ( Error != TentaclesMgr::ERR_NONE )
   {
      ShowError ( Error );                                   // show error message
   }
   // telescope goto command
   Error = MFObserver->GetSubject()->TelescopePos( MFObserver, &RA, &Dec, SelectedObject, false );
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   telescope allign  from object

---------------------------------------------------------------------------*/

void __fastcall TMainForm::ObjAllignmentButtonClick(TObject *Sender)
{
   CMHours::HOURS                      RA;                  // object right ascension
   CMDegree::DEGREE                    Dec;                 // object declination
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   // fill coordinate structured variabile from input field
   RA.hh = atoi ( ObjRA_hhEdit->Text.c_str() );
   RA.mm = atoi ( ObjRA_mmEdit->Text.c_str() );
   RA.ss = atof ( ObjRA_ssEdit->Text.c_str() );
   strcpy( Dec.sign, ObjDec_SignEdit->Text.c_str() );
   Dec.gg = atoi ( ObjDec_ggEdit->Text.c_str() );     
   Dec.mm = atoi ( ObjDec_mmEdit->Text.c_str() );
   Dec.ss = atof ( ObjDec_ssEdit->Text.c_str() );

   // telescope allignment command
   Error = MFObserver->GetSubject()->TelescopePos( MFObserver, &RA, &Dec, SelectedObject, true );
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   Load configuration data from ini file

---------------------------------------------------------------------------*/

void TMainForm::LoadConfigurationData( void )
{
   AnsiString                          Path;                // program path
   AnsiString                          FileName;            // configuration file name
   int                                 Pos;                 // delimiter postion
   TIniFile                            *OptFile;            // ini file object
   AnsiString                          Option;              // option from ini file
   int                                 IndexObserver;       // index for observer scanning

   // get program path
   Path = ParamStr(0);                                      // complete executable path
   Pos = Path.LastDelimiter("\\");                          // find last '\' before it there is the path
   Path = Path.SubString(1,Pos);                            // get path

   // create ini file object
   OptFile = new TIniFile(Path + "Polypus.ini");

   //
   // get Observatory options
   //

   // get observatory longitude in time
   Option = OptFile->ReadString( OPTION_OBSERVATORY, "Long_hh", "0");
   ObsLongitude.hh = atoi( Option.c_str() );
   Option = OptFile->ReadString( OPTION_OBSERVATORY, "Long_mm", "0");
   ObsLongitude.mm = atoi( Option.c_str() );
   Option = OptFile->ReadString( OPTION_OBSERVATORY, "Long_ss", "0");
   ObsLongitude.ss = atoi( Option.c_str() );

   // get observatory latitude in degree
   Option = OptFile->ReadString( OPTION_OBSERVATORY, "Lat_sign", "+");
   strcpy ( ObsLatitude.sign, Option.c_str());
   Option = OptFile->ReadString( OPTION_OBSERVATORY, "Lat_gg", "0");
   ObsLatitude.gg = atoi( Option.c_str() );
   Option = OptFile->ReadString( OPTION_OBSERVATORY, "Lat_mm", "0");
   ObsLatitude.mm = atoi( Option.c_str() );
   Option = OptFile->ReadString( OPTION_OBSERVATORY, "Lat_ss", "0");
   ObsLatitude.ss = atoi( Option.c_str() );

   // get telescope and observer list
   Option = OptFile->ReadString( OPTION_OBSERVATORY, "Telescope", "");
   strcpy( Telescope, Option.c_str() );
   Option = OptFile->ReadString( OPTION_OBSERVATORY, "NumObserver", "0");  // get observer number
   NumObserver = atoi( Option.c_str() );
   if ( NumObserver > MAX_OBSERVER )                        // clamp observer number to the maximum number
      NumObserver = MAX_OBSERVER;
   *ObserverList = 0;                                       // clear observer list
   for ( IndexObserver=0; IndexObserver<NumObserver; IndexObserver++ ) // scan all observers
   {
      Option = "Observer" + AnsiString ( IndexObserver+1);  // create key
      Option = OptFile->ReadString( OPTION_OBSERVATORY, Option, ""); // get observer name
      strcpy( Observers[IndexObserver], Option.c_str() );
      if ( strlen(ObserverList) < 60 )                      // observerlist need for fits header it shoud be less then 60 chars
      {
         if ( IndexObserver > 0 )
            strcat( ObserverList, "," );                    // separator
         strcat(ObserverList, Observers[IndexObserver] );   // concat name
      }
   }
   ObserverList[60] = 0;                                    // cat lenght

   //
   // get directories options
   //

   Option = OptFile->ReadString( OPTION_DIRECTORIES, "ShotFolder", "");
   strcpy( ShotFolder, Option.c_str() );
   Option = OptFile->ReadString( OPTION_DIRECTORIES, "DarkFolder", "");
   strcpy( DarkFolder, Option.c_str() );
   Option = OptFile->ReadString( OPTION_DIRECTORIES, "GSCFolder", "");
   strcpy( GSCFolder, Option.c_str() );
   Option = OptFile->ReadString( OPTION_DIRECTORIES, "GSCResolutionFolder", "");
   strcpy( GSCResolutionFolder, Option.c_str() );
   Option = OptFile->ReadString( OPTION_DIRECTORIES, "ObjectsFolder", "");
   strcpy( ObjectsFolder, Option.c_str() );
   Option = OptFile->ReadString( OPTION_DIRECTORIES, "ScriptFolder", "");
   strcpy( ScriptFolder, Option.c_str() );
   Option = OptFile->ReadString( OPTION_DIRECTORIES, "SNFolder", "");
   strcpy( SNFolder, Option.c_str() );

   //
   // get CCD options
   //

   Option = OptFile->ReadString( OPTION_CCD, "Name", "");
   strcpy( CCDName, Option.c_str() );
   Option = OptFile->ReadString( OPTION_CCD, "Width", "0");
   FrameWidth = atoi( Option.c_str() );
   Option = OptFile->ReadString( OPTION_CCD, "Height", "0");
   FrameHeight = atoi( Option.c_str() );
   Option = OptFile->ReadString( OPTION_CCD, "PixelScale", "1");
   PixelScale = atof( Option.c_str() );

   //
   // get GRB options
   //

   Option = OptFile->ReadString( OPTION_GRB, "Polling", "60");
   PollingEdit->Text = Option;
   Option = OptFile->ReadString( OPTION_GRB, "MinElevation", "20");
   MinElevation = atoi( Option.c_str() );
   Option = OptFile->ReadString( OPTION_GRB, "MaxDelay", "60");
   MaxDelay = atoi( Option.c_str() );

   //
   // get miscellaneus options
   //

   Option = OptFile->ReadString( OPTION_MISC, "SerialPower", "1");
   SerialPower = atoi( Option.c_str() );
   Option = OptFile->ReadString( OPTION_MISC, "SPortNumber", "1");
   SPortNumber = atoi( Option.c_str() );
   Option = OptFile->ReadString( OPTION_MISC, "SPortBaudRate", "9600");
   SPortBaudRate = atoi( Option.c_str() );
   Option = OptFile->ReadString( OPTION_MISC, "SelectionRadius", "10");
   SelectionRadius = atoi( Option.c_str() );
   Option = OptFile->ReadString( OPTION_MISC, "CommTimeout", "0");
   EnableCommTimeout = atoi( Option.c_str() ) == 1;
   Option = OptFile->ReadString( OPTION_MISC, "Diary", "0");
   EnableDiary = atoi( Option.c_str() ) == 1;
   Option = OptFile->ReadString( OPTION_GRB, "SafeElevation", "20");
   SafeElevation = atoi( Option.c_str() );
   Option = OptFile->ReadString( OPTION_GRB, "MaxMagResolution", "17");
   MaxMagResolution = atoi( Option.c_str() );
   Option = OptFile->ReadString( OPTION_MISC, "PEC", "0");
   EnablePEC = atoi( Option.c_str() ) == 1;
   Option = OptFile->ReadString( OPTION_MISC, "PECAmplitude", "0");
   PECAmplitude = atoi( Option.c_str() );
   Option = OptFile->ReadString( OPTION_MISC, "PECPhase", "0");
   PECPhase = atoi( Option.c_str() );

   delete OptFile;                                          // destroy object
}



/*---------------------------------------------------------------------------

   Save configuration data to ini file

---------------------------------------------------------------------------*/

void TMainForm::SaveConfigurationData( void )
{
   AnsiString                             Path;             // program path
   AnsiString                             FileName;         // configuration file name
   int                                    Pos;              // delimiter postion
   TIniFile                               *OptFile;         // ini file object
   AnsiString                             Option;           // option from ini file
   int                                    IndexObserver;    // index for observer scanning

   // get program path
   Path = ParamStr(0);                                      // complete executable path
   Pos = Path.LastDelimiter("\\");                          // find last '\' before it there is the path
   Path = Path.SubString(1,Pos);                            // get path

   // create ini file object
   OptFile = new TIniFile(Path + "Polypus.ini");

   // save Observatory options
   OptFile->WriteString( OPTION_OBSERVATORY, "Long_hh", ObsLongitude.hh);
   OptFile->WriteString( OPTION_OBSERVATORY, "Long_mm", ObsLongitude.mm);
   OptFile->WriteString( OPTION_OBSERVATORY, "Long_ss", ObsLongitude.ss);
   OptFile->WriteString( OPTION_OBSERVATORY, "Lat_sign", ObsLatitude.sign);
   OptFile->WriteString( OPTION_OBSERVATORY, "Lat_gg", ObsLatitude.gg);
   OptFile->WriteString( OPTION_OBSERVATORY, "Lat_mm", ObsLatitude.mm);
   OptFile->WriteString( OPTION_OBSERVATORY, "Lat_ss", ObsLatitude.ss);
   OptFile->WriteString( OPTION_OBSERVATORY, "Telescope", Telescope);
   OptFile->WriteString( OPTION_OBSERVATORY, "NumObserver", NumObserver);  
   for ( IndexObserver=0; IndexObserver<NumObserver; IndexObserver++ ) // scan all observers
   {
      Option = "Observer" + AnsiString ( IndexObserver+1);  // create key
      OptFile->WriteString( OPTION_OBSERVATORY, Option, Observers[IndexObserver]); // set observer name
   }

   // save directories options
   OptFile->WriteString( OPTION_DIRECTORIES, "ShotFolder", ShotFolder);
   OptFile->WriteString( OPTION_DIRECTORIES, "DarkFolder", DarkFolder);
   OptFile->WriteString( OPTION_DIRECTORIES, "GSCFolder", GSCFolder);
   OptFile->WriteString( OPTION_DIRECTORIES, "GSCResolutionFolder", GSCResolutionFolder);
   OptFile->WriteString( OPTION_DIRECTORIES, "ObjectsFolder", ObjectsFolder);
   OptFile->WriteString( OPTION_DIRECTORIES, "ScriptFolder", ScriptFolder);
   OptFile->WriteString( OPTION_DIRECTORIES, "SNFolder", SNFolder);

   // save CCD options
   OptFile->WriteString( OPTION_CCD, "Name", CCDName);
   OptFile->WriteString( OPTION_CCD, "Width", FrameWidth);
   OptFile->WriteString( OPTION_CCD, "Height", FrameHeight);
   OptFile->WriteString( OPTION_CCD, "PixelScale", PixelScale);

   // save GRB options
   OptFile->WriteString( OPTION_GRB, "Polling", PollingEdit->Text);
   OptFile->WriteString( OPTION_GRB, "MinElevation", MinElevation);
   OptFile->WriteString( OPTION_GRB, "MaxDelay", MaxDelay);

   // save miscellaneus options
   OptFile->WriteString( OPTION_MISC, "SerialPower", SerialPower);
   OptFile->WriteString( OPTION_MISC, "SPortNumber", SPortNumber);
   OptFile->WriteString( OPTION_MISC, "SPortBaudRate", SPortBaudRate);
   OptFile->WriteString( OPTION_MISC, "SelectionRadius", SelectionRadius);
   Option = "0";
   if ( EnableCommTimeout == true )
      Option = "1";
   OptFile->WriteString( OPTION_MISC, "CommTimeout", Option);
   Option = "0";
   if ( EnableDiary == true )
      Option = "1";
   OptFile->WriteString( OPTION_MISC, "Diary", Option);
   OptFile->WriteString( OPTION_GRB, "SafeElevation", SafeElevation);
   OptFile->WriteString( OPTION_GRB, "MaxMagResolution", MaxMagResolution);
   Option = "0";
   if ( EnablePEC == true )
      Option = "1";
   OptFile->WriteString( OPTION_MISC, "PEC", Option);
   OptFile->WriteString( OPTION_MISC, "PECAmplitude", PECAmplitude);
   OptFile->WriteString( OPTION_MISC, "PECPhase", PECPhase);

   delete OptFile;                                          // destroy object
}



/*---------------------------------------------------------------------------

   Load retain data registry

---------------------------------------------------------------------------*/

void TMainForm::LoadRetainData( void )
{
   TRegistry                           *Reg;                // registry obejct
   AnsiString                          Str;                 // temporary string
   AnsiString                          Key;                 // registry key
   int                                 MPNumber;            // number of minor planet
   int                                 MPCounter;           // counter of minor planet

   // open registry Polypus key
   Reg = new TRegistry();
   if ( Reg->OpenKey( "Software\\Polypus", false ) == true )
   {
      ObjectNameEdit->Text = Reg->ReadString("Object");
      ExposureComboBox->Text = Reg->ReadString("Exposure");
      SequenceTargetEdit->Text = Reg->ReadString("SequenceTarget");
      SafeThresholdEdit->Text = Reg->ReadString("SafeThreshold");
      SafeTimeEdit->Text = Reg->ReadString("SafeTime");
      VentilationIntervalEdit->Text = Reg->ReadString("VentilationInterval");
      VentilationTimeEdit->Text = Reg->ReadString("VentilationTime");
      VentilationPauseEdit->Text = Reg->ReadString("VentilationPause");
      FlatExposureComboBox->Text = Reg->ReadString("FlatExposure");
      FlatTargetEdit->Text = Reg->ReadString("FlatTarget");
      DarkFlatTargetEdit->Text = Reg->ReadString("DarkFlatTarget");
      DarkExposureComboBox->Text = Reg->ReadString("DarkExposure");
      DarkTargetEdit->Text = Reg->ReadString("DarkTarget");
      AutoScalingCheckBox->Checked = Reg->ReadString("AutoScaling") == "True";
      SigmaMinusEdit->Text = Reg->ReadString("SigmaMinus");
      SigmaPlusEdit->Text = Reg->ReadString("SigmaPlus");
      MainScriptEdit->Text = Reg->ReadString("MainScript");
      TimeScriptEdit->Text = Reg->ReadString("TimeScript");
      CloseScriptEdit->Text = Reg->ReadString("CloseScript");
      GRBScriptEdit->Text = Reg->ReadString("GRBScript");
      IntervalTimeScriptEdit->Text = Reg->ReadString("IntervalTimeScript");
      EnableTimeScriptCheck->Checked = Reg->ReadString("EnableTimeScript") == "True";
      HourCloseScriptEdit->Text = Reg->ReadString("HourCloseScript");
      MinuteCloseScriptEdit->Text = Reg->ReadString("MinuteCloseScript");

      // get number of minor planet
      Str = Reg->ReadString("MinorPlanet");
      MPNumber = atoi ( Str.c_str() );

      // get minor planet code and put in the list
      for ( MPCounter=1; MPCounter<= MPNumber; MPCounter++ )// scan all key in registry
      {
         Key = "MinorPlanet" + AnsiString( MPCounter );     // create key with counter
         Str = Reg->ReadString(Key);                        // get minor planet key
         MFObserver->GetSubject()->AddMinorPlanet( Str.c_str() );   // inert minor planet in the list
      }
   }
   delete Reg;
}



/*---------------------------------------------------------------------------

   Save retain data registry

---------------------------------------------------------------------------*/

void TMainForm::SaveRetainData( void )
{
   TRegistry                           *Reg;                // registry obejct
   AnsiString                          Str;                 // temporary string
   CMObjectList                        *List;               // star list
   CMObject                            *Obj;                // current minor planet
   AnsiString                          Key;                 // registry key
   int                                 MPCounter;           // counter of minor planet

   // open registry Polypus key
   Reg = new TRegistry();
   if ( Reg->OpenKey( "Software\\Polypus", true ) == true )
   {
      Reg->WriteString( "Object", ObjectNameEdit->Text );
      Reg->WriteString( "Exposure", ExposureComboBox->Text );
      Reg->WriteString( "SequenceTarget", SequenceTargetEdit->Text );
      Reg->WriteString( "SafeThreshold", SafeThresholdEdit->Text );
      Reg->WriteString( "SafeTime", SafeTimeEdit->Text );
      Reg->WriteString( "VentilationInterval", VentilationIntervalEdit->Text );
      Reg->WriteString( "VentilationTime", VentilationTimeEdit->Text );
      Reg->WriteString( "VentilationPause", VentilationPauseEdit->Text );
      Reg->WriteString( "FlatExposure", FlatExposureComboBox->Text );
      Reg->WriteString( "FlatTarget", FlatTargetEdit->Text );
      Reg->WriteString( "DarkFlatTarget", DarkFlatTargetEdit->Text );
      Reg->WriteString( "DarkExposure", DarkExposureComboBox->Text );
      Reg->WriteString( "DarkTarget", DarkTargetEdit->Text );
      Str = "True";
      if ( AutoScalingCheckBox->Checked == false )
         Str = "False";
      Reg->WriteString( "AutoScaling", Str );
      Reg->WriteString( "SigmaMinus", SigmaMinusEdit->Text );
      Reg->WriteString( "SigmaPlus", SigmaPlusEdit->Text );
      Reg->WriteString( "MainScript", MainScriptEdit->Text );
      Reg->WriteString( "TimeScript", TimeScriptEdit->Text );
      Reg->WriteString( "CloseScript", CloseScriptEdit->Text );
      Reg->WriteString( "GRBScript", GRBScriptEdit->Text );
      Reg->WriteString( "IntervalTimeScript", IntervalTimeScriptEdit->Text );
      Str = "True";
      if ( EnableTimeScriptCheck->Checked == false )
         Str = "False";
      Reg->WriteString( "EnableTimeScript", Str );
      Reg->WriteString( "HourCloseScript", HourCloseScriptEdit->Text );
      Reg->WriteString( "MinuteCloseScript", MinuteCloseScriptEdit->Text );

      // get minor planet list
      List = MFObserver->GetSubject()->GetObjectList( TentaclesMgr::OBJECT_MINORPLANETS );
      // save number of minor planet
      Str = List->GetCount();
      Reg->WriteString( "MinorPlanet", Str );

      // scan minor planet and save their code
      List->First();                                        // reset internal cursor list
      MPCounter = 0;                                        // reset counter
      Obj = List->Next();                                   // get first object
      while (Obj != 0 )                                     // works only until object is avalable
      {
         Key = "MinorPlanet" + AnsiString( ++MPCounter );   // create key with counter
         Str = AnsiString ( Obj->GetName(CMObject::NAME_CATALOG ));  // get minor planet code
         Reg->WriteString( Key, Str );                      // write key
         Obj = List->Next();                                // goto next minor planet
      }

   }
   delete Reg;
}



/*---------------------------------------------------------------------------

   Save diary file

---------------------------------------------------------------------------*/

void TMainForm::SaveDiary( void )
{
   char                                FileName[MAXPATH];   // diary file name
   HANDLE                              hFile;               // handle del file
   DWORD                               ToWrite;             // byte da scrivere
   DWORD                               Written;             // byte scritti

   // create file name
   strcpy( FileName, PatShotFolder );
   strcat( FileName, "Diario.txt" );

   // open file
   hFile = CreateFile( FileName, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS,	FILE_ATTRIBUTE_ARCHIVE, NULL );
   if ( hFile == INVALID_HANDLE_VALUE )
      return ;

   // save file
   ToWrite = Diary.Length();                                // string len
   WriteFile ( hFile, Diary.c_str(), ToWrite, &Written, NULL);  // write text

   // close file
   CloseHandle( hFile);
}



/*---------------------------------------------------------------------------

   Load dark list from library

---------------------------------------------------------------------------*/

void TMainForm::LoadDarkList( void )
{
   AnsiString                          Path;                // searching argument
   AnsiString                          FileName;            // file name found
   TSearchRec                          FindRec;             // record for file find operations
   int                                 Status;              // request status
   int                                 Lenght;              // string lenght

   DarkLibListBox->Items->Clear();                          // remove all lines in the list
   Path = AnsiString( DarkFolder ) + "*.*";                 // create seaching path
   Status = FindFirst(Path, faAnyFile, FindRec);            // find the first element
   while ( Status == 0 )                                    // until files
   {
      Lenght = FindRec.Name.Length();                       // total lenght
      if ( Lenght > 6 )
      {
         if ( FindRec.Name.SubString(Lenght-5,6) == "_1.fit" ) // detect only the first dark shot
         {
            FileName = FindRec.Name.SubString(1, Lenght-6); // file name
            DarkLibListBox->Items->Add( FileName );
         }
      }
      Status = FindNext( FindRec );                         // find another file
   }
   FindClose(FindRec);                                      // close search
}



/*---------------------------------------------------------------------------

   Show error from devices manages

---------------------------------------------------------------------------*/

void TMainForm::ShowError ( TentaclesMgr::ERR Error )
{
   SYSTEMTIME                          CurrentGMT;          // current GMT time
   char                                Message[200];        // error message

   // no error no message
   if ( Error == TentaclesMgr::ERR_NONE )
      return;

   // get event timestamp
   GetSystemTime( &CurrentGMT );                            // get current GMT time
   // create date time string
   wsprintf( Message, "%02d:%02d \r\n", CurrentGMT.wHour, CurrentGMT.wMinute );

   // depending on error code shows error message
   switch ( Error )
   {
      case TentaclesMgr::ERR_LPOWER_PAR:
         strcat( Message, "Errore nei parametri di comunicazione linea ON/OFF" );
         break;
      case TentaclesMgr::ERR_LPOWER_OPEN:
         strcat( Message, "Impossibile aprire linea seriale ON/OFF");
         break;
      case TentaclesMgr::ERR_LX200_PAR:
         strcat( Message, "Errore nei parametri di comunicazione LX200" );
         break;
      case TentaclesMgr::ERR_LX200_OPEN:
         strcat( Message, "Impossibile aprire linea seriale LX200");
         break;
      case TentaclesMgr::ERR_COMMAND:
         strcat( Message, "Comando errato" );
         break;
      case TentaclesMgr::ERR_CCDBUSY:
         strcat( Message, "Il CCD  correntemente usato da un altro modulo software, operazione non riuscita");
         break;
      case TentaclesMgr::ERR_OBSERVATORYBUSY:
         strcat( Message, "I dispositivi dell'osservatorio sono correntemente usati da un alto modulo software, operazione non riuscita" );
         break;
      case TentaclesMgr::ERR_TOMANYSHOT:
         strcat( Message, "Raggiunto limite nomi oggetti salvati, operazione non riuscita" );
         break;
      case TentaclesMgr::ERR_NOSHOT:
         strcat( Message, "Avvio fotografia non riuscito" );
         break;
      case TentaclesMgr::ERR_NOABORTSHOT:
         strcat( Message, "Arresto fotografia non riuscito");
         break;
      case TentaclesMgr::ERR_POS:
         strcat( Message, "Posizionamento non riuscito" );
         break;
      case TentaclesMgr::ERR_STARNOTFOUD:
         strcat( Message, "Stella non trovata" );
         break;
      case TentaclesMgr::ERR_NOTRACKING:
         strcat( Message, "Moto orario spento, comando non accettato" );
         break;
      case TentaclesMgr::ERR_NOTFOUND:
         strcat( Message, "Pianetino non trovato" );
         break;
      case TentaclesMgr::ERR_ALREADYPRESENT:
         strcat( Message, "Pianetino gi presente nella lista" );
         break;
      case TentaclesMgr::ERR_SHOTALLIGNMENT:
         strcat( Message, "Allineamento da foto non riuscito" );
         break;
      default:
         strcat( Message, "Errore sconosciuto, operazione non riuscita" );
         break;
   }
   MessageBox( this->Handle, Message, "Errore", MB_OK );
}



/*---------------------------------------------------------------------------

   update shot and zoom images with actual scaling values

---------------------------------------------------------------------------*/

void TMainForm::UpdateShotImages(void)
{
   int                                 MinimumValue;        // minimum value
   int                                 MaximumValue;        // maximum value
   Graphics::TBitmap                   *Bmp;                // new bitmap
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   // get scaling values
   MinimumValue = atoi ( MinimumEdit->Text.c_str() );
   MaximumValue = atoi ( MaximumEdit->Text.c_str() );
   // get shot scaled bitmap
   Error = MFObserver->GetSubject()->GetBmp( &Bmp, MinimumValue, MaximumValue);
   if ( Error == TentaclesMgr::ERR_NONE )
      CCDImage->Picture->Bitmap = Bmp;
   // get zoom scaled image
   Error = MFObserver->GetSubject()->GetZoomBmp( &Bmp, MinimumValue, MaximumValue, ZoomX, ZoomY);
   if ( Error == TentaclesMgr::ERR_NONE )
      ZoomImage->Picture->Bitmap = Bmp;
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   update histogram image with actual scaling values

---------------------------------------------------------------------------*/

void  TMainForm::UpdateHistogram(void)
{
   int                                 MinimumValue;        // minimum value
   int                                 MaximumValue;        // maximum value
   Graphics::TBitmap                   *Bmp;                // new bitmap
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   // get scaling valus
   MinimumValue = atoi ( MinimumEdit->Text.c_str() );
   MaximumValue = atoi ( MaximumEdit->Text.c_str() );
   // get histogram scaled bitmap
   Error = MFObserver->GetSubject()->GetHistogramBmp( &Bmp, MinimumValue, MaximumValue);
   if ( Error == TentaclesMgr::ERR_NONE)
      HistogramImage->Picture->Bitmap = Bmp;
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   Refresh shot information

---------------------------------------------------------------------------*/

void TMainForm::RefreshShot(void)
{
   CCD::SHOT_INFO                      ShotInfo;            // last shot information
   char                                Str[50];             // temporary string
   int                                 NumSigma;            // number of sigma for scaling limit computing
   int                                 Limit;               // scaling (upper and lower ) limit computing

   MFObserver->GetSubject()->GetShotInfo(&ShotInfo);        // get shot information
   // update limit information on image sheet
   MinimumValueEdit->Text = ShotInfo.MinValue;
   MaximumValueEdit->Text = ShotInfo.MaxValue;

   // update statistic information on image sheet
   ImageSigmaEdit->Text = ShotInfo.ImgageSigma;
   BackgrondValueEdit->Text = ShotInfo.BackgrondLevel;
   BackgroundSigmaEdit->Text = ShotInfo.BackgroundSigma;
   gcvt ( ShotInfo.FWHM, 3, Str);                           // Full width at half maximum with 3 digit
   ImageFWHMEdit->Text = Str;
   gcvt ( ShotInfo.VisiblityIndex, 3, Str);                 // visibility index with 3 digit
   VisibilityIndexEdit->Text = Str;

   // update status information on status bar
   VisibilityIndexLabel->Caption = "Ind.Vis." + AnsiString(Str);
   PhotoBackgroundLabel->Caption = "Fondo " + AnsiString(ShotInfo.BackgrondLevel);

   // if required automatic scaling is computed
   if ( AutoScalingCheckBox->Checked == true )
   {
      EnableScaligChangeDetect = false;                     // disable change detect of minimum and maximum values
      NumSigma = atoi ( SigmaMinusEdit->Text.c_str() ) ;    // get minimum sigma number
      Limit = ShotInfo.BackgrondLevel - NumSigma * ShotInfo.BackgroundSigma;   // compute lower limit
      if ( Limit < 0 )                                      // clamp to 0
         Limit = 0;
      MinimumEdit->Text = AnsiString( Limit );              // assign to minimum edit
      MinimumScrollBar->Position = Limit;                   // assign to minimum scroll bar

      NumSigma = atoi ( SigmaPlusEdit->Text.c_str() ) ;     // get maximum sigma number
      Limit = ShotInfo.BackgrondLevel + NumSigma * ShotInfo.BackgroundSigma;   // compute upper limit
      if ( Limit > CCD::VALUE_RANGE )                       // clamp to maximum ccd value
         Limit = CCD::VALUE_RANGE;
      MaximumEdit->Text = AnsiString( Limit );              // assign to maximum edit
      MaximumScrollBar->Position = Limit;                   // assign to maximum scrollbar
      EnableScaligChangeDetect = true;                      // enable change detect of minimum and maximum values
   }
   UpdateShotImages();                                      // update shot images
   UpdateHistogram();                                       // update histogram image
   StarShotSelected = false;                                // star selection in no longer available

}



/*---------------------------------------------------------------------------

   Refresh drift information

---------------------------------------------------------------------------*/

/*void TMainForm::RefreshDrift(void)
{
   CCD::SHOT_INFO                      ShotInfo;            // last shot information
   TentaclesMgr::DRIFT                 *CurrentDriftElement;// drift information
   Graphics::TBitmap                   *Bmp;                // Graph image
   XYGraph::XYGRAPH_POINT              Point;               // Graph point
   int                                 PointNum;            // current point number
   TRect                               Rect;                // rectangle for bitmap drawing
   int                                 Second;              // Second in int format
   char                                Str[20];             // temporary string

   // only if refresh is activated
   if ( DeriveRefreshCheckBox->Checked == false )
      return;

   MFObserver->GetSubject()->GetShotInfo(&ShotInfo);        // get shot information

   // get current drift compensation information
   MFObserver->GetSubject()->GetCurrentDriftElement( &CurrentDriftElement );

   // refresch reference graphs
   GrafHRef->Clear();
   GrafVRef->Clear();
   if ( CurrentDriftElement != 0 )
   {
      for ( PointNum=0; PointNum<ShotInfo.Height; PointNum++ )
      {
         Point.X = PointNum;
         Point.Y = CurrentDriftElement->HImageSumRef[PointNum];
         GrafHRef->Add ( &Point );
      }
      for ( PointNum=0; PointNum<ShotInfo.Width; PointNum++ )
      {
         Point.X = PointNum;
         Point.Y = CurrentDriftElement->VImageSumRef[PointNum];
         GrafVRef->Add ( &Point );
      }
   }
   Bmp = ImageHRef->Picture->Bitmap;
   GrafHRef->GetBmp(&Bmp);
   Bmp = ImageVRef->Picture->Bitmap;
   GrafVRef->GetBmp(&Bmp);

   // refresch comparation graphs
   GrafHCmp->Clear();
   GrafVCmp->Clear();
   if ( CurrentDriftElement != 0 )
   {
      for ( PointNum=0; PointNum<ShotInfo.Height; PointNum++ )
      {
         Point.X = PointNum;
         Point.Y = CurrentDriftElement->HImageSumCmp[PointNum];
         GrafHCmp->Add ( &Point );
      }
      for ( PointNum=0; PointNum<ShotInfo.Width; PointNum++ )
      {
         Point.X = PointNum;
         Point.Y = CurrentDriftElement->VImageSumCmp[PointNum];
         GrafVCmp->Add ( &Point );
      }
   }
   Bmp = ImageHCmp->Picture->Bitmap;
   GrafHCmp->GetBmp(&Bmp);
   Bmp = ImageVCmp->Picture->Bitmap;
   GrafVCmp->GetBmp(&Bmp);

   // refresch correlation graphs
   GrafHCor->Clear();
   GrafVCor->Clear();
   if ( CurrentDriftElement != 0 )
   {
      for ( PointNum = 0; PointNum < (TentaclesMgr::MAX_PIXELCOMP*2+1); PointNum++ )
      {
         Point.X = PointNum - TentaclesMgr::MAX_PIXELCOMP;
         Point.Y = CurrentDriftElement->HImageSumCor[PointNum];
         GrafHCor->Add ( &Point );
      }
      for ( PointNum = 0; PointNum < (TentaclesMgr::MAX_PIXELCOMP*2+1); PointNum++ )
      {
         Point.X = PointNum - TentaclesMgr::MAX_PIXELCOMP;
         Point.Y = CurrentDriftElement->VImageSumCor[PointNum];
         GrafVCor->Add ( &Point );
      }
   }
   Bmp = ImageHCor->Picture->Bitmap;
   GrafHCor->GetBmp(&Bmp);
   Bmp = ImageVCor->Picture->Bitmap;
   GrafVCor->GetBmp(&Bmp);

   // refresch delta pixel and offset
   if ( CurrentDriftElement != 0 )
   {
      DeltaVEdit->Text = CurrentDriftElement->VDelta;
      DeltaHEdit->Text = CurrentDriftElement->HDelta;
      Second = CurrentDriftElement->RAOffset.ss;            // convert second in int format
      wsprintf( Str, "%02dh %02d' %02d\"", CurrentDriftElement->RAOffset.hh, CurrentDriftElement->RAOffset.mm, Second ); // create RA string
      OffsetVEdit->Text = Str;                              // offset RA
      Second = CurrentDriftElement->DecOffset.ss;           // convert second in int format
      wsprintf( Str, "%s%02d %02d' %02d\"", CurrentDriftElement->DecOffset.sign, CurrentDriftElement->DecOffset.gg, CurrentDriftElement->DecOffset.mm, Second );  // create declination string
      OffsetHEdit->Text = Str;                              // offset declination
   }
}         */



/*---------------------------------------------------------------------------

   manages notify messages from observer pattern

---------------------------------------------------------------------------*/

void TMainForm::ObsUpdate( int ANotifyCode )
{
   const char                          *ShotName;           // shot name pointer
   TentaclesMgr::STATUS                Status;              // subject status

   // select notify code
   switch ( ANotifyCode )
   {
      // manages shot file saved
      case TentaclesMgr::NOTIFY_SHOTSAVED:
         ShotName = MFObserver->GetSubject()->GetLastShotName();  // get file name
         ShotNameLabel->Caption = AnsiString ( ShotName );  // show file name on status bar
         break;
      // manages shot image downloaded
      case TentaclesMgr::NOTIFY_SHOTDOWNLOADED:
         // activate refresh information
         // this cannot be done immediatly because this can run in a diffent thread
         // refresh of VCL component should be done in the main thread or sincronized
         EnableRefreshShot = true;
         break;
      case TentaclesMgr::NOTYFY_TELESCOPEPOSITIONED:
      case TentaclesMgr::NOTIFY_TELESCOPEALLIGED:
      case TentaclesMgr::NOTIFY_TELESCOPESHOTALLIGED:
         // activate chart set from telescope position
         // this cannot be done immediatly because this can run in a diffent thread
         // refresh of VCL component should be done in the main thread or sincronized
         if ( CharFollowsTelescopeCheckBox->Checked == true )
            SetChartFromTelescope = 4;
         break;
      // power off observatory
      case TentaclesMgr::NOTIFY_GENERIC_COMMAND:
         MFObserver->GetSubject()->GetStatus( &Status );    // get last command
         switch ( Status.CommandSubject )
         {
            // power management
            case TentaclesMgr::COM_SUBJ_POWER:
               if ( Status.CommandState == TentaclesMgr::COM_STA_OFF &&  Diary != "" )
               {
                  Diary += "Ora fine: " +  TimeLabel->Caption + "\r\n";   // set stating time
                  SaveDiary();
                  Diary="";
               }
               break;
         }
         break;
   }
}



/*---------------------------------------------------------------------------

   Add a minor planet to the list of known objects

---------------------------------------------------------------------------*/

void __fastcall TMainForm::AddMinorPlanet(TObject *Sender)
{
   TForm3                              *DlgMPCode;          // dialog for minorpanet code selection
   int                                 ReturnCode;          // return dialog value
   char                                Code[20];            // code of minor planet to be inserted
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   DlgMPCode = new TForm3(this);                            // create dialog for minorpanet code selection
   ReturnCode = DlgMPCode->ShowModal();                     // show dialof
   if ( ReturnCode == mrOk )                                // if confermed
   {
      strcpy( Code,DlgMPCode->MPlanetCodeEdit->Text.c_str() );    // get minor planet code
      Error = MFObserver->GetSubject()->AddMinorPlanet( Code );   // inert minor planet in the list
      ShowError ( Error );                                  // show error message
      UpdateObjectList(Sender);                             // update object list
   }
   delete DlgMPCode;

}



/*---------------------------------------------------------------------------

   Remove a minor planet from the list of known objects

---------------------------------------------------------------------------*/

void __fastcall TMainForm::RemoveMinorPlanet(TObject *Sender)
{
   int                                 SelectedItem;        // selected item in list box
   CMObject                            *SelectedObj;        // selected obejct in the listbos
   char                                Code[20];            // minor planet code
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   SelectedItem = ObjListBox->ItemIndex;                    // get number of selected item
   if ( SelectedItem == -1 )                                // something should be selected
      return;

   // get selected object
   SelectedObj = (CMObject*)ObjListBox->Items->Objects[SelectedItem];
   if ( SelectedObj == 0 )                                  // pointer should be valid
      return;

   // get object name
   strcpy( Code, SelectedObj->GetName(CMObject::NAME_CATALOG) );
   Error = MFObserver->GetSubject()->RemoveMinorPlanet( Code );   // remove minor planet from the list
   ShowError ( Error );                                     // show error message
   UpdateObjectList(Sender);                                // update object list
}



/*---------------------------------------------------------------------------

   Detect set goto telecope coordinates

---------------------------------------------------------------------------*/

void __fastcall TMainForm::SetGotoCoordinates(TObject *Sender)
{
   *SelectedObject = 0;                                     // clear object name because set by handes
}



/*---------------------------------------------------------------------------

   Open load dialog, select file and load it in the main script edit object

---------------------------------------------------------------------------*/

void __fastcall TMainForm::LoadMainScript(TObject *Sender)
{
   AnsiString                          FileName;            // configuration file name
   int                                 Pos;                 // delimiter postion
   TRegistry                           *Reg;                // registry obejct

   // let use to select a file to define the directory
   OpenDialog->InitialDir = ScriptFolder;
   if ( OpenDialog->Execute() == false )
      return;

   FileName = OpenDialog->FileName;                         // get path file
   Pos = FileName.LastDelimiter("\\");                      // find last '\' before it there is the path
   FileName = FileName.SubString(Pos+1,FileName.Length());  // get file
   Pos = FileName.LastDelimiter(".");                       // find last '.' before it there is the path
   FileName = FileName.SubString(1,Pos-1);                  // get file without extension
   MainScriptEdit->Text = FileName;                         // write on the editbox
   LoadMainScriptFile();                                    // execute file load

   // open registry Polypus key
   Reg = new TRegistry();
   if ( Reg->OpenKey( "Software\\Polypus", true ) == true )
      Reg->WriteString( "MainScript", MainScriptEdit->Text );
   delete Reg;
}



/*---------------------------------------------------------------------------

   Load the GRB script file in edit object

---------------------------------------------------------------------------*/

void TMainForm::LoadGRBScriptFile(void)
{
   char                                FileName[MAXPATH];   // file name
   HANDLE                              hFile;               // file handle
   BY_HANDLE_FILE_INFORMATION          Info;                // file information
   char                                Message[200];        // error message
   DWORD                               ToRead;              // bytes to read
   DWORD                               Read;                // bytes read
   char                                *Buffer;             // file buffer

   // create file name to load
   strcpy( FileName, ScriptFolder );
   strcat( FileName, GRBScriptEdit->Text.c_str() );
   strcat( FileName, ".psc" );

   // open file
   hFile = CreateFile( FileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL );  // open file
   if ( hFile == INVALID_HANDLE_VALUE )                     // if open fails show error dialog
      return;

   GetFileInformationByHandle( hFile, &Info );              // get information for buffer size allocation
   ToRead = Info.nFileSizeLow;                              // file lenght
   Buffer = (char*) malloc( ToRead+1 );                     // allocate buffer for all file plus terminator
   if ( Buffer == 0 )                                       // if not enought memory show error dialog
   {
      strcpy( Message, "Memoria insufficiente per caricare file:\n" );
      strcat( Message, FileName );
      MessageBox( 0, Message, "Errore ", 0);
      CloseHandle( hFile);                                  // close file
      return;
   }

   ReadFile ( hFile, Buffer, ToRead, &Read, NULL);          // read all file in one block
   Buffer[ToRead] = 0;                                      // string terminator
   GRBScriptEditor->SetText( Buffer );                     // assign text to editor
   CloseHandle( hFile);                                     // close file
   free (Buffer);                                           // free buffer memory
}



/*---------------------------------------------------------------------------

   Load the main script file in edit object

---------------------------------------------------------------------------*/

void TMainForm::LoadMainScriptFile(void)
{
   char                                FileName[MAXPATH];   // file name
   HANDLE                              hFile;               // file handle
   BY_HANDLE_FILE_INFORMATION          Info;                // file information
   char                                Message[200];        // error message
   DWORD                               ToRead;              // bytes to read
   DWORD                               Read;                // bytes read
   char                                *Buffer;             // file buffer

   // create file name to load
   strcpy( FileName, ScriptFolder );
   strcat( FileName, MainScriptEdit->Text.c_str() );
   strcat( FileName, ".psc" );

   // open file
   hFile = CreateFile( FileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL );  // open file
   if ( hFile == INVALID_HANDLE_VALUE )                     // if open fails show error dialog
      return;

   GetFileInformationByHandle( hFile, &Info );              // get information for buffer size allocation
   ToRead = Info.nFileSizeLow;                              // file lenght
   Buffer = (char*) malloc( ToRead+1 );                     // allocate buffer for all file plus terminator
   if ( Buffer == 0 )                                       // if not enought memory show error dialog
   {
      strcpy( Message, "Memoria insufficiente per caricare file:\n" );
      strcat( Message, FileName );
      MessageBox( 0, Message, "Errore ", 0);
      CloseHandle( hFile);                                  // close file
      return;
   }

   ReadFile ( hFile, Buffer, ToRead, &Read, NULL);          // read all file in one block
   Buffer[ToRead] = 0;                                      // string terminator
   MainScriptEditor->SetText( Buffer );                     // assign text to editor
   CloseHandle( hFile);                                     // close file
   free (Buffer);                                           // free buffer memory
   CPULoaded = false;
}



/*---------------------------------------------------------------------------

   Save edit object in the GRB script file

---------------------------------------------------------------------------*/

void __fastcall TMainForm::SaveGRBScript(TObject *Sender)
{
   char                                FileName[MAXPATH];   // file name
   HANDLE                              hFile;               // file handle
   char                                Message[200];        // error message
   DWORD                               ToWrite;             // byte da scrivere
   DWORD                               Written;             // byte scritti
   char                                *Buffer;             // file buffer

   // create file name to save
   strcpy( FileName, ScriptFolder );
   strcat( FileName, GRBScriptEdit->Text.c_str() );
   strcat( FileName, ".psc" );

   // open file
   hFile = CreateFile( FileName, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS,	FILE_ATTRIBUTE_ARCHIVE, NULL );
   if ( hFile == INVALID_HANDLE_VALUE )                     // if open fails show error dialog
      return;

   ToWrite = GRBScriptEditor->GetTextLenght()-1;           // text lenght
   Buffer = (char*) malloc( ToWrite+1 );                    // allocate buffer for all text
   GRBScriptEditor->GetText(Buffer, ToWrite+1);            // get text
   if ( Buffer == 0 )                                       // if not enought memory show error dialog
   {
      strcpy( Message, "Memoria insufficiente per salvare file:\n" );
      strcat( Message, FileName );
      MessageBox( 0, Message, "Errore ", 0);
      CloseHandle( hFile);                                  // close file
      free (Buffer);                                        // free buffer memory
      return;
   }

   WriteFile ( hFile, Buffer, ToWrite, &Written, NULL);     // write file
   CloseHandle( hFile);                                     // close file
   free (Buffer);                                           // free buffer memory
   GRBScriptChanged = false;                               // reset flag for change
}


/*---------------------------------------------------------------------------

   Save edit object in the main script file

---------------------------------------------------------------------------*/

void __fastcall TMainForm::SaveMainScript(TObject *Sender)
{
   char                                FileName[MAXPATH];   // file name
   HANDLE                              hFile;               // file handle
   char                                Message[200];        // error message
   DWORD                               ToWrite;             // byte da scrivere
   DWORD                               Written;             // byte scritti
   char                                *Buffer;             // file buffer

   // create file name to save
   strcpy( FileName, ScriptFolder );
   strcat( FileName, MainScriptEdit->Text.c_str() );
   strcat( FileName, ".psc" );

   // open file
   hFile = CreateFile( FileName, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS,	FILE_ATTRIBUTE_ARCHIVE, NULL );
   if ( hFile == INVALID_HANDLE_VALUE )                     // if open fails show error dialog
      return;

   ToWrite = MainScriptEditor->GetTextLenght()-1;           // text lenght
   Buffer = (char*) malloc( ToWrite+1 );                    // allocate buffer for all text
   MainScriptEditor->GetText(Buffer, ToWrite+1);            // get text
   if ( Buffer == 0 )                                       // if not enought memory show error dialog
   {
      strcpy( Message, "Memoria insufficiente per salvare file:\n" );
      strcat( Message, FileName );
      MessageBox( 0, Message, "Errore ", 0);
      CloseHandle( hFile);                                  // close file
      free (Buffer);                                        // free buffer memory
      return;
   }

   WriteFile ( hFile, Buffer, ToWrite, &Written, NULL);     // write file
   CloseHandle( hFile);                                     // close file
   free (Buffer);                                           // free buffer memory
   MainScriptChanged = false;                               // reset flag for change
}



/*---------------------------------------------------------------------------

   Open load dialog, select file and load it in the time script edit object

---------------------------------------------------------------------------*/

void __fastcall TMainForm::LoadTimeScript(TObject *Sender)
{
   AnsiString                          FileName;            // configuration file name
   int                                 Pos;                 // delimiter postion
   TRegistry                           *Reg;                // registry obejct

   // let use to select a file to define the directory
   OpenDialog->InitialDir = ScriptFolder;
   if ( OpenDialog->Execute() == false )
      return;

   FileName = OpenDialog->FileName;                         // get path file
   Pos = FileName.LastDelimiter("\\");                      // find last '\' before it there is the path
   FileName = FileName.SubString(Pos+1,FileName.Length());  // get file
   Pos = FileName.LastDelimiter(".");                       // find last '.' before it there is the path
   FileName = FileName.SubString(1,Pos-1);                  // get file without extension
   TimeScriptEdit->Text = FileName;                         // write on the editbox
   LoadTimeScriptFile();                                    // execute file load
   CPULoaded = false;

   // open registry Polypus key
   Reg = new TRegistry();
   if ( Reg->OpenKey( "Software\\Polypus", true ) == true )
      Reg->WriteString( "TimeScript", TimeScriptEdit->Text );
   delete Reg;
}



/*---------------------------------------------------------------------------

   Load the time script file in edit object

---------------------------------------------------------------------------*/

void TMainForm::LoadTimeScriptFile(void)
{
   char                                FileName[MAXPATH];   // file name
   HANDLE                              hFile;               // file handle
   BY_HANDLE_FILE_INFORMATION          Info;                // file information
   char                                Message[200];        // error message
   DWORD                               ToRead;              // bytes to read
   DWORD                               Read;                // bytes read
   char                                *Buffer;             // file buffer

   // create file name to load
   strcpy( FileName, ScriptFolder );
   strcat( FileName, TimeScriptEdit->Text.c_str() );
   strcat( FileName, ".psc" );

   // open file
   hFile = CreateFile( FileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL );  // open file
   if ( hFile == INVALID_HANDLE_VALUE )                     // if open fails show error dialog
      return;

   GetFileInformationByHandle( hFile, &Info );              // get information for buffer size allocation
   ToRead = Info.nFileSizeLow;                              // file lenght
   Buffer = (char*) malloc( ToRead+1 );                     // allocate buffer for all file plus terminator
   if ( Buffer == 0 )                                       // if not enought memory show error dialog
   {
      strcpy( Message, "Memoria insufficiente per caricare file:\n" );
      strcat( Message, FileName );
      MessageBox( 0, Message, "Errore ", 0);
      CloseHandle( hFile);                                  // close file
      return;
   }

   ReadFile ( hFile, Buffer, ToRead, &Read, NULL);          // read all file in one block
   Buffer[ToRead] = 0;                                      // string terminator
   TimeScriptEditor->SetText( Buffer );                     // assign text to editor
   CloseHandle( hFile);                                     // close file
   free (Buffer);                                           // free buffer memory
   CPULoaded = false;
}



/*---------------------------------------------------------------------------

   Save edit object in the time script file

---------------------------------------------------------------------------*/

void __fastcall TMainForm::SaveTimeScript(TObject *Sender)
{
   char                                FileName[MAXPATH];   // file name
   HANDLE                              hFile;               // file handle
   char                                Message[200];        // error message
   DWORD                               ToWrite;             // byte da scrivere
   DWORD                               Written;             // byte scritti
   char                                *Buffer;             // file buffer

   // create file name to save
   strcpy( FileName, ScriptFolder );
   strcat( FileName, TimeScriptEdit->Text.c_str() );
   strcat( FileName, ".psc" );

   // open file
   hFile = CreateFile( FileName, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS,	FILE_ATTRIBUTE_ARCHIVE, NULL );
   if ( hFile == INVALID_HANDLE_VALUE )                     // if open fails show error dialog
      return;

   ToWrite = TimeScriptEditor->GetTextLenght()-1;           // text lenght
   Buffer = (char*) malloc( ToWrite+1 );                    // allocate buffer for all text
   TimeScriptEditor->GetText(Buffer, ToWrite+1);            // get text
   if ( Buffer == 0 )                                       // if not enought memory show error dialog
   {
      strcpy( Message, "Memoria insufficiente per salvare file:\n" );
      strcat( Message, FileName );
      MessageBox( 0, Message, "Errore ", 0);
      CloseHandle( hFile);                                  // close file
      free (Buffer);                                        // free buffer memory
      return;
   }

   WriteFile ( hFile, Buffer, ToWrite, &Written, NULL);     // write file
   CloseHandle( hFile);                                     // close file
   free (Buffer);                                           // free buffer memory
   TimeScriptChanged = false;                               // reset flag for change
}



/*---------------------------------------------------------------------------

   Open load dialog, select file and load it in the close script edit object

---------------------------------------------------------------------------*/

void __fastcall TMainForm::LoadCloseScript(TObject *Sender)
{
   AnsiString                          FileName;            // configuration file name
   int                                 Pos;                 // delimiter postion
   TRegistry                           *Reg;                // registry obejct

   // let use to select a file to define the directory
   OpenDialog->InitialDir = ScriptFolder;
   if ( OpenDialog->Execute() == false )
      return;

   FileName = OpenDialog->FileName;                         // get path file
   Pos = FileName.LastDelimiter("\\");                      // find last '\' before it there is the path
   FileName = FileName.SubString(Pos+1,FileName.Length());  // get file
   Pos = FileName.LastDelimiter(".");                       // find last '.' before it there is the path
   FileName = FileName.SubString(1,Pos-1);                  // get file without extension
   CloseScriptEdit->Text = FileName;                        // write on the editbox
   LoadCloseScriptFile();                                   // execute file load

   // open registry Polypus key
   Reg = new TRegistry();
   if ( Reg->OpenKey( "Software\\Polypus", true ) == true )
      Reg->WriteString( "CloseScript", CloseScriptEdit->Text );
   delete Reg;
}



/*---------------------------------------------------------------------------

   Load the close script file in edit object

---------------------------------------------------------------------------*/

void TMainForm::LoadCloseScriptFile(void)
{
   char                                FileName[MAXPATH];   // file name
   HANDLE                              hFile;               // file handle
   BY_HANDLE_FILE_INFORMATION          Info;                // file information
   char                                Message[200];        // error message
   DWORD                               ToRead;              // bytes to read
   DWORD                               Read;                // bytes read
   char                                *Buffer;             // file buffer

   // create file name to load
   strcpy( FileName, ScriptFolder );
   strcat( FileName, CloseScriptEdit->Text.c_str() );
   strcat( FileName, ".psc" );

   // open file
   hFile = CreateFile( FileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL );  // open file
   if ( hFile == INVALID_HANDLE_VALUE )                     // if open fails show error dialog
      return;

   GetFileInformationByHandle( hFile, &Info );              // get information for buffer size allocation
   ToRead = Info.nFileSizeLow;                              // file lenght
   Buffer = (char*) malloc( ToRead+1 );                     // allocate buffer for all file plus terminator
   if ( Buffer == 0 )                                       // if not enought memory show error dialog
   {
      strcpy( Message, "Memoria insufficiente per caricare file:\n" );
      strcat( Message, FileName );
      MessageBox( 0, Message, "Errore ", 0);
      CloseHandle( hFile);                                  // close file
      return;
   }

   ReadFile ( hFile, Buffer, ToRead, &Read, NULL);          // read all file in one block
   Buffer[ToRead] = 0;                                      // string terminator
   CloseScriptEditor->SetText( Buffer );                    // assign text to editor
   CloseHandle( hFile);                                     // close file
   free (Buffer);                                           // free buffer memory
}



/*---------------------------------------------------------------------------

   Save edit object in the cloee script file

---------------------------------------------------------------------------*/

void __fastcall TMainForm::SaveCloseScript(TObject *Sender)
{
   char                                FileName[MAXPATH];   // file name
   HANDLE                              hFile;               // file handle
   char                                Message[200];        // error message
   DWORD                               ToWrite;             // byte da scrivere
   DWORD                               Written;             // byte scritti
   char                                *Buffer;             // file buffer

   // create file name to save
   strcpy( FileName, ScriptFolder );
   strcat( FileName, CloseScriptEdit->Text.c_str() );
   strcat( FileName, ".psc" );

   // open file
   hFile = CreateFile( FileName, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS,	FILE_ATTRIBUTE_ARCHIVE, NULL );
   if ( hFile == INVALID_HANDLE_VALUE )                     // if open fails show error dialog
      return;

   ToWrite = CloseScriptEditor->GetTextLenght()-1;          // text lenght
   Buffer = (char*) malloc( ToWrite+1 );                    // allocate buffer for all text
   CloseScriptEditor->GetText(Buffer, ToWrite+1);           // get text
   if ( Buffer == 0 )                                       // if not enought memory show error dialog
   {
      strcpy( Message, "Memoria insufficiente per salvare file:\n" );
      strcat( Message, FileName );
      MessageBox( 0, Message, "Errore ", 0);
      CloseHandle( hFile);                                  // close file
      free (Buffer);                                        // free buffer memory
      return;
   }

   WriteFile ( hFile, Buffer, ToWrite, &Written, NULL);     // write file
   CloseHandle( hFile);                                     // close file
   free (Buffer);                                           // free buffer memory
   CloseScriptChanged = false;                              // reset flag for change
}



/*---------------------------------------------------------------------------

   load script in CPU

---------------------------------------------------------------------------*/

void TMainForm::LoadScript(void)
{
   char                                *MainBuffer;         // main script buffer
   char                                *TimeBuffer;         // time script buffer
   char                                *CloseBuffer;        // close script buffer
   int                                 BufferLen;           // buffer lenght
   ScriptErrorList::LIST_ELEMENT       *ErrorElement;       // scanning error element
   AnsiString                          Message;             // error message

   // get main script
   BufferLen = MainScriptEditor->GetTextLenght();           // text lenght
   MainBuffer = (char*) malloc( BufferLen );                // allocate buffer for all text
   MainScriptEditor->GetText(MainBuffer, BufferLen);        // get text from editor

   // get time script
   BufferLen = TimeScriptEditor->GetTextLenght();           // text lenght
   TimeBuffer = (char*) malloc( BufferLen );                // allocate buffer for all text
   TimeScriptEditor->GetText(TimeBuffer, BufferLen);        // get text from editor

   // get close script
   BufferLen = CloseScriptEditor->GetTextLenght();          // text lenght
   CloseBuffer = (char*) malloc( BufferLen );               // allocate buffer for all text
   CloseScriptEditor->GetText(CloseBuffer, BufferLen);      // get text from editor

   CPU->SetScript(MainBuffer, TimeBuffer, CloseBuffer);     // send all script to CPU

   // free all buffers
   free( MainBuffer );
   free( TimeBuffer );
   free( CloseBuffer );

   // show error list
   ErrorList = CPU->GetErrorList();                         // get error list
   ErrorListMemo->Lines->Clear();                           // clear previous error list showed
   ErrorList->First();                                      // pointer on the first element in the list
   ErrorElement = ErrorList->Next();                        // get first element
   while ( ErrorElement != 0 )                              // continue while element are present
   {
      switch( ErrorElement->Script )                        // decode script source of error
      {
         case 1:     Message = "Programma principale ";  break;
         case 2:     Message = "Programma a tempo    ";  break;
         default:    Message = "Programma di chiusura";  break;
      }
      Message += " Linea:" + AnsiString(ErrorElement->Line);   // add error line
      Message += " Colonna:" + AnsiString(ErrorElement->Column);  // add error column
      switch( ErrorElement->Error )                         // decode error code
      {
         case ScriptTask::ERR_PREMATUREEND:
            Message += " fine programma con ciclo non terminato";    break;
         case ScriptTask::ERR_SINTAX:
            Message += " comando non conosciuto";    break;
         case ScriptTask::ERR_WRONGPARAMETER:
            Message += " parametro errato";    break;
         case ScriptTask::ERR_PARAMOUTOFRANGE:
            Message += " parametro fuori limiti consentiti";    break;
         case ScriptTask::ERR_NOPARAMETER:
            Message += " manca parametro";    break;
         case ScriptTask::ERR_STACKEMPTY:
            Message += " fine ciclo senza corrispondete inizio ciclo";    break;
         case ScriptTask::ERR_STACKFULL:
            Message += " superato numero massimo di nidificazioni";    break;
      }
      ErrorListMemo->Lines->Add(Message);                   // add error string in the shoed error list
      ErrorElement = ErrorList->Next();                     // get next error
   }
}



/*---------------------------------------------------------------------------

   Exec single step in script CPU

---------------------------------------------------------------------------*/

void __fastcall TMainForm::StepScriptButtonClick(TObject *Sender)
{
   int                                 TimeScriptInterval;  // interval for time script (minutes)
   bool                                TimeSciptEnable;     // enable time script
   int                                 HourCloseScript;     // hour for close script activation
   int                                 MinuteCloseScript;   // minute for close script activation

   // check communication and CCD activation
   if ( Status.CommOpened == false || Status.CommOk == false || Status.CCDEnabled == false )
   {
      MessageBox( this->Handle, "Non  possibile lanciare programma, una delle sequenti condizioni non  verificata:\n   Comunicazione attiva\n   Comunicazione ok\n   CCD abilitato", "Errore", MB_OK );
      return;
   }
   // load script in the CPU if first time or if script are changed
   if ( CPULoaded == false )
   {
      LoadScript();                                         // load script in the CPU
      if ( ErrorList->GetCount() == 0 )                     // reset the need to load only if no error was found
         CPULoaded = true;
      else
         MessageBox( this->Handle, "Non  possibile lanciare programma, errori presenti nelle istruzioni", "Errore", MB_OK );
   }
   // exec step activity only of CPU is correctly loaded
   if ( CPULoaded == true )
   {
      TimeScriptInterval = atoi( IntervalTimeScriptEdit->Text.c_str() );   // get time script interval
      TimeSciptEnable = EnableTimeScriptCheck->Checked;                    // get time scripe enable
      HourCloseScript = atoi( HourCloseScriptEdit->Text.c_str() );         // get time for close script activation
      MinuteCloseScript = atoi( MinuteCloseScriptEdit->Text.c_str() );
      CPU->SetExecParameters(this->Handle, TimeScriptInterval, TimeSciptEnable, HourCloseScript, MinuteCloseScript, SafeElevation);   // set all parameters into CPU
      CPU->Step();                                          // exec one step
   }
}



/*---------------------------------------------------------------------------

   run script CPU

---------------------------------------------------------------------------*/

void __fastcall TMainForm::RunScriptButtonClick(TObject *Sender)
{
   int                                 TimeScriptInterval;  // interval for time script (minutes)
   bool                                TimeSciptEnable;     // enable time script
   int                                 HourCloseScript;     // hour for close script activation
   int                                 MinuteCloseScript;   // minute for close script activation

   // check communication and CCD activation
   if ( Status.CommOpened == false || Status.CommOk == false || Status.CCDEnabled == false )
   {
      MessageBox( this->Handle, "Non  possibile lanciare programma, una delle sequenti condizioni non  verificata:\n   Comunicazione attiva\n   Comunicazione ok\n   CCD abilitato", "Errore", MB_OK );
      return;
   } 

   // load script in the CPU if first time or if script are changed
   if ( CPULoaded == false )
   {
      LoadScript();                                         // load script in the CPU
      if ( ErrorList->GetCount() == 0 )                     // reset the need to load only if no error was found
         CPULoaded = true;
      else
         MessageBox( this->Handle, "Non  possibile lanciare programma, errori presenti nelle istruzioni", "Errore", MB_OK );
   }
   // exec run activity only of CPU is correctly loaded
   if ( CPULoaded == true )
   {
      TimeScriptInterval = atoi( IntervalTimeScriptEdit->Text.c_str() );   // get time script interval
      TimeSciptEnable = EnableTimeScriptCheck->Checked;                    // get time scripe enable
      HourCloseScript = atoi( HourCloseScriptEdit->Text.c_str() );         // get time for close script activation
      MinuteCloseScript = atoi( MinuteCloseScriptEdit->Text.c_str() );
      CPU->SetExecParameters(this->Handle, TimeScriptInterval, TimeSciptEnable, HourCloseScript, MinuteCloseScript, SafeElevation);   // set all parameters into CPU
      CPU->Start();                                         // exec run
   }
}



/*---------------------------------------------------------------------------

   stop script CPU

---------------------------------------------------------------------------*/

void __fastcall TMainForm::StopScriptButtonClick(TObject *Sender)
{
   // if CPU is correctly loaded exec CPU stop
   if ( CPULoaded == true )
      CPU->Stop();
}



/*---------------------------------------------------------------------------

   reset script CPU to the first instruction

---------------------------------------------------------------------------*/

void __fastcall TMainForm::RewindScriptButtonClick(TObject *Sender)
{
   // if CPU is correctly loaded exec CPU rewind
   if ( CPULoaded == true )
      CPU->Rewind();
}



/*---------------------------------------------------------------------------

   enable chart follow telecope position

---------------------------------------------------------------------------*/

void __fastcall TMainForm::CharFollowsTelescopeCheckBoxClick(
      TObject *Sender)
{
   if ( CharFollowsTelescopeCheckBox->Checked == true )
      SetChartFromTelescope = 4;
}



/*---------------------------------------------------------------------------

   show licence

---------------------------------------------------------------------------*/

void __fastcall TMainForm::LicenceClick(TObject *Sender)
{
   TForm4                              *LicenceForm;        // licence dialog box
   int                                 ReturnCode;          // return value
   TRegistry                           *Reg;                // registry obejct
   AnsiString                          Licence;             // licence enable

   LicenceForm = new TForm4(this);                          // create licence dialog box
   ReturnCode = LicenceForm->ShowModal();                   // show dialog box
   delete LicenceForm;                                      // delete dialog box

   // store selection
   Reg = new TRegistry();
   Licence = "";
   if ( Reg->OpenKey( "Software\\Polypus", false ) == true )
   {
      if ( ReturnCode == mrOk )                             // licence accepted
         Reg->WriteString( "License", "Y" );
      else
         Reg->WriteString( "License", "N" );
   }
   delete Reg;

   if ( ReturnCode != mrOk )                                // licence not accepted
      Close();                                              // program stop
}



/*---------------------------------------------------------------------------

   Enable/Disable mail download

---------------------------------------------------------------------------*/

void __fastcall TMainForm::GRBEnableCheckBoxClick(TObject *Sender)
{
   TentaclesMgr                        *ObjTentacles;       // subject: object managing all observatory devices

   ObjTentacles = MFObserver->GetSubject();                 // get tentacols object from observer

   MailTimer->Enabled = GRBEnableCheckBox->Checked;
   if ( MailTimer->Enabled == false )
   {
      ObjTentacles->Notify( NULL, TentaclesMgr::NOTIFY_GRBDEACTIVE );
      MailTimer->Interval = 1000;
      MailStatus->Text = "Disabilitata";
   }
   else
   {
      ObjTentacles->Notify( NULL, TentaclesMgr::NOTIFY_GRBACTIVE );
      MailStatus->Text = "Attesa e-mail";
   }
}



/*---------------------------------------------------------------------------

   Enable/Disable SWIFT spacecraft

---------------------------------------------------------------------------*/

void __fastcall TMainForm::SwiftEnableCheckBoxClick(TObject *Sender)
{
   TentaclesMgr                        *ObjTentacles;       // subject: object managing all observatory devices

   ObjTentacles = MFObserver->GetSubject();                 // get tentacols object from observer
   if ( SwiftEnableCheckBox->Checked == true )
   {
      ObjTentacles->Notify( NULL, TentaclesMgr::NOTIFY_GRBACTIVESWIFT );
      DetectGRB();
   }
   else
      ObjTentacles->Notify( NULL, TentaclesMgr::NOTIFY_GRBDEACTIVESWIFT );

}



/*---------------------------------------------------------------------------

   Timely check for a new mail

---------------------------------------------------------------------------*/

void __fastcall TMainForm::MailCheck(TObject *Sender)
{
   SYSTEMTIME                          LocalTime;           // current data and time
   char                                Str[20];             // temporary string
   int                                 Polling;

   // disable timer avoiding overrunnig
   MailTimer->Enabled = false;

   // download new mail from server
   GRBDetector->DownloadMail();

   // shows waiting message
   MailStatus->Text = "Attesa e-mail";

   // update mail grid
   UpdateMailGrid();

   // detect visible GRB
   DetectGRB();

   // update last download time
   GetSystemTime ( &LocalTime );                            // get PC time
   wsprintf( Str, "%02d:%02d", LocalTime.wHour, LocalTime.wMinute );
   LoadDownloadEdit->Text = Str;


   // enable timer
   Polling = atoi (PollingEdit->Text.c_str() );
   if ( Polling < 1 )
      Polling = 1;
   MailTimer->Interval = Polling*1000;
   MailTimer->Enabled = GRBEnableCheckBox->Checked;
}



/*---------------------------------------------------------------------------

   Update mail grid

---------------------------------------------------------------------------*/

void TMainForm::UpdateMailGrid( void )
{
   int                                 MailCount;           // current mail number
   int                                 MailIndex;           // mail index
   char                                ScanFrom[50];        // scanning source of mail
   char                                ScanSubject[50];     // scanning mail subject
   char                                ScanDate[30];        // scanning mail date
   char                                *ScanText;
   bool                                CanSelect;           // only for call

   // update mail grid
   MailCount = GRBDetector->GetMailsCount();                // get number of mails
   if ( MailCount == 0 )                                    // if no mail are present
      MailGrid->RowCount = 2;                               // at least 2 rows should be visibile (bug in Grid string component)
   else
      MailGrid->RowCount = MailCount+1;                     // rows number is one more of mails (for heading)
   GRBDetector->First();
   for ( MailIndex=1; MailIndex<=MailCount; MailIndex++)    // scan all mails
   {
      GRBDetector->GetNextMail( ScanFrom, ScanSubject, ScanDate, &ScanText );  // get mail informations
      MailGrid->Cells[0][MailIndex] = ScanFrom;             // put them on the grid
      MailGrid->Cells[1][MailIndex] = ScanDate;
      MailGrid->Cells[2][MailIndex] = ScanSubject;
   }

   if ( MailGrid->Row >= MailGrid->RowCount )               // select the first mail if current is not longer present (removed)
      MailGrid->Row = 1;
   MailGridSelectCell( NULL, 0, MailGrid->Row, CanSelect ); // show mail text for selected mail
}



/*---------------------------------------------------------------------------

   Changed selected mail
   Show text for the new selection

---------------------------------------------------------------------------*/

void __fastcall TMainForm::MailGridSelectCell(TObject *Sender, int ACol,
      int ARow, bool &CanSelect)
{
   int                                 MailCount;           // current mail number
   int                                 MailIndex;           // mail index
   char                                ScanFrom[50];        // scanning source of mail
   char                                ScanSubject[50];     // scanning mail subject
   char                                ScanDate[30];        // scanning mail date
   char                                *ScanText;

   // update mail grid
   MailCount = GRBDetector->GetMailsCount();                // get number of mails
   if ( MailCount == 0 )                                    // if no mail are present
      return;
   GRBDetector->First();
   for ( MailIndex=0; MailIndex<ARow; MailIndex++)         // read mails as row number
   {
      GRBDetector->GetNextMail( ScanFrom, ScanSubject, ScanDate, &ScanText );  // get mail informations
   }
   MailTextMemo->Lines->Clear();
   MailTextMemo->Lines->Add(ScanText);
   MailTextMemo->SelStart = 0;
   MailTextMemo->SelLength = 0;
}



/*---------------------------------------------------------------------------

   Scan all mails in order to detect a visible GRB

---------------------------------------------------------------------------*/

void TMainForm::DetectGRB(void)
{
   int                                 MailCount;           // current mail number
   int                                 MailIndex;           // mail index
   int                                 TextIndex;           // idex for text scan
   char                                ScanFrom[50];        // scanning source of mail
   char                                ScanSubject[50];     // scanning mail subject
   char                                ScanDate[30];        // scanning mail date
   char                                *ScanText;
   char                                SpacecraftName[50];  // spacecraft name
   char                                SpacecraftNotify[50];  // spacecraft notify
   CMEvent                             *Event;              // event object
   CMEvent::DATE                       EventDate;           // data for event object
   SYSTEMTIME                          LocalTime;           // current data and time
   double                              MailJD;              // mail reveive julian day
   AnsiString                          Date;                // mail date in AnsiString
   GRB::MAIL_INFO                      MailInfo;            // information in the mail
   bool                                GRBAlert;            // GRB alert arrived in time window
   CMHours::HOURS                      SidTime;             // current sideral time
   double                              Latitude;            // observatory latitude
   double                              HourAngle;           // GRB hour angle
   double                              Azimut;              // telescope azimut
   double                              Declination;         // GRB declination in degree
   double                              Temp1;               // termporary double for computing
   double                              Temp2;               // termporary double for computing
   double                              Elevation;           // telescope elevation
   TentaclesMgr                        *ObjTentacles;       // subject: object managing all observatory devices

   Event = new CMEvent();                                   // create object for jd computing
   ObjTentacles = MFObserver->GetSubject();                 // get tentacols object from observer

   // get current JD
   GetSystemTime ( &LocalTime );                            // get PC time
   EventDate.Year = LocalTime.wYear;                        // set event structure
   EventDate.Month = LocalTime.wMonth;
   EventDate.Day = LocalTime.wDay;
   EventDate.Hour = LocalTime.wHour;
   EventDate.Minute = LocalTime.wMinute;
   EventDate.Sec = LocalTime.wSecond;
   Event->SetDate( &EventDate );                            // set event

   // set the starting point and mail numbers
   MailCount = GRBDetector->GetMailsCount();
   GRBDetector->First();

   // scan all mails in the list
   for ( MailIndex=1; MailIndex<=MailCount; MailIndex++)
   {
      GRBDetector->GetNextMail( ScanFrom, ScanSubject, ScanDate, &ScanText );  // get mail informations

      // bypass too old mail
      Date = AnsiString(ScanDate);
      EventDate.Year = Date.SubString(1,4).ToInt();            // set event structure
      EventDate.Month = Date.SubString(6,2).ToInt();
      EventDate.Day = Date.SubString(9,2).ToInt();
      EventDate.Hour = Date.SubString(12,2).ToInt();
      EventDate.Minute = Date.SubString(15,2).ToInt();
      EventDate.Sec = 0;
      Event->SetDate( &EventDate );                            // set event
      MailJD = Event->GetGG();                                 // get JD

      if ( (NowJD-MailJD)*1440 > MaxDelay )
         continue;

      // scan subject looking for spacecraf name and notify
      strcpy( SpacecraftName, ScanSubject+4 );
      for ( TextIndex=0; TextIndex<(int)strlen(SpacecraftName); TextIndex++)
      {
         if ( SpacecraftName[TextIndex] == '_' )
         {
            SpacecraftName[TextIndex] = 0;
            strcpy( SpacecraftNotify, SpacecraftName+ TextIndex + 1 );
         }
      }

      GRBAlert = false;                                        // reset alert condition

      // detect GRB alert from SWIFT spacescraft
      if ( SwiftEnableCheckBox->Checked == true )
      {
         // ckeck for Spacecraft rapid movement
         if ( strcmp(SpacecraftNotify, "SC_2SLEW") == 0 )
            GRBAlert = GRBDetector->UnderstandMail( ScanText, &MailInfo );
            if ( MailInfo.TriggerIndex == 0 )
               GRBAlert = false;
         if ( strcmp(SpacecraftNotify, "SC_SLEW") == 0 )
            GRBAlert = GRBDetector->UnderstandMail( ScanText, &MailInfo );
            if ( MailInfo.TriggerIndex == 0 )
               GRBAlert = false;

         // ckeck for factor of merit computing
         if ( strcmp(SpacecraftNotify, "FOM_2OBS") == 0 )
            GRBAlert = GRBDetector->UnderstandMail( ScanText, &MailInfo );
         if ( strcmp(SpacecraftNotify, "FOM_OBSERVE") == 0 )
            GRBAlert = GRBDetector->UnderstandMail( ScanText, &MailInfo );
      }

      if ( GRBAlert == true )
      {
         // compute GRB elevation
         SidTime = CM_SideralTimeComputing( NowJD, false );       // compute Greenwich  sideral time
         CM_AddTime( &ObsLongitude, &SidTime );                   // compute local sideral time
         HourAngle = CM_Convert( &SidTime ) - CM_Convert( &MailInfo.RA );   // GRB hour angle
         Declination = CM_Convert( &MailInfo.Dec );               // GRB declination in degree
         Latitude = CM_Convert ( &ObsLatitude );                  // observatory latitude
         HourAngle /= 180/M_PI;                                   // GRB hour angle in radiant
         Declination /= 180/M_PI;                                 // GRB declination in radiant
         Latitude /= 180/M_PI;                                    // observatory latitude in radiant
         Temp1 = sin(HourAngle);                                  // first parameter for trasnformation
         Temp2 = cos(HourAngle)*sin(Latitude) - tan(Declination)*cos(Latitude);  // second parameter for trasnformation
         if ( Temp2 == 0 )                                        // to evalid zero divide exception
         {
            if ( Temp1 > 0 )                                      // assign azimut depending on sign of first parameter
               Azimut = M_PI_2;
            else
               Azimut = - M_PI_2;
         }
         else
            Azimut = atan( Temp1/Temp2 );                         // compute azimut
         Azimut *= 180/M_PI;
         if ( Temp2 < 0 )                                         // arrange quadrant restut
            Azimut += 180;
         if ( Azimut < 0 )                                        // arrange quadrant restut
            Azimut += 360;
         Temp1 = sin(Latitude)*sin(Declination) + cos(Latitude)*cos(Declination)*cos(HourAngle);   // compute elevetion asin in radiant
         Elevation = asin(Temp1) * 180/M_PI;                      // compute elevation

         // if GRB alert is over the minimum elevation it become GRB detected
         if ( Elevation >= MinElevation && GRBDetected == false )
         {
            // signal GRB detection
            GRBDetected = true;
            ObjTentacles->Notify( NULL, TentaclesMgr::NOTIFY_GRBDETECTED );

            // assign GRB to tentacoles
            ObjTentacles->SetGRBPos(&MailInfo.RA, &MailInfo.Dec); // set GRB position

            // set GRB index
            CPU->SetGRBTrigger(MailInfo.TriggerIndex);

            // update internal coordinates
            GRBRA = MailInfo.RA;
            GRBDec = MailInfo.Dec;

            // update user interface coordintes
            GRBRA_hhEdit->Text      = GRBRA.hh;
            GRBRA_mmEdit->Text      = GRBRA.mm;
            GRBRA_ssEdit->Text      = GRBRA.ss;

            GRBDec_SignEdit->Text   = GRBDec.sign;
            GRBDec_ggEdit->Text     = GRBDec.gg;
            GRBDec_mmEdit->Text     = GRBDec.mm;
            GRBDec_ssEdit->Text     = GRBDec.ss;

            GRBAzimutEdit->Text     = (int)Azimut;
            GRBElevationEdit->Text  = (int)Elevation;

            // stop current program execution if it is running
            if ( CPURunning == true )
            {
               CPU->Stop();
               Sleep( 500 );
            }

            // assign new main program
            MainScriptEdit->Text = GRBScriptEdit->Text;
            LoadMainScriptFile();

            // set automatic program run
            AutomaticRun = true;
         }
      }
   }

   delete Event;
}



/*---------------------------------------------------------------------------

   Pressed button for GRB program selection

---------------------------------------------------------------------------*/

void __fastcall TMainForm::LoadGRBScriptButton2Click(TObject *Sender)
{
   AnsiString                             FileName;         // configuration file name
   int                                    Pos;              // delimiter postion

   // let use to select a file to define the directory
   OpenDialog->InitialDir = ScriptFolder;
   if ( OpenDialog->Execute() == false )
      return;

   FileName = OpenDialog->FileName;                         // get path file
   Pos = FileName.LastDelimiter("\\");                      // find last '\' before it there is the path
   FileName = FileName.SubString(Pos+1,FileName.Length());  // get file
   Pos = FileName.LastDelimiter(".");                       // find last '.' before it there is the path
   FileName = FileName.SubString(1,Pos-1);                  // get file without extension
   GRBScriptEdit->Text = FileName;                          // write on the editbox
}



/*---------------------------------------------------------------------------

   Pressed button for GRB information erasing

---------------------------------------------------------------------------*/

void __fastcall TMainForm::GRBResetButtonClick(TObject *Sender)
{
   // reset all GRB information
   GRBDetected = false;

   GRBRA_hhEdit->Text      = "";
   GRBRA_mmEdit->Text      = "";
   GRBRA_ssEdit->Text      = "";

   GRBDec_SignEdit->Text   = "";
   GRBDec_ggEdit->Text     = "";
   GRBDec_mmEdit->Text     = "";
   GRBDec_ssEdit->Text     = "";

   GRBAzimutEdit->Text     = "";
   GRBElevationEdit->Text  = "";
}



/*---------------------------------------------------------------------------

   Pressed button for GRB goto

---------------------------------------------------------------------------*/

void __fastcall TMainForm::GRBGotoButtonClick(TObject *Sender)
{
   int                                 MailCount;           // current mail number
   int                                 MailIndex;           // mail index
   char                                ScanFrom[50];        // scanning source of mail
   char                                ScanSubject[50];     // scanning mail subject
   char                                ScanDate[30];        // scanning mail date
   char                                *ScanText;
   GRB::MAIL_INFO                      MailInfo;            // information in the mail
   bool                                GRBAlert;
   int                                 Velocity;            // manual velocity
   TentaclesMgr::ERR                   Error;               // error code from devices manager
   char                                Temp[10];

   // update mail grid
   MailCount = GRBDetector->GetMailsCount();                // get number of mails
   if ( MailCount == 0 )                                    // if no mail are present
      return;
   GRBDetector->First();
   for ( MailIndex=0; MailIndex<MailGrid->Row; MailIndex++)         // read mails as row number
   {
      GRBDetector->GetNextMail( ScanFrom, ScanSubject, ScanDate, &ScanText );  // get mail informations
   }

   // get
   GRBAlert = GRBDetector->UnderstandMail( ScanText, &MailInfo );
   if ( GRBAlert == false )
      return;

   // set telescope velocity
   Velocity = atoi ( TelecopeAutoVelocityEdit->Text.c_str() );
   Error = MFObserver->GetSubject()->SetAutoVelocity( MFObserver, Velocity );
   if ( Error != TentaclesMgr::ERR_NONE )
   {
      ShowError ( Error );                                   // show error message
   }
   // set object name
   strcpy( SelectedObject, "GRB_TRIGGER_" );
   itoa( MailInfo.TriggerIndex, Temp, 10);
   strcat( SelectedObject, Temp );
   // telescope goto command
   Error = MFObserver->GetSubject()->TelescopePos( MFObserver, &MailInfo.RA, &MailInfo.Dec, SelectedObject, false );
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   Update the planning graphic

---------------------------------------------------------------------------*/

void TMainForm::UpdatePlanningGraph(void)
{
   int                        SizeX;                        // Horizontal image size
   int                        SizeY;                        // vertical image size
   Graphics::TBitmap          *Bmp;                         // bitmap where create image
   TRect                      Rect;
   double                     JD;

   SizeX = PlanningImage->Width;                            // get image size
   SizeY = PlanningImage->Height;
   Bmp = PlanningImage->Picture->Bitmap;                    // get bitmap in the image
   Bmp->Width = SizeX;                                      // assign image size to bitmap
   Bmp->Height = SizeY;
   Bmp->PixelFormat = pf24bit;                              // true color

   // giorno giuliano alla mezza notte del giorno in corso
   JD = (int)MonthCalendar1->Date + 2415018.5;              // get Julian day from calendar

   PlaningGraph->SetColor( TabColor, MAX_PLANNEDOBJ );      // set color

   PlaningGraph->SetX(15, 7, 1);                            // set horizonatal start and stop time and grid size (hours)
   PlaningGraph->SetY(0, 90, 10);                           // set vertical start and stop elevation (degree)
   PlaningGraph->SetSite( Site );                           // set observatory position
   PlaningGraph->SetDate(JD);                               // set julian day
   PlaningGraph->GetBmp( &Bmp );                            // get the graph in the bitmap
}



/*---------------------------------------------------------------------------

   Deletct date change in panning calendar

---------------------------------------------------------------------------*/

void __fastcall TMainForm::MonthCalendar1Click(TObject *Sender)
{
   UpdatePlanningGraph( );                                  // update planning view
}



/*---------------------------------------------------------------------------

   planning view for previous day

---------------------------------------------------------------------------*/

void __fastcall TMainForm::PreviousDayButtonClick(TObject *Sender)
{
   MonthCalendar1->Date = MonthCalendar1->Date - 1;         // decrease calendar by 1 day
   UpdatePlanningGraph( );                                  // update planning view
}



/*---------------------------------------------------------------------------

   planning view for next day

---------------------------------------------------------------------------*/

void __fastcall TMainForm::NextDayButtonClick(TObject *Sender)
{
   MonthCalendar1->Date = MonthCalendar1->Date + 1;         // increase calendar by 1 day
   UpdatePlanningGraph( );                                  // update planning view
}



/*---------------------------------------------------------------------------

   add currect selected object in obect pages to palling list

---------------------------------------------------------------------------*/

void __fastcall TMainForm::PlaneButtonClick(TObject *Sender)
{
   int                                 SelectedItem;        // selected item in list box
   CMObject                            *SelectedObj;        // selected obejct in the listbos
   char                                ObjName[200];        // object name

   SelectedItem = ObjListBox->ItemIndex;                    // get number of selected item
   if ( SelectedItem == -1 )                                // something should be selected
      return;

   // get selected object
   SelectedObj = (CMObject*)ObjListBox->Items->Objects[SelectedItem];
   if ( SelectedObj == 0 )                                  // pointer should be valid
      return;

   PlanningList->Add(SelectedObj);                          // add object to plannning list
   UpdatePlanningGraph();                                   // update planning view

   strcpy( ObjName, SelectedObj->GetName(CMObject::NAME_OWN) ); // get own name
   if ( *ObjName == 0 )                                     // if object don't have own name
      strcpy( ObjName, SelectedObj->GetName(CMObject::NAME_CATALOG) ); // get owcatalog name
   PlanningListBox->Items->Add( ObjName );                  // insert name into listbox
}




/*---------------------------------------------------------------------------

   planning list draw cell
   in this way it is possibile give to every line a different color

---------------------------------------------------------------------------*/

void __fastcall TMainForm::PlanningDrawGridDrawCell(TObject *Sender, int ACol,
      int ARow, TRect &Rect, TGridDrawState State)
{
   AnsiString                          ObjName;             // object name

   PlanningDrawGrid->Canvas->Font->Color = TabColor[ARow];  // set color from the table
   PlanningDrawGrid->Canvas->Brush->Color = clWhite;        // background is white
   PlanningDrawGrid->Canvas->FillRect( Rect);               // paint background
   if ( ARow < PlanningListBox->Items->Count )              // check if an object is present in current cell
   {
      ObjName = PlanningListBox->Items->Strings[ARow];      // get object name fron hidden listbox
      PlanningDrawGrid->Canvas->TextRect( Rect, Rect.Left, Rect.Top, ObjName); // write object name
   }
}



/*---------------------------------------------------------------------------

   remove all objects from planning list

---------------------------------------------------------------------------*/

void __fastcall TMainForm::ErasePlanningButtonClick(TObject *Sender)
{
   PlanningListBox->Items->Clear();                         // remove all objects fron hidden listbox
   PlanningList->RemoveAll();                               // remove all objects from the list
   PlanningDrawGrid->Invalidate();                          // invalidate colored grid list
   UpdatePlanningGraph();                                   // update planning view
}



/*---------------------------------------------------------------------------

   increase declination

---------------------------------------------------------------------------*/

void __fastcall TMainForm::IncDecButtonClick(TObject *Sender)
{
   CMHours::HOURS                      RA;                  // object right ascension
   CMDegree::DEGREE                    Dec;                 // object declination
   CMDegree::DEGREE                    Delta;               // delta declination
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   // get currect telescope position
   RA = Status.TeleRA;
   Dec = Status.TeleDec;

   // add declination
   strcpy (Delta.sign, "+");
   Delta.gg = 0;
   Delta.mm = atoi( DeltaDecEdit->Text.c_str() );
   Delta.ss = 0;
   CM_AddDec( &Dec, &Delta );

   // telescope goto command
   Error = MFObserver->GetSubject()->TelescopePos( MFObserver, &RA, &Dec, SelectedObject, false );
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   decrease declination

---------------------------------------------------------------------------*/

void __fastcall TMainForm::DecDecButtonClick(TObject *Sender)
{
   CMHours::HOURS                      RA;                  // object right ascension
   CMDegree::DEGREE                    Dec;                 // object declination
   CMDegree::DEGREE                    Delta;               // delta declination
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   // get currect telescope position
   RA = Status.TeleRA;
   Dec = Status.TeleDec;

   // add declination
   strcpy (Delta.sign, "-");
   Delta.gg = 0;
   Delta.mm = atoi( DeltaDecEdit->Text.c_str() );
   Delta.ss = 0;
   CM_AddDec( &Dec, &Delta );

   // telescope goto command
   Error = MFObserver->GetSubject()->TelescopePos( MFObserver, &RA, &Dec, SelectedObject, false );
   ShowError ( Error );                                     // show error message

}



/*---------------------------------------------------------------------------

   increase right ascension

---------------------------------------------------------------------------*/

void __fastcall TMainForm::IncRAButtonClick(TObject *Sender)
{
   CMHours::HOURS                      RA;                  // object right ascension
   CMDegree::DEGREE                    Dec;                 // object declination
   double                              DeltaTime;           // delta righe ascension in time minute
   CMHours::HOURS                      Delta;               // delta right ascension
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   // get currect telescope position
   RA = Status.TeleRA;
   Dec = Status.TeleDec;

   // add declination
   DeltaTime = atoi( DeltaRAEdit->Text.c_str() );
   DeltaTime /= 15;
   Delta.hh = 0;
   Delta.mm = DeltaTime;
   Delta.ss = (DeltaTime - Delta.mm) * 60;
   CM_AddTime( &Delta, &RA );

   // telescope goto command
   Error = MFObserver->GetSubject()->TelescopePos( MFObserver, &RA, &Dec, SelectedObject, false );
   ShowError ( Error );                                     // show error message
}



/*---------------------------------------------------------------------------

   decrease right ascension

---------------------------------------------------------------------------*/

void __fastcall TMainForm::DecRAButtonClick(TObject *Sender)
{
   CMHours::HOURS                      RA;                  // object right ascension
   CMDegree::DEGREE                    Dec;                 // object declination
   double                              DeltaTime;           // delta righe ascension in time minute
   CMHours::HOURS                      Delta;               // delta right ascension
   TentaclesMgr::ERR                   Error;               // error code from devices manager

   // get currect telescope position
   RA = Status.TeleRA;
   Dec = Status.TeleDec;

   // add declination
   DeltaTime = atoi( DeltaRAEdit->Text.c_str() );
   DeltaTime /= 15;
   Delta.hh = 0;
   Delta.mm = DeltaTime;
   Delta.ss = (DeltaTime - Delta.mm) * 60;
   CM_SubTime( &Delta, &RA );

   // telescope goto command
   Error = MFObserver->GetSubject()->TelescopePos( MFObserver, &RA, &Dec, SelectedObject, false );
   ShowError ( Error );                                     // show error message
}
//---------------------------------------------------------------------------


